@@ -1,24 +1,23 @@
-/* gretl - The Gnu Regression, Econometrics and Time-series Library
- * Copyright (C) 1999-2000 Ramu Ramanathan and Allin Cottrell
- *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License 
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
- *
- * This software is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License 
- * along with this software; if not, write to the 
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+/* 
+ *  gretl -- Gnu Regression, Econometrics and Time-series Library
+ *  Copyright (C) 2001 Allin Cottrell and Riccardo "Jack" Lucchetti
+ * 
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ * 
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ * 
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
-/* interactive client program for libgretl - 
-   uses the GNU readline library if available */
+/* command-line client program for libgretl */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -269,7 +268,7 @@ get_an_input_line (ExecState *s, double ***pZ, DATAINFO *pdinfo)
 #else
 	printf("%s", (coding)? "> " : "? ");
 	fflush(stdout);
-	file_get_line(); /* note: "file" = stdin here */
+	file_get_line(s->line, s->cmd); /* note: "file" = stdin here */
 #endif
     }
 
@@ -322,7 +321,7 @@ static void set_errfatal (int code)
     static int hoe = -1;
 
     if (hoe < 0) {
-	hoe = get_halt_on_error();
+	hoe = libset_get_bool(HALT_ON_ERR);
     }
 
     if (code == ERRFATAL_FORCE) {
@@ -338,6 +337,18 @@ static void set_errfatal (int code)
     }
 }
 
+static int xout;
+
+#ifdef HAVE_READLINE
+static int ctrl_x (int count, int key)
+{
+    xout = 1;
+    rl_done = 1;
+    puts("exit");
+    return 0;
+}
+#endif
+
 int main (int argc, char *argv[])
 {
     double **Z = NULL;
@@ -356,6 +367,10 @@ int main (int argc, char *argv[])
     nls_init();
 #endif
 
+#ifdef HAVE_READLINE
+    rl_bind_key(0x18, ctrl_x);
+#endif
+
     datainfo = datainfo_new();
     if (datainfo == NULL) {
 	noalloc(_("data information"));
@@ -457,21 +472,18 @@ int main (int argc, char *argv[])
 	    break;
 	case GRETL_NATIVE_DATA:
 	    err = gretl_get_data(&Z, &datainfo, paths.datfile, &paths, 
-				 DATA_NONE, prn);
+				 prn);
 	    break;
 	case GRETL_XML_DATA:
 	    err = gretl_read_gdt(&Z, &datainfo, paths.datfile, &paths, 
-				 DATA_NONE, prn, 0);
+				 OPT_NONE, prn);
 	    break;
 	case GRETL_CSV_DATA:
-	    err = import_csv(&Z, &datainfo, paths.datfile, prn);
+	    err = import_csv(&Z, &datainfo, paths.datfile, OPT_NONE, prn);
 	    break;
 	case GRETL_OCTAVE:
 	    err = import_octave(&Z, &datainfo, paths.datfile, prn);
 	    break;
-	case GRETL_BOX_DATA:
-	    err = import_box(&Z, &datainfo, paths.datfile, prn);
-	    break;
 	case GRETL_GNUMERIC:
 	case GRETL_EXCEL:
 	case GRETL_WF1:
@@ -555,7 +567,7 @@ int main (int argc, char *argv[])
     set_errfatal(ERRFATAL_AUTO);
 
     /* main command loop */
-    while (strcmp(cmd.word, "quit") && fb != NULL) {
+    while (cmd.ci != QUIT && fb != NULL && !xout) {
 	char linecopy[MAXLINE];
 	int overflow;
 
@@ -652,6 +664,8 @@ static void cli_exec_callback (ExecState *s, double ***pZ,
 	cli_do_autofit_plot(s->models[0], pZ, pdinfo, s->prn);
     } else if (ci == VAR || ci == VECM) {
 	maybe_stack_var(s->var, s->cmd);
+    } else if (ci == END && !strcmp(s->cmd->param, "restrict")) {
+	maybe_stack_var(s->var, s->cmd);
     }
 }
 
@@ -659,7 +673,6 @@ static int cli_open_append (CMD *cmd, const char *line, double ***pZ,
 			    DATAINFO **ppdinfo, MODEL **models,
 			    PRN *prn)
 {
-    DataOpenCode ocode = DATA_NONE;
     DATAINFO *pdinfo = *ppdinfo;
     char datfile[MAXLEN] = {0};
     char response[3];
@@ -675,9 +688,7 @@ static int cli_open_append (CMD *cmd, const char *line, double ***pZ,
 
     if (cmd->opt & OPT_W) {
 	k = GRETL_NATIVE_DB_WWW;
-    } else if (cmd->opt & OPT_B) {
-	k = GRETL_BOX_DATA;
-    } else if (cmd->opt & OPT_O) {
+     } else if (cmd->opt & OPT_O) {
 	k = GRETL_CSV_DATA;
     } else {
 	k = detect_filetype(datfile, &paths, prn);
@@ -698,30 +709,26 @@ static int cli_open_append (CMD *cmd, const char *line, double ***pZ,
     }
 
     if (data_status) {
-	if (dbdata || cmd->ci == APPEND) {
-	    ocode = DATA_APPEND;
-	} else {
+	if (!dbdata && cmd->ci != APPEND) {
 	    clear_data(cmd, pZ, ppdinfo, models);
 	    pdinfo = *ppdinfo;
 	}
     } 
 
     if (k == GRETL_CSV_DATA) {
-	err = import_csv(pZ, ppdinfo, datfile, prn);
+	err = import_csv(pZ, ppdinfo, datfile, OPT_NONE, prn);
     } else if (k == GRETL_OCTAVE) {
 	err = import_octave(pZ, ppdinfo, datfile, prn);
-    } else if (k == GRETL_BOX_DATA) {
-	err = import_box(pZ, ppdinfo, datfile, prn);
     } else if (WORKSHEET_IMPORT(k)) {
 	err = import_other(pZ, ppdinfo, k, datfile, prn);
     } else if (k == GRETL_XML_DATA) {
 	err = gretl_read_gdt(pZ, ppdinfo, datfile, &paths, 
-			     ocode, prn, 0);
+			     OPT_NONE, prn);
     } else if (dbdata) {
 	err = set_db_name(datfile, k, &paths, prn);
     } else {
 	err = gretl_get_data(pZ, ppdinfo, datfile, &paths, 
-			     ocode, prn);
+			     prn);
     }
 
     pdinfo = *ppdinfo;
@@ -744,6 +751,16 @@ static int cli_open_append (CMD *cmd, const char *line, double ***pZ,
     return err;
 }
 
+static gretlopt plot_opt (gretlopt opt, int batch)
+{
+    if (batch) {
+	opt |= OPT_B;
+    }
+    return opt;
+}
+
+#define ENDRUN (NC + 1)
+
 static int exec_line (ExecState *s, double ***pZ, DATAINFO **ppdinfo)
 {
     DATAINFO *pdinfo = *ppdinfo;
@@ -816,7 +833,7 @@ static int exec_line (ExecState *s, double ***pZ, DATAINFO **ppdinfo)
 	cmd->ci != SYSTEM) {
 	printf(_("Command '%s' ignored; not valid within equation system\n"), 
 	       line);
-	gretl_equation_system_destroy(s->sys);
+	equation_system_destroy(s->sys);
 	s->sys = NULL;
 	return 1;
     }
@@ -861,6 +878,13 @@ static int exec_line (ExecState *s, double ***pZ, DATAINFO **ppdinfo)
     switch (cmd->ci) {
 
     case DELEET:
+	if (cmd->opt & OPT_D) {
+	    err = db_delete_series_by_name(cmd->param, prn);
+	    if (err) {
+		errmsg(err, prn);
+	    } 
+	    break;
+	}
 	if (get_matrix_by_name(cmd->param)) {
 	    err = user_matrix_destroy_by_name(cmd->param, prn);
 	    if (err) {
@@ -868,11 +892,6 @@ static int exec_line (ExecState *s, double ***pZ, DATAINFO **ppdinfo)
 	    } 
 	    break;
 	}	    
-	if (complex_subsampled()) {
-	    pputs(prn, _("Can't delete a variable when in sub-sample"
-			 " mode\n"));
-	    break;
-	}	
 	err = dataset_drop_listed_variables(cmd->list, pZ, pdinfo, &k);
 	if (err) {
 	    pputs(prn, _("Failed to shrink the data set"));
@@ -895,11 +914,12 @@ static int exec_line (ExecState *s, double ***pZ, DATAINFO **ppdinfo)
 			     "which is a dummy variable\n(with values 1 or 0)\n"));
 		break;
 	    }
+	    
 	    err = gnuplot(cmd->list, cmd->param, (const double **) *pZ, 
-			  pdinfo, (cmd->opt | OPT_B));
+			  pdinfo, plot_opt(cmd->opt, batch));
 	} else {
 	    err = multi_scatters(cmd->list, (const double **) *pZ, pdinfo, 
-				 (cmd->opt | OPT_B));
+				 plot_opt(cmd->opt, batch));
 	}
 
 	if (err) {
@@ -950,7 +970,7 @@ static int exec_line (ExecState *s, double ***pZ, DATAINFO **ppdinfo)
 	    if (fb == NULL) {
 		pputs(prn, _("Done\n"));
 	    } else {
-		strcpy(cmd->word, "endrun"); /* overwrite "quit" */
+		cmd->ci = ENDRUN;
 	    }
 	    break;
 	}
@@ -995,6 +1015,8 @@ static int exec_line (ExecState *s, double ***pZ, DATAINFO **ppdinfo)
 	    err = load_user_XML_file(runfile);
 	    if (err) {
 		pprintf(prn, _("Error reading %s\n"), runfile);
+	    } else {
+		pprintf(cmdprn, "include \"%s\"\n", runfile);
 	    }
 	    break;
 	}
@@ -1040,9 +1062,9 @@ static int exec_line (ExecState *s, double ***pZ, DATAINFO **ppdinfo)
 	maybe_stack_model(models[0], cmd, prn);
     }
 
-    if (gretl_system_save_flag_set(s->sys)) {
+    if (system_save_flag_is_set(s->sys)) {
 	/* only warrants action in GUI program */
-	gretl_system_unset_save_flag(s->sys);
+	system_unset_save_flag(s->sys);
 	s->sys = NULL;
     }
 
