@@ -1,20 +1,20 @@
-/*
- *  Copyright (c) by Allin Cottrell
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
- *
- *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- *
+/* 
+ *  gretl -- Gnu Regression, Econometrics and Time-series Library
+ *  Copyright (C) 2001 Allin Cottrell and Riccardo "Jack" Lucchetti
+ * 
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ * 
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ * 
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 /* boxplots for gretl */
@@ -652,22 +652,12 @@ make_area (PLOTGROUP *grp)
     return grp->window;
 }
 
-static int 
-compare_doubles (const void *a, const void *b)
-{
-    const double *da = (const double *) a;
-    const double *db = (const double *) b;
-     
-    return (*da > *db) - (*da < *db);
-}
-
-static double 
-median (double *x, const int n)
+static double median (double *x, const int n)
 {
     int n2;
     double xx;
 
-    qsort(x, n, sizeof *x, compare_doubles);
+    qsort(x, n, sizeof *x, gretl_compare_doubles);
 
     n2 = n/2;
     xx = (n % 2)? x[n2] : 0.5 * (x[n2 - 1] + x[n2]);
@@ -769,7 +759,7 @@ median_interval (double *x, int n, double *low, double *high)
     }
 
     /* sort the sample medians */
-    qsort(medians, ITERS, sizeof *medians, compare_doubles);
+    qsort(medians, ITERS, sizeof *medians, gretl_compare_doubles);
     
     /* return the right values */
     j = 100 / ((100 - CONFIDENCE) / 2);
@@ -1004,7 +994,7 @@ int boxplots (int *list, char **bools, double ***pZ, const DATAINFO *pdinfo,
 
 	plotgrp->plots[i].outliers = NULL;
 	plotgrp->plots[i].mean = gretl_mean(0, n, x);
-	qsort(x, n, sizeof *x, compare_doubles);
+	qsort(x, n, sizeof *x, gretl_compare_doubles);
 	plotgrp->plots[i].min = x[0];
 	plotgrp->plots[i].max = x[n-1];
 	quartiles(x, n, &plotgrp->plots[i]);
@@ -1041,7 +1031,7 @@ int boxplots (int *list, char **bools, double ***pZ, const DATAINFO *pdinfo,
     if (plotgrp->show_outliers) {
 	for (i=0; i<plotgrp->nplots; i++) {
 	    n = ztox(list[i+1], x, (const double **) *pZ, pdinfo);
-	    qsort(x, n, sizeof *x, compare_doubles);
+	    qsort(x, n, sizeof *x, gretl_compare_doubles);
 	    add_outliers(x, n, &plotgrp->plots[i]);
 	}
     }
@@ -1234,7 +1224,7 @@ int plot_to_xpm (const char *fname, gpointer data)
     
     fp = gretl_fopen(fname, "w");
     if (fp == NULL) {
-	errbox (_("Couldn't open XPM file for writing"));
+	file_write_errbox(fname);
 	return 1;
     }
 
@@ -1271,19 +1261,37 @@ int plot_to_xpm (const char *fname, gpointer data)
 
 #endif 
 
+static FILE *get_custom_file (void)
+{
+    char boxrc[FILENAME_MAX];
+    FILE *fp;
+    
+    fp = fopen(".boxplotrc", "r");
+
+    if (fp == NULL) {
+	sprintf(boxrc, "%s.boxplotrc", paths.userdir);
+	fp = gretl_fopen(boxrc, "r");
+    }
+
+    if (fp == NULL) {
+	fp = fopen("plotconfig.txt", "r");
+    }
+
+    if (fp == NULL) {
+	sprintf(boxrc, "%splotconfig.txt", paths.userdir);
+	fp = gretl_fopen(boxrc, "r");
+    }    
+
+    return fp;
+}
+
 static void read_boxrc (PLOTGROUP *grp)
 {
     FILE *fp;
 
     grp->gmax = grp->gmin = NADBL;
 
-    fp = fopen(".boxplotrc", "r");
-    if (fp == NULL) {
-	char boxrc[MAXLEN];
-
-	sprintf(boxrc, "%s.boxplotrc", paths.userdir);
-	fp = gretl_fopen(boxrc, "r");
-    }
+    fp = get_custom_file();
 
     if (fp != NULL) {
 	char line[80], key[18], val[32];
@@ -1356,12 +1364,7 @@ static int dump_boxplot (PLOTGROUP *grp)
 
     fp = gretl_fopen(boxplottmp, "w");
     if (fp == NULL) {
-	gchar *msg;
-
-	msg = g_strdup_printf(_("Couldn't open %s for writing"), 
-			      boxplottmp);
-	errbox(msg);
-	g_free(msg);
+	file_write_errbox(boxplottmp);
 	return 1;
     }
 
@@ -1501,15 +1504,11 @@ int retrieve_boxplot (const char *fname)
     PLOTGROUP *grp = NULL;
     BOXPLOT *plt = NULL;
     char line[80], numstr[24];
-    gchar *msg;
     int err = 0;
 
     fp = gretl_fopen(fname, "r");
-
     if (fp == NULL) {
-	msg = g_strdup_printf(_("Couldn't open %s"), fname);
-	errbox(msg);
-	g_free(msg);
+	file_read_errbox(fname);
 	return 1;
     }
 
@@ -1607,7 +1606,7 @@ int retrieve_boxplot (const char *fname)
 
     grp->popup = NULL;
     grp->pixmap = NULL;
-    place_plots (grp);
+    place_plots(grp);
 
     if (make_area(grp) == NULL) {
 	free(grp->plots);
