@@ -1,20 +1,20 @@
-/*
- *   Copyright (C) Allin Cottrell
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
- *
- *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- *
+/* 
+ *  gretl -- Gnu Regression, Econometrics and Time-series Library
+ *  Copyright (C) 2001 Allin Cottrell and Riccardo "Jack" Lucchetti
+ * 
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ * 
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ * 
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 /* callbacks.c for gretl */
@@ -104,7 +104,7 @@ gboolean listbox_drag (GtkWidget *listbox, GdkEventMotion *event,
     GtkTreePath *path;
 
     if (event->is_hint) {
-        gdk_window_get_pointer (event->window, &x, &y, &state);
+        gdk_window_get_pointer(event->window, &x, &y, &state);
     } else {
         x = event->x;
         y = event->y;
@@ -126,7 +126,9 @@ gboolean listbox_drag (GtkWidget *listbox, GdkEventMotion *event,
 	row = tree_path_get_row_number(path);
 
 	select = gtk_tree_view_get_selection(view);
-	if (select == NULL) return FALSE;
+	if (select == NULL) {
+	    return FALSE;
+	}
 	anchor_id = g_strdup_printf("%d", anchor);
 	anchor_path = gtk_tree_path_new_from_string(anchor_id);
 	g_free(anchor_id);
@@ -168,9 +170,6 @@ void open_data (gpointer data, guint code, GtkWidget *widget)
     case APPEND_OCTAVE:
 	file_selector(_("Open Octave file"), code, FSEL_DATA_NONE, NULL);
 	break;
-    case OPEN_BOX:
-	file_selector(_("Open BOX file"), code, FSEL_DATA_NONE, NULL);
-	break;
     case OPEN_GNUMERIC:
     case APPEND_GNUMERIC:
 	file_selector(_("Open Gnumeric file"), code, FSEL_DATA_NONE, NULL);
@@ -191,6 +190,10 @@ void open_data (gpointer data, guint code, GtkWidget *widget)
     case APPEND_JMULTI:
 	file_selector(_("Open JMulTi file"), code, FSEL_DATA_NONE, NULL);
 	break;
+    case OPEN_ODS:
+    case APPEND_ODS:
+	file_selector(_("Open ODS file"), code, FSEL_DATA_NONE, NULL);
+	break;
     case OPEN_MARKERS:
 	file_selector(_("gretl: add markers"), code, FSEL_DATA_NONE, NULL);
 	break;
@@ -325,7 +328,8 @@ void model_genr_callback (gpointer data, guint u, GtkWidget *widget)
 {
     windata_t *mydata = (windata_t *) data;
 
-    edit_dialog(_("gretl: add var"), _("Enter formula for new variable:"),
+    edit_dialog(_("gretl: add var"), 
+		_("Enter formula for new variable:"),
 		"", do_model_genr, mydata, 
 		MODEL_GENR, VARCLICK_INSERT_NAME, NULL);   
 }
@@ -385,7 +389,7 @@ void selector_callback (gpointer data, guint action, GtkWidget *widget)
 	simple_selection(_("gretl: define graph"), do_scatters, action, vwin);
     } else if (action == SPEARMAN) {
 	strcat(title, _("rank correlation"));
-	simple_selection(title, do_spearman, action, vwin);
+	simple_selection(title, do_rankcorr, action, vwin);
     } else {
 	errbox("selector_callback: code was not recognized");
     }
@@ -408,7 +412,8 @@ void gretl_callback (gpointer data, guint action, GtkWidget *widget)
 	break;
     case GENR:
 	title = N_("gretl: add var");
-	query = N_("Enter formula for new variable:");
+	query = N_("Enter formula for new variable\n"
+		   "(or just a name, to enter data manually)");
 	okfunc = do_genr;
 	varclick = VARCLICK_INSERT_NAME;
 	break;
@@ -484,7 +489,22 @@ void file_save_callback (GtkWidget *w, gpointer data)
     windata_t *vwin = (windata_t *) data;
 
     if (g_object_get_data(G_OBJECT(vwin->dialog), "text_out")) {
-	u = SAVE_OUTPUT;
+	const char *opts[] = {
+	    N_("Save to file"),
+	    N_("Save to session as icon")
+	};
+	int resp;
+
+	resp = radio_dialog(_("gretl: save text"), _("Save text"), 
+			    opts, 2, 0, 0);
+	if (resp < 0) {
+	    return;
+	} else if (resp == 1) {
+	    save_output_as_text_icon(vwin);
+	    return;
+	} else {
+	    u = SAVE_OUTPUT;
+	}
     } else {
 	switch (vwin->role) {
 	case EDIT_SCRIPT:
@@ -505,39 +525,6 @@ void file_save_callback (GtkWidget *w, gpointer data)
     file_save(data, u, w);
 }
 
-void add_rand_callback (gpointer data, guint i, GtkWidget *widget) 
-{
-    const char *title[] = {
-	N_("gretl: uniform variable"),
-	N_("gretl: normal variable"),
-	N_("gretl: chi-square variable"),
-	N_("gretl: Student's t variable"),
-	N_("gretl: binomial variable"),
-	N_("gretl: Poisson variable")
-    };
-    const char *info[] = {
-	N_("Enter name for variable, and\n"
-	   "minimum and maximum values:"), 
-	N_("Enter name, mean and standard deviation:"), 
-	N_("Enter name and degrees of freedom:"), 
-	N_("Enter name and degrees of freedom:"), 
-	N_("Enter name, number of trials and probability:"), 
-	N_("Enter name and mean (scalar or series):")
-    };
-    const char *template[] = {
-	"unif 0 1",
-	"norm 0 1", 
-	"chi 5", 
-	"st 20", 
-	"bin 10 0.5", 
-	"pois xbar"
-    };
-	
-    edit_dialog(_(title[i]), _(info[i]), _(template[i]),
-		add_rand_series, GUINT_TO_POINTER(i), 
-		GENR_RANDOM, VARCLICK_NONE, NULL);
-}
-
 void newdata_callback (gpointer data, guint pd_code, GtkWidget *widget) 
 {
     int resp, n = 50;
