@@ -1,20 +1,20 @@
-/*
- *  Copyright (c) by Allin Cottrell
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
- *
- *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- *
+/* 
+ *  gretl -- Gnu Regression, Econometrics and Time-series Library
+ *  Copyright (C) 2001 Allin Cottrell and Riccardo "Jack" Lucchetti
+ * 
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ * 
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ * 
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 /* console.c for gretl */
@@ -252,15 +252,6 @@ static void console_exec (void)
 
     top_n_tail(cbuf);
 
-    if (strcmp(cbuf, "quit") == 0 || 
-	strcmp(cbuf, "q") == 0 ||
-	strcmp(cbuf, "exit") == 0) {
-	gtk_widget_destroy(console_view->parent->parent->parent);
-	g_free(cbuf);
-	cbuf = NULL;
-	return;
-    }
-
     if (console_prn == NULL && bufopen(&console_prn)) {
 	g_free(cbuf);
 	cbuf = NULL;
@@ -292,6 +283,7 @@ static void console_exec (void)
     gtk_text_buffer_get_end_iter(buf, &start);
     gtk_text_buffer_insert(buf, &start, "\n", 1);
 
+#ifdef ENABLE_NLS
     if (!redirected) {
 	const char *cbuf = gretl_print_get_buffer(console_prn);
 
@@ -312,6 +304,31 @@ static void console_exec (void)
     } else {
 	gretl_print_reset_buffer(console_prn);
     }
+#else
+    if (!redirected) {
+	const char *cbuf = gretl_print_get_buffer(console_prn);
+
+        /* put results into console window */
+	gtk_text_buffer_insert(buf, &start, cbuf, strlen(cbuf));
+	gretl_print_destroy(console_prn);
+	console_prn = NULL;
+    } else {
+	gretl_print_reset_buffer(console_prn);
+    }
+#endif
+
+    if (cstate.cmd->ci == QUIT) {
+	gtk_widget_destroy(gtk_widget_get_toplevel(console_view));
+	g_free(cbuf);
+	cbuf = NULL;
+	return;
+    }
+
+#ifdef G_OS_WIN32
+    if (cstate.cmd->ci == SHELL) {
+	gtk_window_present(GTK_WINDOW(gtk_widget_get_toplevel(console_view)));
+    }    
+#endif
 
     coding = gretl_compiling_loop() || gretl_compiling_function();
 
@@ -411,6 +428,20 @@ static int bslash_cont (const gchar *line)
     return bslash;
 }
 
+const char *console_varname_complete (const char *s)
+{
+    size_t n = strlen(s);
+    int i;
+
+    for (i=0; i<datainfo->v; i++) {
+	if (!strncmp(s, datainfo->varname[i], n)) {
+	    return datainfo->varname[i];
+	}
+    }
+
+    return NULL;
+}  
+
 gint console_key_handler (GtkWidget *w, GdkEventKey *key, gpointer d)
 {
     gint last_line, curr_line, line_pos;
@@ -504,28 +535,36 @@ gint console_key_handler (GtkWidget *w, GdkEventKey *key, gpointer d)
 	return TRUE;
     }
 
-    /* tab completion for gretl commands */
+    /* tab completion for gretl commands, variable names */
     if (key->keyval == GDK_Tab) {
+	const char *complete = NULL;
 	GtkTextIter start, end;	
-	gchar *bit;
+	gchar *bit = NULL;
+	int offset;
 
 	start = end = iter;
-	gtk_text_iter_set_line_index(&start, 2);
-	gtk_text_iter_forward_to_line_end(&end);
+	if (!gtk_text_iter_starts_word(&start)) {
+	    gtk_text_iter_backward_word_start(&start);
+	}
+	offset = gtk_text_iter_get_line_offset(&start);
+	if (!gtk_text_iter_ends_word(&end)) {
+	    gtk_text_iter_forward_word_end(&end);
+	}
 	bit = gtk_text_buffer_get_text(buf, &start, &end, FALSE);
 
-	if (bit != NULL) {
-	    if (*bit != 0) {
-		const char *complete = gretl_command_complete(bit);
-
-		if (complete != NULL) {
-		    gtk_text_buffer_delete(buf, &start, &end);
-		    gtk_text_buffer_insert(buf, &start, complete, 
+	if (bit != NULL && *bit != '\0') {
+	    if (offset == 2) {
+		complete = gretl_command_complete(bit);
+	    } else {
+		complete = console_varname_complete(bit);
+	    }
+	    if (complete != NULL) {
+		gtk_text_buffer_delete(buf, &start, &end);
+		gtk_text_buffer_insert(buf, &start, complete, 
 				       strlen(complete));
-		}
 	    }
-	    g_free(bit);
 	}
+	g_free(bit);
 	return TRUE;
     }
 
