@@ -1,25 +1,25 @@
-/*
- *  Copyright (c) by Allin Cottrell
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
- *
- *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+/* 
+ *  gretl -- Gnu Regression, Econometrics and Time-series Library
+ *  Copyright (C) 2001 Allin Cottrell and Riccardo "Jack" Lucchetti
+ * 
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ * 
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ * 
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
  */
 
 /* datafiles.c : for gretl */
 
-#undef COLL_DEBUG 
+#define COLL_DEBUG 0
 
 #include "gretl.h"
 #include "datafiles.h"
@@ -103,7 +103,7 @@ static char *full_path (char *s1, const char *s2)
 	sprintf(fpath, "%s%c%s", s1, SLASH, s2);
     }
 
-#ifdef COLL_DEBUG
+#if COLL_DEBUG
     fprintf(stderr, "full_path: got '%s' from '%s' + '%s'\n",
 	    fpath, s1, s2);
 #endif
@@ -403,7 +403,7 @@ static int get_file_collections_from_dir (const char *dname, DIR *dir)
 
     while (!err && (dirent = readdir(dir))) { 
 	if (strstr(dirent->d_name, "descriptions")) {
-#ifdef COLL_DEBUG
+#if COLL_DEBUG
 	    fprintf(stderr, "   %s: looking at '%s'\n", dname, dirent->d_name);
 #endif
 	    n = strlen(dirent->d_name);
@@ -444,7 +444,7 @@ static int seek_file_collections (const char *topdir)
 	return 1;
     }
 
-#ifdef COLL_DEBUG
+#if COLL_DEBUG
     fprintf(stderr, "seeking file collections in '%s'\n", topdir);
 #endif
 
@@ -457,11 +457,11 @@ static int seek_file_collections (const char *topdir)
 	    }
 	    try = opendir(subdir);
 	    if (try != NULL) {
-#ifdef COLL_DEBUG
+#if COLL_DEBUG
 		fprintf(stderr, " trying in subdir '%s'\n", subdir);
 #endif
 		err = get_file_collections_from_dir(subdir, try);
-#ifdef COLL_DEBUG
+#if COLL_DEBUG
 		fprintf(stderr, " result: err = %d\n", err);
 #endif
 		closedir(try);
@@ -476,7 +476,7 @@ static int seek_file_collections (const char *topdir)
     return err;
 }
 
-#ifdef COLL_DEBUG
+#if COLL_DEBUG
 static void print_collection (const file_collection *coll)
 {
     printf("path = '%s'\n", coll->path);
@@ -530,7 +530,7 @@ static int build_file_collections (void)
 	built = 1;
     }
 
-#ifdef COLL_DEBUG
+#if COLL_DEBUG
     print_data_collections();
     print_script_collections();
 #endif
@@ -705,7 +705,7 @@ static int gui_delete_fn_pkg (const char *fname, windata_t *vwin)
 
     err = remove(fname);
     if (err) {
-	errbox("Couldn't delete %s", fname);
+	file_write_errbox(fname);
     } else {
 	GtkTreeModel *mod;
 	GtkTreeIter iter;
@@ -983,13 +983,6 @@ struct files_item {
     const gchar *icon;
 };
 
-#ifndef GTK_STOCK_INFO
-# define GTK_STOCK_INFO GRETL_STOCK_INFO
-#endif
-#ifndef GTK_STOCK_EDIT
-# define GTK_STOCK_EDIT GRETL_STOCK_EDIT
-#endif
-
 static struct files_item files_items[] = {
     { N_("Open"),      BTN_OPEN,  GTK_STOCK_OK },
     { N_("Edit"),      BTN_EDIT,  GTK_STOCK_EDIT },
@@ -1085,8 +1078,7 @@ static void make_filesbar (windata_t *vwin)
 
 void display_files (gpointer p, guint code, GtkWidget *w)
 {
-    GtkWidget *filebox, *button;
-    GtkWidget *button_box;
+    GtkWidget *filebox;
     windata_t *vwin;
     int col1w = 0;
     int err = 0;
@@ -1186,15 +1178,6 @@ void display_files (gpointer p, guint code, GtkWidget *w)
 	gtk_box_pack_start(GTK_BOX(hbox), vwin->status, FALSE, FALSE, 0);
     } 
 
-    button_box = gtk_hbox_new(FALSE, 5);
-    gtk_box_set_homogeneous(GTK_BOX(button_box), TRUE);
-    gtk_box_pack_start(GTK_BOX(vwin->vbox), button_box, FALSE, FALSE, 0);
-
-    button = gtk_button_new_with_label(_("Close"));
-    gtk_box_pack_start(GTK_BOX(button_box), button, FALSE, TRUE, 0);
-    g_signal_connect(G_OBJECT(button), "clicked",
-		     G_CALLBACK(delete_widget), vwin->w);
-
     /* put stuff into list box(es) */
     if (code == TEXTBOOK_DATA || code == PS_FILES) {
 	err = populate_notebook_filelists(vwin, filebox, code);
@@ -1300,7 +1283,6 @@ gint populate_func_list (windata_t *vwin, int *wid)
     gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter);
 
     /* pick up any function files in system dir */
-
     build_path(fndir, paths.gretldir, "functions", NULL);
     dir = opendir(fndir);
     if (dir != NULL) {
@@ -1310,7 +1292,6 @@ gint populate_func_list (windata_t *vwin, int *wid)
     }
 
     /* pick up any function files in the user's personal dir */
-
     build_path(fndir, paths.userdir, "functions", NULL);
     dir = opendir(fndir);
     if (dir != NULL) {
@@ -1490,6 +1471,7 @@ static GtkWidget *files_window (windata_t *vwin)
 	types = types_5;
 	hidden_col = TRUE;
 	full_width = 560;
+	file_height = 320;
 	break;
     case REMOTE_FUNC_FILES:
 	titles = remote_func_titles;
@@ -1556,6 +1538,7 @@ static GtkWidget *files_notebook (windata_t *vwin, int code)
     build_file_collections(); /* FIXME check for errors */
 
     notebook = gtk_notebook_new();
+    gtk_notebook_set_scrollable(GTK_NOTEBOOK(notebook), TRUE);
 
     j = 0;
 
@@ -1578,7 +1561,7 @@ static GtkWidget *files_notebook (windata_t *vwin, int code)
 	g_object_set_data(G_OBJECT(coll->page), "coll", coll);
 	j++;
     }
-    
+
     if (code == TEXTBOOK_DATA) {
 	reset_data_stack();
     } else {
@@ -1606,6 +1589,7 @@ static int populate_notebook_filelists (windata_t *vwin,
 {
     file_collection *coll;
     const char *title;
+    int gotpref = 0;
     int gotcol = 0;
     int j;
 
@@ -1634,35 +1618,43 @@ static int populate_notebook_filelists (windata_t *vwin,
 	return 1;
     }
 
-    if (code == TEXTBOOK_DATA) {
-	reset_data_stack();
-    } else {
-	reset_ps_stack();
-    }
-
     j = 0;
 
     if (code == TEXTBOOK_DATA) {
+	reset_data_stack();
 	title = get_datapage();
 	if (*title != '\0') {
 	    while ((coll = pop_data_collection())) {
 		if (!strcmp(coll->title, title)) {
+		    gotpref = 1;
 		    break;
 		}
 		j++;
 	    }
+	    if (!gotpref) {
+		reset_data_stack();
+		coll = pop_data_collection();
+		j = 0;
+	    }
 	} else {
 	    coll = pop_data_collection();
 	}
     } else {
+	reset_ps_stack();
 	title = get_scriptpage();
 	if (*title != '\0') {
 	    while ((coll = pop_ps_collection())) {
 		if (!strcmp(coll->title, title)) {
+		    gotpref = 1;
 		    break;
 		}
 		j++;
 	    }
+	    if (!gotpref) {
+		reset_ps_stack();
+		coll = pop_ps_collection();
+		j = 0;
+	    }	    
 	} else {
 	    coll = pop_ps_collection();
 	}
@@ -1677,13 +1669,7 @@ static int populate_notebook_filelists (windata_t *vwin,
     }
 
     gtk_notebook_set_current_page(GTK_NOTEBOOK(notebook), j);
-
     gtk_widget_grab_focus(vwin->listbox);
 
     return 0;
 } 
-
-
-
-
-
