@@ -1,20 +1,20 @@
-/*
- *  Copyright (c) by Allin Cottrell
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
- *
- *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- *
+/* 
+ *  gretl -- Gnu Regression, Econometrics and Time-series Library
+ *  Copyright (C) 2001 Allin Cottrell and Riccardo "Jack" Lucchetti
+ * 
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ * 
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ * 
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 /* dlgutils.c for gretl: utilities for composing dialog boxes */
@@ -26,6 +26,33 @@
 
 #include "system.h"
 
+dialog_opts *dialog_opts_new (int n, int type, 
+			      gretlopt *optp,
+			      const gretlopt *vals,
+			      const char **strs)
+{
+    dialog_opts *opts;
+
+    opts = malloc(sizeof *opts);
+    if (opts == NULL) {
+	nomem();
+	return NULL;
+    }
+
+    opts->n = n;
+    opts->type = type;
+    opts->optp = optp;
+    opts->vals = vals;
+    opts->strs = strs;
+    
+    return opts;
+}
+
+void dialog_opts_free (dialog_opts *opts)
+{
+    free(opts);
+}
+
 void vbox_add_hsep (GtkWidget *vbox)
 {
     GtkWidget *h = gtk_hseparator_new();
@@ -497,7 +524,7 @@ static GtkWidget *dlg_text_edit_new (int *hsize, gboolean s)
     return tview;
 }
 
-static void dlg_text_set_from_sys (gretl_equation_system *sys,
+static void dlg_text_set_from_sys (equation_system *sys,
 				   dialog_t *d)
 {
     const char *buf;
@@ -675,7 +702,7 @@ static void sample_replace_buttons (GtkWidget *box, gpointer data)
 
 static void free_sys_strings (GtkWidget *w, char **strs)
 {
-    free_strings_array(strs,  SYS_MAX - 1); 
+    free_strings_array(strs,  SYS_METHOD_MAX - 1); 
 }
 
 static void set_sys_method (GtkEditable *entry, dialog_t *d)
@@ -688,7 +715,7 @@ static void set_sys_method (GtkEditable *entry, dialog_t *d)
 	s = strrchr(s, '(');
 	if (s != NULL) {
 	    sscanf(s + 1, "%7[^)]", mstr);
-	    d->opt = gretl_system_method_from_string(mstr);
+	    d->opt = system_method_from_string(mstr);
 	}
     } 
 }
@@ -726,6 +753,17 @@ static gboolean opt_b_callback (GtkWidget *w, dialog_t *dlg)
     return FALSE;
 }
 
+static gboolean opt_f_callback (GtkWidget *w, dialog_t *dlg)
+{
+    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w))) {
+	dlg->opt |= OPT_F;
+    } else {
+	dlg->opt &= ~OPT_F;
+    }
+
+    return FALSE;
+}
+
 static void dialog_option_switch (GtkWidget *vbox, dialog_t *dlg,
 				  gretlopt opt)
 {
@@ -743,6 +781,10 @@ static void dialog_option_switch (GtkWidget *vbox, dialog_t *dlg,
 	b = gtk_check_button_new_with_label(_("Use bootstrap"));
 	g_signal_connect(G_OBJECT(b), "toggled", 
 			 G_CALLBACK(opt_b_callback), dlg);
+    } else if (opt == OPT_F) {
+	b = gtk_check_button_new_with_label(_("Show full restricted estimates"));
+	g_signal_connect(G_OBJECT(b), "toggled", 
+			 G_CALLBACK(opt_f_callback), dlg);
     } else {
 	return;
     }
@@ -755,58 +797,86 @@ static void dialog_option_switch (GtkWidget *vbox, dialog_t *dlg,
     gtk_widget_show(hbox);
 }
 
-static void gmm_option_callback (GtkWidget *w, dialog_t *d)
+static void combo_opt_changed (GtkWidget *w, combo_opts *opts)
 {
-    gint i = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(w), "i"));
+    const char *s = gtk_entry_get_text(GTK_ENTRY(w));
+    int i;
 
-    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w))) {
-	switch (i) {
-	case 0:
-	    d->opt &= ~OPT_T;
-	    d->opt &= ~OPT_I;
-	    break;
-	case 1:
-	    d->opt |= OPT_T;
-	    d->opt &= ~OPT_I;
-	    break;
-	case 2:
-	    d->opt &= ~OPT_T;
-	    d->opt |= OPT_I;
-	    break;
-	}
+    if (s != NULL && *s != '\0') {
+        for (i=0; opts->strs[i] != NULL; i++) {
+            if (!strcmp(s, _(opts->strs[i]))) {
+                *opts->optp |= opts->vals[i];
+            } else {
+		*opts->optp &= ~opts->vals[i];
+	    }
+        }
     }
 }
 
-static void build_gmm_radios (GtkWidget *vbox, dialog_t *d)
+GtkWidget *gretl_opts_combo (combo_opts *opts, int deflt)
 {
-    GtkWidget *b, *hbox;
-    GSList *group = NULL;
-    const char *strs[] = {
+    GtkWidget *combo;
+    GList *optlist = NULL;
+    int i, ni, n = 0;
+
+    for (i=0; opts->strs[i] != NULL; i++) {
+	optlist = g_list_append(optlist, _(opts->strs[i]));
+	ni = strlen(_(opts->strs[i]));
+	if (ni > n) {
+	    n = ni;
+	}
+    } 
+
+    *opts->optp |= opts->vals[deflt];
+
+    combo = gtk_combo_new();
+    gtk_combo_set_popdown_strings(GTK_COMBO(combo), optlist); 
+    gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(combo)->entry), 
+		       _(opts->strs[deflt]));
+    gtk_entry_set_width_chars(GTK_ENTRY(GTK_COMBO(combo)->entry), n);
+    gtk_editable_set_editable(GTK_EDITABLE(GTK_COMBO(combo)->entry), 
+			      FALSE);
+    g_signal_connect(G_OBJECT(GTK_COMBO(combo)->entry), "changed",
+		     G_CALLBACK(combo_opt_changed), opts);
+    g_list_free(optlist);
+
+    return combo;
+}
+
+static void build_gmm_combo (GtkWidget *vbox, dialog_t *d)
+{
+    GtkWidget *combo, *hbox;
+    static const char *strs[] = {
 	N_("One-step estimation"),
 	N_("Two-step estimation"),
-	N_("Iterated estimation")
+	N_("Iterated estimation"),
+	NULL
     };
-    int i;
+    static gretlopt opts[] = {
+	OPT_NONE,
+	OPT_T,
+	OPT_I
+    };
+    static combo_opts gmm_opts;
+    int deflt = 0;
 
-    for (i=0; i<3; i++) {
-	b = gtk_radio_button_new_with_label(group, _(strs[i]));
-	g_object_set_data(G_OBJECT(b), "i", GINT_TO_POINTER(i));
-	g_signal_connect(G_OBJECT(b), "toggled", 
-			 G_CALLBACK(gmm_option_callback), d);
-	group = gtk_radio_button_get_group(GTK_RADIO_BUTTON(b));
-	hbox = gtk_hbox_new(FALSE, 5);
-	gtk_box_pack_start(GTK_BOX(hbox), b, TRUE, TRUE, 5);
-	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
-	gtk_widget_show_all(hbox);
-	if (i == 0) {
-	    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(b), TRUE);
-	}
-    }
+    gmm_opts.strs = strs;
+    gmm_opts.vals = opts;
+    gmm_opts.optp = &d->opt;
+
+    combo = gretl_opts_combo(&gmm_opts, deflt);
+
+    hbox = gtk_hbox_new(FALSE, 5);
+    gtk_box_pack_start(GTK_BOX(hbox), combo, FALSE, FALSE, 0); /* 5 ? */
+    gtk_widget_show(combo);
+
+    gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
+    gtk_widget_show(hbox);      
 }
 
 static void system_estimator_list (GtkWidget *vbox, dialog_t *d)
 {
-    gretl_equation_system *sys = NULL;
+    equation_system *sys = NULL;
     GList *items = NULL;
     GtkWidget *w, *hbox;
     gchar **strs;
@@ -814,12 +884,12 @@ static void system_estimator_list (GtkWidget *vbox, dialog_t *d)
     int i;
 
     if (d->data != NULL) {
-	sys = (gretl_equation_system *) d->data;
+	sys = (equation_system *) d->data;
     }
 
-    strs = strings_array_new(SYS_MAX);
+    strs = strings_array_new(SYS_METHOD_MAX);
 
-    for (i=SYS_SUR; i<SYS_MAX; i++) {
+    for (i=SYS_METHOD_SUR; i<SYS_METHOD_MAX; i++) {
 	strs[i] = g_strdup_printf("%s (%s)", _(system_method_full_string(i)),
 				  system_method_short_string(i));
 	items = g_list_append(items, strs[i]);
@@ -853,7 +923,7 @@ static void system_estimator_list (GtkWidget *vbox, dialog_t *d)
 
 static void dlg_display_sys (dialog_t *d)
 {
-    gretl_equation_system *sys = d->data;
+    equation_system *sys = d->data;
     GtkWidget *w;
     int hsize = 62;
 
@@ -926,6 +996,11 @@ static int ols_model_window (windata_t *vwin)
     return 0;
 }
 
+static int vecm_model_window (windata_t *vwin)
+{
+    return vwin->role == VECM;
+}
+
 void edit_dialog (const char *title, const char *info, const char *deflt, 
 		  void (*okfunc)(), void *okptr,
 		  guint cmdcode, guint varclick, 
@@ -991,6 +1066,7 @@ void edit_dialog (const char *title, const char *info, const char *deflt,
     } else {
 	if (info != NULL) {
 	    w = gtk_label_new(info);
+	    gtk_label_set_justify(GTK_LABEL(w), GTK_JUSTIFY_CENTER);
 	    gtk_box_pack_start(GTK_BOX(top_vbox), w, TRUE, TRUE, 5);
 	    gtk_widget_show(w);
 	}
@@ -1024,9 +1100,11 @@ void edit_dialog (const char *title, const char *info, const char *deflt,
 	dialog_option_switch(top_vbox, d, OPT_R);
     } else if (cmdcode == GMM) {
 	dialog_option_switch(top_vbox, d, OPT_V);
-	build_gmm_radios(top_vbox, d);
+	build_gmm_combo(top_vbox, d);
     } else if (cmdcode == RESTRICT && ols_model_window(okptr)) {
 	dialog_option_switch(top_vbox, d, OPT_B);
+    } else if (cmdcode == RESTRICT && vecm_model_window(okptr)) {
+	dialog_option_switch(top_vbox, d, OPT_F);
     }
 
     if (varclick == VARCLICK_INSERT_ID) { 
