@@ -1,20 +1,20 @@
-/*
- *  Copyright (c) by Allin Cottrell
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
- *
- *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- *
+/* 
+ *  gretl -- Gnu Regression, Econometrics and Time-series Library
+ *  Copyright (C) 2001 Allin Cottrell and Riccardo "Jack" Lucchetti
+ * 
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ * 
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ * 
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 #ifndef DLGUTILS_H
@@ -26,6 +26,19 @@ enum {
     GRETL_DLG_RESIZE = 1 << 2
 };
 
+enum {
+    OPT_TYPE_RADIO,
+    OPT_TYPE_COMBO
+};
+
+struct combo_opts_ {
+    gretlopt *optp;
+    gretlopt *vals;
+    const char **strs;
+};
+
+typedef struct combo_opts_ combo_opts;
+
 GtkWidget *gretl_dialog_new (const char *title, GtkWidget *parent,
 			     unsigned char flags);
 
@@ -70,4 +83,13 @@ void close_dialog (dialog_t *dlg);
 
 char *entry_box_get_trimmed_text (GtkWidget *w);
 
+GtkWidget *gretl_opts_combo (combo_opts *opts, int deflt);
+
+dialog_opts *dialog_opts_new (int n, int type, 
+			      gretlopt *optp,
+			      const gretlopt *vals,
+			      const char **strs);
+
+void dialog_opts_free (dialog_opts *opts);
+
 #endif /* DLGUTILS_H */
