@@ -1,20 +1,20 @@
-/*
- *  Copyright (c) by Allin Cottrell
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
- *
- *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- *
+/* 
+ *  gretl -- Gnu Regression, Econometrics and Time-series Library
+ *  Copyright (C) 2001 Allin Cottrell and Riccardo "Jack" Lucchetti
+ * 
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ * 
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ * 
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 /* fileselect.c for gretl -- use the gtk file chooser under X11,
@@ -40,12 +40,12 @@
                               i == OPEN_CSV || \
                               i == OPEN_ASCII || \
                               i == OPEN_OCTAVE || \
-	                      i == OPEN_BOX || \
                               i == OPEN_GNUMERIC || \
 	                      i == OPEN_EXCEL || \
                               i == OPEN_WF1 || \
                               i == OPEN_DTA || \
-                              i == OPEN_JMULTI)
+                              i == OPEN_JMULTI || \
+                              i == OPEN_ODS)
 
 #define APPEND_DATA_ACTION(i) (i == APPEND_DATA || \
                                i == APPEND_CSV || \
@@ -55,7 +55,8 @@
                                i == APPEND_ASCII || \
                                i == APPEND_WF1 || \
                                i == APPEND_DTA || \
-                               i == APPEND_JMULTI)
+                               i == APPEND_JMULTI || \
+                               i == APPEND_ODS)
 
 #define SAVE_GRAPH_ACTION(i) (i == SAVE_GNUPLOT || \
                               i == SAVE_BOXPLOT_EPS || \
@@ -104,7 +105,6 @@ static struct extmap action_map[] = {
     { APPEND_CSV,        ".csv" },
     { OPEN_ASCII,        ".txt" },
     { APPEND_ASCII,      ".txt" },
-    { OPEN_BOX,          ".box" },
     { OPEN_GNUMERIC,     ".gnumeric" },
     { APPEND_GNUMERIC,   ".gnumeric" },
     { OPEN_EXCEL,        ".xls" },
@@ -115,6 +115,8 @@ static struct extmap action_map[] = {
     { APPEND_DTA,        ".dta" },
     { OPEN_JMULTI,       ".dat" },
     { APPEND_JMULTI,     ".dat" },
+    { OPEN_ODS,          ".ods" },
+    { APPEND_ODS,        ".ods" },
     { OPEN_RATS_DB,      ".rat" },
     { OPEN_PCGIVE_DB,    ".bn7" },
     { FILE_OP_MAX,       NULL }
@@ -240,15 +242,25 @@ static int check_maybe_add_ext (char *fname, int action, gpointer data)
 
 static void script_window_update (windata_t *vwin, const char *fname)
 {
-    gchar *title;
+    gchar *trfname, *title;
     const char *p = strrchr(fname, SLASH);
 
+    /* ensure UTF-8 filename for display */
+    if (p != NULL) {
+	trfname = my_filename_to_utf8(p + 1);
+    } else {
+	trfname = my_filename_to_utf8(fname);
+    }
+
     /* update the window title */
-    title = g_strdup_printf("gretl: %s", p? p + 1 : fname);
+    title = g_strdup_printf("gretl: %s", trfname);
     gtk_window_set_title(GTK_WINDOW(vwin->dialog), title);
-    strcpy(vwin->fname, fname);
+    g_free(trfname);
     g_free(title);
 
+    /* and update internal filename record */
+    strcpy(vwin->fname, fname);
+
     if (vwin->role == VIEW_LOG || vwin->role == VIEW_SCRIPT ||
 	vwin->role == VIEW_FUNC_CODE) {
 	/* change role of window for editing */
@@ -280,7 +292,7 @@ save_editable_content (int action, const char *fname, windata_t *vwin)
 
     fp = gretl_fopen(fname, "w");
     if (fp == NULL) {
-	errbox(_("Couldn't open file for writing"));
+	file_write_errbox(fname);
 	g_free(buf);
 	return;
     }
@@ -321,7 +333,7 @@ static void filesel_save_prn_buffer (PRN *prn, const char *fname)
     FILE *fp = gretl_fopen(fname, "w");
 
     if (fp == NULL) {
-	errbox(_("Couldn't write to %s"), fname);
+	file_write_errbox(fname);
     } else {
 	const char *buf = gretl_print_get_buffer(prn);
 
@@ -434,9 +446,7 @@ static void bootstrap_save_callback (const char *fname)
 
     if (err) {
 	gui_errmsg(err);
-    } else {
-	infobox(_("Saved data as %s"), fname);
-    }
+    } 
 }
 
 static void
@@ -452,7 +462,7 @@ file_selector_process_result (const char *in_fname, int action, FselDataSrc src,
 	FILE *fp = gretl_fopen(fname, "r");
 
 	if (fp == NULL) {
-	    errbox(_("Couldn't open %s"), fname);
+	    file_read_errbox(fname);
 	    return;
 	} else {
 	    fclose(fp);
@@ -608,7 +618,6 @@ static struct winfilter get_filter (int action, gpointer data)
 	{ APPEND_CSV,   { N_("CSV files (*.csv)"), "*.csv" }},
 	{ OPEN_ASCII,   { N_("ASCII files (*.txt)"), "*.txt" }},
 	{ APPEND_ASCII, { N_("ASCII files (*.txt)"), "*.txt" }},
-	{ OPEN_BOX,     { N_("BOX data files (*.box)"), "*.box" }},
 	{ OPEN_GNUMERIC,   { N_("Gnumeric files (*.gnumeric)"), "*.gnumeric" }},
 	{ APPEND_GNUMERIC, { N_("Gnumeric files (*.gnumeric)"), "*.gnumeric" }},
 	{ OPEN_EXCEL,   { N_("Excel files (*.xls)"), "*.xls" }},
@@ -619,6 +628,8 @@ static struct winfilter get_filter (int action, gpointer data)
 	{ APPEND_DTA,   { N_("Stata files (*.dta)"), "*.dta" }},
 	{ OPEN_JMULTI,  { N_("JMulTi files (*.dat)"), "*.dat" }},
 	{ APPEND_JMULTI,{ N_("JMulTi files (*.dat)"), "*.dat" }},
+	{ OPEN_ODS,     { N_("Open Document Spreadsheet (*.ods)"), "*.ods" }},
+	{ APPEND_ODS,   { N_("Open Document Spreadsheet (*.ods)"), "*.ods" }},
 	{ OPEN_RATS_DB, { N_("RATS databases (*.rat)"), "*.rat" }},
 	{ OPEN_PCGIVE_DB, { N_("PcGive data files (*.bn7)"), "*.bn7" }},
 	{ SET_PROG,     { N_("program files (*.exe)"), "*.exe" }}
@@ -796,8 +807,6 @@ void file_selector (const char *msg, int action, FselDataSrc src, gpointer data)
 	return;
     }
 
-    my_filename_to_utf8(fname);
-
     file_selector_process_result(fname, action, src, data);
 }
 
