@@ -1,20 +1,20 @@
-/*
- *  Copyright (c) by Allin Cottrell
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
- *
- *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- *
+/* 
+ *  gretl -- Gnu Regression, Econometrics and Time-series Library
+ *  Copyright (C) 2001 Allin Cottrell and Riccardo "Jack" Lucchetti
+ * 
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ * 
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ * 
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 /* filters.c : for gretl */
@@ -111,7 +111,7 @@ static void filter_info_init (filter_info *finfo, int ftype, int v,
     } else if (ftype == FILTER_EMA) {
 	finfo->lambda = 0.1;
     } else if (ftype == FILTER_HP) {
-	finfo->lambda = get_hp_lambda();
+	finfo->lambda = libset_get_double(HP_LAMBDA);
 	if (na(finfo->lambda)) {
 	    finfo->lambda = 100 * datainfo->pd * datainfo->pd;
 	}
@@ -638,8 +638,8 @@ do_filter_graph (filter_info *finfo, const double *fx, const double *u)
 	if (zkeypos == 'L') {
 	    fputs("set key top left\n", fp);
 	}
-	sprintf(xtitle, I_("%s (original data)"), datainfo->varname[v]);
-	sprintf(ztitle, I_("%s (smoothed)"), datainfo->varname[v]);
+	sprintf(xtitle, G_("%s (original data)"), datainfo->varname[v]);
+	sprintf(ztitle, G_("%s (smoothed)"), datainfo->varname[v]);
 	fprintf(fp, "plot '-' using 1:2 title '%s' w lines, \\\n"
 		" '-' using 1:2 title '%s' w lines\n", xtitle, ztitle);
 	print_gp_data(finfo, obs, Z[v], fp);
@@ -650,7 +650,7 @@ do_filter_graph (filter_info *finfo, const double *fx, const double *u)
 	fputs("set size 1.0,0.38\n", fp);
 	fputs("set origin 0.0,0.0\n", fp);
 	fputs("set xzeroaxis\n", fp);
-	sprintf(title, I_("Cyclical component of %s"), datainfo->varname[v]);
+	sprintf(title, G_("Cyclical component of %s"), datainfo->varname[v]);
 	fprintf(fp, "plot '-' using 1:2 title '%s' w lines\n", title);
 	print_gp_data(finfo, obs, u, fp);
 	fputs("e\n", fp);
@@ -659,8 +659,8 @@ do_filter_graph (filter_info *finfo, const double *fx, const double *u)
 	if (zkeypos == 'L') {
 	    fputs("set key top left\n", fp);
 	}
-	sprintf(xtitle, I_("%s (original data)"), datainfo->varname[v]);
-	sprintf(ztitle, I_("%s (smoothed)"), datainfo->varname[v]);
+	sprintf(xtitle, G_("%s (original data)"), datainfo->varname[v]);
+	sprintf(ztitle, G_("%s (smoothed)"), datainfo->varname[v]);
 	fprintf(fp, "plot '-' using 1:2 title '%s' w lines, \\\n"
 		" '-' using 1:2 title '%s' w lines\n", xtitle, ztitle);
 	print_gp_data(finfo, obs, Z[v], fp);
@@ -669,10 +669,10 @@ do_filter_graph (filter_info *finfo, const double *fx, const double *u)
 	fputs("e\n", fp);
     } else if (finfo->graph_opt & FILTER_GRAPH_CYCLE) {
 	if (finfo->ftype == FILTER_BK) {
-	    sprintf(title, I_("Baxter-King component of %s at frequency %d to %d"), 
+	    sprintf(title, G_("Baxter-King component of %s at frequency %d to %d"), 
 		    datainfo->varname[v], finfo->bkl, finfo->bku);
 	} else {
-	    sprintf(title, I_("Cyclical component of %s"), datainfo->varname[v]);
+	    sprintf(title, G_("Cyclical component of %s"), datainfo->varname[v]);
 	}
 	fprintf(fp, "set title '%s'\n", title); 
 	fputs("set xzeroaxis\n", fp);
@@ -758,6 +758,7 @@ static int sma_special (const filter_info *finfo, double *fx,
 	k = t - t1;
 	fx[t] = (tmp[k] + tmp[k+1]) / 2.0;
     }
+
     fx[t2] = NADBL;
 
     free(tmp);
@@ -825,11 +826,11 @@ static int calculate_filter (filter_info *finfo)
 	    fx[t] = finfo->lambda * x[t] + (1.0 - finfo->lambda) * fx[t-1];
 	}
     } else if (finfo->ftype == FILTER_HP) {
-	double l = get_hp_lambda();
+	double l0 = libset_get_double(HP_LAMBDA);
 
-	set_hp_lambda(finfo->lambda);
+	libset_set_double(HP_LAMBDA, finfo->lambda);
 	err = hp_filter(x, fx, datainfo, OPT_T);
-	set_hp_lambda(l);
+	libset_set_double(HP_LAMBDA, l0);
     } else if (finfo->ftype == FILTER_BK) {
 	set_bkbp_k(finfo->k);
 	set_bkbp_periods(finfo->bkl, finfo->bku);
