@@ -1,20 +1,20 @@
-/*
- *  Copyright (c) by Allin Cottrell
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
- *
- *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- *
+/* 
+ *  gretl -- Gnu Regression, Econometrics and Time-series Library
+ *  Copyright (C) 2001 Allin Cottrell and Riccardo "Jack" Lucchetti
+ * 
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ * 
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ * 
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 #include "gretl.h"
@@ -25,6 +25,7 @@
 #include "usermat.h"
 #include "cmd_private.h"
 #include "gretl_www.h"
+#include "gretl_string_table.h"
 #include "database.h"
 #include "guiprint.h"
 #include "ssheet.h"
@@ -35,8 +36,9 @@ typedef struct call_info_ call_info;
 
 struct call_info_ {
     GtkWidget *dlg;
-    GList *lsels;
-    GList *msels;
+    GList *lsels; /* list arg selectors */
+    GList *msels; /* matrix arg selectors */
+    GList *ssels; /* string arg selectors */
     int iface;
     int extracol;
     const ufunc *func;
@@ -47,6 +49,10 @@ struct call_info_ {
     int ok;
 };
 
+#define scalar_arg(t) (t == GRETL_TYPE_DOUBLE || t == GRETL_TYPE_SCALAR_REF)
+#define series_arg(t) (t == GRETL_TYPE_SERIES || t == GRETL_TYPE_SERIES_REF)
+#define matrix_arg(t) (t == GRETL_TYPE_MATRIX || t == GRETL_TYPE_MATRIX_REF)
+
 static call_info *cinfo_new (void)
 {
     call_info *cinfo = mymalloc(sizeof *cinfo);
@@ -63,7 +69,7 @@ static call_info *cinfo_new (void)
     cinfo->func = NULL;
     cinfo->n_params = 0;
 
-    cinfo->rettype = ARG_NONE;
+    cinfo->rettype = GRETL_TYPE_NONE;
 
     cinfo->args = NULL;
     cinfo->ret = NULL;
@@ -93,26 +99,37 @@ static int cinfo_args_init (call_info *cinfo)
 
 static void cinfo_free (call_info *cinfo)
 {
-    free_strings_array(cinfo->args, cinfo->n_params);
-    free(cinfo->ret);
-    g_list_free(cinfo->lsels);
-    g_list_free(cinfo->msels);
+    if (cinfo->n_params > 0) {
+	free_strings_array(cinfo->args, cinfo->n_params);
+    }
+    if (cinfo->ret != NULL) {
+	free(cinfo->ret);
+    }
+    if (cinfo->lsels != NULL) {
+	g_list_free(cinfo->lsels);
+    }
+    if (cinfo->msels != NULL) {
+	g_list_free(cinfo->msels);
+    }
+
     free(cinfo);
 }
 
-#define scalar_type(t) (t == ARG_SCALAR || t == ARG_REF_SCALAR)
-#define series_type(t) (t == ARG_SERIES || t == ARG_REF_SERIES)
-#define matrix_type(t) (t == ARG_MATRIX || t == ARG_REF_MATRIX)
-
-static const char *arg_type_string (int type)
+static const char *arg_type_string (int t)
 {
-    if (type == ARG_BOOL)   return "boolean";
-    if (type == ARG_INT)    return "int";
-    if (type == ARG_LIST)   return "list";
+    if (t == GRETL_TYPE_BOOL)   return "boolean";
+    if (t == GRETL_TYPE_INT)    return "int";
+    if (t == GRETL_TYPE_LIST)   return "list";
+
+    if (t == GRETL_TYPE_DOUBLE) return "scalar";
+    if (t == GRETL_TYPE_SERIES) return "series";
+    if (t == GRETL_TYPE_MATRIX) return "matrix";
     
-    if (scalar_type(type)) return "scalar";
-    if (series_type(type)) return "series";
-    if (matrix_type(type)) return "matrix";
+    if (t == GRETL_TYPE_SCALAR_REF) return "scalar *";
+    if (t == GRETL_TYPE_SERIES_REF) return "series *";
+    if (t == GRETL_TYPE_MATRIX_REF) return "matrix *";
+
+    if (t == GRETL_TYPE_STRING) return "string";
 
     return "";
 }
@@ -205,7 +222,7 @@ static gboolean update_arg (GtkEditable *entry,
     free(cinfo->args[i]);
     s = cinfo->args[i] = entry_box_get_trimmed_text(GTK_WIDGET(entry));
 
-    if (s != NULL && fn_param_type(cinfo->func, i) == ARG_SCALAR) {
+    if (s != NULL && fn_param_type(cinfo->func, i) == GRETL_TYPE_DOUBLE) {
 	if (isdigit(*s) || *s == '-' || *s == '+' || *s == ',') {
 	    charsub(s, ',', '.');
 	}
@@ -223,7 +240,8 @@ static gboolean update_return (GtkEditable *entry,
     return FALSE;
 }
 
-static GList *get_selection_list (call_info *cinfo, int i, int type)
+static GList *get_selection_list (call_info *cinfo, int i, int type,
+				  int set_default)
 {
     GList *list = NULL;
     const char *name;
@@ -235,20 +253,24 @@ static GList *get_selection_list (call_info *cinfo, int i, int type)
 	optional = fn_param_optional(cinfo->func, i);
     }
 
-    if (series_type(type) || scalar_type(type)) {
+    if (!set_default) {
+	list = g_list_append(list, "");
+    }
+
+    if (series_arg(type) || scalar_arg(type)) {
 	for (i=1; i<datainfo->v; i++) {
 	    if (var_is_hidden(datainfo, i)) {
 		continue;
 	    }
-	    if ((series_type(type) && var_is_series(datainfo, i)) ||
-		(scalar_type(type) && var_is_scalar(datainfo, i))) {
+	    if ((series_arg(type) && var_is_series(datainfo, i)) ||
+		(scalar_arg(type) && var_is_scalar(datainfo, i))) {
 		list = g_list_append(list, (gpointer) datainfo->varname[i]);
 	    } 
 	}
-	if (type == ARG_SERIES) {
+	if (type == GRETL_TYPE_SERIES) {
 	    list = g_list_append(list, (gpointer) datainfo->varname[0]);
 	}
-    } else if (type == ARG_LIST) {
+    } else if (type == GRETL_TYPE_LIST) {
 	int nl = n_saved_lists();
 
 	if (optional) {
@@ -259,16 +281,16 @@ static GList *get_selection_list (call_info *cinfo, int i, int type)
 	    name = get_list_name_by_index(i);
 	    list = g_list_append(list, (gpointer) name);
 	}
-    } else if (matrix_type(type)) {
+    } else if (matrix_arg(type)) {
 	int nm = n_user_matrices();
 
 	for (i=0; i<nm; i++) {
 	    name = get_matrix_name_by_index(i);
 	    list = g_list_append(list, (gpointer) name);
 	}	
-    }
+    } 
 
-    if (optional && type != ARG_LIST) {
+    if (optional && type != GRETL_TYPE_LIST) {
 	list = g_list_append(list, "null");
     }
 
@@ -498,7 +520,7 @@ static GtkWidget *combo_arg_selector (call_info *cinfo, int ptype, int i)
     gtk_entry_set_activates_default(GTK_ENTRY(GTK_COMBO(combo)->entry), 
 				    TRUE);
 
-    list = get_selection_list(cinfo, i, ptype);
+    list = get_selection_list(cinfo, i, ptype, 1);
     if (list != NULL) {
 	gtk_combo_set_popdown_strings(GTK_COMBO(combo), list);
 	g_list_free(list);
@@ -506,7 +528,7 @@ static GtkWidget *combo_arg_selector (call_info *cinfo, int ptype, int i)
 
     /* FIXME bool etc */
 
-    if (ptype == ARG_INT || ptype == ARG_SCALAR) {
+    if (ptype == GRETL_TYPE_INT || ptype == GRETL_TYPE_DOUBLE) {
 	double x = fn_param_default(cinfo->func, i);
 
 	if (!na(x)) {
@@ -584,9 +606,9 @@ static void function_call_dialog (call_info *cinfo)
 			     GTK_EXPAND, GTK_FILL, 5, 5);
 	    gtk_widget_show(label);
 
-	    if (ptype == ARG_BOOL) {
+	    if (ptype == GRETL_TYPE_BOOL) {
 		sel = bool_arg_selector(cinfo, i);
-	    } else if (ptype == ARG_INT && spinner_arg(cinfo, i)) {
+	    } else if (ptype == GRETL_TYPE_INT && spinner_arg(cinfo, i)) {
 		sel = spin_arg_selector(cinfo, i);
 	    } else {
 		sel = combo_arg_selector(cinfo, ptype, i);
@@ -596,7 +618,7 @@ static void function_call_dialog (call_info *cinfo)
 			     GTK_EXPAND, GTK_FILL, 5, 5);
 	    gtk_widget_show(sel);
 
-	    if (ptype == ARG_LIST) {
+	    if (ptype == GRETL_TYPE_LIST) {
 		cinfo->lsels = g_list_append(cinfo->lsels, sel);
 		button = gtk_button_new_with_label(_("More..."));
 		gtk_table_attach(GTK_TABLE(tbl), button, 3, 4, i+1, i+2,
@@ -605,7 +627,7 @@ static void function_call_dialog (call_info *cinfo)
 				 G_CALLBACK(launch_list_maker), 
 				 GTK_COMBO(sel)->entry);
 		gtk_widget_show(button);
-	    } else if (ptype == ARG_MATRIX) {
+	    } else if (ptype == GRETL_TYPE_MATRIX) {
 		cinfo->msels = g_list_append(cinfo->msels, sel);
 		button = gtk_button_new_with_label(_("New..."));
 		gtk_table_attach(GTK_TABLE(tbl), button, 3, 4, i+1, i+2,
@@ -614,13 +636,13 @@ static void function_call_dialog (call_info *cinfo)
 				 G_CALLBACK(launch_matrix_maker), 
 				 cinfo);
 		gtk_widget_show(button);
-	    }		
+	    } 
 	}
 
 	gtk_widget_show(tbl);
     }
 
-    if (cinfo->n_params > 0 && cinfo->rettype != ARG_NONE) {
+    if (cinfo->n_params > 0 && cinfo->rettype != GRETL_TYPE_NONE) {
 	GtkWidget *hsep = gtk_hseparator_new();
 
 	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(cinfo->dlg)->vbox),
@@ -630,7 +652,7 @@ static void function_call_dialog (call_info *cinfo)
 
     /* function return(s) assignment */
 
-    if (cinfo->rettype != ARG_NONE) {
+    if (cinfo->rettype != GRETL_TYPE_NONE) {
 	GList *list = NULL;
 
 	hbox = label_hbox(GTK_DIALOG(cinfo->dlg)->vbox, 
@@ -660,7 +682,7 @@ static void function_call_dialog (call_info *cinfo)
 	sel = gtk_combo_new();
 	g_signal_connect(G_OBJECT(GTK_COMBO(sel)->entry), "changed",
 			 G_CALLBACK(update_return), cinfo);
-	list = get_selection_list(cinfo, -1, cinfo->rettype);
+	list = get_selection_list(cinfo, -1, cinfo->rettype, 0);
 	if (list != NULL) {
 	    gtk_combo_set_popdown_strings(GTK_COMBO(sel), list); 
 	    g_list_free(list);
@@ -687,7 +709,7 @@ static void function_call_dialog (call_info *cinfo)
     gtk_widget_show(button);
 
     /* Help button? */
-    if (cinfo->n_params > 0 || cinfo->rettype != ARG_NONE) {
+    if (cinfo->n_params > 0 || cinfo->rettype != GRETL_TYPE_NONE) {
 	button = context_help_button(GTK_DIALOG(cinfo->dlg)->action_area, -1);
 	g_signal_connect(G_OBJECT(button), "clicked", 
 			 G_CALLBACK(fncall_help), cinfo);
@@ -705,24 +727,24 @@ static int function_data_check (call_info *cinfo)
        it doesn't need data loaded? */
 
     if (datainfo == NULL || datainfo->v == 0) {
-	errbox(_("Please open a data file first"));
+	warnbox(_("Please open a data file first"));
 	return 1;
     }
 
     for (i=0; i<cinfo->n_params; i++) {
 	int type = fn_param_type(cinfo->func, i);
 
-	if (type == ARG_SERIES || type == ARG_LIST ||
-	    type == ARG_REF_SERIES) {
+	if (type == GRETL_TYPE_SERIES || type == GRETL_TYPE_LIST ||
+	    type == GRETL_TYPE_SERIES_REF) {
 	    if (datainfo == NULL || datainfo->v == 0) {
-		errbox(_("Please open a data file first"));
+		warnbox(_("Please open a data file first"));
 		err = 1;
 		break;
 	    }
 	}
-	if (type == ARG_LIST) {
+	if (type == GRETL_TYPE_LIST) {
 	    cinfo->extracol = 1;
-	} else if (type == ARG_MATRIX || type == ARG_REF_MATRIX) {
+	} else if (type == GRETL_TYPE_MATRIX || type == GRETL_TYPE_MATRIX_REF) {
 	    cinfo->extracol = 1;
 	}
     }
@@ -749,7 +771,7 @@ static int temp_install_remote_fnpkg (const char *fname, char *target)
     err = load_user_function_file(target);
     if (err) {
 	fprintf(stderr, "load_user_function_file: failed on %s\n", target);
-	errbox(_("Couldn't open %s"), target);
+	file_read_errbox(target);
     }
 
     if (err) {
@@ -759,31 +781,100 @@ static int temp_install_remote_fnpkg (const char *fname, char *target)
     return err;
 }
 
-static int addressify_var (call_info *cinfo, int i)
+/* detect the case where we need a "pointer" variable but
+   have been given a scalar or matrix constant 
+*/
+
+static int should_addressify_var (call_info *cinfo, int i)
 {
-    char *numchars = "0123456789+-.";
+    char *numchars = "0123456789+-.,";
     int t = fn_param_type(cinfo->func, i);
     char *s = cinfo->args[i];
 
-    return (t == ARG_REF_SCALAR && strchr(numchars, *s)) ||
-	(t == ARG_REF_MATRIX && *s == '{');
+    return (t == GRETL_TYPE_SCALAR_REF && strchr(numchars, *s)) ||
+	(t == GRETL_TYPE_MATRIX_REF && *s == '{');
 }
 
-static int needs_amp (call_info *cinfo, int i)
+static int maybe_add_amp (call_info *cinfo, int i, PRN *prn, int *add)
+{
+    int t = fn_param_type(cinfo->func, i);
+    char *s = cinfo->args[i];
+    int err = 0;
+
+    *add = 0;
+
+    if (!gretl_ref_type(t)) {
+	return 0;
+    }
+
+    if (*s == '&' || !strcmp(s, "null")) {
+	return 0;
+    }
+
+    if (t == GRETL_TYPE_MATRIX_REF) {
+	/* handle case where indirect return matrix does not yet exist */
+	if (get_matrix_by_name(s) == NULL) {
+	    gretl_matrix *m = gretl_null_matrix_new();
+
+	    if (m == NULL) {
+		err = E_ALLOC;
+	    } else {
+		err = add_or_replace_user_matrix(m, s);
+	    }
+	    if (!err) {
+		pprintf(prn, "? matrix %s\n", s);
+	    }
+	}
+    } 
+
+    if (!err) {
+	*add = 1;
+    }
+
+    return err;
+}
+
+static int needs_quoting (call_info *cinfo, int i)
 {
     int t = fn_param_type(cinfo->func, i);
     char *s = cinfo->args[i];
 
-    if (*s == '&') {
-	return 0;
+    return (t == GRETL_TYPE_STRING && 
+	    get_named_string(s) == NULL &&
+	    *s != '"');
+}
+
+static int pre_process_args (call_info *cinfo, PRN *prn)
+{
+    char auxline[MAXLINE];
+    char auxname[VNAMELEN+2];
+    int i, add = 0, err = 0;
+
+    for (i=0; i<cinfo->n_params && !err; i++) {
+	if (should_addressify_var(cinfo, i)) {
+	    sprintf(auxname, "FNARG%d", i + 1);
+	    sprintf(auxline, "genr %s=%s", auxname, cinfo->args[i]);
+	    err = generate(auxline, &Z, datainfo, OPT_NONE, NULL);
+	    if (!err) {
+		free(cinfo->args[i]);
+		cinfo->args[i] = g_strdup(auxname);
+		pprintf(prn, "? %s\n", auxline);
+	    } 
+	} 
+	err = maybe_add_amp(cinfo, i, prn, &add);
+	if (add) {
+	    strcpy(auxname, "&");
+	    strncat(auxname, cinfo->args[i], VNAMELEN);
+	    free(cinfo->args[i]);
+	    cinfo->args[i] = g_strdup(auxname);
+	} else if (needs_quoting(cinfo, i)) {
+	    sprintf(auxname, "\"%s\"", cinfo->args[i]);
+	    free(cinfo->args[i]);
+	    cinfo->args[i] = g_strdup(auxname);
+	}	
     }
 
-    if (t != ARG_REF_SCALAR && t != ARG_REF_SERIES &&
-	t != ARG_REF_MATRIX) {
-	return 0;
-    }
-
-    return 1;
+    return err;
 }
 
 void call_function_package (const char *fname, GtkWidget *w,
@@ -792,12 +883,12 @@ void call_function_package (const char *fname, GtkWidget *w,
     ExecState state;
     char tmpfile[FILENAME_MAX];
     char fnline[MAXLINE];
-    char auxline[MAXLINE];
     const char *fnname;
     FuncDataReq dreq;
     float minver;
-    PRN *prn;
-    call_info *cinfo;
+    PRN *prn = NULL;
+    call_info *cinfo = NULL;
+    int orig_v;
     int i, err = 0;
 
     *tmpfile = 0;
@@ -809,8 +900,7 @@ void call_function_package (const char *fname, GtkWidget *w,
 	if (!function_package_is_loaded(fname)) {
 	    err = load_user_function_file(fname);
 	    if (err) {
-		fprintf(stderr, "load_user_function_file: failed on %s\n", fname);
-		errbox(_("Couldn't open %s"), fname);
+		file_read_errbox(fname);
 		*loaderr = 1;
 	    }
 	}
@@ -843,15 +933,13 @@ void call_function_package (const char *fname, GtkWidget *w,
 
     if (cinfo->iface < 0) {
 	errbox(_("Function package is broken"));
-	free(cinfo);
-	return;
+	goto bailout;
     }
 
     err = check_function_needs(datainfo, dreq, minver);
     if (err) {
 	gui_errmsg(err);
-	free(cinfo);
-	return;
+	goto bailout;
     }
 
     cinfo->func = get_user_function_by_index(cinfo->iface);
@@ -860,15 +948,13 @@ void call_function_package (const char *fname, GtkWidget *w,
 	fprintf(stderr, "get_user_function_by_index: got NULL for idx = %d\n", 
 		cinfo->iface);
 	errbox(_("Couldn't get function package information"));
-	free(cinfo);
-	return;
+	goto bailout;
     }
 
     cinfo->n_params = fn_n_params(cinfo->func);
 
     if (function_data_check(cinfo)) {
-	cinfo_free(cinfo);
-	return;
+	goto bailout;
     }
 
     cinfo->rettype = user_func_get_return_type(cinfo->func);
@@ -877,14 +963,13 @@ void call_function_package (const char *fname, GtkWidget *w,
 	fprintf(stderr, "user_func_get_return_types: failed for idx = %d\n",
 		cinfo->iface);
 	errbox(_("Couldn't get function package information"));
-	return;
+	goto bailout;
     }
 
     function_call_dialog(cinfo);
 
-    if (!cinfo->ok || check_args(cinfo)) {
-	cinfo_free(cinfo);
-	return;
+    if (!cinfo->ok || check_args(cinfo) || bufopen(&prn)) {
+	goto bailout;
     }
 
 #if 0
@@ -892,6 +977,14 @@ void call_function_package (const char *fname, GtkWidget *w,
     gtk_widget_destroy(w);
 #endif
 
+    if (cinfo->args != NULL) {
+	err = pre_process_args(cinfo, prn);
+	if (err) {
+	    gui_errmsg(err);
+	    goto bailout;
+	}
+    }
+
     fnname = user_function_name_by_index(cinfo->iface);
     *fnline = 0;
 
@@ -901,60 +994,41 @@ void call_function_package (const char *fname, GtkWidget *w,
     }    
 
     strcat(fnline, fnname);
+    orig_v = datainfo->v;
+
+    strcat(fnline, "(");
 
     if (cinfo->args != NULL) {
-	strcat(fnline, "(");
 	for (i=0; i<cinfo->n_params; i++) {
-	    char auxname[VNAMELEN];
-
-	    if (addressify_var(cinfo, i)) {
-		sprintf(auxname, "FNARG%d", i + 1);
-		sprintf(auxline, "genr %s=%s", auxname, cinfo->args[i]);
-		err = generate(auxline, &Z, datainfo, OPT_NONE, NULL);
-		if (!err) {
-		    free(cinfo->args[i]);
-		    cinfo->args[i] = g_strdup(auxname);
-		}
-	    } 
-	    if (needs_amp(cinfo, i)) {
-		strcat(fnline, "&");
-	    }
 	    strcat(fnline, cinfo->args[i]);
 	    if (i < cinfo->n_params - 1) {
 		strcat(fnline, ", ");
 	    }
 	}
-	strcat(fnline, ")");
-    } else {
-	strcat(fnline, "()");
-    }
+    } 
 
-    /* FIXME destroy any "ARG" vars or matrices that were created? */
+    /* destroy any "ARG" vars or matrices that were created? */
 
-    cinfo_free(cinfo);
-
-    if (bufopen(&prn)) {
-	return;
-    }
-
-#if FCDEBUG
-    fprintf(stderr, "fnline: '%s'\n", fnline);
-#endif
+    strcat(fnline, ")");
+    pprintf(prn, "? %s\n", fnline);
 
     gretl_exec_state_init(&state, SCRIPT_EXEC, fnline, get_lib_cmd(),
 			  models, prn);
 
     err = gui_exec_line(&state, &Z, &datainfo);
+    view_buffer(prn, 80, 400, fnname, SCRIPT_OUT, NULL);
 
-    if (err) {
-	const char *msg = get_funcerr_message();
-
-	if (*msg != 0) {
-	    errbox(msg);
-	} else {
-	    gui_errmsg(err);
-	}
-    } else {
-	view_buffer(prn, 80, 400, fnname, PRINT, NULL);
+    if (datainfo->v > orig_v) {
+	mark_dataset_as_modified();
+	populate_varlist();
     }
+
+    cinfo_free(cinfo);
+
+    return;
+
+ bailout:
+
+    cinfo_free(cinfo);
+    gretl_print_destroy(prn);
 }
