@@ -1,20 +1,20 @@
-/*
- *  Copyright (c) by Allin Cottrell
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
- *
- *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- *
+/* 
+ *  gretl -- Gnu Regression, Econometrics and Time-series Library
+ *  Copyright (C) 2001 Allin Cottrell and Riccardo "Jack" Lucchetti
+ * 
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ * 
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ * 
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 /* gpt_control.c for gretl -- gnuplot controller */
@@ -26,6 +26,7 @@
 #include "gpt_dialog.h"
 #include "fileselect.h"
 #include "calculator.h"
+#include "guiprint.h"
 
 #define GPDEBUG 0
 #define POINTS_DEBUG 0
@@ -130,10 +131,11 @@ struct png_plot_t {
     unsigned char format;
 };
 
-static void render_pngfile (png_plot *plot, int view);
+static int render_pngfile (png_plot *plot, int view);
 static int zoom_unzoom_png (png_plot *plot, int view);
 static void create_selection_gc (png_plot *plot);
 static int get_plot_ranges (png_plot *plot);
+static void graph_display_pdf (GPT_SPEC *spec);
 
 #ifdef PNG_COMMENTS
 
@@ -189,6 +191,11 @@ GtkWidget *plot_get_shell (png_plot *plot)
     return plot->shell;
 }
 
+GPT_SPEC *plot_get_spec (png_plot *plot) 
+{
+    return plot->spec;
+}
+
 int plot_is_mouseable (const png_plot *plot)
 {
     return !(plot->status & PLOT_DONT_MOUSE);
@@ -225,19 +232,30 @@ void plot_label_position_click (GtkWidget *w, png_plot *plot)
 
 static void line_to_file (const char *s, FILE *fp, int lv)
 {
+    int done = 0;
+
 #ifdef ENABLE_NLS
-    if (lv == -2) {
-	fprintf(stderr, "line_to_file: lv=-2, doing print_as_html\n");
-	print_as_html(s, fp);
-    } else if (lv == 2) {
-	fprintf(stderr, "line_to_file: lv=2, doing print_as_locale\n");
-	print_as_locale(s, fp);
-    } else {
+    static int pngterm = -1;
+
+    if (lv == -2 || lv == 2) {
+	if (pngterm < 0) {
+	    pngterm = gnuplot_png_terminal();
+	}
+	if (pngterm != GP_PNG_CAIRO) {
+	    if (lv == -2) {
+		print_as_html(s, fp);
+		done = 1;
+	    } else if (lv == 2) {
+		print_as_locale(s, fp);
+		done = 1;
+	    }
+	}
+    } 
+#endif
+
+    if (!done) {
 	fputs(s, fp);
     }
-#else
-    fputs(s, fp);
-#endif
 }
 
 static FILE *open_gp_file (const char *fname, const char *mode)
@@ -246,9 +264,9 @@ static FILE *open_gp_file (const char *fname, const char *mode)
 
     if (fp == NULL) {
 	if (*mode == 'w') {
-	    errbox(_("Couldn't write to %s"), fname);
+	    file_write_errbox(fname);
 	} else {
-	    errbox(_("Couldn't open %s"), fname);
+	    file_read_errbox(fname);
 	}
     }
 
@@ -265,13 +283,25 @@ static int set_output_line (const char *s)
     return !strncmp(s, "set output", 10);
 }
 
+static int set_print_line (const char *s)
+{
+    return (!strncmp(s, "set print ", 10) ||
+	    !strncmp(s, "print \"pixe", 11) ||
+	    !strncmp(s, "print \"data", 11));
+}
+
+enum {
+    REMOVE_PNG,
+    ADD_PNG
+};
+
 static int 
-add_or_remove_png_term (const char *fname, int add, GPT_SPEC *spec)
+add_or_remove_png_term (const char *fname, int action, GPT_SPEC *spec)
 {
     FILE *fsrc, *ftmp;
     char temp[MAXLEN], fline[MAXLEN];
     char restore_line[MAXLEN] = {0};
-    int png_line_saved = 0;
+    GptFlags flags = 0;
 #ifdef ENABLE_NLS
     int lv = iso_latin_version();
 #else
@@ -290,36 +320,35 @@ add_or_remove_png_term (const char *fname, int add, GPT_SPEC *spec)
 	return 1;
     }
 
-    if (add && spec == NULL) {
-	/* see if there's a commented out term setting to restore */
+    if (action == ADD_PNG) {
+	/* see if there's already a png term setting, possibly commented
+	   out, that can be reused */
 	while (fgets(fline, sizeof fline, fsrc)) {
-	    if (commented_term_line(fline)) {
+	    if (!strncmp(fline, "set term png", 12)) {
+		strcat(restore_line, fline);
+		break;
+	    } else if (commented_term_line(fline) && *restore_line == '\0') {
 		strcat(restore_line, fline + 2);
 		break;
+	    } else if (strstr(fline, "letterbox")) {
+		flags = GPT_LETTERBOX;
+	    } else if (!strncmp(fline, "plot", 4)) {
+		break;
 	    }
 	}
+
 	rewind(fsrc);
-    }
 
-    fprintf(stderr, "add_or_remove_png_term: add = %d\n", add);
-
-    if (add) {
-	int need_term_line = 1;
-
-	if (spec == NULL) {
-	    /* we're reconstituting a session graph from a 
-	       saved gnuplot command file */
-	    if (*restore_line) {
-		/* found a saved png term specification */
-		fputs(restore_line, ftmp);
-		need_term_line = 0;
-	    }
-	}
-	if (need_term_line) {
+	if (*restore_line) {
+	    fputs(restore_line, ftmp);
+	} else {
 	    int ptype = (spec != NULL)? spec->code : PLOT_REGULAR;
-	    int flags = (spec != NULL)? spec->flags : 0;
-	    const char *pline = get_gretl_png_term_line(ptype, flags);
+	    const char *pline;
 
+	    if (spec != NULL) {
+		flags = spec->flags;
+	    }
+	    pline = get_gretl_png_term_line(ptype, flags);
 	    fprintf(ftmp, "%s\n", pline);
 	}	    
 	fprintf(ftmp, "set output '%sgretltmp.png'\n", 
@@ -327,15 +356,27 @@ add_or_remove_png_term (const char *fname, int add, GPT_SPEC *spec)
     }
 
     /* now for the body of the plot file */
-    while (fgets(fline, sizeof fline, fsrc)) {
-	if (add) {
-	    if (!commented_term_line(fline) && !set_output_line(fline)) {
+
+    if (action == ADD_PNG) {
+	int got_set_print = 0;
+
+	while (fgets(fline, sizeof fline, fsrc)) {
+	    if (set_print_line(fline)) {
+		got_set_print = 1;
+		fputs(fline, ftmp);
+	    } else if (!commented_term_line(fline) && !set_output_line(fline)) {
 		line_to_file(fline, ftmp, -lv);
 	    }
-	} else {
-	    /* we're removing the png term line */
-	    int printit = 1;
-
+	}
+	if (gnuplot_png_terminal() == GP_PNG_CAIRO && !got_set_print) {
+	    print_plot_bounding_box_request(ftmp);
+	}
+    } else {
+	/* we're removing the png term line */
+	int printit, png_line_saved = 0;
+	
+	while (fgets(fline, sizeof fline, fsrc)) {
+	    printit = 1;
 	    if (!strncmp(fline, "set term png", 12)) {
 		if (!png_line_saved) {
 		    /* comment it out, for future reference */
@@ -344,12 +385,16 @@ add_or_remove_png_term (const char *fname, int add, GPT_SPEC *spec)
 		} 
 		printit = 0;
 	    } else if (commented_term_line(fline)) {
-		printit = 0;
+		if (png_line_saved) {
+		    printit = 0;
+		}
 	    } else if (set_output_line(fline)) {
 		printit = 0;
 	    } else if (spec != NULL && (spec->flags & GPT_FIT_HIDDEN)
 		       && is_auto_fit_string(fline)) {
 		printit = 0;
+	    } else if (set_print_line(fline)) {
+		printit = 0;
 	    }
 	    if (printit) {
 		line_to_file(fline, ftmp, lv);
@@ -366,19 +411,19 @@ add_or_remove_png_term (const char *fname, int add, GPT_SPEC *spec)
 
 static int add_png_term_to_plotfile (const char *fname)
 {
-    return add_or_remove_png_term(fname, 1, NULL);
+    return add_or_remove_png_term(fname, ADD_PNG, NULL);
 }
 
 static int remove_png_term_from_plotfile (const char *fname, GPT_SPEC *spec)
 {
-    return add_or_remove_png_term(fname, 0, spec);
+    return add_or_remove_png_term(fname, REMOVE_PNG, spec);
 }
 
-/* public because called from session.c when saving a graph file */
+/* public because called from session.c when editing plot commands */
 
 int remove_png_term_from_plotfile_by_name (const char *fname)
 {
-    return add_or_remove_png_term(fname, 0, NULL);
+    return add_or_remove_png_term(fname, REMOVE_PNG, NULL);
 }
 
 static void mark_plot_as_saved (GPT_SPEC *spec)
@@ -393,7 +438,7 @@ static int gnuplot_png_init (GPT_SPEC *spec, FILE **fpp)
     *fpp = gretl_fopen(spec->fname, "w");
 
     if (*fpp == NULL) {
-	errbox(_("Couldn't write to %s"), spec->fname);
+	file_write_errbox(spec->fname);
 	return 1;
     }
 
@@ -437,14 +482,18 @@ static int get_full_term_string (const GPT_SPEC *spec, char *termstr)
     } else if (!strcmp(spec->termtype, "postscript")) {
 	strcpy(termstr, "postscript eps"); 
     } else if (!strcmp(spec->termtype, "PDF")) {
-	strcpy(termstr, "pdf");
+	if (gnuplot_png_terminal() == GP_PNG_CAIRO) {
+	    strcpy(termstr, "pdfcairo");
+	} else {
+	    strcpy(termstr, "pdf");
+	}
     } else if (!strcmp(spec->termtype, "fig")) {
 	strcpy(termstr, "fig");
     } else if (!strcmp(spec->termtype, "latex")) {
 	strcpy(termstr, "latex");
-    } else if (!strcmp(spec->termtype, "png")) { 
+    } else if (!strncmp(spec->termtype, "png", 3)) { 
 	const char *png_str = 
-	    get_gretl_png_term_line(spec->code, 0);
+	    get_gretl_png_term_line(spec->code, spec->flags);
 
 	strcpy(termstr, png_str + 9);
     } else if (!strcmp(spec->termtype, "emf color")) {
@@ -461,77 +510,167 @@ static int get_full_term_string (const GPT_SPEC *spec, char *termstr)
     return cmds;
 }
 
-void save_graph_to_file (gpointer data, const char *fname)
+#ifndef ENABLE_NLS /* bodge */
+static void pprint_as_latin (PRN *prn, const char *s, int emf)
 {
-    GPT_SPEC *spec = (GPT_SPEC *) data;
-    char plottmp[MAXLEN], plotline[MAXLEN], termstr[MAXLEN];
+    pputs(prn, s);
+}
+
+static int html_encoded (const char *s)
+{
+    return 0;
+}
+#endif
+
+static char *get_insert_point (char *s)
+{
+    char *p = strstr(s, ", \\");
+
+    if (p == NULL) {
+	p = s + strlen(s) - 1;
+    }
+
+    if (p - s > 0 && *(p-1) == ' ') {
+	p--;
+    }
+
+    return p;
+}
+
+/* old gnuplot: can't do "rgb" line spec; we'll do what we can,
+   namely switch line type 2 for 3
+*/
+
+static void 
+print_line_with_color (PRN *prn, char *s, int lnum, int *contd)
+{
+    const gretlRGB *color = get_graph_color(lnum - 1);
+    char cstr[8];
+    char *p;
+
+    if (strstr(s, ", \\") == NULL) {
+	*contd = 0;
+    }
+
+    if (color == NULL) {
+	pputs(prn, s);
+	return;
+    }
+
+    print_rgb_hash(cstr, color);
+
+#if GPDEBUG
+    fprintf(stderr, "lnum=%d, cstr='%s', rgb=%d\n", lnum, cstr, rgb);
+    fprintf(stderr, "s='%s'\n", s);
+#endif
+    
+    if (lnum == 2 && strcmp(cstr, "#00ff00") &&
+	strstr(s, " lt ") == NULL) {
+	p = get_insert_point(s);
+	*p = '\0';
+	strcpy(p, " lt 3");
+    } else {
+	pputs(prn, s);
+	return;
+    }
+
+    if (*contd) {
+	pprintf(prn, "%s , \\\n", s);
+    } else {
+	pprintf(prn, "%s\n", s);
+    }
+}
+
+#define is_color_line(s) (strstr(s, "set style line") && strstr(s, "rgb"))
+
+static int filter_plot_file (const char *inname, 
+			     const char *term,
+			     const char *fname)
+{
+    char pline[MAXLEN];
+    char plottmp[MAXLEN];
     gchar *plotcmd = NULL;
-    FILE *fq;
-    PRN *prn;
-    int mono = 0, png = 0, emf = 0;
-    int cmds, err;
+    FILE *fp = NULL;
+    PRN *prn = NULL;
+    int ttype = 0, mono = 0;
+    int lnum = -1, recolor = 0;
+    int contd = 1, err = 0;
 
     if (user_fopen("gptout.tmp", plottmp, &prn)) {
-	return;
+	return 1;
     }
 
-    fq = gretl_fopen(spec->fname, "r");
-    if (fq == NULL) {
-	errbox(_("Couldn't access graph info"));
+    fp = gretl_fopen(inname, "r");
+    if (fp == NULL) {
+	file_read_errbox(inname);
 	gretl_print_destroy(prn);
-	return;
-    }
- 
-    cmds = get_full_term_string(spec, termstr);
-
-    if (strstr(termstr, " mono")) {
-	mono = 1;
+	return 1;
     }
 
-    if (!strncmp(termstr, "png", 3)) {
-	png = 1;
-    } else if (!strncmp(termstr, "emf", 3)) {
-	emf = 1;
+    if (!strncmp(term, "emf", 3)) {
+	ttype = GP_TERM_EMF;
+    } else if (!strncmp(term, "png", 3)) {
+	ttype = GP_TERM_PNG;
+    } else if (!strncmp(term, "post", 4)) {
+	ttype = GP_TERM_EPS;
+    } else if (!strncmp(term, "pdf", 3)) {
+	ttype = GP_TERM_PDF;
     }
 
-    if (cmds) {
-	if (copyfile(spec->fname, fname)) { 
-	    errbox(_("Failed to copy graph file"));
-	}
-	return;
-    } else {
 #ifdef ENABLE_NLS
-	pprint_gnuplot_encoding(termstr, prn);
-#endif
-	pprintf(prn, "set term %s\n", termstr);
-	pprintf(prn, "set output '%s'\n", fname);
-	while (fgets(plotline, MAXLEN-1, fq)) {
-	    if (!strncmp(plotline, "set term", 8) ||
-		!strncmp(plotline, "set output", 10)) {
-		continue;
-	    }
-	    if (mono && strstr(plotline, "set style fill solid")) {
-		pputs(prn, "set style fill solid 0.3\n");
-	    } else if (!png && html_encoded(plotline)) {
-		pprint_as_latin(prn, plotline, emf);
-	    } else {
-		pputs(prn, plotline);
-	    }
-#if 0
-	    if (!gretl_is_ascii(plotline)) {
-		fprintf(stderr, "non-ascii line: '%s'\n", plotline);
-		if (g_utf8_validate(plotline, -1, NULL)) {
-		    fprintf(stderr, " valid UTF-8\n");
-		} else {
-		    fprintf(stderr, " not valid UTF-8\n");
-		}
-	    }
+    pprint_gnuplot_encoding(term, prn);
 #endif
+    pprintf(prn, "set term %s\n", term);
+    pprintf(prn, "set output '%s'\n", fname);
+
+    if (strstr(term, " mono") || 
+	(strstr(term, "postscr") && !strstr(term, "color"))) {
+	mono = 1;
+    }    
+
+    if (!mono && (ttype == GP_TERM_EPS || ttype == GP_TERM_PDF)
+	&& !gnuplot_has_rgb()) {
+	recolor = 1;
+    }
+
+    while (fgets(pline, sizeof pline, fp)) {
+	if (!strncmp(pline, "set term", 8) ||
+	    !strncmp(pline, "set output", 10)) {
+	    continue;
 	}
+
+	if (!strncmp(pline, "plot ", 5)) {
+	    lnum = 0;
+	} else if (lnum >= 0) {
+	    lnum++;
+	}
+
+	if (mono && strstr(pline, "set style fill solid")) {
+	    pputs(prn, "set style fill solid 0.3\n");
+	} else if (mono && is_color_line(pline)) {
+	    ; /* skip */
+	} else if (ttype != GP_TERM_PNG && html_encoded(pline)) {
+	    pprint_as_latin(prn, pline, ttype == GP_TERM_EMF);
+	} else if (recolor && lnum > 0 && contd) {
+	    print_line_with_color(prn, pline, lnum, &contd);
+	} else if (!set_print_line(pline)) {
+	    pputs(prn, pline);
+	}
+
+#if 0
+	if (!gretl_is_ascii(pline)) {
+	    fprintf(stderr, "non-ascii line: '%s'\n", pline);
+	    if (g_utf8_validate(pline, -1, NULL)) {
+		fprintf(stderr, " valid UTF-8\n");
+	    } else {
+		fprintf(stderr, " not valid UTF-8\n");
+	    }
+	}
+#endif
     }
 
     gretl_print_destroy(prn);
-    fclose(fq);
+    fclose(fp);
 
     plotcmd = g_strdup_printf("\"%s\" \"%s\"", paths.gnuplot, 
 			      plottmp);
@@ -541,8 +680,58 @@ void save_graph_to_file (gpointer data, const char *fname)
     g_free(plotcmd);
 
     if (err) {
-	errbox(_("Gnuplot error creating graph"));
+	gui_errmsg(err);
     } 
+
+    return err;
+}
+
+void save_graph_to_file (gpointer data, const char *fname)
+{
+    GPT_SPEC *spec = (GPT_SPEC *) data;
+    char term[MAXLEN];
+    int cmds;
+
+    cmds = get_full_term_string(spec, term);
+
+    if (cmds) {
+	if (copyfile(spec->fname, fname)) { 
+	    errbox(_("Failed to copy graph file"));
+	}
+    } else {
+	filter_plot_file(spec->fname, term, fname);
+    }
+}
+
+#define GRETL_PDF_TMP "gretltmp.pdf"
+
+static void graph_display_pdf (GPT_SPEC *spec)
+{
+    char pdfname[FILENAME_MAX];
+    static char term[9];
+
+    build_path(pdfname, paths.userdir, GRETL_PDF_TMP, NULL);
+
+    if (*term == '\0') {
+	if (gnuplot_pdf_terminal() == GP_PDF_CAIRO) {
+	    strcpy(term, "pdfcairo");
+	} else {
+	    strcpy(term, "pdf");
+	}
+    }
+
+    if (filter_plot_file(spec->fname, term, pdfname)) {
+	errbox("Error creating graph file");
+	return;
+    }
+
+#if defined(G_OS_WIN32)
+    win32_open_file(pdfname);
+#elif defined(OSX_BUILD)
+    osx_open_file(pdfname);
+#else
+    gretl_fork(viewpdf, pdfname);
+#endif
 }
 
 /* chop trailing comma, if present; return 1 if comma chopped,
@@ -796,6 +985,8 @@ read_plotspec_range (const char *obj, const char *s, GPT_SPEC *spec)
 	i = 1;
     } else if (!strcmp(obj, "y2range")) {
 	i = 2;
+    } else if (!strcmp(obj, "trange")) {
+	i = 3;
     } else {
 	err = 1;
     }
@@ -818,31 +1009,34 @@ read_plotspec_range (const char *obj, const char *s, GPT_SPEC *spec)
     return err;
 }
 
-static void catch_value (char *targ, const char *src, int maxlen)
+static int catch_value (char *targ, const char *src, int maxlen)
 {
     int i, n;
 
     src += strspn(src, " \t\r\n");
-
     if (*src == '\'' || *src == '"') {
 	src++;
     }
 
     *targ = '\0';
-    strncat(targ, src, maxlen - 1);
-    n = strlen(targ);
-    
-    for (i=n-1; i>=0; i--) {
-	if (isspace((unsigned char) targ[i])) {
-	    targ[i] = '\0';
-	} else {
-	    break;
-	}
-    }    
 
-    if (targ[i] == '\'' || targ[i] == '"') {
-	targ[i] = '\0';
+    if (*src != '\0') {
+	strncat(targ, src, maxlen - 1);
+	n = strlen(targ);
+
+	for (i=n-1; i>=0; i--) {
+	    if (isspace((unsigned char) targ[i])) {
+		targ[i] = '\0';
+	    } else {
+		break;
+	    }
+	}  
+	if (targ[i] == '\'' || targ[i] == '"') {
+	    targ[i] = '\0';
+	}
     }
+
+    return (*targ != '\0');
 }
 
 static int parse_gp_set_line (GPT_SPEC *spec, const char *s, int *labelno)
@@ -861,15 +1055,33 @@ static int parse_gp_set_line (GPT_SPEC *spec, const char *s, int *labelno)
 	return 1;
     }
 
-    if (strcmp(variable, "y2tics") == 0) {
+    if (!strcmp(variable, "y2tics")) {
 	spec->flags |= GPT_Y2AXIS;
 	return 0;
-    } else if (strcmp(variable, "border 3") == 0) {
+    } else if (!strcmp(variable, "border 3")) {
 	spec->flags |= GPT_MINIMAL_BORDER;
 	return 0;
-    }    
+    } else if (!strcmp(variable, "parametric")) {
+	spec->flags |= GPT_PARAMETRIC;
+	return 0;
+    } else if (!strcmp(variable, "xzeroaxis")) {
+	spec->flags |= GPT_XZEROAXIS;
+	return 0;
+    } else if (!strcmp(variable, "yzeroaxis")) {
+	spec->flags |= GPT_YZEROAXIS;
+	return 0;
+    } else if (!strcmp(variable, "nokey")) {
+	strcpy(spec->keyspec, "none");
+	return 0;
+    } else if (!strcmp(variable, "label")) {
+	parse_label_line(spec, s, *labelno);
+	*labelno += 1;
+	return 0;
+    }
 
-    catch_value(value, s + 4 + strlen(variable), MAXLEN);
+    if (!catch_value(value, s + 4 + strlen(variable), MAXLEN)) {
+	return 0;
+    }
 
     if (strstr(variable, "range")) {
 	if (read_plotspec_range(variable, value, spec)) {
@@ -891,20 +1103,15 @@ static int parse_gp_set_line (GPT_SPEC *spec, const char *s, int *labelno)
 	strcpy(spec->titles[3], value);
     } else if (!strcmp(variable, "key")) {
 	strcpy(spec->keyspec, value);
-    } else if (!strcmp(variable, "nokey")) {
-	strcpy(spec->keyspec, "none");
     } else if (!strcmp(variable, "xtics")) { 
 	safecpy(spec->xtics, value, 15);
     } else if (!strcmp(variable, "mxtics")) { 
 	safecpy(spec->mxtics, value, 3);
-    } else if (!strcmp(variable, "xzeroaxis")) {
-	spec->xzeroaxis = 1;
     } else if (!strcmp(variable, "boxwidth")) {
 	spec->boxwidth = (float) atof(value);
-    } else if (!strcmp(variable, "label")) {
-	parse_label_line(spec, s, *labelno);
-	*labelno += 1;
-    }
+    } else if (!strcmp(variable, "samples")) {
+	spec->samples = atoi(value);
+    } 
 
     return 0;
 }
@@ -938,9 +1145,9 @@ static int allocate_plotspec_markers (GPT_SPEC *spec)
     return 0;
 }
 
-/* Determine the number of data points in a plot (also the type
-   of plot, and whether there are any data-point markers along with
-   the data).
+/* Determine the number of data points in a plot.  While we're at it,
+   determine the type of plot, and check whether there are any
+   data-point markers along with the data.
 */
 
 static int get_plot_nobs (FILE *fp, PlotType *ptype, int *do_markers)
@@ -1063,10 +1270,17 @@ static void grab_line_title (char *targ, const char *src)
      using XX axes XX title XX w XX lt XX lw XX
 */
 
-static int parse_gp_line_line (const char *s, GPT_SPEC *spec, int i)
+static int parse_gp_line_line (const char *s, GPT_SPEC *spec)
 {
     const char *p;
-    int err = 0;
+    int i, err;
+
+    err = plotspec_add_line(spec);
+    if (err) {
+	return err;
+    }
+
+    i = spec->n_lines - 1;
 
     if ((p = strstr(s, " using "))) {
 	/* data column spec */
@@ -1233,10 +1447,12 @@ static int uneditable_get_markers (GPT_SPEC *spec, FILE *fp, int *polar)
 	fseek(fp, offset, SEEK_SET);
     }
 
-    spec->n_lines = 1;
-    spec->lines[0].ncols = 2;
-
-    err = plot_get_data_and_markers(spec, fp, 2, 1);
+    err = plotspec_add_line(spec);
+    
+    if (!err) {
+	spec->lines[0].ncols = 2;
+	err = plot_get_data_and_markers(spec, fp, 2, 1);
+    }
 
     if (!err) {
 	maybe_set_all_markers_ok(spec);
@@ -1294,7 +1510,7 @@ static int read_plotspec_from_file (GPT_SPEC *spec, int *plot_pd, int *polar)
     /* open the plot file */
     fp = gretl_fopen(spec->fname, "r");
     if (fp == NULL) {
-	errbox(_("Couldn't open graph file"));
+	file_read_errbox(spec->fname);
 	return 1;
     }
 
@@ -1331,17 +1547,21 @@ static int read_plotspec_from_file (GPT_SPEC *spec, int *plot_pd, int *polar)
 	int v;
 
 	if (!strncmp(gpline, "# timeseries", 12)) {
-	    int pd;
-
-	    if (sscanf(gpline, "# timeseries %d", &pd)) {
-		*plot_pd = pd;
+	    if (sscanf(gpline, "# timeseries %d", &spec->pd)) {
+		*plot_pd = spec->pd;
 	    }
 	    spec->flags |= GPT_TS;
 	    if (strstr(gpline, "letterbox")) {
 		spec->flags |= GPT_LETTERBOX;
 	    }
 	    continue;
-	}
+	} else if (!strncmp(gpline, "# multiple timeseries", 21)) {
+	    if (sscanf(gpline, "# multiple timeseries %d", &spec->pd)) {
+		*plot_pd = spec->pd;
+	    }
+	    spec->flags |= GPT_TS;
+	    continue;
+	}	    
 
 	if (sscanf(gpline, "# X = '%15[^\']' (%d)", vname, &v) == 2) {
 	    if (plot_ols_var_ok(vname, v)) {
@@ -1424,8 +1644,8 @@ static int read_plotspec_from_file (GPT_SPEC *spec, int *plot_pd, int *polar)
 	goto plot_bailout;
     }
 
-    i = 0;
     done = 0;
+
     while (!err) {
 	top_n_tail(gpline);
 
@@ -1435,15 +1655,9 @@ static int read_plotspec_from_file (GPT_SPEC *spec, int *plot_pd, int *polar)
 	    done = 1;
 	} 
 
-	err = parse_gp_line_line(gpline, spec, i);
+	err = parse_gp_line_line(gpline, spec);
 
-	if (done) {
-	    break;
-	}
-
-	i++;
-
-	if ((got = fgets(gpline, MAXLEN - 1, fp)) == NULL) {
+	if (err || done || (got = fgets(gpline, MAXLEN - 1, fp)) == NULL) {
 	    break;
 	}
     }
@@ -1453,14 +1667,6 @@ static int read_plotspec_from_file (GPT_SPEC *spec, int *plot_pd, int *polar)
 	goto plot_bailout;
     }
 
-    spec->n_lines = i + 1; /* i is a zero-based index */
-
-    /* free any unused lines */
-    if (spec->n_lines < MAX_PLOT_LINES) {
-	spec->lines = myrealloc(spec->lines, 
-				spec->n_lines * sizeof *spec->lines);
-    }
-
     /* determine total number of required data columns */
     for (i=0; i<spec->n_lines; i++) {
 	if (spec->lines[i].ncols == 0) {
@@ -1494,10 +1700,6 @@ static int read_plotspec_from_file (GPT_SPEC *spec, int *plot_pd, int *polar)
     return err;
 }
 
-#ifdef USE_GNOME
-extern void gnome_print_graph (const char *fname);
-#endif
-
 static int get_data_xy (png_plot *plot, int x, int y, 
 			double *data_x, double *data_y)
 {
@@ -2042,11 +2244,19 @@ static gint plot_popup_activated (GtkWidget *w, gpointer data)
     if (!strcmp(item, _("Add another curve..."))) {
 	stats_calculator(plot, CALC_GRAPH_ADD, NULL);
     } else if (!strcmp(item, _("Save as PNG..."))) {
-	strcpy(plot->spec->termtype, "png");
+	if (gnuplot_png_terminal() == GP_PNG_CAIRO) {
+	    strcpy(plot->spec->termtype, "pngcairo");
+	} else {
+	    strcpy(plot->spec->termtype, "png");
+	}
         file_selector(_("Save gnuplot graph"), SAVE_GNUPLOT, 
 		      FSEL_DATA_MISC, plot->spec);
     } else if (!strcmp(item, _("Save as PDF..."))) {
-	strcpy(plot->spec->termtype, "PDF");
+	if (gnuplot_pdf_terminal() == GP_PDF_CAIRO) {
+	    strcpy(plot->spec->termtype, "pdfcairo");
+	} else {
+	    strcpy(plot->spec->termtype, "pdf");
+	}
         file_selector(_("Save gnuplot graph"), SAVE_GNUPLOT, 
 		      FSEL_DATA_MISC, plot->spec);
     } else if (!strcmp(item, _("Save to session as icon"))) { 
@@ -2057,7 +2267,7 @@ static gint plot_popup_activated (GtkWidget *w, gpointer data)
 	context_help(NULL, GINT_TO_POINTER(HURST));
     } else if (!strcmp(item, _("Freeze data labels"))) {
 	plot->spec->flags |= GPT_ALL_MARKERS;
-	redisplay_edited_png(plot);
+	redisplay_edited_plot(plot);
     } else if (!strcmp(item, _("Clear data labels"))) { 
 	zoom_unzoom_png(plot, PNG_REDISPLAY);
     } else if (!strcmp(item, _("Zoom..."))) { 
@@ -2073,12 +2283,14 @@ static gint plot_popup_activated (GtkWidget *w, gpointer data)
     } else if (!strcmp(item, _("Restore full view"))) { 
 	zoom_unzoom_png(plot, PNG_UNZOOM);
     }
-#ifdef USE_GNOME 
+#if defined(USE_GNOME) || defined(GTK_PRINTING)
     else if (!strcmp(item, _("Print..."))) { 
-	gnome_print_graph(plot->spec->fname);
+	gtk_print_graph(plot->spec->fname);
     }
 #endif 
-    else if (!strcmp(item, _("OLS estimates"))) { 
+    else if (!strcmp(item, _("Display PDF"))) { 
+	graph_display_pdf(plot->spec);
+    } else if (!strcmp(item, _("OLS estimates"))) { 
 	if (plot->spec != NULL) {
 	    do_graph_model(plot->spec->reglist, plot->spec->fit);
 	}
@@ -2146,12 +2358,13 @@ static void build_plot_menu (png_plot *plot)
 	N_("Freeze data labels"),
 	N_("Clear data labels"),
 	N_("Zoom..."),
-#ifdef USE_GNOME
+#if defined(USE_GNOME) || defined(GTK_PRINTING)
 	N_("Print..."),
 #endif
 #ifdef G_OS_WIN32
 	N_("Print"),
 #endif
+	N_("Display PDF"),
 	N_("OLS estimates"),
 	N_("Numerical values"),
 	N_("Edit"),
@@ -2169,7 +2382,7 @@ static void build_plot_menu (png_plot *plot)
     int i;
 
     if (pdf_ok == -1) {
-	pdf_ok = gnuplot_has_pdf();
+	pdf_ok = gnuplot_pdf_terminal();
     }
 
     plot->popup = gtk_menu_new();
@@ -2207,7 +2420,14 @@ static void build_plot_menu (png_plot *plot)
 	    i++;
 	    continue;
 	}
-	if (!pdf_ok && !strcmp(plot_items[i], "Save as PDF...")) {
+	if (!pdf_ok && (!strcmp(plot_items[i], "Save as PDF...") ||
+			!strcmp(plot_items[i], "Display PDF"))) {
+	    i++;
+	    continue;
+	}
+	if (pdf_ok && !strcmp(plot_items[i], "Print...")) {
+	    /* Print... is currently very funky for graphs.  If
+	       we're able to display PDF, bypass this option */
 	    i++;
 	    continue;
 	}	    
@@ -2254,14 +2474,14 @@ static void build_plot_menu (png_plot *plot)
 		     &plot->popup);
 }
 
-int redisplay_edited_png (png_plot *plot)
+int redisplay_edited_plot (png_plot *plot)
 {
     gchar *plotcmd;
     FILE *fp;
     int err = 0;
 
 #if GPDEBUG
-    fprintf(stderr, "redisplay_edited_png: plot = %p\n", (void *) plot);
+    fprintf(stderr, "redisplay_edited_plot: plot = %p\n", (void *) plot);
 #endif
 
     /* open file in which to dump plot specification */
@@ -2282,8 +2502,8 @@ int redisplay_edited_png (png_plot *plot)
     g_free(plotcmd);
 
     if (err) {
-	errbox(_("Failed to generate PNG file"));
-	return 1;
+	gui_errmsg(err);
+	return err;
     }
 
     /* reset format flags */
@@ -2293,9 +2513,7 @@ int redisplay_edited_png (png_plot *plot)
     get_plot_ranges(plot);
 
     /* put the newly created PNG onto the plot canvas */
-    render_pngfile(plot, PNG_REDISPLAY);
-
-    return 0;
+    return render_pngfile(plot, PNG_REDISPLAY);
 }
 
 static int zoom_unzoom_png (png_plot *plot, int view)
@@ -2354,13 +2572,11 @@ static int zoom_unzoom_png (png_plot *plot, int view)
     }
 
     if (err) {
-	errbox(_("Failed to generate PNG file"));
-	return 1;
+	gui_errmsg(err);
+	return err;
     }
 
-    render_pngfile(plot, view);
-
-    return 0;
+    return render_pngfile(plot, view);
 }
 
 static gint plot_button_release (GtkWidget *widget, GdkEventButton *event, 
@@ -2490,7 +2706,7 @@ void plot_expose (GtkWidget *widget, GdkEventExpose *event,
 			 event->area.width, event->area.height);
 }
 
-static void render_pngfile (png_plot *plot, int view)
+static int render_pngfile (png_plot *plot, int view)
 {
     gint width;
     gint height;
@@ -2505,7 +2721,7 @@ static void render_pngfile (png_plot *plot, int view)
         errbox(error->message);
         g_error_free(error);
 	remove(pngname);
-	return;
+	return 1;
     }
 
     width = gdk_pixbuf_get_width(pbuf);
@@ -2515,7 +2731,7 @@ static void render_pngfile (png_plot *plot, int view)
 	errbox(_("Malformed PNG file for graph"));
 	g_object_unref(pbuf);
 	remove(pngname);
-	return;
+	return 1;
     }
 
     /* scrap any old record of which points are labeled */
@@ -2547,6 +2763,8 @@ static void render_pngfile (png_plot *plot, int view)
 	    plot->status ^= PLOT_ZOOMED;
 	}
     }
+
+    return 0;
 }
 
 static void destroy_png_plot (GtkWidget *w, png_plot *plot)
@@ -2620,6 +2838,14 @@ static void set_approx_pixel_bounds (png_plot *plot,
 #endif
 }
 
+int ok_dumb_line (const char *s)
+{
+    if (strstr(s, "x2tics")) return 0;
+    if (strstr(s, "set style line")) return 0;
+    if (strstr(s, "set style inc")) return 0;
+    return 1;
+}
+
 /* Attempt to read y-range info from the ascii representation
    of a gnuplot graph (the "dumb" terminal): return 0 on
    success, non-zero on failure.
@@ -2653,7 +2879,7 @@ static int get_dumb_plot_yrange (png_plot *plot)
 	    fputs("set term dumb\n", fpout);
 	} else if (strstr(line, "set output")) { 
 	    fprintf(fpout, "set output '%s'\n", dumbtxt);
-	} else {
+	} else if (ok_dumb_line(line)) {
 	    fputs(line, fpout);
 	}
 	if (strstr(line, "x2range")) {
@@ -2664,8 +2890,7 @@ static int get_dumb_plot_yrange (png_plot *plot)
     fclose(fpin);
     fclose(fpout);
 
-    plotcmd = g_strdup_printf("\"%s\" \"%s\"", paths.gnuplot,
-			      dumbgp);
+    plotcmd = g_strdup_printf("\"%s\" \"%s\"", paths.gnuplot, dumbgp);
     err = gretl_spawn(plotcmd);
     g_free(plotcmd);
 
@@ -2675,6 +2900,7 @@ static int get_dumb_plot_yrange (png_plot *plot)
 #if POINTS_DEBUG
 	fputs("get_dumb_plot_yrange(): plot command failed\n", stderr);
 #endif
+	gretl_error_clear();
 	return 1;
     } else {
 	double y[16] = {0};
@@ -2819,7 +3045,9 @@ static int get_plot_ranges (png_plot *plot)
     fclose(fp);
 
 #ifdef PNG_COMMENTS
-    /* now try getting accurate coordinate info from PNG file */
+    /* now try getting accurate coordinate info from PNG file
+       or auxiliary file
+    */
     if (get_png_bounds_info(&b) == GRETL_PNG_OK) {
 	plot->status |= PLOT_PNG_COORDS;
 	got_x = got_y = 1;
@@ -2839,6 +3067,7 @@ static int get_plot_ranges (png_plot *plot)
 	fprintf(stderr, "using px_height %d, px_width %d\n",
 		plot->pixel_height, plot->pixel_width);
 # endif
+	fprintf(stderr, "get_png_bounds_info(): OK\n");
     } else {
 	fprintf(stderr, "get_png_bounds_info(): failed\n");
     }
@@ -2994,7 +3223,6 @@ gnuplot_show_png (const char *plotfile, GPT_SPEC *spec, int saved)
     if (!err) {
 	get_plot_ranges(plot);
     }
-	
 
     plot->shell = gtk_window_new(GTK_WINDOW_TOPLEVEL);
 
@@ -3117,7 +3345,11 @@ gnuplot_show_png (const char *plotfile, GPT_SPEC *spec, int saved)
     g_signal_connect(G_OBJECT(plot->canvas), "expose_event",
 		     G_CALLBACK(plot_expose), plot->pixmap);
 
-    render_pngfile(plot, PNG_START);
+    err = render_pngfile(plot, PNG_START);
+    if (err) {
+	gtk_widget_destroy(plot->shell);
+	plot = NULL;
+    }
 
     return plot;
 }
@@ -3148,7 +3380,7 @@ void display_session_graph_png (const char *fname)
     g_free(plotcmd);
 
     if (err) {
-	errbox(_("Gnuplot error creating graph"));
+	gui_errmsg(err);
     } else {
 	gnuplot_show_png(fullname, NULL, 1);
     }
@@ -3165,15 +3397,37 @@ static int get_png_plot_bounds (const char *str, png_bounds *bounds)
 {
     int ret = GRETL_PNG_OK;
 
-    if (sscanf(str, "xleft=%d xright=%d ybot=%d ytop=%d", 
+    bounds->xleft = bounds->xright = 0;
+    bounds->ybot = bounds->ytop = 0;
+
+    if (sscanf(str, "pixel_bounds: %d %d %d %d",
+	       &bounds->xleft, &bounds->xright,
+	       &bounds->ybot, &bounds->ytop) == 4) {
+	ret = GRETL_PNG_OK;
+    } else if (sscanf(str, "xleft=%d xright=%d ybot=%d ytop=%d", 
 	       &bounds->xleft, &bounds->xright,
 	       &bounds->ybot, &bounds->ytop) != 4) {
 	ret = GRETL_PNG_BAD_COMMENTS;
-    } else if (bounds->xleft == 0 && bounds->xright == 0 &&
-	       bounds->ybot == 0 && bounds->ytop == 0) {
-	ret = GRETL_PNG_NO_COORDS;
     } 
 
+    if (ret == GRETL_PNG_OK && bounds->xleft == 0 && 
+	bounds->xright == 0 && bounds->ybot == 0 && 
+	bounds->ytop == 0) {
+	ret = GRETL_PNG_NO_COORDS;
+    }
+
+    if (bounds->xright > 1024) {
+	bounds->xleft /= 20;
+	bounds->xright /= 20;
+	bounds->ybot /= 20;
+	bounds->ytop /= 20;
+    }
+
+#if POINTS_DEBUG
+    fprintf(stderr, "Got: xleft=%d, xright=%d, ybot=%d, ytop=%d\n",
+	    bounds->xleft, bounds->xright, bounds->ybot, bounds->ytop);
+#endif
+
     return ret;
 }
 
@@ -3187,22 +3441,70 @@ static int get_png_data_bounds (char *str, png_bounds *bounds)
 	p++;
     }
 
+    bounds->xmin = bounds->xmax = 0.0;
+    bounds->ymin = bounds->ymax = 0.0;
+
     gretl_push_c_numeric_locale();
 
-    if (sscanf(str, "xmin=%lf xmax=%lf ymin=%lf ymax=%lf", 
+    if (sscanf(str, "data_bounds: %lf %lf %lf %lf",
+	       &bounds->xmin, &bounds->xmax,
+	       &bounds->ymin, &bounds->ymax) == 4) {
+	ret = GRETL_PNG_OK;
+    } else if (sscanf(str, "xmin=%lf xmax=%lf ymin=%lf ymax=%lf", 
 	       &bounds->xmin, &bounds->xmax,
 	       &bounds->ymin, &bounds->ymax) != 4) {
 	ret = GRETL_PNG_BAD_COMMENTS;
-    } else if (bounds->xmin == 0.0 && bounds->xmax == 0.0 &&
-	       bounds->ymin == 0.0 && bounds->ymax == 0.0) {
+    } 
+
+    if (ret == GRETL_PNG_OK && bounds->xmin == 0.0 && 
+	bounds->xmax == 0.0 && bounds->ymin == 0.0 && 
+	bounds->ymax == 0.0) {
 	ret = GRETL_PNG_NO_COORDS;
     } 
 
+#if POINTS_DEBUG
+    fprintf(stderr, "Got: xmin=%g, xmax=%g, ymin=%g, ymax=%g\n",
+	    bounds->xmin, bounds->xmax, bounds->ymin, bounds->ymax);
+#endif
+
     gretl_pop_c_numeric_locale();
 
     return ret;
 }
 
+static int new_get_png_bounds_info (png_bounds *bounds, FILE *fp)
+{
+    char line[128];
+    int plot_ret = -1, data_ret = -1;
+    int ret = GRETL_PNG_OK;
+
+    if (fgets(line, sizeof line, fp) == NULL) {
+	plot_ret = GRETL_PNG_NO_COMMENTS;
+    } else {
+	plot_ret = get_png_plot_bounds(line, bounds);
+    }
+
+    if (fgets(line, sizeof line, fp) == NULL) {
+	data_ret = GRETL_PNG_NO_COMMENTS;
+    } else {
+	data_ret = get_png_data_bounds(line, bounds);
+    }
+
+    if (plot_ret == GRETL_PNG_NO_COORDS && data_ret == GRETL_PNG_NO_COORDS) {
+	/* comments were present and correct, but all zero */
+	ret = GRETL_PNG_NO_COORDS;
+    } else if (plot_ret != GRETL_PNG_OK || data_ret != GRETL_PNG_OK) {
+	/* one or both set of coordinates bad or missing */
+	if (plot_ret >= 0 || data_ret >= 0) {
+	    ret = GRETL_PNG_BAD_COMMENTS;
+	} else {
+	    ret = GRETL_PNG_NO_COMMENTS;
+	}
+    }
+
+    return ret;
+}
+
 #define PNG_CHECK_BYTES 4
 
 static int get_png_bounds_info (png_bounds *bounds)
@@ -3216,6 +3518,17 @@ static int get_png_bounds_info (png_bounds *bounds)
     int i, num_text;
     volatile int ret = GRETL_PNG_OK;
 
+#if 1
+    build_path(pngname, paths.userdir, "gretltmp.png", ".bounds"); 
+    fp = fopen(pngname, "r");
+    if (fp != NULL) {
+	ret = new_get_png_bounds_info(bounds, fp);
+	fclose(fp);
+	remove(pngname);
+	return ret;
+    }
+#endif
+
     build_path(pngname, paths.userdir, "gretltmp.png", NULL); 
 
     fp = gretl_fopen(pngname, "rb");
@@ -3297,157 +3610,3 @@ static int get_png_bounds_info (png_bounds *bounds)
 
 #endif /* PNG_COMMENTS */
 
-/* for graphs in the stats calculator */
-
-static int get_dist_and_df (const char *s, int *d, int *m, int *n)
-{
-    int ret = 1;
-
-    if (!strncmp(s, "# standard", 10)) {
-	*d = NORMAL_DIST;
-    } else if (sscanf(s, "# t(%d)", m) == 1) {
-	*d = T_DIST;
-    } else if (sscanf(s, "# chi-square(%d)", m) == 1) {
-	*d = CHISQ_DIST;
-    } else if (sscanf(s, "# F(%d,%d)", m, n) == 2) {
-	*d = F_DIST;
-    } else {
-	ret = 0;
-    }
-
-    return ret;
-}
-
-void revise_distribution_plotspec (png_plot *plot, int d, int df1, int df2)
-{
-    GPT_SPEC *spec = plot->spec;
-    gchar *title = NULL;
-    const char *s;
-    char dfstr[16];
-    int got[5] = {0};
-    int bigchi = 0;
-    int m, n, prevd, dfmax = 0;
-    int i, err = 0;
-
-    for (i=0; i<spec->n_literal; i++) {
-	int dfid;
-
-	s = spec->literal[i];
-	m = n = 0;
-	prevd = -1;
-	if (*s == '#') {
-	    if (get_dist_and_df(s, &prevd, &m, &n)) {
-		if (prevd == d && m == df1 && n == df2) {
-		    /* line is already present */
-		    return;
-		}
-		if (prevd == CHISQ_DIST && m > 69) {
-		    got[4] += 1;
-		} else {
-		    got[prevd] += 1;
-		}
-	    }
-	} else if (sscanf(s, "df%d=", &dfid) == 1) {
-	    if (dfid > dfmax) {
-		dfmax = dfid;
-	    }
-	}
-    }
-
-    /* adjust x-range if needed */
-    if (d == CHISQ_DIST || d == F_DIST) {
-	double x = dist_xmax(d, df1, df2);
-
-	if (x > spec->range[0][1]) {
-	    spec->range[0][1] = x;
-	}
-    }
-
-    /* add comment for current plot */
-    title = dist_marker_line(d, df1, df2);
-    err = strings_array_add(&spec->literal, &spec->n_literal, title);
-    g_free(title);
-
-    if (!err) {
-	/* add df line(s) if needed */
-	m = dfmax + 1;
-	if (d == T_DIST || d == CHISQ_DIST || d == F_DIST) {
-	    sprintf(dfstr, "df%d=%.1f", m, (double) df1);
-	    err = strings_array_add(&spec->literal, &spec->n_literal, dfstr);
-	}
-	if (d == F_DIST && !err) {
-	    sprintf(dfstr, "df%d=%.1f", m + 1, (double) df2);
-	    err = strings_array_add(&spec->literal, &spec->n_literal, dfstr);
-	}
-    }
-
-    if (err) {
-	gui_errmsg(err);
-	return;
-    }  
-
-    if (d == CHISQ_DIST && df1 > 69) {
-	bigchi = 1;
-    }
-
-    /* add any required formula lines */
-
-    if (d == T_DIST) {
-	if (!got[T_DIST] && !got[F_DIST]) {
-	    err = strings_array_add(&spec->literal, &spec->n_literal, 
-				    dist_formula(F_BINV));
-	}
-    } else if (d == CHISQ_DIST) {
-	if (bigchi && !got[4]) {
-	    err = strings_array_add(&spec->literal, &spec->n_literal, 
-				    dist_formula(F_LOG2));
-	    if (!err) {
-		err = strings_array_add(&spec->literal, &spec->n_literal, 
-					dist_formula(F_BIGCHI));
-	    }
-	} else if (!bigchi && !got[CHISQ_DIST]) {
-	    err = strings_array_add(&spec->literal, &spec->n_literal, 
-				    dist_formula(F_CHI));
-	}
-    } else if (d == F_DIST) {
-	if (!got[F_DIST] && !got[T_DIST]) {
-	    err = strings_array_add(&spec->literal, &spec->n_literal, 
-				    dist_formula(F_BINV));
-	} 
-	if (!err && !got[F_DIST]) {
-	    err = strings_array_add(&spec->literal, &spec->n_literal, 
-				    dist_formula(F_F));
-	}
-    }
-
-    if (!err) {
-	/* add new plot line */
-	err = plotspec_add_line(spec);
-    }
-
-    if (err) {
-	gui_errmsg(err);
-	return;
-    }
-
-    n = spec->n_lines - 1;
-
-    strcpy(spec->lines[n].scale, "NA");
-    strcpy(spec->lines[n].style, "lines");
-    title = dist_graph_title(d, NADBL, df1, df2);
-    strcpy(spec->lines[n].title, title);
-    g_free(title);
-
-    if (d == NORMAL_DIST) {
-	strcpy(spec->lines[n].formula, "(1/(sqrt(2*pi))*exp(-(x)**2/2))");
-    } else if (d == T_DIST) {
-	sprintf(spec->lines[n].formula, "Binv(0.5*df%d,0.5)/sqrt(df%d)*(1.0+(x*x)/df%d)"
-		"**(-0.5*(df%d+1.0))", m, m, m, m);
-    } else if (d == CHISQ_DIST) {
-	sprintf(spec->lines[n].formula, "%s(x,df%d)", (bigchi)? "bigchi" : "chi", m);
-    } else if (d == F_DIST) {
-	sprintf(spec->lines[n].formula, "f(x,df%d,df%d)", m, m + 1);
-    }
-
-    redisplay_edited_png(plot);
-}
