@@ -1,20 +1,20 @@
-/*
- *  Copyright (c) by Allin Cottrell
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
- *
- *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- *
+/* 
+ *  gretl -- Gnu Regression, Econometrics and Time-series Library
+ *  Copyright (C) 2001 Allin Cottrell and Riccardo "Jack" Lucchetti
+ * 
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ * 
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ * 
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 /* gpt_dialog.c for gretl -- GUI gnuplot controller dialog */
@@ -41,15 +41,15 @@ struct gpt_range_t {
     GtkWidget *max;
 };
 
-GtkWidget *linetitle[MAX_PLOT_LINES];
-GtkWidget *stylecombo[MAX_PLOT_LINES];
-GtkWidget *yaxiscombo[MAX_PLOT_LINES];
-GtkWidget *linescale[MAX_PLOT_LINES];
-GtkWidget *linewidth[MAX_PLOT_LINES];
+static GtkWidget **linetitle;
+static GtkWidget **stylecombo;
+static GtkWidget **yaxiscombo;
+static GtkWidget **linescale;
+static GtkWidget **linewidth;
 
-GtkWidget *labeltext[MAX_PLOT_LABELS];
-GtkWidget *labeljust[MAX_PLOT_LABELS];
-GtkWidget *labelpos[MAX_PLOT_LABELS];
+static GtkWidget *labeltext[MAX_PLOT_LABELS];
+static GtkWidget *labeljust[MAX_PLOT_LABELS];
+static GtkWidget *labelpos[MAX_PLOT_LABELS];
 
 static GtkWidget *gpt_control;
 static GtkWidget *keycombo;
@@ -135,6 +135,7 @@ static void entry_to_gp_string (GtkWidget *w, char *targ, size_t n)
     g_return_if_fail(GTK_IS_ENTRY(w));
     wstr = gtk_entry_get_text(GTK_ENTRY(w));
 
+#ifdef ENABLE_NLS
     if (wstr != NULL && *wstr != '\0') {
 	gchar *trstr = gp_locale_from_utf8(wstr);
 
@@ -145,6 +146,11 @@ static void entry_to_gp_string (GtkWidget *w, char *targ, size_t n)
 	    strncat(targ, wstr, n-1);
 	}
     }
+#else
+    if (wstr != NULL && *wstr != '\0') {
+	strncat(targ, wstr, n-1);
+    }
+#endif
 }
 
 static FitType fit_type_from_string (const char *s)
@@ -279,8 +285,7 @@ static void gp_string_to_entry (GtkWidget *w, const char *str)
 {
 #ifdef ENABLE_NLS
     int lv = iso_latin_version();
-#else
-    int lv = 0;
+    int pngterm = gnuplot_png_terminal();
 #endif
     gchar *trstr = NULL;
 
@@ -289,7 +294,9 @@ static void gp_string_to_entry (GtkWidget *w, const char *str)
 	return;
     }
 
-    if (lv == 2) {
+
+#ifdef ENABLE_NLS
+    if (lv == 2 && pngterm < GP_PNG_CAIRO) {
 	char lstr[MAXTITLE];
 	
 	sprint_html_to_l2(lstr, str);
@@ -299,6 +306,9 @@ static void gp_string_to_entry (GtkWidget *w, const char *str)
     } else {
 	trstr = g_strdup(str);
     }
+#else
+    trstr = g_strdup(str);
+#endif
 
     if (trstr != NULL) {
 	gtk_entry_set_text(GTK_ENTRY(w), trstr);
@@ -497,7 +507,7 @@ static void apply_gpt_changes (GtkWidget *widget, GPT_SPEC *spec)
 	    png_plot *plot = (png_plot *) spec->ptr;
 
 	    set_plot_has_y2_axis(plot, spec->flags & GPT_Y2AXIS);
-	    redisplay_edited_png(plot);
+	    redisplay_edited_plot(plot);
 	}
 	mark_session_changed();
     }
@@ -556,10 +566,20 @@ static const char *get_font_filename (const char *showname)
 
 static int font_is_ok (const char *fname)
 {
+    static int pngterm;
     char cmd[64];
     int err;
 
-    sprintf(cmd, "set term png font %s 10", fname);
+    if (pngterm == 0) {
+	pngterm = gnuplot_png_terminal();
+    }
+
+    if (pngterm == GP_PNG_CAIRO) {
+	sprintf(cmd, "set term pngcairo font \"%s,10\"", fname);
+    } else {
+	sprintf(cmd, "set term png font %s 10", fname);
+    }
+
     err = gnuplot_test_command(cmd);
 
     return err == 0;
@@ -637,9 +657,6 @@ static void toggle_axis_selection (GtkWidget *w, GPT_SPEC *spec)
     int i;
 
     for (i=0; i<spec->n_lines; i++) {
-	if (i >= MAX_PLOT_LINES) {
-	    break;
-	}
 	if (yaxiscombo[i] != NULL) {
 	    gtk_widget_set_sensitive(yaxiscombo[i], !no_y2);
 	}
@@ -735,8 +752,27 @@ static void add_color_selector (int i, GtkWidget *tbl, int *rows,
     }
 }
 
+#if 0
+static void gp_commands_window (GtkWidget *w, GPT_SPEC *spec)
+{
+    remove_png_term_from_plotfile_by_name(spec->fname);
+    view_file(spec->fname, 1, 0, 78, 400, GR_PLOT);
+}
+#endif
+
+/* PNG anti-aliasing switch */
+
+static void set_aa_status (GtkWidget *w, int *ok)
+{
+    *ok = GTK_TOGGLE_BUTTON(w)->active;
+
+    gnuplot_png_set_use_aa(*ok);
+}
+
 static void gpt_tab_main (GtkWidget *notebook, GPT_SPEC *spec) 
 {
+    static int aa_ok = 1;
+    static int show_aa_check = -1;
     GtkWidget *label, *vbox, *tbl;
     int i, rows = 1;
     GList *keypos_list = NULL;
@@ -886,6 +922,26 @@ static void gpt_tab_main (GtkWidget *notebook, GPT_SPEC *spec)
 	markers_check = NULL;
     }
 
+    if (show_aa_check < 0) {
+	show_aa_check = (gnuplot_png_terminal() == GP_PNG_GD2);
+    }
+
+    /* give option of suppressing anti-aliasing for PNGs */
+    if (show_aa_check) {
+	GtkWidget *aa_check;
+
+	table_add_row(tbl, &rows, TAB_MAIN_COLS);
+	aa_check = gtk_check_button_new_with_label(_("Allow anti-aliasing of lines"));
+	gtk_table_attach_defaults(GTK_TABLE(tbl), 
+				  aa_check, 0, TAB_MAIN_COLS, 
+				  rows-1, rows);
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(aa_check),
+				     aa_ok);
+	g_signal_connect(G_OBJECT(aa_check), "clicked", 
+			 G_CALLBACK(set_aa_status), &aa_ok);
+	gtk_widget_show(aa_check);
+    }	
+
     /* set TT font (if gnuplot uses libgd and freetype) */
     if (gnuplot_has_ttf(0)) {
 	GtkWidget *ebox, *hsep;
@@ -950,7 +1006,7 @@ static void gpt_tab_main (GtkWidget *notebook, GPT_SPEC *spec)
 	ttfspin = NULL;
     }
 
-    if (gnuplot_has_specified_colors()) { 
+    if (gnuplot_png_terminal() != GP_PNG_OLD) {
 	GtkWidget *hsep = gtk_hseparator_new();
 
 	table_add_row(tbl, &rows, TAB_MAIN_COLS);
@@ -966,6 +1022,30 @@ static void gpt_tab_main (GtkWidget *notebook, GPT_SPEC *spec)
 	    }
 	}
     }
+
+#if 0 /* this is not thought out properly */
+    if (1) {
+	GtkWidget *hsep = gtk_hseparator_new();
+	GtkWidget *hbox, *button;
+
+	table_add_row(tbl, &rows, TAB_MAIN_COLS);
+	gtk_table_attach_defaults(GTK_TABLE(tbl), hsep, 0, TAB_MAIN_COLS, 
+				  rows - 1, rows);  
+	gtk_widget_show(hsep);
+
+	table_add_row(tbl, &rows, TAB_MAIN_COLS);
+	hbox = gtk_hbox_new(FALSE, 2);
+	button = gtk_button_new_with_label(_("Edit plot commands"));
+	gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 10);
+	g_signal_connect(G_OBJECT(button), "clicked", 
+			 G_CALLBACK(gp_commands_window), 
+			 spec);
+	gtk_table_attach(GTK_TABLE(tbl), hbox, 0, 2, rows - 1, rows,
+			 GTK_FILL, 0, 0, 5);
+	gtk_widget_show(button);
+	gtk_widget_show(hbox);
+    }
+#endif
 }
 
 static void gpt_tab_output (GtkWidget *notebook, GPT_SPEC *spec) 
@@ -983,8 +1063,7 @@ static void gpt_tab_output (GtkWidget *notebook, GPT_SPEC *spec)
 	"plot commands",
 	NULL
     }; 
-    int pdf_ok = gnuplot_has_pdf();
-    
+    int pdf_ok = gnuplot_pdf_terminal();
 
     for (i=0; termtypes[i] != NULL; i++) {
 	if (!pdf_ok && !strcmp(termtypes[i], "PDF")) {
@@ -1519,9 +1598,52 @@ static void gpt_tab_XY (GtkWidget *notebook, GPT_SPEC *spec, gint axis)
 
 void close_gnuplot_dialog (GtkWidget *w, gpointer p)
 {
+    free(linetitle);
+    free(stylecombo);
+    free(yaxiscombo);
+    free(linescale);
+    free(linewidth);
+
+    linetitle = NULL;
+    stylecombo = NULL;
+    yaxiscombo = NULL;
+    linescale = NULL;
+    linewidth = NULL;
+
     gtk_widget_destroy(GTK_WIDGET(p));
 }
 
+static int gpt_allocate_widgets (int n)
+{
+    int i;
+
+    if (n == 0) {
+	return 0;
+    }
+
+    linetitle = malloc(n * sizeof *linetitle);
+    stylecombo = malloc(n * sizeof *stylecombo);
+    yaxiscombo = malloc(n * sizeof *yaxiscombo);
+    linescale = malloc(n * sizeof *linescale);
+    linewidth = malloc(n * sizeof *linewidth);
+    
+    if (linetitle == NULL || stylecombo == NULL ||
+	yaxiscombo == NULL || linescale == NULL ||
+	linewidth == NULL) {
+	return E_ALLOC;
+    }
+
+    for (i=0; i<n; i++) {
+	linetitle[i] = NULL;
+	stylecombo[i] = NULL;
+	yaxiscombo[i] = NULL;
+	linescale[i] = NULL;
+	linewidth[i] = NULL;
+    }
+
+    return 0;
+}
+
 int show_gnuplot_dialog (GPT_SPEC *spec) 
 {
     png_plot *plot = (png_plot *) spec->ptr;
@@ -1535,6 +1657,11 @@ int show_gnuplot_dialog (GPT_SPEC *spec)
 	return 1;
     }
 
+    if (gpt_allocate_widgets(spec->n_lines)) {
+	nomem();
+	return 1;
+    }
+
     for (i=0; i<MAX_AXES; i++) {
 	axis_range[i].isauto = NULL;
     }
@@ -1569,7 +1696,9 @@ int show_gnuplot_dialog (GPT_SPEC *spec)
 	gpt_tab_XY(notebook, spec, 2);
     }
 
-    gpt_tab_lines(notebook, spec);
+    if (spec->lines != NULL) {
+	gpt_tab_lines(notebook, spec);
+    }
     gpt_tab_labels(notebook, spec); 
     gpt_tab_output(notebook, spec);
 
