@@ -1,20 +1,20 @@
-/*
- *  Copyright (c) 2004 by Allin Cottrell
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
- *
- *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- *
+/* 
+ *  gretl -- Gnu Regression, Econometrics and Time-series Library
+ *  Copyright (C) 2001 Allin Cottrell and Riccardo "Jack" Lucchetti
+ * 
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ * 
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ * 
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 #include "gretl.h"
@@ -607,7 +607,7 @@ int display_graph_page (void)
     gpage_filenames_init(NULL);
 
     if (!strncmp(latex, "pdf", 3)) {
-	if (gnuplot_has_pdf()) {
+	if (gnuplot_pdf_terminal()) {
 	    gpage.output = PDF_OUTPUT;
 	} else {
 	    latex_orig = g_strdup(latex);
@@ -678,7 +678,7 @@ int save_graph_page (const char *fname)
     gpage_filenames_init(fname);
 
     if (!strncmp(latex, "pdf", 3)) {
-	if (gnuplot_has_pdf()) {
+	if (gnuplot_pdf_terminal()) {
 	    gpage.output = PDF_OUTPUT;
 	} else {
 	    latex_orig = g_strdup(latex);
