@@ -1,20 +1,20 @@
-/*
- *  Copyright (c) by Allin Cottrell
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
- *
- *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
- *
+/* 
+ *  gretl -- Gnu Regression, Econometrics and Time-series Library
+ *  Copyright (C) 2001 Allin Cottrell and Riccardo "Jack" Lucchetti
+ * 
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ * 
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ * 
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 /* gretl.c : main for gretl */
@@ -59,7 +59,6 @@ static GtkWidget *make_main_window (void);
 
 static gboolean main_popup_handler (GtkWidget *w, GdkEventButton *event,
 				    gpointer data);
-static int selection_count (GtkTreeSelection *select, int *vnum);
 static void set_up_main_menu (void);
 static void startRcallback (gpointer p, guint opt, GtkWidget *w);
 static void auto_store (void);
@@ -72,14 +71,14 @@ GtkTargetEntry gretl_drag_targets[] = {
 };
 
 static void  
-drag_data_received  (GtkWidget          *widget,
-		     GdkDragContext     *dc,
-		     gint                x,
-		     gint                y,
-		     GtkSelectionData   *data,
-		     guint               info,
-		     guint               time,
-		     gpointer            p);
+mdata_handle_drag  (GtkWidget          *widget,
+		    GdkDragContext     *dc,
+		    gint                x,
+		    gint                y,
+		    GtkSelectionData   *data,
+		    guint               info,
+		    guint               time,
+		    gpointer            p);
 
 #ifdef USE_GNOME
 static char *optrun, *optdb;
@@ -191,13 +190,9 @@ GtkItemFactoryEntry data_items[] = {
     { N_("/File/Open data/Import/_CSV..."), NULL, open_data, OPEN_CSV, NULL, GNULL },
     { N_("/File/Open data/Import/_ASCII..."), NULL, open_data, OPEN_ASCII, NULL, GNULL },
     { N_("/File/Open data/Import/_Octave..."), NULL, open_data, OPEN_OCTAVE, NULL, GNULL },
-#ifdef G_OS_WIN32
-    { N_("/File/Open data/Import/_Excel..."), NULL, open_data, OPEN_EXCEL, NULL, GNULL },
-    { N_("/File/Open data/Import/_Gnumeric..."), NULL, open_data, OPEN_GNUMERIC, NULL, GNULL },
-#else
     { N_("/File/Open data/Import/_Gnumeric..."), NULL, open_data, OPEN_GNUMERIC, NULL, GNULL },
     { N_("/File/Open data/Import/_Excel..."), NULL, open_data, OPEN_EXCEL, NULL, GNULL },
-#endif
+    { N_("/File/Open data/Import/_Open Document..."), NULL, open_data, OPEN_ODS, NULL, GNULL },
     { N_("/File/Open data/Import/_Eviews..."), NULL, open_data, OPEN_WF1, NULL, GNULL },
     { N_("/File/Open data/Import/_Stata..."), NULL, open_data, OPEN_DTA, NULL, GNULL },
     { N_("/File/Open data/Import/_JMulTi..."), NULL, open_data, OPEN_JMULTI, NULL, GNULL },
@@ -210,6 +205,7 @@ GtkItemFactoryEntry data_items[] = {
     { N_("/File/Append data/_Octave..."), NULL, open_data, APPEND_OCTAVE, NULL, GNULL },
     { N_("/File/Append data/_Gnumeric..."), NULL, open_data, APPEND_GNUMERIC, NULL, GNULL },
     { N_("/File/Append data/_Excel..."), NULL, open_data, APPEND_EXCEL, NULL, GNULL },
+    { N_("/File/Append data/_Open Document..."), NULL, open_data, OPEN_ODS, NULL, GNULL },
     { N_("/File/Append data/_Eviews..."), NULL, open_data, APPEND_WF1, NULL, GNULL },
     { N_("/File/Append data/_Stata..."), NULL, open_data, APPEND_DTA, NULL, GNULL },
     { N_("/File/Append data/_JMulTi..."), NULL, open_data, APPEND_JMULTI, NULL, GNULL },
@@ -245,7 +241,6 @@ GtkItemFactoryEntry data_items[] = {
     { N_("/File/Script files/_Practice file..."), "", display_files, PS_FILES, 
       "<StockItem>", GTK_STOCK_OPEN },
     { N_("/File/Script files/_New script"), "", do_new_script, 0, "<StockItem>", GTK_STOCK_NEW },
-    { "/File/sep2", NULL, NULL, 0, "<Separator>", GNULL },
 
     /* File, session files */
     { N_("/File/_Session files"), NULL, NULL, 0, "<Branch>", GNULL },
@@ -255,7 +250,6 @@ GtkItemFactoryEntry data_items[] = {
       "<StockItem>", GTK_STOCK_SAVE },
     { N_("/File/Session files/Save session _as..."), "", save_session_callback, SAVE_RENAME, 
       "<StockItem>", GTK_STOCK_SAVE_AS }, 
-    { "/File/sep3", NULL, NULL, 0, "<Separator>", GNULL },
 
     /* File, databases */
     { N_("/File/_Databases"), NULL, NULL, 0, "<Branch>", GNULL },
@@ -267,7 +261,6 @@ GtkItemFactoryEntry data_items[] = {
       "<StockItem>", GTK_STOCK_OPEN },
     { N_("/File/Databases/On database _server..."), "", display_files, REMOTE_DB, 
       "<StockItem>", GTK_STOCK_NETWORK },
-    { "/File/sep4", NULL, NULL, 0, "<Separator>", GNULL },
 
     /* File, function packages */
     { N_("/File/_Function files"), NULL, NULL, 0, "<Branch>", GNULL },
@@ -284,12 +277,12 @@ GtkItemFactoryEntry data_items[] = {
     { N_("/_Tools"), NULL, NULL, 0, "<Branch>", GNULL },
     { N_("/Tools/_Statistical tables"), NULL, stats_calculator, CALC_DIST, NULL, GNULL },
     { N_("/Tools/_P-value finder"), NULL, stats_calculator, CALC_PVAL, NULL, GNULL },
+    { N_("/Tools/_Distribution graphs"), NULL, stats_calculator, CALC_GRAPH, 
+      NULL, GNULL },
     { N_("/Tools/_Test statistic calculator"), NULL, stats_calculator, CALC_TEST, 
       NULL, GNULL },
     { N_("/Tools/_Nonparametric tests"), NULL, stats_calculator, CALC_NPTEST, 
       NULL, GNULL },
-    { N_("/Tools/_Probability distributions"), NULL, stats_calculator, CALC_GRAPH, 
-      NULL, GNULL },
     { N_("/Tools/_Seed for random numbers"), NULL, rand_seed_dialog, 0, 
       NULL, GNULL },
     { "/Tools/sep1", NULL, NULL, 0, "<Separator>", GNULL },
@@ -300,19 +293,17 @@ GtkItemFactoryEntry data_items[] = {
     { N_("/Tools/_Sort variables"), NULL, NULL, 0, "<Branch>", GNULL },
     { N_("/Tools/Sort variables/By _ID number"), NULL, sort_varlist, 0, NULL, GNULL },
     { N_("/Tools/Sort variables/By _name"), NULL, sort_varlist, 1, NULL, GNULL },
-    { "/Tools/sep3", NULL, NULL, 0, "<Separator>", GNULL },
     { N_("/Tools/_NIST test suite"), NULL, NULL, 0, "<Branch>", GNULL },
     { N_("/Tools/NIST test suite/_Basic"), NULL, do_nistcheck, 0, NULL, GNULL },
     { N_("/Tools/NIST test suite/_Verbose"), NULL, do_nistcheck, 1, NULL, GNULL },
     { N_("/Tools/NIST test suite/V_ery verbose"), NULL, do_nistcheck, 2, NULL, GNULL },
-    { "/Tools/sep4", NULL, NULL, 0, "<Separator>", GNULL },
     /* Tools, preferences */
     { N_("/Tools/_Preferences"), NULL, NULL, 0, "<Branch>", GNULL },
     { N_("/Tools/_Preferences/_General..."), NULL, options_dialog_callback, 0, 
       "<StockItem>", GTK_STOCK_PREFERENCES },
     { N_("/Tools/Preferences/_Fixed font..."), NULL, font_selector, 
       FIXED_FONT_SELECTION, "<StockItem>", GTK_STOCK_SELECT_FONT },
-#ifndef USE_GNOME
+#if !defined(USE_GNOME)
     { N_("/Tools/Preferences/_Menu font..."), NULL, font_selector, 
       APP_FONT_SELECTION, "<StockItem>", GTK_STOCK_SELECT_FONT },
 #endif
@@ -338,6 +329,7 @@ GtkItemFactoryEntry data_items[] = {
     { N_("/Data/_Compact data..."), NULL, do_compact_data_set, 0, NULL, GNULL },
     { N_("/Data/_Expand data..."), NULL, do_expand_data_set, 0, NULL, GNULL },
     { N_("/Data/_Transpose data..."), NULL, gui_transpose_data, 0, NULL, GNULL },
+    { N_("/Data/_Sort data..."), NULL, gui_sort_data, 0, NULL, GNULL },
     { "/Data/sep3", NULL, NULL, 0, "<Separator>", GNULL },
     { N_("/Data/_Refresh window"), NULL, refresh_data, 0, NULL, GNULL },
 
@@ -368,7 +360,6 @@ GtkItemFactoryEntry data_items[] = {
     { "/View/sep1", NULL, NULL, 0, "<Separator>", GNULL },
     /* descriptive statistics */
     { N_("/View/_Summary statistics"), NULL, do_menu_op, SUMMARY, NULL, GNULL },
-    { "/View/sep2", NULL, NULL, 0, "<Separator>", GNULL },
     { N_("/View/_Correlation matrix"), NULL, do_menu_op, CORR, NULL, GNULL },
     { N_("/View/Cross _Tabulation"), NULL, do_menu_op, XTAB, NULL, GNULL },
     { N_("/View/_Principal components"), NULL, do_menu_op, PCA, NULL, GNULL },
@@ -395,19 +386,8 @@ GtkItemFactoryEntry data_items[] = {
     { N_("/Add/_Index variable"), NULL, add_index, 0, NULL, GNULL },
     { N_("/Add/_Time trend"), NULL, add_index, 1, NULL, GNULL },
     { "/Add/sep2", NULL, NULL, 0, "<Separator>", GNULL },
-    { N_("/Add/_Random variable"), NULL, NULL, 0, "<Branch>", GNULL },
-    { N_("/Add/Random variable/_Uniform..."), NULL, add_rand_callback, 
-      RANDOM_UNIFORM, NULL, GNULL },
-    { N_("/Add/Random variable/_Normal..."), NULL, add_rand_callback, 
-      RANDOM_NORMAL, NULL, GNULL },
-    { N_("/Add/Random variable/_Chi-square..."), NULL, add_rand_callback, 
-      RANDOM_CHISQ, NULL, GNULL },
-    { N_("/Add/Random variable/Student's _t..."), NULL, add_rand_callback, 
-      RANDOM_ST, NULL, GNULL },
-    { N_("/Add/Random variable/Binomial..."), NULL, add_rand_callback, 
-      RANDOM_BIN, NULL, GNULL },
-    { N_("/Add/Random variable/Poisson..."), NULL, add_rand_callback, 
-      RANDOM_POIS, NULL, GNULL },
+    { N_("/Add/_Random variable..."), NULL, stats_calculator, CALC_RAND, 
+      NULL, GNULL },
     { "/Add/sep3", NULL, NULL, 0, "<Separator>", GNULL },
     /* dummies */
     { N_("/Add/_Periodic dummies"), NULL, add_dummies, TS_DUMMIES, NULL, GNULL },
@@ -500,26 +480,22 @@ GtkItemFactoryEntry data_items[] = {
     { N_("/Model/Other linear models/High _precision OLS..."), NULL, 
       model_callback, MPOLS, NULL, GNULL },
 #endif
-    { "/Model/sep1",  NULL, NULL, 0, "<Separator>", GNULL },
     { N_("/Model/_Time series"), NULL, NULL, 0, "<Branch>", GNULL },
-    { "/Model/sep2",  NULL, NULL, 0, "<Separator>", GNULL },
     { N_("/Model/_Panel"), NULL, NULL, 0, "<Branch>", GNULL },
-    { "/Model/sep3",  NULL, NULL, 0, "<Separator>", GNULL },
     { N_("/Model/_Nonlinear models"), NULL, NULL, 0, "<Branch>", GNULL }, 
     { N_("/Model/Nonlinear models/_Logit..."), NULL, model_callback, LOGIT, NULL, GNULL },
     { N_("/Model/Nonlinear models/_Probit..."), NULL, model_callback, PROBIT, NULL, GNULL },
     { N_("/Model/Nonlinear models/To_bit..."), NULL, model_callback, TOBIT, NULL, GNULL },
+    { N_("/Model/Nonlinear models/_Heckit..."), NULL, model_callback, HECKIT, NULL, GNULL },
     { N_("/Model/Nonlinear models/Poi_sson..."), NULL, model_callback, POISSON, NULL, GNULL },
     { N_("/Model/Nonlinear models/Lo_gistic..."), NULL, model_callback, LOGISTIC, NULL, GNULL },
     { N_("/Model/Nonlinear models/_Nonlinear Least Squares..."), NULL, 
       gretl_callback, NLS, NULL, GNULL },
-    { "/Model/sep4",  NULL, NULL, 0, "<Separator>", GNULL },
     { N_("/Model/_Robust estimation"), NULL, NULL, 0, "<Branch>", GNULL }, 
     { N_("/Model/Robust estimation/Least _Absolute Deviation..."), NULL, 
       model_callback, LAD, NULL, GNULL },
     { N_("/Model/Robust estimation/_Rank correlation..."), NULL, 
       selector_callback, SPEARMAN, NULL, GNULL },
-    { "/Model/sep5",  NULL, NULL, 0, "<Separator>", GNULL },
     { N_("/Model/_Maximum likelihood..."), NULL, gretl_callback, MLE, NULL, GNULL },
     { N_("/Model/_GMM..."), NULL, gretl_callback, GMM, NULL, GNULL },
     { N_("/Model/_Simultaneous equations..."), NULL, gretl_callback, SYSTEM, NULL, GNULL },
@@ -559,7 +535,7 @@ static void gui_usage (void)
 	      " -h or --help      Print this info and exit.\n"
 	      " -v or --version   Print version info and exit.\n"));
 
-    exit(0);
+    exit(EXIT_SUCCESS);
 }
 
 static void noalloc (const char *str)
@@ -573,11 +549,13 @@ static void get_runfile (char *fname)
     int i;
 
     *tryfile = '\0';
+
 #ifdef G_OS_WIN32
     if (unmangle(fname, tryfile)) return;
 #else
     strncat(tryfile, fname, MAXLEN - 1);
 #endif
+
     if (addpath(tryfile, &paths, 1) == NULL) {
 	fprintf(stderr, I_("Couldn't find script '%s'\n"), tryfile);
 	exit(EXIT_FAILURE);
@@ -619,7 +597,7 @@ static void fix_dbname (char *db)
 
 #ifdef ENABLE_NLS
 
-# if defined(G_OS_WIN32)
+# if defined(G_OS_WIN32) 
 
 static void real_nls_init (void)
 {
@@ -638,6 +616,31 @@ static void real_nls_init (void)
     bind_textdomain_codeset(PACKAGE, "UTF-8");
 }
 
+# elif defined(OSX_BUILD) 
+
+static void real_nls_init (void)
+{
+    char *gretlhome = getenv("GRETL_HOME");
+    char localedir[MAXSTR];
+    char *p, *loc;
+
+    if (gretlhome == NULL) {
+	return;
+    }
+
+    strcpy(localedir, gretlhome);
+    p = strstr(localedir, "share/gretl");
+    if (p != NULL) {
+	strcpy(p, "share/locale");
+    }
+
+    loc = setlocale(LC_ALL, "");
+    set_gretl_charset(loc);
+    bindtextdomain(PACKAGE, localedir);
+    textdomain(PACKAGE);
+    bind_textdomain_codeset(PACKAGE, "UTF-8");
+}
+
 # else /* regular *nix treatment */
 
 static void real_nls_init (void)
@@ -651,7 +654,7 @@ static void real_nls_init (void)
     bind_textdomain_codeset(PACKAGE, "UTF-8");
 }
 
-# endif
+# endif /* NLS init variants */
 
 void nls_init (void)
 {
@@ -738,7 +741,9 @@ int main (int argc, char *argv[])
 
 #ifdef G_OS_WIN32
     putenv("PANGO_WIN32_NO_UNISCRIBE=a");
+# if GTK_MINOR_VERSION < 6
     putenv("G_FILENAME_ENCODING=@locale");
+# endif
 #endif     
 
     *tryfile = '\0';
@@ -862,7 +867,6 @@ int main (int argc, char *argv[])
 	if (unmangle(filearg, paths.datfile)) {
 	    exit(EXIT_FAILURE);
 	}
-	my_filename_to_utf8(paths.datfile);
 #else
 	record_filearg(paths.datfile, filearg);
 #endif
@@ -874,27 +878,24 @@ int main (int argc, char *argv[])
 
 	switch (ftype) {
 	case GRETL_NATIVE_DATA:
-	    err = gretl_get_data(&Z, &datainfo, paths.datfile, &paths, DATA_NONE, 
-				 prn);
+	    err = gretl_get_data(&Z, &datainfo, paths.datfile, &paths, prn);
 	    break;
 	case GRETL_XML_DATA:
-	    err = gretl_read_gdt(&Z, &datainfo, paths.datfile, &paths, DATA_NONE, 
-				 prn, 0);
+	    err = gretl_read_gdt(&Z, &datainfo, paths.datfile, &paths, OPT_NONE, 
+				 prn);
 	    break;
 	case GRETL_CSV_DATA:
-	    err = import_csv(&Z, &datainfo, paths.datfile, prn);
+	    err = import_csv(&Z, &datainfo, paths.datfile, OPT_NONE, prn);
 	    break;
 	case GRETL_OCTAVE:
 	    err = import_octave(&Z, &datainfo, paths.datfile, prn);
 	    break;
-	case GRETL_BOX_DATA:
-	    err = import_box(&Z, &datainfo, paths.datfile, prn);
-	    break;
 	case GRETL_EXCEL:
 	case GRETL_GNUMERIC:
 	case GRETL_WF1:
 	case GRETL_DTA:
 	case GRETL_JMULTI:
+	case GRETL_ODS:
 	    err = get_worksheet_data(paths.datfile, ftype, 0);
 	    break;
 	case GRETL_SCRIPT:
@@ -918,12 +919,6 @@ int main (int argc, char *argv[])
 	    break;
 	}
 
-#if 0
-	if (paths.datfile[0] != 0) {
-	    my_filename_to_utf8(paths.datfile);
-	}
-#endif
-
 	if (err == E_CANCEL) {
 	    err = 0;
 	    ftype = 0;
@@ -949,6 +944,7 @@ int main (int argc, char *argv[])
     if (make_main_window() == NULL) 
 	noalloc(_("main window"));
     if (have_data()) {
+	/* redundant? */
 	set_sample_label(datainfo);
     }
 
@@ -963,6 +959,8 @@ int main (int argc, char *argv[])
     restore_sample_state(FALSE);
     main_menubar_state(FALSE);
 
+    /* run init script, if found? */
+
     if (have_data()) {
 	register_data(paths.datfile, tryfile, 1);
 	maybe_display_string_table();
@@ -1027,7 +1025,7 @@ static void check_varmenu_state (GtkTreeSelection *select, gpointer p)
 {
     if (mdata->ifac != NULL) {
 	int vnum = 0;
-	int sc = selection_count(select, &vnum);
+	int sc = tree_selection_count(select, &vnum);
 
 	if (sc == 1 && vnum > 0) {
 	    mdata->active_var = vnum;
@@ -1081,9 +1079,7 @@ static gint catch_mdata_key (GtkWidget *w, GdkEventKey *key, windata_t *vwin)
 	) {
 	int selcount, vnum = 0;
 
-	selcount = 
-	    selection_count(gtk_tree_view_get_selection(GTK_TREE_VIEW(mdata->listbox)),
-			    &vnum);
+	selcount = vwin_selection_count(mdata, &vnum);
 
 	if (selcount == 1 && vnum != 0) {
 	    mdata->active_var = vnum;
@@ -1289,6 +1285,28 @@ void populate_varlist (void)
     variable_menu_state(TRUE);
 }
 
+void mdata_select_last_var (void)
+{
+    GtkTreeIter iter, last;
+    GtkTreeStore *store;
+    GtkTreeSelection *select;
+
+    store = GTK_TREE_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(mdata->listbox)));
+    gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter);
+
+    while (1) {
+	last = iter;
+	if (!gtk_tree_model_iter_next(GTK_TREE_MODEL(store), &iter)) {
+	    iter = last;
+	    break;
+	}
+    }
+
+    select = gtk_tree_view_get_selection(GTK_TREE_VIEW(mdata->listbox));
+    gtk_tree_selection_unselect_all(select);
+    gtk_tree_selection_select_iter(select, &iter);
+}
+
 static gint 
 compare_var_ids (GtkTreeModel *model, GtkTreeIter *a, GtkTreeIter *b,
 		 gpointer p)
@@ -1506,7 +1524,7 @@ static GtkWidget *make_main_window (void)
 		      GDK_ACTION_COPY);
 
     g_signal_connect(G_OBJECT(mdata->listbox), "drag_data_received",
-		     G_CALLBACK(drag_data_received),
+		     G_CALLBACK(mdata_handle_drag),
 		     NULL);
 
     gtk_box_pack_start(GTK_BOX(main_vbox), box, TRUE, TRUE, 0);
@@ -1537,12 +1555,6 @@ static GtkWidget *make_main_window (void)
 	gtk_window_move(GTK_WINDOW(mdata->w), main_x, main_y);
     }
 
-#ifdef G_OS_WIN32
-    if (wimp) {
-	menu_font_option_off();
-    }
-#endif
-
     return main_vbox;
 }
 
@@ -1554,12 +1566,16 @@ static void set_up_main_menu (void)
     accel_group = gtk_accel_group_new();
     mdata->ifac = gtk_item_factory_new(GTK_TYPE_MENU_BAR, "<main>", 
 				       accel_group);
+
     gtk_window_add_accel_group(GTK_WINDOW(mdata->w), accel_group);
     g_object_unref(accel_group);
+
 #ifdef ENABLE_NLS
     gtk_item_factory_set_translate_func(mdata->ifac, menu_translate, NULL, NULL);
 #endif    
+
     gtk_item_factory_create_items(mdata->ifac, n_items, data_items, NULL);
+
     mdata->mbar = gtk_item_factory_get_widget(mdata->ifac, "<main>");
 }
 
@@ -1640,14 +1656,14 @@ void set_wm_icon (GtkWidget *w, gpointer data)
 /* Drag 'n' drop */
 
 static void  
-drag_data_received  (GtkWidget *widget,
-		     GdkDragContext *context,
-		     gint x,
-		     gint y,
-		     GtkSelectionData *data,
-		     guint info,
-		     guint time,
-		     gpointer p)
+mdata_handle_drag  (GtkWidget *widget,
+		    GdkDragContext *context,
+		    gint x,
+		    gint y,
+		    GtkSelectionData *data,
+		    guint info,
+		    guint time,
+		    gpointer p)
 {
     gchar *dfname;
     char tmp[MAXLEN];
@@ -1685,8 +1701,6 @@ drag_data_received  (GtkWidget *widget,
 
 #ifdef G_OS_WIN32
     slash_convert(tmp, TO_BACKSLASH);
-    /* also convert to UTF-8 here? */
-    my_filename_to_utf8(tmp);
 #endif
 
     strcpy(tryfile, tmp);
@@ -1720,56 +1734,16 @@ static void auto_store (void)
     }	
 }
 
-static void get_selected_varnum (GtkTreeModel *model, GtkTreePath *path,
-				 GtkTreeIter *iter, int *v)
-{
-    gchar *id;
-
-    gtk_tree_model_get(model, iter, 0, &id, -1);  
-    *v = atoi(id);
-    g_free(id);
-}
-
-static void count_selections (GtkTreeModel *model, GtkTreePath *path,
-			      GtkTreeIter *iter, int *selcount)
-{
-    *selcount += 1;
-}
-
-static int selection_count (GtkTreeSelection *select, int *vnum)
-{
-    int selcount = 0;
-
-    if (select != NULL) {
-	gtk_tree_selection_selected_foreach(select, 
-					    (GtkTreeSelectionForeachFunc) 
-					    count_selections,
-					    &selcount);
-    }
-    
-    if (vnum != NULL && selcount == 1) {
-	gtk_tree_selection_selected_foreach(select, 
-					    (GtkTreeSelectionForeachFunc) 
-					    get_selected_varnum,
-					    vnum);	
-    }
-
-    return selcount;
-}
-
 int mdata_selection_count (void)
 {
-    return selection_count(gtk_tree_view_get_selection(GTK_TREE_VIEW(mdata->listbox)),
-			   NULL);
+    return vwin_selection_count(mdata, NULL);
 }
 
 int mdata_active_var (void)
 {
     int selcount, v = 0;
 
-    selcount = 
-	selection_count(gtk_tree_view_get_selection(GTK_TREE_VIEW(mdata->listbox)),
-			&v);
+    selcount = vwin_selection_count(mdata, &v);
 
     if (selcount == 1 && v != 0) {
 	mdata->active_var = v;
@@ -1787,7 +1761,7 @@ main_popup_handler (GtkWidget *w, GdkEventButton *event, gpointer data)
 
     if (mods & GDK_BUTTON3_MASK) {
 	/* ignore all but right-clicks */
-	int selcount = mdata_selection_count();
+	int selcount = vwin_selection_count(mdata, NULL);
 
 	if (mdata->popup) {
 	    gtk_widget_destroy(mdata->popup);
