@@ -1,20 +1,20 @@
-/*
- *   Copyright (c) by Allin Cottrell
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
- *
- *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- *
+/* 
+ *  gretl -- Gnu Regression, Econometrics and Time-series Library
+ *  Copyright (C) 2001 Allin Cottrell and Riccardo "Jack" Lucchetti
+ * 
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ * 
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ * 
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 /* gretl.h, main header file for gretl gui */
@@ -39,6 +39,16 @@
 #include <stdlib.h>
 #include <time.h>
 
+#if defined(G_OS_WIN32) || defined(USE_GNOME) || (GTK_MINOR_VERSION >= 10)
+# define NATIVE_PRINTING
+#endif
+
+#ifndef G_OS_WIN32
+# if GTK_MINOR_VERSION >= 10
+#  define GTK_PRINTING
+# endif
+#endif
+
 #define GNULL (gconstpointer) NULL
 
 #include <gtksourceview/gtksourceview.h>
@@ -50,9 +60,9 @@
 #include "library.h"
 #include "settings.h"
 #include "helpfiles.h"
-#include "gui_recode.h"
 
 #ifdef ENABLE_NLS
+# include "gui_recode.h"
 # include "locale.h"
 #endif
 
@@ -87,14 +97,10 @@ extern int main_y;
 extern int mainwin_width;
 extern int mainwin_height;
 
-#if defined(G_OS_WIN32)
-extern int wimp;
-#else
-# ifndef OSX_BUILD
+#if !defined(G_OS_WIN32) && !defined(OSX_BUILD)
 extern char viewps[MAXSTR];
 extern char viewpdf[MAXSTR];
 extern char Browser[MAXSTR];
-# endif
 #endif
 
 extern char calculator[MAXSTR];
@@ -123,6 +129,7 @@ int mdata_selection_count (void);
 int mdata_active_var (void);
 void populate_varlist (void);
 void clear_varlist (GtkWidget *widget);
+void mdata_select_last_var (void);
 int gui_restore_sample (double ***pZ, DATAINFO **ppdinfo);
 
 /* functions defined in files other than gretl.c */
