@@ -92,6 +92,8 @@ enum extra_cmds {
     PANEL_B,
     IMPORT,
     BOOTSTRAP,
+    TRANSPOS,
+    DATASORT,
     GUI_CMD_MAX
 };
 
@@ -105,7 +107,6 @@ enum file_ops {
     APPEND_CSV,
     OPEN_ASCII,
     APPEND_ASCII,
-    OPEN_BOX,
     OPEN_OCTAVE,
     APPEND_OCTAVE,
     OPEN_GNUMERIC,
@@ -118,6 +119,8 @@ enum file_ops {
     APPEND_DTA,
     OPEN_JMULTI,
     APPEND_JMULTI,
+    OPEN_ODS,
+    APPEND_ODS,
     OPEN_SESSION,
     OPEN_MARKERS,
     END_OPEN,      /* marker for end of file open section */
@@ -225,7 +228,8 @@ enum calc_functions {
     CALC_TEST,
     CALC_NPTEST,
     CALC_GRAPH,
-    CALC_GRAPH_ADD
+    CALC_GRAPH_ADD,
+    CALC_RAND
 };
 
 enum auto_dummies {
@@ -240,25 +244,6 @@ enum dynamic_forecast_status {
     DYNAMIC_FORCED
 };
 
-enum random_types {
-    RANDOM_UNIFORM,
-    RANDOM_NORMAL,
-    RANDOM_CHISQ,
-    RANDOM_ST,
-    RANDOM_BIN,
-    RANDOM_POIS
-};
-
-enum pdists {
-    NORMAL_DIST,
-    T_DIST,
-    CHISQ_DIST,
-    F_DIST,
-    DW_DIST,
-    BINOMIAL_DIST,
-    POISSON_DIST
-};    
-
 #define MULTI_FORMAT_ENABLED(c) (c == SUMMARY || c == VAR_SUMMARY || \
                                  c == ALL_SUMMARY || \
 	                         c == CORR || c == ALL_CORR || c == FCASTERR || \
