@@ -1,20 +1,20 @@
-/*
- *  Copyright (c) by Allin Cottrell
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
- *
- *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
- *
+/* 
+ *  gretl -- Gnu Regression, Econometrics and Time-series Library
+ *  Copyright (C) 2001 Allin Cottrell and Riccardo "Jack" Lucchetti
+ * 
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ * 
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ * 
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 #undef CHILD_DEBUG
@@ -36,8 +36,6 @@
 
 #define MAX_CONSOLE_LINES 500
 
-extern int wimp; /* settings.c */
-
 void redirect_io_to_console (void)
 {
     CONSOLE_SCREEN_BUFFER_INFO coninfo;
@@ -152,7 +150,7 @@ void startR (char *Rcommand)
     int err = 0;
 
     if (!data_status) {
-	errbox(_("Please open a data file first"));
+	warnbox(_("Please open a data file first"));
 	return;
     }
 
@@ -200,6 +198,8 @@ void startR (char *Rcommand)
 	if (fq != NULL) {
 	    fputs("# load data from gretl\n", fq);
 	    fputs("library(stats)\n", fq);
+	    fputs("vnum <- as.double(R.version$major) + (as.double(R.version$minor) / 10.0)\n", fq);
+	    fputs("if (vnum > 2.41) library(utils)\n", fq);
 	    fprintf(fq, "gretldata <- read.table(\"%s\", header=TRUE)\n", 
 		    slash_convert(Rdata, FROM_BACKSLASH));
 	    fprintf(fq, "attach(gretldata)\n");
@@ -419,28 +419,32 @@ static void try_to_get_windows_font (void)
     }
 }
 
-void menu_font_option_off (void)
+int use_wimp;
+
+static int wimp_init (void)
 {
-    if (mdata->ifac != NULL) {
-	flip(mdata->ifac, "/File/Preferences/Menu font...", FALSE);
+    char tmp[4] = {0};
+
+    /* Are we using the XP theme (as opposed to "classic")? */
+
+    read_reg_val(HKEY_CURRENT_USER, 
+		 "Microsoft\\Windows\\CurrentVersion\\ThemeManager", 
+		 "ThemeActive", tmp);
+
+    if (!strcmp(tmp, "1")) {
+	use_wimp = 1;
     }
 }
 
 void set_up_windows_look (void)
 {
-    if (wimp) { 
-	/* "Windows Impersonator" wanted */
+    if (use_wimp) { 
 	size_t n = strlen(paths.gretldir);
 	int needslash = (paths.gretldir[n-1] != SLASH);
 	gchar *wimprc;
 
-#ifdef NEW_GTK
 	wimprc = g_strdup_printf("%s%sshare\\themes\\MS-Windows\\gtk-2.0\\gtkrc", 
 				 paths.gretldir, (needslash)? "\\" : "");
-#else
-	wimprc = g_strdup_printf("%s%setc\\gtk-2.0\\gtkrc.wimp", 
-				 paths.gretldir, (needslash)? "\\" : "");
-#endif
 	gtk_rc_parse(wimprc);
 	g_free(wimprc);
     } else {
@@ -506,6 +510,7 @@ void gretl_win32_init (const char *progname)
 {
     set_network_cfg_filename(progname);
 
+    wimp_init();
     read_rc(); /* get config info from registry */
     set_gd_fontpath();
     hush_warnings();
@@ -623,14 +628,34 @@ int win_buf_to_clipboard (const char *buf)
 
 int fnamecmp_win32 (const char *f1, const char *f2)
 {
-    gchar *c1, *c2;
-    int ret;
+    GError *err = NULL;
+    gchar *u1 = NULL, *u2 = NULL;
+    gchar *c1 = NULL, *c2 = NULL;
+    gsize bytes;
+    int ret = 0;
 
-    c1 = g_utf8_casefold(f1, -1);
-    c2 = g_utf8_casefold(f2, -1);
+    u1 = g_locale_to_utf8(f1, -1, NULL, &bytes, &err);
+    if (err != NULL) {
+	errbox(err->message);
+	g_error_free(err);
+	return 0;
+    }
+
+    u2 = g_locale_to_utf8(f2, -1, NULL, &bytes, &err);
+    if (err != NULL) {
+	errbox(err->message);
+	g_error_free(err);
+	g_free(u1);
+	return 0;
+    }
+
+    c1 = g_utf8_casefold(u1, -1);
+    c2 = g_utf8_casefold(u2, -1);
 
     ret = strcmp(c1, c2);
 
+    g_free(u1);
+    g_free(u2);
     g_free(c1);
     g_free(c2);
 
@@ -926,32 +951,4 @@ int win32_open_file (const char *fname)
     return err;
 }
 
-int win32_delete_dir (const char *path)
-{
-    SHFILEOPSTRUCT op;
-    char *from;
-    int err = 0;
-
-    from = calloc(strlen(path) + 2, 1);
-    if (from == NULL) {
-	return E_ALLOC;
-    }
-
-    strcpy(from, path);
-
-    op.hwnd = NULL;
-    op.wFunc = FO_DELETE;
-    op.pFrom = from;
-    op.pTo = NULL;
-    op.fFlags = FOF_SILENT | FOF_NOCONFIRMATION | FOF_NOERRORUI;
-    op.fAnyOperationsAborted = FALSE;
-    op.hNameMappings = NULL;
-    op.lpszProgressTitle = NULL;
-
-    err = SHFileOperation(&op);
-
-    free(from);
-
-    return err;
-}
 
