@@ -1,24 +1,26 @@
-/*
- *  Copyright (c) by Allin Cottrell
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
- *
- *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
- *
+/* 
+ *  gretl -- Gnu Regression, Econometrics and Time-series Library
+ *  Copyright (C) 2001 Allin Cottrell and Riccardo "Jack" Lucchetti
+ * 
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ * 
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ * 
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 #include "gretl.h"
 
+#ifdef ENABLE_NLS
+
 static int seven_bit_string (const unsigned char *s)
 {
     while (*s) {
@@ -108,6 +110,7 @@ int maybe_recode_file (const char *fname)
 
 gchar *my_filename_from_utf8 (char *fname)
 {
+    const gchar *cset;
     gchar *trfname = NULL;
     GError *err = NULL;
     gsize bytes;
@@ -116,6 +119,11 @@ gchar *my_filename_from_utf8 (char *fname)
 	return fname;
     }
 
+    if (g_get_charset(&cset)) {
+	/* is this right? */
+	return fname;
+    }
+
     trfname = g_filename_from_utf8(fname, -1, NULL, &bytes, &err);
 
     if (err != NULL) {
@@ -157,43 +165,35 @@ gchar *my_locale_from_utf8 (const gchar *src)
     return trstr;
 }
 
-#define FNAME_DEBUG 0
+/* returns new copy of fname, converted if need be */
 
-gchar *my_filename_to_utf8 (char *fname)
+gchar *my_filename_to_utf8 (const char *fname)
 {
     gchar *trfname = NULL;
     GError *err = NULL;
     gsize bytes;
 
-#if FNAME_DEBUG
-    fprintf(stderr, "my_filename_to_utf8: fname='%s'\n", fname);
-    fflush(stderr);
-#endif
-
     if (g_utf8_validate(fname, -1, NULL)) {
-#if FNAME_DEBUG
-	fprintf(stderr, " validates as utf8, returning fname\n");
-	fflush(stderr);
+	trfname = g_strdup(fname);
+    } else {
+	/* On Windows, with GTK >= 2.6, the GLib filename
+	   encoding is UTF-8; however, filenames coming from
+	   a native Windows file dialog will be in the
+	   locale charset 
+	*/
+#if defined(G_OS_WIN32) && GTK_MINOR_VERSION >= 6
+	trfname = g_locale_to_utf8(fname, -1, NULL, &bytes, &err);
+#else
+	trfname = g_filename_to_utf8(fname, -1, NULL, &bytes, &err);
 #endif
-	return fname;
     }
 
-    trfname = g_filename_to_utf8(fname, -1, NULL, &bytes, &err);
-
     if (err != NULL) {
-	errbox("g_filename_to_utf8 failed");
+	errbox(err->message);
 	g_error_free(err);
-    } else {
-	strcpy(fname, trfname);
-#if FNAME_DEBUG
-	fprintf(stderr, " converted fname='%s'\n", fname);
-	fflush(stderr);
-#endif
-    }
+    } 
 
-    g_free(trfname);
-
-    return fname;
+    return trfname;
 }
 
 static gchar *real_my_locale_to_utf8 (const gchar *src,
@@ -232,7 +232,7 @@ static gchar *real_my_locale_to_utf8 (const gchar *src,
 static const char *gp_cset (void)
 {
     if (iso_latin_version() == 2) {
-#ifdef GO_OS_WIN32
+#ifdef G_OS_WIN32
 	return "CP1250";
 #else
 	return "ISO-8859-2";
@@ -330,13 +330,13 @@ gchar *latin2_to_utf8 (const gchar *src)
 {
     gsize read, wrote;
 
-#ifdef G_OS_WIN32
+# ifdef G_OS_WIN32
     return g_convert(src, -1, "UTF-8", "CP1250",
 		     &read, &wrote, NULL);
-#else
+# else
     return g_convert(src, -1, "UTF-8", "ISO-8859-2",
 		     &read, &wrote, NULL);
-#endif
+# endif
 }
 
 int html_encoded (const char *s)
@@ -351,4 +351,6 @@ int html_encoded (const char *s)
     return 0;
 }
 
+#endif /* ENABLE_NLS */
+
 
