@@ -9,7 +9,7 @@ gchar *my_locale_from_utf8 (const gchar *src);
 
 gchar *force_locale_from_utf8 (const gchar *src);
 
-gchar *my_filename_to_utf8 (char *fname);
+gchar *my_filename_to_utf8 (const char *fname);
 
 gchar *my_locale_to_utf8 (const gchar *src);
 
@@ -27,4 +27,4 @@ gchar *latin2_to_utf8 (const gchar *src);
 
 int html_encoded (const char *s);
 
-#endif
+#endif /* GUI_RECODE_H */
