@@ -1,20 +1,20 @@
-/*
- *  Copyright (c) by Ramu Ramanathan and Allin Cottrell
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
- *
- *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111, USA.
- *
+/* 
+ *  gretl -- Gnu Regression, Econometrics and Time-series Library
+ *  Copyright (C) 2001 Allin Cottrell and Riccardo "Jack" Lucchetti
+ * 
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ * 
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ * 
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 /*  guiprint.c - RTF and LaTeX generation for gretl, plus native
@@ -29,10 +29,13 @@
 
 #ifdef G_OS_WIN32
 # include <windows.h>
+# include "gretlwin32.h"
 #endif
 
-#if defined(G_OS_WIN32) || defined(USE_GNOME)
-#define NATIVE_PRINTING
+#ifndef GTK_PRINTING
+# ifdef USE_GNOME
+#  define USE_GNOMEPRINT
+# endif
 #endif
 
 #ifdef NATIVE_PRINTING
@@ -250,7 +253,7 @@ int winprint_graph (char *emfname)
 
     hemf = GetEnhMetaFile(emfname);
     if (hemf == NULL) {
-	errbox(_("Couldn't open %s"), emfname);
+	file_read_errbox(emfname);
 	return 1;
     }
 
@@ -329,7 +332,7 @@ int winprint_graph (char *emfname)
     return !printok;
 }
 
-#elif defined(USE_GNOME)
+#elif defined(USE_GNOMEPRINT)
 
 #include <libgnomeprint/gnome-print.h>
 #include <libgnomeprint/gnome-print-job.h>
@@ -342,6 +345,7 @@ int winprint_graph (char *emfname)
 
 #define GRETL_PRINT_CONFIG_FILE "gretl-print-config"
 #define GRETL_PBM_TMP           "gretltmp.pbm"
+#define GRETL_PNG_TMP "gretltmp.png"
 
 static GdkPixbuf *png_mono_pixbuf (const char *fname);
 
@@ -451,8 +455,8 @@ void winprint (char *fullbuf, char *selbuf)
     gnome_print_setfont(gpc, font);
     /* gnome_print_setrgbcolor(gpc, 0, 0, 0); */
 
-    if (selbuf != NULL) p = selbuf;
-    else p = fullbuf;
+    p = (selbuf != NULL)? selbuf : fullbuf;
+
     page = 1;
     x = 72;
     hdrstart = header_string();
@@ -528,7 +532,7 @@ print_image_from_pixbuf (GnomePrintContext *gpc, GdkPixbuf *pixbuf)
     }
 }
 
-void gnome_print_graph (const char *fname)
+void gtk_print_graph (const char *fname)
 {
     GnomePrintJob *job;
     GnomePrintConfig *config;
@@ -610,9 +614,9 @@ void gnome_print_graph (const char *fname)
     gtk_widget_destroy(dialog);
 }
 
-#endif /* G_OS_WIN32, USE_GNOME */
+#endif /* G_OS_WIN32, USE_GNOMEPRINT */
 
-#ifdef USE_GNOME
+#ifdef USE_GNOMEPRINT
 
 static GdkPixbuf *png_mono_pixbuf (const char *fname)
 {
@@ -664,7 +668,318 @@ static GdkPixbuf *png_mono_pixbuf (const char *fname)
     return pbuf;
 }
 
-#endif /* USE_GNOME */
+#endif /* USE_GNOMEPRINT */
+
+#ifdef GTK_PRINTING
+
+#define GRETL_PNG_TMP "gretltmp.png"
+
+struct print_info {
+    int n_pages;
+    int pagelines;
+    gdouble x, y;
+    const char *buf;
+    const char *p;
+    char *hdr;
+    cairo_t *cr;
+    PangoLayout *layout;
+};
+
+static void begin_text_print (GtkPrintOperation *op,
+			      GtkPrintContext *context,
+			      struct print_info *pinfo)
+{
+    PangoFontDescription *desc;
+    GtkPageSetup *setup;
+    gdouble x, y;
+    int lines = 0;
+    const char *p;
+
+    pinfo->pagelines = 54; /* FIXME? */
+ 
+    setup = gtk_print_context_get_page_setup(context);
+
+    x = gtk_page_setup_get_left_margin(setup, GTK_UNIT_POINTS);
+    pinfo->x = 72 - x; /* pad left to 72 points */
+    if (pinfo->x < 0) {
+	pinfo->x = 0;
+    }
+
+    y = gtk_page_setup_get_top_margin(setup, GTK_UNIT_POINTS);
+    pinfo->y = 26 - y; /* pad top to 26 points */
+    if (pinfo->y < 0) {
+	pinfo->y = 0;
+    }
+
+#if 0  
+    /* redundant? */
+    gtk_page_setup_set_top_margin(setup, 72, GTK_UNIT_POINTS);
+    gtk_page_setup_set_bottom_margin(setup, 72, GTK_UNIT_POINTS);
+    gtk_page_setup_set_left_margin(setup, 72, GTK_UNIT_POINTS);
+    gtk_page_setup_set_right_margin(setup, 72, GTK_UNIT_POINTS);
+
+    gdouble x = gtk_print_context_get_width(context);
+    fprintf(stderr, "context width = %g\n", x);
+    x = gtk_print_context_get_height(context);
+    fprintf(stderr, "context height = %g\n", x);
+#endif
+
+    pinfo->cr = gtk_print_context_get_cairo_context(context);
+    cairo_set_source_rgb(pinfo->cr, 0, 0, 0);
+
+    pinfo->layout = gtk_print_context_create_pango_layout(context);
+
+    /* FIXME let the user choose a font? */
+    desc = pango_font_description_from_string("mono 10");
+    pango_layout_set_font_description(pinfo->layout, desc);
+    pango_font_description_free(desc);
+    pango_layout_set_width(pinfo->layout, -1);
+    pango_layout_set_alignment(pinfo->layout, PANGO_ALIGN_LEFT);
+
+    p = pinfo->buf;
+    while (*p) {
+	if (*p == '\n') {
+	    lines++;
+	}
+	p++;
+    }
+
+    pinfo->n_pages = lines / pinfo->pagelines + 
+	(lines % pinfo->pagelines != 0);
+    gtk_print_operation_set_n_pages(op, pinfo->n_pages);
+}
+
+static void
+draw_text_page (GtkPrintOperation *op, GtkPrintContext *context,
+		gint pagenum, struct print_info *pinfo)
+{
+    gchar *hdr;
+    gdouble y = pinfo->y;
+    gint lheight;
+
+    hdr = g_strdup_printf(_("%s page %d of %d"), pinfo->hdr,
+			  pagenum + 1, pinfo->n_pages);
+    pango_layout_set_text(pinfo->layout, hdr, -1);
+    g_free(hdr);
+
+    cairo_move_to(pinfo->cr, pinfo->x, y);
+    pango_cairo_show_layout(pinfo->cr, pinfo->layout);
+
+    pango_layout_get_size(pinfo->layout, NULL, &lheight);
+    y += 8 + (gdouble) lheight / PANGO_SCALE;
+
+    if (pinfo->n_pages - pagenum > 1) {
+	/* carve out the current page */
+	const char *p = pinfo->p;
+	int nc = 0, nl = 0;
+
+	while (*p && nl <= pinfo->pagelines) {
+	    if (*p == '\n') {
+		nl++;
+	    }
+	    nc++;
+	    p++;
+	}
+	pango_layout_set_text(pinfo->layout, pinfo->p, nc);
+	pinfo->p += nc;
+    } else {
+	/* print all that's left */
+	pango_layout_set_text(pinfo->layout, pinfo->p, -1);
+    }
+
+    cairo_move_to(pinfo->cr, pinfo->x, y);
+    pango_cairo_show_layout(pinfo->cr, pinfo->layout);
+}
+
+static GtkPrintSettings *settings = NULL;
+
+void winprint (char *fullbuf, char *selbuf)
+{
+    GtkPrintOperation *op;
+    GtkPrintOperationResult res;
+    GError *err = NULL;
+    struct print_info pinfo;
+
+    op = gtk_print_operation_new();
+
+    if (settings != NULL) {
+	gtk_print_operation_set_print_settings(op, settings);
+    }
+
+    gtk_print_operation_set_use_full_page(op, FALSE);
+    gtk_print_operation_set_unit(op, GTK_UNIT_POINTS);
+
+    pinfo.buf = (selbuf != NULL)? selbuf : fullbuf;
+    pinfo.p = pinfo.buf;
+    pinfo.hdr = header_string();
+    pinfo.layout = NULL;
+
+    g_signal_connect(op, "begin_print", G_CALLBACK(begin_text_print), &pinfo);
+    g_signal_connect(op, "draw_page", G_CALLBACK(draw_text_page), &pinfo);
+
+    res = gtk_print_operation_run(op, GTK_PRINT_OPERATION_ACTION_PRINT_DIALOG,
+				  GTK_WINDOW(mdata->w), &err);
+
+    if (res == GTK_PRINT_OPERATION_RESULT_ERROR) {
+	errbox("Error printing:\n%s", err->message);
+	g_error_free(err);
+    } else if (res == GTK_PRINT_OPERATION_RESULT_APPLY) {
+	if (settings != NULL) {
+	    g_object_unref(settings);
+	}
+	settings = g_object_ref(gtk_print_operation_get_print_settings(op));
+    }
+
+    free(pinfo.hdr);
+    if (pinfo.layout != NULL) {
+	g_object_unref(G_OBJECT(pinfo.layout));
+    }
+    g_object_unref(G_OBJECT(op));
+}
+
+/* 
+   we're better off using PDF here, if possible
+*/
+
+static int make_png_file (const char *fname,
+			  char *pngname)
+{
+    FILE *fsrc, *ftmp;
+    char cmd[MAXLEN], temp[MAXLEN], fline[MAXLEN];
+
+    sprintf(temp, "%sgpttmp", paths.userdir);
+
+    ftmp = gretl_tempfile_open(temp);
+    if (ftmp == NULL) {
+	return 1;
+    }
+
+    fsrc = gretl_fopen(fname, "r");
+    if (fsrc == NULL) {
+	fclose(ftmp);
+	remove(temp);
+	return 1;
+    }
+
+    build_path(pngname, paths.userdir, GRETL_PNG_TMP, NULL);
+
+    while (fgets(fline, MAXLEN-1, fsrc)) {
+	if (!strncmp(fline, "set output", 10)) {
+	    fprintf(ftmp, "set output '%s'\n", pngname);
+	} else {
+	    fputs(fline, ftmp);
+	}
+    }
+
+    fclose(fsrc);
+    fclose(ftmp);
+
+    /* run gnuplot on the temp plotfile */
+    sprintf(cmd, "\"%s\" \"%s\"", paths.gnuplot, temp);
+    if (system(cmd)) {
+	remove(temp);
+	return 1;
+    }
+
+    remove(temp);
+
+    return 0;
+}
+
+static void begin_image_print (GtkPrintOperation *op,
+			       GtkPrintContext *context,
+			       char *pngname)
+{
+    GtkPageSetup *setup;
+    cairo_surface_t *cs;
+    cairo_t *cr;
+    gdouble x, y;
+
+    setup = gtk_print_context_get_page_setup(context);
+
+    x = gtk_print_context_get_width(context);
+    y = gtk_print_context_get_height(context);
+
+#if 0
+    fprintf(stderr, "context width = %g\n", x);
+    fprintf(stderr, "context height = %g\n", y);
+#endif
+
+    cs = cairo_image_surface_create_from_png(pngname);
+    if (cairo_surface_status(cs) != CAIRO_STATUS_SUCCESS) {
+	errbox("Error reading image");
+	return;
+    }
+
+    x = cairo_image_surface_get_width(cs);
+    y = cairo_image_surface_get_height(cs);
+
+#if 0
+    fprintf(stderr, "surface: width=%g, height=%g\n", x, y);
+#endif
+
+    /* FIXME get this aligned properly!! */
+
+    cr = gtk_print_context_get_cairo_context(context);
+    cairo_translate(cr, x/2 - 20, y/2 + 200);
+    cairo_rotate(cr, -M_PI / 2);
+    cairo_set_source_surface(cr, cs, -x/2, -y/2);
+    cairo_set_antialias(cr, CAIRO_ANTIALIAS_NONE);
+
+    gtk_print_operation_set_n_pages(op, 1);
+}
+
+static void
+draw_image (GtkPrintOperation *op, GtkPrintContext *context,
+	    gint page, gpointer p)
+{
+    cairo_t *cr;
+
+    cr = gtk_print_context_get_cairo_context(context);
+    cairo_paint(cr);
+}
+
+void gtk_print_graph (const char *fname)
+{
+    char pngname[FILENAME_MAX];
+    GtkPrintOperation *op;
+    GtkPrintOperationResult res;
+    GError *err = NULL;
+
+    if (make_png_file(fname, pngname)) {
+	errbox("Error creating graph file");
+	return;
+    }
+
+    op = gtk_print_operation_new();
+
+    if (settings != NULL) {
+	gtk_print_operation_set_print_settings(op, settings);
+    }
+
+    gtk_print_operation_set_use_full_page(op, TRUE);
+    gtk_print_operation_set_unit(op, GTK_UNIT_POINTS);
+
+    g_signal_connect(op, "begin_print", G_CALLBACK(begin_image_print), pngname);
+    g_signal_connect(op, "draw_page", G_CALLBACK(draw_image), NULL);
+
+    res = gtk_print_operation_run(op, GTK_PRINT_OPERATION_ACTION_PRINT_DIALOG,
+				  GTK_WINDOW(mdata->w), &err);
+
+    if (res == GTK_PRINT_OPERATION_RESULT_ERROR) {
+	errbox("Error printing:\n%s", err->message);
+	g_error_free(err);
+    } else if (res == GTK_PRINT_OPERATION_RESULT_APPLY) {
+	if (settings != NULL) {
+	    g_object_unref(settings);
+	}
+	settings = g_object_ref(gtk_print_operation_get_print_settings(op));
+    }
+
+    g_object_unref(op);
+}
+
+#endif /* GTK_PRINTING */
 
 void rtf_print_obs_marker (int t, const DATAINFO *pdinfo, PRN *prn)
 {
@@ -1403,8 +1718,10 @@ static void texprint_fcast_with_errs (const FITRESID *fr,
 	    /* xgettext:no-c-format */
 	    I_("95\\% confidence interval"));
 
-    pputs(prn, "& & & & \\multicolumn{1}{c}{low} & "
-	  "\\multicolumn{1}{c}{high} \\\\\n");
+    pprintf(prn, "& & & & \\multicolumn{1}{c}{%s} & "
+	    "\\multicolumn{1}{c}{%s} \\\\\n", 
+	    I_("low"), I_("high"));
+
 
     if (pmax < 4) {
 	errpmax = pmax + 1;
@@ -1834,5 +2151,3 @@ int csv_copy_listed_vars (windata_t *vwin, int fmt, int action)
 
     return err;
 }
-
-
