@@ -1,20 +1,20 @@
-/*
- *  Copyright (c) by Allin Cottrell
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
- *
- *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
- *
+/* 
+ *  gretl -- Gnu Regression, Econometrics and Time-series Library
+ *  Copyright (C) 2001 Allin Cottrell and Riccardo "Jack" Lucchetti
+ * 
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ * 
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ * 
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 /* helpfiles.c for gretl */
@@ -120,6 +120,8 @@ static struct gui_help_item gui_help_items[] = {
     { PANEL_WLS,      "panel-wls" },
     { PANEL_B,        "panel-between" },
     { BOOTSTRAP,      "bootstrap" },
+    { TRANSPOS,       "transpos" },
+    { DATASORT,       "datasort" },
     { -1,          NULL },
 };
 
@@ -305,7 +307,7 @@ static int get_byte_positions (help_head **heads, int nh, int gui, FILE *fp)
     char word[16];
     int m = 0, n = 0;
     int c, cbak = 0;
-    int i, j, b = 0;
+    int i, j = 0, b = 0;
     int err = 0;
 
     rewind(fp);
@@ -635,7 +637,7 @@ static int real_helpfile_init (int gui, int en)
     err = get_helpfile_structure(&heads, gui, helpfile);
 
     if (err) {
-	errbox(_("help file %s is not up to date\n"), helpfile);
+	warnbox(_("help file %s is not up to date\n"), helpfile);
     } else {
 	if (gui) {
 	    if (en) {
@@ -1112,6 +1114,7 @@ gint edit_script_help (GtkWidget *widget, GdkEventButton *b,
 	    if (!gtk_text_iter_starts_word(&iter)) {
 		gtk_text_iter_backward_word_start(&w_start);
 	    }
+
 	    if (!gtk_text_iter_ends_word(&iter)) {
 		gtk_text_iter_forward_word_end(&w_end);
 	    }
