@@ -1,20 +1,20 @@
-/*
- *  Copyright (c) by Allin Cottrell
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
- *
- *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- *
+/* 
+ *  gretl -- Gnu Regression, Econometrics and Time-series Library
+ *  Copyright (C) 2001 Allin Cottrell and Riccardo "Jack" Lucchetti
+ * 
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ * 
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ * 
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 /* library.h for gretl */
@@ -65,6 +65,8 @@ void do_menu_op (gpointer data, guint action, GtkWidget *w);
 
 void do_run_script (GtkWidget *w, gpointer data);
 
+void run_script_fragment (windata_t *vwin, gchar *buf);
+
 void gui_errmsg (const int errcode);
 
 void register_graph (void);
@@ -91,6 +93,8 @@ int maybe_restore_full_data (int action);
 
 void gui_transpose_data (gpointer p, guint u, GtkWidget *w);
 
+void gui_sort_data (gpointer p, guint u, GtkWidget *w);
+
 int dataset_is_subsampled (void);
 
 void set_original_n (int n);
@@ -157,7 +161,7 @@ void do_coeff_intervals (gpointer data, guint i, GtkWidget *w);
 
 void do_panel_diagnostics (gpointer data, guint u, GtkWidget *w);
 
-int do_spearman (selector *sr);
+int do_rankcorr (selector *sr);
 
 int out_of_sample_info (int add_ok, int *t2);
 
@@ -169,8 +173,6 @@ void do_genr (GtkWidget *w, dialog_t *dlg);
 
 void do_model_genr (GtkWidget *w, dialog_t *dlg);
 
-void add_rand_series (GtkWidget *w, dialog_t *dlg);
-
 void do_global_setmiss (GtkWidget *w, dialog_t *dlg);
 
 void do_variable_setmiss (GtkWidget *w, dialog_t *dlg);
@@ -267,7 +269,7 @@ void open_info (gpointer p, guint edit, GtkWidget *w);
 
 void do_new_script (gpointer p, guint action, GtkWidget *w);
 
-void do_open_csv_box (char *fname, int code, int append);
+void do_open_csv_octave (char *fname, int code, int append);
 
 int do_store (char *mydatfile, gretlopt oflag);
 
@@ -291,6 +293,8 @@ void add_pca_data (windata_t *vwin);
 
 void add_fcast_data (windata_t *vwin);
 
+void VECM_add_EC_data (gpointer p, int i, GtkWidget *w);
+
 void maybe_display_string_table (void);
 
 #endif /* LIBRARY_H */
