@@ -1,20 +1,20 @@
-/*
- *  Copyright (c) by Allin Cottrell
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
- *
- *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
- *
+/* 
+ *  gretl -- Gnu Regression, Econometrics and Time-series Library
+ *  Copyright (C) 2001 Allin Cottrell and Riccardo "Jack" Lucchetti
+ * 
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ * 
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ * 
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 /* menustate.c: status of menus etc. */
@@ -160,7 +160,7 @@ static GtkItemFactoryEntry panel_model_items[] = {
 
 #define extended_ts(d) ((d)->structure == TIME_SERIES || \
 			(d)->structure == SPECIAL_TIME_SERIES || \
-                        (d)->structure == STACKED_TIME_SERIES)
+			(d)->structure == STACKED_TIME_SERIES)
 
 #define tramo_ts(d) ((d)->structure == TIME_SERIES && \
                      (d->pd == 1 || d->pd == 4 || d->pd == 12))
@@ -252,6 +252,8 @@ void ts_or_panel_menu_state (gboolean s)
 {
     if (mdata->ifac == NULL) return;
 
+    flip(mdata->ifac, "/Data/Sort data...", !s);
+
     flip(mdata->ifac, "/Add/Time trend", s);
     flip(mdata->ifac, "/Add/Lags of selected variables", s);
     flip(mdata->ifac, "/Add/First differences of selected variables", s);
@@ -478,40 +480,40 @@ void clear_sample_label (void)
 void set_sample_label (DATAINFO *pdinfo)
 {
     GtkWidget *dlabel = g_object_get_data(G_OBJECT(mdata->w), "dlabel");
-    char pdstr[16];
     char stobs[OBSLEN], endobs[OBSLEN];
-    char labeltxt[80];
+    char labeltxt[128];
+    const char *pdstr;
 
     ntodate(stobs, 0, pdinfo);
     ntodate(endobs, pdinfo->n - 1, pdinfo);
 
     if (custom_time_series(pdinfo)) {
-	strcpy(pdstr, _("Time series"));
+	pdstr = N_("Time series");
     } else if (dataset_is_time_series(pdinfo)) {
 	switch (pdinfo->pd) {
 	case 1:
-	    strcpy(pdstr, _("Annual")); break;
+	    pdstr = N_("Annual"); break;
 	case 4:
-	    strcpy(pdstr, _("Quarterly")); break;
+	    pdstr = N_("Quarterly"); break;
 	case 12:
-	    strcpy(pdstr, _("Monthly")); break;
+	    pdstr = N_("Monthly"); break;
 	case 24:
-	    strcpy(pdstr, _("Hourly")); break;
+	    pdstr = N_("Hourly"); break;
 	case 52:
-	    strcpy(pdstr, _("Weekly")); break;
+	    pdstr = N_("Weekly"); break;
 	case 5:
 	case 6:
 	case 7:
-	    strcpy(pdstr, _("Daily")); break;
+	    pdstr = N_("Daily"); break;
 	case 10:
-	    strcpy(pdstr, _("Decennial")); break;
+	    pdstr = N_("Decennial"); break;
 	default:
-	    strcpy(pdstr, _("Unknown")); break;
+	    pdstr = N_("Unknown"); break;
 	}
     } else if (dataset_is_panel(pdinfo)) {
-	strcpy(pdstr, _("Panel"));
+	pdstr = N_("Panel");
     } else {
-	strcpy(pdstr, _("Undated"));
+	pdstr = N_("Undated");
     }
 
     time_series_menu_state(dataset_is_time_series(pdinfo));
@@ -529,12 +531,12 @@ void set_sample_label (DATAINFO *pdinfo)
 			    " n = %d"), get_full_length_n(), datainfo->n);
     } else {
 	sprintf(labeltxt, _("%s: Full range %s - %s"), 
-		pdstr, stobs, endobs);
+		_(pdstr), stobs, endobs);
     }
 
     if (pdinfo->t1 > 0 || pdinfo->t2 < pdinfo->n - 1) {
 	char t1str[OBSLEN], t2str[OBSLEN];
-	char biglabel[128];
+	char biglabel[160];
 
 	ntodate(t1str, pdinfo->t1, pdinfo);
 	ntodate(t2str, pdinfo->t2, pdinfo);
@@ -546,11 +548,17 @@ void set_sample_label (DATAINFO *pdinfo)
 
     if (strlen(paths.datfile) > 2) {
 	/* data file open already */
-	if (strrchr(paths.datfile, SLASH) == NULL) {
-	    sprintf(labeltxt, " %s ", paths.datfile);
+	const char *p = strrchr(paths.datfile, SLASH);
+	gchar *trfname;
+
+	if (p != NULL) {
+	    trfname = my_filename_to_utf8(p + 1);
 	} else {
-	    sprintf(labeltxt, " %s ", strrchr(paths.datfile, SLASH) + 1);
+	    trfname = my_filename_to_utf8(paths.datfile);
 	}
+
+	sprintf(labeltxt, " %s ", trfname);
+	g_free(trfname);
 	if (data_status & MODIFIED_DATA) { 
 	    strcat(labeltxt, "* ");
 	} 
