@@ -1,20 +1,20 @@
-/*
- *  Copyright (c) by Allin Cottrell
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
- *
- *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- *
+/* 
+ *  gretl -- Gnu Regression, Econometrics and Time-series Library
+ *  Copyright (C) 2001 Allin Cottrell and Riccardo "Jack" Lucchetti
+ * 
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ * 
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ * 
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 /* session.c for gretl */
@@ -56,8 +56,6 @@
 # include "gretlwin32.h"
 #endif
 
-static void auto_save_gp (windata_t *vwin);
-
 #include "../pixmaps/model.xpm"
 #include "../pixmaps/boxplot.xpm"
 #include "../pixmaps/gnuplot.xpm"
@@ -72,7 +70,7 @@ static void auto_save_gp (windata_t *vwin);
 
 #define SESSION_DEBUG 0
 
-#define SHOWNAMELEN 12
+#define SHOWNAMELEN 15
 #define ICONVIEW_MIN_COLS 4
 
 typedef struct SESSION_ SESSION;
@@ -127,6 +125,7 @@ struct gui_obj_ {
 };
 
 struct sample_info {
+    char datafile[MAXLEN];
     int t1;
     int t2;
     char *mask;
@@ -158,6 +157,7 @@ static char *global_items[] = {
 static char *model_items[] = {
     N_("Display"),
     N_("Add to model table"),
+    N_("Rename"),
     N_("Delete")
 };
 
@@ -175,14 +175,16 @@ static char *graph_page_items[] = {
     N_("Help")
 };
 
-static char *var_items[] = {
+static char *generic_items[] = {
     N_("Display"),
+    N_("Rename"),
     N_("Delete")
 };
 
 static char *graph_items[] = {
     N_("Display"),
     N_("Edit plot commands"),
+    N_("Rename"),
     N_("Delete")
 };
 
@@ -202,6 +204,7 @@ static char *matrix_items[] = {
     N_("View"),
     N_("Edit"),
     N_("Properties"),
+    N_("Rename"),
     N_("Delete")
 };
 
@@ -216,7 +219,7 @@ static GtkWidget *icon_table;
 static GtkWidget *global_popup;
 static GtkWidget *model_popup;
 static GtkWidget *model_table_popup;
-static GtkWidget *var_popup;
+static GtkWidget *generic_popup;
 static GtkWidget *graph_popup;
 static GtkWidget *graph_page_popup;
 static GtkWidget *boxplot_popup;
@@ -225,14 +228,14 @@ static GtkWidget *info_popup;
 static GtkWidget *matrix_popup;
 static GtkWidget *save_item;
 
-static GList *icon_list;
+static GList *iconlist;
 static gui_obj *active_object;
 static gint iconview_width;
 static gint iconview_cols;
 static gint in_icon;
 
 /* private functions */
-static gui_obj *gui_object_new (gchar *name, int sort);
+static gui_obj *gui_object_new (gchar *name, int sort, gpointer data);
 static gui_obj *session_add_icon (gpointer data, int sort, int mode);
 static void session_build_popups (void);
 static void global_popup_activated (GtkWidget *widget, gpointer data);
@@ -247,7 +250,7 @@ static gboolean session_view_click (GtkWidget *widget,
 				    GdkEventButton *event,
 				    gpointer data);
 static int real_delete_model_from_session (SESSION_MODEL *model);
-static void rename_session_object (gui_obj *obj, const char *newname);
+static void make_short_label_string (char *targ, const char *src);
 
 static int session_graph_count;
 static int session_bplot_count;
@@ -544,13 +547,105 @@ int real_add_text_to_session (PRN *prn, const char *tname)
     
     mark_session_changed();
 
-    if (icon_list != NULL && !replace) {
+    if (iconlist != NULL && !replace) {
 	session_add_icon(text, GRETL_OBJ_TEXT, ICON_ADD_SINGLE);
     }
 
     return (replace)? ADD_OBJECT_REPLACE : ADD_OBJECT_OK;
 }
 
+static int ends_with_number (const char *s, int *k)
+{
+    int ret = 0;
+
+    s = strrchr(s, '(');
+
+    if (s != NULL) {
+	int n = strspn(s + 1, "0123456789");
+	
+	if (n > 0 && *(s + n + 1) == ')' && *(s + n + 2) == '\0') {
+	    sscanf(s + 1, "%d", k);
+	    ret = 1;
+	}
+    }
+
+    return ret;
+}
+
+static void ensure_unique_text_name (char *tname)
+{
+    char tmp[MAXSAVENAME];
+    const char *p, *oldname;
+    int i, n, id, idmax = 0;
+
+    for (i=0; i<session.ntexts; i++) {
+	id = 0;
+	oldname = session.texts[i]->name;
+	if (!strcmp(tname, oldname)) {
+	    id = 1;
+	} else if (ends_with_number(oldname, &id)) {
+	    p = strrchr(oldname, '(');
+	    *tmp = '\0';
+	    strncat(tmp, oldname, p - oldname);
+	    if (strcmp(tmp, tname)) {
+		id = 0;
+	    }
+	}
+	if (id > idmax) {
+	    idmax = id;
+	}	
+    }
+
+    if (idmax > 0) {
+	char num[8];
+
+	sprintf(num, "(%d)", ++idmax);
+	n = MAXSAVENAME - strlen(num) - 1;
+	*tmp = '\0';
+	strncat(tmp, tname, n);
+	strcat(tmp, num);
+	strcpy(tname, tmp);
+    }
+}
+
+void save_output_as_text_icon (windata_t *vwin)
+{
+    const gchar *title = NULL;
+    char tname[MAXSAVENAME];
+    gchar *buf;
+    int err;
+
+    buf = textview_get_text(vwin->w);
+    if (buf == NULL) {
+	errbox("Couldn't retrieve buffer");
+	return;
+    }
+
+    if (vwin->dialog != NULL) {
+	title = gtk_window_get_title(GTK_WINDOW(vwin->dialog));
+    }
+
+    if (title != NULL && !strncmp(title, "gretl: ", 7)) {
+	int n = strlen(title + 7);
+
+	if (n > MAXSAVENAME - 1) {
+	    n = MAXSAVENAME - 1;
+	}
+	*tname = '\0';
+	strncat(tname, title + 7, n);
+    } else {
+	strcpy(tname, "text");
+    }
+
+    ensure_unique_text_name(tname);
+    err = session_append_text(tname, buf);
+
+    if (!err && iconlist != NULL) {
+	SESSION_TEXT * text = get_session_text_by_name(tname);
+
+	session_add_icon(text, GRETL_OBJ_TEXT, ICON_ADD_SINGLE);
+    }
+}
 
 static int real_add_model_to_session (void *ptr, const char *name,
 				      GretlObjType type)
@@ -567,7 +662,7 @@ static int real_add_model_to_session (void *ptr, const char *name,
 	return 1;
     } 
 
-    if (icon_list != NULL) {
+    if (iconlist != NULL) {
 	session_add_icon(mod, type, ICON_ADD_SINGLE);
     }
 
@@ -594,7 +689,7 @@ real_add_graph_to_session (const char *fname, const char *grname,
 
     mark_session_changed();
 
-    if (icon_list != NULL && !replace) {
+    if (iconlist != NULL && !replace) {
 	session_add_icon(graph, type, ICON_ADD_SINGLE);
     }
 
@@ -636,7 +731,7 @@ int add_graph_to_session (char *fname, char *fullname)
     if (Z == NULL) {
 	/* we may be called via the "stats calculator" when
 	   there's no dataset yet */
-	err = open_nulldata(&Z, datainfo, DATA_NONE, 10, NULL);
+	err = open_nulldata(&Z, datainfo, 0, 10, NULL);
 	if (err) {
 	    gui_errmsg(err);
 	    return 1;
@@ -874,9 +969,11 @@ static int unzip_session_file (const char *fname)
 
     err = (*gretl_unzip_file)(fname, &gerr);
     if (gerr != NULL) {
+	fprintf(stderr, "gretl_unzip_file: '%s'\n", gerr->message);
 	ret = SAVEFILE_ERROR;
 	g_error_free(gerr);
     } else if (err) {
+	fprintf(stderr, "gretl_unzip_file: err = %d\n", err);
 	ret = SAVEFILE_ERROR;
     }
 
@@ -889,6 +986,7 @@ static int unzip_session_file (const char *fname)
 
 static void sinfo_init (struct sample_info *sinfo)
 {
+    strcpy(sinfo->datafile, "data.gdt");
     sinfo->t1 = 0;
     sinfo->t2 = 0;
     sinfo->mask = NULL;
@@ -936,7 +1034,7 @@ void do_open_session (void)
     if (fp != NULL) {
 	fclose(fp);
     } else {
-	errbox(_("Couldn't open %s\n"), tryfile);
+	file_read_errbox(tryfile);
 	delete_from_filelist(FILE_LIST_SESSION, tryfile);
 	delete_from_filelist(FILE_LIST_SCRIPT, tryfile);
 	return;
@@ -953,7 +1051,7 @@ void do_open_session (void)
 
     if (status == SAVEFILE_ERROR) {
 	/* FIXME more explicit error message */
-	errbox(_("Couldn't open %s"), sessionfile);
+	file_read_errbox(sessionfile);
 	return;
     } else if (status == SAVEFILE_SCRIPT) {
 	strcpy(scriptfile, sessionfile);
@@ -967,7 +1065,8 @@ void do_open_session (void)
     
     err = set_session_dirname(sname);
     if (err) {
-	errbox(_("Couldn't open %s"), "session.xml");
+	fprintf(stderr, "Failed on set_session_dirname\n");
+	file_read_errbox("session.xml");
 	return;
     }
 
@@ -976,17 +1075,20 @@ void do_open_session (void)
 
     if (err) {
 	/* FIXME more explicit error message */
-	errbox(_("Couldn't open %s"), "session.xml");
+	fprintf(stderr, "Failed on read_session_xml\n");
+	file_read_errbox("session.xml");
 	return;
     }
 
-    session_file_make_path(paths.datfile, "data.gdt");
-    err = gretl_read_gdt(&Z, &datainfo, paths.datfile, &paths, DATA_NONE, 
-			 NULL, 1);
+    session_file_make_path(paths.datfile, sinfo.datafile);
+    err = gretl_read_gdt(&Z, &datainfo, paths.datfile, &paths, OPT_P, 
+			 NULL);
     if (err) {
 	/* FIXME more explicit error message */
-	errbox(_("Couldn't open %s"), "data.gdt");
+	file_read_errbox(sinfo.datafile);
 	return;
+    } else {
+	fprintf(stderr, "Opened session datafile '%s'\n", paths.datfile);
     }
 
     session_file_make_path(fname, "matrices.xml");
@@ -1064,17 +1166,6 @@ void verify_clear_data (void)
     close_session(NULL, &Z, &datainfo);
 }
 
-#ifndef G_OS_WIN32
-
-static const char *readd (DIR *d)
-{
-    struct dirent *e = readdir(d);
-
-    return (e == NULL)? NULL : e->d_name;
-}
-
-#endif
-
 static void remove_session_dir (void)
 {
 #ifdef G_OS_WIN32
@@ -1085,23 +1176,8 @@ static void remove_session_dir (void)
     win32_delete_dir(fullpath);
     g_free(fullpath);
 #else
-    const char *fname;
-    DIR *dir;
-
     chdir(paths.userdir);
-    dir = opendir(session.dirname);
-
-    if (dir != NULL) {
-	chdir(session.dirname);
-	while ((fname = readd(dir)) != NULL) {
-	    if (strcmp(fname, ".") && strcmp(fname, "..")) {
-		remove(fname);
-	    }
-	}
-	closedir(dir);
-	chdir(paths.userdir);
-	remove(session.dirname);
-    }
+    gretl_deltree(session.dirname);
 #endif
 }
 
@@ -1175,7 +1251,7 @@ int highest_numbered_variable_in_session (void)
 {
     MODEL *pmod;
     GRETL_VAR *var;
-    gretl_equation_system *sys;
+    equation_system *sys;
     int i, mvm, vmax = 0;
 
     if (session.models) {
@@ -1191,7 +1267,7 @@ int highest_numbered_variable_in_session (void)
 	    } else if (session.models[i]->type == GRETL_OBJ_VAR) {
 		var = session.models[i]->ptr;
 		if (var != NULL) {
-		    mvm = gretl_VAR_get_highest_variable(var, datainfo);
+		    mvm = gretl_VAR_get_highest_variable(var);
 		    if (mvm > vmax) {
 			vmax = mvm;
 		    }
@@ -1341,7 +1417,7 @@ static void relpath_from_fname (char *path, const char *fname)
     }
 }
 
-static int save_session_dataset (void)
+static int save_session_dataset (const char *dname)
 {
     char tmpname[MAXLEN];
     const double **dZ = NULL;
@@ -1350,6 +1426,7 @@ static int save_session_dataset (void)
     int err = 0;
 
     /* dump current dataset into session dir */
+
     if (complex_subsampled()) {
 	/* save full version of dataset */
 	double ***fullZ = fetch_full_Z();
@@ -1365,16 +1442,49 @@ static int save_session_dataset (void)
     t2 = dinfo->t2;
     dinfo->t1 = 0;
     dinfo->t2 = dinfo->n - 1;
-    session_file_make_path(tmpname, "data.gdt");
+
+    session_file_make_path(tmpname, dname);
     err = gretl_write_gdt(tmpname, NULL, dZ, dinfo, 0, NULL);
+
+    fprintf(stderr, "Save session datafile as '%s', err = %d\n",
+	    tmpname, err);
+
     dinfo->t1 = t1;
     dinfo->t2 = t2;
     
     return err;
 }
 
+static const char *unpath (const char *fname)
+{
+    int i, n = strlen(fname);
+
+    for (i=n-1; i>=0; i--) {
+	if (fname[i] == '/') {
+	    return fname + i + 1;
+	}
+#ifdef G_OS_WIN32
+	if (fname[i] == '\\') {
+	    return fname + i + 1;
+	}
+#endif
+    }
+
+    return fname;
+}
+
+static void make_session_dataname (char *datname)
+{
+    if (*paths.datfile != '\0') {
+	strcpy(datname, unpath(paths.datfile));
+    } else {
+	strcpy(datname, "data.gdt");
+    }
+}
+
 int save_session (char *fname) 
 {
+    char datname[MAXLEN];
     char dirname[MAXLEN];
     void *handle;
     int (*gretl_make_zipfile) (const char *, const char *, GError **);
@@ -1408,8 +1518,9 @@ int save_session (char *fname)
 	strcpy(session.dirname, dirname);
     }
 
-    write_session_xml();
-    err = save_session_dataset();
+    make_session_dataname(datname);
+    write_session_xml(datname);
+    err = save_session_dataset(datname);
     
     if (!err) {
 	gretl_make_zipfile = gui_get_plugin_function("gretl_make_zipfile", 
@@ -1495,6 +1606,7 @@ static gchar *graph_str (SESSION_GRAPH *graph)
 	    } 
 	}
 
+#ifdef ENABLE_NLS
 	if (gottitle) {
 	    buf = my_locale_to_utf8(title);
 	} else if (gotxy == 2) {
@@ -1505,6 +1617,13 @@ static gchar *graph_str (SESSION_GRAPH *graph)
 		free(s);
 	    }
 	}
+#else
+	if (gottitle) {
+	    buf = g_strdup(title);
+	} else if (gotxy == 2) {
+	    buf = g_strdup_printf("%s %s %s", ylabel, _("versus"), xlabel);
+	}
+#endif
 
 	fclose(fp);
     }
@@ -1578,7 +1697,7 @@ static int display_session_model (SESSION_MODEL *sm)
 	gretl_VAR_print(var, datainfo, OPT_NONE, prn);
 	view_buffer(prn, 78, 450, sm->name, var->ci, var);
     } else if (sm->type == GRETL_OBJ_SYS) {
-	gretl_equation_system *sys = (gretl_equation_system *) sm->ptr;
+	equation_system *sys = (equation_system *) sm->ptr;
 
 	edit_dialog(sm->name, NULL, NULL, do_saved_eqn_system, sys, 
 		    SYSTEM, VARCLICK_NONE, NULL); 
@@ -1793,7 +1912,7 @@ static void maybe_delete_session_object (gui_obj *obj)
 
 static gui_obj *get_gui_obj_by_data (void *finddata)
 {
-    GList *mylist = icon_list;
+    GList *mylist = iconlist;
     gui_obj *obj = NULL;
 
     while (mylist != NULL) {
@@ -1814,9 +1933,12 @@ int session_matrix_destroy_by_name (const char *name, PRN *prn)
 
     if (u == NULL) {
 	err = E_UNKVAR;
+    } else if (winstack_match_data(u)) {
+	errbox(_("Please close this object's window first"));
+	return 0;
     } else {
 	gui_obj *obj = get_gui_obj_by_data(u);
-    
+
 	if (obj != NULL) {
 	    session_delete_icon(obj);
 	}
@@ -1839,23 +1961,90 @@ static void rename_session_graph (SESSION_GRAPH *graph, const char *newname)
     }
 }
 
-static void rename_session_object (gui_obj *obj, const char *newname)
+static int rename_session_object (gui_obj *obj, const char *newname)
 {
+    int err = 0;
+
     if (obj->sort == GRETL_OBJ_EQN || obj->sort == GRETL_OBJ_SYS || 
 	obj->sort == GRETL_OBJ_VAR) { 
-	SESSION_MODEL *sm = obj->data;
+	SESSION_MODEL *sm;
 
-	gretl_object_rename(sm->ptr, sm->type, newname);
-	sm->name[0] = '\0';
-	strncat(sm->name, newname, MAXSAVENAME - 1);
-    } else if (obj->sort == GRETL_OBJ_GRAPH || obj->sort == GRETL_OBJ_PLOT) { 
-	rename_session_graph(obj->data, newname);
+	sm = get_session_model_by_name(newname);
+	if (sm != NULL) {
+	    err = 1;
+	} else {
+	    sm = obj->data;
+	    gretl_object_rename(sm->ptr, sm->type, newname);
+	    sm->name[0] = '\0';
+	    strncat(sm->name, newname, MAXSAVENAME - 1);
+	}
+    } else if (obj->sort == GRETL_OBJ_GRAPH || obj->sort == GRETL_OBJ_PLOT) {
+	SESSION_GRAPH *sg;
+
+	sg = get_session_graph_by_name(newname);
+	if (sg != NULL) {
+	    err = 1;
+	} else {
+	    sg = obj->data;
+	    rename_session_graph(sg, newname);
+	}
     } else if (obj->sort == GRETL_OBJ_MATRIX) {
 	user_matrix_set_name(obj->data, newname);
+    } else if (obj->sort == GRETL_OBJ_TEXT) {
+	SESSION_TEXT *st;
+
+	st = get_session_text_by_name(newname);
+	if (st != NULL) {
+	    err = 1;
+	} else {
+	    st = obj->data;
+	    *st->name = '\0';
+	    strncat(st->name, newname, MAXSAVENAME - 1);
+	}
     }
 
-    free(obj->name);
-    obj->name = g_strdup(newname);
+    if (err) {
+	errbox(_("'%s': there is already an object of this name"), newname);
+    } else {
+	free(obj->name);
+	obj->name = g_strdup(newname);
+    }
+
+    return err;
+}
+
+static void rename_object_callback (GtkWidget *widget, dialog_t *dlg) 
+{
+    gui_obj *obj = (gui_obj *) edit_dialog_get_data(dlg);
+    const gchar *newname;
+    int err = 0;
+
+    newname = edit_dialog_get_text(dlg);
+
+    if (newname != NULL && *newname != '\0' &&
+	strcmp(newname, obj->name)) {
+	gchar str[SHOWNAMELEN + 1];
+
+	err = rename_session_object(obj, newname);
+	if (!err) {
+	    make_short_label_string(str, obj->name);
+	    gtk_label_set_text(GTK_LABEL(obj->label), str);
+	    mark_session_changed();
+	}
+    }
+
+    if (!err) {
+	close_dialog(dlg);
+    }
+}
+
+static void rename_object_dialog (gui_obj *obj) 
+{
+    edit_dialog (_("gretl: rename object"), 
+		 _("Enter new name\n"
+		   "(max. 31 characters)"),
+		 obj->name, rename_object_callback, obj, 
+		 0, VARCLICK_NONE, NULL);
 }
 
 void delete_text_from_session (void *p)
@@ -1888,7 +2077,7 @@ void display_saved_text (void *p)
 
 static void session_view_init (void)
 {
-    icon_list = NULL;
+    iconlist = NULL;
     icon_table = NULL;
     iconview_width = 0;
     iconview_cols = ICONVIEW_MIN_COLS;
@@ -1913,28 +2102,30 @@ static void session_view_free (GtkWidget *w, gpointer data)
 {
     iconview = NULL;
 
-    g_list_foreach(icon_list, (GFunc) gui_obj_destroy, NULL);
+    g_list_foreach(iconlist, (GFunc) gui_obj_destroy, NULL);
 
-    g_list_free(icon_list);
-    icon_list = NULL;
+    g_list_free(iconlist);
+    iconlist = NULL;
 }
 
 static void session_delete_icon (gui_obj *obj)
 {
     if (obj == NULL) return;
 
-    if (obj->icon && GTK_IS_WIDGET(obj->icon))
+    if (obj->icon != NULL && GTK_IS_WIDGET(obj->icon)) {
 	gtk_container_remove(GTK_CONTAINER(icon_table), obj->icon);
-    if (obj->label && GTK_IS_WIDGET(obj->label))
+    }
+    if (obj->label != NULL && GTK_IS_WIDGET(obj->label)) {
 	gtk_container_remove(GTK_CONTAINER(icon_table), obj->label);
+    }
 
-    icon_list = g_list_first(icon_list);
-    icon_list = g_list_remove(icon_list, obj);
+    iconlist = g_list_first(iconlist);
+    iconlist = g_list_remove(iconlist, obj);
 
     gui_obj_destroy(obj, NULL);
 
-    if (icon_list == NULL) {
-	fprintf(stderr, "Bad: icon_list has gone NULL\n");
+    if (iconlist == NULL) {
+	fprintf(stderr, "Bad: iconlist has gone NULL\n");
     }
 }
 
@@ -1990,12 +2181,12 @@ static void pack_single_icon (gui_obj *obj)
     int row, col;
     gui_obj *last;
 
-    icon_list = g_list_last(icon_list);
-    last = icon_list->data;
+    iconlist = g_list_last(iconlist);
+    last = iconlist->data;
     row = last->row;
     col = last->col;  
 
-    icon_list = g_list_append(icon_list, obj);
+    iconlist = g_list_append(iconlist, obj);
 
     col++;
     if (col > 0 && (col % iconview_cols == 0)) {
@@ -2009,7 +2200,7 @@ static void pack_single_icon (gui_obj *obj)
 
 static void batch_pack_icons (void)
 {
-    GList *mylist = g_list_first(icon_list);
+    GList *mylist = g_list_first(iconlist);
     gui_obj *obj;
     int row = 0, col = 0;
 
@@ -2107,28 +2298,16 @@ static void undisplay_icon (gui_obj *obj, gpointer p)
 
 static void rearrange_icons (void) 
 {
-    g_list_foreach(icon_list, (GFunc) undisplay_icon, NULL);
-
+    iconlist = g_list_first(iconlist);
+    g_list_foreach(iconlist, (GFunc) undisplay_icon, NULL);
     batch_pack_icons();
 }
 
 static gint catch_iconview_key (GtkWidget *w, GdkEventKey *key, 
 				gpointer p)
 {
-    /* 'q' quits, but not if we're editing a label */
-
+    /* 'q' quits iconview */
     if (key->keyval == GDK_q) { 
-	GList *mylist = icon_list;
-	gui_obj *obj = NULL;
-
-	while (mylist != NULL) {
-	    obj = (gui_obj *) mylist->data;
-	    if (gtk_editable_get_editable(GTK_EDITABLE(obj->label))) {
-		return FALSE;
-	    }
-	    mylist = mylist->next;
-	}
-
         gtk_widget_destroy(w);
     }
 
@@ -2154,7 +2333,7 @@ static void object_popup_show (gui_obj *obj, GdkEventButton *event)
     case GRETL_OBJ_VAR: 
     case GRETL_OBJ_SYS:
     case GRETL_OBJ_TEXT:
-	w = var_popup; 
+	w = generic_popup; 
 	break;
     case GRETL_OBJ_GRAPH: 
 	w = graph_popup; 
@@ -2187,7 +2366,7 @@ static void display_model_table_wrapper (void)
 static void graph_page_save_wrapper (void)
 {
     if (graph_page_get_n_graphs() == 0) {
-	errbox(_("The graph page is empty"));
+	warnbox(_("The graph page is empty"));
     } else {
 	file_selector(_("Save LaTeX file"), SAVE_TEX, FSEL_DATA_NONE, NULL);
     }
@@ -2327,6 +2506,8 @@ static void matrix_popup_activated (GtkWidget *widget, gpointer data)
     } else if (!strcmp(item, _("Properties"))) {
 	m = user_matrix_get_matrix(u);
 	view_matrix_properties(m, name);
+    } else if (!strcmp(item, _("Rename"))) {
+	rename_object_dialog(obj);
     } else if (!strcmp(item, _("Delete"))) {
 	maybe_delete_session_object(obj);
     }
@@ -2379,6 +2560,8 @@ static void object_popup_activated (GtkWidget *widget, gpointer data)
 	    view_file(fullname, 1, 0, 78, 400, 
 		      (obj->sort == GRETL_OBJ_GRAPH)? GR_PLOT : GR_BOX);
 	}
+    } else if (!strcmp(item, _("Rename"))) {
+	rename_object_dialog(obj);
     } else if (!strcmp(item, _("Delete"))) {
 	maybe_delete_session_object(obj);
     } else if (!strcmp(item, _("Add to model table"))) {
@@ -2576,7 +2759,7 @@ static gui_obj *session_add_icon (gpointer data, int sort, int mode)
 	return NULL;
     }
 
-    obj = gui_object_new(name, sort);
+    obj = gui_object_new(name, sort, data);
 
     /* full-length object name as tooltip */
     if (strlen(name) > SHOWNAMELEN) {
@@ -2584,15 +2767,6 @@ static gui_obj *session_add_icon (gpointer data, int sort, int mode)
 	icon_named = 1;
     }
 
-    /* attach specific data items */
-    if (sort == GRETL_OBJ_EQN || sort == GRETL_OBJ_VAR || sort == GRETL_OBJ_SYS) {
-	obj->data = mod;
-    } else if (sort == GRETL_OBJ_GRAPH || sort == GRETL_OBJ_PLOT) {
-	obj->data = graph;
-    } else if (sort == GRETL_OBJ_MATRIX) {
-	obj->data = um;
-    }
-
     /* set up for drag and drop */
     if (sort == GRETL_OBJ_EQN) {
 	model_drag_connect(obj->icon, obj->data);
@@ -2619,14 +2793,14 @@ static gui_obj *session_add_icon (gpointer data, int sort, int mode)
 	}
     }
 
-    if      (sort == GRETL_OBJ_TEXT)   obj->data = text;
-    else if (sort == GRETL_OBJ_DSET)   obj->data = paths.datfile;
-    else if (sort == GRETL_OBJ_MODTAB) obj->data = NULL;
+    if (sort == GRETL_OBJ_DSET) {
+	obj->data = paths.datfile;
+    }
 
     if (mode == ICON_ADD_SINGLE) {
 	pack_single_icon(obj);
     } else if (mode == ICON_ADD_BATCH) {
-	icon_list = g_list_append(icon_list, obj);
+	iconlist = g_list_append(iconlist, obj);
     }
 
     return obj;
@@ -2682,11 +2856,11 @@ static void session_build_popups (void)
 	}
     }
 
-    if (var_popup == NULL) {
-	var_popup = gtk_menu_new();
-	for (i=0; i<sizeof var_items / sizeof var_items[0]; i++) {
-	    create_popup_item(var_popup, 
-			      _(var_items[i]), 
+    if (generic_popup == NULL) {
+	generic_popup = gtk_menu_new();
+	for (i=0; i<sizeof generic_items / sizeof generic_items[0]; i++) {
+	    create_popup_item(generic_popup, 
+			      _(generic_items[i]), 
 			      object_popup_activated);
 	}
     }
@@ -2835,89 +3009,6 @@ void view_session (void)
     gtk_widget_grab_focus(icon_table);
 }
 
-/* apparatus for renaming session objects */
-
-static void size_name_entry (GtkWidget *w, const char *name)
-{
-    PangoLayout *layout;
-    PangoRectangle rect;
-    PangoFontDescription *pfd;
-    int width;
-#ifdef USE_GNOME
-    GtkSettings *settings;
-    gchar *fontname;
-
-    settings = gtk_settings_get_default();
-    g_object_get(G_OBJECT(settings), "gtk-font-name", &fontname, NULL);
-#endif    
-
-    layout = gtk_entry_get_layout(GTK_ENTRY(w));
-
-#ifdef USE_GNOME
-    pfd = pango_font_description_from_string(fontname);
-    g_free(fontname);
-#else
-    pfd = pango_font_description_from_string(get_app_fontname());
-#endif
-
-    pango_layout_set_font_description(layout, pfd);
-    pango_font_description_free(pfd);
-
-    pango_layout_get_pixel_extents(layout, NULL, &rect);
-    width = rect.width;
-    width = (width < 10)? width + 3 : 1.1 * width;
-
-    gtk_widget_set_size_request(w, width, -1); 
-}
-
-static gboolean object_name_return (GtkWidget *w,
-				    GdkEventKey *key,
-				    gui_obj *obj)
-{
-    if (!gtk_editable_get_editable(GTK_EDITABLE(obj->label))) {
-	return FALSE;
-    }
-
-    if (key->keyval == GDK_Return) {
-	const gchar *newname = gtk_entry_get_text(GTK_ENTRY(obj->label));
-
-	gtk_editable_set_position(GTK_EDITABLE(obj->label), 0);
-	gtk_entry_set_has_frame(GTK_ENTRY(obj->label), FALSE);
-	gtk_editable_set_editable(GTK_EDITABLE(obj->label), FALSE);
-	
-	if (newname != NULL && *newname != '\0' &&
-	    strcmp(newname, obj->name)) {
-	    rename_session_object(obj, newname);
-	}
-
-	size_name_entry(obj->label, newname);
-	gtk_widget_grab_focus(icon_table);
-
-	return TRUE;
-    } 
-
-    return FALSE;
-}
-
-static gboolean start_rename_object (GtkWidget *w,
-				     GdkEventButton *event,
-				     gui_obj *obj)
-{
-    if (gtk_editable_get_editable(GTK_EDITABLE(obj->label))) {
-	return FALSE;
-    }
-
-    gtk_widget_set_size_request(obj->label, -1, -1);
-    gtk_entry_set_width_chars(GTK_ENTRY(obj->label), SHOWNAMELEN);
-    gtk_entry_set_has_frame(GTK_ENTRY(obj->label), TRUE); 
-    gtk_editable_set_editable(GTK_EDITABLE(obj->label), TRUE);
-    gtk_editable_select_region(GTK_EDITABLE(obj->label), 0, -1);
-    gtk_editable_set_position(GTK_EDITABLE(obj->label), -1);
-    gtk_widget_grab_focus(obj->label);
-    
-    return TRUE;
-}
-
 static void make_short_label_string (char *targ, const char *src)
 {
     if (strlen(src) > SHOWNAMELEN) {
@@ -2931,6 +3022,7 @@ static void make_short_label_string (char *targ, const char *src)
 
 static void create_gobj_icon (gui_obj *obj, const char **xpm)
 {
+    gchar str[SHOWNAMELEN + 1];
     GdkPixbuf *pbuf;
     GtkWidget *image;
 
@@ -2949,25 +3041,8 @@ static void create_gobj_icon (gui_obj *obj, const char **xpm)
 	session_drag_setup(obj);
     }
 
-    if (obj->sort == GRETL_OBJ_EQN || obj->sort == GRETL_OBJ_GRAPH ||
-	obj->sort == GRETL_OBJ_VAR || obj->sort == GRETL_OBJ_PLOT || 
-	obj->sort == GRETL_OBJ_SYS || obj->sort == GRETL_OBJ_MATRIX) { 
-	obj->label = gtk_entry_new();
-	gtk_entry_set_text(GTK_ENTRY(obj->label), obj->name);
-	gtk_editable_set_editable(GTK_EDITABLE(obj->label), FALSE);
-	gtk_entry_set_has_frame(GTK_ENTRY(obj->label), FALSE);
-	gtk_entry_set_max_length(GTK_ENTRY(obj->label), MAXSAVENAME);
-	size_name_entry(obj->label, obj->name);
-	g_signal_connect(G_OBJECT(obj->label), "button-press-event",
-			 G_CALLBACK(start_rename_object), obj);
-	g_signal_connect(G_OBJECT(obj->label), "key-press-event",
-			 G_CALLBACK(object_name_return), obj);
-    } else {
-	gchar str[SHOWNAMELEN + 1];
-
-	make_short_label_string(str, obj->name);
-	obj->label = gtk_label_new(str);
-    }
+    make_short_label_string(str, obj->name);
+    obj->label = gtk_label_new(str);
 
     g_object_ref(obj->icon);
     g_object_ref(obj->label);
@@ -2980,7 +3055,7 @@ static void create_gobj_icon (gui_obj *obj, const char **xpm)
 		     G_CALLBACK(icon_left), obj);
 }
 
-static gui_obj *gui_object_new (gchar *name, int sort)
+static gui_obj *gui_object_new (gchar *name, int sort, gpointer data)
 {
     gui_obj *obj;
     char **xpm = NULL;
@@ -2988,7 +3063,7 @@ static gui_obj *gui_object_new (gchar *name, int sort)
     obj = mymalloc(sizeof *obj);
     obj->name = name; 
     obj->sort = sort;
-    obj->data = NULL;
+    obj->data = data;
 
 #if SESSION_DEBUG
     fprintf(stderr, "Allocated obj at %p (%s)\n", (void *) obj, obj->name);
@@ -3017,88 +3092,73 @@ static gui_obj *gui_object_new (gchar *name, int sort)
     return obj;
 } 
 
-static void auto_save_gp (windata_t *vwin)
+static char *dump_current_plot (windata_t *vwin)
 {
     FILE *fp;
-    gchar *buf;
+    char *tmpfile = NULL;
+    int gotpause = 0;
+    char bufline[1024];
+    char *buf;
 # ifdef ENABLE_NLS
     gchar *trbuf;
 # endif
 
     buf = textview_get_text(vwin->w);
-    if (buf == NULL) return;
-
-    if ((fp = gretl_fopen(vwin->fname, "w")) == NULL) {
-	g_free(buf);
-	buf = g_strdup_printf(_("Couldn't write to %s"), vwin->fname);
-	errbox(buf); 
-	g_free(buf);
-	return;
-    }
-
-# ifdef ENABLE_NLS
-    trbuf = gp_locale_from_utf8(buf);
-    if (trbuf != NULL) {
-	fputs(trbuf, fp);
-	g_free(trbuf);
-    } else {
-	fputs(buf, fp);
-    }
-# else
-    fputs(buf, fp);
-# endif
-
-    g_free(buf); 
-    fclose(fp);
-}
-
-#ifdef G_OS_WIN32
-
-static char *add_pause_to_plotfile (const char *fname)
-{
-    FILE *fin, *fout;
-    char fline[MAXLEN];
-    char *tmpfile = NULL;
-    int gotpause = 0;
-
-    fin = gretl_fopen(fname, "r");
-    if (fin == NULL) return NULL;
+    if (buf == NULL) return NULL;
 
     tmpfile = g_strdup_printf("%showtmp.gp", paths.userdir);
 
-    fout = gretl_fopen(tmpfile, "w");
-    if (fout == NULL) {
-	fclose(fin);
+    fp = gretl_fopen(tmpfile, "w");
+    if (fp == NULL) {
 	g_free(tmpfile);
+	g_free(buf);
+	file_write_errbox(tmpfile);
 	return NULL;
     }
 
-    while (fgets(fline, MAXLEN - 1, fin)) {
-	fputs(fline, fout);
-	if (strstr(fline, "pause -1")) {
+    bufgets_init(buf);
+
+    while (bufgets(bufline, sizeof bufline, buf)) {
+# ifdef ENABLE_NLS
+	trbuf = gp_locale_from_utf8(bufline);
+	if (trbuf != NULL) {
+	    fputs(trbuf, fp);
+	    g_free(trbuf);
+	} else {
+	    fputs(bufline, fp);
+	}
+# else
+	fputs(bufline, fp);
+# endif
+	fputc('\n', fp);
+	if (strstr(bufline, "pause -1")) {
 	    gotpause = 1;
 	}
     }
 
-    if (!gotpause) {
-	fputs("pause -1\n", fout);
-    }
+    bufgets_finalize(buf);
+    g_free(buf);
 
-    fclose(fin);
-    fclose(fout);
+#ifdef G_OS_WIN32
+    if (!gotpause) {
+	fputs("pause -1\n", fp);
+    }
+#endif
+
+    fclose(fp);
 
     return tmpfile;
 }
 
+#ifdef G_OS_WIN32
+
 void gp_to_gnuplot (gpointer data, guint i, GtkWidget *w)
 {
     windata_t *vwin = (windata_t *) data;
     gchar *tmpfile;
     int err = 0;
 
-    auto_save_gp(vwin);
-
-    tmpfile = add_pause_to_plotfile(vwin->fname);
+    tmpfile = dump_current_plot(vwin);
 
     if (tmpfile != NULL) {
 	gchar *buf = g_strdup_printf("\"%s\" \"%s\"", paths.gnuplot, tmpfile);
@@ -3112,44 +3172,84 @@ void gp_to_gnuplot (gpointer data, guint i, GtkWidget *w)
     }
 
     if (err) {
-	errbox(_("gnuplot command failed"));
+	gui_errmsg(err);
     }
 }
 
 #else /* !G_OS_WIN32 */
 
+#include <sys/types.h>
+#include <sys/wait.h>
+
 void gp_to_gnuplot (gpointer data, guint i, GtkWidget *w)
 {
     windata_t *vwin = (windata_t *) data;
+    gchar *tmpfile;
+    GError *error = NULL;
     gchar *argv[4];
+    GPid pid = 0;
+    gint fd = -1;
     gboolean run;
 
-    auto_save_gp(vwin);
+    tmpfile = dump_current_plot(vwin);
+    if (tmpfile == NULL) {
+	return;
+    }
 
     argv[0] = g_strdup(paths.gnuplot);
     argv[1] = g_strdup("-persist");
-    argv[2] = g_strdup(vwin->fname);
+    argv[2] = g_strdup(tmpfile);
     argv[3] = NULL;
-    
-    run = g_spawn_async(NULL, argv, NULL, G_SPAWN_SEARCH_PATH, 
-			NULL, NULL, NULL, NULL);
 
-    if (!run) {
+    run = g_spawn_async_with_pipes(NULL, argv, NULL, 
+				   G_SPAWN_SEARCH_PATH | 
+				   G_SPAWN_DO_NOT_REAP_CHILD,
+				   NULL, NULL, &pid, NULL, NULL,
+				   &fd, &error);
+
+    if (error != NULL) {
+	errbox(error->message);
+	g_error_free(error);
+    } else if (!run) {
 	errbox(_("gnuplot command failed"));
+    } else if (pid > 0) {
+	int status = 0, err = 0;
+
+	/* bodge below: try to give gnuplot time to bomb
+	   out, if it's going to -- but we don't want to
+	   hold things up if we're doing OK */
+	
+	sleep(1);
+	waitpid(pid, &status, WNOHANG);
+	if (WIFEXITED(status)) {
+	    err = WEXITSTATUS(status);
+	} 
+
+	if (err && fd > 0) {
+	    char buf[128] = {0};
+
+	    if (read(fd, buf, 127) > 0) {
+		errbox(buf);
+	    }
+	}
     }
 
+    if (fd > 0) {
+	close(fd);
+    }
+
+    g_spawn_close_pid(pid);
+
     g_free(argv[0]);
     g_free(argv[1]);
     g_free(argv[2]);
+
+    remove(tmpfile);
+    g_free(tmpfile);
 }
 
 #endif /* ? G_OS_WIN32 */
 
-void save_plot_commands_callback (GtkWidget *w, gpointer p)
-{
-    auto_save_gp((windata_t *) p);
-}
-
 static void open_gui_graph (gui_obj *obj)
 {
     SESSION_GRAPH *graph = (SESSION_GRAPH *) obj->data;
