@@ -1,20 +1,20 @@
-/*
- *  Copyright (c) by Allin Cottrell
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
- *
- *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- *
+/* 
+ *  gretl -- Gnu Regression, Econometrics and Time-series Library
+ *  Copyright (C) 2001 Allin Cottrell and Riccardo "Jack" Lucchetti
+ * 
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ * 
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ * 
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 /* session.h for gretl */
@@ -51,6 +51,8 @@ const char *get_session_dirname (void);
 
 int real_add_text_to_session (PRN *prn, const char *tname);
 
+void save_output_as_text_icon (windata_t *vwin);
+
 int add_graph_to_session (char *fname, char *fullname);
 
 void add_boxplot_to_session (const char *boxtmp);
@@ -104,8 +106,6 @@ int clear_or_save_model (MODEL **ppmod, DATAINFO *pdinfo, int rebuild);
 
 void gp_to_gnuplot (gpointer data, guint i, GtkWidget *w);
 
-void save_plot_commands_callback (GtkWidget *w, gpointer p);
-
 void disable_graph_page (void);
 
 void display_session_graph_by_data (void *p);
