@@ -14,7 +14,7 @@ static int check_graph_file (const char *fname)
     session_file_make_path(fullname, fname);
     fp = gretl_fopen(fullname, "r");
     if (fp == NULL) {
-	errbox(_("Warning: couldn't open graph file %s"), fname);
+	file_read_errbox(fname);
 	err = 1;
     } else {
 	fclose(fp);
@@ -149,6 +149,7 @@ static gpointer rebuild_session_model (const char *fname,
 				   (type == GRETL_OBJ_VAR)? "gretl-VAR" :
 				   "gretl-equation-system", &doc, &node);
     if (*err) {
+	fprintf(stderr, "Failed on gretl_xml_open_doc_root\n");
 	return NULL;
     }
 
@@ -161,7 +162,7 @@ static gpointer rebuild_session_model (const char *fname,
     } else if (type == GRETL_OBJ_VAR) {
 	ptr = gretl_VAR_from_XML(node, doc, err);
     } else {
-	ptr = gretl_system_from_XML(node, doc, err);
+	ptr = equation_system_from_XML(node, doc, err);
     }
 
     xmlFreeDoc(doc);
@@ -248,7 +249,13 @@ static int restore_session_models (xmlNodePtr node, xmlDocPtr doc)
 	    err = reattach_model(ptr, type, (const char *) name, flags);
 	    if (!err) {
 		model_count_plus();
+	    } else {
+		fprintf(stderr, "reattach_model: failed on %s (err = %d)\n",
+			name, err);
 	    }
+	} else {
+	    fprintf(stderr, "rebuild_session_model: failed on %s (err = %d)\n",
+		    fullname, err);
 	}
 
 	free(fname);
@@ -283,6 +290,14 @@ read_session_xml (const char *fname, struct sample_info *sinfo)
 	return 1;
     }
 
+    /* read datafile attribute, if present */
+    tmp = xmlGetProp(cur, (XUC) "datafile");
+    if (tmp != NULL) {
+	strcpy(sinfo->datafile, (char *) tmp);
+	my_filename_from_utf8(sinfo->datafile);
+	free(tmp);
+    }
+
     /* Now walk the tree */
     cur = cur->xmlChildrenNode;
     while (cur != NULL && !err) {
@@ -469,7 +484,7 @@ static int maybe_write_lists_file (void)
     return gretl_serialize_lists(fullname);
 }
 
-static int write_session_xml (void)
+static int write_session_xml (const char *datname)
 {
     MODEL *pmod;
     char fname[MAXLEN];
@@ -485,12 +500,22 @@ static int write_session_xml (void)
     fp = gretl_fopen(fname, "w");
 
     if (fp == NULL) {
-	errbox("Couldn't write session file");
+	file_write_errbox(fname);
 	return E_FOPEN;
     }
 
     gretl_xml_header(fp);
-    fputs("<gretl-session>\n", fp);
+
+    if (*datname != '\0') {
+	/* ensure UTF-8 inside XML file */
+	gchar *trname = my_filename_to_utf8(datname);
+
+	fprintf(fp, "<gretl-session datafile=\"%s\">\n", trname);
+	g_free(trname);
+    } else {
+	fputs("<gretl-session>\n", fp);
+    }
+
     fprintf(fp, " <sample t1=\"%d\" t2=\"%d\"/>\n", datainfo->t1, datainfo->t2);
     write_datainfo_submask(datainfo, fp);
 
@@ -515,7 +540,7 @@ static int write_session_xml (void)
 	fq = fopen(tmpname, "w");
 
 	if (fq == NULL) {
-	    errbox("Couldn't write session model file");
+	    file_write_errbox(tmpname);
 	    err = E_FOPEN;
 	} else {
 	    sprintf(tmpname, "model.%d", i + 1);
@@ -528,7 +553,7 @@ static int write_session_xml (void)
 	    } else if (type == GRETL_OBJ_VAR) {
 		gretl_VAR_serialize(ptr, sflags, fq);
 	    } else if (type == GRETL_OBJ_SYS) {
-		gretl_system_serialize(ptr, sflags, fq);
+		equation_system_serialize(ptr, sflags, fq);
 	    }
 	    fclose(fq);
 	}
@@ -540,7 +565,7 @@ static int write_session_xml (void)
 	    sprintf(tmpname, "%s%cmodel.%d", session.dirname, SLASH, i+1);
 	    fq = fopen(tmpname, "w");
 	    if (fq == NULL) {
-		errbox("Couldn't write session model file");
+		file_write_errbox(tmpname);
 		err = E_FOPEN;
 	    } else {
 		sprintf(tmpname, "model.%d", i+1);
