@@ -1,20 +1,20 @@
-/*
- *  Copyright (c) by Allin Cottrell
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
- *
- *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
- *
+/* 
+ *  gretl -- Gnu Regression, Econometrics and Time-series Library
+ *  Copyright (C) 2001 Allin Cottrell and Riccardo "Jack" Lucchetti
+ * 
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ * 
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ * 
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 /* settings.c for gretl */
@@ -104,14 +104,14 @@ static char hc_tseri[5] = "HAC";
 static char hc_panel[9] = "Arellano";
 static char hc_garch[5] = "QML";
 
-#ifdef G_OS_WIN32
-int wimp;
-#endif
-
 #ifdef ENABLE_NLS
 static int lcnumeric = 1;
 #endif
 
+#ifdef G_OS_WIN32
+extern int use_wimp;
+#endif
+
 #if defined(HAVE_AUDIO) && !defined(G_OS_WIN32)
 char midiplayer[MAXSTR];
 #endif
@@ -184,9 +184,9 @@ RCVAR rc_vars[] = {
     { "lcnumeric", N_("Use locale setting for decimal point"), NULL, &lcnumeric, 
       BOOLSET, 0, TAB_MAIN, NULL },
 #endif
-#ifdef G_OS_WIN32
-    { "wimp", N_("Emulate Windows look"), NULL, &wimp, 
-      BOOLSET, 0, TAB_MAIN, NULL },
+#ifdef G_OS_WIN32 	 
+    { "wimp", N_("Emulate Windows look"), NULL, &use_wimp, 	 
+      BOOLSET, 0, TAB_MAIN, NULL }, 	 
 #endif
 #if !defined(G_OS_WIN32) && !defined(OSX_BUILD)
     { "browser", N_("Web browser"), NULL, Browser, 
@@ -1058,7 +1058,7 @@ static void make_prefs_tab (GtkWidget *notebook, int tab)
 	    }
 
 	    /* special case: warning */
-	    if (!strcmp(rc->key, "wimp") || !strcmp(rc->key, "lcnumeric")) {
+	    if (!strcmp(rc->key, "lcnumeric") || !strcmp(rc->key, "wimp")) {
 		g_signal_connect(G_OBJECT(rc->widget), "toggled",
 				 G_CALLBACK(takes_effect_on_restart), 
 				 NULL);
@@ -1270,6 +1270,7 @@ static void set_lcnumeric (void)
     if (lcnumeric) {
 	struct langname names[] = {
 	    { "es", "Spanish" },
+	    { "eu", "Basque" },
 	    { "fr", "French" },
 	    { "it", "Italian" },
 	    { "pl", "Polish" },
@@ -1349,7 +1350,7 @@ static void set_gp_colors (void)
 		cstr[0], cstr[1], cstr[2], cstr[3]);
 
     for (i=0; i<nc; i++) {
-	set_graph_palette(i, cstr[i]);
+	set_graph_palette_from_string(i, cstr[i]);
     }
 }
 
@@ -1371,8 +1372,6 @@ static void maybe_revise_tramo_x12a_status (void)
     }
 # endif
 
-    fprintf(stderr, "maybe_revise_tramo_x12a_status: doit = %d\n", doit);
-
     if (doit) {
 	set_tramo_x12a_status();
     }
@@ -1427,9 +1426,9 @@ static void apply_changes (GtkWidget *widget, gpointer data)
     show_or_hide_toolbar(want_toolbar);
 
     /* register these for session using libset apparatus */
-    set_use_qr(useqr);
-    set_use_cwd(usecwd);
-    set_shell_ok(shellok);
+    libset_set_bool(USE_QR, useqr);
+    libset_set_bool(USE_CWD, usecwd);
+    libset_set_bool(SHELL_OK, shellok);
     set_xsect_hccme(hc_xsect);
     set_tseries_hccme(hc_tseri);
     set_panel_hccme(hc_panel);
@@ -1503,9 +1502,9 @@ static int common_read_rc_setup (void)
 {
     int err = 0;
 
-    set_use_qr(useqr);
-    set_use_cwd(usecwd);
-    set_shell_ok(shellok);
+    libset_set_bool(USE_QR, useqr);
+    libset_set_bool(USE_CWD, usecwd);
+    libset_set_bool(SHELL_OK, shellok);
     set_gp_colors();
     
     set_xsect_hccme(hc_xsect);
@@ -1802,11 +1801,9 @@ void write_rc (void)
     char *strvar;
     int i;
 
-    fprintf(stderr, "old write_rc\n");
-
     rc = fopen(rcfile, "w");
     if (rc == NULL) {
-	errbox(_("Couldn't open config file for writing"));
+	file_write_errbox(rcfile);
 	return;
     }
 
@@ -1838,8 +1835,6 @@ static void read_rc (void)
     char *strvar;
     int i, j;
 
-    fprintf(stderr, "old read_rc\n");
-
     fp = fopen(rcfile, "r");
     if (fp == NULL) {
 	fprintf(stderr, "Couldn't read %s\n", rcfile);
@@ -2102,13 +2097,18 @@ void font_selector (gpointer data, guint which, GtkWidget *widget)
 
 #define scale_round(v) ((v) * 255.0 / 65535.0)
 
-static GtkWidget *get_image_for_color (const char *colstr)
+static GtkWidget *get_image_for_color (const gretlRGB *color)
 {
+    static char **xpm = NULL;
     GdkPixbuf *icon;
     GtkWidget *image;
-    static char **xpm = NULL;
+    char colstr[8] = {0};
     int i;
 
+    if (color == NULL) {
+	return NULL;
+    }
+
     if (xpm == NULL) {
 	xpm = strings_array_new_with_length(XPMROWS, XPMCOLS);
 	if (xpm != NULL) {
@@ -2132,8 +2132,10 @@ static GtkWidget *get_image_for_color (const char *colstr)
 	return NULL;
     }
 
+    print_rgb_hash(colstr, color);
+
     for (i=0; i<6; i++) {
-	xpm[1][10+i] = colstr[1+i];
+	xpm[1][10+i] = colstr[i+1];
     }    
 
     icon = gdk_pixbuf_new_from_xpm_data((const char **) xpm);
@@ -2146,28 +2148,28 @@ static void color_select_callback (GtkWidget *button, GtkWidget *w)
 {
     GtkWidget *csel;
     GtkWidget *color_button, *image;
-    GdkColor color;
-    char color_string[12];
+    GdkColor gcolor;
+    gretlRGB rgb;
     gint i;
 
     color_button = g_object_get_data(G_OBJECT(w), "color_button");
     csel = GTK_COLOR_SELECTION_DIALOG(w)->colorsel;
 
-    gtk_color_selection_get_current_color(GTK_COLOR_SELECTION(csel), &color);
-    sprintf(color_string, "x%02x%02x%02x",
-	    (guint) (scale_round(color.red)),
-	    (guint) (scale_round(color.green)),
-	    (guint) (scale_round(color.blue)));
+    gtk_color_selection_get_current_color(GTK_COLOR_SELECTION(csel), &gcolor);
+
+    rgb.r = (unsigned char) (scale_round(gcolor.red));
+    rgb.g = (unsigned char) (scale_round(gcolor.red));
+    rgb.b = (unsigned char) (scale_round(gcolor.red));
 
     i = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(w), "colnum"));
 
-    set_graph_palette(i, color_string);
+    set_graph_palette(i, rgb);
     print_palette_string(gpcolors);
 
     /* update the "image" widget */
     image = g_object_get_data(G_OBJECT(color_button), "image");
     gtk_widget_destroy(image);
-    image = get_image_for_color(color_string);
+    image = get_image_for_color(&rgb);
     gtk_widget_show(image);
     gtk_container_add(GTK_CONTAINER(color_button), image);
     g_object_set_data(G_OBJECT(color_button), "image", image);
@@ -2184,7 +2186,7 @@ GtkWidget *color_patch_button (int cnum)
 {
     GtkWidget *image, *button;
 
-    image = get_image_for_color(graph_color_string(cnum));
+    image = get_image_for_color(get_graph_color(cnum));
 
     if (image == NULL) {
 	button = gtk_button_new_with_label(_("Select color"));
@@ -2206,7 +2208,7 @@ void color_patch_button_reset (GtkWidget *button, int cnum)
 
     image = g_object_get_data(G_OBJECT(button), "image");
     gtk_widget_destroy(image);
-    image = get_image_for_color(graph_color_string(cnum));
+    image = get_image_for_color(get_graph_color(cnum));
     gtk_widget_show(image);
     gtk_container_add(GTK_CONTAINER(button), image);
     g_object_set_data(G_OBJECT(button), "image", image);
@@ -2222,11 +2224,17 @@ void graph_color_selector (GtkWidget *w, gpointer p)
     GtkWidget *button;
     gint i = GPOINTER_TO_INT(p);
     char colstr[8];
-    GdkColor color;
+    const gretlRGB *rgb;
+    GdkColor gcolor;
 
-    strcpy(colstr, graph_color_string(i));
-    *colstr = '#';
-    gdk_color_parse(colstr, &color);
+    rgb = get_graph_color(i);
+    if (rgb == NULL) {
+	fprintf(stderr, "graph_get_color(%d) gave NULL\n", i);
+	return;
+    }
+
+    print_rgb_hash(colstr, rgb);
+    gdk_color_parse(colstr, &gcolor);
 
     cdlg = gtk_color_selection_dialog_new(_("gretl: graph color selection"));
 
@@ -2235,7 +2243,7 @@ void graph_color_selector (GtkWidget *w, gpointer p)
 
     gtk_color_selection_set_current_color(GTK_COLOR_SELECTION
 					  (GTK_COLOR_SELECTION_DIALOG(cdlg)->colorsel),
-					  &color);					  
+					  &gcolor);					  
 
     button = GTK_COLOR_SELECTION_DIALOG(cdlg)->ok_button;
     g_signal_connect(G_OBJECT(button), "clicked", 
@@ -2309,7 +2317,7 @@ void dump_rc (void)
 
     fp = gretl_fopen(dumper, "w");
     if (fp == NULL) {
-	errbox(_("Couldn't open config file for writing"));
+	file_write_errbox(dumper);
 	return;
     }
 
