@@ -1,18 +1,18 @@
 #ifndef SETTINGS_H
 #define SETTINGS_H
 
-#define COLOR_MAX 3
-
 #ifdef G_OS_WIN32
 void read_rc (void);
+#else
+void gretl_config_init (void);
 #endif
 
+void set_program_startdir (const char *callname);
+
 int using_hc_by_default (void);
 
 int get_manpref (void);
 
-void set_rcfile (void);
-
 void write_rc (void);
 
 void dump_rc (void);
@@ -35,9 +35,11 @@ const char *get_app_fontname (void);
 
 #endif
 
-void gnuplot_color_selector (GtkWidget *w, gpointer p);
+void graph_color_selector (GtkWidget *w, gpointer p);
 
-GtkWidget *color_patch_button (int colnum);
+GtkWidget *color_patch_button (int cnum);
+
+void color_patch_button_reset (GtkWidget *button, int cnum);
 
 void get_default_dir (char *s, int action);
 
@@ -51,14 +53,6 @@ const char *get_datapage (void);
 
 const char *get_scriptpage (void);
 
-#ifdef HAVE_TRAMO
-void set_tramo_ok (int set);
-#endif
-
-#ifdef HAVE_X12A
-void set_x12a_ok (int set);
-#endif
-
 #ifndef G_OS_WIN32
 void first_time_set_user_dir (void);
 #endif
