@@ -1,20 +1,20 @@
-/*
- *  Copyright (c) by Allin Cottrell
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
- *
- *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
- *
+/* 
+ *  gretl -- Gnu Regression, Econometrics and Time-series Library
+ *  Copyright (C) 2001 Allin Cottrell and Riccardo "Jack" Lucchetti
+ * 
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ * 
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ * 
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 #include "gretl.h"
@@ -217,7 +217,9 @@ static int source_buffer_load_file (GtkSourceBuffer *sbuf,
 {
     char readbuf[MAXSTR], *chunk = NULL;
     GtkTextIter iter;
+#ifdef ENABLE_NLS
     int i = 0;
+#endif
 
     gtk_source_buffer_begin_not_undoable_action(sbuf);
 
@@ -317,7 +319,7 @@ real_sourceview_insert (windata_t *vwin, const char *fname, const char *buf)
     if (fname != NULL) {
 	fp = gretl_fopen(fname, "rb");
 	if (fp == NULL) {
-	    errbox(_("Couldn't open %s"), fname);
+	    file_read_errbox(fname);
 	    return;
 	}
     }
@@ -410,6 +412,8 @@ void create_source (windata_t *vwin, int hsize, int vsize,
     g_object_unref(cmap);
 }
 
+#ifdef ENABLE_NLS
+
 static gchar *my_utf_string (char *t)
 {
     static gchar *s = NULL;
@@ -476,6 +480,8 @@ static gchar *my_utf_string (char *t)
     return s;
 }
 
+#endif /* ENABLE_NLS */
+
 static GtkTextTagTable *gretl_tags_new (void)
 {
     GtkTextTagTable *table;
@@ -679,6 +685,33 @@ void textview_insert_file (windata_t *vwin, const char *fname)
     fclose(fp);
 }
 
+void textview_insert_from_tempfile (windata_t *vwin, PRN *prn)
+{
+    GtkTextBuffer *tbuf;
+    GtkTextIter iter;    
+    char readbuf[MAXSTR];
+    FILE *fp;
+
+    fp = gretl_print_read_tempfile(prn);
+    if (fp == NULL) return;
+
+    tbuf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(vwin->w));
+
+    gtk_text_buffer_get_iter_at_offset(tbuf, &iter, -1);
+    memset(readbuf, 0, sizeof readbuf);
+
+    while (fgets(readbuf, sizeof readbuf, fp)) {
+	gtk_text_buffer_insert(tbuf, &iter, readbuf, -1);
+	memset(readbuf, 0, sizeof readbuf);
+    }
+
+    gretl_print_stop_tempfile_read(prn, fp);
+
+    while (gtk_events_pending()) {
+        gtk_main_iteration();
+    }
+}
+
 static void insert_link (GtkTextBuffer *tbuf, GtkTextIter *iter, 
 			 const char *text, gint page, 
 			 const char *indent)
@@ -1037,6 +1070,251 @@ help_popup_handler (GtkWidget *w, GdkEventButton *event, gpointer p)
     return FALSE;
 }
 
+gchar *textview_get_current_line (GtkWidget *view)
+{
+    GtkTextBuffer *buf;
+    GtkTextIter start, end;
+
+    buf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(view));
+    gtk_text_buffer_get_iter_at_mark(buf, &start, 
+				     gtk_text_buffer_get_insert(buf));
+    gtk_text_iter_set_line_offset(&start, 0);
+    gtk_text_buffer_get_iter_at_mark(buf, &end, 
+				     gtk_text_buffer_get_insert(buf));
+    gtk_text_iter_forward_to_line_end(&end);
+
+    return gtk_text_buffer_get_text(buf, &start, &end, FALSE);
+}
+
+/* Determine whether or not a chunk of text is commented,
+   in the form of each line beginning with '#' (with possible
+   leading white space).  If some lines are commented and
+   others are not, return -1.
+*/
+
+static int text_is_commented (const gchar *s)
+{
+    int gotc = 0, comm = 0;
+    int lines = 1;
+
+    if (s == NULL) {
+	return -1;
+    }
+
+    while (*s) {
+	if (!gotc) {
+	    if (*s == '#') {
+		comm++;
+		gotc = 1;
+	    } else if (!isspace(*s)) {
+		gotc = 1;
+	    }
+	} else if (*s == '\n') {
+	    gotc = 0;
+	    if (*(s+1)) {
+		lines++;
+	    }
+	}
+	s++;
+    }
+
+    if (comm > 0 && comm < lines) {
+	/* mixed */
+	comm = -1;
+    }
+
+    return comm;
+}
+
+struct textcomm {
+    windata_t *vwin;
+    GtkTextBuffer *buf;
+    GtkTextIter start;
+    GtkTextIter end;
+    gchar *chunk;
+    int commented;
+    int selected;
+};
+
+/* either insert or remove '#' comment markers at the
+   start of the line(s) of a chunk of text 
+*/
+
+static void comment_or_uncomment_text (GtkWidget *w, gpointer p)
+{
+    struct textcomm *tc = (struct textcomm *) p;
+    gchar *s;
+
+    gtk_text_buffer_delete(tc->buf, &tc->start, &tc->end);
+
+    if (tc->selected) {
+	char line[1024];
+
+	bufgets_init(tc->chunk);
+	while (bufgets(line, sizeof line, tc->chunk)) {
+	    if (tc->commented) {
+		s = strchr(line, '#');
+		if (s != NULL) {
+		    s++;
+		    if (*s == ' ') s++;
+		    gtk_text_buffer_insert(tc->buf, &tc->start, s, -1);
+		    gtk_text_buffer_insert(tc->buf, &tc->start, "\n", -1);
+		}
+	    } else {
+		gtk_text_buffer_insert(tc->buf, &tc->start, "# ", -1);
+		gtk_text_buffer_insert(tc->buf, &tc->start, line, -1);
+		gtk_text_buffer_insert(tc->buf, &tc->start, "\n", -1);
+	    }
+	}
+	bufgets_finalize(tc->chunk);
+    } else {
+	if (tc->commented) {
+	    s = strchr(tc->chunk, '#');
+	    if (s != NULL) {
+		s++;
+		if (*s == ' ') s++;
+		gtk_text_buffer_insert(tc->buf, &tc->start, s, -1);
+	    }
+	} else {
+	    gtk_text_buffer_insert(tc->buf, &tc->start, "# ", -1);
+	    gtk_text_buffer_insert(tc->buf, &tc->start, tc->chunk, -1);
+	}
+    }
+}
+
+static void exec_script_text (GtkWidget *w, gpointer p)
+{
+    struct textcomm *tc = (struct textcomm *) p;
+
+    run_script_fragment(tc->vwin, tc->chunk);
+    tc->chunk = NULL; /* will be freed already */
+}
+
+static GtkWidget *build_script_popup (windata_t *vwin, struct textcomm **ptc)
+{
+    const char *items[] = {
+	N_("Comment line"),
+	N_("Uncomment line"),
+	N_("Comment region"),
+	N_("Uncomment region"),
+    };
+    GtkWidget *pmenu = NULL;
+    GtkWidget *item;
+    struct textcomm *tc;
+
+    g_return_val_if_fail(GTK_IS_TEXT_VIEW(vwin->w), NULL);
+
+    tc = malloc(sizeof *tc);
+    if (tc == NULL) {
+	return NULL;
+    }
+
+    *ptc = tc;
+    tc->vwin = vwin;
+    tc->buf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(vwin->w));
+
+    if (gtk_text_buffer_get_selection_bounds(tc->buf, &tc->start, &tc->end)) {
+	tc->selected = 1;
+	gtk_text_iter_set_line_offset(&tc->start, 0);
+	if (!gtk_text_iter_ends_line(&tc->end)) {
+	    gtk_text_iter_forward_to_line_end(&tc->end);
+	}
+	tc->chunk = gtk_text_buffer_get_text(tc->buf, &tc->start, &tc->end, FALSE);
+    } else {
+	tc->selected = 0;
+	gtk_text_buffer_get_iter_at_mark(tc->buf, &tc->start, 
+					 gtk_text_buffer_get_insert(tc->buf));
+	gtk_text_iter_set_line_offset(&tc->start, 0);
+	gtk_text_buffer_get_iter_at_mark(tc->buf, &tc->end, 
+					 gtk_text_buffer_get_insert(tc->buf));
+	gtk_text_iter_forward_to_line_end(&tc->end);
+	tc->chunk = gtk_text_buffer_get_text(tc->buf, &tc->start, &tc->end, FALSE);
+    }
+
+    tc->commented = text_is_commented(tc->chunk);
+
+    if (tc->commented <= 0 || (vwin->role == EDIT_SCRIPT && tc->commented >= 0)) {
+	pmenu = gtk_menu_new();
+    }
+
+    if (tc->commented <= 0) {
+	/* we have some uncommented material: allow exec option */
+	if (tc->selected) {
+	    item = gtk_menu_item_new_with_label(_("Execute region"));
+	} else {
+	    item = gtk_menu_item_new_with_label(_("Execute line"));
+	}
+	g_signal_connect(G_OBJECT(item), "activate",
+			 G_CALLBACK(exec_script_text),
+			 *ptc);
+	gtk_widget_show(item);
+	gtk_menu_shell_append(GTK_MENU_SHELL(pmenu), item);
+    }
+
+    if (vwin->role == EDIT_SCRIPT && tc->commented >= 0) {
+	/* material is either all commented or all uncommented:
+	   allow comment/uncomment option */
+	int i = (tc->selected && !tc->commented)? 2 : 
+	    (tc->selected && tc->commented)? 3 :
+	    (!tc->selected && !tc->commented)? 0 : 1;
+
+	item = gtk_menu_item_new_with_label(_(items[i]));
+	g_signal_connect(G_OBJECT(item), "activate",
+			 G_CALLBACK(comment_or_uncomment_text),
+			 *ptc);
+	gtk_widget_show(item);
+	gtk_menu_shell_append(GTK_MENU_SHELL(pmenu), item);
+    }
+
+    if (pmenu == NULL) {
+	g_free(tc->chunk);
+	free(tc);
+	*ptc = NULL;
+    }
+
+    return pmenu;
+}
+
+static gboolean destroy_textcomm (GtkWidget **pw, struct textcomm *tc)
+{
+    tc->vwin->popup = NULL;
+    g_free(tc->chunk);
+    free(tc);
+    return FALSE;
+}
+
+gboolean 
+script_popup_handler (GtkWidget *w, GdkEventButton *event, gpointer p)
+{
+    GdkModifierType mods;
+
+    gdk_window_get_pointer(w->window, NULL, NULL, &mods);
+
+    if (mods & GDK_BUTTON3_MASK) {
+	windata_t *vwin = (windata_t *) p;
+	struct textcomm *tc = NULL;
+
+	if (vwin->popup) {
+	    gtk_widget_destroy(vwin->popup);
+	    vwin->popup = NULL;
+	}
+
+	vwin->popup = build_script_popup(vwin, &tc);
+
+	if (vwin->popup != NULL) {
+	    gtk_menu_popup(GTK_MENU(vwin->popup), NULL, NULL, NULL, NULL,
+			   event->button, event->time);
+	    g_signal_connect(G_OBJECT(vwin->popup), "destroy",
+			     G_CALLBACK(destroy_textcomm), 
+			     tc);
+	}
+
+	return TRUE;
+    }
+
+    return FALSE;
+}
+
 enum {
     INSERT_NONE,
     INSERT_REF,
@@ -1254,6 +1532,9 @@ void set_help_topic_buffer (windata_t *hwin, int hcode, int pos, int en)
 	hwin->active_var = 0;
 	return;
     }
+
+    maybe_connect_help_signals(hwin, en);
+    maybe_set_help_tabs(hwin);
     
     gtk_text_buffer_get_iter_at_offset(textb, &iter, 0);
 
