@@ -1,20 +1,20 @@
-/*
- *  Copyright (c) by Allin Cottrell
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
- *
- *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
- *
+/* 
+ *  gretl -- Gnu Regression, Econometrics and Time-series Library
+ *  Copyright (C) 2001 Allin Cottrell and Riccardo "Jack" Lucchetti
+ * 
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ * 
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ * 
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 #ifndef TEXTBUF_H
@@ -38,6 +38,8 @@ gchar *textview_get_text (GtkWidget *view);
 gchar *textview_get_selection_or_all (GtkWidget *view,
 				      int *sel);
 
+gchar *textview_get_current_line (GtkWidget *view);
+
 int textview_set_text (GtkWidget *view, const gchar *text);
 
 int textview_set_cursor_at_line (GtkWidget *view, int line);
@@ -54,6 +56,8 @@ void textview_append_text_colorized (GtkWidget *view, const char *buf);
 
 void textview_insert_file (windata_t *vwin, const char *fname);
 
+void textview_insert_from_tempfile (windata_t *vwin, PRN *prn);
+
 void create_text (windata_t *vwin, int hsize, int vsize, 
 		  gboolean editable);
 
@@ -66,6 +70,10 @@ void set_help_topic_buffer (windata_t *hwin, int hcode, int pos, int en);
 gboolean help_popup_handler (GtkWidget *w, GdkEventButton *event, 
 			     gpointer p);
 
+gboolean 
+script_popup_handler (GtkWidget *w, GdkEventButton *event, gpointer p);
+
+
 void create_source (windata_t *vwin, int hsize, int vsize, 
 		    gboolean editable);
 
