@@ -1,20 +1,20 @@
-/*
- *  Copyright (c) by Allin Cottrell
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
- *
- *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- *
+/* 
+ *  gretl -- Gnu Regression, Econometrics and Time-series Library
+ *  Copyright (C) 2001 Allin Cottrell and Riccardo "Jack" Lucchetti
+ * 
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ * 
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ * 
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 #include "gretl.h"
@@ -409,7 +409,11 @@ static gchar *text_window_get_copy_buf (windata_t *vwin, int select)
 
 static gchar *maybe_amend_buffer (gchar *inbuf, int fmt)
 {
+#ifdef ENABLE_NLS
     gchar *outbuf = my_locale_from_utf8(inbuf);
+#else
+    gchar *outbuf = g_strdup(inbuf);
+#endif
 
     free(inbuf);
     inbuf = outbuf;
@@ -483,7 +487,7 @@ void window_save (windata_t *vwin, guint fmt)
 
 /* native printing from gretl windows */
 
-#if defined(G_OS_WIN32) || defined (USE_GNOME)
+#ifdef NATIVE_PRINTING
 
 void window_print (windata_t *vwin, guint u, GtkWidget *w) 
 {
