@@ -1,20 +1,20 @@
-/*
- *  Copyright (c) by Allin Cottrell
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
- *
- *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111, USA.
- *
+/* 
+ *  gretl -- Gnu Regression, Econometrics and Time-series Library
+ *  Copyright (C) 2001 Allin Cottrell and Riccardo "Jack" Lucchetti
+ * 
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ * 
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ * 
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 #define ADF_DEBUG 0
@@ -25,8 +25,7 @@
 
 enum {
     ADF_EG_TEST   = 1 << 0,
-    ADF_PRINT_ACK = 1 << 1,
-    ADF_EG_RESIDS = 1 << 2
+    ADF_EG_RESIDS = 1 << 1
 } adf_flags;
 
 static int *
@@ -516,9 +515,6 @@ static int real_adf_test (int varno, int order, int niv,
 	if (!(flags & ADF_EG_TEST) || (flags & ADF_EG_RESIDS)) {
 	    record_test_result(DFt, pv, "Dickey-Fuller");
 	}
-	if ((flags & ADF_PRINT_ACK) && !(opt & OPT_Q)) {
-	    pputs(prn, _("P-values based on MacKinnon (JAE, 1996)\n"));
-	}	
     }
 
  bailout:
@@ -537,7 +533,7 @@ static int real_adf_test (int varno, int order, int niv,
 /**
  * adf_test:
  * @order: lag order for the test.
- * @varno: ID number of the variable to test.
+ * @list: list of variables to test.
  * @pZ: pointer to data matrix.
  * @pdinfo: data information struct.
  * @opt: option flag.
@@ -549,30 +545,22 @@ static int real_adf_test (int varno, int order, int niv,
  * Returns: 0 on successful completion, non-zero on error.
  */
 
-int adf_test (int order, int varno, double ***pZ,
+int adf_test (int order, const int *list, double ***pZ,
 	      DATAINFO *pdinfo, gretlopt opt, PRN *prn)
 {
-    return real_adf_test(varno, order, 1, pZ, pdinfo, opt, 
-			 ADF_PRINT_ACK, prn);
+    int i, err = 0;
+
+    for (i=1; i<=list[0] && !err; i++) {
+	err = real_adf_test(list[i], order, 1, pZ, pdinfo, opt, 
+			    0, prn);
+    }
+
+    return err;
 }
 
-/**
- * kpss_test:
- * @order: window size for Bartlett smoothing.
- * @varno: ID number of the variable to test.
- * @pZ: pointer to data matrix.
- * @pdinfo: data information struct.
- * @opt: option flag.
- * @prn: gretl printing struct.
- *
- * Carries out and prints the results of the KPSS test for 
- * stationarity.
- *
- * Returns: 0 on successful completion, non-zero on error.
- */
-
-int kpss_test (int order, int varno, double ***pZ,
-	       DATAINFO *pdinfo, gretlopt opt, PRN *prn)
+static int 
+real_kpss_test (int order, int varno, double ***pZ,
+		DATAINFO *pdinfo, gretlopt opt, PRN *prn)
 {
     MODEL KPSSmod;
     int list[4];
@@ -699,6 +687,34 @@ int kpss_test (int order, int varno, double ***pZ,
     return 0;
 }
 
+/**
+ * kpss_test:
+ * @order: window size for Bartlett smoothing.
+ * @list: list of variables to test.
+ * @pZ: pointer to data matrix.
+ * @pdinfo: data information struct.
+ * @opt: option flag.
+ * @prn: gretl printing struct.
+ *
+ * Carries out and prints the results of the KPSS test for 
+ * stationarity.
+ *
+ * Returns: 0 on successful completion, non-zero on error.
+ */
+
+int kpss_test (int order, const int *list, double ***pZ,
+	       DATAINFO *pdinfo, gretlopt opt, PRN *prn)
+{
+    int i, err = 0;
+
+    for (i=1; i<=list[0] && !err; i++) {
+	err = real_kpss_test(order, list[i], pZ, pdinfo,
+			     opt, prn);
+    }
+
+    return err;
+}
+
 static int *make_coint_list (const int *list, int detcode, int *nv, 
 			     double ***pZ, DATAINFO *pdinfo,
 			     int *err)
@@ -859,8 +875,7 @@ int coint (int order, const int *list, double ***pZ,
 
     /* Run (A)DF test on the residuals */
     real_adf_test(k, order, nv, pZ, pdinfo, adf_opt, 
-		  ADF_EG_TEST | ADF_EG_RESIDS | ADF_PRINT_ACK, 
-		  prn);
+		  ADF_EG_TEST | ADF_EG_RESIDS, prn); 
 
     pputs(prn, _("\nThere is evidence for a cointegrating relationship if:\n"
 		 "(a) The unit-root hypothesis is not rejected for the individual"
