@@ -1,20 +1,20 @@
 /* 
- * Copyright (C) 2004 Allin Cottrell
- *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License 
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
- *
- * This software is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License 
- * along with this software; if not, write to the 
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ *  gretl -- Gnu Regression, Econometrics and Time-series Library
+ *  Copyright (C) 2001 Allin Cottrell and Riccardo "Jack" Lucchetti
+ * 
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ * 
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ * 
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 #include "libgretl.h"
@@ -471,6 +471,7 @@ static int expand_model_series (MODEL *pmod, model_info *minfo)
  * on for use by @loglik).
  * @init_coeff: starting values for coefficients.
  * @minfo: model info struct; see below.
+ * @opt: can include %OPT_V for verbose output.
  * @prn: printing struct for iteration info (or %NULL).
  *
  * Maximize likelihood using the BHHH conditional ML method,
@@ -503,6 +504,7 @@ int bhhh_max (LL_FUNC loglik,
 	      const double **X, 
 	      const double *init_coeff,
 	      model_info *minfo, 
+	      gretlopt opt,
 	      PRN *prn)
 {
     /* OPG model */
@@ -569,7 +571,7 @@ int bhhh_max (LL_FUNC loglik,
     bmod = gretl_model_new();
 
     iters = 0;
-    itermax = get_bhhh_maxiter();
+    itermax = libset_get_int(BHHH_MAXITER);
 
     while (crit > minfo->tol && iters++ < itermax) {
 
@@ -643,12 +645,19 @@ int bhhh_max (LL_FUNC loglik,
 	}
 
 	/* print interation info, if wanted */
-	print_iter_info(iters, minfo->ll, C_LOGLIK, k, minfo->theta, delta, 
-			stepsize, prn);
+	if (opt & OPT_V) {
+	    print_iter_info(iters, minfo->ll, C_LOGLIK, k, minfo->theta, delta, 
+			    stepsize, prn);
+	}
 
 	crit = minfo->ll2 - minfo->ll;  
     }
 
+    if (opt & OPT_V) {
+	print_iter_info(-1, minfo->ll, C_LOGLIK, k, minfo->theta, delta, 
+			stepsize, prn);
+    }
+
     if (crit > minfo->tol || err != 0) {
 	fprintf(stderr, "bhhh_max: iters = %d, crit = %g, tol = %g, err = %d\n",
 		iters, crit, minfo->tol, err);
@@ -669,12 +678,12 @@ int bhhh_max (LL_FUNC loglik,
 	    gretl_matrix_free(G);
 	}
 	if (minfo->opts & PRESERVE_OPG_MODEL) {
-	    int qr_bak = get_use_qr();
+	    int qr_bak = libset_get_bool(USE_QR);
 
 	    /* run OPG once more using QR, to get packed VCV */
-	    set_use_qr(1);
+	    libset_set_bool(USE_QR, 1);
 	    *bmod = lsq(blist, &tZ, tinfo, OLS, OPT_A);
-	    set_use_qr(qr_bak);
+	    libset_set_bool(USE_QR, qr_bak);
 	    minfo->pmod = bmod;
 	    gretl_model_set_int(bmod, "iters", iters);
 	    if (minfo->bign > minfo->t2) {
