@@ -4,6 +4,7 @@
 #define CMD_PRIVATE_H
 
 #include "gretl_restrict.h"
+#include "gretl_func.h"
 
 typedef struct Laginfo_ Laginfo;
 
@@ -16,7 +17,7 @@ enum {
 #define cmd_ignore(c) (c->flags & CMD_IGNORE)
 
 struct CMD_ {
-    char word[9];               /* command word */
+    char word[FN_NAMELEN];      /* command word */
     int ci;                     /* command index number */
     int context;                /* context for subsetted commands */
     int order;                  /* lag order, for various commands */
@@ -41,12 +42,13 @@ struct ExecState_ {
     char *line;
     char runfile[MAXLEN];
     MODEL **models;
-    gretl_equation_system *sys;
-    gretl_restriction_set *rset;
+    equation_system *sys;
+    gretl_restriction *rset;
     GRETL_VAR *var;
     DATAINFO *subinfo; /* record of incoming sub-sample for functions */
     int alt_model;
     int in_comment;
+    int funcerr;
     EXEC_CALLBACK callback;
 };
 
@@ -59,7 +61,10 @@ void gretl_exec_state_init (ExecState *s,
 
 void gretl_exec_state_clear (ExecState *s);
 
-int maybe_exec_line (ExecState *s, double ***pZ, DATAINFO **ppdinfo,
-		     int *funcerr);
+int gretl_exec_state_finalize (void);
+
+int maybe_exec_line (ExecState *s, double ***pZ, DATAINFO **ppdinfo);
+
+int plausible_genr_start (const char *s, const DATAINFO *pdinfo);
 
 #endif /* CMD_PRIVATE_H */
