@@ -1,20 +1,20 @@
-/*
- *  Copyright (c) by Ramu Ramanathan and Allin Cottrell
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
- *
- *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- *
+/* 
+ *  gretl -- Gnu Regression, Econometrics and Time-series Library
+ *  Copyright (C) 2001 Allin Cottrell and Riccardo "Jack" Lucchetti
+ * 
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ * 
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ * 
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 /*
@@ -147,63 +147,69 @@ mask_from_test_list (const int *list, const MODEL *pmod, int *err)
     return mask;
 }
 
-/* Wald (chi-square or F) test for a set of zero restrictions on the
-   parameters of a given model, based on the covariance matrix of the
-   unrestricted model. Suitable for use where the original model is
-   estimated by FGLS or IV.  Note that if list is NULL, we do an
+/* Wald (chi-square and/or F) test for a set of zero restrictions on
+   the parameters of a given model, based on the covariance matrix of
+   the unrestricted model. Suitable for use where the original model
+   is estimated by FGLS or IV.  Note that if list is NULL, we do an
    automatic test, for the significance of all vars but the constant.
 */
 
-static double wald_test (const int *list, MODEL *pmod, int form,
-			 int *err)
+static int 
+wald_test (const int *list, MODEL *pmod, double *chisq, double *F)
 {
     char *mask = NULL;
     gretl_matrix *C = NULL;
     gretl_vector *b = NULL;
-    double w = NADBL;
+    double wX = NADBL;
+    double wF = NADBL;
+    int err = 0;
 
-    mask = mask_from_test_list(list, pmod, err);
+    mask = mask_from_test_list(list, pmod, &err);
 
-    if (!*err) {
+    if (!err) {
 	C = gretl_vcv_matrix_from_model(pmod, mask);
 	if (C == NULL) {
-	    *err = E_ALLOC;
+	    err = E_ALLOC;
 	} 
     }
 
-    if (!*err) {
+    if (!err) {
 	b = gretl_coeff_vector_from_model(pmod, mask);
 	if (b == NULL) {
-	    *err = E_ALLOC;
+	    err = E_ALLOC;
 	} 
     }  
 
-    if (!*err) {
+    if (!err) {
 #if WDEBUG
 	gretl_matrix_print(C, "Wald VCV matrix");
 	gretl_matrix_print(b, "Wald coeff vector");
 #endif
-	*err = gretl_invert_symmetric_matrix(C);
+	err = gretl_invert_symmetric_matrix(C);
     }
 
-    if (!*err) {
-	w = gretl_scalar_qform(b, C, err);
+    if (!err) {
+	wX = gretl_scalar_qform(b, C, &err);
     }
 
-    if (!*err && form == F_FORM) {
-	w /= gretl_vector_get_length(b);
+    if (!err) {
+	wF = wX / gretl_vector_get_length(b);
+    }
+
+    if (!err) {
+	if (chisq != NULL) *chisq = wX;
+	if (F != NULL) *F = wF;
     }
 
 #if WDEBUG
-    fprintf(stderr, "Wald test: %s = %g\n", 
-	    (form == F_FORM)? "F" : "Chi^2", w);
+    fprintf(stderr, "Wald test: F = %g, Chi^2 = %g\n", wF, wX);
 #endif
 
     free(mask);
     gretl_matrix_free(C);
     gretl_matrix_free(b);
 
-    return w;
+    return err;
 }
 
 /**
@@ -222,9 +228,10 @@ static double wald_test (const int *list, MODEL *pmod, int form,
 
 double robust_omit_F (const int *list, MODEL *pmod)
 {
-    int err = 0;
+    double F = NADBL;
 
-    return wald_test(list, pmod, F_FORM, &err);
+    wald_test(list, pmod, NULL, &F);
+    return F;
 }
 
 /* ----------------------------------------------------- */
@@ -303,24 +310,7 @@ gretl_make_compare (const struct COMPARE *cmp, const int *diffvars,
 	}
     }
 
-    if (!na(cmp->F)) {
-	pval = f_cdf_comp(cmp->F, cmp->dfn, cmp->dfd);
-	if (verbosity > 0) {
-	    pprintf(prn, "\n  %s: %s(%d, %d) = %g, ", _("Test statistic"), 
-		    (cmp->robust)? _("Robust F") : "F",
-		    cmp->dfn, cmp->dfd, cmp->F);
-	    pprintf(prn, _("with p-value = %g\n"), pval);
-	}
-	record_test_result(cmp->F, pval, (cmp->cmd == OMIT)? _("omit") : _("add"));
-
-	if (test != NULL) {
-	    model_test_set_teststat(test, GRETL_STAT_F);
-	    model_test_set_dfn(test, cmp->dfn);
-	    model_test_set_dfd(test, cmp->dfd);
-	    model_test_set_value(test, cmp->F);
-	    model_test_set_pvalue(test, pval);
-	}
-    } else if (!na(cmp->chisq)) {
+    if (!na(cmp->chisq)) {
 	pval = chisq_cdf_comp(cmp->chisq, cmp->dfn);
 	if (verbosity > 0) {
 	    pprintf(prn, "\n  %s:%s%s(%d) = %g, ",  
@@ -330,6 +320,7 @@ gretl_make_compare (const struct COMPARE *cmp, const int *diffvars,
 		    _("Chi-square"), cmp->dfn, cmp->chisq);
 	    pprintf(prn, _("with p-value = %g\n\n"), pval);
 	}
+
 	record_test_result(cmp->chisq, pval, (cmp->cmd == OMIT)? _("omit") : _("add"));
 
 	if (test != NULL) {
@@ -339,7 +330,33 @@ gretl_make_compare (const struct COMPARE *cmp, const int *diffvars,
 	    model_test_set_value(test, cmp->chisq);
 	    model_test_set_pvalue(test, pval);
 	}
-    } 	
+
+	if (verbosity > 0 && !na(cmp->F)) {
+	    /* alternate form */
+	    pval = snedecor_cdf_comp(cmp->F, cmp->dfn, cmp->dfd);
+	    pprintf(prn, "  %s: F(%d, %d) = %g, ", _("F-form"), 
+		    cmp->dfn, cmp->dfd, cmp->F);
+	    pprintf(prn, _("with p-value = %g\n"), pval);
+	}	    
+    } else if (!na(cmp->F)) {
+	pval = snedecor_cdf_comp(cmp->F, cmp->dfn, cmp->dfd);
+	if (verbosity > 0) {
+	    pprintf(prn, "\n  %s: %s(%d, %d) = %g, ", _("Test statistic"), 
+		    (cmp->robust)? _("Robust F") : "F",
+		    cmp->dfn, cmp->dfd, cmp->F);
+	    pprintf(prn, _("with p-value = %g\n"), pval);
+	}
+
+	record_test_result(cmp->F, pval, (cmp->cmd == OMIT)? _("omit") : _("add"));
+
+	if (test != NULL) {
+	    model_test_set_teststat(test, GRETL_STAT_F);
+	    model_test_set_dfn(test, cmp->dfn);
+	    model_test_set_dfd(test, cmp->dfd);
+	    model_test_set_value(test, cmp->F);
+	    model_test_set_pvalue(test, pval);
+	}
+    }
 
     if (test != NULL) {
 	add_diffvars_to_test(test, diffvars, pdinfo);
@@ -399,7 +416,7 @@ add_or_omit_compare (MODEL *pmodA, MODEL *pmodB, int flag,
     /* FIXME TSLS (use F or chi-square?) */
 
     if (flag == OMIT_WALD || flag == ADD_WALD) {
-	cmp.chisq = wald_test(testvars, umod, CHISQ_FORM, &cmp.err);
+	cmp.err = wald_test(testvars, umod, &cmp.chisq, &cmp.F);
     } else if (gretl_model_get_int(pmodA, "robust") || pmodA->ci == HCCM) {
 	cmp.F = robust_omit_F(testvars, umod);
 	cmp.robust = 1;
@@ -407,8 +424,8 @@ add_or_omit_compare (MODEL *pmodA, MODEL *pmodB, int flag,
 	cmp.chisq = 2.0 * (umod->lnL - rmod->lnL);
     } else if (cmp.ci == OLS) {
 	cmp.F = ((rmod->ess - umod->ess) / umod->ess) * cmp.dfd / cmp.dfn;
-    } else if (cmp.dfn > 1) {
-	cmp.chisq = wald_test(testvars, umod, CHISQ_FORM, &cmp.err);
+    } else if (cmp.dfn >= 1) {
+	cmp.err = wald_test(testvars, umod, &cmp.chisq, NULL);
     }
 
     if (pmodB != NULL && flag != ADD_WALD) {
@@ -553,9 +570,6 @@ static MODEL replicate_estimator (MODEL *orig, int **plist,
 	}
 	rho = estimate_rho(list, pZ, pdinfo, orig->ci, 
 			   &rep.errcode, hlopt, prn);
-    } else if (gretl_model_get_int(orig, "unit_weights")) {
-	/* panel model with per-unit weights */
-	myopt |= OPT_W;
     } else if (orig->ci == WLS || orig->ci == AR || 
 	       (orig->ci == POISSON && gretl_model_get_int(orig, "offset_var"))) {
 	int *full_list = full_model_list(orig, list);
@@ -582,6 +596,11 @@ static MODEL replicate_estimator (MODEL *orig, int **plist,
     } else if (orig->ci == PANEL) {
 	if (gretl_model_get_int(orig, "random-effects")) {
 	    myopt |= OPT_U;
+	} else if (gretl_model_get_int(orig, "unit-weights")) {
+	    myopt |= OPT_W;
+	    if (gretl_model_get_int(orig, "iters")) {
+		myopt |= OPT_T;
+	    }
 	}
     }
 
@@ -616,6 +635,9 @@ static MODEL replicate_estimator (MODEL *orig, int **plist,
     case POISSON:
 	rep = poisson_model(list, pZ, pdinfo, NULL);
 	break;
+    case HECKIT:
+	rep = heckit_model(list, pZ, pdinfo, myopt, NULL);
+	break;
     case TSLS:
 	rep = tsls_func(list, TSLS, pZ, pdinfo, myopt);
 	break;
@@ -630,7 +652,7 @@ static MODEL replicate_estimator (MODEL *orig, int **plist,
 	}
 	break;
     case PANEL:
-	rep = real_panel_model(list, pZ, pdinfo, myopt, prn);
+	rep = panel_model(list, pZ, pdinfo, myopt, prn);
 	break;
     default:
 	/* handles OLS, WLS, HSK, HCCM, etc. */
@@ -643,7 +665,6 @@ static MODEL replicate_estimator (MODEL *orig, int **plist,
 	if (rho != 0.0) {
 	    rep = ar1_lsq(list, pZ, pdinfo, repci, myopt, rho);
 	} else {
-	    /* FIXME panel WLS by unit? */
 	    rep = lsq(list, pZ, pdinfo, repci, myopt);
 	}
 	break;
@@ -847,6 +868,9 @@ static void remove_special_flags (gretlopt *popt)
     if (opt & OPT_P) {
 	opt &= ~OPT_P;
     }
+    if (opt & OPT_A) {
+	opt &= ~OPT_A;
+    }    
 
     *popt = opt;
 }
@@ -995,6 +1019,148 @@ static int wald_omit_test (const int *list, MODEL *pmod,
     return err;
 }
 
+/* determine if a model contains a variable with p-value
+   greater than some cutoff alpha_max; and if so, remove
+   this variable from the regression list 
+*/
+
+static int auto_drop_var (const MODEL *pmod, int *list,
+			  DATAINFO *pdinfo, int d0, 
+			  PRN *prn, int *err)
+{
+    double alpha_max = .10; /* FIXME make this configurable? */
+    double tstat, pv = 0.0, tmin = 4.0;
+    int i, k = -1;
+    int ret = 0;
+
+    if (pmod->ncoeff == 1) {
+	return 0;
+    }
+    
+    for (i=pmod->ifc; i<pmod->ncoeff; i++) {
+	tstat = fabs(pmod->coeff[i] / pmod->sderr[i]);
+	if (tstat < tmin) {
+	    tmin = tstat;
+	    k = i;
+	}
+    }
+
+    if (k >= 0) {
+	pv = coeff_pval(pmod->ci, tmin, pmod->dfd);
+    }
+
+    if (pv > alpha_max) {
+	char pname[VNAMELEN];
+
+	if (d0) {
+	    pputc(prn, '\n');
+	    pprintf(prn, _("Sequential elimination using two-sided alpha = %.2f"),
+		    alpha_max);
+	    pputs(prn, "\n\n");
+	}
+
+	gretl_model_get_param_name(pmod, pdinfo, k, pname);
+	pprintf(prn, _(" Dropping %-16s (p-value %.3f)\n"), pname, pv);
+	*err = gretl_list_delete_at_pos(list, k + 2);
+	ret = 1;
+    }
+
+    return ret;
+}
+
+static void list_copy_values (int *targ, const int *src)
+{
+    int i;
+
+    for (i=0; i<=src[0]; i++) {
+	targ[i] = src[i];
+    }
+}
+
+/* run a loop in which the least significant variable is dropped
+   from the regression list, provided its p-value exceeds some
+   specified cutoff.  FIXME this probably still needs work for 
+   estimators other than OLS.
+*/
+
+static int auto_omit (MODEL *orig, MODEL *new, 
+		      double ***pZ, DATAINFO *pdinfo, 
+		      gretlopt est_opt, gretlopt opt,
+		      PRN *prn)
+{
+    int *tmplist = NULL;
+    int err = 0;
+
+    tmplist = gretl_list_copy(orig->list);
+    if (tmplist == NULL) {
+	return E_ALLOC;
+    }
+
+    if (!auto_drop_var(orig, tmplist, pdinfo, 1, prn, &err)) {
+	free(tmplist);
+	return (err)? err : E_NOOMIT;
+    }    
+
+    while (!err) {
+	*new = replicate_estimator(orig, &tmplist, pZ, pdinfo, 
+				   est_opt, prn);
+	if (new->errcode) {
+	    err = new->errcode;
+	} else {
+	    list_copy_values(tmplist, new->list);
+	    if (auto_drop_var(new, tmplist, pdinfo, 0, prn, &err)) {
+		model_count_minus();
+		clear_model(new);
+	    } else {
+		break;
+	    }
+	}
+    }
+
+    free(tmplist);
+
+    return err;
+}
+
+/* create reduced list for "omit" test on model, based on
+   the list of variables to be dropped, omitvars
+*/
+
+static int make_short_list (MODEL *orig, const int *omitvars,
+			    gretlopt opt, int *omitlast,
+			    int **plist)
+{
+    int *list = NULL;
+    int err = 0;
+
+    if (omitvars == NULL || omitvars[0] == 0) {
+	if (orig->ci == TSLS) {
+	    return E_PARSE;
+	} else {
+	    *omitlast = 1;
+	}
+    }
+
+    if (orig->ci == TSLS) {
+	list = tsls_list_omit(orig->list, omitvars, opt, &err);
+    } else if (orig->ci == PANEL || orig->ci == ARBOND) {
+	list = panel_list_omit(orig, omitvars, &err);
+    } else if (*omitlast) {
+	/* special: just drop the last variable */
+	list = gretl_list_omit_last(orig->list, &err);
+    } else {
+	list = gretl_list_omit(orig->list, omitvars, 2, &err);
+    }
+
+    if (list != NULL && list[0] == 1) {
+	err = E_NOVARS;
+    }
+
+    *plist = list;
+
+    return err;
+}
+
 static int omit_options_inconsistent (gretlopt opt)
 {
     if ((opt & OPT_T) || (opt & OPT_B)) {
@@ -1017,16 +1183,19 @@ static int omit_options_inconsistent (gretlopt opt)
  * @pdinfo: information on the data set.
  * @opt: can contain %OPT_Q (quiet) to suppress printing
  * of the new model, %OPT_O to print covariance matrix,
- * %OPT_I for silent operation.  Additional options that
- * are applicable only for 2SLS models: %OPT_B (omit from
- * both list of regressors and list of instruments), %OPT_T
- * (omit from list of instruments only).
+ * %OPT_I for silent operation; for %OPT_A, see below.
+ * Additional options that are applicable only for 2SLS models: 
+ * %OPT_B (omit from both list of regressors and list of 
+ * instruments), %OPT_T (omit from list of instruments only).
  * @prn: gretl printing struct.
  *
  * Re-estimate a given model after removing the variables
- * specified in @omitvars, or if @omitvars is %NULL and
- * @orig was not estimated using two-stage least squares, after
- * removing the last independent variable in @orig.
+ * specified in @omitvars.  Or if %OPT_A is given, proceed 
+ * sequentially, at each step dropping the least significant 
+ * variable provided its p-value is above a certain threshold 
+ * (currently 0.10, two-sided).  Or if @omitvars is %NULL 
+ * and @orig was not estimated using two-stage least squares,
+ * drop the last independent variable in @orig.
  * 
  * Returns: 0 on successful completion, error code on error.
  */
@@ -1063,53 +1232,34 @@ int omit_test (const int *omitvars, MODEL *orig, MODEL *new,
 	return err;
     }
 
-    if (omitvars == NULL || omitvars[0] == 0) {
-	if (orig->ci == TSLS) {
-	    return E_PARSE;
-	} else {
-	    omitlast = 1;
+    if (!(opt & OPT_A)) {
+	err = make_short_list(orig, omitvars, opt, &omitlast, &tmplist);
+	if (err) {
+	    return err;
 	}
     }
 
-    /* create list for test model */
-    if (orig->ci == TSLS) {
-	tmplist = tsls_list_omit(orig->list, omitvars, opt, &err);
-    } else if (orig->ci == PANEL || orig->ci == ARBOND) {
-	tmplist = panel_list_omit(orig, omitvars, &err);
-    } else if (omitlast) {
-	/* special: just drop the last variable */
-	tmplist = gretl_list_omit_last(orig->list, &err);
-    } else {
-	tmplist = gretl_list_omit(orig->list, omitvars, 2, &err);
-    }
-
-    if (tmplist == NULL) {
-	return err;
-    }
-
-    if (tmplist[0] == 1) {
-	return E_NOVARS;
-    }
-
     /* impose the sample range used for the original model */ 
     impose_model_smpl(orig, pdinfo);
 
     /* set the mask for missing obs within the sample range, based
        on the original model */
-    set_reference_missmask(orig);
+    set_reference_missmask_from_model(orig);
 
     /* extract option flags that should not be passed to estimator
        functions */
     remove_special_flags(&est_opt);
 
-    *new = replicate_estimator(orig, &tmplist, pZ, pdinfo, est_opt, prn);
-
-    if (new->errcode) {
+    if (opt & OPT_A) {
+	err = auto_omit(orig, new, pZ, pdinfo, est_opt, opt, prn);
+    } else {
+	*new = replicate_estimator(orig, &tmplist, pZ, pdinfo, est_opt, prn);
 	err = new->errcode;
-	errmsg(err, prn);
     }
 
-    if (!err) {
+    if (err) {
+	errmsg(err, prn);
+    } else {
 	if (orig->ci == LOGIT || orig->ci == PROBIT) {
 	    new->aux = AUX_OMIT;
 	}
@@ -1237,7 +1387,7 @@ int reset_test (MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
 	aux.aux = AUX_RESET;
 	printmodel(&aux, pdinfo, OPT_NONE, prn);
 	RF = ((pmod->ess - aux.ess) / 2) / (aux.ess / aux.dfd);
-	pval = f_cdf_comp(RF, 2, aux.dfd);
+	pval = snedecor_cdf_comp(RF, 2, aux.dfd);
 
 	pprintf(prn, "\n%s: F = %f,\n", _("Test statistic"), RF);
 	pprintf(prn, "%s = P(F(%d,%d) > %g) = %.3g\n", _("with p-value"), 
@@ -1266,18 +1416,204 @@ int reset_test (MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
     return err;
 }
 
-static void bg_test_header (int order, PRN *prn)
+static void bg_test_header (int order, PRN *prn, int tsls)
 {
-    pprintf(prn, "\n%s ", _("Breusch-Godfrey test for"));
+    if (tsls) {
+	pprintf(prn, "\n%s ", _("Godfrey (1994) test for"));
+    } else {
+	pprintf(prn, "\n%s ", _("Breusch-Godfrey test for")); 
+    }
+
     if (order > 1) {
 	pprintf(prn, "%s %d\n", _("autocorrelation up to order"),
 		order);
     } else {
 	pprintf(prn, "%s\n", _("first-order autocorrelation"));
     }
+
     pputc(prn, '\n');
 }
 
+/**
+ * tsls_autocorr_test:
+ * @pmod: pointer to model to be tested.
+ * @order: lag order for test.
+ * @pZ: pointer to data matrix.
+ * @pdinfo: information on the data set.
+ * @opt: if flags include %OPT_S, save test results to model;
+ * if %OPT_Q, be less verbose.
+ * @prn: gretl printing struct.
+ *
+ * Tests the given two-stage model for autocorrelation of order equal
+ * to the specified value, or equal to the frequency of the data if
+ * the supplied @order is zero, as per Godfrey (1999), "Testing for
+ * Serial Correlation by Variable Addition in Dynamic Models Estimated
+ * by Instrumental Variables", RES. Note that none of the
+ * asymptotically equivalent tests given on page 553 is used
+ * here. Instead, we estimate the model augmented with lags and then
+ * perform a Wald-type test. The resulting chi-square statistic is
+ * divided by its degrees of freedom as a finite-sample adjustment and
+ * compared to an F distribution.
+ * 
+ * Returns: 0 on successful completion, error code on error.
+ */
+
+static int tsls_autocorr_test (MODEL *pmod, int order, 
+			       double ***pZ, DATAINFO *pdinfo, 
+			       gretlopt opt, PRN *prn)
+{
+    int smpl_t1 = pdinfo->t1;
+    int smpl_t2 = pdinfo->t2;
+    int *addlist = NULL;
+    int *testlist = NULL;
+    MODEL aux;
+    int i, j, t, n = pdinfo->n, v = pdinfo->v;
+    double x, pval = 1.0;
+    int err = 0;
+
+    if (dataset_is_panel(pdinfo)) { 
+	return E_NOTIMP;
+    }
+
+    if (pmod->missmask != NULL) {
+	return E_DATA;
+    }
+
+    /* impose original sample range */
+    impose_model_smpl(pmod, pdinfo);
+
+    gretl_model_init(&aux);
+
+    if (order <= 0) {
+	order = pdinfo->pd;
+    }
+
+    if (pmod->ncoeff + order >= pdinfo->t2 - pdinfo->t1) {
+	return E_DF;
+    }
+
+    addlist = gretl_list_new(order);
+
+    if (addlist == NULL) {
+	err = E_ALLOC;
+    } else {
+	err = dataset_add_series(1, pZ, pdinfo);
+    }
+
+    if (!err) {
+	/* add uhat to data set */
+	for (t=0; t<n; t++) {
+	    (*pZ)[v][t] = pmod->uhat[t];
+	}
+	strcpy(pdinfo->varname[v], "uhat");
+	strcpy(VARLABEL(pdinfo, v), _("residual"));
+	/* then lags of same */
+	for (i=1; i<=order; i++) {
+	    int lnum;
+
+	    lnum = laggenr(v, i, pZ, pdinfo);
+	    /* 
+	       set the first entries to 0 for compatibility with Godfrey (1994)
+	       and PcGive (not perfect)
+	    */
+	    for (t=smpl_t1; t<smpl_t1+i; t++) {
+		(*pZ)[lnum][t] = 0;
+	    }
+
+	    if (lnum < 0) {
+		sprintf(gretl_errmsg, _("lagging uhat failed"));
+		err = E_LAGS;
+	    } else {
+		addlist[i] = lnum;
+	    }
+	}
+    }
+
+    if (!err) {
+	testlist = tsls_list_add(pmod->list, addlist, OPT_B, &err);
+	aux = tsls_func(testlist, TSLS, pZ, pdinfo, OPT_NONE);
+	err = aux.errcode;
+	if (err) {
+	    errmsg(aux.errcode, prn);
+	}
+    } 
+
+    if (!err) {
+	gretl_matrix *V0;
+	gretl_vector *b = gretl_vector_alloc(order);
+	gretl_matrix *V1 = gretl_matrix_alloc(order, order);
+	gretl_vector *WT = gretl_matrix_alloc(1, 1);
+	int ki, kj;
+
+	V0 = gretl_model_get_matrix(&aux, M_VCV, &err);
+	ki = aux.ncoeff - order;
+
+	for (i=0; i<order; i++) {
+	    x = aux.coeff[ki];
+	    gretl_vector_set(b, i, x);
+	    kj = ki;
+	    for (j=i; j<order; j++) {
+		x = gretl_matrix_get(V0, ki, kj); 
+		gretl_matrix_set(V1, i, j, x);
+		gretl_matrix_set(V1, j, i, x);
+		kj++;
+	    }
+	    ki++;
+	}
+
+	err = gretl_invert_symmetric_matrix(V1);
+	gretl_matrix_qform(b, GRETL_MOD_NONE, V1, WT, GRETL_MOD_NONE);
+	x = gretl_vector_get(WT,0) / order;
+	
+	gretl_vector_free(WT);
+	gretl_vector_free(b);
+	gretl_matrix_free(V1);
+	gretl_matrix_free(V0);
+
+	aux.aux = AUX_AR;
+	gretl_model_set_int(&aux, "BG_order", order);
+	pval = snedecor_cdf_comp(x, order, aux.nobs - pmod->ncoeff - order);
+
+	if (opt & OPT_Q) {
+	    bg_test_header(order, prn, 1);
+	} else {
+	    printmodel(&aux, pdinfo, OPT_S, prn);
+	} 
+
+	pprintf(prn, "%s: Pseudo-LMF = %f,\n", _("Test statistic"), x);
+	pprintf(prn, "%s = P(F(%d,%d) > %g) = %.3g\n", _("with p-value"), 
+		order, aux.nobs - pmod->ncoeff, x, pval);
+	pputc(prn, '\n');
+	record_test_result(x/order, pval, _("autocorrelation"));
+
+	if (opt & OPT_S) {
+	    ModelTest *test = model_test_new(GRETL_TEST_AUTOCORR);
+	    
+	    if (test != NULL) {
+		model_test_set_teststat(test, GRETL_STAT_LMF);
+		model_test_set_dfn(test, order);
+		model_test_set_dfd(test, aux.nobs - pmod->ncoeff);
+		model_test_set_order(test, order);
+		model_test_set_value(test, x);
+		model_test_set_pvalue(test, pval);
+		maybe_add_test_to_model(pmod, test);
+	    }	    
+	}
+    }
+
+    free(addlist);
+    free(testlist);
+
+    dataset_drop_last_variables(pdinfo->v - v, pZ, pdinfo); 
+    clear_model(&aux); 
+
+    /* reset sample as it was */
+    pdinfo->t1 = smpl_t1;
+    pdinfo->t2 = smpl_t2;
+
+    return err;
+}
+
 /**
  * autocorr_test:
  * @pmod: pointer to model to be tested.
@@ -1307,6 +1643,10 @@ int autocorr_test (MODEL *pmod, int order,
     double trsq, LMF, lb, pval = 1.0;
     int err = 0;
 
+    if (pmod->ci == TSLS) {
+	return tsls_autocorr_test(pmod, order, pZ, pdinfo, opt, prn);
+    }
+
     if (pmod->ci != OLS && pmod->ci != VAR) { 
 	return E_NOTIMP;
     }
@@ -1383,11 +1723,11 @@ int autocorr_test (MODEL *pmod, int order,
 	trsq = aux.rsq * aux.nobs;
 	LMF = (aux.rsq/(1.0 - aux.rsq)) * 
 	    (aux.nobs - pmod->ncoeff - order)/order; 
-	pval = f_cdf_comp(LMF, order, aux.nobs - pmod->ncoeff - order);
+	pval = snedecor_cdf_comp(LMF, order, aux.nobs - pmod->ncoeff - order);
 
 	if (pmod->aux != AUX_VAR) {
 	    if (opt & OPT_Q) {
-		bg_test_header(order, prn);
+		bg_test_header(order, prn, 0);
 	    } else {
 		printmodel(&aux, pdinfo, OPT_NONE, prn);
 	    } 
@@ -1669,7 +2009,8 @@ int chow_test (const char *line, MODEL *pmod, double ***pZ,
     } else {
 	split = dateton(chowdate, pdinfo);
 	if (split <= 0 || split >= pdinfo->n) { 
-	    err = E_SPLIT;
+	    strcpy(gretl_errmsg, _("Invalid sample split for Chow test"));
+	    err = E_DATA;
 	}
 	smax = split;
     }
@@ -1746,7 +2087,7 @@ int chow_test (const char *line, MODEL *pmod, double ***pZ,
 	    printmodel(&chow_mod, pdinfo, OPT_NONE, prn);
 	    F = (pmod->ess - chow_mod.ess) * chow_mod.dfd / 
 		(chow_mod.ess * dfn);
-	    pval = f_cdf_comp(F, dfn, chow_mod.dfd);
+	    pval = snedecor_cdf_comp(F, dfn, chow_mod.dfd);
 	    pprintf(prn, _("\nChow test for structural break at observation %s:\n"
 		    "  F(%d, %d) = %f with p-value %f\n\n"), chowdate,
 		    dfn, chow_mod.dfd, F, pval);
@@ -1855,6 +2196,13 @@ static int cusum_do_graph (double a, double b, const double *W,
     double x0 = 0.0;
     int j, t;
     int err = 0;
+    double frac = 1;
+    double slope = b;
+
+    if( pdinfo->pd==4 || pdinfo->pd==12 ) {
+	slope *= pdinfo->pd;
+	frac = 1.0/pdinfo->pd;
+    }
 
     err = gnuplot_init(PLOT_CUSUM, &fq);
     if (err) {
@@ -1877,16 +2225,16 @@ static int cusum_do_graph (double a, double b, const double *W,
 	fprintf(fq, "set title '%s'\n",
 		/* xgettext:no-c-format */
 		G_("CUSUMSQ plot with 95% confidence band"));
-	fprintf(fq, "plot \\\n%g*(x-%g) title '' w dots lt 2, \\\n", b, x0 - 1);
-	fprintf(fq, "%g+%g*(x-%g) title '' w lines lt 2, \\\n", -a, b, x0 - 1);
-	fprintf(fq, "%g+%g*(x-%g) title '' w lines lt 2, \\\n", a, b, x0 - 1);
+	fprintf(fq, "plot \\\n%g*(x-%g) title '' w dots lt 2, \\\n", slope, x0 - frac);
+	fprintf(fq, "%g+%g*(x-%g) title '' w lines lt 2, \\\n", -a, slope, x0 - frac);
+	fprintf(fq, "%g+%g*(x-%g) title '' w lines lt 2, \\\n", a, slope, x0 - frac);
     } else {
 	fputs("set xzeroaxis\n", fq);
 	fprintf(fq, "set title '%s'\n",
 		/* xgettext:no-c-format */
 		G_("CUSUM plot with 95% confidence band"));
-	fprintf(fq, "plot \\\n%g+%g*(x-%g) title '' w lines lt 2, \\\n", a, b, x0);
-	fprintf(fq, "%g-%g*(x-%g) title '' w lines lt 2, \\\n", -a, b, x0);
+	fprintf(fq, "plot \\\n%g+%g*(x-%g) title '' w lines lt 2, \\\n", a, slope, x0);
+	fprintf(fq, "%g-%g*(x-%g) title '' w lines lt 2, \\\n", -a, slope, x0);
     }	
 
     fputs("'-' using 1:2 w linespoints lt 1\n", fq);
@@ -1916,7 +2264,7 @@ static void cusum_harvey_collier (double wbar, double sigma, int m,
     double hct, pval;
 
     hct = (sqrt((double) m) * wbar) / sigma;
-    pval = t_pvalue_2(hct, m - 1);
+    pval = student_pvalue_2(hct, m - 1);
     pprintf(prn, _("\nHarvey-Collier t(%d) = %g with p-value %.4g\n\n"), 
 	    m - 1, hct, pval);
 
@@ -2239,7 +2587,9 @@ int leverage_test (MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
     gretl_matrix *m;
     int err = 0;
 
-    if (pmod->ci != OLS) return E_OLSONLY;
+    if (pmod->ci != OLS) {
+	return E_OLSONLY;
+    }
 
     model_leverage = get_plugin_function("model_leverage", &handle);
     if (model_leverage == NULL) {
@@ -2326,6 +2676,12 @@ int lmtest_driver (const char *param,
 	return 0;
     }
 
+    err = incompatible_options(opt, OPT_A | OPT_H | OPT_L | OPT_S |
+			       OPT_P | OPT_W);
+    if (err) {
+	return err;
+    }
+
     ptr = get_last_model(&type);  
     if (ptr == NULL) {
 	return E_DATA;
@@ -2357,7 +2713,7 @@ int lmtest_driver (const char *param,
 	}
     }
 
-    /* heteroskedasticity, White */
+    /* heteroskedasticity (White) */
     if (!err && (opt & OPT_W)) {
 	if (type == GRETL_OBJ_EQN) {
 	    err = whites_test(ptr, pZ, pdinfo, testopt, prn);
@@ -2369,8 +2725,7 @@ int lmtest_driver (const char *param,
     /* autocorrelation */
     if (!err && (opt & OPT_A)) {
 	if (type == GRETL_OBJ_EQN) {
-	    err = autocorr_test(ptr, k, pZ, pdinfo, 
-				testopt, prn);
+	    err = autocorr_test(ptr, k, pZ, pdinfo, testopt, prn);
 	} else if (type == GRETL_OBJ_VAR) {
 	    err = gretl_VAR_autocorrelation_test(ptr, k, pZ, pdinfo, prn);
 	} else {
