@@ -1,20 +1,20 @@
-/*
- *  Copyright (c) 2005 by Allin Cottrell
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
- *
- *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
- *
+/* 
+ *  gretl -- Gnu Regression, Econometrics and Time-series Library
+ *  Copyright (C) 2001 Allin Cottrell and Riccardo "Jack" Lucchetti
+ * 
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ * 
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ * 
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 #include "libgretl.h"
@@ -632,6 +632,7 @@ create_new_dataset (double ***pZ, int nvar, int nobs, int markers)
 
     pdinfo->v = nvar;
     pdinfo->n = nobs;
+
     *pZ = NULL;
 
     if (start_new_Z(pZ, pdinfo, 0)) {
@@ -639,8 +640,7 @@ create_new_dataset (double ***pZ, int nvar, int nobs, int markers)
 	return NULL;
     }
 
-    pdinfo->markers = (unsigned char) markers;
-    if (pdinfo->markers) {
+    if (markers) {
 	if (dataset_allocate_obs_markers(pdinfo)) {
 	    free_datainfo(pdinfo);
 	    return NULL;
@@ -1854,6 +1854,135 @@ int dataset_destroy_hidden_variables (double ***pZ, DATAINFO *pdinfo,
     return err;
 }
 
+/* apparatus for sorting entire dataset */
+
+typedef struct spoint_t_ spoint_t;
+
+struct spoint_t_ {
+    int obsnum;
+    double val;
+};
+
+static int compare_vals_up (const void *a, const void *b)
+{
+    const spoint_t *pa = (const spoint_t *) a;
+    const spoint_t *pb = (const spoint_t *) b;
+     
+    return (pa->val > pb->val) - (pa->val < pb->val);
+}
+
+static int compare_vals_down (const void *a, const void *b)
+{
+    const spoint_t *pa = (const spoint_t *) a;
+    const spoint_t *pb = (const spoint_t *) b;
+     
+    return (pa->val < pb->val) - (pa->val > pb->val);
+}
+
+int dataset_sort_by (int v, double **Z, DATAINFO *pdinfo, gretlopt opt)
+{
+    spoint_t *sv = NULL;
+    double *x = NULL;
+    char **S = NULL;
+    int i, t;
+    int err = 0;
+
+    if (v < 1 || v >= pdinfo->v || var_is_scalar(pdinfo, v)) {
+	return E_DATA;
+    }
+
+    sv = malloc(pdinfo->n * sizeof *sv);
+    if (sv == NULL) {
+	return E_ALLOC;
+    }
+
+    x = malloc(pdinfo->n * sizeof *x);
+    if (x == NULL) {
+	free(sv);
+	return E_ALLOC;
+    }    
+
+    if (pdinfo->S != NULL) {
+	S = strings_array_new_with_length(pdinfo->n, OBSLEN);
+	if (S == NULL) {
+	    err = E_ALLOC;
+	    goto bailout;
+	}
+    }
+
+    for (t=0; t<pdinfo->n; t++) {
+	sv[t].obsnum = t;
+	sv[t].val = Z[v][t];
+    }
+
+    if (opt & OPT_D) {
+	/* descending */
+	qsort(sv, pdinfo->n, sizeof *sv, compare_vals_down);
+    } else {
+	qsort(sv, pdinfo->n, sizeof *sv, compare_vals_up);
+    }
+
+    for (i=1; i<pdinfo->v; i++) {
+	if (var_is_scalar(pdinfo, i)) {
+	    continue;
+	}
+	for (t=0; t<pdinfo->n; t++) {
+	    x[t] = Z[i][sv[t].obsnum];
+	}
+	for (t=0; t<pdinfo->n; t++) {
+	    Z[i][t] = x[t];
+	}
+    }
+
+    if (S != NULL) {
+	for (t=0; t<pdinfo->n; t++) {
+	    strcpy(S[t], pdinfo->S[sv[t].obsnum]);
+	}
+	free_strings_array(pdinfo->S, pdinfo->n);
+	pdinfo->S = S;
+    }
+
+ bailout:
+
+    free(sv);
+    free(x);
+
+    return err;
+}
+
+static int dataset_sort (const char *s, double **Z, DATAINFO *pdinfo,
+			 gretlopt opt)
+{
+    char fmt[10];
+    char vname[VNAMELEN];
+    int v;
+
+    if (dataset_is_time_series(pdinfo) ||
+	dataset_is_panel(pdinfo)) {
+	strcpy(gretl_errmsg, "You can only do this with undated data");
+	return E_DATA;
+    }
+
+    s += strspn(s, " \t");
+
+    sprintf(fmt, "%%%ds", VNAMELEN - 1);
+
+    if (sscanf(s, fmt, vname) != 1) {
+	return E_DATA;
+    }
+
+    v = varindex(pdinfo, vname);
+    if (v == pdinfo->v) {
+	return E_UNKVAR;
+    }
+
+    if (v < 1 || var_is_scalar(pdinfo, v)) {
+	return E_DATA;
+    }
+    
+    return dataset_sort_by(v, Z, pdinfo, opt);
+}
+
 /**
  * dataset_drop_last_variables:
  * @delvars: number of variables to be dropped.
@@ -2473,4 +2602,372 @@ const char *var_get_graph_name (const DATAINFO *pdinfo, int i)
     return ret;
 }
 
+static int add_obs (int n, double ***pZ, DATAINFO *pdinfo, PRN *prn)
+{
+    int err = 0;
 
+    if (complex_subsampled()) {
+	pprintf(prn, _("The data set is currently sub-sampled.\n"));
+	err = E_DATA;
+    } else if (n <= 0) {
+	err = E_PARSE;
+    } else {
+	err = dataset_add_observations(n, pZ, pdinfo, OPT_A);
+	if (!err) {
+	    pprintf(prn, _("Dataset extended by %d observations"), n);
+	    pputc(prn, '\n');
+	}
+    }
+
+    return err;
+}
+
+int dataset_op_from_string (const char *s)
+{
+    int op = DS_NONE;
+
+    if (!strcmp(s, "addobs")) {
+	op = DS_ADDOBS;
+    } else if (!strcmp(s, "compact")) {
+	op = DS_COMPACT;
+    } else if (!strcmp(s, "expand")) {
+	op = DS_EXPAND;
+    } else if (!strcmp(s, "transpose")) {
+	op = DS_TRANSPOSE;
+    } else if (!strcmp(s, "delete")) {
+	op = DS_DELETE;
+    } else if (!strcmp(s, "keep")) {
+	op = DS_KEEP;
+    } else if (!strcmp(s, "sortby")) {
+	op = DS_SORTBY;
+    } else if (!strcmp(s, "dsortby")) {
+	op = DS_DSORTBY;
+    } else if (!strcmp(s, "resample")) {
+	op = DS_RESAMPLE;
+    }
+
+    return op;
+}
+
+static int dataset_int_param (const char *s, int op, 
+			      double **Z, DATAINFO *pdinfo, 
+			      int *err)
+{
+    char test[32];
+    int k = 0;
+
+    if ((op == DS_COMPACT || op == DS_EXPAND) &&
+	!dataset_is_time_series(pdinfo)) {
+	*err = E_PDWRONG;
+	return 0;
+    }
+
+    *test = '\0';
+    sscanf(s, "%31s", test);
+
+    k = gretl_int_from_string(test, (const double **) Z, pdinfo, err);
+    if (*err) {
+	return 0;
+    }
+
+    if (k <= 0 || (op == DS_RESAMPLE && k < 1)) {
+	*err = E_DATA;
+    } else if (op == DS_COMPACT) {
+	int ok = 0;
+
+	if (pdinfo->pd == 12 && (k == 4 || k == 1)) {
+	    ok = 1;
+	} else if (pdinfo->pd == 4 && k == 1) {
+	    ok = 1;
+	} else if (pdinfo->pd == 52 && k == 12) {
+	    ok = 1;
+	} else if (dated_daily_data(pdinfo) && (k == 52 || k == 12)) {
+	    ok = 1;
+	}
+
+	if (!ok) {
+	    *err = E_PDWRONG;
+	    strcpy(gretl_errmsg, "This conversion is not supported");
+	}
+    } else if (op == DS_EXPAND) {
+	int ok = 0;
+
+	if (pdinfo->pd == 1 && (k == 4 || k == 12)) {
+	    ok = 1;
+	} else if (pdinfo->pd == 4 && k == 12) {
+	    ok = 1;
+	} 
+
+	if (!ok) {
+	    *err = E_PDWRONG;
+	    strcpy(gretl_errmsg, "This conversion is not supported");
+	}
+    }
+
+    return k;
+}
+
+static int compact_data_set_wrapper (const char *s, double ***pZ, 
+				     DATAINFO *pdinfo, int k)
+{
+    CompactMethod method = COMPACT_AVG;
+
+    if (strstr(s, "sum")) {
+	method = COMPACT_SUM;
+    } else if (strstr(s, "first")) {
+	method = COMPACT_SOP;
+    } else if (strstr(s, "last")) {
+	method = COMPACT_EOP;
+    }
+
+    return compact_data_set(pZ, pdinfo, k, method, 0, 0);
+}
+
+/* resample the dataset by observation, with replacement */
+
+static int dataset_resample (int n, double ***pZ, DATAINFO *pdinfo)
+{
+    double **RZ = NULL;
+    char **S = NULL;
+    int T = pdinfo->t2 - pdinfo->t1 + 1;
+    int v = pdinfo->v;
+    int i, j, s, t;
+    int err = 0;
+
+    if (v < 2) {
+	return E_DATA;
+    }
+
+    RZ = malloc(v * sizeof *RZ);
+    if (RZ == NULL) {
+	return E_ALLOC;
+    }
+
+    for (i=0; i<v; i++) {
+	RZ[i] = NULL;
+    }
+
+    j = 0;
+    for (i=0; i<pdinfo->v && !err; i++) {
+	if (i > 0 && var_is_scalar(pdinfo, i)) {
+	    RZ[j] = malloc(sizeof **RZ);
+	    if (RZ[j] != NULL) {
+		RZ[j][0] = (*pZ)[i][0];
+	    }
+	} else {
+	    RZ[j] = malloc(n * sizeof **RZ);
+	    if (RZ[j] != NULL && i == 0) {
+		for (t=0; t<n; t++) {
+		    RZ[j][t] = 1.0;
+		}
+	    }
+	}
+	if (RZ[j] == NULL) {
+	    err = E_ALLOC;
+	}
+	j++;
+    }
+
+    if (err) {
+	for (i=0; i<v; i++) {
+	    free(RZ[i]);
+	}
+	free(RZ);
+	return E_ALLOC;
+    }
+
+    if (pdinfo->markers == REGULAR_MARKERS) {
+	S = strings_array_new_with_length(n, OBSLEN);
+    }
+
+    for (t=0; t<n; t++) {
+	s = gretl_rand_int_max(T) + pdinfo->t1;
+	j = 1;
+	for (i=1; i<pdinfo->v; i++) {
+	    if (var_is_series(pdinfo, i)) {
+		RZ[j][t] = (*pZ)[i][s];
+	    } 
+	    j++;
+	}
+	if (S != NULL) {
+	    strcpy(S[t], pdinfo->S[s]);
+	}
+    }
+
+    free_Z(*pZ, pdinfo);
+    *pZ = RZ;
+
+    if (S != NULL) {
+	dataset_destroy_obs_markers(pdinfo);
+	pdinfo->S = S;
+	pdinfo->markers = REGULAR_MARKERS;
+    } else if (pdinfo->S != NULL) {
+	dataset_destroy_obs_markers(pdinfo);
+    }
+
+    pdinfo->v = v;
+    pdinfo->n = n;
+    pdinfo->t1 = 0;
+    pdinfo->t2 = n - 1;
+    dataset_obs_info_default(pdinfo);
+
+    return err;
+}
+
+int modify_dataset (int op, const int *list, const char *s, 
+		    double ***pZ, DATAINFO *pdinfo, 
+		    PRN *prn)
+{
+    int k = 0, err = 0;
+
+    if (complex_subsampled()) {
+	strcpy(gretl_errmsg, _("The data set is currently sub-sampled"));
+	return 1;
+    }
+
+    if (op == DS_ADDOBS || op == DS_COMPACT || 
+	op == DS_EXPAND || op == DS_RESAMPLE) {
+	k = dataset_int_param(s, op, *pZ, pdinfo, &err);
+	if (err) {
+	    return err;
+	}
+    }
+
+    if (op == DS_ADDOBS) {
+	err = add_obs(k, pZ, pdinfo, prn);
+    } else if (op == DS_COMPACT) {
+	err = compact_data_set_wrapper(s, pZ, pdinfo, k);
+    } else if (op == DS_EXPAND) {
+	err = expand_data_set(pZ, pdinfo, k);
+    } else if (op == DS_TRANSPOSE) {
+	err = transpose_data(pZ, pdinfo);
+    } else if (op == DS_SORTBY) {
+	err = dataset_sort(s, *pZ, pdinfo, OPT_NONE);
+    } else if (op == DS_DSORTBY) {
+	err = dataset_sort(s, *pZ, pdinfo, OPT_D);
+    } else if (op == DS_RESAMPLE) {
+	err = dataset_resample(k, pZ, pdinfo);
+    } else if (op == DS_DELETE) {
+	pprintf(prn, "dataset delete: not ready yet\n");
+    } else if (op == DS_KEEP) {
+	pprintf(prn, "dataset keep: not ready yet\n");
+    } else {
+	err = E_PARSE;
+    }
+
+    return err;
+}
+
+/**
+ * dataset_purge_missing_rows:
+ * @Z: data array.
+ * @pdinfo: pointer to data information struct.
+ * 
+ * Removes empty rows from the dataset -- that is, observations
+ * at which there are no non-missing values.  This is intended
+ * for daily data only.
+ *
+ * Returns: 0 on success, non-zero code on error.
+ */
+
+int dataset_purge_missing_rows (double **Z, DATAINFO *pdinfo)
+{
+    int new_n, missrow, totmiss = 0;
+    int t1 = pdinfo->t1;
+    int t2 = pdinfo->t2;
+    char **S = NULL;
+    double *Zi = NULL;
+    size_t sz;
+    int i, t, s;
+    int err = 0;
+
+    for (t=0; t<pdinfo->n; t++) {
+	missrow = 1;
+	for (i=1; i<pdinfo->v; i++) {
+	    if (!var_is_scalar(pdinfo, i) && !na(Z[i][t])) {
+		missrow = 0;
+		break;
+	    }
+	}
+	if (missrow) {
+	    totmiss++;
+	    if (t < pdinfo->t1) {
+		t1--;
+	    }
+	    if (t < pdinfo->t2) {
+		t2--;
+	    }
+	}
+    }
+
+    if (totmiss == 0) {
+	/* no-op */
+	return 0;
+    }
+
+    if (dated_daily_data(pdinfo) && pdinfo->S == NULL) {
+	err = dataset_allocate_obs_markers(pdinfo);
+	if (!err) {
+	    for (t=0; t<pdinfo->n; t++) {
+		calendar_date_string(pdinfo->S[t], t, pdinfo);
+	    }
+	}
+    }
+
+    for (t=0; t<pdinfo->n; t++) {
+	missrow = 1;
+	for (i=1; i<pdinfo->v; i++) {
+	    if (!var_is_scalar(pdinfo, i) && !na(Z[i][t])) {
+		missrow = 0;
+		break;
+	    }
+	}
+	if (missrow) {
+	    sz = (pdinfo->n - t) * sizeof **Z;
+	    for (i=1; i<pdinfo->v; i++) {
+		if (!var_is_scalar(pdinfo, i)) {
+		    memmove(Z[i] + t, Z[i] + t + 1, sz);
+		}
+	    }
+	    if (pdinfo->S != NULL) {
+		free(pdinfo->S[t]);
+		for (s=t; s<pdinfo->n - 1; s++) {
+		    pdinfo->S[s] = pdinfo->S[s+1];
+		}
+	    }
+	}
+    } 
+
+    new_n = pdinfo->n - totmiss;
+
+    for (i=1; i<pdinfo->v; i++) {
+	if (!var_is_scalar(pdinfo, i)) {
+	    Zi = realloc(Z[i], new_n * sizeof *Zi);
+	    if (Zi == NULL) {
+		err = E_ALLOC;
+	    } else {
+		Z[i] = Zi;
+	    }
+	}
+    }
+
+    if (!err && pdinfo->S != NULL) {
+	S = realloc(pdinfo->S, new_n * sizeof *S);
+	if (S == NULL) {
+	    err = E_ALLOC;
+	} else {
+	    pdinfo->S = S;
+	    if (dated_daily_data(pdinfo)) {
+		strcpy(pdinfo->stobs, pdinfo->S[0]);
+		strcpy(pdinfo->endobs, pdinfo->S[new_n-1]);
+		pdinfo->sd0 = get_epoch_day(pdinfo->stobs);
+	    }
+	}
+    }
+
+    pdinfo->n = new_n;
+    pdinfo->t1 = t1;
+    pdinfo->t2 = t2;
+
+    return err;
+}
