@@ -1,20 +1,20 @@
-/*
- *   Copyright (c) by Allin Cottrell
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
- *
- *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- *
+/* 
+ *  gretl -- Gnu Regression, Econometrics and Time-series Library
+ *  Copyright (C) 2001 Allin Cottrell and Riccardo "Jack" Lucchetti
+ * 
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ * 
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ * 
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 /* dbread.c for gretl */
@@ -23,6 +23,10 @@
 #include "swap_bytes.h"
 #include "gretl_www.h"
 
+#include <glib.h>
+#include <unistd.h>
+#include <errno.h>
+
 #if WORDS_BIGENDIAN
 # include <netinet/in.h>
 #endif
@@ -144,21 +148,24 @@ int get_native_db_data (const char *dbbase, SERIESINFO *sinfo,
     char numstr[32];
     FILE *fp;
     dbnumber x;
-    int t, err = 0;
+    int v = sinfo->v;
+    int t, t2, err = 0;
 
     fp = open_binfile(dbbase, GRETL_NATIVE_DB, sinfo->offset, &err);
     if (err) {
 	return err;
     }
 
-    for (t=0; t<sinfo->nobs && !err; t++) {
+    t2 = (sinfo->t2 > 0)? sinfo->t2 : sinfo->nobs - 1;
+
+    for (t=sinfo->t1; t<=t2 && !err; t++) {
 	if (fread(&x, sizeof x, 1, fp) != 1) {
 	    err = DB_PARSE_ERROR;
 	} else {
 	    sprintf(numstr, "%.7g", (double) x); /* N.B. converting a float */
-	    Z[1][t] = atof(numstr);
-	    if (Z[1][t] == DBNA) {
-		Z[1][t] = NADBL;
+	    Z[v][t] = atof(numstr);
+	    if (Z[v][t] == DBNA) {
+		Z[v][t] = NADBL;
 	    }
 	}
     }
@@ -168,10 +175,12 @@ int get_native_db_data (const char *dbbase, SERIESINFO *sinfo,
     return err;
 }
 
-int get_remote_db_data (const char *dbbase, SERIESINFO *sinfo, double **Z)
+int get_remote_db_data (const char *dbbase, SERIESINFO *sinfo, 
+			double **Z)
 {
     char *getbuf = NULL;
-    int t, err, n = sinfo->nobs;
+    int t, t2, err;
+    int v = sinfo->v;
     dbnumber x;
     size_t offset;
 #if WORDS_BIGENDIAN
@@ -191,8 +200,10 @@ int get_remote_db_data (const char *dbbase, SERIESINFO *sinfo, double **Z)
 	return E_FOPEN;
     } 
 
+    t2 = (sinfo->t2 > 0)? sinfo->t2 : sinfo->nobs - 1;
+
     offset = 0L;
-    for (t=0; t<n; t++) {
+    for (t=sinfo->t1; t<=t2; t++) {
 #if WORDS_BIGENDIAN
 	/* go via network byte order */
 	memcpy(&(nf.frac), getbuf + offset, sizeof nf.frac);
@@ -205,11 +216,7 @@ int get_remote_db_data (const char *dbbase, SERIESINFO *sinfo, double **Z)
 	memcpy(&x, getbuf + offset, sizeof x);
 	offset += sizeof x;
 #endif
-	if (x == -999.0) {
-	    Z[1][t] = NADBL;
-	} else {
-	    Z[1][t] = x;
-	}
+	Z[v][t] = (x == DBNA)? NADBL : x;
     }
 
     free(getbuf);
@@ -222,14 +229,17 @@ int get_pcgive_db_data (const char *dbbase, SERIESINFO *sinfo,
 {
     FILE *fp;
     double x;
-    int t, err = 0;
+    int v = sinfo->v;
+    int t, t2, err = 0;
 
     fp = open_binfile(dbbase, GRETL_PCGIVE_DB, sinfo->offset, &err);
     if (err) {
 	return err;
     }
 
-    for (t=0; t<sinfo->nobs; t++) {
+    t2 = (sinfo->t2 > 0)? sinfo->t2 : sinfo->nobs - 1;
+
+    for (t=sinfo->t1; t<=t2; t++) {
 	if (fread(&x, sizeof x, 1, fp) != 1) {
 	    err = E_DATA;
 	    break;
@@ -238,10 +248,10 @@ int get_pcgive_db_data (const char *dbbase, SERIESINFO *sinfo,
 	reverse_double(x);
 #endif
 	if (x == -9999.99 || isnan(x)) {
-	    Z[1][t] = NADBL;
+	    Z[v][t] = NADBL;
 	    err = DB_MISSING_DATA;
 	} else {
-	    Z[1][t] = x;
+	    Z[v][t] = x;
 	}
     }
 
@@ -280,7 +290,8 @@ static int get_native_series_obs (SERIESINFO *sinfo,
 				  const char *stobs,
 				  const char *endobs)
 {
-    if (strchr(stobs, '/')) { /* calendar data */
+    if (strchr(stobs, '/')) { 
+	/* calendar data */
 	const char *q = stobs;
 	const char *p = strchr(stobs, '/');
 
@@ -302,40 +313,105 @@ static int get_native_series_obs (SERIESINFO *sinfo,
     return 0;
 }
 
+static int 
+open_native_db_files (const char *dname, FILE **f1, char *name1, 
+		      FILE **f2, char *name2)
+{
+    char dbbase[FILENAME_MAX];
+    char fname[FILENAME_MAX];
+    FILE *fidx = NULL, *fbin = NULL;
+    int err = 0;
+
+    if (dname != NULL) {
+	strcpy(dbbase, dname);
+    } else {
+	strcpy(dbbase, db_name);
+    }
+
+    if (has_suffix(dbbase, ".bin")) {
+	dbbase[strlen(dbbase) - 4] = '\0';
+    }
+
+    if (f1 != NULL) {
+	strcpy(fname, dbbase);
+	strcat(fname, ".idx");
+
+	if (name1 != NULL) {
+	    err = gretl_write_access(fname);
+	    if (!err) {
+		strcpy(name1, fname);
+	    }
+	}
+
+	if (!err) {
+	    fidx = gretl_fopen(fname, "r");
+	    if (fidx == NULL) {
+		strcpy(gretl_errmsg, _("Couldn't open database index file"));
+		err = E_FOPEN;
+	    } 
+	}
+    }
+
+    if (f2 != NULL && !err) {
+	strcpy(fname, dbbase);
+	strcat(fname, ".bin");
+
+	if (name2 != NULL) {
+	    err = gretl_write_access(fname);
+	    if (!err) {
+		strcpy(name2, fname);
+	    }
+	}
+
+	if (!err) {
+	    fbin = gretl_fopen(fname, "rb");
+	    if (fbin == NULL) {
+		strcpy(gretl_errmsg, _("Couldn't open database binary file"));
+		err = E_FOPEN;
+	    } 
+	}
+    }
+
+    if (err) {
+	if (fidx != NULL) {
+	    fclose(fidx);
+	}
+    } else {
+	if (f1 != NULL) {
+	    *f1 = fidx;
+	}
+	if (f2 != NULL) {
+	    *f2 = fbin;
+	}
+    }
+
+    return err;
+}
+
 static int 
 get_native_series_info (const char *series, SERIESINFO *sinfo)
 {
-    FILE *fp;
-    char dbidx[MAXLEN];
+    FILE *fp = NULL;
     char sername[VNAMELEN];
-    char line1[256], line2[72];
+    char s1[256], s2[72];
     char stobs[11], endobs[11];
-    char *p, pdc;
+    char pdc;
     int offset = 0;
     int gotit = 0, err = 0;
     int n;
 
-    strcpy(dbidx, db_name);
-    p = strstr(dbidx, ".bin");
-    if (p != NULL) {
-	strcpy(p, ".idx");
-    } else {
-	strcat(dbidx, ".idx");
+    err = open_native_db_files(db_name, &fp, NULL, NULL, NULL);
+    if (err) {
+	return err;
     }
 
-    fp = gretl_fopen(dbidx, "r");
-    if (fp == NULL) {
-	strcpy(gretl_errmsg, _("Couldn't open database index file"));
-	return E_FOPEN;
-    }
+    while (fgets(s1, sizeof s1, fp) && !gotit) {
 
-    while (fgets(line1, sizeof line1, fp) && !gotit) {
-
-	if (*line1 == '#') {
+	if (*s1 == '#') {
 	    continue;
 	}
 
-	if (sscanf(line1, "%15s", sername) != 1) {
+	if (sscanf(s1, "%15s", sername) != 1) {
 	    break;
 	}
 
@@ -344,21 +420,22 @@ get_native_series_info (const char *series, SERIESINFO *sinfo)
 	    strcpy(sinfo->varname, sername);
 	}
 
-	fgets(line2, sizeof line2, fp);
+	fgets(s2, sizeof s2, fp);
 
 	if (gotit) {
-	    get_native_series_comment(sinfo, line1);
-	    if (sscanf(line2, "%c %10s %*s %10s %*s %*s %d", 
-		       &pdc, stobs, endobs, &(sinfo->nobs)) != 4) {
+	    get_native_series_comment(sinfo, s1);
+	    if (sscanf(s2, "%c %10s %*s %10s %*s %*s %d", 
+		       &pdc, stobs, endobs, &sinfo->nobs) != 4) {
 		strcpy(gretl_errmsg, _("Failed to parse series information"));
 		err = DB_PARSE_ERROR;
 	    } else {
 		get_native_series_pd(sinfo, pdc);
 		get_native_series_obs(sinfo, stobs, endobs);
 		sinfo->offset = offset;
+		sinfo->t2 = sinfo->nobs - 1;
 	    }
 	} else {
-	    if (sscanf(line2, "%*c %*s %*s %*s %*s %*s %d", &n) != 1) {
+	    if (sscanf(s2, "%*c %*s %*s %*s %*s %*s %d", &n) != 1) {
 		strcpy(gretl_errmsg, _("Failed to parse series information"));
 		err = DB_PARSE_ERROR;
 	    } else {
@@ -382,7 +459,7 @@ get_remote_series_info (const char *series, SERIESINFO *sinfo)
 {
     char *buf = NULL;
     char sername[VNAMELEN];
-    char line1[256], line2[72];
+    char s1[256], s2[72];
     char stobs[11], endobs[11];
     char pdc;
     int gotit = 0;
@@ -397,12 +474,12 @@ get_remote_series_info (const char *series, SERIESINFO *sinfo)
 
     bufgets_init(buf);
 
-    while (bufgets(line1, sizeof line1, buf) && !gotit) {
-	if (*line1 == '#') {
+    while (bufgets(s1, sizeof s1, buf) && !gotit) {
+	if (*s1 == '#') {
 	    continue;
 	}
 
-	if (sscanf(line1, "%15s", sername) != 1) {
+	if (sscanf(s1, "%15s", sername) != 1) {
 	    break;
 	}
 
@@ -412,21 +489,22 @@ get_remote_series_info (const char *series, SERIESINFO *sinfo)
 
 	gotit = 1;
 
-	if (bufgets(line2, sizeof line2, buf) == NULL) {
+	if (bufgets(s2, sizeof s2, buf) == NULL) {
 	    err = DB_PARSE_ERROR;
 	    break;
 	} 
 
 	strcpy(sinfo->varname, sername);
-	get_native_series_comment(sinfo, line1);
+	get_native_series_comment(sinfo, s1);
 
-	if (sscanf(line2, "%c %10s %*s %10s %*s %*s %d", 
-		   &pdc, stobs, endobs, &(sinfo->nobs)) != 4) {
+	if (sscanf(s2, "%c %10s %*s %10s %*s %*s %d", 
+		   &pdc, stobs, endobs, &sinfo->nobs) != 4) {
 	    strcpy(gretl_errmsg, _("Failed to parse series information"));
 	    err = DB_PARSE_ERROR;
 	} else {
 	    get_native_series_pd(sinfo, pdc);
 	    get_native_series_obs(sinfo, stobs, endobs);
+	    sinfo->t2 = sinfo->nobs - 1;
 	}
     }
 
@@ -442,9 +520,46 @@ get_remote_series_info (const char *series, SERIESINFO *sinfo)
     return err;
 }
 
-static int in7_nobs (int y0, int p0, int y1, int p1, int pd)
+static int in7_get_obs (int y0, int p0, int y1, int p1, 
+			SERIESINFO *sinfo)
 {
-    return  (y1 - y0 + 1) * pd - (p0 - 1) - (pd - p1);
+    int pd = sinfo->pd;
+    int n = (y1 - y0 + 1) * pd - (p0 - 1) - (pd - p1);
+    int err = 0;
+
+    if (n <= 0) {
+	err = 1;
+    } else {
+	sinfo->nobs = n;
+	sinfo->t2 = n - 1;
+    }
+
+    return err;
+}
+
+static int 
+pcgive_set_stobs_endobs (int y0, int p0, int y1, int p1, 
+			 SERIESINFO *sinfo)
+{
+    int err = 0;
+
+    if (sinfo->pd == 1) {
+	sprintf(sinfo->stobs, "%d", y0);
+	sprintf(sinfo->endobs, "%d", y1);
+	if (y0 == 1) {
+	    sinfo->undated = 1;
+	}
+    } else if (sinfo->pd == 4) {
+	sprintf(sinfo->stobs, "%d:%d", y0, p0);
+	sprintf(sinfo->endobs, "%d:%d", y1, p1);
+    } else if (sinfo->pd == 12 || sinfo->pd == 52) {
+	sprintf(sinfo->stobs, "%d:%02d", y0, p0);
+	sprintf(sinfo->endobs, "%d:%02d", y1, p1);
+    } else {
+	err = E_DATA; /* FIXME? */
+    }
+    
+    return err;
 }
 
 static int 
@@ -504,21 +619,9 @@ get_pcgive_series_info (const char *series, SERIESINFO *sinfo)
 		    }
 		}
 		/* transcribe info */
-		sinfo->nobs = in7_nobs(y0, p0, y1, p1, sinfo->pd);
-		if (sinfo->pd == 1) {
-		    sprintf(sinfo->stobs, "%d", y0);
-		    sprintf(sinfo->endobs, "%d", y1);
-		    if (y0 == 1) {
-			sinfo->undated = 1;
-		    }
-		} else if (sinfo->pd == 4) {
-		    sprintf(sinfo->stobs, "%d:%d", y0, p0);
-		    sprintf(sinfo->endobs, "%d:%d", y1, p1);
-		} else if (sinfo->pd == 12 || sinfo->pd == 52) {
-		    sprintf(sinfo->stobs, "%d:%02d", y0, p0);
-		    sprintf(sinfo->endobs, "%d:%02d", y1, p1);
-		} else {
-		    err = E_DATA; /* FIXME? */
+		err = in7_get_obs(y0, p0, y1, p1, sinfo);
+		if (!err) {
+		    err = pcgive_set_stobs_endobs(y0, p0, y1, p1, sinfo);
 		}
 	    } else {
 		err = E_DATA;
@@ -631,15 +734,12 @@ static int dinfo_to_sinfo (const DATEINFO *dinfo, SERIESINFO *sinfo,
 
     sinfo->pd = dinfo->info;
     sinfo->nobs = n;
-
-    *sinfo->varname = 0;
-    strncat(sinfo->varname, varname, VNAMELEN - 1);
-
-    *sinfo->descrip = 0;
-    strncat(sinfo->descrip, comment, MAXLABEL - 1);
-
+    sinfo->t2 = n - 1;
     sinfo->offset = offset;
 
+    strncat(sinfo->varname, varname, VNAMELEN - 1);
+    strncat(sinfo->descrip, comment, MAXLABEL - 1);
+
 #if DB_DEBUG
     fprintf(stderr, "dinfo_to_sinfo: '%s': set sinfo->offset = %d\n", varname, 
 	    (int) offset);
@@ -673,10 +773,8 @@ static int in7_to_sinfo (const char *varname, const char *comment,
     }
 
     if (!err) {
-	sinfo->nobs = in7_nobs(y0, p0, y1, p1, pd);
-	if (sinfo->nobs <= 0) {
-	    err = 1;
-	}
+	sinfo->pd = pd;
+	err = in7_get_obs(y0, p0, y1, p1, sinfo);
     }
 
     if (!err) {
@@ -776,81 +874,87 @@ static RECNUM read_rats_directory (FILE *fp, const char *series_name,
 
 static void series_info_init (SERIESINFO *sinfo)
 {
-    *sinfo->varname = 0;
-    *sinfo->descrip = 0;
+    sinfo->t1 = sinfo->t2 = 0;
     sinfo->nobs = 0;
-    *sinfo->stobs = 0;
-    *sinfo->endobs = 0;
+    sinfo->v = 1;
     sinfo->pd = 1;
     sinfo->offset = -1;
     sinfo->err = 0;
     sinfo->undated = 0;
+
+    *sinfo->varname = '\0';
+    *sinfo->descrip = '\0';
+    *sinfo->stobs = '\0';
+    *sinfo->endobs = '\0';
 }
 
 #define DB_INIT_ROWS 32
 
-static db_table *db_table_new (int nr)
+void dbwrapper_destroy (dbwrapper *dw)
 {
-    db_table *tbl;
+    if (dw != NULL) {
+	free(dw->sinfo);
+	free(dw);
+    }
+}
+
+dbwrapper *dbwrapper_new (int n)
+{
+    dbwrapper *dw;
     int i;
 
-    if (nr == 0) {
-	nr = DB_INIT_ROWS;
+    if (n == 0) {
+	n = DB_INIT_ROWS;
     }
 
-    tbl = malloc(sizeof *tbl);
-    if (tbl == NULL) return NULL;
-
-    tbl->sinfo = malloc(nr * sizeof *tbl->sinfo);
-
-    if (tbl->sinfo == NULL) {
-	free(tbl);
+    dw = malloc(sizeof *dw);
+    if (dw == NULL) {
 	return NULL;
     }
 
-    for (i=0; i<nr; i++) {
-	series_info_init(&tbl->sinfo[i]);
+    dw->sinfo = malloc(n * sizeof *dw->sinfo);
+    if (dw->sinfo == NULL) {
+	free(dw);
+	return NULL;
     }
 
-    tbl->nvars = 0;
-    tbl->nalloc = nr;
+    for (i=0; i<n; i++) {
+	series_info_init(&dw->sinfo[i]);
+    }
 
-    return tbl;
+    dw->nv = 0;
+    dw->nalloc = n;
+
+    return dw;
 }
 
-static int db_table_expand (db_table *tbl)
+static int dbwrapper_expand (dbwrapper *dw)
 {
     SERIESINFO *sinfo;
     int i, newsz;
 
-    newsz = (tbl->nvars / DB_INIT_ROWS) + 1;
+    newsz = (dw->nv / DB_INIT_ROWS) + 1;
     newsz *= DB_INIT_ROWS;
 
-    sinfo = realloc(tbl->sinfo, newsz * sizeof *sinfo);
+    sinfo = realloc(dw->sinfo, newsz * sizeof *sinfo);
     if (sinfo == NULL) {
-	free(tbl->sinfo);
-	tbl->sinfo = NULL;
+	free(dw->sinfo);
+	dw->sinfo = NULL;
 	return 1;
     }
 
-    tbl->sinfo = sinfo;
+    dw->sinfo = sinfo;
 
-    for (i=tbl->nalloc; i<newsz; i++) {
-	series_info_init(&tbl->sinfo[i]);
+    for (i=dw->nalloc; i<newsz; i++) {
+	series_info_init(&dw->sinfo[i]);
     }
 
-    tbl->nalloc = newsz;
+    dw->nalloc = newsz;
 
     return 0;
 }
 
-static void db_table_free (db_table *tbl)
-{
-    free(tbl->sinfo);
-    free(tbl);
-}
-
-static int read_in7_series_info (FILE *fp, db_table *tbl)
+static int read_in7_series_info (FILE *fp, dbwrapper *dw)
 {
     char line[1024];
     char sname[VNAMELEN];
@@ -887,9 +991,9 @@ static int read_in7_series_info (FILE *fp, db_table *tbl)
 		}
 		/* record info */
 		err = in7_to_sinfo(sname, desc, y0, p0, y1, p1,
-				   pd, offset, &tbl->sinfo[i++]);
+				   pd, offset, &dw->sinfo[i++]);
 		if (!err) {
-		    tbl->nvars += 1;
+		    dw->nv += 1;
 		}
 	    }
 	}
@@ -934,13 +1038,13 @@ static int count_in7_series (FILE *fp, int *err)
  *
  * Read the series info from a PcGive database, .in7 file
  * 
- * Returns: pointer to a #db_table containing the series info,
+ * Returns: pointer to a #dbwrapper containing the series info,
  * or NULL in case of failure.
  */
 
-db_table *read_pcgive_db (FILE *fp) 
+dbwrapper *read_pcgive_db (FILE *fp) 
 {
-    db_table *tbl;
+    dbwrapper *dw;
     int ns, err = 0;
 
     gretl_error_clear();
@@ -958,8 +1062,8 @@ db_table *read_pcgive_db (FILE *fp)
 #endif
     
     /* allocate table for series rows */
-    tbl = db_table_new(ns);
-    if (tbl == NULL) {
+    dw = dbwrapper_new(ns);
+    if (dw == NULL) {
 	strcpy(gretl_errmsg, _("Out of memory!"));
 	return NULL;
     }
@@ -967,14 +1071,14 @@ db_table *read_pcgive_db (FILE *fp)
     rewind(fp);
     
     /* Go find the series info */
-    err = read_in7_series_info(fp, tbl);
+    err = read_in7_series_info(fp, dw);
 
     if (err) {
-	db_table_free(tbl);
-	return NULL;
+	dbwrapper_destroy(dw);
+	dw = NULL;
     }
 
-    return tbl;
+    return dw;
 }
 
 /**
@@ -983,15 +1087,15 @@ db_table *read_pcgive_db (FILE *fp)
  *
  * Read the series info from a RATS 4.0 database: read the base 
  * block at offset 0 in the data file, and recurse through the 
- * directory entries
+ * directory entries.
  * 
- * Returns: pointer to a #db_table containing the series info,
+ * Returns: pointer to a #dbwrapper containing the series info,
  * or NULL in case of failure.
  */
 
-db_table *read_rats_db (FILE *fp) 
+dbwrapper *read_rats_db (FILE *fp) 
 {
-    db_table *tbl;
+    dbwrapper *dw;
     long forward;
     int i, err = 0;
 
@@ -1010,8 +1114,8 @@ db_table *read_rats_db (FILE *fp)
     }
 
     /* allocate table for series rows */
-    tbl = db_table_new(0);
-    if (tbl == NULL) {
+    dw = dbwrapper_new(0);
+    if (dw == NULL) {
 	strcpy(gretl_errmsg, _("Out of memory!"));
 	return NULL;
     }
@@ -1019,13 +1123,13 @@ db_table *read_rats_db (FILE *fp)
     /* Go find the series */
     i = 0;
     while (forward && !err) {
-	tbl->nvars += 1;
+	dw->nv += 1;
 #if DB_DEBUG
-	fprintf(stderr, "read_rats_db: forward = %d, nvars = %d\n",
-		(int) forward, tbl->nvars);
+	fprintf(stderr, "read_rats_db: forward = %d, nv = %d\n",
+		(int) forward, dw->nv);
 #endif    
-	if (tbl->nvars > 0 && tbl->nvars % DB_INIT_ROWS == 0) {
-	    err = db_table_expand(tbl);
+	if (dw->nv > 0 && dw->nv % DB_INIT_ROWS == 0) {
+	    err = dbwrapper_expand(dw);
 	    if (err) {
 		strcpy(gretl_errmsg, _("Out of memory!"));
 	    }
@@ -1033,7 +1137,7 @@ db_table *read_rats_db (FILE *fp)
 	if (!err) {
 	    err = fseek(fp, (forward - 1) * 256L, SEEK_SET);
 	    if (!err) {
-		forward = read_rats_directory(fp, NULL, &tbl->sinfo[i++]);
+		forward = read_rats_directory(fp, NULL, &dw->sinfo[i++]);
 		if (forward == RATS_PARSE_ERROR) {
 		    err = 1;
 		}
@@ -1045,16 +1149,16 @@ db_table *read_rats_db (FILE *fp)
     }
 
 #if DB_DEBUG
-    fprintf(stderr, "read_rats_db: err = %d, tbl = %p\n",
-	    err, (void *) tbl);
+    fprintf(stderr, "read_rats_db: err = %d, dw = %p\n",
+	    err, (void *) dw);
 #endif    
 
     if (err) {
-	db_table_free(tbl);
+	dbwrapper_destroy(dw);
 	return NULL;
     }
 
-    return tbl;
+    return dw;
 }
 
 /* retrieve the actual data values from the data blocks */
@@ -1063,18 +1167,27 @@ static int get_rats_series (int offset, SERIESINFO *sinfo, FILE *fp,
 			    double **Z)
 {
     RATSData rdata;
-    int miss = 0, i, t = 0;
     double x;
-
+    int miss = 0;
+    int v = sinfo->v;
+    int i, t, T;
+ 
     fprintf(stderr, "get_rats_series: starting from offset %d\n", offset);
+
+    if (sinfo->t2 > 0) {
+	T = sinfo->t2 + 1;
+    } else {
+	T = sinfo->nobs;
+    }
     
     rdata.forward_point = offset;
+    t = sinfo->t1;
 
     while (rdata.forward_point) {
 	fseek(fp, (rdata.forward_point - 1) * 256L, SEEK_SET);
 	/* the RATSData struct is actually 256 bytes.  Yay! */
 	fread(&rdata, sizeof rdata, 1, fp);
-	for (i=0; i<31 && t<sinfo->nobs; i++) {
+	for (i=0; i<31 && t<T; i++) {
 	    x = rdata.data[i];
 #if WORDS_BIGENDIAN
 	    reverse_double(x);
@@ -1083,7 +1196,7 @@ static int get_rats_series (int offset, SERIESINFO *sinfo, FILE *fp,
 		x = NADBL;
 		miss = 1;
 	    }
-	    Z[1][t++] = x;
+	    Z[v][t++] = x;
 	}
     }
 
@@ -1394,6 +1507,11 @@ set_db_name (const char *fname, int filetype, const PATHS *ppaths, PRN *prn)
     return err;
 }
 
+const char *get_db_name (void)
+{
+    return db_name;
+}
+
 int db_set_sample (const char *line, DATAINFO *pdinfo)
 {
     char cmd[5], start[OBSLEN], stop[OBSLEN];
@@ -1501,6 +1619,7 @@ get_compact_method_and_advance (const char *s, CompactMethod *method)
 static double **new_dbZ (int n)
 {
     double **Z;
+    int t;
 
     Z = malloc(2 * sizeof *Z);
     if (Z == NULL) return NULL;
@@ -1513,9 +1632,21 @@ static double **new_dbZ (int n)
 	return NULL;
     }
 
+    for (t=0; t<n; t++) {
+	Z[1][t] = NADBL;
+    }
+
     return Z;
 }
 
+static void free_dbZ (double **dbZ)
+{
+    if (dbZ != NULL) {
+	free(dbZ[1]);
+	free(dbZ);
+    }
+}
+
 /* main function for getting a series out of a database, using the
    command-line client or in script or console mode
 */
@@ -1540,16 +1671,16 @@ int db_get_series (const char *line, double ***pZ, DATAINFO *pdinfo,
 	return 1;
     }   
 
-    series_info_init(&sinfo);
-
     line = get_compact_method_and_advance(line, &method);
 
     /* now loop over variable names given on the line */
 
-    while ((line = get_word_and_advance(line, series, VNAMELEN - 1)) && !err) {
+    while ((line = get_word_and_advance(line, series, VNAMELEN-1)) && !err) {
 	int v, this_var_method = method; 
 	int newdata = 0;
 
+	series_info_init(&sinfo);
+
 	/* see if the series is already in the dataset */
 	v = varindex(pdinfo, series);
 	if (v < pdinfo->v && method == COMPACT_NONE) {
@@ -1616,8 +1747,7 @@ int db_get_series (const char *line, double ***pZ, DATAINFO *pdinfo,
 	}
 
 	/* free up temp stuff */
-	free(dbZ[1]);
-	free(dbZ);
+	free_dbZ(dbZ);
 
 	if (!err) {
 	    pprintf(prn, _("Series imported OK"));
@@ -1631,6 +1761,215 @@ int db_get_series (const char *line, double ***pZ, DATAINFO *pdinfo,
     return err;
 }
 
+static FILE *tempfile_open (char *fname, int *err)
+{
+    FILE *fp = NULL;
+    int fd;
+
+    strcat(fname, ".XXXXXX");
+    fd = g_mkstemp(fname);
+
+    if (fd < 0) {
+	*err = E_FOPEN;
+    } else {
+	fp = fdopen(fd, "w+");
+	if (fp == NULL) {
+	    *err = E_FOPEN;
+	    sprintf(gretl_errmsg, _("Couldn't open %s"), fname);
+	    close(fd);
+	    remove(fname);
+	}
+    } 
+
+    return fp;
+}
+
+static void maybe_fclose (FILE *fp)
+{
+    if (fp != NULL) {
+	fclose(fp);
+    }
+}
+
+#define DBUFLEN 1024
+
+static int db_delete_series (const char *line, const int *list,
+			     const char *fname, PRN *prn)
+{
+    dbnumber buf[DBUFLEN];
+    char src1[FILENAME_MAX];
+    char src2[FILENAME_MAX];
+    char tmp1[FILENAME_MAX];
+    char tmp2[FILENAME_MAX];
+    char series[VNAMELEN];
+    char *p, s[512];
+    char **snames = NULL;
+    FILE *fidx = NULL, *fbin = NULL;
+    FILE *f1 = NULL, *f2 = NULL;
+    int i, j, k, print, n, ns;
+    int ndel = 0;
+    int err = 0;
+
+    if (fname == NULL) {
+	if (*db_name == '\0') {
+	    strcpy(gretl_errmsg, _("No database has been opened"));
+	    err = 1;
+	} else if (db_type != GRETL_NATIVE_DB) {
+	    strcpy(gretl_errmsg, "This only works for gretl databases");
+	    err = 1;
+	} else {
+	    err = open_native_db_files(db_name, &fidx, src1, &fbin, src2);
+	}
+    } else {
+	err = open_native_db_files(fname, &fidx, src1, &fbin, src2);
+    }
+
+    if (err) {
+	return err;
+    }
+
+    strcpy(tmp1, gretl_user_dir());
+    strcat(tmp1, "tmpidx");
+    f1 = tempfile_open(tmp1, &err);
+    if (err) {
+	goto bailout;
+    }
+
+    strcpy(tmp2, gretl_user_dir());
+    strcat(tmp2, "tmpbin");
+    f2 = tempfile_open(tmp2, &err);
+    if (err) {
+	goto bailout;
+    }    
+
+    if (line != NULL) {
+	/* extract the variable names given on the line */
+	ns = 0;
+	while ((line = get_word_and_advance(line, series, VNAMELEN-1)) 
+	       && !err) {
+	    err = strings_array_add(&snames, &ns, series);
+	}
+	if (!err && ns == 0) {
+	    fprintf(stderr, "Found no series names\n");
+	    err = E_PARSE;
+	}
+    }
+
+    print = k = 1;
+    i = j = 0;
+
+    while (fgets(s, sizeof s, fidx) && !err) {
+	if (i == 0) {
+	    /* always reprint the header */
+	    fputs(s, f1);
+	    i++;
+	    continue;
+	}
+
+	if (i % 2 != 0) {
+	    /* odd lines contain varnames */
+	    print = 1;
+	    if (snames != NULL) {
+		sscanf(s, "%s", series);
+		for (j=0; j<ns; j++) {
+		    if (!strcmp(series, snames[j])) {
+			print = 0;
+			ndel++;
+			break;
+		    }
+		}
+	    } else {
+		if (k <= list[0] && list[k] == j) {
+		    k++;
+		    print = 0;
+		    ndel++;
+		}
+		j++;
+	    }
+	    if (print) {
+		fputs(s, f1);
+	    }
+	} else {
+	    /* even lines have obs information */
+	    p = strstr(s, "n = ");
+	    if (p != NULL) {
+		sscanf(p + 4, "%d", &n);
+	    } else {
+		err = E_DATA;
+		fprintf(stderr, "couldn't find obs for series\n");
+	    }
+
+	    if (!print) {
+		fseek(fbin, n * sizeof(dbnumber), SEEK_CUR);
+	    } else {
+		int get, got, rem = n;
+
+		fputs(s, f1);
+
+		while (rem > 0 && !err) {
+		    get = (rem > DBUFLEN)? DBUFLEN : rem;
+		    got = fread(buf, sizeof(dbnumber), get, fbin);
+		    if (got != get) {
+			fprintf(stderr, "error reading binary data\n");
+			err = E_DATA;
+		    } else {
+			fwrite(buf, sizeof(dbnumber), got, f2);
+			rem -= got;
+		    }
+		}
+	    }	    
+	}
+	i++;
+    }
+
+    if (snames != NULL) {
+	free_strings_array(snames, ns);
+    }
+
+ bailout:
+
+    maybe_fclose(fidx);
+    maybe_fclose(fbin);
+    maybe_fclose(f1);
+    maybe_fclose(f2);
+
+    if (!err && ndel > 0) {
+#ifdef G_OS_WIN32
+	err = win32_rename(tmp1, src1);
+	if (!err) {
+	    err = win32_rename(tmp2, src2);
+	}
+#else
+	err = rename(tmp1, src1);
+	if (!err) {
+	    err = rename(tmp2, src2);
+	}
+	if (err) {
+	    gretl_errmsg_set_from_errno();
+	}
+#endif
+    } else {
+	remove(tmp1);
+	remove(tmp2);
+    }
+
+    if (!err && prn != NULL) {
+	pprintf(prn, "Deleted %d series from %s\n", ndel, src2);
+    }
+
+    return err;
+}
+
+int db_delete_series_by_name (const char *line, PRN *prn)
+{
+    return db_delete_series(line, NULL, NULL, prn);
+}
+
+int db_delete_series_by_number (const int *list, const char *fname)
+{
+    return db_delete_series(NULL, list, fname, NULL);
+}
+
 void get_db_padding (SERIESINFO *sinfo, DATAINFO *pdinfo, 
 		     int *pad1, int *pad2)
 {
@@ -1642,28 +1981,38 @@ void get_db_padding (SERIESINFO *sinfo, DATAINFO *pdinfo,
 	    pdinfo->n, sinfo->nobs, *pad1, *pad2);
 } 
 
+int db_range_check (SERIESINFO *sinfo, DATAINFO *pdinfo)
+{
+    double sdn_orig = get_date_x(pdinfo->pd, pdinfo->endobs);
+    double sd0 = get_date_x(sinfo->pd, sinfo->stobs);
+    double sdn = get_date_x(sinfo->pd, sinfo->endobs);
+    int err = 0;
+
+    if (sd0 > sdn_orig || sdn < pdinfo->sd0) {
+	sprintf(gretl_errmsg, _("%s: observation range does not overlap\n"
+				"with the working data set"),
+		sinfo->varname);
+	err = 1;
+    }
+
+    return err;
+}
+
 int check_db_import (SERIESINFO *sinfo, DATAINFO *pdinfo)
 {
-    double sd0, sdn_new, sdn_old;
     int err = 0;
 
     if (sinfo->pd < pdinfo->pd) {
 	if (sinfo->pd != 1 && sinfo->pd != 4 && 
 	    pdinfo->pd != 4 && pdinfo->pd != 12) {
-	    strcpy(gretl_errmsg, _("Sorry, can't handle this conversion yet!"));
+	    sprintf(gretl_errmsg, _("%s: can't handle conversion"),
+		    sinfo->varname);
 	    err = 1;
 	} 
     }
 
     if (!err) {
-	sd0 = get_date_x(sinfo->pd, sinfo->stobs);
-	sdn_new = get_date_x(sinfo->pd, sinfo->endobs);
-	sdn_old = get_date_x(pdinfo->pd, pdinfo->endobs);
-	if (sd0 > sdn_old || sdn_new < pdinfo->sd0) {
-	    strcpy(gretl_errmsg, _("Observation range does not overlap\n"
-				   "with the working data set"));
-	    err = 1;
-	}
+	err = db_range_check(sinfo, pdinfo);
     }
 
 #if DB_DEBUG
@@ -1677,7 +2026,8 @@ int check_db_import (SERIESINFO *sinfo, DATAINFO *pdinfo)
     return err;
 }
 
-void init_datainfo_from_sinfo (DATAINFO *pdinfo, SERIESINFO *sinfo)
+static void 
+init_datainfo_from_sinfo (DATAINFO *pdinfo, SERIESINFO *sinfo)
 {
     pdinfo->pd = sinfo->pd;
 
@@ -2866,16 +3216,19 @@ int expand_data_set (double ***pZ, DATAINFO *pdinfo, int newpd)
 {
     char stobs[12];
     int oldn = pdinfo->n;
-    int mult, newn, addobs;
+    int oldpd = pdinfo->pd;
+    int t1 = pdinfo->t1;
+    int t2 = pdinfo->t2;
+    int mult, newn, nadd;
     double *x = NULL;
     int i, j, s, t;
     int err = 0;
 
-    if (pdinfo->pd != 1 && pdinfo->pd != 4) {
+    if (oldpd != 1 && oldpd != 4) {
 	return E_PDWRONG;
-    } else if (pdinfo->pd == 1 && newpd != 4 && newpd != 12) {
+    } else if (oldpd == 1 && newpd != 4 && newpd != 12) {
 	return E_DATA;
-    } else if (pdinfo->pd == 4 && newpd != 12) {
+    } else if (oldpd == 4 && newpd != 12) {
 	return E_DATA;
     }
 
@@ -2884,11 +3237,11 @@ int expand_data_set (double ***pZ, DATAINFO *pdinfo, int newpd)
 	return E_ALLOC;
     }
 
-    mult = newpd / pdinfo->pd;
+    mult = newpd / oldpd;
     newn = mult * pdinfo->n;
-    addobs = newn - oldn;
+    nadd = newn - oldn;
 
-    err = dataset_add_observations(addobs, pZ, pdinfo, OPT_NONE);
+    err = dataset_add_observations(nadd, pZ, pdinfo, OPT_NONE);
     if (err) {
 	goto bailout;
     }
@@ -2908,7 +3261,7 @@ int expand_data_set (double ***pZ, DATAINFO *pdinfo, int newpd)
 	}
     }
 
-     if (pdinfo->pd == 1) {
+    if (pdinfo->pd == 1) {
 	strcpy(stobs, pdinfo->stobs);
 	if (newpd == 4) {
 	    strcat(stobs, ":1");
@@ -2923,9 +3276,18 @@ int expand_data_set (double ***pZ, DATAINFO *pdinfo, int newpd)
 	sprintf(stobs, "%d:%02d", yr, mo);
     }
 
+    if (pdinfo->t1 > 0) {
+	pdinfo->t1 *= mult;
+    }
+
+    if (pdinfo->t2 < oldn - 1) {
+	pdinfo->t2 = pdinfo->t1 + (t2 - t1 + 1) * mult - 1;
+    }    
+
     strcpy(pdinfo->stobs, stobs);
     pdinfo->pd = newpd;
     pdinfo->sd0 = get_date_x(pdinfo->pd, pdinfo->stobs);
+    ntodate(pdinfo->endobs, pdinfo->n - 1, pdinfo);
 
     if (pdinfo->markers) {
 	dataset_destroy_obs_markers(pdinfo);
