@@ -1,20 +1,20 @@
-/* gretl - The Gnu Regression, Econometrics and Time-series Library
- * Copyright (C) 1999-2000 Ramu Ramanathan and Allin Cottrell
- *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License 
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
- *
- * This software is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License 
- * along with this software; if not, write to the 
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+/* 
+ *  gretl -- Gnu Regression, Econometrics and Time-series Library
+ *  Copyright (C) 2001 Allin Cottrell and Riccardo "Jack" Lucchetti
+ * 
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ * 
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ * 
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 #ifndef DESCRIBE_H
@@ -140,6 +140,16 @@ int crosstab (const int *list, const double **Z,
 	      const DATAINFO *pdinfo, gretlopt opt,
 	      PRN *prn);
 
+int compare_xtab_rows (const void *a, const void *b);
+
+Xtab *single_crosstab (const int *list, const double **Z, 
+		       const DATAINFO *pdinfo, gretlopt opt, 
+		       PRN *prn, int *err);
+
+gretl_matrix *xtab_to_matrix (const Xtab *tab);
+
+void free_xtab (Xtab *tab);
+
 int model_error_dist (const MODEL *pmod, double ***pZ,
 		      DATAINFO *pdinfo, PRN *prn);
 
@@ -148,15 +158,15 @@ int auto_acf_order (int pd, int nobs);
 int auto_spectrum_order (int T, gretlopt opt);
 
 int corrgram (int varno, int order, int nparam,
-	      double ***pZ, DATAINFO *pdinfo, 
+	      const double **Z, DATAINFO *pdinfo, 
 	      PRN *prn, gretlopt opt);
 
 int xcorrgram (const int *list, int order, 
-	      double ***pZ, DATAINFO *pdinfo, 
-	      PRN *prn, gretlopt opt);
+	       const double **Z, DATAINFO *pdinfo, 
+	       PRN *prn, gretlopt opt);
 
 int periodogram (int varno, int width, 
-		 double ***pZ, const DATAINFO *pdinfo, 
+		 const double **Z, const DATAINFO *pdinfo, 
 		 gretlopt opt, PRN *prn);
 
 Summary *summary (const int *list, const double **Z, 
@@ -194,7 +204,7 @@ double dh_b2_to_z2 (double b1, double b2, double n);
 
 double doornik_chisq (double skew, double xkurt, int n);
 
-int gretl_system_normality_test (const gretl_matrix *E, 
+int multivariate_normality_test (const gretl_matrix *E, 
 				 const gretl_matrix *Sigma, 
 				 PRN *prn);
 
