@@ -1,20 +1,20 @@
-/*
- *  Copyright (c) by Ramu Ramanathan and Allin Cottrell
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
- *
- *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
- *
+/* 
+ *  gretl -- Gnu Regression, Econometrics and Time-series Library
+ *  Copyright (C) 2001 Allin Cottrell and Riccardo "Jack" Lucchetti
+ * 
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ * 
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ * 
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 /* discrete.c for gretl: logit and probit models */
@@ -70,6 +70,9 @@ static void Lr_chisq (MODEL *pmod, double **Z)
     double Lr, chisq;
     
     for (t=pmod->t1; t<=pmod->t2; t++) {
+	if (model_missing(pmod, t)) {
+	    continue;
+	}
 	if (Z[pmod->list[1]][t] == 1.0) {
 	    ones++;
 	} 
@@ -129,7 +132,7 @@ static int add_slopes_to_model (MODEL *pmod, double fbx)
     }
 
     if (gretl_model_set_data(pmod, "slopes", slopes, 
-			     MODEL_DATA_DOUBLE_ARRAY,
+			     GRETL_TYPE_DOUBLE_ARRAY,
 			     ssize)) {
 	free(slopes);
 	return 1;
@@ -395,36 +398,166 @@ static int logit_probit_vcv (MODEL *dmod, gretlopt opt, const double **Z)
     return err;
 }
 
-/* BRMR, Davidson and MacKinnon, ETM, p. 461 */
+#if 0 /* not yet */
 
-static int do_BRMR (const int *list, int *dmodlist, 
-		    MODEL *dmod, int ci, double *beta,
-		    double ***pZ, DATAINFO *pdinfo,
-		    PRN *prn)
+static double 
+matrix_probit_llhood (const gretl_matrix *y,
+		      const gretl_matrix *yhat)
 {
-    double lldiff, llbak = -1.0e9;
-    double xx, fx, Fx;
+    double q, ll = 0.0;
+    int t;
+
+    for (t=0; t<y->rows; t++) {
+	q = 2.0 * y->val[t] - 1.0;
+	ll += log(normal_cdf(q * yhat->val[t]));
+    }
+
+    return ll;
+}
+
+int gretl_matrix_probit (const gretl_matrix *X,
+			 const gretl_matrix *y,
+			 gretl_matrix **pb)
+{
+    gretl_matrix *bX = NULL;
+    gretl_matrix *by = NULL;
+    gretl_matrix *bi = NULL;
+    gretl_matrix *b = NULL;
+    gretl_matrix *yhat = NULL;
+    double ll, lldiff, llbak = -1.0e9;
+    double wt, xti, yt, fx, Fx;
     double tol = 1.0e-9;
+    int nobs = X->rows;
+    int nc = X->cols;
     int itermax = 250;
-    int v = dmodlist[1];
-    int depvar = list[1];
     int i, t, iter;
+    int err = 0;
+
+    bX = gretl_matrix_alloc(nobs, nc);
+    by = gretl_column_vector_alloc(nobs);
+    bi = gretl_column_vector_alloc(nc);
+    yhat = gretl_column_vector_alloc(nobs);
+    b = gretl_column_vector_alloc(nc);
+
+    if (bX == NULL || by == NULL || bi == NULL || b == NULL) {
+	gretl_matrix_free(bX);
+	gretl_matrix_free(by);
+	gretl_matrix_free(bi);
+	gretl_matrix_free(b);
+	return E_ALLOC;
+    }
+
+    err = gretl_matrix_ols(y, X, b, NULL, NULL, NULL);
+
+    gretl_matrix_multiply(X, b, yhat);
 
     for (iter=0; iter<itermax; iter++) {
-	double wt;
-
 	/* construct BRMR dataset */
-	for (t=dmod->t1; t<=dmod->t2; t++) {
-	    xx = dmod->yhat[t];
-	    if (na(xx)) {
-		(*pZ)[v][t] = NADBL;
+	for (t=0; t<nobs; t++) {
+	    yt = yhat->val[t];
+	    fx = normal_pdf(yt);
+	    Fx = normal_cdf(yt);
+
+	    if (Fx < 1.0) {
+		wt = 1.0 / sqrt(Fx * (1.0 - Fx));
 	    } else {
+		wt = 0.0;
+	    }
+
+	    by->val[t] = wt * (y->val[t] - Fx);
+	    wt *= fx;
+	    for (i=0; i<nc; i++) {
+		xti = gretl_matrix_get(X, t, i);
+		gretl_matrix_set(bX, t, i, wt * xti);
+	    }
+	}
+
+	ll = matrix_probit_llhood(y, yhat);
+
+	lldiff = fabs(ll - llbak);
+	if (lldiff < tol) {
+	    break; 
+	}
+
+	llbak = ll;
+
+	err = gretl_matrix_ols(by, bX, bi, NULL, NULL, NULL);
+	if (err) {
+	    fprintf(stderr, "logit_probit: err = %d\n", err);
+	    if (iter > 0) {
+		err = E_NOCONV;
+	    }
+	    break;
+	}
+
+	/* update coefficient estimates */
+	gretl_matrix_add_to(b, bi);
+
+	/* calculate yhat */
+	gretl_matrix_multiply(X, b, yhat);
+    }
+
+    if (!err && lldiff > tol) {
+	err = E_NOCONV;
+    }
+
+    gretl_matrix_free(bX);
+    gretl_matrix_free(by);
+    gretl_matrix_free(bi);
+    gretl_matrix_free(yhat);
+
+    if (!err) {
+	*pb = b;
+    } else {
+	gretl_matrix_free(b);
+    }
+
+    return err;
+}
+
+#endif
+
+/* BRMR, Davidson and MacKinnon, ETM, p. 461 */
+
+static int do_BRMR (const int *list, MODEL *dmod, int ci,
+		    double *beta, const double **Z, 
+		    const DATAINFO *pdinfo, PRN *prn)
+{
+    gretl_matrix *X = NULL;
+    gretl_matrix *y = NULL;
+    gretl_matrix *b = NULL;
+    const double *yvar = Z[list[1]];
+    double lldiff, llbak = -1.0e9;
+    double wt, yt, fx, Fx;
+    double tol = 1.0e-9;
+    int itermax = 250;
+    int nc = dmod->ncoeff;
+    int i, s, t, iter;
+    int err = 0;
+
+    X = gretl_matrix_alloc(dmod->nobs, nc);
+    y = gretl_column_vector_alloc(dmod->nobs);
+    b = gretl_column_vector_alloc(nc);
+
+    if (X == NULL || y == NULL || b == NULL) {
+	gretl_matrix_free(X);
+	gretl_matrix_free(y);
+	gretl_matrix_free(b);
+	return E_ALLOC;
+    }
+
+    for (iter=0; iter<itermax; iter++) {
+	/* construct BRMR dataset */
+	s = 0;
+	for (t=dmod->t1; t<=dmod->t2; t++) {
+	    yt = dmod->yhat[t];
+	    if (!na(yt)) {
 		if (ci == LOGIT) {
-		    fx = logit_pdf(xx);
-		    Fx = logit(xx);
+		    fx = logit_pdf(yt);
+		    Fx = logit(yt);
 		} else {
-		    fx = normal_pdf(xx);
-		    Fx = normal_cdf(xx);
+		    fx = normal_pdf(yt);
+		    Fx = normal_cdf(yt);
 		}
 
 		if (Fx < 1.0) {
@@ -433,15 +566,22 @@ static int do_BRMR (const int *list, int *dmodlist,
 		    wt = 0.0;
 		}
 
-		(*pZ)[v][t] = wt * ((*pZ)[depvar][t] - Fx);
+		gretl_vector_set(y, s, wt * (yvar[t] - Fx));
 		wt *= fx;
-		for (i=2; i<=dmodlist[0]; i++) {
-		    (*pZ)[dmodlist[i]][t] = wt * (*pZ)[list[i]][t];
+		for (i=0; i<nc; i++) {
+		    gretl_matrix_set(X, s, i, wt * Z[list[i+2]][t]);
 		}
+		s++;
 	    }
 	}
 
-	dmod->lnL = logit_probit_llhood((*pZ)[depvar], dmod, ci);
+	if (s != dmod->nobs) {
+	    fprintf(stderr, "logit_probit: data error\n");
+	    err = E_DATA;
+	    break;
+	}
+
+	dmod->lnL = logit_probit_llhood(yvar, dmod, ci);
 
 	lldiff = fabs(dmod->lnL - llbak);
 	if (lldiff < tol) {
@@ -451,69 +591,62 @@ static int do_BRMR (const int *list, int *dmodlist,
 	if (na(dmod->lnL)) {
 	    pprintf(prn, _("Iteration %d: log likelihood = NA"), iter);	
 	} else {
-	    pprintf(prn, _("Iteration %d: log likelihood = %#.12g"), iter, dmod->lnL);
+	    pprintf(prn, _("Iteration %d: log likelihood = %#.12g"), 
+		    iter, dmod->lnL);
 	}
 	pputc(prn, '\n');
 
 	llbak = dmod->lnL;
-	clear_model(dmod);
-	*dmod = lsq(dmodlist, pZ, pdinfo, OLS, OPT_A);
-	if (dmod->errcode) {
-	    fprintf(stderr, "logit_probit: dmod errcode = %d\n", dmod->errcode);
+
+	err = gretl_matrix_ols(y, X, b, NULL, NULL, NULL);
+	if (err) {
+	    fprintf(stderr, "logit_probit: err = %d\n", err);
 	    if (iter > 0) {
-		dmod->errcode = E_NOCONV;
+		err = E_NOCONV;
 	    }
 	    break;
 	}
 
 	/* update coefficient estimates */
-	for (i=0; i<dmod->ncoeff; i++) {
-	    if (isnan(dmod->coeff[i])) {
-		fprintf(stderr, "BRMR produced NaN coeff\n");
-		dmod->errcode = E_NOCONV;
-		break;
-	    }
-	    beta[i] += dmod->coeff[i];
-	}
-
-	if (dmod->errcode) {
-	    break;
+	for (i=0; i<nc; i++) {
+	    beta[i] += b->val[i];
 	}
 
 	/* calculate yhat */
 	for (t=dmod->t1; t<=dmod->t2; t++) {
-	    if (na(dmod->yhat[t])) {
-		continue;
-	    }
-	    dmod->yhat[t] = 0.0;
-	    for (i=0; i<dmod->ncoeff; i++) {
-		dmod->yhat[t] += beta[i] * (*pZ)[list[i+2]][t];
+	    if (!na(dmod->yhat[t])) {
+		dmod->yhat[t] = 0.0;
+		for (i=0; i<nc; i++) {
+		    dmod->yhat[t] += beta[i] * Z[list[i+2]][t];
+		}
 	    }
 	}
     }
 
-    if (lldiff > tol) {
-	dmod->errcode = E_NOCONV;
+    if (!err && lldiff > tol) {
+	err = E_NOCONV;
     }
 
-    if (!dmod->errcode) {
+    if (!err) {
 	gretl_model_set_int(dmod, "iters", iter);
 	pputc(prn, '\n');
     }
 
-    return dmod->errcode;
+    gretl_matrix_free(X);
+    gretl_matrix_free(y);
+    gretl_matrix_free(b);
+
+    return err;
 }
 
 static MODEL 
 binary_logit_probit (const int *list, double ***pZ, DATAINFO *pdinfo, 
 		     int ci, gretlopt opt, PRN *prn)
 {
-    int i, t, v, depvar = list[1];
+    int i, t, depvar = list[1];
     int nx = list[0] - 1;
     int oldt1 = pdinfo->t1;
     int oldt2 = pdinfo->t2;
-    int oldv = pdinfo->v;
-    int *dmodlist = NULL;
     int *act_pred = NULL;
     MODEL dmod;
     double xx, fbx, f, F;
@@ -522,12 +655,6 @@ binary_logit_probit (const int *list, double ***pZ, DATAINFO *pdinfo,
 
     gretl_model_init(&dmod);
     
-    dmodlist = gretl_list_new(list[0]);
-    if (dmodlist == NULL) {
-	dmod.errcode = E_ALLOC;
-	return dmod;
-    } 
-
     /* allocate space for means of indep vars, coeffs */
     xbar = malloc(nx * sizeof *xbar);
     beta = malloc(nx * sizeof *beta);
@@ -544,46 +671,33 @@ binary_logit_probit (const int *list, double ***pZ, DATAINFO *pdinfo,
 	}
     }
 
-    /* make room for full set of transformed vars */
-    if (dataset_add_series(list[0], pZ, pdinfo)) {
-	dmod.errcode = E_ALLOC;
-	goto bailout;
-    }
-
-    v = oldv; /* the first newly created variable */
-
     varlist_adjust_sample(list, &pdinfo->t1, &pdinfo->t2, 
 			  (const double **) *pZ);
 
     dmod = lsq(list, pZ, pdinfo, OLS, OPT_A);
-    if (dmod.list[0] != list[0]) {
+    if (dmod.errcode == 0 && dmod.ncoeff != nx) {
 	dmod.errcode = E_DATA;
-	goto bailout;
     }
 
-    for (i=2; i<=list[0]; i++) {
-	dmodlist[i] = v + i - 1;
-	beta[i-2] = dmod.coeff[i-2];
-	xbar[i-2] = 0.0;
-	for (t=dmod.t1; t<=dmod.t2; t++) {
-	    if (!model_missing(&dmod, t)) {
-		xbar[i-2] += (*pZ)[list[i]][t];
-	    }
-	}
-	xbar[i-2] /= dmod.nobs;
-    }
-
-    dmodlist[0] = list[0];
-    dmodlist[1] = v; /* dep var is the newly created one */
-
-    do_BRMR(list, dmodlist, &dmod, ci, beta, pZ, pdinfo, prn);
     if (dmod.errcode) {
 	goto bailout;
     }
 
-    /* re-establish original list in model */
-    for (i=1; i<=list[0]; i++) {
-	dmod.list[i] = list[i];
+    for (i=0; i<dmod.ncoeff; i++) {
+	beta[i] = dmod.coeff[i];
+	xbar[i] = 0.0;
+	for (t=dmod.t1; t<=dmod.t2; t++) {
+	    if (!model_missing(&dmod, t)) {
+		xbar[i] += (*pZ)[list[i+2]][t];
+	    }
+	}
+	xbar[i] /= dmod.nobs;
+    }
+
+    dmod.errcode = do_BRMR(list, &dmod, ci, beta, (const double **) *pZ, 
+			   pdinfo, prn);
+    if (dmod.errcode) {
+	goto bailout;
     }
 
     /* transcribe coefficients */
@@ -616,7 +730,9 @@ binary_logit_probit (const int *list, double ***pZ, DATAINFO *pdinfo,
 	fbx = normal_pdf(xx);
     }
 
-    if (add_slopes_to_model(&dmod, fbx)) {
+    if (opt & OPT_P) {
+	gretl_model_set_int(&dmod, "show-pvals", 1);
+    } else if (add_slopes_to_model(&dmod, fbx)) {
 	dmod.errcode = E_ALLOC;
 	goto bailout;
     }
@@ -656,32 +772,34 @@ binary_logit_probit (const int *list, double ***pZ, DATAINFO *pdinfo,
 
     if (act_pred != NULL) {
 	gretl_model_set_data(&dmod, "discrete_act_pred", act_pred, 
-			     MODEL_DATA_INT_ARRAY, 
+			     GRETL_TYPE_INT_ARRAY, 
 			     4 * sizeof *act_pred);
     }
 
     mle_criteria(&dmod, 0);
 
-    dmod.ID = model_count_plus();
+    if (opt & OPT_A) {
+	dmod.aux = AUX_AUX;
+    } else {
+	dmod.ID = model_count_plus();
+    }
 
  bailout:
 
     free(xbar);
-    free(dmodlist);
     free(beta);
 
     pdinfo->t1 = oldt1;
     pdinfo->t2 = oldt2;
 
-    dataset_drop_last_variables(pdinfo->v - oldv, pZ, pdinfo);
-
     return dmod;
 }
 
 static int 
 ordered_model_ok (double **Z, const DATAINFO *pdinfo, int v)
 {
-    if (!var_is_discrete(pdinfo, v)) {
+    if (!var_is_discrete(pdinfo, v) && 
+	!gretl_is_oprobit_ok(pdinfo->t1, pdinfo->t2, Z[v])) {
 	sprintf(gretl_errmsg, "The variable '%s' is not discrete",
 		pdinfo->varname[v]);
 	return 0;
@@ -706,7 +824,9 @@ ordered_model_ok (double **Z, const DATAINFO *pdinfo, int v)
  * @ci: command index: if = %LOGIT, perform logit regression, otherwise
  * perform probit regression.
  * @opt: if includes %OPT_R form robust (QML) estimates of standard
- * errors and covariance matrix, in binary case.
+ * errors and covariance matrix, in binary case; if %OPT_P arrange for
+ * printing of p-values, not slopes at mean; if %OPT_A treat as an
+ * auxiliary regression.
  * @prn: printing struct in case additional information is
  * wanted (%OPT_V).
  *
@@ -734,8 +854,6 @@ MODEL logit_probit (const int *list, double ***pZ, DATAINFO *pdinfo,
 
 	gretl_model_init(&dmod);
 	dmod.errcode = E_UNSPEC;
-	sprintf(gretl_errmsg, _("The dependent variable '%s' is not a 0/1 "
-				"variable.\n"), pdinfo->varname[yv]);
 	return dmod;
     }
 }
@@ -1041,3 +1159,91 @@ MODEL logistic_model (const int *list, double ***pZ, DATAINFO *pdinfo,
     
     return lmod;
 }
+
+static double choose (double n, double k)
+{
+    double c = 1.0;
+    int i;
+
+    for (i=0; i<k; i++) {
+	c *= (n - i) / (k - i);
+    }
+
+    return c;
+}
+
+static double table_prob (double a, double b, double c, double d,
+			  double n)
+{
+    double p1 = choose(a+b, a);
+    double p2 = choose(c+d, c);
+    double p3 = choose(n, a+c);
+    double P = p1 * p2 / p3;
+
+#if 0
+    fprintf(stderr, "\ntable_prob: a=%g, b=%g, c=%g, d=%g, n=%g\n", 
+	    a, b, c, d, n);
+    fprintf(stderr, " p1=%g, p2=%g, p3=%g; P = %g\n", 
+	    p1, p2, p3, P);
+#endif
+
+    return P;
+}
+
+/* for use with cross-tabulation (xtab command) in the
+   2 x 2 case
+*/
+
+int fishers_exact_test (const Xtab *tab, PRN *prn)
+{
+    double a, b, c, d, n, E0;
+    double P0, Pi, PL, PR, P2;
+
+    a = tab->f[0][0];
+    b = tab->f[0][1];
+    c = tab->f[1][0];
+    d = tab->f[1][1];
+    n = tab->n;
+
+    E0 = (tab->rtotal[0] * tab->ctotal[0]) / n;
+
+    /* Probability of the observed table */
+    PL = PR = P2 = P0 = table_prob(a, b, c, d, n);
+
+    while (a > 0 && d > 0) {
+	a -= 1; d -= 1;
+	c += 1; b += 1;
+	Pi = table_prob(a, b, c, d, n);
+	if (Pi <= P0 || tab->f[0][0] > E0) {
+	    PL += Pi;
+	}
+	if (Pi <= P0) {
+	    P2 += Pi;
+	}
+    }
+
+    a = tab->f[0][0];
+    b = tab->f[0][1];
+    c = tab->f[1][0];
+    d = tab->f[1][1];
+
+    while (c > 0 && b > 0) {
+	c -= 1; b -= 1;
+	a += 1; d += 1;
+	Pi = table_prob(a, b, c, d, n);
+	if (Pi <= P0 || tab->f[0][0] < E0) {
+	    PR += Pi;
+	}
+	if (Pi <= P0) {
+	    P2 += Pi;
+	}
+    } 
+
+    pprintf(prn, "\n%s:\n", _("Fisher's Exact Test"));
+    pprintf(prn, "  Left:   P-value = %g\n", PL);
+    pprintf(prn, "  Right:  P-value = %g\n", PR);
+    pprintf(prn, "  2-Tail: P-value = %g\n", P2);
+    pputc(prn, '\n');
+
+    return 0;
+}
