@@ -1,20 +1,20 @@
-/*
- *  Copyright (c) by Ramu Ramanathan and Allin Cottrell
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
- *
- *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- *
+/* 
+ *  gretl -- Gnu Regression, Econometrics and Time-series Library
+ *  Copyright (C) 2001 Allin Cottrell and Riccardo "Jack" Lucchetti
+ * 
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ * 
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ * 
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 /* estimate.c - basic gretl estimation procedures */
@@ -498,7 +498,7 @@ lsq_check_for_missing_obs (MODEL *pmod, gretlopt opts,
 
     /* can't do HAC VCV with missing obs in middle */
     if ((opts & OPT_R) && dataset_is_time_series(pdinfo) &&
-	!get_force_hc()) {
+	!libset_get_bool(FORCE_HC)) {
 	reject_missing = 1;
     } 
 
@@ -568,16 +568,22 @@ log_depvar_ll (MODEL *pmod, const double **Z, const DATAINFO *pdinfo)
 
 static int check_weight_var (MODEL *pmod, const double *w, int *effobs)
 {
+    const char *wtzero = 
+	N_("Weight variable is all zeros, aborting regression");
+    const char *wtneg = 
+	N_("Weight variable contains negative values");
     int t;
 
     if (gretl_iszero(pmod->t1, pmod->t2, w)) {
-	pmod->errcode = E_WTZERO;
+	strcpy(gretl_errmsg, _(wtzero));
+	pmod->errcode = E_DATA;
 	return 1;
     }
 
     for (t=pmod->t1; t<=pmod->t2; t++) {
 	if (w[t] < 0.0) {
-	    pmod->errcode = E_WTNEG;
+	    strcpy(gretl_errmsg, _(wtneg));
+	    pmod->errcode = E_DATA;
 	    return 1;
 	}
     }
@@ -689,7 +695,7 @@ MODEL ar1_lsq (const int *list, double ***pZ, DATAINFO *pdinfo,
     int effobs = 0;
     int missv = 0, misst = 0;
     int jackknife = 0;
-    int use_qr = get_use_qr();
+    int use_qr = libset_get_bool(USE_QR);
     int pwe = (ci == PWE || (opt & OPT_P));
     int yno, i;
 
@@ -746,7 +752,7 @@ MODEL ar1_lsq (const int *list, double ***pZ, DATAINFO *pdinfo,
 
     /* sanity check */
     if (mdl.t1 < 0 || mdl.t2 > pdinfo->n - 1) {
-        mdl.errcode = E_NODATA;
+        mdl.errcode = E_DATA;
         goto lsq_abort;
     }
 
@@ -836,7 +842,7 @@ MODEL ar1_lsq (const int *list, double ***pZ, DATAINFO *pdinfo,
 	opt |= OPT_T;
     }
 
-    if (mdl.ci == HCCM || ((opt & OPT_R) && get_hc_version() == 4)) {
+    if (mdl.ci == HCCM || ((opt & OPT_R) && libset_get_int(HC_VERSION) == 4)) {
 	jackknife = 1;
     }
 
@@ -1841,6 +1847,7 @@ double estimate_rho (const int *list, double ***pZ, DATAINFO *pdinfo,
     int t1 = pdinfo->t1, t2 = pdinfo->t2;
     int missv = 0, misst = 0;
     gretlopt lsqopt = OPT_A;
+    int quiet = (opt & OPT_Q);
     int ascii = !(opt & OPT_P);
     MODEL corc_model;
 
@@ -1873,7 +1880,7 @@ double estimate_rho (const int *list, double ***pZ, DATAINFO *pdinfo,
 		goto bailout;
 	    }
 	    ess = corc_model.ess;
-	    if (ascii) {
+	    if (ascii && !quiet) {
 		char num[16];
 		int chk;
 		
@@ -1894,7 +1901,7 @@ double estimate_rho (const int *list, double ***pZ, DATAINFO *pdinfo,
 			bufspace(3, prn);
 		    }
 		} 
-	    } else {
+	    } else if (!quiet) {
 		ssr[nn] = ess;
 		rh[nn++] = rho;
 	    }
@@ -1939,12 +1946,14 @@ double estimate_rho (const int *list, double ***pZ, DATAINFO *pdinfo,
 	}
 
 	rho0 = rho = finalrho;
-	pprintf(prn, _("\n\nESS is minimum for rho = %g\n\n"), rho);
-	if (ascii) {
-	    graphyzx(NULL, ssr, NULL, rh, nn, "ESS", "RHO", NULL, 0, prn); 
-	    pputs(prn, "\n");
-	} else {
-	    hilu_plot(ssr, rh, nn);
+	if (!quiet) {
+	    pprintf(prn, _("\n\nESS is minimum for rho = %g\n\n"), rho);
+	    if (ascii) {
+		graphyx(ssr, rh, nn, "ESS", "RHO", prn); 
+		pputs(prn, "\n");
+	    } else {
+		hilu_plot(ssr, rh, nn);
+	    }
 	}
     } else { 
 	/* Go straight to Cochrane-Orcutt (or Prais-Winsten) */
@@ -1967,20 +1976,24 @@ double estimate_rho (const int *list, double ***pZ, DATAINFO *pdinfo,
 
     if (ci != HILU || !(opt & OPT_B)) {
 
-	if (ci == HILU) {
-	    pputs(prn, _("\nFine-tune rho using the CORC procedure...\n\n"));
-	} else {
-	    pputs(prn, _("\nPerforming iterative calculation of rho...\n\n"));
-	}
+	if (!quiet) {
+	    if (ci == HILU) {
+		pputs(prn, _("\nFine-tune rho using the CORC procedure...\n\n"));
+	    } else {
+		pputs(prn, _("\nPerforming iterative calculation of rho...\n\n"));
+	    }
 
-	pputs(prn, _("                 ITER       RHO        ESS"));
-	pputc(prn, '\n');
+	    pputs(prn, _("                 ITER       RHO        ESS"));
+	    pputc(prn, '\n');
+	}
 
 	iter = 0;
 	diff = 1.0;
 
 	while (diff > 0.001) {
-	    pprintf(prn, "          %10d %12.5f", ++iter, rho);
+	    if (!quiet) {
+		pprintf(prn, "          %10d %12.5f", ++iter, rho);
+	    }
 	    clear_model(&corc_model);
 	    corc_model = ar1_lsq(list, pZ, pdinfo, OLS, lsqopt, rho);
 #if AR_DEBUG
@@ -1993,7 +2006,9 @@ double estimate_rho (const int *list, double ***pZ, DATAINFO *pdinfo,
 		clear_model(&corc_model);
 		goto bailout;
 	    }
-	    pprintf(prn, "   %g\n", corc_model.ess);
+	    if (!quiet) {
+		pprintf(prn, "   %g\n", corc_model.ess);
+	    }
 
 	    rho = autores(&corc_model, (const double **) *pZ, ci);
 
@@ -2009,12 +2024,14 @@ double estimate_rho (const int *list, double ***pZ, DATAINFO *pdinfo,
 		goto bailout;
 	    }
 
-	    diff = (rho > rho0) ? rho - rho0 : rho0 - rho;
+	    diff = (rho > rho0)? rho - rho0 : rho0 - rho;
 	    rho0 = rho;
 	    if (iter == 30) break;
 	}
 
-	pprintf(prn, _("                final %11.5f\n\n"), rho);
+	if (!quiet) {
+	    pprintf(prn, _("                final %11.5f\n\n"), rho);
+	}
     }
 
     clear_model(&corc_model);
@@ -2125,9 +2142,8 @@ int *augment_regression_list (const int *orig, int aux,
 /* get_hsk_weights: take the residuals from the model pmod, square them
    and take logs; find the fitted values for this series using an
    auxiliary regression including the original independent variables
-   and their squares; transform the fitted values by exponentiating
-   and taking the square root; and add the resulting series to the
-   data set
+   and their squares; exponentiate the fitted values; and add the
+   resulting series to the data set.
 */
 
 static int get_hsk_weights (MODEL *pmod, double ***pZ, DATAINFO *pdinfo)
@@ -2146,10 +2162,10 @@ static int get_hsk_weights (MODEL *pmod, double ***pZ, DATAINFO *pdinfo)
 
     /* add transformed pmod residuals to data set */
     for (t=0; t<pdinfo->n; t++) {
-	if (na(pmod->uhat[t])) {
+	xx = pmod->uhat[t];
+	if (na(xx)) {
 	    (*pZ)[oldv][t] = NADBL;
 	} else {
-	    xx = pmod->uhat[t];
 	    (*pZ)[oldv][t] = log(xx * xx);
 	}
     }
@@ -2173,10 +2189,10 @@ static int get_hsk_weights (MODEL *pmod, double ***pZ, DATAINFO *pdinfo)
     } else {
 	/* write into the data set the required weights */
 	for (t=aux.t1; t<=aux.t2; t++) {
-	    if (na(aux.yhat[t])) {
+	    xx = aux.yhat[t];
+	    if (na(xx)) {
 		(*pZ)[oldv][t] = NADBL;
 	    } else {
-		xx = aux.yhat[t];
 		(*pZ)[oldv][t] = 1.0 / exp(xx);
 	    }
 	}
@@ -2426,13 +2442,186 @@ static int jackknife_vcv (MODEL *pmod, const double **Z)
     return err;
 }
 
+static void print_HET_1 (double z, double pval, PRN *prn)
+{
+    pprintf(prn, "\n%s\n", _("Pesaran-Taylor test for heteroskedasticity"));
+    pprintf(prn, "\n%s: HET_1 = %f,\n", _("Test statistic"), z);
+    pprintf(prn, "%s = 2 * P(z > %f) = %.3g\n\n", 
+	    _("with p-value"), z, pval);
+}
+
 static void print_whites_test (double TR2, int df, double pval, PRN *prn)
 {
     pprintf(prn, "\n%s\n", _("White's test for heteroskedasticity"));
     pprintf(prn, "\n%s: TR^2 = %f,\n", _("Test statistic"), TR2);
     pprintf(prn, "%s = P(%s(%d) > %f) = %f\n\n", 
 	    _("with p-value"), _("Chi-square"), 
-	    df, TR2, chisq_cdf_comp(TR2, df)); 
+	    df, TR2, pval); 
+}
+
+/* For White's test, see if we have enough degrees of freedom
+   to add squares -- and if so, if we have enough df to add
+   cross-products also.
+*/
+
+static int get_whites_aux (const MODEL *pmod, const double **Z) 
+{
+    int aux = AUX_WHITE;
+    int rem = pmod->ncoeff - pmod->ifc - 1;
+    int nsq = 0, nxpx = 0;
+    int i, v;
+
+    for (i=2; i<=pmod->list[0]; i++) {
+	v = pmod->list[i];
+	if (v > 0) {
+	    if (!gretl_isdummy(pmod->t1, pmod->t2, Z[v])) {
+		nsq++;
+	    }
+	    nxpx += rem--;
+	}
+    }
+
+    if (pmod->dfd - nsq < 1) {
+	aux = AUX_NONE;
+    } else if (pmod->dfd - nsq - nxpx < 1) {
+	aux = AUX_SQ;
+    }
+
+    return aux;
+}
+
+#define PT_DEBUG 0
+
+/**
+ * tsls_hetero_test:
+ * @pmod: pointer to model.
+ * @pZ: pointer to data matrix.
+ * @pdinfo: information on the data set.
+ * @opt: if flags include %OPT_S, save results to model; %OPT_Q
+ * means don't print the auxiliary regression.
+ * @prn: gretl printing struct.
+ *
+ * Runs Pesaran and Taylor's (1999) HET_1 test for heteroskedasticity
+ * on the given tsls model. The statistic is just a t-statistic, so
+ * under the null it is distributed as a standard normal.
+ * 
+ * Returns: 0 on successful completion, error code on error.
+ */
+
+static int tsls_hetero_test (MODEL *pmod, double ***pZ, 
+			     DATAINFO *pdinfo, gretlopt opt, 
+			     PRN *prn)
+{
+    int pos, v = pmod->list[1], newv = pdinfo->v;
+    int *auxlist = NULL, *testlist = NULL;
+    int i, h, t;
+    int savet1 = pdinfo->t1;
+    int savet2 = pdinfo->t2;
+    MODEL ptmod;
+    double x;
+    int err = 0;
+
+    pos = gretl_list_separator_position(pmod->list);
+    h = pmod->list[0] - pos;
+
+#if PT_DEBUG
+    pprintf(prn, "v = %d, h = %d\n", v, h);
+#endif
+
+    auxlist = gretl_list_new(h + 1);
+    testlist = gretl_list_new(3);
+
+    if (auxlist == NULL || testlist == NULL) {
+	free(auxlist);
+	free(testlist);
+	return E_ALLOC;
+    }
+
+    auxlist[1] = v;
+    for (i=2; i<=auxlist[0]; i++) {
+	auxlist[i] = pmod->list[i + pos - 1];
+    }	
+
+    testlist[1] = newv;
+    testlist[2] = 0;
+    testlist[3] = newv + 1;
+
+#if PT_DEBUG
+    printlist(auxlist, "auxlist");
+    printlist(testlist, "testlist");
+#endif
+
+    ptmod = lsq(auxlist, pZ, pdinfo, OLS, OPT_A);
+    err = ptmod.errcode;
+    if (err) {
+	goto bailout;
+    }
+
+#if PT_DEBUG
+    printmodel(&ptmod, pdinfo, OPT_S, prn);
+#endif
+
+    err = dataset_add_series(2, pZ, pdinfo);
+    if (err) {
+	clear_model(&ptmod);
+	goto bailout;
+    }
+
+    strcpy(pdinfo->varname[newv+1], "yhat^2");
+
+    for (t=pmod->t1; t<=pmod->t2; t++) {
+	x = pmod->uhat[t];
+	(*pZ)[newv][t] = x*x;
+	x = ptmod.yhat[t];
+	(*pZ)[newv+1][t] = x*x;
+    }
+
+    clear_model(&ptmod);
+
+    pdinfo->t1 = pmod->t1;
+    pdinfo->t2 = pmod->t2;
+
+    ptmod = lsq(testlist, pZ, pdinfo, OLS, OPT_A);
+    err = ptmod.errcode;
+
+    if (!err) {
+	double z = fabs(ptmod.coeff[1]) / ptmod.sderr[1];
+	double pval = 2.0 * (1 - normal_cdf(z));
+
+	if (opt & OPT_Q) {
+	    print_HET_1(z, pval, prn);
+	} else {
+	    ptmod.aux = AUX_HET_1;
+	    printmodel(&ptmod, pdinfo, OPT_NONE, prn);
+	}
+
+	if (opt & OPT_S) {
+	    ModelTest *test = model_test_new(GRETL_TEST_HET_1);
+
+	    if (test != NULL) {
+		model_test_set_teststat(test, GRETL_STAT_Z);
+		model_test_set_value(test, z);
+		model_test_set_pvalue(test, pval);
+		maybe_add_test_to_model(pmod, test);
+	    }	  
+	}
+
+	record_test_result(x, pval, _("HET_1"));
+    }
+
+    clear_model(&ptmod);
+
+    dataset_drop_last_variables(2, pZ, pdinfo); 
+
+ bailout:
+
+    free(auxlist);
+    free(testlist);
+
+    pdinfo->t1 = savet1;
+    pdinfo->t2 = savet2;
+
+    return err;
 }
 
 /**
@@ -2452,14 +2641,24 @@ static void print_whites_test (double TR2, int df, double pval, PRN *prn)
 int whites_test (MODEL *pmod, double ***pZ, DATAINFO *pdinfo, 
 		 gretlopt opt, PRN *prn)
 {
-    int lo, ncoeff, yno, t;
+    int aux, lo, ncoeff, yno, t;
     int v = pdinfo->v;
     int *list = NULL;
     double zz;
     MODEL white;
     int err = 0;
 
-    if (pmod->ci == NLS || pmod->ci == ARMA || pmod->ci == LOGISTIC) { 
+    if (pmod->ci == TSLS) {
+	return tsls_hetero_test(pmod, pZ, pdinfo, opt, prn);
+    }
+
+    if (pmod->list == NULL || gretl_list_has_separator(pmod->list)) {
+	return E_NOTIMP;
+    }
+
+    if (pmod->ci == NLS || pmod->ci == MLE ||
+	pmod->ci == GMM || pmod->ci == ARMA || 
+	pmod->ci == LOGISTIC) { 
 	return E_NOTIMP;
     }
 
@@ -2467,11 +2666,17 @@ int whites_test (MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
 	return err;
     }
 
+    /* what can we do, with the degrees of freedom available? */
+    aux = get_whites_aux(pmod, (const double **) *pZ);
+    if (aux == AUX_NONE) {
+	return E_DF;
+    } 
+
     gretl_model_init(&white);
 
     lo = pmod->list[0];
     yno = pmod->list[1];
-    ncoeff = pmod->list[0] - 1;
+    ncoeff = pmod->ncoeff;    
 
     /* make space in data set */
     if (dataset_add_series(1, pZ, pdinfo)) {
@@ -2493,8 +2698,9 @@ int whites_test (MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
 
     if (!err) {
 	/* build aux regression list, adding squares and
-	   cross-products of the original independent vars */
-	list = augment_regression_list(pmod->list, AUX_WHITE, pZ, pdinfo);
+	   (possibly) cross-products of the original 
+	   independent vars */
+	list = augment_regression_list(pmod->list, aux, pZ, pdinfo);
 	if (list == NULL) {
 	    err = E_ALLOC;
 	} else {
@@ -2509,19 +2715,15 @@ int whites_test (MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
     }
 
     if (!err) {
-	double TR2, pval;
-
-	white.aux = AUX_WHITE;
-
-	if (!(opt & OPT_Q)) {
-	    printmodel(&white, pdinfo, OPT_NONE, prn);
-	}
-
-	TR2 = white.rsq * white.nobs;
-	pval = chisq_cdf_comp(TR2, white.ncoeff - 1);
+	double TR2 = white.rsq * white.nobs;
+	int df = white.ncoeff - 1;
+	double pval = chisq_cdf_comp(TR2, df);
 
 	if (opt & OPT_Q) {
-	    print_whites_test(TR2, white.ncoeff - 1, pval, prn);
+	    print_whites_test(TR2, df, pval, prn);
+	} else {
+	    white.aux = AUX_WHITE;
+	    printmodel(&white, pdinfo, OPT_NONE, prn);
 	}
 
 	if (opt & OPT_S) {
@@ -2529,7 +2731,7 @@ int whites_test (MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
 
 	    if (test != NULL) {
 		model_test_set_teststat(test, GRETL_STAT_TR2);
-		model_test_set_dfn(test, white.ncoeff - 1);
+		model_test_set_dfn(test, df);
 		model_test_set_value(test, TR2);
 		model_test_set_pvalue(test, pval);
 		maybe_add_test_to_model(pmod, test);
@@ -2943,6 +3145,16 @@ lagdepvar (const int *list, const double **Z, const DATAINFO *pdinfo)
     return ret;
 }
 
+static void arch_test_print_simple (int order, double LM,
+				    double pval, PRN *prn)
+{
+    pprintf(prn, "\n%s %d\n", _("Test for ARCH of order"), order);
+    pprintf(prn, "\n%s: TR^2 = %f,\n", _("Test statistic"), LM);
+    pprintf(prn, "%s = P(%s(%d) > %f) = %f\n\n", 
+	    _("with p-value"), _("Chi-square"), 
+	    order, LM, pval); 
+}
+
 static void 
 arch_test_save_or_print (const gretl_matrix *b, const gretl_matrix *V,
 			 int T, int order, double rsq, MODEL *pmod, 
@@ -2972,7 +3184,9 @@ arch_test_save_or_print (const gretl_matrix *b, const gretl_matrix *V,
 	model_test_set_value(test, LM);
 	model_test_set_pvalue(test, pv);
 
-	if (!(opt & OPT_Q)) {
+	if (opt & OPT_Q) {
+	    arch_test_print_simple(order, LM, pv, prn);
+	} else {
 	    int heading = (V == NULL);
 
 	    gretl_model_test_print_direct(test, heading, prn);
@@ -3224,10 +3438,10 @@ MODEL arch_model (const int *list, int order, double ***pZ, DATAINFO *pdinfo,
 	if (!amod.errcode) {
 	    gretl_model_set_int(&amod, "arch_order", order);
 	    gretl_model_set_data(&amod, "arch_coeff", a,
-				 MODEL_DATA_DOUBLE_ARRAY,
+				 GRETL_TYPE_DOUBLE_ARRAY,
 				 (order + 1) * sizeof *a);
 	    gretl_model_set_data(&amod, "arch_sderr", se,
-				 MODEL_DATA_DOUBLE_ARRAY,
+				 GRETL_TYPE_DOUBLE_ARRAY,
 				 (order + 1) * sizeof *se);
 	}
     }
@@ -3290,6 +3504,7 @@ MODEL lad (const int *list, double ***pZ, DATAINFO *pdinfo)
  * arma:
  * @list: dependent variable, AR and MA orders, and any exogenous
  * regressors.
+ * @pqspec: string giving specific non-seasonal AR/MA lags (or %NULL).
  * @Z: data array.
  * @pdinfo: information on the data set.
  * @opt: options: may include %OPT_S to suppress intercept, %OPT_V
@@ -3303,12 +3518,14 @@ MODEL lad (const int *list, double ***pZ, DATAINFO *pdinfo)
  * Returns: a #MODEL struct, containing the estimates.
  */
 
-MODEL arma (const int *list, const double **Z, const DATAINFO *pdinfo, 
+MODEL arma (const int *list, const char *pqspec, 
+	    const double **Z, const DATAINFO *pdinfo, 
 	    gretlopt opt, PRN *prn)
 {
     MODEL armod;
     void *handle;
-    MODEL (*arma_func) (const int *, const double **, const DATAINFO *, 
+    MODEL (*arma_func) (const int *, const char *,
+			const double **, const DATAINFO *, 
 			gretlopt, PRN *);
 
     gretl_error_clear();
@@ -3333,7 +3550,7 @@ MODEL arma (const int *list, const double **Z, const DATAINFO *pdinfo,
 	return armod;
     }
 
-    armod = (*arma_func) (list, Z, pdinfo, opt, prn);
+    armod = (*arma_func) (list, pqspec, Z, pdinfo, opt, prn);
 
     close_plugin(handle);
     set_model_id(&armod);
@@ -3452,6 +3669,48 @@ MODEL poisson_model (const int *list, double ***pZ, DATAINFO *pdinfo, PRN *prn)
     return pmodel;
 }
 
+/**
+ * heckit_model:
+ * @list: dependent variable plus list of regressors.
+ * @pZ: pointer to data array.
+ * @pdinfo: information on the data set.
+ * @opt: option flags (may include %OPT_V for verbose output).
+ * @prn: printing struct for iteration info (or %NULL is this is not
+ * wanted).
+ *
+ * Produce Heckit estimates of the model given in @list. The list must
+ * include a separator to divide the main equation from the selection
+ * equation.
+ * 
+ * Returns: a #MODEL struct, containing the estimates.
+ */
+
+MODEL heckit_model (const int *list, double ***pZ, DATAINFO *pdinfo, 
+		    gretlopt opt, PRN *prn)
+{
+    MODEL hmod;
+    void *handle;
+    MODEL (* heckit_estimate) (const int *, double ***, DATAINFO *, 
+			       gretlopt, PRN *);
+
+    gretl_error_clear();
+
+    heckit_estimate = get_plugin_function("heckit_estimate", &handle);
+    if (heckit_estimate == NULL) {
+	gretl_model_init(&hmod);
+	hmod.errcode = E_FOPEN;
+	return hmod;
+    }
+
+    hmod = (*heckit_estimate) (list, pZ, pdinfo, opt, prn);
+
+    close_plugin(handle);
+
+    set_model_id(&hmod);
+
+    return hmod;
+}
+
 /**
  * garch:
  * @list: dependent variable plus arch and garch orders.
