@@ -1,20 +1,20 @@
-/* gretl - The Gnu Regression, Econometrics and Time-series Library
- * Copyright (C) 1999-2000 Ramu Ramanathan and Allin Cottrell
- *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License 
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
- *
- * This software is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License 
- * along with this software; if not, write to the 
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+/* 
+ *  gretl -- Gnu Regression, Econometrics and Time-series Library
+ *  Copyright (C) 2001 Allin Cottrell and Riccardo "Jack" Lucchetti
+ * 
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ * 
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ * 
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 /* estimate.h for gretl:  set out structs and functions employed */
@@ -33,7 +33,8 @@ double estimate_rho (const int *list, double ***pZ, DATAINFO *pdinfo,
 
 MODEL lad (const int *list, double ***pZ, DATAINFO *pdinfo); 
 
-MODEL arma (const int *list, const double **Z, const DATAINFO *pdinfo, 
+MODEL arma (const int *list, const char *pqspec,
+	    const double **Z, const DATAINFO *pdinfo, 
 	    gretlopt opt, PRN *prn);
 
 MODEL tobit_model (const int *list, double ***pZ, DATAINFO *pdinfo, 
@@ -42,6 +43,9 @@ MODEL tobit_model (const int *list, double ***pZ, DATAINFO *pdinfo,
 MODEL poisson_model (const int *list, double ***pZ, DATAINFO *pdinfo, 
 		     PRN *prn);
 
+MODEL heckit_model (const int *list, double ***pZ, DATAINFO *pdinfo, 
+		    gretlopt, PRN *prn);
+
 MODEL garch (const int *list, double ***pZ, DATAINFO *pdinfo, gretlopt opt,
 	     PRN *prn);
 
