@@ -1,20 +1,20 @@
-/*
- * Copyright (C) 1999-2005 Allin Cottrell
- *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License 
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
- *
- * This software is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License 
- * along with this software; if not, write to the 
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+/* 
+ *  gretl -- Gnu Regression, Econometrics and Time-series Library
+ *  Copyright (C) 2001 Allin Cottrell and Riccardo "Jack" Lucchetti
+ * 
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ * 
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ * 
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 #include "libgretl.h"
@@ -96,8 +96,17 @@ allocate_basic_fit_resid_arrays (FITRESID *fr)
 	return E_ALLOC;
     }
 
+    fr->resid = malloc(fr->nobs * sizeof *fr->resid);
+    if (fr->resid == NULL) {
+	free(fr->actual);
+	free(fr->fitted);
+	fr->actual = NULL;
+	fr->fitted = NULL;
+	return E_ALLOC;
+    }
+
     for (t=0; t<fr->nobs; t++) {
-	fr->actual[t] = fr->fitted[t] = NADBL;
+	fr->actual[t] = fr->fitted[t] = fr->resid[t] = NADBL;
     }
 
     fr->sderr = NULL;
@@ -161,6 +170,7 @@ static FITRESID *fit_resid_new (int n)
     } else {
 	fr->actual = NULL;
 	fr->fitted = NULL;
+	fr->resid = NULL;
 	fr->sderr = NULL;
     } 
     
@@ -179,6 +189,7 @@ void free_fit_resid (FITRESID *fr)
 {
     free(fr->actual);
     free(fr->fitted);
+    free(fr->resid);
     free(fr->sderr);
     free(fr);
 }
@@ -242,6 +253,7 @@ FITRESID *get_fit_resid (const MODEL *pmod, const double **Z,
     for (t=0; t<fr->nobs; t++) {
 	fr->actual[t] = Z[depvar][t];
 	fr->fitted[t] = pmod->yhat[t];
+	fr->resid[t] = pmod->uhat[t];
     }
 
     fit_resid_set_dec_places(fr);
@@ -1821,8 +1833,9 @@ static int real_get_fcast (FITRESID *fr, MODEL *pmod,
 	if (t < fr->t1) {
 	    if (t >= pmod->t1 && t <= pmod->t2) {
 		fr->fitted[t] = pmod->yhat[t];
+		fr->resid[t] = pmod->uhat[t];
 	    } else {
-		fr->fitted[t] = NADBL;
+		fr->fitted[t] = fr->resid[t] = NADBL;
 	    }
 	    if (fr->sderr != NULL) {
 		fr->sderr[t] = NADBL;
@@ -2120,30 +2133,9 @@ int add_forecast (const char *str, MODEL *pmod, double ***pZ,
     return err;
 }
 
-/**
- * display_forecast:
- * @str: string giving starting and ending observations, separated
- * by a space.
- * @pmod: the model from which forecasts are wanted.
- * @pZ: pointer to data array using which @pmod was estimated.
- * @pdinfo: dataset information.
- * @opt: if OPT_D, force a dynamic forecast; if OPT_S, force
- * a static forecast.  By default, the forecast is static within
- * the data range over which the model was estimated, and dynamic
- * out of sample (in cases where this distinction is meaningful).
- * @prn: printing structure.
- *
- * Computes forecasts based on @pmod, over the range of observations
- * given in @str.  Forecast standard errors are also computed
- * if possible.  The results are printed to @prn, and are also
- * plotted if %OPT_P is given.
- *
- * Returns: 0 on success, non-zero error code on error.
- */
-
-int display_forecast (const char *str, MODEL *pmod, 
-		      double ***pZ, DATAINFO *pdinfo, 
-		      gretlopt opt, PRN *prn)
+static int model_display_forecast (const char *str, MODEL *pmod, 
+				   double ***pZ, DATAINFO *pdinfo, 
+				   gretlopt opt, PRN *prn)
 {
     FITRESID *fr;
     int t1, t2;
@@ -2175,6 +2167,94 @@ int display_forecast (const char *str, MODEL *pmod,
     return err;
 }
 
+static int VAR_display_forecast (const char *str, GRETL_VAR *var, 
+				 double ***pZ, DATAINFO *pdinfo, 
+				 gretlopt opt, PRN *prn)
+{
+    FITRESID *fr;
+    const MODEL *pmod;
+    int t1, t2;
+    int i, n, err;
+
+    pmod = gretl_VAR_get_model(var, 0);
+
+    err = parse_forecast_string(str, pmod, pdinfo, &t1, &t2);
+    if (err) {
+	return err;
+    }
+
+    n = gretl_VAR_get_n_equations(var);
+
+    for (i=0; i<n && !err; i++) {
+
+	if (i > 0) {
+	    opt |= OPT_Q;
+	}
+
+	fr = get_VAR_forecast(var, i, t1, t1, t2, (const double **) *pZ, 
+			      pdinfo, opt);
+
+	if (fr == NULL) {
+	    return E_ALLOC;
+	}
+
+	err = fr->err;
+
+	if (!err) {
+	    err = text_print_forecast(fr, pZ, pdinfo, opt, prn);
+	}
+
+	free_fit_resid(fr);
+    }
+
+    return err;
+}
+
+/**
+ * display_forecast:
+ * @str: string giving starting and ending observations, separated
+ * by a space.
+ * @pZ: pointer to data array.
+ * @pdinfo: dataset information.
+ * @opt: if OPT_D, force a dynamic forecast; if OPT_S, force
+ * a static forecast.  By default, the forecast is static within
+ * the data range over which the model was estimated, and dynamic
+ * out of sample (in cases where this distinction is meaningful).
+ * @prn: printing structure.
+ *
+ * Computes forecasts based on the last model estimated, over the 
+ * range of observations given in @str.  Forecast standard errors 
+ * are also computed if possible.  The results are printed to @prn, 
+ * and are also plotted if %OPT_P is given.
+ *
+ * Returns: 0 on success, non-zero error code on error.
+ */
+
+int display_forecast (const char *str, double ***pZ, DATAINFO *pdinfo, 
+		      gretlopt opt, PRN *prn)
+{
+    GretlObjType type;
+    void *ptr;
+    int err;
+
+    ptr = get_last_model(&type);  
+    if (ptr == NULL) {
+	return E_DATA;
+    }
+
+    if (type == GRETL_OBJ_EQN) {
+	err = model_display_forecast(str, ptr, pZ, pdinfo, opt, prn);
+    } else if (type == GRETL_OBJ_SYS) {
+	err = E_NOTIMP;
+    } else if (type == GRETL_OBJ_VAR) {
+	err = VAR_display_forecast(str, ptr, pZ, pdinfo, opt, prn);
+    } else {
+	err = E_DATA;
+    }
+
+    return err;
+}
+
 /* try to determine in advance how far we can go with a forecast,
    either dynamic or static (ftype) */
 
@@ -2267,11 +2347,9 @@ FITRESID *get_VAR_forecast (GRETL_VAR *var, int i, int t0, int t1, int t2,
 	return NULL;
     }
 
-    if (!var->ecm) {
-	pmod = gretl_VAR_get_model(var, i);
-	if (pmod == NULL) {
-	    return NULL;
-	}
+    pmod = gretl_VAR_get_model(var, i);
+    if (pmod == NULL) {
+	return NULL;
     }
 
     F = gretl_VAR_get_forecast_matrix(var, t0, t1, t2, Z, pdinfo, opt);
@@ -2298,13 +2376,9 @@ FITRESID *get_VAR_forecast (GRETL_VAR *var, int i, int t0, int t1, int t2,
     fr->t1 = t1;
     fr->t2 = t2;
 
-    if (var->ecm) {
-	yno = var->jinfo->list[i+1];
-    } else {
-	yno = pmod->list[1];
-    }
+    yno = pmod->list[1];
 
-    strcpy(fr->depvar, pdinfo->varname[yno]);
+    strcpy(fr->depvar, gretl_model_get_depvar_name(pmod, pdinfo));
 
     m = var->neqns;
 
@@ -2324,9 +2398,9 @@ FITRESID *get_VAR_forecast (GRETL_VAR *var, int i, int t0, int t1, int t2,
     if (nf == 0) {
 	fr->err = E_MISSDATA;
     } else {
-	if (var->ecm) {
-	    fr->df = var->T;
+	if (var->ci == VECM) {
 	    /* asymptotic normal */
+	    fr->df = var->T;
 	    fr->tval = 1.96;
 	} else {
 	    fr->df = pmod->dfd;
