@@ -1,20 +1,20 @@
-/*
- * Copyright (C) 1999-2005 Allin Cottrell
- *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License 
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
- *
- * This software is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License 
- * along with this software; if not, write to the 
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+/* 
+ *  gretl -- Gnu Regression, Econometrics and Time-series Library
+ *  Copyright (C) 2001 Allin Cottrell and Riccardo "Jack" Lucchetti
+ * 
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ * 
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ * 
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 #ifndef FORECAST_H
@@ -34,6 +34,7 @@ struct FITRESID_ {
     int method;
     double *actual;
     double *fitted;
+    double *resid;
     double *sderr;
     double sigma;
     double tval;
@@ -58,8 +59,7 @@ FITRESID *get_VAR_forecast (GRETL_VAR *var, int i, int t0, int t1, int t2,
 			    const double **Z, DATAINFO *pdinfo,
 			    gretlopt opt);
 
-int display_forecast (const char *str, MODEL *pmod, 
-		      double ***pZ, DATAINFO *pdinfo, 
+int display_forecast (const char *str, double ***pZ, DATAINFO *pdinfo, 
 		      gretlopt opt, PRN *prn);
 
 int add_forecast (const char *str, MODEL *pmod, 
