@@ -1,20 +1,20 @@
-/*
- *  Copyright (c) by Allin Cottrell and Riccardo Lucchetti
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
- *
- *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
- *
+/* 
+ *  gretl -- Gnu Regression, Econometrics and Time-series Library
+ *  Copyright (C) 2001 Allin Cottrell and Riccardo "Jack" Lucchetti
+ * 
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ * 
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ * 
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 /* various functions called by 'genr' */
@@ -127,6 +127,142 @@ int sort_series (const double *x, double *y, int f,
     return 0;
 }
 
+struct pair_sorter {
+    double x;
+    double y;
+};
+
+/* sort the series y by the values of x, putting the result
+   into z */
+
+int gretl_sort_by (const double *x, const double *y, 
+		   double *z, const DATAINFO *pdinfo)
+{
+    struct pair_sorter *xy;
+    int n = pdinfo->t2 - pdinfo->t1 + 1;
+    int i, t;
+
+    for (t=pdinfo->t1; t<=pdinfo->t2; t++) {
+	if (na(x[t])) {
+	    return E_MISSDATA;
+	}
+    }
+
+    xy = malloc(n * sizeof *xy);
+    if (xy == NULL) {
+	return E_ALLOC;
+    }
+
+    i = 0;
+    for (t=pdinfo->t1; t<=pdinfo->t2; t++) {
+	xy[i].x = x[t];
+	xy[i].y = y[t];
+	i++;
+    }
+
+    qsort(xy, n, sizeof *xy, gretl_compare_doubles);
+
+    i = 0;
+    for (t=pdinfo->t1; t<=pdinfo->t2; t++) {
+	z[t] = xy[i++].y;
+    }
+
+    free(xy);
+
+    return 0;
+}
+
+static void genrank (const double *sz, int m,
+		     const double *z, int n,
+		     double *rz)
+{
+    int cases, k, i, j;
+    double avg, r = 1;
+
+    for (i=0; i<m; i++) {
+	/* scan sorted z */
+	cases = k = 0;
+
+	if (i > 0 && sz[i] == sz[i-1]) {
+	    continue;
+	}
+
+	for (j=0; j<n; j++) {
+	    /* scan raw z for matches */
+	    if (!na(z[j])) {
+		if (z[j] == sz[i]) {
+		    rz[k] = r;
+		    cases++;
+		}
+		k++;
+	    }
+	}
+
+	if (cases > 1) {
+	    avg = (r + r + cases - 1.0) / 2.0;
+	    for (j=0; j<m; j++) {
+		if (rz[j] == r) {
+		    rz[j] = avg;
+		}
+	    }
+	} 
+
+	r += cases;
+    }
+}
+
+int rank_series (const double *x, double *y, int f, 
+		 const DATAINFO *pdinfo)
+{
+    double *sx = NULL;
+    double *rx = NULL;
+    int n = pdinfo->t2 - pdinfo->t1 + 1;
+    int m = n;
+    int i, t;
+
+    for (t=pdinfo->t1; t<=pdinfo->t2; t++) {
+	if (na(x[t])) m--;
+    }
+
+    sx = malloc(m * sizeof *sx);
+    rx = malloc(m * sizeof *rx);
+
+    if (sx == NULL || rx == NULL) {
+	free(sx);
+	free(rx);
+	return E_ALLOC;
+    }
+
+    i = 0;
+    for (t=pdinfo->t1; t<=pdinfo->t2; t++) {
+	if (!na(x[t])) {
+	    sx[i] = x[t];
+	    rx[i] = 0.0;
+	    i++;
+	}
+    }
+    
+    if (f == DSORT) {
+	qsort(sx, m, sizeof *sx, gretl_inverse_compare_doubles);
+    } else {
+	qsort(sx, m, sizeof *sx, gretl_compare_doubles);
+    }
+
+    genrank(sx, m, x + pdinfo->t1, n, rx);
+
+    i = 0;
+    for (t=pdinfo->t1; t<=pdinfo->t2; t++) {
+	if (!na(x[t])) {
+	    y[t] = rx[i++];
+	}
+    }    
+
+    free(sx);
+    free(rx);
+
+    return 0;
+}
+
 /**
  * diff_series:
  * @x: array of original data.
@@ -185,7 +321,7 @@ int diff_series (const double *x, double *y, int f,
  * @pdinfo: data set information.
  *
  * Calculates in @y the forward orthogonal deviations of the input 
- * series @x.  That is, y[t+1] is the scaled difference between x[t]
+ * series @x.  That is, y[t] is the scaled difference between x[t]
  * and the mean of the subsequent observations on x.
  *
  * Returns: 0 on success, non-zero error code on failure.
@@ -219,7 +355,7 @@ int orthdev_series (const double *x, double *y, const DATAINFO *pdinfo)
 
 	if (n > 0) {
 	    xbar /= n;
-	    y[t+1] = sqrt(n / (n + 1.0)) * (x[t] - xbar);
+	    y[t] = sqrt(n / (n + 1.0)) * (x[t] - xbar);
 	}
     }
 
@@ -320,7 +456,7 @@ int resample_series (const double *x, double *y, const DATAINFO *pdinfo)
     }
 
     /* generate uniform random series */
-    gretl_uniform_dist(z, 0, n - 1);
+    gretl_rand_uniform(z, 0, n - 1);
 
     /* sample from source series based on indices */
     for (t=t1; t<=t2; t++) {
@@ -448,7 +584,7 @@ int maybe_pick_up_sorted_markers (parser *p)
 
 static double hp_lambda (const DATAINFO *pdinfo)
 {
-    double la = get_hp_lambda();
+    double la = libset_get_double(HP_LAMBDA);
 
     if (na(la)) {
 	la = 100 * pdinfo->pd * pdinfo->pd;
@@ -810,7 +946,7 @@ int dummy (double ***pZ, DATAINFO *pdinfo, int center)
 {
     char vname[VNAMELEN];
     char vlabel[MAXLABEL];
-    int vi, t, yy, pp, mm;
+    int vi, t, pp;
     int ndums = pdinfo->pd, nnew = 0;
     int di, di0 = pdinfo->v;
     double xx, dx;
@@ -847,26 +983,38 @@ int dummy (double ***pZ, DATAINFO *pdinfo, int center)
 	return 0;
     }
 
-    pp = pdinfo->pd;
-    mm = 10;
-    while ((pp = pp / 10)) {
-	mm *= 10;
-    }
-
     for (vi=1, di = di0; vi<=ndums; vi++, di++) {
 	make_dummy_name_and_label(vi, pdinfo, center, vname, vlabel);
 	strcpy(pdinfo->varname[di], vname);
 	strcpy(VARLABEL(pdinfo, di), vlabel);
+    }
+
+    if (dataset_is_daily(pdinfo)) {
+	int yy, mm = 10;
+
+	pp = pdinfo->pd;
+	while ((pp = pp / 10)) {
+	    mm *= 10;
+	}
+
+	for (vi=1, di = di0; vi<=ndums; vi++, di++) {
+	    for (t=0; t<pdinfo->n; t++) {
+		xx = date(t, pdinfo->pd, pdinfo->sd0) + .1;
+		yy = (int) xx;
+		pp = (int) (mm * (xx - yy) + 0.5);
+		dx = (pp == vi)? 1.0 : 0.0;
+		(*pZ)[di][t] = dx;
+	    }
+	}
+    } else {
+	int p0 = get_subperiod(0, pdinfo, NULL);
 
 	for (t=0; t<pdinfo->n; t++) {
-	    xx = date(t, pdinfo->pd, pdinfo->sd0);
-	    if (dataset_is_daily(pdinfo)) { /* FIXME weekly? */
-		xx += .1;
+	    pp = (t + p0) % pdinfo->pd;
+	    for (vi=0, di = di0; vi<ndums; vi++, di++) {
+		dx = (pp == vi)? 1 : 0;
+		(*pZ)[di][t] = dx;
 	    }
-	    yy = (int) xx;
-	    pp = (int) (mm * (xx - yy) + 0.5);
-	    dx = (pp == vi)? 1.0 : 0.0;
-	    (*pZ)[di][t] = dx;
 	}
     }
 
@@ -1189,8 +1337,7 @@ int plotvar_code (const DATAINFO *pdinfo)
 	return PLOTVAR_MONTHS;
     } else if (pdinfo->pd == 24) {
 	return PLOTVAR_HOURLY;
-    } else if ((dated_daily_data(pdinfo) && pdinfo->n > 365) ||
-	    (dated_weekly_data(pdinfo) && pdinfo->n > 52)) {
+    } else if (calendar_data(pdinfo)) {
 	return PLOTVAR_CALENDAR;
     } else if (dataset_is_decennial(pdinfo)) {
 	return PLOTVAR_DECADES;
@@ -1536,139 +1683,6 @@ int get_t_from_obs_string (const char *s, const double **Z,
     return t;
 }
 
-void fn_args_init (fnargs *args)
-{
-    args->types = NULL;
-    args->nx = 0;
-    args->nX = 0;
-    args->nM = 0;
-    args->nl = 0;
-    args->nrefv = 0;
-    args->nrefm = 0;
-    args->nnull = 0;
-    args->nnames = 0;
-    args->x = NULL;
-    args->X = NULL;
-    args->M = NULL;
-    args->lists = NULL;
-    args->refv = NULL;
-    args->refm = NULL;
-    args->upnames = NULL;
-}
-
-/* note: this is not a "deep free"; that should be
-   handled in geneval.c */
-
-void fn_args_free (fnargs *args)
-{
-    free(args->types);
-    free(args->x);
-    free(args->X);
-    free(args->M);
-    free(args->lists);
-    free(args->refv);
-    free(args->refm);
-    free_strings_array(args->upnames, args->nnames);
-}
-
-int push_fn_arg (fnargs *args, int type, void *p)
-{
-    char *types;
-    int n, err = 0;
-
-    n = args->nx + args->nX + args->nM + args->nl + 
-	args->nrefv + args->nrefm + args->nnull + 1;
-
-    types = realloc(args->types, n * sizeof *types);
-    if (types == NULL) {
-	return E_ALLOC;
-    }
-
-    types[n-1] = type;
-    args->types = types;
-
-    if (type == ARG_NONE) {
-	args->nnull += 1;
-    } else if (type == ARG_SCALAR) {
-	double *x;
-
-	n = args->nx + 1;
-	x = realloc(args->x, n * sizeof *x);
-	if (x == NULL) {
-	    err = E_ALLOC;
-	} else {
-	    x[n-1] = *(double *) p;
-	    args->x = x;
-	    args->nx = n;
-	}
-    } else if (type == ARG_SERIES) {
-	double **X;
-
-	n = args->nX + 1;
-	X = realloc(args->X, n * sizeof *X);
-	if (X == NULL) {
-	    err = E_ALLOC;
-	} else {
-	    X[n-1] = (double *) p;
-	    args->X = X;
-	    args->nX = n;
-	}
-    } else if (type == ARG_MATRIX) {
-	gretl_matrix **M;
-
-	n = args->nM + 1;
-	M = realloc(args->M, n * sizeof *M);
-	if (M == NULL) {
-	    err = E_ALLOC;
-	} else {
-	    M[n-1] = (gretl_matrix *) p;
-	    args->M = M;
-	    args->nM = n;
-	}
-    } else if (type == ARG_LIST) {
-	char **lists;
-
-	n = args->nl + 1;
-	lists = realloc(args->lists, n * sizeof *lists);
-	if (lists == NULL) {
-	    err = E_ALLOC;
-	} else {
-	    lists[n-1] = (char *) p;
-	    args->lists = lists;
-	    args->nl = n;
-	}
-    } else if (type == ARG_REF_SCALAR ||
-	       type == ARG_REF_SERIES) {
-	int *refv;
-
-	n = args->nrefv + 1;
-	refv = realloc(args->refv, n * sizeof *refv);
-	if (refv == NULL) {
-	    err = E_ALLOC;
-	} else {
-	    refv[n-1] = * (int *) p;
-	    args->refv = refv;
-	    args->nrefv = n;
-	}
-    } else if (type == ARG_REF_MATRIX) {
-	user_matrix **M;
-
-	n = args->nrefm + 1;
-	M = realloc(args->refm, n * sizeof *M);
-	if (M == NULL) {
-	    err = E_ALLOC;
-	} else {
-	    M[n-1] = (user_matrix *) p;
-	    args->refm = M;
-	    args->nrefm = n;
-	}
-    } else {
-	err = E_TYPES;
-    }
-
-    return err;
-}
-
 int check_declarations (char ***pS, parser *p)
 {
     char **S;
