@@ -1,20 +1,20 @@
-/*
- *   Copyright (c) by Allin Cottrell
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
- *
- *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
- *
+/* 
+ *  gretl -- Gnu Regression, Econometrics and Time-series Library
+ *  Copyright (C) 2001 Allin Cottrell and Riccardo "Jack" Lucchetti
+ * 
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ * 
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ * 
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 /* lexer module for 'genr' and related commands */
@@ -23,6 +23,7 @@
 #include "usermat.h"
 #include "loop_private.h"
 #include "gretl_func.h"
+#include "gretl_string_table.h"
 
 #define NUMLEN 32
 #define MAXQUOTE 64
@@ -70,6 +71,7 @@ struct str_table dvars[] = {
     { R_PD,        "$pd" },
     { R_TEST_STAT, "$test" },
     { R_TEST_PVAL, "$pvalue" },
+    { R_TEST_LNL,  "$rlnl" },
     { R_INDEX,     "t" },
     { R_INDEX,     "obs" },
     { R_T1,        "$t1" },
@@ -86,6 +88,7 @@ struct str_table mvars[] = {
     { M_DF,      "$df" },
     { M_NCOEFF,  "$ncoeff" },
     { M_LNL,     "$lnl" },
+    { M_GMMCRIT, "$gmmcrit" },
     { M_AIC,     "$aic" },
     { M_BIC,     "$bic" },
     { M_HQC,     "$hqc" },
@@ -98,6 +101,7 @@ struct str_table mvars[] = {
     { M_SE,      "$stderr" },
     { M_VCV,     "$vcv" },
     { M_RHO,     "$rho" },
+    { M_COMPAN,  "$compan" },
     { M_JALPHA,  "$jalpha" }, 
     { M_JBETA,   "$jbeta" },
     { M_JVBETA,  "$jvbeta" },
@@ -123,18 +127,19 @@ struct str_table funcs[] = {
     { LDIF,     "ldiff" },
     { SDIF,     "sdiff" },
     { TOINT,    "int" },
+    { CEIL,     "ceil" },
+    { FLOOR,    "floor" },
     { SORT,     "sort" }, 
     { DSORT,    "dsort" }, 
+    { SORTBY,   "sortby" }, 
+    { RANKING,  "ranking" },
     { ODEV,     "orthdev" },
     { NOBS,     "nobs" },
     { T1,       "firstobs" },
     { T2,       "lastobs" },
-    { UNIFORM,  "uniform" }, 
-    { NORMAL,   "normal" }, 
-    { CHISQ,    "chisq" }, 
-    { STUDENT,  "student" },
-    { BINOMIAL, "binomial" },
-    { GENPOIS,  "poisson" },
+    { RUNIFORM, "uniform" }, 
+    { RNORMAL,  "normal" }, 
+    { RPOISSON, "poisson" },
     { CUM,      "cum" }, 
     { MISSING,  "missing" },
     { OK,       "ok" },        /* opposite of missing */
@@ -154,7 +159,7 @@ struct str_table funcs[] = {
     { CNORM,    "cnorm" },
     { DNORM,    "dnorm" },
     { QNORM,    "qnorm" },
-    { GAMMA,    "gamma" },
+    { GAMMA,    "gammafunc" },
     { LNGAMMA,  "lngamma" },
     { RESAMPLE, "resample" },
     { PMEAN,    "pmean" },     /* panel mean */
@@ -164,6 +169,7 @@ struct str_table funcs[] = {
     { FRACDIF,  "fracdiff" },  /* fractional difference */
     { COV,      "cov" },
     { COR,      "corr" },
+    { MOVAVG,   "movavg" },
     { IMAT,     "I" },
     { ZEROS,    "zeros" },
     { ONES,     "ones" },
@@ -185,16 +191,21 @@ struct str_table funcs[] = {
     { FFT,      "fft" },
     { FFTI,     "ffti" },
     { CMULT,    "cmult" },
+    { CDIV,     "cdiv" },
     { MCOV,     "mcov" },
     { MCORR,    "mcorr" },
+    { MXTAB,    "mxtab" },
     { CDEMEAN,  "cdemean" },
     { CHOL,     "cholesky" },
     { INV,      "inv" },
+    { GINV,     "ginv" },
     { DIAG,     "diag" },
     { TRANSP,   "transp" },
     { TVEC,     "vec" },
     { VECH,     "vech" },
     { UNVECH,   "unvech" },
+    { UPPER,    "upper" },
+    { LOWER,    "lower" },
     { ROWS,     "rows" },
     { COLS,     "cols" },
     { DET,      "det" },
@@ -214,7 +225,6 @@ struct str_table funcs[] = {
     { MEXP,     "mexp" },
     { FDJAC,    "fdjac" },
     { BFGSMAX,  "BFGSmax" },
-    { VARNUM,   "varnum" },
     { OBSNUM,   "obsnum" },
     { ISSERIES, "isseries" },
     { ISLIST,   "islist" },
@@ -222,13 +232,23 @@ struct str_table funcs[] = {
     { ISNULL,   "isnull" },
     { LISTLEN,  "nelem" },
     { CDF,      "cdf" },
+    { INVCDF,   "invcdf" },
     { PVAL,     "pvalue" },
     { CRIT,     "critical" },
+    { RANDGEN,  "randgen" },
     { MAKEMASK, "makemask" },
     { VALUES,   "values" },
     { MSHAPE,   "mshape" },
     { SVD,      "svd" },
-    { 0,     NULL }
+    { MOLS,     "mols" },
+    { MREAD,    "mread" },
+    { MWRITE,   "mwrite" },
+    { 0,        NULL }
+};
+
+struct str_table func_alias[] = {
+    { GAMMA,    "gamma" },
+    { 0,        NULL }
 };
 
 int const_lookup (const char *s)
@@ -257,7 +277,12 @@ const char *constname (int c)
     return "unknown";
 }
 
-int function_lookup (const char *s)
+enum {
+    NO_ALIAS,
+    ALLOW_ALIAS
+};
+
+static int real_function_lookup (const char *s, int a)
 {
     int i;
 
@@ -267,9 +292,27 @@ int function_lookup (const char *s)
 	}
     }
 
+    if (a == ALLOW_ALIAS) {
+	for (i=0; func_alias[i].id != 0; i++) {
+	    if (!strcmp(s, func_alias[i].str)) {
+		return func_alias[i].id;
+	    }
+	} 
+    }   
+
     return 0;
 }
 
+int function_lookup (const char *s)
+{
+    return real_function_lookup(s, NO_ALIAS);
+}
+
+static int function_lookup_with_alias (const char *s)
+{
+    return real_function_lookup(s, ALLOW_ALIAS);
+}
+
 static const char *funname (int t)
 {
     int i;
@@ -432,8 +475,7 @@ void context_error (int c, parser *p)
 {
     parser_print_input(p);
     if (c != 0) {
-	pprintf(p->prn, _("The symbol '%c' is not valid in this context\n"), 
-		p->ch);
+	pprintf(p->prn, _("The symbol '%c' is not valid in this context\n"), c);
     } else {
 	pprintf(p->prn, _("The symbol '%s' is not valid in this context\n"), 
 		getsymb(p->sym, p));
@@ -569,7 +611,7 @@ static void look_up_word (const char *s, parser *p)
 {
     int fsym, err = 0;
 
-    fsym = p->sym = function_lookup(s);
+    fsym = p->sym = function_lookup_with_alias(s);
 
     if (p->sym == 0 || p->ch != '(') {
 	p->idnum = const_lookup(s);
@@ -592,9 +634,12 @@ static void look_up_word (const char *s, parser *p)
 		} else if (get_list_by_name(s)) {
 		    p->sym = LIST;
 		    p->idstr = gretl_strdup(s);
-		} else if (gretl_is_user_function(s)) {
+		} else if (get_user_function_by_name(s)) {
 		    p->sym = UFUN;
 		    p->idstr = gretl_strdup(s);
+		} else if (string_is_defined(s)) {
+		    p->sym = STR;
+		    p->idstr = gretl_strdup(get_named_string(s));
 		} else {
 		    err = 1;
 		}
@@ -616,7 +661,8 @@ static void look_up_word (const char *s, parser *p)
 static void word_check_next_char (const char *s, parser *p)
 {
 #if LDEBUG
-    fprintf(stderr, "word_check_next_char: ch = '%c'\n", p->ch);
+    if (p->ch) fprintf(stderr, "word_check_next_char: ch = '%c'\n", p->ch);
+    else fprintf(stderr, "word_check_next_char: ch = NUL\n");
 #endif
 
     if (p->ch == '(') {
@@ -629,7 +675,6 @@ static void word_check_next_char (const char *s, parser *p)
 	} else if (p->sym == MVAR && model_data_matrix(p->idnum)) {
 	    /* old-style "$coeff(x1)" etc. */
 	    p->sym = DMSTR;
-	    p->idstr = gretl_strdup(s);
 	} else if (!func_symb(p->sym) && !func2_symb(p->sym) &&
 		   !funcn_symb(p->sym) && p->sym != UFUN) {
 	    p->err = 1;
@@ -641,7 +686,6 @@ static void word_check_next_char (const char *s, parser *p)
 	} else if (p->sym == MVAR && could_be_matrix(p->idnum)) {
 	    /* slice of $ matrix */
 	    p->sym = DMSL;
-	    p->idstr = gretl_strdup(s);
 	} else if (p->sym == UVAR && var_is_series(p->dinfo, p->idnum)) {
 	    /* observation from series */
 	    p->sym = OBS;
@@ -717,10 +761,30 @@ static void getword (parser *p)
 #endif
 }
 
+static int doing_matrix_slice;
+
+void set_matrix_slice_on (void)
+{
+    doing_matrix_slice = 1;
+}
+
+void set_matrix_slice_off (void)
+{
+    doing_matrix_slice = 0;
+}
+
 static int colon_ok (char *s, int n)
 {
     int i;
 
+    if (doing_matrix_slice) {
+	/* colon is a separator in this context */
+#if LDEBUG
+	fprintf(stderr, "colon_ok: doing matrix slice\n");
+#endif
+	return 0;
+    }
+
     if (n != 1 && n != 3) {
 	return 0;
     }
@@ -760,6 +824,7 @@ static int ok_dbl_char (int ch, char *s, int i)
 	return !strchr(s, 'e') && !strchr(s, 'E') && 
 	    !strchr(s, ':');
     case ':':
+	/* allow for obs numbers in the form, e.g., "1995:10" */
 	return colon_ok(s, i);
     default:
 	break;
@@ -803,10 +868,11 @@ static double getdbl (parser *p)
     } else {
 	d = dot_atof(xstr);
     }
-
+    
     return d;
 }
 
+#if 0 /* later? */
 static void deprecation_note (parser *p)
 {
     if (p->sym == B_AND) {
@@ -817,6 +883,7 @@ static void deprecation_note (parser *p)
 		"OR is '||'\n");
     }
 }
+#endif
 
 #define matrix_gen(p) (p->lh.t == MAT || p->targ == MAT)
 
@@ -824,6 +891,16 @@ static void deprecation_note (parser *p)
 
 void lex (parser *p)
 {
+#if LDEBUG
+    if (p->ch) fprintf(stderr, "lex: p->ch = '%c'\n", p->ch);
+    else fprintf(stderr, "lex: p->ch = NUL\n");
+#endif
+
+    if (p->ch == 0) {
+	p->sym = EOT;
+	return;
+    }
+
     while (p->ch != 0) {
 	switch (p->ch) {
 	case ' ':
@@ -843,11 +920,7 @@ void lex (parser *p)
         case '*': 
 	    parser_getc(p);
 	    if (p->ch == '*') {
-		if (matrix_gen(p)) {
-		    p->sym = KRON;
-		} else {
-		    p->sym = AST2;
-		}
+		p->sym = KRON;
 		parser_getc(p);
 	    } else {
 		p->sym = B_MUL;
@@ -896,11 +969,7 @@ void lex (parser *p)
 	    p->sym = (matrix_gen(p))? MRCAT : B_OR;
 	    parser_getc(p);
 	    if (p->ch == '|') {
-		if (p->sym == B_OR) {
-		   deprecation_note(p);
-		} else {
-		    p->sym = B_OR;
-		}
+		p->sym = B_OR;
 		parser_getc(p);
 	    }
 	    return;
@@ -1017,6 +1086,14 @@ void lex (parser *p)
 		p->sym = DOTEQ;
 		parser_getc(p);
 		return;
+	    } else if (p->ch == '>') {
+		p->sym = DOTGT;
+		parser_getc(p);
+		return;
+	    } else if (p->ch == '<') {
+		p->sym = DOTLT;
+		parser_getc(p);
+		return;
 	    } else {
 		/* not a "dot operator", back up */
 		parser_ungetc(p);
@@ -1051,6 +1128,10 @@ const char *getsymb (int t, const parser *p)
 	return funname(t);
     }
 
+    if (t == EOT) {
+	return "";
+    }
+
     /* yes, well */
     if (t == OBS) {
 	return "OBS";
@@ -1162,6 +1243,10 @@ const char *getsymb (int t, const parser *p)
 	return ".-";
     case DOTEQ: 
 	return ".=";
+    case DOTGT: 
+	return ".>";
+    case DOTLT: 
+	return ".<";
     case KRON: 
 	return "**";
     case MCCAT: 
