@@ -1,20 +1,20 @@
-/* gretl - The Gnu Regression, Econometrics and Time-series Library
- * Copyright (C) 1999-2006 Allin Cottrell
- *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License 
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
- *
- * This software is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License 
- * along with this software; if not, write to the 
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+/* 
+ *  gretl -- Gnu Regression, Econometrics and Time-series Library
+ *  Copyright (C) 2001 Allin Cottrell and Riccardo "Jack" Lucchetti
+ * 
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ * 
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ * 
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 #ifndef GENMAIN_H
@@ -31,6 +31,7 @@ typedef enum {
     R_DSET_MAX,   /* separator */
     R_TEST_STAT,  /* test statistic from last explicit test performed */
     R_TEST_PVAL,  /* p-value from last explicit test performed */
+    R_TEST_LNL,   /* log-likelihood from last test (if applicable) */
     R_STOPWATCH,  /* stopwatch */ 
     R_SCALAR_MAX, /* separator: scalars vs series */
     R_INDEX,      /* consecutive observations index */
@@ -45,6 +46,7 @@ typedef enum {
     M_DF,         /* degrees of freedom */
     M_NCOEFF,     /* total number of estimated coefficients */
     M_LNL,        /* log-likelihood */
+    M_GMMCRIT,    /* GMM criterion */
     M_AIC,        /* Akaike info criterion */
     M_BIC,        /* Bayesian info criterion */
     M_HQC,        /* Hannan-Quinn criterion */
@@ -59,10 +61,10 @@ typedef enum {
     M_SE,         /* parameter standard errors */
     M_VCV,        /* parameter covariance matrix */
     M_RHO,        /* autoregressive coefficients */
-    M_OLDMAT_MAX, /* -- separator -- */
+    M_COMPAN,     /* VAR companion matrix */
     M_JALPHA,     /* Johansen's alpha */
     M_JBETA,      /* Johansen's beta */
-    M_JVBETA,     /* Covariance matrix for Johansen's normalised beta */
+    M_JVBETA,     /* Covariance matrix for Johansen's normalized beta */
     M_JS00,       /* VECM residual covariance matrix (1st differences) */
     M_JS11,       /* VECM residual covariance matrix (levels) */
     M_JS01,       /* VECM residual cross-product matrix */
@@ -108,8 +110,12 @@ int print_object_var (const char *oname, const char *param,
 
 int gretl_reserved_word (const char *str);
 
+int genr_special_word (const char *s);
+
 /* following functions used in nls.c */
 
+int genr_is_print (const GENERATOR *p);
+
 void genr_set_na_check (GENERATOR *genr);
 
 void genr_unset_na_check (GENERATOR *genr);
