@@ -1,20 +1,20 @@
-/*
- *   Copyright (c) by Allin Cottrell
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
- *
- *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
- *
+/* 
+ *  gretl -- Gnu Regression, Econometrics and Time-series Library
+ *  Copyright (C) 2001 Allin Cottrell and Riccardo "Jack" Lucchetti
+ * 
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ * 
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ * 
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 /* parser module for 'genr' and related commands */
@@ -29,10 +29,7 @@
 # define MDEBUG 0
 #endif
 
-enum {
-    STR_COPY,
-    STR_STEAL
-};
+#define set_transpose(n) (n->flags |= TRANSP_NODE)
 
 static NODE *powterm (parser *p);
 
@@ -55,44 +52,39 @@ static NODE *newempty (int t)
     if (n != NULL) {
 	n->t = t;
 	n->v.idnum = 0;
-	n->ext = 0;
-	n->aux = 0;
-	n->tmp = 0;
+	n->flags = 0;
     }
 
     return n;
 }
 
-static NODE *newref (parser *p)
+static NODE *newref (parser *p, int t)
 {  
     NODE *n = malloc(sizeof *n);
 
 #if MDEBUG
-    fprintf(stderr, "newref: allocated node at %p (type = %d, idnum = %d)\n", 
-	    (void *) n, p->sym, p->idnum);
+    fprintf(stderr, "newref: allocated node at %p (type %d)\n", 
+	    (void *) n, t);
 #endif
 
     if (n != NULL) {
-	n->t = p->sym;
-	if (n->t == UMAT || n->t == UOBJ ||
-	    n->t == LOOPIDX || n->t == LIST) {
+	n->t = t;
+	if (t == UMAT || t == UOBJ || t == LOOPIDX || t == LIST) {
 	    n->v.str = p->idstr;
 	} else {
 	    n->v.idnum = p->idnum;
 	}
-	n->ext = 0;
-	n->aux = 0;
-	n->tmp = 0;
+	n->flags = 0;
     }
 
     return n;
 }
 
-static NODE *newstr (char *s, int ext, int flag)
+static NODE *newstr (parser *p)
 {  
     NODE *n;
 
-    if (s == NULL) {
+    if (p->idstr == NULL) {
 	fprintf(stderr, "newstr: input is NULL\n");
 	return NULL;
     }
@@ -101,19 +93,13 @@ static NODE *newstr (char *s, int ext, int flag)
 
 #if MDEBUG
     fprintf(stderr, "newstr: allocated node at %p (s = '%s')\n", 
-	    (void *) n, s);
+	    (void *) n, p->idstr);
 #endif
 
     if (n != NULL) {
 	n->t = STR;
-	if (flag == STR_COPY) {
-	    n->v.str = gretl_strdup(s);
-	} else {
-	    n->v.str = s;
-	}
-	n->tmp = 0;
-	n->ext = ext;
-	n->aux = 0;
+	n->v.str = p->idstr;
+	n->flags = 0;
     }
 
     return n;
@@ -133,9 +119,7 @@ NODE *newdbl (double x)
     if (n != NULL) {
 	n->t = NUM;
 	n->v.xval = x;
-	n->tmp = 0;
-	n->ext = 0;
-	n->aux = 0;
+	n->flags = 0;
     }
 
     return n;
@@ -143,7 +127,7 @@ NODE *newdbl (double x)
 
 /* node for unary operator, or single-argument function */
 
-static NODE *newb1 (int t, NODE *b, int ext)
+static NODE *newb1 (int t, NODE *b)
 {  
     NODE *n = malloc(sizeof *n);
 
@@ -155,9 +139,7 @@ static NODE *newb1 (int t, NODE *b, int ext)
     if (n != NULL) {
 	n->t = t;
 	n->v.b1.b = b;
-	n->tmp = 0;
-	n->ext = ext;
-	n->aux = 0;
+	n->flags = 0;
     }
 
     return n;
@@ -178,14 +160,7 @@ static NODE *newb2 (int t, NODE *l, NODE *r)
 	n->t = t;
 	n->v.b2.l = l;
 	n->v.b2.r = r;
-	n->tmp = 0;
-	n->ext = 0;
-	n->aux = 0;
-
-	if (n->t == AST2) {
-	    n->t = B_POW;
-	    n->ext = 1; /* record possible ambiguity */
-	}
+	n->flags = 0;
     }
 
     return n;
@@ -207,9 +182,7 @@ static NODE *newb3 (int t, NODE *l, NODE *m, NODE *r)
 	n->v.b3.l = l;
 	n->v.b3.m = m;
 	n->v.b3.r = r;
-	n->tmp = 0;
-	n->ext = 0;
-	n->aux = 0;
+	n->flags = 0;
     }
 
     return n;
@@ -230,9 +203,7 @@ static NODE *newbn (int t)
 	n->t = t;
 	n->v.bn.n_nodes = 0;
 	n->v.bn.n = NULL;
-	n->tmp = 0;
-	n->ext = 0;
-	n->aux = 0;
+	n->flags = 0;
     }
 
     return n;
@@ -329,9 +300,9 @@ static NODE *base (parser *p, NODE *up)
     case OBS:
     case LIST:
     case LOOPIDX:
-	t = newref(p);
+	t = newref(p, p->sym);
 	if (matrix_ref_node(p) && unary_apost(p)) {
-	    t->ext = TRANSP;
+	    set_transpose(t);
 	    parser_getc(p);
 	}
 	lex(p);
@@ -340,7 +311,7 @@ static NODE *base (parser *p, NODE *up)
 	if (p->idnum == DUM_NULL) {
 	    t = newempty(EMPTY);
 	} else {
-	    t = newref(p);
+	    t = newref(p, p->sym);
 	}
 	lex(p);
 	break;
@@ -355,7 +326,7 @@ static NODE *base (parser *p, NODE *up)
 	t = expr(p);
 	if (p->sym == RPR) {
 	    if (up != NULL && up->t != LAG && unary_apost(p)) {
-		up->ext = TRANSP; 
+		set_transpose(up);
 		parser_getc(p);
 	    } 
 	    lex(p);
@@ -373,7 +344,7 @@ static NODE *base (parser *p, NODE *up)
 	if (p->sym == RBR) {
 	    if (up->t == MSL || up->t == DMSL) {
 		if (p->ch == '\'') {
-		    up->ext = TRANSP;
+		    set_transpose(up);
 		    parser_getc(p);
 		}
 	    }
@@ -435,7 +406,10 @@ static NODE *get_string_arg (parser *p)
     parser_getc(p);
     lex(p);
 
-    return newstr(tailstrip(str), 0, STR_COPY);
+    tailstrip(str);
+    p->idstr = gretl_strdup(str); /* leakage possible? */
+
+    return newstr(p);
 }
 
 enum {
@@ -516,7 +490,7 @@ static void get_matrix_def (NODE *t, parser *p, int *sub)
 		lex(p);
 	    } else if (p->sym == RCB) {
 		if (p->ch == '\'') {
-		    t->ext = TRANSP;
+		    set_transpose(t);
 		    parser_getc(p);
 		} else if (p->ch == '[') {
 		    parser_ungetc(p);
@@ -548,7 +522,9 @@ static void get_slice_parts (NODE *t, parser *p)
 
 #if SDEBUG
     fprintf(stderr, "get_slice_parts, p->sym = %d\n", p->sym);
-#endif    
+#endif  
+
+    set_matrix_slice_on();
 
     if (p->sym == LBR) {
 	lex(p);
@@ -568,6 +544,7 @@ static void get_slice_parts (NODE *t, parser *p)
 	    /* co comma, no second arg string: may be OK */
 	    t->v.b2.r = newempty(ABSENT);
 	    lex(p);
+	    set_matrix_slice_off();
 	    return;
 	}
 	if (p->sym == COM) {
@@ -586,7 +563,7 @@ static void get_slice_parts (NODE *t, parser *p)
 	    }
 	    if (p->sym == RBR) {
 		if (p->ch == '\'') {
-		    t->ext = TRANSP; /* ?? */
+		    set_transpose(t); /* ?? */
 		    parser_getc(p);
 		}
 		lex(p);
@@ -603,6 +580,8 @@ static void get_slice_parts (NODE *t, parser *p)
     if (cexp && p->err == 0) {
 	expected_symbol_error(cexp, p);
     }
+
+    set_matrix_slice_off();
 }
 
 /* get up to two comma-separated arguments 
@@ -637,7 +616,7 @@ static void get_args (NODE *t, parser *p, int opt)
 	    t->v.b2.r = expr(p);
 	    if (p->sym == RPR) {
 		if (p->ch == '\'') {
-		    t->ext = TRANSP; 
+		    set_transpose(t);
 		    parser_getc(p);
 		}
 		lex(p);
@@ -677,17 +656,13 @@ static void get_ovar_ref (NODE *t, parser *p)
 	/* followed by '[' slice mechanism? */
 	t->v.b2.r = powterm(p);
     } else {
-	t->v.b2.r = newref(p);
+	t->v.b2.r = newref(p, p->sym);
 	lex(p);
     }
 }
 
-#define idnum_to_ext(t) (t == LAG || t == OBS || t == MVAR || \
-                         t == DMSL || t == DMSTR)
-
 static NODE *powterm (parser *p)
 {  
-    int ext = (idnum_to_ext(p->sym)) ? p->idnum : 0;
     int opt = 0;
     NODE *t;
 
@@ -700,7 +675,7 @@ static NODE *powterm (parser *p)
 	    p->sym, p->ch? p->ch : '0');
 #endif
 
-    if (p->sym == UNIFORM || p->sym == NORMAL) {
+    if (p->sym == RUNIFORM || p->sym == RNORMAL) {
 	opt = BOTH_OPT;
     }
 
@@ -711,7 +686,7 @@ static NODE *powterm (parser *p)
 	    get_args(t, p, opt);
 	}
     } else if (string0_func(p->sym)) {
-	t = newb1(p->sym, NULL, 0);
+	t = newb1(p->sym, NULL);
 	if (t != NULL) {
 	    lex(p);
 	    t->v.b1.b = newbn(FARGS);
@@ -721,22 +696,32 @@ static NODE *powterm (parser *p)
 	    }
 	}	
     } else if (string_arg_func(p->sym)) {
-	t = newb1(p->sym, NULL, ext);
+	t = newb1(p->sym, NULL);
 	if (t != NULL) {
 	    lex(p);
 	    t->v.b1.b = get_string_arg(p);
 	}	
     } else if (func_symb(p->sym)) {
-	/* includes LAG, OBS */
-	t = newb1(p->sym, NULL, ext);
+	t = newb1(p->sym, NULL);
 	if (t != NULL) {
 	    lex(p);
 	    t->v.b1.b = base(p, t);
 	}
+    } else if (p->sym == LAG || p->sym == OBS) {
+	t = newb2(p->sym, NULL, NULL);
+	if (t != NULL) {
+	    t->v.b2.l = newref(p, UVAR); 
+	    lex(p);
+	    t->v.b2.r = base(p, t);
+	}
     } else if (p->sym == MSL || p->sym == DMSL) {
 	t = newb2(p->sym, NULL, NULL);
 	if (t != NULL) {
-	    t->v.b2.l = newstr(p->idstr, ext, STR_STEAL);
+	    if (p->sym == MSL) {
+		t->v.b2.l = newstr(p);
+	    } else {
+		t->v.b2.l = newref(p, MVAR);
+	    }
 	    t->v.b2.r = newb2(MSL2, NULL, NULL);
 	    if (t->v.b2.r != NULL) {
 		lex(p);
@@ -746,20 +731,20 @@ static NODE *powterm (parser *p)
     } else if (p->sym == DMSTR) {
 	t = newb2(p->sym, NULL, NULL);
 	if (t != NULL) {
-	    t->v.b2.l = newstr(p->idstr, ext, STR_STEAL);
+	    t->v.b2.l = newref(p, MVAR);
 	    lex(p);
 	    t->v.b2.r = get_string_arg(p);
 	}
     } else if (p->sym == OVAR) {
 	t = newb2(p->sym, NULL, NULL);
 	if (t != NULL) {
-	    t->v.b2.l = newstr(p->idstr, 0, STR_STEAL);
+	    t->v.b2.l = newstr(p);
 	    get_ovar_ref(t, p);
 	}
     } else if (p->sym == LPR) {
 	/* dummy root for parenthesized expressions, to facilitate
 	   taking the transpose of matrix stuff, e.g. (A*B)' */
-	t = newb1(EROOT, NULL, 0);
+	t = newb1(EROOT, NULL);
 	if (t != NULL) {
 	    t->v.b1.b = base(p, t);
 	}
@@ -785,7 +770,7 @@ static NODE *powterm (parser *p)
     } else if (p->sym == UFUN) {
 	t = newb2(p->sym, NULL, NULL);
 	if (t != NULL) {
-	    t->v.b2.l = newstr(p->idstr, 0, STR_STEAL);
+	    t->v.b2.l = newstr(p);
 	    lex(p);
 	    t->v.b2.r = newbn(FARGS);
 	    if (t != NULL) {
@@ -793,7 +778,7 @@ static NODE *powterm (parser *p)
 	    }
 	}
     } else if (funcn_symb(p->sym)) {
-	t = newb1(p->sym, NULL, 0);
+	t = newb1(p->sym, NULL);
 	if (t != NULL) {
 	    lex(p);
 	    t->v.b1.b = newbn(FARGS);
@@ -802,7 +787,7 @@ static NODE *powterm (parser *p)
 	    }
 	}
     } else if (p->sym == STR) {
-	t = newstr(p->idstr, 0, STR_STEAL);
+	t = newstr(p);
 	lex(p);
     } else {
 	t = base(p, NULL);
@@ -828,7 +813,6 @@ if (unary_op(sym)) {
 }
 #endif
 
-
 static NODE *factor (parser *p)
 {  
     int sym = p->sym == B_SUB ? U_NEG : 
@@ -846,7 +830,7 @@ static NODE *factor (parser *p)
 	    context_error(0, p);
 	    return NULL;
 	}
-        t = newb1(sym, NULL, 0);
+        t = newb1(sym, NULL);
         if (t != NULL) {
             lex(p);
             t->v.b1.b = factor(p);
@@ -854,14 +838,18 @@ static NODE *factor (parser *p)
     } else {
 	t = powterm(p);
 	if (t != NULL) {
-	    while (!p->err && (p->sym == B_POW || 
-			       p->sym == AST2 ||
-			       p->sym == DOTPOW ||
-			       p->sym == B_TRMUL)) {
-		t = newb2(p->sym, t, NULL);
-		if (t != NULL) {
-		    lex(p);
-		    t->v.b2.r = powterm(p);
+	    if (p->sym == B_TRMUL && p->ch == 0) {
+		/* can't really be TRMUL at end of input */
+		set_transpose(t);
+	    } else {
+		while (!p->err && (p->sym == B_POW || 
+				   p->sym == DOTPOW ||
+				   p->sym == B_TRMUL)) {
+		    t = newb2(p->sym, t, NULL);
+		    if (t != NULL) {
+			lex(p);
+			t->v.b2.r = powterm(p);
+		    }
 		}
 	    }
 	}
@@ -933,6 +921,7 @@ static NODE *expr3 (parser *p)
     }
 
     while (!p->err && (p->sym == B_GT || p->sym == B_LT || 
+		       p->sym == DOTGT || p->sym == DOTLT || 
 		       p->sym == B_GTE || p->sym == B_LTE)) {
 	t = newb2(p->sym, t, NULL);
 	if (t != NULL) {
@@ -1030,10 +1019,10 @@ NODE *expr (parser *p)
 	t = newb3(p->sym, t, NULL, NULL);
 	if (t != NULL) {
 	    lex(p);
-	    t->v.b3.m = expr0(p);
+	    t->v.b3.m = expr(p);
 	    if (p->sym == COL) {
 		lex(p);
-		t->v.b3.r = expr0(p);
+		t->v.b3.r = expr(p);
 	    } else {
 		expected_symbol_error(':', p);
 	    }
