@@ -1,20 +1,20 @@
-/*
- *  Copyright (c) 2007 Allin Cottrell and Riccardo "Jack" Lucchetti
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
- *
- *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- *
+/* 
+ *  gretl -- Gnu Regression, Econometrics and Time-series Library
+ *  Copyright (C) 2001 Allin Cottrell and Riccardo "Jack" Lucchetti
+ * 
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ * 
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ * 
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 #include "libgretl.h" 
@@ -53,11 +53,11 @@ struct ocset_ {
     colsrc *ecols;       /* info on provenance of columns of 'e' */
     int noc;             /* total number of orthogonality conds. */
     int step;            /* number of estimation steps */
-    int free_e;          /* indicator: should 'e' be freed? */
-    int free_Z;          /* indicator: should 'Z' be freed? */
     hac_info hinfo;      /* HAC characteristics */
 };
 
+#define using_HAC(s) (s->oc->hinfo.kern >= KERNEL_BARTLETT)
+
 /* destructor for set of O.C. info */
 
 void oc_set_destroy (ocset *oc)
@@ -66,14 +66,8 @@ void oc_set_destroy (ocset *oc)
 	return;
     }
 
-    if (oc->free_e) {
-	gretl_matrix_free(oc->e);
-    }
-
-    if (oc->free_Z) {
-	gretl_matrix_free(oc->Z);
-    }
-
+    gretl_matrix_free(oc->e);
+    gretl_matrix_free(oc->Z);
     gretl_matrix_free(oc->tmp);
     gretl_matrix_free(oc->sum);
     gretl_matrix_free(oc->S);
@@ -99,9 +93,6 @@ static ocset *oc_set_new (void)
 	oc->ecols = NULL;
 	oc->noc = 0;
 	oc->step = 0;
-
-	oc->free_e = 0;
-	oc->free_Z = 0;
     }
 
     return oc;
@@ -135,15 +126,15 @@ static int oc_get_type (const char *name, const DATAINFO *pdinfo,
     if (v >= 0 && v < pdinfo->v) {
 	if (var_is_scalar(pdinfo, v)) {
 	    *err = E_TYPES;
-	    return ARG_NONE;
+	    return GRETL_TYPE_NONE;
 	} else {
-	    return ARG_SERIES;
+	    return GRETL_TYPE_SERIES;
 	}
     }
 
     /* then a matrix */
     if (get_matrix_by_name(name) != NULL) {
-	return ARG_MATRIX;
+	return GRETL_TYPE_MATRIX;
     }
 
     /* failing that, a list */
@@ -154,18 +145,18 @@ static int oc_get_type (const char *name, const DATAINFO *pdinfo,
 	for (j=1; j<=list[0]; j++) {
 	    if (list[j] < 0 || list[j] >= pdinfo->v) {
 		*err = E_DATA;
-		return ARG_NONE;
+		return GRETL_TYPE_NONE;
 	    } else if (var_is_scalar(pdinfo, list[j])) {
 		*err = E_TYPES;
-		return ARG_NONE;
+		return GRETL_TYPE_NONE;
 	    }
 	}
-	return ARG_LIST;
+	return GRETL_TYPE_LIST;
     }
 
     *err = gmm_unkvar(name);
 
-    return ARG_NONE;
+    return GRETL_TYPE_NONE;
 }
 
 /* See if column j in matrix b is present anywhere in matrix a.  If
@@ -400,8 +391,6 @@ static int oc_add_matrices (nlspec *s, int ltype, const char *lname,
     gretl_matrix *M = NULL;
     int oldecols = 0;
     int i, k, t, v;
-    int free_e = 0;
-    int free_M = 0;
     double x;
     int err = 0;
 
@@ -417,10 +406,12 @@ static int oc_add_matrices (nlspec *s, int ltype, const char *lname,
     fprintf(stderr, "oc_add_matrices: lname = '%s'\n", lname);
 #endif
 
-    if (ltype == ARG_MATRIX) {
-	e = get_matrix_by_name(lname);
-	err = push_column_source(s, 0, lname);
-    } else if (ltype == ARG_LIST) {
+    if (ltype == GRETL_TYPE_MATRIX) {
+	e = get_matrix_copy_by_name(lname, &err);
+	if (!err) {
+	    err = push_column_source(s, 0, lname);
+	}
+    } else if (ltype == GRETL_TYPE_LIST) {
 	int *list = get_list_by_name(lname);
 
 	k = list[0];
@@ -438,9 +429,8 @@ static int oc_add_matrices (nlspec *s, int ltype, const char *lname,
 		}
 		err = push_column_source(s, v, NULL);
 	    }
-	    free_e = s->oc->free_e = 1;
 	}	
-    } else if (ltype == ARG_SERIES) {
+    } else if (ltype == GRETL_TYPE_SERIES) {
 	v = varindex(pdinfo, lname);
 	e = gretl_column_vector_alloc(s->nobs);
 	if (e == NULL) {
@@ -452,7 +442,6 @@ static int oc_add_matrices (nlspec *s, int ltype, const char *lname,
 		x = Z[v][t + s->t1];
 		gretl_vector_set(e, t, x);
 	    }
-	    free_e = s->oc->free_e = 1;
 	    err = push_column_source(s, v, NULL);
 	}
     } else {
@@ -473,10 +462,12 @@ static int oc_add_matrices (nlspec *s, int ltype, const char *lname,
     fprintf(stderr, "oc_add_matrices: rname = '%s'\n", rname);
 #endif
 
-    if (rtype == ARG_MATRIX) {
-	M = get_matrix_by_name(rname);
-	k = gretl_matrix_cols(M);
-    } else if (rtype == ARG_LIST) {
+    if (rtype == GRETL_TYPE_MATRIX) {
+	M = get_matrix_copy_by_name(rname, &err);
+	if (!err) {
+	    k = gretl_matrix_cols(M);
+	}
+    } else if (rtype == GRETL_TYPE_LIST) {
 	int *list = get_list_by_name(rname);
 
 	k = list[0];
@@ -493,7 +484,6 @@ static int oc_add_matrices (nlspec *s, int ltype, const char *lname,
 		    gretl_matrix_set(M, t, i, x);
 		}
 	    }
-	    free_M = s->oc->free_Z = 1;
 	}
     } else {
 	/* name of single series */
@@ -509,7 +499,6 @@ static int oc_add_matrices (nlspec *s, int ltype, const char *lname,
 		x = Z[v][t + s->t1];
 		gretl_vector_set(M, t, x);
 	    }
-	    free_M = s->oc->free_Z = 1;
 	}
     } 
 
@@ -525,50 +514,22 @@ static int oc_add_matrices (nlspec *s, int ltype, const char *lname,
 #endif
 	} else {
 	    /* it's an additional set of O.C.s */
-	    if (!s->oc->free_e) {
-		/* current 'e' is pointer to external matrix: this
-		   must now be changed */
-		s->oc->e = gretl_matrix_copy(s->oc->e);
-		if (s->oc->e == NULL) {
-		    err = E_ALLOC;
-		}
-	    }
+	    err = gretl_matrix_inplace_colcat(s->oc->e, e, NULL);
+#if GMM_DEBUG > 1
+	    fprintf(stderr, "oc_add_matrices: expanded e\n");
+	    gretl_matrix_print(s->oc->e, "e");
+#endif
+	    /* don't leak temporary matrix */
+	    gretl_matrix_free(e);
 
 	    if (!err) {
-		err = gretl_matrix_inplace_colcat(s->oc->e, e, NULL);
+		err = add_new_cols_to_Z(s, M, oldecols);
 #if GMM_DEBUG > 1
-		fprintf(stderr, "oc_add_matrices: expanded e\n");
-		gretl_matrix_print(s->oc->e, "e");
+		fprintf(stderr, "oc_add_matrices: expanded Z\n");
+		gretl_matrix_print(s->oc->Z, "Z");
 #endif
 	    }
-
-	    if (free_e) {
-		/* don't leak temporary matrix */
-		gretl_matrix_free(e);
-	    }
-
-	    if (!err) {
-		if (!s->oc->free_Z) {
-		    /* current 'Z' is pointer to external matrix */
-		    s->oc->Z = gretl_matrix_copy(s->oc->Z);
-		    if (s->oc->Z == NULL) {
-			err = E_ALLOC;
-		    }
-		}
-		if (!err) {
-		    err = add_new_cols_to_Z(s, M, oldecols);
-#if GMM_DEBUG > 1
-		    fprintf(stderr, "oc_add_matrices: expanded Z\n");
-		    gretl_matrix_print(s->oc->Z, "Z");
-#endif
-		}
-	    }
-
-	    if (free_M) {
-		gretl_matrix_free(M);
-	    }
-
-	    s->oc->free_e = s->oc->free_Z = 1;
+	    gretl_matrix_free(M);
 	}
     }
 
@@ -591,7 +552,7 @@ nlspec_add_orthcond (nlspec *s, const char *str,
 {
     char lname[VNAMELEN];
     char rname[VNAMELEN];
-    int ltype = ARG_NONE, rtype = ARG_NONE;
+    int ltype = GRETL_TYPE_NONE, rtype = GRETL_TYPE_NONE;
     int err = 0;
 
     if (s->ci != GMM) {
@@ -1250,7 +1211,7 @@ int gmm_add_vcv (MODEL *pmod, nlspec *s)
     int T = s->nobs;
 
     doublereal *wa4 = NULL;
-    doublereal epsfcn = 0.0;
+    doublereal epsfcn = 0.0; /* could be > 0.0 */
     integer m, n, ldjac;
     integer iflag = 0;
 
@@ -1276,7 +1237,7 @@ int gmm_add_vcv (MODEL *pmod, nlspec *s)
 	goto bailout;
     }
 
-    if (s->oc->hinfo.kern != 0) {
+    if (using_HAC(s)) {
 	err = gmm_HAC(s->oc->tmp, S, &s->oc->hinfo);
 	gmm_HAC_cleanup();
     } else {
@@ -1286,8 +1247,9 @@ int gmm_add_vcv (MODEL *pmod, nlspec *s)
     } 
 
     if (!err) {
-	gretl_matrix_divide_by_scalar(S, s->nobs);
+	double Tfac = sqrt((double) T) / T;
 
+	gretl_matrix_divide_by_scalar(S, T);
 	f = s->oc->sum->val;
 
 	for (i=0; i<m; i++) {
@@ -1295,13 +1257,14 @@ int gmm_add_vcv (MODEL *pmod, nlspec *s)
 	    for (j=0; j<T; j++) {
 		f[i] += gretl_matrix_get(s->oc->tmp, j, i);
 	    }
-	    f[i] *= sqrt((double) T) / T;
+	    f[i] *= Tfac;
 	}
 
 	fdjac2_(gmm_jacobian_calc, &m, &n, s->coeff, f, 
 		J->val, &ldjac, &iflag, &epsfcn, wa4, s);
 
 	if (iflag != 0) {
+	    fprintf(stderr, "fdjac2_: iflag = %d\n", (int) iflag);
 	    err = 1;
 	}
     }
@@ -1341,7 +1304,8 @@ int gmm_add_vcv (MODEL *pmod, nlspec *s)
 		} 
 	    }
 	}
-	if (s->oc->hinfo.kern) {
+	if (using_HAC(s)) {
+	    gretl_model_set_int(pmod, "using_hac", 1);
 	    gretl_model_set_int(pmod, "hac_kernel", s->oc->hinfo.kern);
 	    if (s->oc->hinfo.kern == KERNEL_QS) {
 		gretl_model_set_double(pmod, "qs_bandwidth", s->oc->hinfo.bt);
@@ -1356,18 +1320,25 @@ int gmm_add_vcv (MODEL *pmod, nlspec *s)
 
     if (!err) {
 	/* set additional GMM info */
+	double TGcrit = - s->crit / s->nobs;
 	int l = s->oc->noc;
 	
-	pmod->ess = - s->crit; /* note the borrowing! */
+	pmod->ess = TGcrit / s->nobs; /* note the borrowing! */
 
 	if (l > k && ((s->opt & OPT_V) || s->oc->step > 1)) {
 	    gretl_model_set_int(pmod, "J_df", l - k);
-	    gretl_model_set_double(pmod, "J_test", pmod->ess / s->nobs);
+	    gretl_model_set_double(pmod, "J_test", TGcrit);
 	}
 
 	if (s->oc->step > 1) {
 	    gretl_model_set_int(pmod, "step", s->oc->step);
 	}
+
+	if (s->opt & OPT_T) {
+	    gretl_model_set_int(pmod, "two-step", 1);
+	} else if (s->opt & OPT_I) {
+	    gretl_model_set_int(pmod, "iterated", 1);
+	}
     }
 
  bailout:
@@ -1410,7 +1381,7 @@ static int gmm_recompute_weights (nlspec *s)
     gretl_matrix *W = s->oc->W;
     int err = 0;
 
-    if (s->oc->hinfo.kern) {
+    if (using_HAC(s)) {
 	err = gmm_HAC(s->oc->tmp, W, &s->oc->hinfo);
     } else {
 	err = gretl_matrix_multiply_mod(s->oc->tmp, GRETL_MOD_TRANSPOSE,
@@ -1439,7 +1410,7 @@ static void gmm_print_oc (nlspec *s, PRN *prn)
 	return;
     }
 
-    if (s->oc->hinfo.kern) {
+    if (using_HAC(s)) {
 	err = gmm_HAC(s->oc->tmp, V, &s->oc->hinfo);
     } else {
 	err = gretl_matrix_multiply_mod(s->oc->tmp, GRETL_MOD_TRANSPOSE,
@@ -1477,7 +1448,8 @@ int gmm_calculate (nlspec *s, double *fvec, double *jac, PRN *prn)
     int full_grcount = 0;
     double itol = 1.0e-12, icrit = 1;
     double *oldcoeff = NULL;
-    int maxit = get_bfgs_maxiter();
+    int maxit = libset_get_int(BFGS_MAXITER);
+    gretlopt iopt = s->opt;
     int outer_iters = 0;
     int outer_max = 1;
     int converged = 0;
@@ -1508,7 +1480,10 @@ int gmm_calculate (nlspec *s, double *fvec, double *jac, PRN *prn)
 	err = BFGS_max(s->coeff, s->ncoeff, maxit, s->tol, 
 		       &s->fncount, &s->grcount, 
 		       get_gmm_crit, C_GMM, NULL, s,
-		       s->opt, s->prn);
+		       iopt, s->prn);
+
+	/* don't keep displaying certain things */
+	iopt |= OPT_Q;
 
 	full_fncount += s->fncount;
 	full_grcount += s->grcount;
@@ -1618,18 +1593,18 @@ static void gmm_set_HAC_info (nlspec *s)
 {
     hac_info *hinfo = &s->oc->hinfo;
 
-    if (dataset_is_time_series(s->dinfo) && !get_force_hc()) {
-	hinfo->whiten = get_hac_prewhiten();
-	hinfo->kern = get_hac_kernel();
+    if (dataset_is_time_series(s->dinfo) && !libset_get_bool(FORCE_HC)) {
+	hinfo->whiten = libset_get_bool(PREWHITEN);
+	hinfo->kern = libset_get_int(HAC_KERNEL);
 	if (hinfo->kern == KERNEL_QS) {
-	    hinfo->bt = get_qs_bandwidth();
+	    hinfo->bt = libset_get_double(QS_BANDWIDTH);
 	    hinfo->h = s->nobs - 1;
 	} else {
 	    hinfo->h = get_hac_lag(s->nobs);
 	    hinfo->bt = 0.0;
 	}
     } else {
-	hinfo->kern = 0;
+	hinfo->kern = -1;
 	hinfo->h = 0;
 	hinfo->bt = 0.0;
 	hinfo->whiten = 0;
