@@ -1,20 +1,20 @@
-/*
- *  Copyright (c) by Allin Cottrell and Riccardo Lucchetti
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
- *
- *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- *
+/* 
+ *  gretl -- Gnu Regression, Econometrics and Time-series Library
+ *  Copyright (C) 2001 Allin Cottrell and Riccardo "Jack" Lucchetti
+ * 
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ * 
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ * 
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 /* graphing.c for gretl */
@@ -66,6 +66,8 @@ struct gnuplot_info_ {
     double *yvar2;
 };
 
+#define MAX_LETTERBOX_LINES 8
+
 #define use_impulses(g) ((g)->flags & GPT_IMPULSES)
 #define ts_plot(g) ((g)->flags & GPT_TS)
 
@@ -98,6 +100,7 @@ struct plot_type_info ptinfo[] = {
     { PLOT_MULTI_IRF,      "multiple impulse responses" },
     { PLOT_PANEL,          "multiple panel plots" },
     { PLOT_BI_GRAPH,       "double time-series plot" },
+    { PLOT_MANY_TS,        "multiple timeseries" },
     { PLOT_TYPE_MAX,       NULL }
 };
 
@@ -128,10 +131,16 @@ int gnuplot_test_command (const char *cmd)
     int child_pid = 0, sinp = 0, serr = 0;
     GError *error = NULL;
     gchar *argv[] = {
-	(*gnuplot_path == 0)? "gnuplot" : gnuplot_path,
+	NULL,
 	NULL
     };
 
+    if (*gnuplot_path == 0) {
+	strcpy(gnuplot_path, gretl_gnuplot_path());
+    }
+
+    argv[0] = gnuplot_path;
+
     signal(SIGCHLD, SIG_DFL);
 
     ok = g_spawn_async_with_pipes (NULL,
@@ -340,6 +349,8 @@ static int factorized_vars (gnuplot_info *gi, const double **Z)
 
 #ifdef WIN32
 
+#define WIN32_USE_CAIRO 0 /* close, but not just yet */
+
 int gnuplot_has_ttf (int reset)
 {
     /* we know the gnuplot supplied with gretl for win32
@@ -347,27 +358,33 @@ int gnuplot_has_ttf (int reset)
     return 1;
 }
 
-int gnuplot_has_pdf (void)
+int gnuplot_pdf_terminal (void)
 {
-    /* ... and we know it does PDF output */
-    return 1;
+#if WIN32_USE_CAIRO
+    return GP_PDF_CAIRO;
+#else
+    return GP_PDF_PDFLIB;
+#endif
 }
 
-int gnuplot_has_specified_colors (void)
+int gnuplot_png_terminal (void)
 {
-    /* ... and we know it does specified colors */
-    return 1;
+#if WIN32_USE_CAIRO
+    return GP_PNG_CAIRO;
+#else
+    return GP_PNG_GD2;
+#endif
 }
-
+   
 int gnuplot_has_style_fill (void)
 {
     /* ... and that it does style fill */
     return 1;
 }
 
-static int gnuplot_has_specified_emf_colors (void)
+static int gnuplot_uses_datafile_missing (void)
 {
-    /* ... and we know it does specified emf colors */
+    /* yup */
     return 1;
 }
 
@@ -377,6 +394,18 @@ const char *gnuplot_label_front_string (void)
     return " front";
 }
 
+int gnuplot_has_latin5 (void)
+{
+    /* ... and that it supports ISO8859-9 */
+    return 1;
+}
+
+int gnuplot_has_rgb (void)
+{
+    /* ... and that it supports rgb line-color specs */
+    return 1;
+}
+
 #else
 
 int gnuplot_has_ttf (int reset)
@@ -399,28 +428,85 @@ int gnuplot_has_ttf (int reset)
     return !err;
 }
 
-int gnuplot_has_pdf (void)
+static int gnuplot_has_size (void)
 {
     static int err = -1; 
-
+    
     if (err == -1) {
-	err = gnuplot_test_command("set term pdf");
+	err = gnuplot_test_command("set term png size 640,480");
     }
 
     return !err;
 }
 
-int gnuplot_has_specified_colors (void)
+int gnuplot_has_latin5 (void)
 {
     static int err = -1; 
 
     if (err == -1) {
-	/* try the old-style command: 
-	   if it fails, we have the new driver, we hope */
-	err = gnuplot_test_command("set term png color");
+	err = gnuplot_test_command("set encoding iso_8859_9");
     }
 
-    return err;
+    return !err;
+}
+
+int gnuplot_pdf_terminal (void)
+{
+    static int ret = -1;
+
+    if (ret == -1) {
+	int err = gnuplot_test_command("set term pdfcairo");
+
+	if (!err) {
+	    ret = GP_PDF_CAIRO;
+	} else {
+	    err = gnuplot_test_command("set term pdf");
+	    if (!err) {
+		ret = GP_PDF_PDFLIB;
+	    } else {
+		ret = GP_PDF_NONE;
+	    }
+	}
+    }
+
+    return ret;
+}
+
+/* We should enable pngcairo as the default as soon as possible -- but
+   for the present this poses a problem with regard to guessing the
+   pixel bounds in the PNG file.  So we'll accept pngcairo only if
+   we find a (hacked) version that supports the TERM_XMIN printable
+   variable.
+*/
+
+int gnuplot_png_terminal (void)
+{
+    static int ret = -1;
+
+    if (ret == -1) {
+	int err = gnuplot_test_command("set term pngcairo ; "
+				       "set output '/dev/null' ; "
+				       "plot x ; print TERM_XMIN");
+
+	if (!err) {
+	    fprintf(stderr, "gnuplot: using pngcairo driver\n");
+	    ret = GP_PNG_CAIRO;
+	} else {
+	    /* try the old-style command: if it fails, we have 
+	       the libgd driver, we hope! */
+	    err = gnuplot_test_command("set term png color");
+	    if (!err) {
+		fprintf(stderr, "gnuplot: got old png driver\n");
+		ret = GP_PNG_OLD;
+	    } else {
+		fprintf(stderr, "gnuplot: using libgd png driver\n");
+		err = gnuplot_test_command("set term png truecolor");
+		ret = (err)? GP_PNG_GD1 : GP_PNG_GD2;
+	    }
+	}
+    }
+
+    return ret;
 }
 
 int gnuplot_has_style_fill (void)
@@ -434,12 +520,23 @@ int gnuplot_has_style_fill (void)
     return !err;
 }
 
-static int gnuplot_has_specified_emf_colors (void)
+static int gnuplot_uses_datafile_missing (void)
 {
     static int err = -1; 
 
     if (err == -1) {
-	err = gnuplot_test_command("set term emf color xff0000");
+	err = gnuplot_test_command("set datafile missing \"?\"");
+    }
+
+    return !err;
+}
+
+int gnuplot_has_rgb (void)
+{
+    static int err = -1; 
+
+    if (err == -1) {
+	err = gnuplot_test_command("set style line 2 lc rgb \"#0000ff\"");
     }
 
     return !err;
@@ -460,102 +557,231 @@ const char *gnuplot_label_front_string (void)
     }
 }
 
-#endif /* WIN32 */
+#endif /* !WIN32 */
+
+static int gnuplot_png_use_aa = 1;
+
+void gnuplot_png_set_use_aa (int s)
+{
+    gnuplot_png_use_aa = s;
+}
 
 /* apparatus for handling plot colors */
 
-static const char default_palette[N_GP_COLORS][8] = {
-    "xff0000", 
-    "x0000ff", 
-    "x00cc00",
-    "x9ba6bb"
+enum {
+    OLD_PNG_COLOR,  /* plain old "color" option */
+    GD_PNG_COLOR,   /* pre-4.2.0 libgd-based PNG color spec */
+    RGB_LINE_COLOR  /* per-line rgb settings */
 };
 
-static char graph_palette[N_GP_COLORS][8] = {
-    "xff0000", 
-    "x0000ff", 
-    "x00cc00",  /* full-intensity green is not very legible */
-    "x9ba6bb"   /* color for box fill */
+static const gretlRGB default_color[N_GP_COLORS] = {
+    { 0xff, 0x00, 0x00 },
+    { 0x00, 0x00, 0xff },
+    { 0x00, 0xcc, 0x00 }, /* full-intensity green is not very legible */
+    { 0x9b, 0xa6, 0xbb }  /* color for box fill */
 };
 
-void print_palette_string (char *targ)
+static gretlRGB user_color[N_GP_COLORS] = {
+    { 0xff, 0x00, 0x00 },
+    { 0x00, 0x00, 0xff },
+    { 0x00, 0xcc, 0x00 },
+    { 0x9b, 0xa6, 0xbb }
+};
+
+static void print_rgb_x (char *s, gretlRGB color)
 {
-    sprintf(targ, "%s %s %s %s", graph_palette[0],
-	    graph_palette[1], graph_palette[2],
-	    graph_palette[3]);
+    sprintf(s, "x%02x%02x%02x", color.r, color.g, color.b);
 }
 
-const char *graph_color_string (int i)
+void print_rgb_hash (char *s, const gretlRGB *color)
 {
-#if GP_DEBUG
-    fprintf(stderr, "get_graph_palette: i=%d\n", i);
-#endif
-
-    return (i >= 0 && i < N_GP_COLORS)? graph_palette[i] : "";
+    sprintf(s, "#%02x%02x%02x", color->r, color->g, color->b);
 }
 
-static int colstr_is_valid (const char *colstr)
+void print_palette_string (char *s)
 {
-    char *test = NULL;
-    int cnum;
-
-    if (*colstr != 'x' || strlen(colstr) != 7) {
-	return 0;
-    }
-
-    cnum = strtol(colstr + 1, &test, 16);
-
-    if (*test != '\0' || cnum < 0 || cnum > 0xffffff) {
-	return 0;
-    }
-
-    return 1;
+    sprintf(s, "x%02x%02x%02x x%02x%02x%02x x%02x%02x%02x x%02x%02x%02x",
+	    user_color[0].r, user_color[0].g, user_color[0].b,
+	    user_color[1].r, user_color[1].g, user_color[1].b,
+	    user_color[2].r, user_color[2].g, user_color[2].b,
+	    user_color[3].r, user_color[3].g, user_color[3].b);
 }
 
-void set_graph_palette (int i, const char *colstr)
+const gretlRGB *get_graph_color (int i)
 {
-    if (i >= 0 && i < N_GP_COLORS && colstr_is_valid(colstr)) {
-	*graph_palette[i] = '\0';
-	strncat(graph_palette[i], colstr, 7);
+    return (i >= 0 && i < N_GP_COLORS)? &user_color[i] : NULL;
+}
+
+void set_graph_palette (int i, gretlRGB color)
+{
+    if (i >= 0 && i < N_GP_COLORS) {
+	user_color[i] = color;
     } else {
-	fprintf(stderr, "Invalid color spec, '%s'\n", colstr);
+	fprintf(stderr, "Out of bounds color index %d\n", i);
+    }
+}
+
+void set_graph_palette_from_string (int i, const char *s)
+{
+    int err = 0;
+
+    if (i >= 0 && i < N_GP_COLORS) {
+	unsigned int x[3];
+
+	if (sscanf(s + 1, "%02x%02x%02x", &x[0], &x[1], &x[2]) == 3) {
+	    user_color[i].r = x[0];
+	    user_color[i].g = x[1];
+	    user_color[i].b = x[2];
+	} else {
+	    err = 1;
+	}
+    } else {
+	err = 1;
+    }
+
+    if (err) {
+	fprintf(stderr, "Error in set_graph_palette_from_string(%d, '%s')\n", 
+		i, s);
     }
 }
 
 void graph_palette_reset (int i)
 {
     if (i == BOXCOLOR) {
-	strcpy(graph_palette[BOXCOLOR], default_palette[BOXCOLOR]);
+	user_color[BOXCOLOR] = default_color[BOXCOLOR];
     } else {
 	for (i=0; i<BOXCOLOR; i++) {
-	    strcpy(graph_palette[i], default_palette[i]);
+	    user_color[i] = default_color[i];
 	}
     }
 }
 
 static void 
-write_gnuplot_font_string (char *fstr, const char *grfont, PlotType ptype)
+write_gnuplot_font_string (char *fstr, const char *grfont, PlotType ptype,
+			   int pngterm)
 {
-    int shrink = 0;
+    if (pngterm == GP_PNG_CAIRO) {
+	char fname[128];
+	int fsize, nf;
 
-    if ((ptype == PLOT_MULTI_IRF || ptype == PLOT_MULTI_SCATTER) 
-	&& gp_small_font_size > 0) {
-	char fname[64];
-	int fsize;
-
-	if (sscanf(grfont, "%s %d", fname, &fsize) == 2) {
-	    sprintf(fstr, " font %s %d", fname, gp_small_font_size);
-	    shrink = 1;
+	nf = sscanf(grfont, "%s %d", fname, &fsize);
+	if (nf == 2) {
+	    if ((ptype == PLOT_MULTI_IRF || ptype == PLOT_MULTI_SCATTER) 
+		&& gp_small_font_size > 0) {
+		sprintf(fstr, " font \"%s,%d\"", fname, gp_small_font_size);
+	    } else {
+		sprintf(fstr, " font \"%s,%d\"", fname, fsize);
+	    }
+	} else if (nf == 1) {
+	    sprintf(fstr, " font \"%s\"", fname);
 	}
-    }
+    } else {
+	int shrink = 0;
 
-    if (!shrink) {
-	sprintf(fstr, " font %s", grfont);
+	if ((ptype == PLOT_MULTI_IRF || ptype == PLOT_MULTI_SCATTER) 
+	    && gp_small_font_size > 0) {
+	    char fname[64];
+	    int fsize;
+
+	    if (sscanf(grfont, "%s %d", fname, &fsize) == 2) {
+		sprintf(fstr, " font %s %d", fname, gp_small_font_size);
+		shrink = 1;
+	    }
+	}
+
+	if (!shrink) {
+	    sprintf(fstr, " font %s", grfont);
+	}
     }
 }
 
+#ifndef WIN32
+
+static void 
+write_old_gnuplot_font_string (char *fstr, PlotType ptype)
+{
+    if (ptype == PLOT_MULTI_IRF || ptype == PLOT_MULTI_SCATTER) {
+	strcpy(fstr, " tiny");
+    } else {
+	strcpy(fstr, " small");
+    }
+}
+
+#endif
+
+/* we need this only if we don't have per-line rgb
+   settings, which are in gnuplot 4.2 and higher */
+
+static char *make_png_colorspec (char *targ, int ptype)
+{
+    char cstr[8];
+    int i;
+
+    /* background etc. */
+    strcpy(targ, " xffffff x000000 x202020");
+
+    if (frequency_plot_code(ptype)) {
+	strcat(targ, " ");
+	print_rgb_x(cstr, user_color[BOXCOLOR]);
+	strcat(targ, cstr);
+	strcat(targ, " x000000");
+    } else {
+	for (i=0; i<BOXCOLOR; i++) {
+	    strcat(targ, " ");
+	    print_rgb_x(cstr, user_color[i]);
+	    strcat(targ, cstr);
+	}
+    }
+
+    return targ;
+}
+
+/* we use this mechanism with gnuplot 4.2 and higher */
+
+void write_plot_line_styles (int ptype, FILE *fp)
+{
+    char cstr[8];
+    int i;
+    
+    if (frequency_plot_code(ptype)) {
+	print_rgb_hash(cstr, &user_color[BOXCOLOR]);
+	fprintf(fp, "set style line 1 lc rgb \"%s\"\n", cstr);
+	fputs("set style line 2 lc rgb \"#000000\"\n", fp);
+    } else {
+	for (i=0; i<BOXCOLOR; i++) {
+	    print_rgb_hash(cstr, &user_color[i]);
+	    fprintf(fp, "set style line %d lc rgb \"%s\"\n", i+1, cstr);
+	}
+    }
+
+    fputs("set style increment user\n", fp);
+}
+
 /* end colors apparatus */
 
+/* Get gnuplot to print the dimensions of a PNG plot, in terms
+   of both pixels and data bounds, if gnuplot supports this.
+*/
+
+void print_plot_bounding_box_request (FILE *fp)
+{
+    fprintf(fp, "set print '%sgretltmp.png.bounds'\n", gretl_user_dir());
+    fputs("print \"pixel_bounds: \", TERM_XMIN, TERM_XMAX, "
+	  "TERM_YMIN, TERM_YMAX\n", fp);
+    fputs("print \"data_bounds: \", GPVAL_X_MIN, GPVAL_X_MAX, "
+	  "GPVAL_Y_MIN, GPVAL_Y_MAX\n", fp);
+}
+
+static void do_plot_bounding_box (void)
+{
+    FILE *fp = fopen(gretl_plotfile(), "a");
+
+    if (fp != NULL) {
+	print_plot_bounding_box_request(fp);
+	fclose(fp);
+    }
+}
+
 /**
  * get_gretl_png_term_line:
  * @ptype: indication of the sort of plot to be made, which
@@ -573,21 +799,39 @@ write_gnuplot_font_string (char *fstr, const char *grfont, PlotType ptype)
 const char *get_gretl_png_term_line (PlotType ptype, GptFlags flags)
 {
     static char png_term_line[256];
+    char truecolor_string[12] = {0};
     char font_string[128];
     char size_string[16];
     char color_string[64];
-    int gpcolors = 1, gpttf = 1;
+    int gpcolors, gpttf = 1, gpsize = 1;
+    int pngterm = 0;
     const char *grfont = NULL;
 
     *font_string = 0;
     *size_string = 0;
     *color_string = 0;
 
-#ifndef WIN32
-    gpcolors = gnuplot_has_specified_colors();
+    pngterm = gnuplot_png_terminal();
+
+#ifdef WIN32
+    gpcolors = RGB_LINE_COLOR;
+#else
+    if (gnuplot_has_rgb()) {
+	gpcolors = RGB_LINE_COLOR;
+    } else if (pngterm == GP_PNG_OLD) {
+	gpcolors = OLD_PNG_COLOR;
+    } else {
+	gpcolors = GD_PNG_COLOR;
+    }
+
     gpttf = gnuplot_has_ttf(0);
+    gpsize = gnuplot_has_size();
 #endif
 
+    if (pngterm == GP_PNG_GD2 && gnuplot_png_use_aa) {
+	strcpy(truecolor_string, " truecolor");
+    }    
+
     /* plot font setup */
     if (gpttf) {
 	grfont = gretl_png_font();
@@ -595,39 +839,43 @@ const char *get_gretl_png_term_line (PlotType ptype, GptFlags flags)
 	    grfont = getenv("GRETL_PNG_GRAPH_FONT");
 	}
 	if (grfont != NULL && *grfont != 0) {
-	    write_gnuplot_font_string(font_string, grfont, ptype);
+	    write_gnuplot_font_string(font_string, grfont, ptype, pngterm);
 	}
+    } 
+
+#ifndef WIN32
+    if (!gpttf) {
+	write_old_gnuplot_font_string(font_string, ptype);
     }
+#endif
 
     /* plot color setup */
-    if (gpcolors) {
-	int i;
-
-	/* background etc. */
-	strcpy(color_string, " xffffff x000000 x202020");
-
-	if (frequency_plot_code(ptype)) {
-	    strcat(color_string, " ");
-	    strcat(color_string, graph_palette[BOXCOLOR]);
-	    strcat(color_string, " x000000");
-	} else {
-	    for (i=0; i<BOXCOLOR; i++) {
-		strcat(color_string, " ");
-		strcat(color_string, graph_palette[i]);
-	    }
-	}
+    if (gpcolors == GD_PNG_COLOR) {
+	make_png_colorspec(color_string, ptype);
+    } else if (gpcolors == OLD_PNG_COLOR) {
+	strcpy(color_string, " color");
     } else {
-	strcpy(color_string, " color"); /* old PNG driver */
+	/* handled via styles */
+	*color_string = '\0';
     }
 
-    if (flags & GPT_LETTERBOX) {
-	strcpy(size_string, " size 680,400");
-    } else if (ptype == PLOT_VAR_ROOTS) {
-	strcpy(size_string, " size 480,480");
+    if (gpsize) {
+	if (flags & GPT_LETTERBOX) {
+	    strcpy(size_string, " size 680,400");
+	} else if (ptype == PLOT_VAR_ROOTS) {
+	    strcpy(size_string, " size 480,480");
+	}
     }
 
-    sprintf(png_term_line, "set term png%s%s%s",
-	    font_string, size_string, color_string);
+    if (pngterm == GP_PNG_CAIRO) {
+	sprintf(png_term_line, "set term pngcairo%s%s",
+		font_string, size_string);
+	strcat(png_term_line, "\nset encoding utf8"); /* FIXME? */
+    } else {
+	sprintf(png_term_line, "set term png%s%s%s%s",
+		truecolor_string, font_string, size_string, 
+		color_string);
+    }
 
 #if GP_DEBUG
     fprintf(stderr, "png term line:\n'%s'\n", png_term_line);
@@ -688,20 +936,6 @@ const char *get_gretl_emf_term_line (PlotType ptype, int color)
 	png_font_to_emf(grfont, emf_term_line);
     }
 
-    if (color && gnuplot_has_specified_emf_colors()) {
-	if (frequency_plot_code(ptype)) {
-	    strcat(emf_term_line, graph_palette[BOXCOLOR]);
-	    strcat(emf_term_line, " x000000");
-	} else {
-	    int i;
-
-	    for (i=0; i<BOXCOLOR; i++) {
-		strcat(emf_term_line, graph_palette[i]);
-		strcat(emf_term_line, " ");
-	    }
-	}
-    } 
-
     return emf_term_line;
 }
 
@@ -716,10 +950,11 @@ const char *get_gretl_emf_term_line (PlotType ptype, int color)
 
 PlotType plot_type_from_string (const char *str)
 {
-    int i, ret = PLOT_REGULAR;
+    int i, len, ret = PLOT_REGULAR;
 
     for (i=1; i<PLOT_TYPE_MAX; i++) {
-	if (!strcmp(str + 2, ptinfo[i].pstr)) {
+	len = strlen(ptinfo[i].pstr);
+	if (!strncmp(str + 2, ptinfo[i].pstr, len)) {
 	    ret = ptinfo[i].ptype;
 	    break;
 	}
@@ -781,6 +1016,10 @@ static int real_gnuplot_init (PlotType ptype, int flags, FILE **fpp)
 
     write_plot_type_string(ptype, *fpp);
 
+    if (gnuplot_has_rgb()) {
+	write_plot_line_styles(ptype, *fpp);
+    }
+
 #if GP_DEBUG
     fprintf(stderr, "gnuplot_init: set plotfile = '%s'\n", 
 	    plotfile);
@@ -825,7 +1064,7 @@ static int html_recode_gnuplot_file (const char *fname)
     char rname[FILENAME_MAX];
     int ttf = 0;
 
-    fp = fopen(fname, "r");
+    fp = gretl_fopen(fname, "r");
     if (fp == NULL) {
 	return 1;
     }
@@ -833,7 +1072,7 @@ static int html_recode_gnuplot_file (const char *fname)
     strcpy(rname, fname);
     strcat(rname, "l2");
 
-    fq = fopen(rname, "w");
+    fq = gretl_fopen(rname, "w");
     if (fq == NULL) {
 	fclose(fp);
 	return 1;
@@ -885,7 +1124,8 @@ int gnuplot_make_graph (void)
     int err = 0;
 
 #ifdef ENABLE_NLS  
-    if (iso_latin_version() == 2 && gnuplot_has_ttf(0)) {
+    if (iso_latin_version() == 2 && gnuplot_png_terminal() != GP_PNG_CAIRO && 
+	gnuplot_has_ttf(0)) {
 # if GP_DEBUG
 	fprintf(stderr, "gnuplot_make_graph: calling recode_gnuplot_file()\n");
 # endif
@@ -893,6 +1133,11 @@ int gnuplot_make_graph (void)
     } 
 #endif
 
+    if (gretl_in_gui_mode() && gnuplot_png_terminal() == GP_PNG_CAIRO) {
+	/* FIXME? */
+	do_plot_bounding_box();
+    }
+
 #ifdef WIN32
     sprintf(plotcmd, "\"%s\" \"%s\"", gretl_gnuplot_path(), gretl_plotfile());
     err = winfork(plotcmd, NULL, SW_SHOWMINIMIZED, 0);
@@ -958,11 +1203,7 @@ static void make_gtitle (gnuplot_info *gi, int code,
     }
 
     if (*title != '\0') {
-	if (strchr(s1, '\'') || strchr(s1, '\'')) {
-	    fprintf(gi->fp, "set title \"%s\"\n", title);
-	} else {
-	    fprintf(gi->fp, "set title '%s'\n", title);
-	}
+	fprintf(gi->fp, "set title \"%s\"\n", title);
     }
 }
 
@@ -1170,13 +1411,13 @@ static int get_fitted_line (gnuplot_info *gi,
 				 &y, &X);
 
     if (!err) {
-	b = gretl_vector_alloc(k);
+	b = gretl_column_vector_alloc(k);
 	if (b == NULL) {
 	    err = E_ALLOC;
 	}
     }
 
-    if (gi->fit == PLOT_FIT_NONE) {
+    if (f == PLOT_FIT_OLS) {
 	V = gretl_matrix_alloc(2, 2);
 	if (V == NULL) {
 	    err = E_ALLOC;
@@ -1193,7 +1434,7 @@ static int get_fitted_line (gnuplot_info *gi,
 	if (gi->fit == PLOT_FIT_NONE) {
 	    double v = gretl_matrix_get(V, 1, 1);
 	    int T = gretl_vector_get_length(y);
-	    double pv = t_pvalue_2(c[1] / sqrt(v), T - k);
+	    double pv = student_pvalue_2(c[1] / sqrt(v), T - k);
 
 	    if (pv < .10) {
 		sprintf(title, "Y = %#.3g %c %#.3gX", b->val[0],
@@ -1527,10 +1768,10 @@ static void set_lwstr (const DATAINFO *pdinfo, int v, char *s)
 {
     int w = var_get_linewidth(pdinfo, v);
 
-    if (w > 0) {
+    if (w > 1) {
 	sprintf(s, " lw %d", w);
     } else {
-	strcpy(s, " lw 1");
+	*s = '\0';
     }
 }
 
@@ -1552,12 +1793,13 @@ static void graph_list_adjust_sample (int *list,
     int t1min = ginfo->t1;
     int t2max = ginfo->t2;
     int t_ok;
-    int i, t;
+    int i, t, vi;
 
     for (t=t1min; t<=t2max; t++) {
 	t_ok = 0;
 	for (i=1; i<=list[0]; i++) {
-	    if (!na(Z[list[i]][t])) {
+	    vi = list[i];
+	    if (vi > 0 && !na(Z[vi][t])) {
 		t_ok = 1;
 		break;
 	    }
@@ -1571,7 +1813,8 @@ static void graph_list_adjust_sample (int *list,
     for (t=t2max; t>t1min; t--) {
 	t_ok = 0;
 	for (i=1; i<=list[0]; i++) {
-	    if (!na(Z[list[i]][t])) {
+	    vi = list[i];
+	    if (vi > 0 && !na(Z[vi][t])) {
 		t_ok = 1;
 		break;
 	    }
@@ -1586,8 +1829,9 @@ static void graph_list_adjust_sample (int *list,
 	for (i=1; i<=list[0]; i++) {
 	    int all_missing = 1;
 
+	    vi = list[i];
 	    for (t=t1min; t<=t2max; t++) {
-		if (!na(Z[list[i]][t])) {
+		if (!na(Z[vi][t])) {
 		    all_missing = 0;
 		    break;
 		}
@@ -1610,7 +1854,7 @@ static int maybe_add_plotx (gnuplot_info *gi,
     int add0 = 0;
 
     /* are we really doing a time-series plot? */
-    if (!strcmp(pdinfo->varname[gi->list[k]], "time")) {
+    if (k > 1 && !strcmp(pdinfo->varname[gi->list[k]], "time")) {
 	; /* yes */
     } else if (gi->flags & GPT_IDX) {
 	add0 = 1; /* yes */
@@ -1643,7 +1887,84 @@ static int maybe_add_plotx (gnuplot_info *gi,
     return 0;
 }
 
-#define ts_letterbox 1
+void gnuplot_missval_string (FILE *fp)
+{
+    if (gnuplot_uses_datafile_missing()) {
+	fputs("set datafile missing \"?\"\n", fp);
+    } else {
+	fputs("set missing \"?\"\n", fp);
+    }
+}
+
+static void graph_month_name (char *mname, int m)
+{
+    struct tm mt;
+
+    mt.tm_sec = 0;
+    mt.tm_min = 0;
+    mt.tm_hour = 0;
+    mt.tm_mday = 1;
+    mt.tm_mon = m - 1;
+    mt.tm_year = 100;
+
+    strftime(mname, 7, "%b", &mt);
+    mname[4] = '\0';
+}
+
+/* for short daily time-series plots: write month names
+   into the xtics */
+
+static void make_named_month_tics (gnuplot_info *gi, double yrs, PRN *prn)
+{
+    double t0 = gi->x[gi->t1];
+    double t1 = gi->x[gi->t2];
+    double x, tw = 1.0/12;
+    int i, m, n = 0;
+    char mname[8];
+    int notfirst = 0;
+    int scale = (int) (yrs * 1.5);
+
+    if (scale == 0) {
+	scale = 1;
+    }
+
+    t0 += (1.0 - (t0 - floor(t0)) * 12.0) / 12.0;
+    for (x=t0; x<t1; x+=tw) n++;
+
+    x = (t0 - floor(t0)) * 12;
+    m = 1 + ((x - floor(x) > .8)? ceil(x) : floor(x));
+    if (m > 12) m -= 12;
+
+    pputs(prn, "# literal lines = 1\n"); 
+    pputs(prn, "set xtics ("); 
+    x = t0;
+
+    gretl_push_c_numeric_locale();
+
+    for (i=0; i<n; i++) {
+	if (m == 1) {
+	    if (notfirst) {
+		pputs(prn, ", ");
+	    }
+	    pprintf(prn, "\"%4.0f\" %.8g", x, x);
+	    notfirst = 1;
+	} else if ((scale == 1) || (m % scale == 1)) {
+	    graph_month_name(mname, m);
+	    if (notfirst) {
+		pputs(prn, ", ");
+	    }
+	    pprintf(prn, "\"%s\" %.8g", mname, x);
+	    notfirst = 1;
+	}
+	m++;
+	x += tw;
+	if (m > 12) m -= 12;
+    }
+
+    gretl_pop_c_numeric_locale();
+
+    pputs(prn, ")\n");
+}
 
 /**
  * gnuplot:
@@ -1674,7 +1995,7 @@ int gnuplot (const int *plotlist, const char *literal,
     char keystr[48] = {0};
     char fit_line[128] = {0};
     int oddman = 0;
-    int toomany = 0;
+    int many = 0;
     int i, err = 0;
 
     gnuplot_info gi;
@@ -1699,8 +2020,8 @@ int gnuplot (const int *plotlist, const char *literal,
 	return loess_plot(&gi, Z, pdinfo);
     }
 
-    if (gi.list[0] > MAX_PLOT_LINES + 1) {
-	toomany = 1;
+    if (gi.list[0] > MAX_LETTERBOX_LINES + 1) {
+	many = 1;
     }
 
     err = maybe_add_plotx(&gi, pdinfo);
@@ -1751,30 +2072,48 @@ int gnuplot (const int *plotlist, const char *literal,
 	}
     } 
 
-    /* special tics for short time series plots */
+    /* special tics for time series plots */
     if (gi.flags & GPT_TS) {
-	if (toomany) {
-	    pprintf(prn, "# multiple timeseries %d", pdinfo->pd);
+	if (many) {
+	    pprintf(prn, "# multiple timeseries %d\n", pdinfo->pd);
 	} else {
+	    int gpsize = 1;
+
+#ifndef WIN32
+	    gpsize = gnuplot_has_size();
+#endif
 	    pprintf(prn, "# timeseries %d", pdinfo->pd);
-	}
-	if (ts_letterbox) {
-	    gi.flags |= GPT_LETTERBOX;
-	    pputs(prn, " (letterbox)\n");
-	} else {
-	    pputc(prn, '\n');
-	}
+	    if (gpsize) {
+		gi.flags |= GPT_LETTERBOX;
+		pputs(prn, " (letterbox)\n");
+	    } else {
+		pputc(prn, '\n');
+	    }
+	} 
 	if (pdinfo->pd == 4 && (gi.t2 - gi.t1) / 4 < 8) {
 	    pputs(prn, "set xtics nomirror 0,1\n"); 
 	    pputs(prn, "set mxtics 4\n");
-	}
-	if (pdinfo->pd == 12 && (gi.t2 - gi.t1) / 12 < 8) {
+	} else if (pdinfo->pd == 12 && (gi.t2 - gi.t1) / 12 < 8) {
 	    pputs(prn, "set xtics nomirror 0,1\n"); 
 	    pputs(prn, "set mxtics 12\n");
+	} else if (dated_daily_data(pdinfo)) {
+	    double yrs = (gi.t2 - gi.t1 + 1.0) / (pdinfo->pd * 52.0);
+
+	    if (yrs <= 3) {
+		make_named_month_tics(&gi, yrs, prn);
+	    } else if (yrs < 6) {
+		/* don't show ugly "fractions of years" */
+		pputs(prn, "set xtics 1\n");
+		if (yrs < 3) {
+		    /* put monthly minor tics */
+		    pputs(prn, "set mxtics 12\n");
+		} else if (yrs < 5) {
+		    /* quarterly minor tics */
+		    pputs(prn, "set mxtics 4\n");
+		}
+	    }
 	}
-    } else if (toomany) {
-	pputs(prn, "# multiple data series\n");
-    }
+    } 
 
     /* open file and dump the prn into it: we delaying writing
        the file header till we know a bit more about the plot
@@ -1791,7 +2130,7 @@ int gnuplot (const int *plotlist, const char *literal,
 
     print_axis_label('x', xlabel, fp);
     fputs("set xzeroaxis\n", fp); 
-    fputs("set missing \"?\"\n", fp);
+    gnuplot_missval_string(fp);
 
     if (list[0] == 2) {
 	/* only two variables */
@@ -1828,7 +2167,7 @@ int gnuplot (const int *plotlist, const char *literal,
 	print_axis_label('y', var_get_graph_name(pdinfo, list[2]), fp);
     } 
 
-    if (toomany) {
+    if (many) {
 	strcpy(keystr, "set key outside\n");
     }
 
@@ -2152,7 +2491,7 @@ maybe_add_surface (const int *list, double ***pZ, DATAINFO *pdinfo,
     smod = lsq(olslist, pZ, pdinfo, OLS, OPT_A);
 
     if (!smod.errcode && !na(smod.fstt) &&
-	(f_cdf_comp(smod.fstt, smod.dfn, smod.dfd) < .10 || (opt & OPT_F))) {
+	(snedecor_cdf_comp(smod.fstt, smod.dfn, smod.dfd) < .10 || (opt & OPT_F))) {
 	double uadj = (umax - umin) * 0.02;
 	double vadj = (vmax - vmin) * 0.02;
 
@@ -2220,7 +2559,7 @@ int gnuplot_3d (int *list, const char *literal,
     print_axis_label('y', var_get_graph_name(pdinfo, list[1]), fq);
     print_axis_label('z', var_get_graph_name(pdinfo, list[3]), fq);
 
-    fputs("set missing \"?\"\n", fq);
+    gnuplot_missval_string(fq);
 
     if (literal != NULL && *literal != 0) {
 	print_gnuplot_literal_lines(literal, fq);
@@ -2576,7 +2915,7 @@ int plot_fcast_errs (int t1, int t2, const double *obs,
 
     gretl_pop_c_numeric_locale();
 
-    fputs("set missing \"?\"\n", fp);
+    gnuplot_missval_string(fp);
 
     if (time_series) {
 	fprintf(fp, "# timeseries %d\n", time_series);
