@@ -1,20 +1,20 @@
-/* gretl - The Gnu Regression, Econometrics and Time-series Library
- * Copyright (C) 1999-2000 Ramu Ramanathan and Allin Cottrell
- *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License 
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
- *
- * This software is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License 
- * along with this software; if not, write to the 
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+/* 
+ *  gretl -- Gnu Regression, Econometrics and Time-series Library
+ *  Copyright (C) 2001 Allin Cottrell and Riccardo "Jack" Lucchetti
+ * 
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ * 
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ * 
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 /* graphing.h for gretl */
@@ -46,12 +46,22 @@ typedef enum {
     GPT_PNG_OUTPUT     = 1 << 16, /* output is to PNG file */
     GPT_ALL_MARKERS    = 1 << 17, /* all observation markers displayed */
     GPT_ALL_MARKERS_OK = 1 << 18, /* OK to show all observation markers */
-    GPT_LETTERBOX      = 1 << 19  /* special format for time series graphs */
+    GPT_LETTERBOX      = 1 << 19, /* special format for time series graphs */
+    GPT_PARAMETRIC     = 1 << 20, /* gnuplot should be in parametric mode */
+    GPT_XZEROAXIS      = 1 << 21, /* show x = 0 line */
+    GPT_YZEROAXIS      = 1 << 22  /* show y = 0 line */
 } GptFlags; 
 
+typedef struct gretlRGB_ gretlRGB;
+
+struct gretlRGB_ {
+    unsigned char r;
+    unsigned char g;
+    unsigned char b;
+};
+
 #define MAXTITLE 128
 #define MAX_PLOT_LABELS 3
-#define MAX_PLOT_LINES 8
 #define N_GP_COLORS 4
 #define BOXCOLOR (N_GP_COLORS - 1)
 
@@ -79,6 +89,7 @@ typedef enum {
     PLOT_MULTI_IRF,
     PLOT_PANEL,
     PLOT_BI_GRAPH,
+    PLOT_MANY_TS,
     PLOT_TYPE_MAX
 } PlotType;
 
@@ -103,6 +114,20 @@ typedef enum {
     GP_TERM_PLT
 } TermType;
 
+typedef enum {
+    GP_PNG_NONE,  /* no PNG terminal available */
+    GP_PNG_OLD,   /* old driver from gnuplot 3.N */
+    GP_PNG_GD1,   /* libgd driver, no truecolor option */
+    GP_PNG_GD2,   /* libgd with truecolor option */
+    GP_PNG_CAIRO  /* newer cairo-based driver */
+} PNGTerm;
+
+typedef enum {
+    GP_PDF_NONE,
+    GP_PDF_PDFLIB,
+    GP_PDF_CAIRO
+} PDFTerm;
+
 #define frequency_plot_code(c) (c == PLOT_FREQ_SIMPLE || \
 				c == PLOT_FREQ_NORMAL || \
 				c == PLOT_FREQ_GAMMA)
@@ -118,10 +143,16 @@ const char *gp_justification_string (int j);
 
 const char *gnuplot_label_front_string (void);
 
+void gnuplot_missval_string (FILE *fp);
+
 int gnuplot_init (PlotType ptype, FILE **fpp);
 
 int write_plot_type_string (PlotType ptype, FILE *fp);
 
+void write_plot_line_styles (int ptype, FILE *fp);
+
+void print_plot_bounding_box_request (FILE *fp);
+
 PlotType plot_type_from_string (const char *str);
 
 int gnuplot_make_graph (void);
@@ -181,21 +212,31 @@ int is_auto_fit_string (const char *s);
 
 int gnuplot_has_ttf (int reset);
 
-int gnuplot_has_pdf (void);
+int gnuplot_pdf_terminal (void);
 
-int gnuplot_has_specified_colors (void);
+int gnuplot_png_terminal (void);
+
+int gnuplot_has_rgb (void);
 
 int gnuplot_has_style_fill (void);
 
-void set_graph_palette (int i, const char *colstr);
+int gnuplot_has_latin5 (void);
+
+void set_graph_palette (int i, gretlRGB color);
+
+void set_graph_palette_from_string (int i, const char *cstr);
 
 void graph_palette_reset (int i);
 
-void print_palette_string (char *targ);
+void print_rgb_hash (char *s, const gretlRGB *color);
 
-const char *graph_color_string (int i);
+void print_palette_string (char *s);
+
+const gretlRGB *get_graph_color (int i);
 
 int gnuplot_test_command (const char *cmd);
 
+void gnuplot_png_set_use_aa (int s);
+
 #endif /* GRAPHING_H */
 
