@@ -1,20 +1,20 @@
-/*
- *  Copyright (c) by Ramu Ramanathan and Allin Cottrell
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
- *
- *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
- *
+/* 
+ *  gretl -- Gnu Regression, Econometrics and Time-series Library
+ *  Copyright (C) 2001 Allin Cottrell and Riccardo "Jack" Lucchetti
+ * 
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ * 
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ * 
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 /* gretl_commands.c */
@@ -33,7 +33,6 @@ struct gretl_cmd {
 static struct gretl_cmd gretl_cmds[] = {
     { SEMIC,    ";" },     
     { ADD,      "add" },
-    { ADDOBS,   "addobs" },
     { ADDTO,    "addto" },
     { ADF,      "adf" }, 
     { APPEND,   "append" },
@@ -53,6 +52,7 @@ static struct gretl_cmd gretl_cmds[] = {
     { CRITERIA, "criteria" },
     { CUSUM,    "cusum" },
     { DATA,     "data" },
+    { DATAMOD,  "dataset" },
     { DELEET,   "delete" },
     { DIFF,     "diff" },
     { DIFFTEST, "difftest" },
@@ -78,6 +78,7 @@ static struct gretl_cmd gretl_cmds[] = {
     { GRAPH,    "graph" }, 
     { HAUSMAN,  "hausman" },
     { HCCM,     "hccm" }, 
+    { HECKIT,   "heckit" },
     { HELP,     "help" },    
     { HILU,     "hilu" },    
     { HSK,      "hsk" }, 
@@ -147,7 +148,6 @@ static struct gretl_cmd gretl_cmds[] = {
     { TABPRINT, "tabprint" },
     { TESTUHAT, "testuhat" },
     { TOBIT,    "tobit" },
-    { TRANSPOSE, "transpos" },
     { TSLS,     "tsls" },    
     { VAR,      "var" },
     { VARLIST,  "varlist" },
