@@ -1,29 +1,28 @@
-/* gretl - The Gnu Regression, Econometrics and Time-series Library
- * Copyright (C) 1999-2000 Ramu Ramanathan and Allin Cottrell
- *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License 
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
- *
- * This software is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License 
- * along with this software; if not, write to the 
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+/* 
+ *  gretl -- Gnu Regression, Econometrics and Time-series Library
+ *  Copyright (C) 2001 Allin Cottrell and Riccardo "Jack" Lucchetti
+ * 
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ * 
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ * 
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 #ifndef COMMANDS_H
 #define COMMANDS_H
 
 typedef enum {
-    SEMIC,    
+    SEMIC = 0,    
     ADD,
-    ADDOBS,
     ADDTO,
     ADF,
     APPEND,
@@ -43,6 +42,7 @@ typedef enum {
     CRITERIA,
     CUSUM,
     DATA,
+    DATAMOD,
     DELEET,
     DIFF,
     DIFFTEST,
@@ -68,6 +68,7 @@ typedef enum {
     GRAPH,
     HAUSMAN,
     HCCM,    
+    HECKIT,
     HELP,    
     HILU,    
     HSK,
@@ -137,7 +138,6 @@ typedef enum {
     TABPRINT,
     TESTUHAT,
     TOBIT,
-    TRANSPOSE,
     TSLS,    
     VAR,
     VARLIST,
@@ -160,7 +160,6 @@ typedef enum {
 #define TEXTSAVE_OK(c) (c == ADD || \
                         c == ADDTO || \
                         c == ADF || \
-                        c == ARCH || \
                         c == CHOW || \
                         c == COEFFSUM || \
                         c == COINT || \
@@ -218,6 +217,13 @@ typedef enum {
                               c == LMTEST || \
                               c == TESTUHAT)
 
+#define USES_BFGS(c) ((c) == ARMA || \
+                      (c) == GARCH || \
+                      (c) == GMM || \
+		      (c) == HECKIT || \
+                      (c) == MLE || \
+                      (c) == TOBIT)
+
 int gretl_command_number (const char *s);
 
 const char *gretl_command_word (int i);
