@@ -1,24 +1,26 @@
-/*
- *  Copyright (c) by Ramu Ramanathan and Allin Cottrell
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
- *
- *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- *
+/* 
+ *  gretl -- Gnu Regression, Econometrics and Time-series Library
+ *  Copyright (C) 2001 Allin Cottrell and Riccardo "Jack" Lucchetti
+ * 
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ * 
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ * 
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 #include "libgretl.h"
 
+#include <errno.h>
+
 char gretl_errmsg[ERRLEN];
 
 static const char *gretl_error_messages[] = {
@@ -27,11 +29,9 @@ static const char *gretl_error_messages[] = {
     N_("Data error"),                                            /* E_DATA = 2 */
     N_("Exact or near collinearity encountered"),                /* E_SINGULAR */
     N_("Insufficient degrees of freedom for regression"),        /* E_DF */
-    N_("Y-prime * Y equals zero"),                               /* E_YPY */
     N_("Dependent variable is all zeros, aborting regression"),  /* E_ZERO */
     N_("Total sum of squares was not positive"),                 /* E_TSS */
     N_("Sum of squared residuals negative!"),                    /* E_ESS */
-    N_("Unbalanced parentheses in genr command"),                /* E_UNBAL */
     N_("Sorry, command not available for this estimator"),       /* E_NOTIMP */
     N_("Unspecified error -- FIXME"),                            /* E_UNSPEC */
     N_("Syntax error in genr formula"),                          /* E_SYNTAX */
@@ -40,17 +40,12 @@ static const char *gretl_error_messages[] = {
     N_("Out of memory error"),                                   /* E_ALLOC */
     N_("No formula supplied in genr"),                           /* E_EQN */
     N_("Unknown variable name in command"),                      /* E_UNKVAR */
-    N_("The observations specified for the regression "
-       "exceed those in the data set"),                          /* E_NODATA */
     N_("Command has insufficient arguments"),                    /* E_ARGS */
     N_("This command is implemented only for OLS models"),       /* E_OLSONLY */
     N_("Invalid argument for function"),                         /* E_INVARG */
-    N_("Invalid sample split for Chow test"),                    /* E_SPLIT */
     N_("Syntax error in command line"),                          /* E_PARSE */
     N_("No independent variables left after omissions"),         /* E_NOVARS */
     N_("No independent variables were omitted"),                 /* E_NOOMIT */
-    N_("Can't do this: some vars in original model "
-       "have been redefined"),                                   /* E_VARCHANGE */
     N_("No new independent variables were added"),               /* E_NOADD */
     N_("One or more \"added\" vars were already present"),       /* E_ADDDUP */
     N_("Error generating logarithms"),                           /* E_LOGS */
@@ -58,13 +53,8 @@ static const char *gretl_error_messages[] = {
     N_("Error generating lagged variables"),                     /* E_LAGS */
     N_("Attempting to take square root of negative number"),     /* E_SQRT */
     N_("Excessive exponent in genr formula"),                    /* E_HIGH */
-    N_("Weight variable is all zeros, aborting regression"),     /* E_WTZERO */
-    N_("Weight variable contains negative values"),              /* E_WTNEG */
     N_("Need valid starting and ending observations"),           /* E_OBS */
     N_("You must include a constant in this sort of model"),     /* E_NOCONST */
-    N_("There were missing observations for the added "
-       "variable(s).\nReset the sample and rerun the original "
-       "regression first"),                                      /* E_MISS */
     N_("The statistic you requested is not available"),          /* E_BADSTAT */
     N_("Missing sub-sample information; can't merge data"),      /* E_NOMERGE */
     N_("The convergence criterion was not met"),                 /* E_NOCONV */
@@ -75,6 +65,8 @@ static const char *gretl_error_messages[] = {
     N_("Data types not conformable for operation"),              /* E_TYPES */
     N_("Wrong data type"),                                       /* E_DATATYPE */
     N_("Incompatible options"),                                  /* E_BADOPT */
+    N_("The restrictions do not identify the parameters"),       /* E_NOIDENT */
+    N_("External command failed"),                               /* E_EXTERNAL */
     NULL,                                                        /* E_DB_DUP */
     NULL,                                                        /* E_OK */
     NULL                                                         /* E_MAX */
@@ -179,6 +171,28 @@ void gretl_errmsg_set (const char *str)
     }
 }
 
+/**
+ * gretl_errmsg_set_from_errno:
+ *
+ * If %gretl_errmsg is currently blank, copy the string 
+ * returned by %strerror into the message space; or if the 
+ * error message is not blank but sufficient space remains, 
+ * append the new error info to the message.
+ */
+
+void gretl_errmsg_set_from_errno (void)
+{
+    char *msg = NULL;
+
+    if (errno) {
+	msg = strerror(errno);
+    }
+
+    if (msg != NULL) {
+	gretl_errmsg_set(msg);
+    }
+}
+
 /**
  * gretl_error_clear:
  *
