@@ -1,20 +1,20 @@
-/*
- *  Copyright (c) 2005 by Allin Cottrell
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
- *
- *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- *
+/* 
+ *  gretl -- Gnu Regression, Econometrics and Time-series Library
+ *  Copyright (C) 2001 Allin Cottrell and Riccardo "Jack" Lucchetti
+ * 
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ * 
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ * 
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 #include "libgretl.h"
@@ -25,13 +25,14 @@
 #include "usermat.h"
 #include "gretl_xml.h"
 #include "cmd_private.h"
+#include "gretl_string_table.h"
 
 #define FN_DEBUG 0
 #define PKG_DEBUG 0
 #define EXEC_DEBUG 0
+#define UDEBUG 0
 
 typedef struct fn_param_ fn_param;
-typedef struct fncall_ fncall;
 
 struct fn_param_ {
     char *name;
@@ -54,6 +55,7 @@ struct ufunc_ {
     int rettype;
     char *retname;
     int *in_use;
+    fnargs *args;
 };
 
 struct fnpkg_ {
@@ -71,15 +73,29 @@ struct fnpkg_ {
     int n_priv;
 };
 
+struct fnarg {
+    int type;
+    char *upname;  /* name of arg at caller level */
+    union {
+	int idnum;
+	double x;
+	double *px;
+	gretl_matrix *m;
+	user_matrix *um;
+	char *str;
+    } val;
+};
+
+struct fnargs_ {
+    int argc;
+    struct fnarg **arg;
+};
+
 enum {
     ARG_OPTIONAL = 1 << 0,
     ARG_CONST    = 1 << 1
 };
 
-#define ref_type(t) (t == ARG_REF_SCALAR || \
-		     t == ARG_REF_SERIES || \
-		     t == ARG_REF_MATRIX)
-
 static int n_ufuns;
 static ufunc **ufuns;
 static ufunc *current_ufun;
@@ -99,7 +115,6 @@ static int function_package_remove_by_ID (int ID);
 
 static int compiling;
 static int fn_executing;
-static int fn_comment;
 
 int gretl_compiling_function (void)
 {
@@ -108,7 +123,6 @@ int gretl_compiling_function (void)
 
 static void set_compiling_on (void)
 {
-    fn_comment = 0;
     compiling = 1;
 }
 
@@ -122,6 +136,113 @@ int gretl_function_depth (void)
     return fn_executing;
 }
 
+/* handling of function arguments */
+
+struct fnarg *fn_arg_new (int type, void *p, int *err)
+{
+    struct fnarg *arg;
+
+    arg = malloc(sizeof *arg);
+    if (arg == NULL) {
+	*err = E_ALLOC;
+	return NULL;
+    }
+
+    arg->type = type;
+    arg->upname = NULL;
+    
+    if (type == GRETL_TYPE_NONE) {
+	arg->val.x = 0;
+    } else if (type == GRETL_TYPE_DOUBLE) {
+	arg->val.x = *(double *) p;
+    } else if (type == GRETL_TYPE_SERIES) {
+	arg->val.px = (double *) p;
+    } else if (type == GRETL_TYPE_MATRIX) {
+	arg->val.m = (gretl_matrix *) p;
+    } else if (type == GRETL_TYPE_LIST ||
+	       type == GRETL_TYPE_STRING) {
+	arg->val.str = (char *) p;
+    } else if (type == GRETL_TYPE_SCALAR_REF ||
+	       type == GRETL_TYPE_SERIES_REF ||
+	       type == GRETL_TYPE_UVAR) {
+	arg->val.idnum = * (int *) p;
+    } else if (type == GRETL_TYPE_MATRIX_REF) {
+	arg->val.um = (user_matrix *) p;
+    } else {
+	*err = E_TYPES;
+	free(arg);
+	arg = NULL;
+    }
+
+    return arg;
+}
+
+fnargs *fn_args_new (void)
+{
+    fnargs *args = malloc(sizeof *args);
+
+    if (args == NULL) {
+	return NULL;
+    }
+
+    args->argc = 0;
+    args->arg = NULL;
+
+    return args;
+}
+
+/* note: this is not supposed to be a "deep free" (in case the arg
+   carries a pointer member); that is handled in geneval.c 
+*/
+
+static void free_fn_arg (struct fnarg *arg)
+{
+    free(arg->upname);
+    free(arg);
+}
+
+void fn_args_free (fnargs *args)
+{
+    int i;
+
+    for (i=0; i<args->argc; i++) {
+	free_fn_arg(args->arg[i]);
+    }
+    
+    free(args->arg);
+    free(args);
+}
+
+int push_fn_arg (fnargs *args, int type, void *p)
+{
+    int err = 0;
+
+#if 0
+    fprintf(stderr, "push_fn_arg: starting on type %d\n", type);
+#endif
+
+    if (args == NULL) {
+	err = E_DATA;
+    } else {
+	struct fnarg **arg;
+	int n = args->argc + 1;
+
+	arg = realloc(args->arg, n * sizeof *arg);
+	if (arg == NULL) {
+	    err = E_ALLOC;
+	} else {
+	    args->arg = arg;
+	    args->arg[n-1] = fn_arg_new(type, p, &err);
+	}
+
+	if (!err) {
+	    args->argc = n;
+	}
+    }
+
+    return err;
+}
+
 /* Switch to delete, or not, all the "private" variables internal to a
    function when execution terminates.  This is used in nls.c: if a
    user function will be called repeatedly in the context of
@@ -250,14 +371,14 @@ int fn_param_optional (const ufunc *fun, int i)
 
     t = fun->params[i].type;
 
-    return ((ref_type(t) || t == ARG_LIST) && 
+    return ((gretl_ref_type(t) || t == GRETL_TYPE_LIST) && 
 	    (fun->params[i].flags & ARG_OPTIONAL));
 }
 
 int user_func_get_return_type (const ufunc *fun)
 {
     if (fun == NULL) {
-	return ARG_NONE;
+	return GRETL_TYPE_NONE;
     } else {
 	return fun->rettype;
     }
@@ -331,10 +452,34 @@ int current_func_pkgID (void)
     return 0;
 }
 
+static ufunc *currently_called_function (void)
+{
+    ufunc *fun;
+    int i, f0;
+
+    for (i=0; i<n_ufuns; i++) {
+	fun = ufuns[i];
+	if (fun->in_use != NULL) {
+	    f0 = fun->in_use[0];
+	    if (fun->in_use[f0] == fn_executing) {
+		return fun;
+	    }
+	}
+    }
+
+    return NULL;
+}
+
 ufunc *get_user_function_by_name (const char *name)
 {
-    int i, ID = current_func_pkgID();
     ufunc *fun = NULL;
+    int i, ID;
+
+    if (n_ufuns == 0) {
+	return NULL;
+    }
+
+    ID = current_func_pkgID();
 
     /* if we're currently exec'ing a function from a package,
        look first for functions from the same package */
@@ -418,10 +563,11 @@ static ufunc *ufunc_new (void)
     fun->n_params = 0;
     fun->params = NULL;
 
-    fun->rettype = ARG_NONE;
+    fun->rettype = GRETL_TYPE_NONE;
     fun->retname = NULL;
 
     fun->in_use = NULL;
+    fun->args = NULL;
 
     return fun;
 }
@@ -452,10 +598,11 @@ static void clear_ufunc_data (ufunc *fun)
     fun->n_lines = 0;
     fun->n_params = 0;
 
-    fun->rettype = ARG_NONE;
+    fun->rettype = GRETL_TYPE_NONE;
     fun->retname = NULL;
     
     fun->in_use = NULL;
+    fun->args = NULL;
 }
 
 static void free_ufunc (ufunc *fun)
@@ -527,64 +674,82 @@ enum {
     FUNCS_CODE
 };
 
-static const char *arg_type_string (int type)
+static const char *arg_type_string (int t)
 {
-    if (type == ARG_BOOL) {
-	return "bool";
-    } else if (type == ARG_INT) {
-	return "int";
-    } else if (type == ARG_SCALAR) {
-	return "scalar";
-    } else if (type == ARG_SERIES) {
-	return "series";
-    } else if (type == ARG_LIST) {
-	return "list";
-    } else if (type == ARG_MATRIX) {
-	return "matrix";
-    } else if (type == ARG_REF_SCALAR) {
-	return "scalar *";
-    } else if (type == ARG_REF_SERIES) {
-	return "series *";
-    } else if (type == ARG_REF_MATRIX) {
-	return "matrix *";
+    switch (t) {
+    case GRETL_TYPE_BOOL:       return "bool";
+    case GRETL_TYPE_INT:        return "int";
+    case GRETL_TYPE_DOUBLE:     return "scalar";
+    case GRETL_TYPE_SERIES:     return "series";
+    case GRETL_TYPE_LIST:       return "list";
+    case GRETL_TYPE_MATRIX:     return "matrix";
+    case GRETL_TYPE_SCALAR_REF: return "scalar *";
+    case GRETL_TYPE_SERIES_REF: return "series *";
+    case GRETL_TYPE_MATRIX_REF: return "matrix *";
+    case GRETL_TYPE_STRING:     return "string";	
+    }
+
+    return "unknown";
+}
+
+static const char *arg_type_xml_string (int t)
+{
+    if (t == GRETL_TYPE_SCALAR_REF) {
+	return "scalarref";
+    } else if (t == GRETL_TYPE_SERIES_REF) {
+	return "seriesref";
+    } else if (t == GRETL_TYPE_MATRIX_REF) {
+	return "matrixref";
     } else {
-	return "unknown";
+	return arg_type_string(t);
     }
 }
 
-static const char *arg_type_xml_string (int type)
-{
-    if (type == ARG_REF_SCALAR) {
-	return "scalarref";
-    } else if (type == ARG_REF_SERIES) {
-	return "seriesref";
-    } else if (type == ARG_REF_MATRIX) {
-	return "matrixref";
-    } else {
-	return arg_type_string(type);
-    }
-}
+/* FIXME updating and placement of these function */
 
 static int arg_type_from_string (const char *s)
 {
-    if (!strncmp(s, "bool", 4)) return ARG_BOOL;
-    if (!strcmp(s, "int"))      return ARG_INT;
-    if (!strcmp(s, "scalar"))   return ARG_SCALAR;
-    if (!strcmp(s, "series"))   return ARG_SERIES;
-    if (!strcmp(s, "list"))     return ARG_LIST;
-    if (!strcmp(s, "matrix"))   return ARG_MATRIX;
+    if (!strncmp(s, "bool", 4)) return GRETL_TYPE_BOOL;
+    if (!strcmp(s, "int"))      return GRETL_TYPE_INT;
+    if (!strcmp(s, "scalar"))   return GRETL_TYPE_DOUBLE;
+    if (!strcmp(s, "series"))   return GRETL_TYPE_SERIES;
+    if (!strcmp(s, "list"))     return GRETL_TYPE_LIST;
+    if (!strcmp(s, "matrix"))   return GRETL_TYPE_MATRIX;
 
-    if (!strcmp(s, "scalar *"))  return ARG_REF_SCALAR;
-    if (!strcmp(s, "series *"))  return ARG_REF_SERIES;
-    if (!strcmp(s, "matrix *"))  return ARG_REF_MATRIX;
+    if (!strcmp(s, "scalar *"))  return GRETL_TYPE_SCALAR_REF;
+    if (!strcmp(s, "series *"))  return GRETL_TYPE_SERIES_REF;
+    if (!strcmp(s, "matrix *"))  return GRETL_TYPE_MATRIX_REF;
 
-    if (!strcmp(s, "scalarref"))  return ARG_REF_SCALAR;
-    if (!strcmp(s, "seriesref"))  return ARG_REF_SERIES;
-    if (!strcmp(s, "matrixref"))  return ARG_REF_MATRIX;
+    if (!strcmp(s, "scalarref"))  return GRETL_TYPE_SCALAR_REF;
+    if (!strcmp(s, "seriesref"))  return GRETL_TYPE_SERIES_REF;
+    if (!strcmp(s, "matrixref"))  return GRETL_TYPE_MATRIX_REF;
+
+    if (!strcmp(s, "string")) return GRETL_TYPE_STRING;
 
     return 0;
 }
 
+/* backward compatibility for nasty old numeric type references */
+
+static int arg_type_from_int (const char *s)
+{
+    int i = atoi(s);
+
+    switch (i) {
+    case 1: return GRETL_TYPE_DOUBLE;
+    case 2: return GRETL_TYPE_SERIES;
+    case 3: return GRETL_TYPE_LIST;
+    case 4: return GRETL_TYPE_MATRIX;
+    case 5: return GRETL_TYPE_BOOL;
+    case 6: return GRETL_TYPE_INT;
+    case 7: return GRETL_TYPE_SCALAR_REF;
+    case 8: return GRETL_TYPE_SERIES_REF;
+    case 9: return GRETL_TYPE_MATRIX_REF;
+    }
+
+    return GRETL_TYPE_NONE;
+}
+
 static int field_to_type (const char *s)
 {
 #if FN_DEBUG
@@ -592,14 +757,12 @@ static int field_to_type (const char *s)
 #endif
 
     if (isdigit(*s)) {
-	return atoi(s);
+	return arg_type_from_int(s);
     } else {
 	return arg_type_from_string(s);
     }
 }    
 
-#define scalar_arg(t) (t == ARG_BOOL || t == ARG_INT || t == ARG_SCALAR)
-
 static int func_read_params (xmlNodePtr node, ufunc *fun)
 {
     xmlNodePtr cur;
@@ -636,11 +799,11 @@ static int func_read_params (xmlNodePtr node, ufunc *fun)
 	    if (gretl_xml_get_prop_as_string(cur, "type", &field)) {
 		fun->params[n].type = field_to_type(field);
 		free(field);
-		if (scalar_arg(fun->params[n].type)) {
+		if (gretl_scalar_type(fun->params[n].type)) {
 		    gretl_xml_get_prop_as_double(cur, "default", 
 						 &fun->params[n].deflt);
 		}
-		if (fun->params[n].type == ARG_INT) {
+		if (fun->params[n].type == GRETL_TYPE_INT) {
 		    gretl_xml_get_prop_as_double(cur, "min", 
 						 &fun->params[n].min);
 		    gretl_xml_get_prop_as_double(cur, "max", 
@@ -779,14 +942,14 @@ static void print_function_start (ufunc *fun, PRN *prn)
 	} else {
 	    pprintf(prn, "%s %s", s, fun->params[i].name);
 	}
-	if (fun->params[i].type == ARG_BOOL) {
+	if (fun->params[i].type == GRETL_TYPE_BOOL) {
 	    if (!na(fun->params[i].deflt)) {
 		pprintf(prn, "[%g]", fun->params[i].deflt);
 	    }
-	} else if (scalar_arg(fun->params[i].type)) {
+	} else if (gretl_scalar_type(fun->params[i].type)) {
 	    print_deflt_min_max(&fun->params[i], prn);
-	} else if (ref_type(fun->params[i].type) || 
-		   fun->params[i].type == ARG_LIST) {
+	} else if (gretl_ref_type(fun->params[i].type) || 
+		   fun->params[i].type == GRETL_TYPE_LIST) {
 	    print_opt_flags(&fun->params[i], prn);
 	}
 	if (i == fun->n_params - 1) {
@@ -804,7 +967,7 @@ static void print_function_end (ufunc *fun, PRN *prn)
 {
     const char *typestr;
 
-    if (fun->rettype != ARG_NONE) {
+    if (fun->rettype != GRETL_TYPE_NONE) {
 	typestr = arg_type_string(fun->rettype);
 	pprintf(prn, "  return %s %s\n", typestr, fun->retname);
     }
@@ -933,27 +1096,36 @@ static int read_ufunc_from_xml (xmlNodePtr node, xmlDocPtr doc, fnpkg *pkg)
     return err;
 }
 
+static int wordmatch (const char *line, const char *s)
+{
+    int n = strlen(s);
+    
+    return (strncmp(line, s, n) == 0 && 
+	    (line[n] == '\0' || isspace(line[n])));
+}
+
 static void adjust_indent (const char *line, int *this_indent,
 			   int *next_indent)
 {
     int ti = *next_indent;
     int ni = *next_indent;
 
-    if (!strncmp(line, "loop", 4)) {
+    if (wordmatch(line, "loop")) {
 	ni++;
-    } else if (!strncmp(line, "if", 2)) {
+    } else if (wordmatch(line, "if")) {
 	ni++;
-    } else if (!strncmp(line, "nls", 3)) {
+    } else if (wordmatch(line, "nls")) {
 	ni++;
-    } else if (!strncmp(line, "mle", 3)) {
+    } else if (wordmatch(line, "mle")) {
 	ni++;
-    } else if (!strncmp(line, "gmm", 3)) {
+    } else if (wordmatch(line, "gmm")) {
 	ni++;
-    } else if (!strncmp(line, "end", 3)) {
+    } else if (wordmatch(line, "end") ||
+	       wordmatch(line, "endloop")) {
 	ti--;
 	ni--;
-    } else if (!strncmp(line, "else", 4) ||
-	       !strncmp(line, "elif", 4)) {
+    } else if (wordmatch(line, "else") ||
+	       wordmatch(line, "elif")) {
 	ni = ti;
 	ti--;
     } 
@@ -1006,7 +1178,7 @@ static int write_function_xml (const ufunc *fun, FILE *fp)
 	gretl_pop_c_numeric_locale();
     }
 
-    if (fun->rettype != ARG_NONE) {
+    if (fun->rettype != GRETL_TYPE_NONE) {
 	fprintf(fp, " <return name=\"%s\" type=\"%s\"/>\n", fun->retname, 
 		arg_type_string(fun->rettype));
     }
@@ -2134,53 +2306,6 @@ char *get_function_file_header (const char *fname, char **pver,
     return descrip;
 }
 
-static char *function_name_from_line (const char *line, char *name)
-{
-    if (*line == '#' || !strncmp(line, "(*", 2)) {
-	*name = '\0';
-    } else {
-	const char *p = strchr(line, '=');
-	char *q;
-
-	if (p == NULL) {
-	    p = line;
-	} else {
-	    p++;
-	}
-
-	sscanf(p, "%31s", name);
-	q = strchr(name, '(');
-	if (q != NULL) {
-	    *q = 0;
-	}
-#if FN_DEBUG
-	fprintf(stderr, "function_name_from_line: line='%s', got '%s'\n", 
-		line, name);
-#endif
-    }
-
-    return name;
-}
-
-int gretl_is_user_function (const char *line)
-{
-    int ret = 0;
-
-    if (n_ufuns > 0 && !string_is_blank(line)) {
-	char name[FN_NAMELEN];
-
-#if FN_DEBUG > 1
-	fprintf(stderr, "gretl_is_user_function: testing '%s'\n", line);
-#endif
-	function_name_from_line(line, name);
-	if (get_user_function_by_name(name) != NULL) {
-	    ret = 1;
-	}
-    }
-
-    return ret;
-}
-
 int gretl_is_public_user_function (const char *name)
 {
     ufunc *fun = get_user_function_by_name(name);
@@ -2192,27 +2317,6 @@ int gretl_is_public_user_function (const char *name)
     }
 }
 
-int gretl_get_user_function (const char *line)
-{
-    int ret = 0;
-
-    if (n_ufuns > 0 && !string_is_blank(line)) {
-	char name[FN_NAMELEN];
-
-#if FN_DEBUG > 1
-	fprintf(stderr, "gretl_is_user_function: testing '%s'\n", line);
-#endif
-	function_name_from_line(line, name);
-	if (get_user_function_by_name(name) != NULL) {
-	    ret = 1;
-	} else if (function_from_string(name)) {
-	    ret = 1;
-	}
-    }
-
-    return ret;
-}
-
 static void delete_ufunc_from_list (ufunc *fun)
 {
     if (n_ufuns == 0 || fun == NULL) {
@@ -2313,7 +2417,7 @@ static int read_deflt_min_max (char *s, fn_param *param,
     if (p != NULL) {
 	double x, y, z;
 
-	if (param->type == ARG_BOOL) {
+	if (param->type == GRETL_TYPE_BOOL) {
 	    if (sscanf(p, "[%lf]", &x) == 1) {
 		param->deflt = x;
 	    } else {
@@ -2406,12 +2510,12 @@ static int parse_function_param (char *s, fn_param *param, int i)
 	return err;
     }    
 
-    if (scalar_arg(type)) {
+    if (gretl_scalar_type(type)) {
 	param->type = type;
 	err = read_deflt_min_max(s, param, &len);
     }
 
-    if (ref_type(type) || type == ARG_LIST) {
+    if (gretl_ref_type(type) || type == GRETL_TYPE_LIST) {
 	param->type = type;
 	err = read_param_option(s, param, &len);
     }    
@@ -2603,7 +2707,7 @@ static int add_function_return (ufunc *fun, const char *line)
     int type;
     int err = 0;
 
-    if (fun->rettype != ARG_NONE) {
+    if (fun->rettype != GRETL_TYPE_NONE) {
 	sprintf(gretl_errmsg, "Function %s: return value is already defined",
 		fun->name);
 	return 1;
@@ -2614,6 +2718,12 @@ static int add_function_return (ufunc *fun, const char *line)
     }
 
     type = field_to_type(s1);
+
+#if FN_DEBUG
+    fprintf(stderr, "add_function_return: s1='%s', s2='%s'\n", s1, s2);
+    fprintf(stderr, "field_to_type on '%s' gives %d\n", s1, type);
+#endif
+
     if (type == 0) {
 	return E_PARSE;
     } 
@@ -2672,16 +2782,6 @@ static int real_function_append_line (const char *line, ufunc *fun)
 	return 0;
     }
 
-    if (!strncmp(line, "/*", 2)) {
-	fn_comment = 1;
-    } else if (!strncmp(line, "*/", 2)) {
-	fn_comment = 0;
-    }
-
-    if (fn_comment) {
-	return strings_array_add(&fun->lines, &fun->n_lines, line);
-    }
-
     if (end_of_function(line)) {
 	if (fun->n_lines == 0) {
 	    sprintf(gretl_errmsg, "%s: empty function", fun->name);
@@ -2750,7 +2850,7 @@ int update_function_from_script (const char *fname, int idx)
 	return E_DATA;
     }
 
-    fp = fopen(fname, "r");
+    fp = gretl_fopen(fname, "r");
     if (fp == NULL) {
 	return E_FOPEN;
     }
@@ -2835,6 +2935,7 @@ static int localize_list (const char *oldname, fn_param *fp,
 			  DATAINFO *pdinfo)
 {
     const int *list;
+    int level = fn_executing + 1;
     int i, err;
 
     list = get_list_by_name(oldname);
@@ -2845,10 +2946,15 @@ static int localize_list (const char *oldname, fn_param *fp,
 	err = copy_named_list_as(oldname, fp->name);
     }
 
+#if UDEBUG
+    fprintf(stderr, "localize_list: localizing '%s' as '%s'\n",
+	    oldname, fp->name);
+#endif
+
     if (!err) {
 	for (i=1; i<=list[0]; i++) {
 	    if (list[i] != 0) {
-		STACK_LEVEL(pdinfo, list[i]) += 1;
+		STACK_LEVEL(pdinfo, list[i]) = level;
 		if (fp->flags & ARG_CONST) {
 		    set_var_const(pdinfo, list[i]);
 		}
@@ -2873,7 +2979,7 @@ static int add_scalar_arg_default (fn_param *param, double ***pZ,
 	return E_DATA;
     }
 
-    if (param->type == ARG_BOOL || param->type == ARG_INT) {
+    if (param->type == GRETL_TYPE_BOOL || param->type == GRETL_TYPE_INT) {
 	x = floor(param->deflt);
     } else {
 	x = param->deflt;
@@ -2883,69 +2989,85 @@ static int add_scalar_arg_default (fn_param *param, double ***pZ,
 }
 
 static int allocate_function_args (ufunc *fun,
-				   int argc, fnargs *args, 
+				   fnargs *args, 
 				   double ***pZ,
 				   DATAINFO *pdinfo)
 {
+    struct fnarg *arg;
     fn_param *fp;
-    int xi = 0, Xi = 0, Mi = 0, li = 0;
     int i, err = 0;
 
-    /* regular arguments, passed by value */
-    
-    for (i=0; i<fun->n_params && !err; i++) {
+    for (i=0; i<args->argc && !err; i++) {
+	arg = args->arg[i];
 	fp = &fun->params[i];
-	if (scalar_arg(fp->type)) {
-	    if (i >= argc || args->types[i] == ARG_NONE) {
+
+	if (gretl_scalar_type(fp->type)) {
+	    if (arg->type == GRETL_TYPE_UVAR) {
+		err = dataset_copy_variable_as(arg->val.idnum, fp->name,
+					       pZ, pdinfo);
+	    } else if (arg->type == GRETL_TYPE_NONE) {
 		err = add_scalar_arg_default(fp, pZ, pdinfo);
 	    } else {
-		err = dataset_add_scalar_as(args->x[xi++], fp->name, 
+		err = dataset_add_scalar_as(arg->val.x, fp->name, 
 					    pZ, pdinfo);
-	    } 
-	} else if (fp->type == ARG_SERIES) {
-	    err = dataset_add_series_as(args->X[Xi++], fp->name, 
-					pZ, pdinfo);
-	} else if (fp->type == ARG_MATRIX) {
-	    err = copy_matrix_as(args->M[Mi++], fp->name);
-	} else if (fp->type == ARG_LIST) {
-	    if (i >= argc || args->types[i] == ARG_NONE) {
+	    }
+	} else if (fp->type == GRETL_TYPE_SERIES) {
+	    if (arg->type == GRETL_TYPE_UVAR) {
+		err = dataset_copy_variable_as(arg->val.idnum, fp->name,
+					       pZ, pdinfo);
+	    } else {
+		err = dataset_add_series_as(arg->val.px, fp->name, 
+					    pZ, pdinfo);
+	    }	    
+	} else if (fp->type == GRETL_TYPE_MATRIX) {
+	    err = copy_matrix_as(arg->val.m, fp->name);
+	} else if (fp->type == GRETL_TYPE_LIST) {
+	    if (arg->type == GRETL_TYPE_NONE) {
 		err = create_named_null_list(fp->name);
 	    } else {
-		err = localize_list(args->lists[li++], fp, pdinfo);
-	    } 
-	} 
+		err = localize_list(arg->val.str, fp, pdinfo);
+	    }
+	} else if (fp->type == GRETL_TYPE_STRING) {
+	    err = add_string_as(arg->val.str, fp->name);
+	} else if (fp->type == GRETL_TYPE_SCALAR_REF ||
+		   fp->type == GRETL_TYPE_SERIES_REF) {
+	    int v = arg->val.idnum;
+
+	    arg->upname = gretl_strdup(pdinfo->varname[v]);
+	    if (arg->upname == NULL) {
+		err = E_ALLOC;
+	    } else {
+		STACK_LEVEL(pdinfo, v) += 1;
+		strcpy(pdinfo->varname[v], fp->name);
+	    }
+	} else if (fp->type == GRETL_TYPE_MATRIX_REF) {
+	    user_matrix *u = arg->val.um;
+
+	    arg->upname = gretl_strdup(user_matrix_get_name(u));
+	    if (arg->upname == NULL) {
+		err = E_ALLOC;
+	    } else {	    
+		user_matrix_adjust_level(u, 1);
+		user_matrix_set_name(u, fp->name);
+	    }
+	}
+
+	if (arg->type == GRETL_TYPE_UVAR && !err) {
+	    arg->upname = gretl_strdup(pdinfo->varname[arg->val.idnum]);
+	}
     }
 
-    xi = Mi = 0;
+    /* now for any parameters withut matching arguments */
 
-    /* "pointer" parameters, passed by reference: make these vars
-       visible at function level, under their parameter names
-    */
-
-    for (i=0; i<argc && !err; i++) {
+    for (i=args->argc; i<fun->n_params && !err; i++) {
 	fp = &fun->params[i];
-	if (ref_type(fp->type)) {
-	    if (args->types[i] == ARG_REF_SCALAR ||
-		args->types[i] == ARG_REF_SERIES) {
-		err = strings_array_add(&args->upnames, &args->nnames, 
-					pdinfo->varname[args->refv[xi]]);
-		if (!err) {
-		    STACK_LEVEL(pdinfo, args->refv[xi]) += 1;
-		    strcpy(pdinfo->varname[args->refv[xi]], fp->name);
-		    xi++;
-		}
-	    } else if (args->types[i] == ARG_REF_MATRIX) {
-		err = strings_array_add(&args->upnames, &args->nnames, 
-					user_matrix_get_name(args->refm[Mi]));
-		if (!err) {
-		    user_matrix_adjust_level(args->refm[Mi], 1);
-		    user_matrix_set_name(args->refm[Mi], fp->name);
-		    Mi++;
-		}
-	    } 
-	} 
+	if (gretl_scalar_type(fp->type)) {
+	    err = add_scalar_arg_default(fp, pZ, pdinfo);
+	} else if (fp->type == GRETL_TYPE_LIST) {
+	    err = create_named_null_list(fp->name);
+	}
     }
-
+    
     return err;
 }
 
@@ -3070,12 +3192,17 @@ get_matrix_return (const char *mname, int action, int *err)
     return ret;
 }
 
-static int unlocalize_list (const char *listname, DATAINFO *pdinfo)
+static int unlocalize_list (const char *lname, DATAINFO *pdinfo)
 {
-    const int *list = get_list_by_name(listname);
+    const int *list = get_list_by_name(lname);
     int d = gretl_function_depth();
     int i, vi, err = 0;
 
+#if UDEBUG
+    fprintf(stderr, "unlocalize_list: %s, d = %d\n", lname, d);
+    printlist(list, lname);
+#endif	    
+
     if (list == NULL) {
 	err = E_DATA;
     } else {
@@ -3115,7 +3242,7 @@ static void
 maybe_set_return_description (ufunc *u, int rtype, DATAINFO *pdinfo, 
 			      char **descrip)
 {
-    if (rtype == ARG_SCALAR || rtype == ARG_SERIES) {
+    if (rtype == GRETL_TYPE_DOUBLE || rtype == GRETL_TYPE_SERIES) {
 	int v = varindex(pdinfo, u->retname);
 
 	if (v < pdinfo->v) {
@@ -3125,24 +3252,24 @@ maybe_set_return_description (ufunc *u, int rtype, DATAINFO *pdinfo,
 }
 
 static int 
-function_assign_returns (ufunc *u, fnargs *args, int argc, int rtype, 
+function_assign_returns (ufunc *u, fnargs *args, int rtype, 
 			 double **Z, DATAINFO *pdinfo, 
 			 void *ret, char **descrip, PRN *prn, 
 			 int *perr)
 {
+    struct fnarg *arg;
     fn_param *fp;
-    int vi = 0, mi = 0, li = 0;
-    int i, j, err = 0;
+    int i, err = 0;
 
     if (*perr == 0) {
 	/* direct return value */
-	if (rtype == ARG_SCALAR) {
+	if (rtype == GRETL_TYPE_DOUBLE) {
 	    *(double *) ret = get_scalar_return(u->retname, Z, pdinfo, &err);
-	} else if (rtype == ARG_SERIES) {
+	} else if (rtype == GRETL_TYPE_SERIES) {
 	    *(double **) ret = get_series_return(u->retname, Z, pdinfo, GET_COPY, &err);
-	} else if (rtype == ARG_MATRIX) {
+	} else if (rtype == GRETL_TYPE_MATRIX) {
 	    *(gretl_matrix **) ret = get_matrix_return(u->retname, GET_COPY, &err);
-	} else if (rtype == ARG_LIST) {
+	} else if (rtype == GRETL_TYPE_LIST) {
 	    *(char **) ret = get_list_return(u->retname, pdinfo, &err);
 	}
 
@@ -3162,24 +3289,24 @@ function_assign_returns (ufunc *u, fnargs *args, int argc, int rtype,
        function bombed
     */
 
-    j = 0;
-    for (i=0; i<argc; i++) {
+    for (i=0; i<args->argc; i++) {
+	arg = args->arg[i];
 	fp = &u->params[i];
-	if (ref_type(fp->type)) {
-	    if (args->types[i] == ARG_REF_SCALAR ||
-		args->types[i] == ARG_REF_SERIES) {
-		STACK_LEVEL(pdinfo, args->refv[vi]) -= 1;
-		strcpy(pdinfo->varname[args->refv[vi]], args->upnames[j++]);
-		vi++;
-	    } else if (args->types[i] == ARG_REF_MATRIX) {
-		user_matrix_adjust_level(args->refm[mi], -1);
-		user_matrix_set_name(args->refm[mi], args->upnames[j++]);
-		mi++;
+	if (gretl_ref_type(fp->type)) {
+	    if (arg->type == GRETL_TYPE_SCALAR_REF ||
+		arg->type == GRETL_TYPE_SERIES_REF) {
+		int v = arg->val.idnum;
+
+		STACK_LEVEL(pdinfo, v) -= 1;
+		strcpy(pdinfo->varname[v], arg->upname);
+	    } else if (arg->type == GRETL_TYPE_MATRIX_REF) {
+		user_matrix *u = arg->val.um;
+
+		user_matrix_adjust_level(u, -1);
+		user_matrix_set_name(u, arg->upname);
 	    }
-	} else if (fp->type == ARG_LIST) {
-	    if (li < args->nl) {
-		unlocalize_list(args->lists[li++], pdinfo);
-	    } 
+	} else if (fp->type == GRETL_TYPE_LIST) {
+	    unlocalize_list(fp->name, pdinfo);
 	}
     }
 
@@ -3198,6 +3325,8 @@ static int stop_fncall (ufunc *u, double ***pZ, DATAINFO *pdinfo,
 	    "function '%s' at depth %d\n", u->name, d);
 #endif
 
+    u->args = NULL;
+
     anyerr = destroy_saved_lists_at_level(d);
     if (anyerr && !err) {
 	err = anyerr;
@@ -3214,6 +3343,14 @@ static int stop_fncall (ufunc *u, double ***pZ, DATAINFO *pdinfo,
 #endif
     }  
 
+    anyerr = destroy_saved_strings_at_level(d);
+    if (anyerr && !err) {
+	err = anyerr;
+#if FN_DEBUG
+	fprintf(stderr, "destroy_saved_strings_at_level(%d): err = %d\n", d, err);
+#endif
+    }  
+
     /* below: delete some or all local variables, taking care not to
        delete any 'local' vars that have been "promoted" to caller
        level via their inclusion in a returned list
@@ -3251,7 +3388,7 @@ static int stop_fncall (ufunc *u, double ***pZ, DATAINFO *pdinfo,
 
 	for (i=0; i<u->n_params && !err; i++) {
 	    fp = &u->params[i];
-	    if (scalar_arg(fp->type) || fp->type == ARG_SERIES) {
+	    if (gretl_scalar_type(fp->type) || fp->type == GRETL_TYPE_SERIES) {
 		v = varindex(pdinfo, fp->name);
 		if (STACK_LEVEL(pdinfo, v) == d) {
 		    anyerr = dataset_drop_variable(v, pZ, pdinfo);
@@ -3269,9 +3406,10 @@ static int stop_fncall (ufunc *u, double ***pZ, DATAINFO *pdinfo,
     return err;
 }
 
-static void start_fncall (ufunc *u)
+static void start_fncall (ufunc *u, fnargs *args)
 {
     set_executing_on(u);
+    u->args = args;
     push_program_state();
     set_gretl_echo(0);
     set_gretl_messages(0);
@@ -3293,6 +3431,117 @@ static void set_funcerr_message (ufunc *u, const char *s)
     strncat(funcerr_msg, s, 255 - n);
 }
 
+static void func_exec_callback (ExecState *s, double ***pZ,
+				DATAINFO *pdinfo)
+{
+    int ci = s->cmd->ci;
+
+    if (ci == VAR || ci == VECM) {
+	maybe_stack_var(s->var, s->cmd);
+    } else if (ci == END && !strcmp(s->cmd->param, "restrict")) {
+	maybe_stack_var(s->var, s->cmd);
+    }
+}
+
+static int uvar_scalar (struct fnarg *arg, const DATAINFO *pdinfo)
+{
+    if (arg->type == GRETL_TYPE_UVAR) {
+	int v = arg->val.idnum;
+
+	if (v >= 0 && v < pdinfo->v) {
+	    return var_is_scalar(pdinfo, v);
+	}
+    }
+
+    return 0;
+}
+
+static int uvar_series (struct fnarg *arg, const DATAINFO *pdinfo)
+{
+    if (arg->type == GRETL_TYPE_UVAR) {
+	int v = arg->val.idnum;
+
+	if (v >= 0 && v < pdinfo->v) {
+	    return var_is_series(pdinfo, v);
+	}
+    }
+
+    return 0;
+}
+
+static double arg_get_double_val (struct fnarg *arg,
+				  const double **Z)
+{
+    if (gretl_scalar_type(arg->type)) {
+	return arg->val.x;
+    } else if (arg->type == GRETL_TYPE_UVAR) {
+	return Z[arg->val.idnum][0];
+    } else {
+	return NADBL;
+    }
+}
+
+static int check_function_args (ufunc *u, fnargs *args, 
+				const double **Z,
+				const DATAINFO *pdinfo,
+				PRN *prn)
+{
+    struct fnarg *arg;
+    fn_param *fp;
+    double x;
+    int i, err = 0;
+
+    for (i=0; i<args->argc && !err; i++) {
+	arg = args->arg[i];
+	fp = &u->params[i];
+
+	if ((fp->flags & ARG_OPTIONAL) && arg->type == GRETL_TYPE_NONE) {
+	    ; /* this is OK */
+	} else if (gretl_scalar_type(fp->type) && arg->type == GRETL_TYPE_DOUBLE) {
+	    ; /* OK */
+	} else if (gretl_scalar_type(fp->type) && uvar_scalar(arg, pdinfo)) {
+	    ; /* OK */
+	} else if (fp->type == GRETL_TYPE_SERIES && uvar_series(arg, pdinfo)) {
+	    ; /* OK */
+	} else if (fp->type != arg->type) {
+	    pprintf(prn, "argv[%d] is of wrong type (got %s, should be %s)\n", 
+		    i, arg_type_string(arg->type), arg_type_string(fp->type));
+	    err = E_TYPES;
+	}
+
+	if (!err && fp->type == GRETL_TYPE_DOUBLE) {
+	    x = arg_get_double_val(arg, Z);
+	    if ((!na(fp->min) && x < fp->min) ||
+		(!na(fp->max) && x > fp->max)) {
+		pprintf(prn, "argv[%d]: scalar value %g out of bounds\n", i, x);
+		err = E_INVARG;
+	    }
+	}
+    }
+
+    for (i=args->argc; i<u->n_params && !err; i++) {
+	/* do we have defaults for any empty args? */
+	fp = &u->params[i];
+	if (!(fp->flags & ARG_OPTIONAL) && na(fp->deflt)) {
+	    pprintf(prn, "%s: not enough arguments\n", u->name);
+	    err = E_ARGS;
+	}
+    }
+
+    return err;
+}
+
+static void fn_state_init (CMD *cmd, ExecState *state)
+{
+    cmd->list = NULL;
+    cmd->param = NULL;
+    cmd->extra = NULL;
+    cmd->linfo = NULL;
+
+    state->cmd = NULL;
+    state->models = NULL;
+}
+
 int gretl_function_exec (ufunc *u, fnargs *args, int rtype,
 			 double ***pZ, DATAINFO *pdinfo,
 			 void *ret, char **descrip, 
@@ -3302,73 +3551,35 @@ int gretl_function_exec (ufunc *u, fnargs *args, int rtype,
     MODEL **models = NULL;
     char line[MAXLINE];
     CMD cmd;
-    fn_param *fp;
-    int started = 0;
-    int funcerr = 0;
-    int argc, i, j;
-    int err = 0;
-
     int orig_v = pdinfo->v;
     int orig_t1 = pdinfo->t1;
     int orig_t2 = pdinfo->t2;
-
+    int started = 0;
+    int i, err = 0;
+ 
     *funcerr_msg = '\0';
 
+#if FN_DEBUG
+    fprintf(stderr, "gretl_function_exec: starting\n");
+#endif
+
     err = maybe_check_function_needs(pdinfo, u);
     if (err) {
 	return err;
     }
 
-    /* precautions */
-    cmd.list = NULL;
-    cmd.param = NULL;
-    cmd.extra = NULL;
-    cmd.linfo = NULL;
-    state.cmd = NULL;
-    state.models = NULL;
-
-    argc = args->nx + args->nX + args->nM + args->nl +
-	args->nrefv + args->nrefm + args->nnull;
+    /* precaution */
+    fn_state_init(&cmd, &state);
 
 #if FN_DEBUG
-    fprintf(stderr, "gretl_function_exec: argc = %d\n", argc);
+    fprintf(stderr, "gretl_function_exec: argc = %d\n", args->argc);
     fprintf(stderr, "u->n_params = %d\n", u->n_params);
 #endif
 
-    j = 0;
-    for (i=0; i<argc && !err; i++) {
-	fp = &u->params[i];
-	if ((fp->flags & ARG_OPTIONAL) && args->types[i] == ARG_NONE) {
-	    ; /* this is OK */
-	} else if (scalar_arg(fp->type) && args->types[i] == ARG_SCALAR) {
-	    ; /* this is OK too */
-	} else if (fp->type != args->types[i]) {
-	    pprintf(prn, "argv[%d] is of wrong type (got %s, should be %s)\n", 
-		    i, arg_type_string(args->types[i]), 
-		    arg_type_string(fp->type));
-	    err = E_TYPES;
-	} else if (fp->type == ARG_SCALAR) {
-	    if ((!na(fp->min) && args->x[j] < fp->min) ||
-		(!na(fp->max) && args->x[j] > fp->max)) {
-		pprintf(prn, "argv[%d]: scalar value %g out of bounds\n", 
-			i, args->x[j]);
-		err = E_INVARG;
-	    }
-	    j++;
-	}
-    }
-
-    for (i=argc; i<u->n_params && !err; i++) {
-	/* do we have defaults for any empty args? */
-	fp = &u->params[i];
-	if (!(fp->flags & ARG_OPTIONAL) && na(fp->deflt)) {
-	    pprintf(prn, "%s: not enough arguments\n", u->name);
-	    err = E_ARGS;
-	}
-    }
+    err = check_function_args(u, args, (const double **) *pZ, pdinfo, prn);
 
     if (!err) {
-	err = allocate_function_args(u, argc, args, pZ, pdinfo);
+	err = allocate_function_args(u, args, pZ, pdinfo);
     }
 
     if (err) {
@@ -3392,10 +3603,11 @@ int gretl_function_exec (ufunc *u, fnargs *args, int rtype,
 	if (pdinfo->submode) {
 	    state.subinfo = pdinfo;
 	}
+	state.callback = func_exec_callback;
     }
 
     if (!err) {
-	start_fncall(u);
+	start_fncall(u, args);
 	started = 1;
     }
 
@@ -3403,17 +3615,31 @@ int gretl_function_exec (ufunc *u, fnargs *args, int rtype,
 
     for (i=0; i<u->n_lines && !err; i++) {
 	strcpy(line, u->lines[i]);
-	err = maybe_exec_line(&state, pZ, &pdinfo, &funcerr);
-	if (funcerr) {
+	err = maybe_exec_line(&state, pZ, &pdinfo);
+	if (state.funcerr) {
+#if UDEBUG
+	    fprintf(stderr, "funcerr: gretl_function_exec: i=%d, line: '%s'\n", 
+		    i, line);
+#endif
 	    pprintf(prn, "%s: %s\n", u->name, state.cmd->param);
 	    set_funcerr_message(u, state.cmd->param);
 	}
 	if (gretl_execute_loop()) { 
+#if UDEBUG
+	    fprintf(stderr, "gretl_function_exec: calling gretl_loop_exec\n");
+#endif
 	    err = gretl_loop_exec(&state, pZ, &pdinfo);
+	    if (state.funcerr) {
+		pprintf(prn, "%s: %s\n", u->name, state.cmd->param);
+		set_funcerr_message(u, state.cmd->param);
+	    }
 	    if (err) {
 		fprintf(stderr, "function_exec: breaking on error in loop\n");
 		break;
 	    }
+#if UDEBUG
+	    fprintf(stderr, "gretl_function_exec: gretl_loop_exec done, err = %d\n", err);
+#endif
 	}
     }
 
@@ -3432,7 +3658,7 @@ int gretl_function_exec (ufunc *u, fnargs *args, int rtype,
     pdinfo->t1 = orig_t1;
     pdinfo->t2 = orig_t2;
 
-    function_assign_returns(u, args, argc, rtype, *pZ, pdinfo,
+    function_assign_returns(u, args, rtype, *pZ, pdinfo,
 			    ret, descrip, prn, &err);
 
     gretl_exec_state_clear(&state);
@@ -3452,6 +3678,34 @@ int gretl_function_exec (ufunc *u, fnargs *args, int rtype,
     return err;    
 }
 
+/* look up name of supplied argument based on name of variable
+   inside function */
+
+char *gretl_func_get_arg_name (const char *argvar)
+{
+    ufunc *u = currently_called_function();
+    char *ret = NULL;
+
+    if (u != NULL && u->args != NULL) {
+	int i, n = u->args->argc;
+
+	for (i=0; i<n; i++) {
+	    if (!strcmp(argvar, u->params[i].name)) {
+		if (u->args->arg[i]->upname != NULL) {
+		    ret = gretl_strdup(u->args->arg[i]->upname); 
+		}
+		break;
+	    }
+	}
+    }
+
+    if (ret == NULL) {
+	ret = gretl_strdup("");
+    }
+
+    return ret;
+}
+
 void gretl_functions_cleanup (void)
 {
     ufuncs_destroy();
@@ -3486,7 +3740,7 @@ static void real_user_function_help (ufunc *fun, int ci, PRN *prn)
 	pputs(prn, "Parameters: none\n\n");
     }
 
-    if (fun->rettype != ARG_NONE) {
+    if (fun->rettype != GRETL_TYPE_NONE) {
 	pprintf(prn, "Return value: %s (%s)\n\n", 
 		fun->retname, arg_type_string(fun->rettype));
     } else {
