@@ -1,20 +1,20 @@
-/* gretl - The Gnu Regression, Econometrics and Time-series Library
- * Copyright (C) 1999-2004 Ramu Ramanathan and Allin Cottrell
- *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License 
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
- *
- * This software is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License 
- * along with this software; if not, write to the 
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+/* 
+ *  gretl -- Gnu Regression, Econometrics and Time-series Library
+ *  Copyright (C) 2001 Allin Cottrell and Riccardo "Jack" Lucchetti
+ * 
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ * 
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ * 
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 #ifndef GRETL_FUNC_H
@@ -40,21 +40,6 @@ typedef struct ufunc_ ufunc;
 typedef struct fnpkg_ fnpkg;
 typedef struct fnargs_ fnargs;
 
-struct fnargs_ {
-    char *types;
-    int nx, nX, nM, nl;
-    int nrefv, nrefm;
-    int nnull;
-    int nnames;
-    double *x;
-    double **X;
-    gretl_matrix **M;
-    char **lists;
-    int *refv;
-    user_matrix **refm;
-    char **upnames;
-};
-
 int n_free_functions (void);
 
 ufunc *get_user_function_by_name (const char *name);
@@ -95,17 +80,15 @@ int gretl_start_compiling_function (const char *line, PRN *prn);
 
 int gretl_function_append_line (const char *line);
 
-int gretl_is_user_function (const char *line);
-
 int gretl_is_public_user_function (const char *name);
 
-int gretl_get_user_function (const char *line);
-
 int gretl_function_exec (ufunc *u, fnargs *args, int rtype,
 			 double ***pZ, DATAINFO *pdinfo,
 			 void *ret, char **descrip,
 			 PRN *prn);
 
+char *gretl_func_get_arg_name (const char *argvar);
+
 void set_drop_function_vars (int s);
 
 const char *get_funcerr_message (void);
@@ -166,4 +149,10 @@ int user_function_help (const char *fnname, PRN *prn);
 
 void gretl_functions_cleanup (void);
 
+fnargs *fn_args_new (void);
+
+void fn_args_free (fnargs *args);
+
+int push_fn_arg (fnargs *args, int type, void *p);
+
 #endif /* GRETL_FUNC_H */
