@@ -1,20 +1,20 @@
-/*
- *  Copyright (c) 2005 by Allin Cottrell
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
- *
- *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
- *
+/* 
+ *  gretl -- Gnu Regression, Econometrics and Time-series Library
+ *  Copyright (C) 2001 Allin Cottrell and Riccardo "Jack" Lucchetti
+ * 
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ * 
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ * 
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 #include "libgretl.h"
@@ -111,7 +111,9 @@ int doing_nls (void)
 #endif
 }
 
+#ifdef ENABLE_NLS
 static int decpoint;
+#endif
 
 /**
  * reset_local_decpoint:
@@ -166,6 +168,8 @@ static int gretl_cset_min;
 # ifdef WIN32
 static int gretl_cpage;
 # endif
+static int using_utf8;
+
 
 /* Use g_get_charset() to determine the current local character set,
    and record this information.  If we get an "ISO-XXXX-Y" locale,
@@ -178,11 +182,12 @@ void set_gretl_charset (const char *s)
 {
     const char *charset = NULL;
     char gretl_charset[32];
-    int using_utf8 = 0;
 
     using_utf8 = g_get_charset(&charset);
+
     if (using_utf8) {
 	set_tex_use_utf(1);
+	set_fopen_use_utf8();
     }
 
     *gretl_charset = '\0';
@@ -241,15 +246,32 @@ static const char *get_gretl_charset (void)
 
 static const char *gnuplot_encoding_string (void)
 {
-    static char gp_enc[12];
+    static char gp_enc[12] = {0};
 
-    if (gretl_cset_maj == 8859 && 
-	(gretl_cset_min == 1 || 
-	 gretl_cset_min == 2 ||
-	 gretl_cset_min == 15)) {
-	sprintf(gp_enc, "iso_%d_%d", gretl_cset_maj, gretl_cset_min);
+    if (gretl_cset_maj != 8859) {
+	return NULL;
+    }
+
+    if (*gp_enc != '\0') {
 	return gp_enc;
-    } 
+    }
+
+    if (gretl_cset_min == 1 || 
+	gretl_cset_min == 2 ||
+	gretl_cset_min == 15) {
+	sprintf(gp_enc, "iso_8859_%d", gretl_cset_min);
+	return gp_enc;
+    } else if (gretl_cset_min == 9) {
+	static int l5 = -1;
+
+	if (l5 < 0) {
+	    l5 = gnuplot_has_latin5();
+	}
+	if (l5) {
+	    strcpy(gp_enc, "iso_8859_9");
+	    return gp_enc;
+	}
+    }
 
     return NULL;
 }
@@ -283,6 +305,7 @@ int iso_latin_version (void)
     if (gretl_cset_maj == 8859 &&
 	(gretl_cset_min == 1 || 
 	 gretl_cset_min == 2 ||
+	 gretl_cset_min == 9 ||
 	 gretl_cset_min == 15)) {
 	return gretl_cset_min;
     }
@@ -295,10 +318,14 @@ int iso_latin_version (void)
     }
 # endif
 
-    /* handle Polish UTF-8 locale? */
+    /* Polish, Turkish: UTF-8 locale? */
     lang = getenv("LANG");
-    if (lang != NULL && !strncmp(lang, "pl", 2)) {
-	return 2;
+    if (lang != NULL) {
+	if (!strncmp(lang, "pl", 2)) {
+	    return 2;
+	} else if (!strncmp(lang, "tr", 2)) {
+	    return 9;
+	}
     } 
 
     return 1;
@@ -312,6 +339,8 @@ static const char *get_gp_charset (void)
 #else
 	return "ISO-8859-2";
 #endif
+    } else if (iso_latin_version() == 9) {
+	return "ISO-8859-9";
     } else {
 	return "ISO-8859-1";
     }
@@ -322,6 +351,11 @@ char *gp_gettext (const char *msgid)
     static const char *cset;
     char *ret;
 
+    if (gnuplot_png_terminal() == GP_PNG_CAIRO) {
+	/* UTF-8 wanted */
+	return gettext(msgid);
+    }
+
     if (cset == NULL) {
 	cset = get_gp_charset();
     }
@@ -354,11 +388,7 @@ char *iso_gettext (const char *msgid)
 	/* not yet determined */
 	cset = get_gretl_charset();
 	fprintf(stderr, "get_gretl_charset gave %s\n", cset);
-	if (cset == NULL) {
-	    iso_ok = 0;
-	} else {
-	    iso_ok = 1;
-	}
+	iso_ok = (cset != NULL);
     }
 
     if (iso_ok) {
@@ -618,6 +648,27 @@ char *iso_to_ascii (char *s)
     return real_iso_to_ascii(s, 1);
 }
 
+char *get_month_name (char *mname, int m)
+{
+    struct tm mt;
+
+    mt.tm_sec = 0;
+    mt.tm_min = 0;
+    mt.tm_hour = 0;
+    mt.tm_mday = 1;
+    mt.tm_mon = m - 1;
+    mt.tm_year = 100;
+
+    strftime(mname, 7, "%b", &mt);
+    *mname = tolower(*mname);
+
+#ifdef ENABLE_NLS
+    real_iso_to_ascii(mname, gretl_cset_min);
+#endif
+
+    return mname;
+}
+
 #ifdef ENABLE_NLS
 
 struct l2sym {
@@ -1002,25 +1053,6 @@ int gretl_is_ascii (const char *buf)
     return 1;
 }
 
-char *get_month_name (char *mname, int m)
-{
-    struct tm mt;
-
-    mt.tm_sec = 0;
-    mt.tm_min = 0;
-    mt.tm_hour = 0;
-    mt.tm_mday = 1;
-    mt.tm_mon = m - 1;
-    mt.tm_year = 100;
-
-    strftime(mname, 7, "%b", &mt);
-    *mname = tolower(*mname);
-
-    real_iso_to_ascii(mname, gretl_cset_min);
-
-    return mname;
-}
-
 int get_utf_width (const char *str, int width)
 {
     width += strlen(str) - g_utf8_strlen(str, -1);
@@ -1050,12 +1082,9 @@ char *maybe_iso_gettext (const char *msgid)
 
 void check_for_console (PRN *prn)
 {
-    if (prn == NULL) return;
-
-    if (printing_to_standard_stream(prn)) {
-	printing_to_console = 1;
-    } else {
-	printing_to_console = 0;
+    if (prn != NULL) {
+	printing_to_console = 
+	    printing_to_standard_stream(prn);
     }
 }
 
@@ -1064,4 +1093,11 @@ void console_off (void)
     printing_to_console = 0;
 }
 
-#endif /* ENABLE_NLS */
+#else
+
+int gretl_is_ascii (const char *buf)
+{
+    return 1;
+}
+
+#endif /* !ENABLE_NLS */
