@@ -1,20 +1,20 @@
-/*
- *  Copyright (c) Allin Cottrell
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
- *
- *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- *
+/* 
+ *  gretl -- Gnu Regression, Econometrics and Time-series Library
+ *  Copyright (C) 2001 Allin Cottrell and Riccardo "Jack" Lucchetti
+ * 
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ * 
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ * 
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 #include "libgretl.h"
@@ -179,7 +179,7 @@ static int real_remember_list (const int *list, const char *name,
 	orig->list = gretl_list_copy(list);
 	if (orig->list == NULL) {
 	    err = E_ALLOC;
-	} else if (gretl_messages_on()) {
+	} else if (gretl_messages_on() && !gretl_looping_quietly()) {
 	    pprintf(prn, _("Replaced list '%s'\n"), name);
 	}
     } else {
@@ -1484,6 +1484,8 @@ int gretl_list_insert_list_minus (int **targ, const int *src, int pos)
 int list_members_replaced (const int *list, const DATAINFO *pdinfo,
 			   int ref_id)
 {
+    const char *errmsg = N_("Can't do this: some vars in original "
+			    "model have been redefined");
     const char *label;
     char rword[16];
     int j, mc, repl, err = 0;
@@ -1505,7 +1507,8 @@ int list_members_replaced (const int *list, const DATAINFO *pdinfo,
 	    repl = 0;
 	    sscanf(label, "%*s %*s %*s %d", &repl);
 	    if (repl >= mc) {
-		err = E_VARCHANGE;
+		strcpy(gretl_errmsg, _(errmsg));
+		err = E_DATA;
 		break;
 	    }
 	}
@@ -1562,14 +1565,14 @@ int gretl_list_const_pos (const int *list, int minpos, const double **Z,
  *
  * Returns: if @list contains the separator for compound
  * lists, #LISTSEP, the position in @list at which this is found,
- * else 0.  The search begins at position 2.
+ * else 0.  The search begins at position 1.
  */
 
 int gretl_list_separator_position (const int *list)
 {
     int i;
 
-    for (i=2; i<=list[0]; i++) {
+    for (i=1; i<=list[0]; i++) {
         if (list[i] == LISTSEP) {
 	    return i;
 	}
@@ -1584,7 +1587,7 @@ int gretl_list_separator_position (const int *list)
  * of which holds a count of the number of elements following.
  *
  * Returns: 1 if @list contains the separator for compound
- * lists, #LISTSEP, else 0.  The search begins at position 2.
+ * lists, #LISTSEP, else 0.  The search begins at position 1.
  */
 
 int gretl_list_has_separator (const int *list)
@@ -1623,13 +1626,14 @@ int gretl_list_split_on_separator (const int *list, int **plist1, int **plist2)
 	return E_PARSE;
     }
 
-    list1 = gretl_list_new(n - 1);
-    if (list1 == NULL) {
-	return E_ALLOC;
-    }
-
-    for (i=1; i<n; i++) {
-	list1[i] = list[i];
+    if (n > 1) {
+	list1 = gretl_list_new(n - 1);
+	if (list1 == NULL) {
+	    return E_ALLOC;
+	}
+	for (i=1; i<n; i++) {
+	    list1[i] = list[i];
+	}
     }
 
     list2 = gretl_list_new(list[0] - n);
@@ -1683,7 +1687,9 @@ int gretl_list_duplicates (const int *list, GretlCmdIndex ci)
     int start = 2;
     int i, ret = -1;
 
-    if (ci == ARCH) {
+    if (ci == COINT) {
+	start = 1;
+    } else if (ci == ARCH) {
 	start = 3;
     } else if (ci == ARMA) {
 	for (i=list[0]-1; i>2; i--) {
@@ -1701,7 +1707,7 @@ int gretl_list_duplicates (const int *list, GretlCmdIndex ci)
 		break;
 	    }
 	}
-    } else if (ci == TSLS) {
+    } else if (ci == TSLS || ci == HECKIT) {
 	multi = 1;
 	for (i=2; i<list[0]; i++) {
 	    if (list[i] == LISTSEP) {
@@ -1713,6 +1719,18 @@ int gretl_list_duplicates (const int *list, GretlCmdIndex ci)
 	if (ret == -1) {
 	    ret = real_list_dup(list, 2, start - 2);
 	}
+    } else if (ci == VAR || ci == VECM || ci == COINT2) {
+	multi = 1;
+	for (i=1; i<list[0]; i++) {
+	    if (list[i] == LISTSEP) {
+		start = i+1;
+		break;
+	    }
+	}
+	ret = real_list_dup(list, start, list[0]);
+	if (ret == -1) {
+	    ret = real_list_dup(list, 1, start - 2);
+	}
     } else if (ci == ARBOND) {
 	int stop = 0;
 
@@ -1748,6 +1766,32 @@ int gretl_list_duplicates (const int *list, GretlCmdIndex ci)
     return ret;
 }
 
+/**
+ * gretl_lists_share_members:
+ * @list1: 
+ * @list2: 
+ *
+ * Returns: 1 if there are any elements in common between @list1
+ * and @list2, otherwise 0.
+ */
+
+int gretl_lists_share_members (const int *list1, const int *list2)
+{
+    int i;
+
+    if (list1 == NULL || list2 == NULL) {
+	return 0;
+    }
+
+    for (i=1; i<=list1[0]; i++) {
+	if (in_gretl_list(list2, list1[i])) {
+	    return 1;
+	}
+    }
+
+    return 0;
+}
+
 /**
  * full_var_list:
  * @pdinfo: dataset information.
