@@ -1,20 +1,20 @@
-/*
- *  Copyright (c) by Allin Cottrell
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
- *
- *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- *
+/* 
+ *  gretl -- Gnu Regression, Econometrics and Time-series Library
+ *  Copyright (C) 2001 Allin Cottrell and Riccardo "Jack" Lucchetti
+ * 
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ * 
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ * 
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 #ifndef GRETL_MATRIX_H
@@ -41,7 +41,8 @@ typedef enum {
     GRETL_MATRIX_UPPER_TRIANGULAR,
     GRETL_MATRIX_SYMMETRIC,
     GRETL_MATRIX_DIAGONAL,
-    GRETL_MATRIX_SCALAR
+    GRETL_MATRIX_IDENTITY,
+    GRETL_MATRIX_SCALAR,
 } GretlMatrixStructure;
 
 typedef struct _gretl_matrix gretl_matrix;
@@ -166,6 +167,12 @@ struct _gretl_matrix {
                                    (m)->rows == 1 && \
                                    (m)->cols == 1)
 
+#define gretl_is_null_matrix(m) (m == NULL || m->rows == 0 || m->cols == 0)
+
+int get_gretl_matrix_err (void);
+
+void clear_gretl_matrix_err (void);
+
 int gretl_matrix_is_symmetric (const gretl_matrix *m);
 
 int gretl_matrix_is_idempotent (const gretl_matrix *m);
@@ -222,6 +229,8 @@ void gretl_matrix_multiply_by_scalar (gretl_matrix *m, double x);
 
 int gretl_matrix_divide_by_scalar (gretl_matrix *m, double x);
 
+void gretl_matrix_switch_sign (gretl_matrix *m);
+
 gretl_matrix *
 gretl_matrix_dot_op (const gretl_matrix *a, const gretl_matrix *b,
 		     int op, int *err);
@@ -230,6 +239,10 @@ gretl_matrix *gretl_matrix_complex_multiply (const gretl_matrix *a,
 					     const gretl_matrix *b,
 					     int *err);
 
+gretl_matrix *gretl_matrix_complex_divide (const gretl_matrix *a, 
+					   const gretl_matrix *b,
+					   int *err);
+
 gretl_matrix *gretl_matrix_exp (const gretl_matrix *m, int *err);
 
 void gretl_matrix_raise (gretl_matrix *m, double x);
@@ -241,6 +254,9 @@ double *gretl_matrix_steal_data (gretl_matrix *m);
 int gretl_matrix_copy_values (gretl_matrix *targ, 
 			      const gretl_matrix *src);
 
+int gretl_matrix_copy_values_shaped (gretl_matrix *targ, 
+				     const gretl_matrix *src);
+
 int gretl_matrix_add_to (gretl_matrix *targ, const gretl_matrix *src);
 
 int gretl_matrix_add_transpose_to (gretl_matrix *targ, 
@@ -249,6 +265,9 @@ int gretl_matrix_add_transpose_to (gretl_matrix *targ,
 int 
 gretl_matrix_subtract_from (gretl_matrix *targ, const gretl_matrix *src);
 
+int 
+gretl_matrix_subtract_reversed (const gretl_matrix *a, gretl_matrix *b);
+
 int gretl_matrix_I_minus (gretl_matrix *m);
 
 int gretl_matrix_transpose (gretl_matrix *targ, const gretl_matrix *src);
@@ -286,13 +305,32 @@ int gretl_matrix_multiply_mod (const gretl_matrix *a, GretlMatrixMod amod,
 int gretl_matrix_multiply (const gretl_matrix *a, const gretl_matrix *b,
 			   gretl_matrix *c);
 
+gretl_matrix *gretl_matrix_multiply_new (const gretl_matrix *a, 
+					 const gretl_matrix *b,
+					 int *err);
+
 int
 gretl_matrix_kronecker_product (const gretl_matrix *A, const gretl_matrix *B,
 				gretl_matrix *K);
 
 gretl_matrix *
 gretl_matrix_kronecker_product_new (const gretl_matrix *A, 
-				    const gretl_matrix *B);
+				    const gretl_matrix *B,
+				    int *err);
+
+int
+gretl_matrix_I_kronecker (int p, const gretl_matrix *B,
+			  gretl_matrix *K);
+
+gretl_matrix *
+gretl_matrix_I_kronecker_new (int p, const gretl_matrix *B, int *err);
+
+int
+gretl_matrix_kronecker_I (const gretl_matrix *A, int r,
+			  gretl_matrix *K);
+
+gretl_matrix *
+gretl_matrix_kronecker_I_new (const gretl_matrix *A, int r, int *err);
 
 gretl_matrix *gretl_matrix_pow (const gretl_matrix *A, 
 				int s, int *err);
@@ -304,13 +342,15 @@ double gretl_matrix_dot_product (const gretl_matrix *a, GretlMatrixMod amod,
 double gretl_vector_dot_product (const gretl_vector *a, const gretl_vector *b,
 				 int *errp);
 
-gretl_matrix *gretl_matrix_row_sum (const gretl_matrix *m);
+gretl_matrix *gretl_matrix_row_sum (const gretl_matrix *m, int *err);
 
-gretl_matrix *gretl_matrix_column_sum (const gretl_matrix *m);
+gretl_matrix *gretl_matrix_column_sum (const gretl_matrix *m, int *err);
 
-gretl_matrix *gretl_matrix_row_mean (const gretl_matrix *m);
+gretl_matrix *gretl_matrix_row_mean (const gretl_matrix *m, int *err);
 
-gretl_matrix *gretl_matrix_column_mean (const gretl_matrix *m);
+gretl_matrix *gretl_matrix_column_mean (const gretl_matrix *m, int *err);
+
+gretl_matrix *gretl_matrix_column_sd (const gretl_matrix *m, int *err);
 
 double gretl_matrix_row_i_mean (const gretl_matrix *m, int row);
 
@@ -336,7 +376,7 @@ double gretl_matrix_infinity_norm (const gretl_matrix *m);
 
 int gretl_LU_solve (gretl_matrix *a, gretl_vector *b);
 
-int gretl_cholesky_decomp_solve (gretl_matrix *a, gretl_vector *b);
+int gretl_cholesky_decomp_solve (gretl_matrix *a, gretl_matrix *b);
 
 int gretl_cholesky_solve (const gretl_matrix *a, gretl_vector *b);
 
@@ -359,6 +399,8 @@ int gretl_invert_diagonal_matrix (gretl_matrix *a);
 
 int gretl_invert_matrix (gretl_matrix *a);
 
+int gretl_matrix_moore_penrose (gretl_matrix *A);
+
 int gretl_SVD_invert_matrix (gretl_matrix *a);
 
 int gretl_matrix_SVD (const gretl_matrix *a, gretl_matrix **pu, 
@@ -391,6 +433,10 @@ gretl_gensymm_eigenvals (const gretl_matrix *A,
 gretl_matrix *gretl_matrix_right_nullspace (const gretl_matrix *M, 
 					    int *err);
 
+gretl_matrix *gretl_matrix_left_nullspace (const gretl_matrix *M,
+					   GretlMatrixMod mod,
+					   int *err);
+
 gretl_matrix *
 gretl_matrix_row_concat (const gretl_matrix *a, const gretl_matrix *b,
 			 int *err);
@@ -422,11 +468,37 @@ int gretl_matrix_ols (const gretl_vector *y, const gretl_matrix *X,
 		      gretl_vector *b, gretl_matrix *vcv,
 		      gretl_vector *uhat, double *s2);
 
+int gretl_matrix_multi_ols (const gretl_matrix *Y, 
+			    const gretl_matrix *X,
+			    gretl_matrix *B, 
+			    gretl_matrix *E,
+			    gretl_matrix **XTXi);
+
+int gretl_matrix_multi_SVD_ols (const gretl_matrix *Y, 
+				const gretl_matrix *X,
+				gretl_matrix *B, 
+				gretl_matrix *E,
+				gretl_matrix **XTXi);
+
+int gretl_matrix_QR_ols (const gretl_matrix *Y,
+			 const gretl_matrix *X,
+			 gretl_matrix *B,
+			 gretl_matrix *E,
+			 gretl_matrix **XTXi,
+			 gretl_matrix **Qout);
+
 double gretl_matrix_r_squared (const gretl_matrix *y,
 			       const gretl_matrix *X,
 			       const gretl_matrix *b,
 			       int *err);
 
+int gretl_matrix_SVD_johansen_solve (const gretl_matrix *R0,
+				     const gretl_matrix *R1,
+				     gretl_matrix *evals,
+				     gretl_matrix *B,
+				     gretl_matrix *A,
+				     int jrank);
+
 int 
 gretl_matrix_restricted_ols (const gretl_vector *y, const gretl_matrix *X,
 			     const gretl_matrix *R, const gretl_vector *q,
@@ -501,6 +573,13 @@ gretl_matrix *gretl_matrix_minmax (const gretl_matrix *A,
 
 gretl_matrix *gretl_matrix_pca (const gretl_matrix *X, int p, int *err);
 
+gretl_matrix *gretl_matrix_xtab (int t1, int t2, const double *x, 
+				 const double *y, int *err);
+
+gretl_matrix *matrix_matrix_xtab (const gretl_matrix *x,
+				  const gretl_matrix *y,
+				  int *err);
+
 void lapack_mem_free (void);
 
 #endif /* GRETL_MATRIX_H */
