@@ -1,20 +1,20 @@
-/* gretl - The Gnu Regression, Econometrics and Time-series Library
- * Copyright (C) 1999-2004 Ramu Ramanathan and Allin Cottrell
- *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License 
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
- *
- * This software is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License 
- * along with this software; if not, write to the 
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+/* 
+ *  gretl -- Gnu Regression, Econometrics and Time-series Library
+ *  Copyright (C) 2001 Allin Cottrell and Riccardo "Jack" Lucchetti
+ * 
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ * 
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ * 
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 #ifndef GRETL_MODEL_H
@@ -22,19 +22,6 @@
 
 #include "objstack.h"
 
-typedef enum {
-    MODEL_DATA_NONE,
-    MODEL_DATA_INT,
-    MODEL_DATA_LIST,
-    MODEL_DATA_DOUBLE,
-    MODEL_DATA_INT_ARRAY,
-    MODEL_DATA_DOUBLE_ARRAY,
-    MODEL_DATA_STRING,
-    MODEL_DATA_CHAR_ARRAY,
-    MODEL_DATA_CMPLX_ARRAY,
-    MODEL_DATA_STRUCT
-} ModelDataType;
-
 typedef enum {
     ARMA_SEAS  = 1 << 0, /* includes seasonal component */
     ARMA_DSPEC = 1 << 1, /* input list includes differences */
@@ -76,6 +63,7 @@ struct CoeffIntervals_ {
 
 #define ML_ESTIMATOR(c) (c == ARMA || \
                          c == GARCH || \
+                         c == HECKIT || \
                          c == LOGIT || \
                          c == MLE || \
                          c == POISSON || \
@@ -98,6 +86,7 @@ struct CoeffIntervals_ {
                              c == ARMA || \
                              c == GARCH || \
                              c == GMM || \
+                             c == HECKIT || \
                              c == LOGIT || \
                              c == MLE || \
                              c == POISSON || \
@@ -138,6 +127,7 @@ typedef enum {
     GRETL_TEST_PANEL_F,
     GRETL_TEST_PANEL_BP,
     GRETL_TEST_PANEL_TIMEDUM,
+    GRETL_TEST_HET_1,
     GRETL_TEST_MAX
 } ModelTestType;
 
@@ -172,11 +162,11 @@ void gretl_model_free (MODEL *pmod);
 void gretl_model_free_on_exit (MODEL *pmod);
 
 int gretl_model_set_data_with_destructor (MODEL *pmod, const char *key, void *ptr, 
-					  ModelDataType type, size_t size, 
+					  GretlType type, size_t size, 
 					  void (*destructor) (void *));
 
 int gretl_model_set_data (MODEL *pmod, const char *key, void *ptr, 
-			  ModelDataType type, size_t size);
+			  GretlType type, size_t size);
 
 int gretl_model_set_list_as_data (MODEL *pmod, const char *key, int *list);
 
@@ -217,6 +207,9 @@ gretl_model_get_coeff_intervals (const MODEL *pmod,
 
 int gretl_model_get_depvar (const MODEL *pmod);
 
+const char *gretl_model_get_depvar_name (const MODEL *pmod,
+					 const DATAINFO *pdinfo);
+
 int *gretl_model_get_x_list (const MODEL *pmod);
 
 int arma_model_nonseasonal_AR_order (const MODEL *pmod);
@@ -241,6 +234,10 @@ int regarima_model_get_AR_coeffs (const MODEL *pmod,
 				  double **phi0,
 				  int *pp);
 
+int gretl_model_set_coeff_separator (MODEL *pmod, const char *s, int pos);
+
+int gretl_model_get_coeff_separator (const MODEL *pmod, const char **ps, int *ppos);
+
 int gretl_model_new_vcv (MODEL *pmod, int *nelem);
 
 VMatrix *gretl_model_get_vcv (MODEL *pmod, const DATAINFO *pdinfo);
@@ -312,7 +309,9 @@ double coeff_pval (int ci, double x, int df);
 int gretl_model_allocate_params (MODEL *pmod, int k);
 
 int gretl_model_add_arma_varnames (MODEL *pmod, const DATAINFO *pdinfo,
-				   int yno, int p, int q, int P, int Q, 
+				   int yno, int p, int q, 
+				   const char *pmask, const char *qmask,
+				   int P, int Q, 
 				   int r);
 
 int gretl_model_add_panel_varnames (MODEL *pmod, const DATAINFO *pdinfo,
