@@ -1,20 +1,20 @@
-/*
- *   Copyright (c) by Allin Cottrell and Riccardo "Jack" Lucchetti
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
- *
- *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
- *
+/* 
+ *  gretl -- Gnu Regression, Econometrics and Time-series Library
+ *  Copyright (C) 2001 Allin Cottrell and Riccardo "Jack" Lucchetti
+ * 
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ * 
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ * 
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 /* panel estimation and dataset handling for gretl */
@@ -503,7 +503,7 @@ panel_robust_vcv (MODEL *pmod, panelmod_t *pan, const double **Z)
     }
 
     /* call the appropriate function */
-    if (get_panel_beck_katz()) {
+    if (libset_get_bool(PCSE)) {
 	err = beck_katz_vcv(pmod, pan, Z, XX, W, V);
     } else {
 	err = arellano_vcv(pmod, pan, Z, XX, W, V);
@@ -647,7 +647,7 @@ static void print_panel_coeff (const MODEL *pmod,
     mc.b = pmod->coeff[i];
     mc.se = pmod->sderr[i];
     mc.tval = mc.b / mc.se;
-    mc.pval = t_pvalue_2(mc.tval, pmod->dfd);
+    mc.pval = student_pvalue_2(mc.tval, pmod->dfd);
     strcpy(mc.name, vname);
     print_coeff(&mc, prn);
 }
@@ -663,7 +663,7 @@ static void print_panel_coeff (const MODEL *pmod,
     char pvstr[18];
  
     sprintf(errstr, "(%.5g)", pmod->sderr[i]);
-    sprintf(pvstr, "[%.5f]", t_pvalue_2(tstat, pmod->dfd));
+    sprintf(pvstr, "[%.5f]", student_pvalue_2(tstat, pmod->dfd));
     pprintf(prn, "%*s: %14.5g %15s %15s\n", VNAMELEN, vname,
  	    pmod->coeff[i], errstr, pvstr);
 }
@@ -1237,7 +1237,7 @@ static int print_fe_results (panelmod_t *pan,
 
     pprintf(prn, _("Joint significance of differing group means:\n"));
     pprintf(prn, " F(%d, %d) = %g %s %g\n", pan->Fdfn, pan->Fdfd, pan->F, 
-	    _("with p-value"), f_cdf_comp(pan->F, pan->Fdfn, pan->Fdfd));
+	    _("with p-value"), snedecor_cdf_comp(pan->F, pan->Fdfn, pan->Fdfd));
 
     pputs(prn, _("(A low p-value counts against the null hypothesis that "
 		 "the pooled OLS model\nis adequate, in favor of the fixed "
@@ -1330,7 +1330,7 @@ static void save_fixed_effects_F (panelmod_t *pan, MODEL *wmod)
 	model_test_set_dfn(test, pan->Fdfn);
 	model_test_set_dfd(test, pan->Fdfd);
 	model_test_set_value(test, pan->F);
-	model_test_set_pvalue(test, f_cdf_comp(pan->F, pan->Fdfn, pan->Fdfd));
+	model_test_set_pvalue(test, snedecor_cdf_comp(pan->F, pan->Fdfn, pan->Fdfd));
 	maybe_add_test_to_model(wmod, test);
     }	    
 }
@@ -1476,7 +1476,7 @@ static int fe_model_add_ahat (MODEL *pmod, const double **Z,
     }
 
     err = gretl_model_set_data(pmod, "ahat", ahat, 
-			       MODEL_DATA_DOUBLE_ARRAY, 
+			       GRETL_TYPE_DOUBLE_ARRAY, 
 			       pdinfo->n * sizeof *ahat);
 
     return err;
@@ -2880,8 +2880,8 @@ static double pooled_ll (const MODEL *pmod)
    heteroskedasticity
 */
 
-static double panel_ML_ll (const MODEL *pmod, const double *uvar, 
-			   int nunits, const int *unit_obs)
+static void panel_ML_ll (MODEL *pmod, const double *uvar, 
+			 int nunits, const int *unit_obs)
 {
     double ll = -(pmod->nobs / 2.0) * LN_2_PI;
     int i, Ti;
@@ -2893,7 +2893,8 @@ static double panel_ML_ll (const MODEL *pmod, const double *uvar,
 	}
     }
 
-    return ll;
+    pmod->lnL = ll;
+    mle_criteria(pmod, 0);
 }
 
 /* we can't estimate a group-specific variance based on just one
@@ -3091,18 +3092,19 @@ MODEL panel_wls_by_unit (const int *list, double ***pZ, DATAINFO *pdinfo,
     }
 
     if (!mdl.errcode) {
+	mdl.ci = PANEL;
 	if (!(opt & OPT_A)) {
 	    set_model_id(&mdl);
 	}
 	gretl_model_set_int(&mdl, "n_included_units", pan.effn);
-	gretl_model_set_int(&mdl, "unit_weights", 1);
+	gretl_model_set_int(&mdl, "unit-weights", 1);
 	mdl.nwt = 0;
 
 	if (opt & OPT_T) {
 	    gretl_model_set_int(&mdl, "iters", iter);
 	    ml_hetero_test(&mdl, s2, uvar, pan.nunits, pan.unit_obs);
 	    unit_error_variances(uvar, &mdl, pdinfo, &pan);
-	    mdl.lnL = panel_ML_ll(&mdl, uvar, pan.nunits, pan.unit_obs);
+	    panel_ML_ll(&mdl, uvar, pan.nunits, pan.unit_obs);
 	    if (opt & OPT_V) {
 		pputc(prn, '\n');
 	    }
@@ -3287,7 +3289,7 @@ int panel_autocorr_test (MODEL *pmod, int order,
 	printmodel(&aux, tmpinfo, OPT_NONE, prn);
 	trsq = aux.rsq * aux.nobs;
 	LMF = (aux.rsq / (1.0 - aux.rsq)) * dfd / order; 
-	pval = f_cdf_comp(LMF, order, dfd);
+	pval = snedecor_cdf_comp(LMF, order, dfd);
 
 	pprintf(prn, "\n%s: LMF = %f,\n", _("Test statistic"), LMF);
 	pprintf(prn, "%s = P(F(%d,%d) > %g) = %.3g\n", _("with p-value"), 
@@ -3562,8 +3564,8 @@ static int compose_panel_indices (DATAINFO *pdinfo, int T,
     return err;
 }
 
-static int dataset_sort_by (double **Z, DATAINFO *pdinfo,
-			    int uv, int tv)
+static int panel_data_sort_by (double **Z, DATAINFO *pdinfo,
+			       int uv, int tv)
 {
     int n = pdinfo->n;
     char **S = NULL;
@@ -3901,7 +3903,7 @@ int set_panel_structure_from_vars (int uv, int tv,
 #endif
 
     /* sort full dataset by unit and period */
-    err = dataset_sort_by(Z, pdinfo, uv, tv);
+    err = panel_data_sort_by(Z, pdinfo, uv, tv);
 
 #if PDEBUG
     print_unit_var(uv, Z, n, 1);
