@@ -1,20 +1,20 @@
-/*
- *  Copyright (c) Allin Cottrell
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
- *
- *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
- *
+/* 
+ *  gretl -- Gnu Regression, Econometrics and Time-series Library
+ *  Copyright (C) 2001 Allin Cottrell and Riccardo "Jack" Lucchetti
+ * 
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ * 
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ * 
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 /* gretl_paths.c for gretl  */
@@ -34,11 +34,6 @@
 #endif
 
 #include <glib.h>
-#if (GLIB_MAJOR_VERSION >= 2) && (GLIB_MINOR_VERSION >= 6)
-# ifdef WIN32
-#  define USE_G_FOPEN
-# endif
-#endif
 
 enum {
     CURRENT_DIR,
@@ -60,51 +55,110 @@ static int add_suffix (char *fname, const char *sfx)
     return 0;
 }
 
-FILE *gretl_fopen (const char *filename, const char *mode)
+/* Heuristic: filename contains non-ascii characters, and
+   validates as UTF-8 */
+
+static int fname_is_utf8 (const unsigned char *s)
 {
+    const unsigned char *p = s;
+    int sevenbit = 1;
+    int ret = 0;
+
+    while (*p) {
+	if (*p > 127) {
+	    sevenbit = 0;
+	    break;
+	}
+	p++;
+    }
+
+    if (!sevenbit && g_utf8_validate((gchar *) s, -1, NULL)) {
+	ret = 1;
+    }
+
+    return ret;
+}
+
+static int fopen_use_utf8;
+
+void set_fopen_use_utf8 (void)
+{
+    fopen_use_utf8 = 1;
+}
+
+/* Below: try to guard against a situation where a filename
+   is UTF-8 encoded, but we should be using locale encoding
+   with the stdio functions.
+*/
+
+FILE *gretl_fopen (const char *fname, const char *mode)
+{
+    gchar *fconv;
+    gsize wrote;
     FILE *fp = NULL;
 
-#if defined(USE_G_FOPEN)
-    fp = g_fopen((const gchar *) filename, (const gchar *) mode);
-#elif defined(WIN32)
-    fp = fopen(filename, mode);
-    if (fp == NULL) {
-	int save_errno = errno;
-	gchar *fconv;
-	gsize wrote;
+    if (mode != NULL && *mode == 'r') {
+	/* opening for reading */
+	fp = fopen(fname, mode);
+	if (fp == NULL && !fopen_use_utf8 && 
+	    fname_is_utf8((unsigned char *) fname)) {
+	    int save_errno = errno;
 
-	fconv = g_locale_from_utf8(filename, -1, NULL, &wrote, NULL);
-	if (fconv != NULL) {
-	    fp = fopen(fconv, mode);
-	    g_free(fconv);
+	    fconv = g_locale_from_utf8(fname, -1, NULL, &wrote, NULL);
+	    if (fconv != NULL) {
+		fp = fopen(fconv, mode);
+		g_free(fconv);
+	    }
+	    errno = save_errno;
+	}
+    } else {
+	/* opening for appending/writing */
+	if (!fopen_use_utf8 && fname_is_utf8((unsigned char *) fname)) {
+	    fconv = g_locale_from_utf8(fname, -1, NULL, &wrote, NULL);
+	    if (fconv != NULL) {
+		fp = fopen(fconv, mode);
+		g_free(fconv);
+	    }
+	} else {
+	    fp = fopen(fname, mode);
 	}
-	errno = save_errno;
     }
-#else    
-    fp = fopen(filename, mode);
-#endif
 
     return fp;
 }
 
-gzFile gretl_gzopen (const char *filename, const char *mode)
+gzFile gretl_gzopen (const char *fname, const char *mode)
 {
-    gzFile fz = NULL;
-
-#if defined(USE_G_FOPEN) || defined(WIN32)
-    int save_errno = errno;
     gchar *fconv;
     gsize wrote;
+    gzFile fz = NULL;
 
-    fconv = g_locale_from_utf8(filename, -1, NULL, &wrote, NULL);
-    if (fconv != NULL) {
-	fz = gzopen(fconv, mode);
-	g_free(fconv);
-    }    
-    errno = save_errno;
-#else
-    fz = gzopen(filename, mode);
-#endif
+    if (mode != NULL && *mode == 'r') {
+	/* opening for reading */
+	fz = gzopen(fname, mode);
+	if (fz == NULL && !fopen_use_utf8 && 
+	    fname_is_utf8((unsigned char *) fname)) {
+	    int save_errno = errno;
+
+	    fconv = g_locale_from_utf8(fname, -1, NULL, &wrote, NULL);
+	    if (fconv != NULL) {
+		fz = gzopen(fconv, mode);
+		g_free(fconv);
+	    }
+	    errno = save_errno;
+	}
+    } else {
+	/* opening for writing */
+	if (!fopen_use_utf8 && fname_is_utf8((unsigned char *) fname)) {
+	    fconv = g_locale_from_utf8(fname, -1, NULL, &wrote, NULL);
+	    if (fconv != NULL) {
+		fz = gzopen(fconv, mode);
+		g_free(fconv);
+	    }
+	} else {
+	    fz = gzopen(fname, mode);
+	}
+    }
 
     return fz;
 }
@@ -146,8 +200,83 @@ int gretl_mkdir (const char *path)
     return err;
 }
 
+static const char *gretl_readd (DIR *d)
+{
+    struct dirent *e = readdir(d);
+
+    return (e == NULL)? NULL : e->d_name;
+}
+
+static int gretl_isdir (const char *path)
+{
+    struct stat buf;
+
+    return (stat(path, &buf) == 0 && S_ISDIR(buf.st_mode)); 
+}
+
+/* recursive deletion of directory tree: must be located
+   in the directory above the one to be deleted at the
+   outset */
+
+int gretl_deltree (const char *path)
+{
+    const char *fname;
+    DIR *dir;
+    int err = 0;
+
+    errno = 0;
+
+    dir = opendir(path);
+
+    if (dir == NULL) {
+	err = 1;
+    } else {
+	err = chdir(path);
+	while ((fname = gretl_readd(dir)) != NULL && !err) {
+	    if (strcmp(fname, ".") && strcmp(fname, "..")) {
+		if (gretl_isdir(fname)) {
+		    err = gretl_deltree(fname);
+		} else {
+		    err = remove(fname);
+		}
+	    }
+	}
+	if (!err) {
+	    closedir(dir);
+	    chdir("..");
+	    err = remove(path);
+	}
+    }
+
+    if (err) {
+	gretl_errmsg_set_from_errno();
+	err = E_FOPEN;
+    }
+    
+    return err;
+}
+
 #endif
 
+/* like access(), returns 0 on success */
+
+int gretl_write_access (char *fname)
+{
+    int err = 0;
+
+#ifdef WIN32
+    err = !win32_write_access(fname);
+#else
+    err = access(fname, W_OK);
+#endif
+
+    if (err) {
+	sprintf(gretl_errmsg, "Can't write to %s", fname);
+    }
+
+    return err;
+}
+
 int gretl_is_xml_file (const char *fname)
 {
     gzFile fz;
@@ -443,6 +572,31 @@ static void make_path_absolute (char *fname, const char *orig)
     }
 }
 
+/* When given a filename such as "./foo", see if shelldir is
+   set -- if so, try opening the file as if shelldir was
+   the CWD.
+*/
+
+static int shelldir_open_dotfile (char *fname, char *orig)
+{
+    char *sdir = get_shelldir();
+    FILE *test;
+    int ret = 0;
+
+    if (sdir != NULL && *sdir != '\0') {
+	sprintf(fname, "%s%c%s", sdir, SLASH, orig);
+	test = gretl_fopen(fname, "r");
+	if (test != NULL) {
+	    fclose(test);
+	    ret = 1;
+	} else {
+	    strcpy(fname, orig);
+	}
+    }
+
+    return ret;
+}
+
 /**
  * addpath:
  * @fname: initially given file name.
@@ -464,6 +618,10 @@ char *addpath (char *fname, PATHS *ppaths, int script)
 
     strcpy(orig, fname);
 
+    if (dotpath(fname) && shelldir_open_dotfile(fname, orig)) {
+	return fname;
+    }
+
     /* try opening filename as given */
     test = gretl_fopen(fname, "r");
     if (test != NULL) { 
@@ -776,16 +934,40 @@ static void copy_paths_to_internal (PATHS *paths)
     gretl_insert_builtin_string("x12adir",   paths->x12adir);
     gretl_insert_builtin_string("tramo",     paths->tramo);
     gretl_insert_builtin_string("tramodir",  paths->tramodir);
+
+    if (*paths->tramo) {
+	char s[MAXLEN];
+	int n;
+
+	*s = '\0';
+	strncat(s, paths->tramo, MAXLEN - 1);
+	n = strlen(s);
+#ifdef WIN32
+	if (n >= 9 && !strcmp(s + n - 9, "tramo.exe")) {
+	    strcpy(s + n - 9, "seats.exe");
+	    gretl_insert_builtin_string("seats", s);
+	    return;
+	}
+#endif
+	if (n >= 5 && !strcmp(s + n - 5, "tramo")) {
+	    strcpy(s + n - 5, "seats");
+	    gretl_insert_builtin_string("seats", s);
+	}
+    }
 }
 
 const char *gretl_lib_path (void)
 {
-    if (*gretl_paths.libpath == '\0') {
+    static int set;
+
+    if (!set) {
 	char *epath = getenv("GRETL_PLUGIN_PATH");
 
 	if (epath != NULL) {
+	    gretl_paths.libpath[0] = '\0';
 	    strncat(gretl_paths.libpath, epath, MAXLEN - 1);
 	}
+	set = 1;
     }
 
     return gretl_paths.libpath;
@@ -1101,6 +1283,14 @@ int gretl_set_paths (PATHS *ppaths, gretlopt opt)
 	    *ppaths->userdir = '\0';
 	}
 
+#ifdef HAVE_X12A 	 
+	strcpy(ppaths->x12a, "x12a"); 	 
+#endif 	 
+	  	 
+#ifdef HAVE_TRAMO 	 
+	strcpy(ppaths->tramo, "tramo"); 	 
+#endif
+
 	*gretl_paths.plotfile = '\0';
     } else {
 	/* check validity of gretldir, userdir */
@@ -1163,7 +1353,7 @@ int gretl_set_paths (PATHS *ppaths, gretlopt opt)
 void gretl_maybe_switch_dir (const char *fname)
 {
     if (!g_path_is_absolute(fname)) {
-	if (dotpath(fname) || get_use_cwd()) {
+	if (dotpath(fname) || libset_get_bool(USE_CWD)) {
 	    char *sdir = get_shelldir();
 
 	    if (sdir != NULL) {
