@@ -1,24 +1,26 @@
-/*
- *  Copyright (c) 2005 by Allin Cottrell
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
- *
- *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111, USA.
- *
+/* 
+ *  gretl -- Gnu Regression, Econometrics and Time-series Library
+ *  Copyright (C) 2001 Allin Cottrell and Riccardo "Jack" Lucchetti
+ * 
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ * 
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ * 
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 #include "libgretl.h"
+#include "libset.h"
 #include <stdarg.h>
+#include <glib.h>
 
 struct PRN_ {
     FILE *fp;
@@ -29,6 +31,7 @@ struct PRN_ {
     PrnFormat format;
     int fixed;
     char delim;
+    char *fname;
 };
 
 #undef PRN_DEBUG
@@ -54,6 +57,11 @@ void gretl_print_destroy (PRN *prn)
 	fclose(prn->fp);
     }
 
+    if (prn->fname != NULL) {
+	remove(prn->fname);
+	free(prn->fname);
+    }
+
     if (!fpdup && prn->fpaux != NULL && 
 	prn->fpaux != stdout && prn->fpaux != stderr) {
 	fclose(prn->fpaux);
@@ -70,6 +78,28 @@ void gretl_print_destroy (PRN *prn)
     free(prn);
 }
 
+static int prn_add_tempfile (PRN *prn)
+{
+    char fname[MAXLEN];
+    int fd;
+
+    sprintf(fname, "%sprntmp.XXXXXX", gretl_user_dir());
+
+#ifdef WIN32
+    fd = g_mkstemp(fname);
+#else
+    fd = mkstemp(fname);
+#endif
+    if (fd == -1) {
+	return E_FOPEN;
+    } else {
+	prn->fname = gretl_strdup(fname);
+	prn->fp = fdopen(fd, "w");
+    }
+
+    return 0;
+}
+
 static PRN *real_gretl_print_new (PrnType ptype, 
 				  const char *fname,
 				  char *buf)
@@ -94,6 +124,7 @@ static PRN *real_gretl_print_new (PrnType ptype,
     prn->format = GRETL_FORMAT_TXT;
     prn->fixed = 0;
     prn->delim = ',';
+    prn->fname = NULL;
 
     if (ptype == GRETL_PRINT_FILE) {
 	prn->fp = gretl_fopen(fname, "w");
@@ -101,6 +132,14 @@ static PRN *real_gretl_print_new (PrnType ptype,
 	    fprintf(stderr, _("gretl_prn_new: couldn't open %s\n"), fname);
 	    free(prn);
 	    prn = NULL;
+	} 
+    } else if (ptype == GRETL_PRINT_TEMPFILE) {
+	if (prn_add_tempfile(prn)) {
+	    fprintf(stderr, "gretl_prn_new: couldn't open tempfile\n");
+	    free(prn);
+	    prn = NULL;
+	} else {
+	    prn->savepos = 0;
 	}
     } else if (ptype == GRETL_PRINT_STDOUT) {
 	prn->fp = stdout;
@@ -182,6 +221,33 @@ PRN *gretl_print_new_with_filename (const char *fname)
     return real_gretl_print_new(GRETL_PRINT_FILE, fname, NULL);
 }
 
+/**
+ * gretl_print_new_with_tempfile:
+ * 
+ * Create and initialize a gretl printing struct, with
+ * output directed to a temporary file, which is deleted
+ * when the printing struct is destroyed.
+ *
+ * Returns: pointer to newly created struct, or %NULL on failure.
+ */
+
+PRN *gretl_print_new_with_tempfile (void)
+{
+    return real_gretl_print_new(GRETL_PRINT_TEMPFILE, NULL, NULL);
+}
+
+/**
+ * gretl_print_has_tempfile:
+ * @prn: printing struct to test.
+ * 
+ * Returns: 1 if @prn has a tempfile attached, else 0.
+ */
+
+int gretl_print_has_tempfile (PRN *prn)
+{
+    return (prn != NULL && prn->fname != NULL && prn->fp != NULL);
+}
+
 /**
  * gretl_print_new_with_buffer:
  * @buf: pre-allocated text buffer.
@@ -245,6 +311,60 @@ const char *gretl_print_get_buffer (PRN *prn)
     return (prn != NULL)? prn->buf : NULL;
 }
 
+/**
+ * gretl_print_read_tempfile:
+ * @prn: printing struct.
+ * 
+ * Obtain a read handle to the tempfile associated with
+ * @prn, if any.  This should be fclosed once you're
+ * finished with it.
+ *
+ * Returns: %FILE pointer, or %NULL on failure.
+ */
+
+FILE *gretl_print_read_tempfile (PRN *prn)
+{
+    FILE *fp = NULL;
+
+    if (prn->fp != NULL) {
+	fflush(prn->fp);
+    }
+
+    if (prn->fname != NULL) {
+	fp = gretl_fopen(prn->fname, "r");
+	if (fp != NULL && prn->savepos > 0) {
+	    fseek(fp, (long) prn->savepos, SEEK_SET);
+	}
+    }
+
+    return fp;
+}
+
+/**
+ * gretl_print_stop_tempfile_read:
+ * @prn: printing struct.
+ * 
+ * For @prn with tempfile attached, stops reading
+ * of the tempfile and closes @fp (recording the
+ * position at which reading stopped).
+ *
+ * Returns: 0 on success, error code if @prn is not
+ * connected to a tempfile.
+ */
+
+int gretl_print_stop_tempfile_read (PRN *prn, FILE *fp)
+{
+    if (prn == NULL || prn->fp == NULL || 
+	prn->fname == NULL || fp == NULL) {
+	return E_DATA;
+    }
+
+    prn->savepos = ftell(fp);
+    fclose(fp);
+
+    return 0;
+}
+
 /**
  * gretl_print_set_save_position:
  * @prn: printing struct.
@@ -791,3 +911,35 @@ char prn_delim (PRN *prn)
 {
     return (prn != NULL)? prn->delim : ',';
 }
+
+/**
+ * set_up_verbose_printer.
+ * @opt: %OPT_V for verbosity.
+ * @prn: currently active printing struct
+ *
+ * Returns: printing struct for verbose output.
+ */
+
+PRN *set_up_verbose_printer (gretlopt opt, PRN *prn)
+{
+    PRN *vprn = NULL;
+
+    if (opt & OPT_V) {
+	if (iter_print_func_installed()) {
+	    vprn = gretl_print_new_with_tempfile();
+	} else {
+	    vprn = prn;
+	}
+    } 
+
+    return vprn;
+}
+
+void close_down_verbose_printer (PRN *vprn)
+{
+    if (vprn != NULL) {
+	iter_print_callback(-1, NULL);
+	gretl_print_destroy(vprn);
+    }
+}
+ 
