@@ -1,20 +1,20 @@
-/*
- *  Copyright (c) 2005 by Allin Cottrell
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
- *
- *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111, USA.
- *
+/* 
+ *  gretl -- Gnu Regression, Econometrics and Time-series Library
+ *  Copyright (C) 2001 Allin Cottrell and Riccardo "Jack" Lucchetti
+ * 
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ * 
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ * 
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 #ifndef GRETL_PRN_H
@@ -24,7 +24,8 @@ typedef enum {
     GRETL_PRINT_STDOUT,
     GRETL_PRINT_STDERR,
     GRETL_PRINT_FILE,
-    GRETL_PRINT_BUFFER
+    GRETL_PRINT_BUFFER,
+    GRETL_PRINT_TEMPFILE
 } PrnType;
 
 typedef enum {
@@ -49,12 +50,20 @@ PRN *gretl_print_new (PrnType ptype);
 
 PRN *gretl_print_new_with_filename (const char *fname);
 
+PRN *gretl_print_new_with_tempfile (void);
+
+int gretl_print_has_tempfile (PRN *prn);
+
 PRN *gretl_print_new_with_buffer (char *buf);
 
 int gretl_print_reset_buffer (PRN *prn);
 
 const char *gretl_print_get_buffer (PRN *prn);
 
+FILE *gretl_print_read_tempfile (PRN *prn);
+
+int gretl_print_stop_tempfile_read (PRN *prn, FILE *fp);
+
 int gretl_print_set_save_position (PRN *prn);
 
 void gretl_print_unset_save_position (PRN *prn);
@@ -103,4 +112,8 @@ int prn_format (PRN *prn);
 
 char prn_delim (PRN *prn);
 
+PRN *set_up_verbose_printer (gretlopt opt, PRN *prn);
+
+void close_down_verbose_printer (PRN *vprn);
+
 #endif /* GRETL_PRN_H */
