@@ -1,32 +1,36 @@
-/*
- *  Copyright (c) 2004 by Allin Cottrell
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
- *
- *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- *
+/* 
+ *  gretl -- Gnu Regression, Econometrics and Time-series Library
+ *  Copyright (C) 2001 Allin Cottrell and Riccardo "Jack" Lucchetti
+ * 
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ * 
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ * 
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 #include "libgretl.h"
 #include "libset.h"
+#include "gretl_func.h"
 #include "gretl_string_table.h"
 
+#include <glib.h>
+
 typedef struct _col_table col_table;
 
 struct _col_table {
     int idx;
     int n_strs;
     char **strs;
+    GHashTable *hash;
 };
 
 struct _gretl_string_table {
@@ -34,6 +38,8 @@ struct _gretl_string_table {
     col_table **cols;
 };
 
+static void clean_up_codevars (void);
+
 static col_table *col_table_new (int colnum)
 {
     col_table *ct = malloc(sizeof *ct);
@@ -42,16 +48,19 @@ static col_table *col_table_new (int colnum)
 	ct->strs = NULL;
 	ct->n_strs = 0;
 	ct->idx = colnum;
+	ct->hash = g_hash_table_new(g_str_hash, g_str_equal);
     }
 
     return ct;
 }
 
-gretl_string_table *gretl_string_table_new (void)
+gretl_string_table *gretl_string_table_new (int *err)
 {
     gretl_string_table *st = malloc(sizeof *st);
 
-    if (st != NULL) {
+    if (st == NULL) {
+	*err = E_ALLOC;
+    } else {
 	st->cols = NULL;
 	st->n_cols = 0;
     }
@@ -92,14 +101,11 @@ gretl_string_table *string_table_new_from_cols_list (int *list)
 
 static int col_table_get_index (const col_table *ct, const char *s)
 {
-    int ret = -1;
-    int i;
+    gpointer p = g_hash_table_lookup(ct->hash, s);
+    int ret = 0;
 
-    for (i=0; i<ct->n_strs; i++) {
-	if (!strcmp(s, ct->strs[i])) {
-	    ret = i + 1;
-	    break;
-	}
+    if (p != NULL) {
+	ret = GPOINTER_TO_INT(p);
     }
 
     return ret;
@@ -123,6 +129,8 @@ col_table_add_string (col_table *ct, const char *s)
 	    ret = -1;
 	} else {
 	    ct->n_strs += 1;
+	    g_hash_table_insert(ct->hash, (gpointer) strs[n-1], 
+				GINT_TO_POINTER(n));
 	}
     }
 
@@ -152,7 +160,7 @@ gretl_string_table_index (gretl_string_table *st, const char *s, int col,
 			  int addcol, PRN *prn)
 {
     col_table *ct = NULL;
-    int i, idx = -1;
+    int i, idx = 0;
 
     if (st == NULL) return idx;
 
@@ -170,12 +178,13 @@ gretl_string_table_index (gretl_string_table *st, const char *s, int col,
 	/* no table for this column yet: start one now */
 	ct = gretl_string_table_add_column(st, col);
 	if (ct != NULL) {
-	    pprintf(prn, M_("variable %d: translating from strings to code numbers\n"), 
+	    pprintf(prn, M_("variable %d: translating from strings to "
+			    "code numbers\n"), 
 		    col);
 	}
     }
 
-    if (idx < 0 && ct != NULL) {
+    if (idx == 0 && ct != NULL) {
 	idx = col_table_add_string(ct, s);
     }
 
@@ -192,6 +201,11 @@ static void col_table_destroy (col_table *ct)
 	free(ct->strs[i]);
     }
     free(ct->strs);
+
+    if (ct->hash != NULL) {
+	g_hash_table_destroy(ct->hash);
+    }
+
     free(ct);
 }
 
@@ -205,6 +219,7 @@ void gretl_string_table_destroy (gretl_string_table *st)
 	col_table_destroy(st->cols[i]);
     }
     free(st->cols);
+
     free(st);
 }
 
@@ -218,15 +233,14 @@ int gretl_string_table_print (gretl_string_table *st, DATAINFO *pdinfo,
     FILE *fp;
     int err = 0;
 
-    if (st == NULL) return 1;
+    if (st == NULL) return E_DATA;
 
     strcpy(stname, "string_table.txt");
     gretl_path_prepend(stname, gretl_user_dir());
 
     fp = gretl_fopen(stname, "w");
     if (fp == NULL) {
-	err = E_FOPEN;
-	goto bailout;
+	return E_FOPEN;
     }
 
     fshort = strrchr(fname, SLASH);
@@ -264,10 +278,6 @@ int gretl_string_table_print (gretl_string_table *st, DATAINFO *pdinfo,
 	set_string_table_written();
     }
 
- bailout:
-
-    gretl_string_table_destroy(st);
-
     return err;
 }
 
@@ -277,6 +287,7 @@ typedef struct saved_string_ saved_string;
 
 struct saved_string_ {
     char name[VNAMELEN];
+    int level;
     char *s;
 };
 
@@ -284,19 +295,21 @@ static int n_saved_strings;
 static saved_string *saved_strings;
 
 static saved_string built_ins[] = {
-    { "gretldir", NULL },
-    { "userdir",  NULL },
-    { "gnuplot",  NULL },
-    { "x12a",     NULL },
-    { "x12adir",  NULL },
-    { "tramo",    NULL },
-    { "tramodir", NULL }
+    { "gretldir", 0, NULL },
+    { "userdir",  0, NULL },
+    { "gnuplot",  0, NULL },
+    { "x12a",     0, NULL },
+    { "x12adir",  0, NULL },
+    { "tramo",    0, NULL },
+    { "tramodir", 0, NULL },
+    { "seats",    0, NULL },
+    { "shelldir", 0, NULL }
 };
 
 #ifdef WIN32
-static saved_string dsep = { "dirsep",  "\\" };
+static saved_string dsep = { "dirsep", 0, "\\" };
 #else
-static saved_string dsep = { "dirsep",  "/" };
+static saved_string dsep = { "dirsep", 0, "/" };
 #endif
 
 void gretl_insert_builtin_string (const char *name, const char *s)
@@ -331,7 +344,7 @@ static void gretl_free_builtin_strings (void)
 static saved_string *get_saved_string_by_name (const char *name,
 					       int *builtin)
 {
-    int i;
+    int i, d;
 
     if (builtin != NULL && !strcmp(name, "dirsep")) {
 	*builtin = 1;
@@ -347,10 +360,13 @@ static saved_string *get_saved_string_by_name (const char *name,
 		return &built_ins[i];
 	    }
 	}
-    }	
+    }
+
+    d = gretl_function_depth();
 
     for (i=0; i<n_saved_strings; i++) {
-	if (!strcmp(name, saved_strings[i].name)) {
+	if (saved_strings[i].level == d &&
+	    !strcmp(name, saved_strings[i].name)) {
 	    return &saved_strings[i];
 	}
     }
@@ -363,7 +379,7 @@ static saved_string *get_saved_string_by_name (const char *name,
 
 char *get_named_string (const char *name)
 {
-    int i, n;
+    int i, n, d;
 
     if (!strcmp(name, "dirsep")) {
 	return dsep.s;
@@ -377,8 +393,11 @@ char *get_named_string (const char *name)
 	}
     }
 
+    d = gretl_function_depth();
+
     for (i=0; i<n_saved_strings; i++) {
-	if (!strcmp(name, saved_strings[i].name)) {
+	if (saved_strings[i].level == d &&
+	    !strcmp(name, saved_strings[i].name)) {
 	    return saved_strings[i].s;
 	}
     }
@@ -435,6 +454,7 @@ static saved_string *add_named_string (const char *name)
     }
 
     strcpy(S[n].name, name);
+    S[n].level = gretl_function_depth();
     S[n].s = NULL;
     saved_strings = S;
     n_saved_strings += 1;
@@ -442,6 +462,35 @@ static saved_string *add_named_string (const char *name)
     return &S[n];
 }
 
+int add_string_as (const char *s, const char *name)
+{
+    int n = n_saved_strings;
+    saved_string *S;
+    int err = 0;
+
+    if (name == NULL || s == NULL) {
+	return E_DATA;
+    }
+
+    S = realloc(saved_strings, (n + 1) * sizeof *S);
+    if (S == NULL) {
+	return E_ALLOC;
+    }
+
+    saved_strings = S;
+
+    S[n].s = gretl_strdup(s);
+    if (S[n].s == NULL) {
+	err = E_ALLOC;
+    } else {  
+	strcpy(S[n].name, name);
+	S[n].level = gretl_function_depth() + 1;
+	n_saved_strings += 1;
+    }
+
+    return err;
+}
+
 void saved_strings_cleanup (void)
 {
     int i;
@@ -455,6 +504,59 @@ void saved_strings_cleanup (void)
     n_saved_strings = 0;
 
     gretl_free_builtin_strings();
+
+    clean_up_codevars();
+}
+
+/* called on exiting a user-defined function to clean
+   up any strings defined therein */
+
+int destroy_saved_strings_at_level (int d)
+{
+    int i, ndel = 0;
+    int err = 0;
+
+    for (i=0; i<n_saved_strings; i++) {
+	if (saved_strings[i].level == d) {
+	    ndel++;
+	}
+    }
+
+    if (ndel > 0) {
+	int nnew = n_saved_strings - ndel;
+
+	if (nnew == 0) {
+	    for (i=0; i<n_saved_strings; i++) {
+		free(saved_strings[i].s);
+	    }
+	    free(saved_strings);
+	    saved_strings = NULL;
+	    n_saved_strings = 0;
+	} else {	    
+	    saved_string *S = malloc(nnew * sizeof *S);
+	    int j = 0;
+
+	    if (S == NULL) {
+		err = E_ALLOC;
+	    } else {
+		for (i=0; i<n_saved_strings; i++) {
+		    if (saved_strings[i].level == d) {
+			free(saved_strings[i].s);
+		    } else {
+			strcpy(S[j].name, saved_strings[i].name);
+			S[j].level = saved_strings[i].level;
+			S[j].s = saved_strings[i].s;
+			j++;
+		    }
+		}
+		free(saved_strings);
+		saved_strings = S;
+		n_saved_strings = nnew;
+	    }
+	}
+    }
+
+    return err;
 }
 
 static void copy_unescape (char *targ, const char *src, int n)
@@ -508,6 +610,175 @@ static char *retrieve_string_var (const char **pline,
     return ret;
 }
 
+#ifdef WIN32
+
+/* FIXME */
+
+static int shell_grab (const char *arg, char **sout)
+{
+    gchar *serr = NULL;
+    GError *gerr = NULL;
+    char *wdir;
+    int status, err = 0;
+ 
+    if (!libset_get_bool(SHELL_OK)) {
+	strcpy(gretl_errmsg, _("The shell command is not activated."));
+	return 1;
+    }
+
+    wdir = get_shelldir();
+    if (wdir != NULL && chdir(wdir)) {
+	sprintf(gretl_errmsg, _("Couldn't open %s"), wdir);
+	return E_FOPEN;
+    }
+
+    arg += strspn(arg, " \t");
+
+    g_spawn_command_line_sync(arg, sout, &serr, 
+			      &status, &gerr);
+
+    if (gerr != NULL) {
+	gretl_errmsg_set(gerr->message);
+	g_error_free(gerr);
+	err = 1;
+    }
+
+    if (serr != NULL) {
+	g_free(serr);
+    }
+
+    if (sout != NULL && *sout != NULL) {
+	/* trim trailing newline */
+	int n = strlen(*sout);
+
+	if ((*sout)[n-1] == '\n') {
+	    (*sout)[n-1] = '\0';
+	}
+    }
+
+    return err;
+}
+
+#else
+
+static int shell_grab (const char *arg, char **sout)
+{
+    gchar *serr = NULL;
+    GError *gerr = NULL;
+    gchar *argv[5];
+    const char *theshell, *namep;
+    char shellnam[40];
+    char *wdir;
+    int status, err = 0;
+    
+    if (arg == NULL || *arg == '\0') {
+	return E_PARSE;
+    }
+
+    if (!libset_get_bool(SHELL_OK)) {
+	strcpy(gretl_errmsg, _("The shell command is not activated."));
+	return 1;
+    }
+
+    wdir = get_shelldir();
+    if (wdir == NULL || *wdir == '\0') {
+	wdir = NULL;
+    }
+
+    arg += strspn(arg, " \t");
+    theshell = getenv("SHELL"); 
+
+    if (theshell == NULL) {
+# ifdef HAVE_PATHS_H
+	theshell =_PATH_BSHELL;
+# else
+	theshell = "/bin/sh"; 
+# endif
+    }
+
+    namep = strrchr(theshell, '/');
+    if (namep == NULL) {
+	namep = theshell;
+    }
+
+    strcpy(shellnam, "-");
+    strcat(shellnam, ++namep);
+    if (strcmp(namep, "sh") != 0) {
+	shellnam[0] = '+';
+    }
+
+    argv[0] = g_strdup(theshell);
+    argv[1] = shellnam;
+    argv[2] = g_strdup("-c");
+    argv[3] = g_strdup(arg);
+    argv[4] = NULL;
+
+    g_spawn_sync(wdir, argv, NULL, 0, NULL, NULL,
+		 sout, &serr, &status, &gerr); 
+
+    g_free(argv[0]);
+    g_free(argv[2]);
+    g_free(argv[3]);
+
+    if (gerr != NULL) {
+	gretl_errmsg_set(gerr->message);
+	g_error_free(gerr);
+	err = 1;
+    }
+
+    if (serr != NULL) {
+	g_free(serr);
+    }
+
+    if (sout != NULL && *sout != NULL) {
+	/* trim trailing newline */
+	int n = strlen(*sout);
+
+	if ((*sout)[n-1] == '\n') {
+	    (*sout)[n-1] = '\0';
+	}
+    }
+
+    return err;
+}
+
+#endif
+
+static char *gretl_backtick (const char **pline, int *err)
+{
+    const char *s = *pline + 2;
+    char *arg = NULL;
+    char *val = NULL;
+    int n = 0, pcount = 1;
+
+    while (*s && pcount) {
+	if (*s == '(') pcount++;
+	else if (*s == ')') pcount--;
+	s++;
+	n++;
+    }
+
+    if (pcount || n < 2) {
+	*err = E_PARSE;
+	return NULL;
+    }
+
+    arg = gretl_strndup(*pline + 2, n - 1);
+    if (arg == NULL) {
+	*err = E_ALLOC;
+	return NULL;
+    }
+
+    fprintf(stderr, "arg = '%s'\n", arg);
+
+    *pline = s;
+    *err = shell_grab(arg, &val);
+
+    free(arg);
+    
+    return val;
+}
+
 static char *gretl_getenv (const char **pline, int *err)
 {
     const char *s = strchr(*pline, '"') + 1;
@@ -547,6 +818,21 @@ static char *gretl_getenv (const char **pline, int *err)
     return val;
 }
 
+static char *retrieve_arg_name (const char **pline, int *err)
+{
+    char argvar[VNAMELEN];
+    char *ret = NULL;
+
+    if (sscanf(*pline, "argname(%15[^)])", argvar) == 1) {
+	*pline = strchr(*pline, ')') + 1;
+	ret = gretl_func_get_arg_name(argvar);
+    } else {
+	*err = E_PARSE;
+    }
+
+    return ret;
+}
+
 static char *get_string_element (const char **pline, int *err)
 {
     const char *line = *pline;
@@ -557,6 +843,7 @@ static char *get_string_element (const char **pline, int *err)
     int n = 0;
 
     line += strspn(line, " \t");
+    *pline = line;
 
     if (*line == '@') {
 	/* should be saved string variable */
@@ -567,6 +854,14 @@ static char *get_string_element (const char **pline, int *err)
 	return gretl_getenv(pline, err);
     }
 
+    if (!strncmp(line, "argname(", 8)) {
+	return retrieve_arg_name(pline, err);
+    }
+
+    if (!strncmp(line, "$(", 2)) {
+	return gretl_backtick(pline, err);
+    }    
+
     if (*line != '"') {
 	*err = E_PARSE;
 	return NULL;
@@ -921,3 +1216,60 @@ int substitute_named_strings (char *line)
 
     return err;
 }
+
+/* setting of names of imported variables that should be treated
+   as string codes */
+
+static char **codevars;
+static int n_codevars;
+
+static void clean_up_codevars (void)
+{
+    if (codevars != NULL && n_codevars > 0) {
+	free_strings_array(codevars, n_codevars);
+	codevars = NULL;
+	n_codevars = 0;
+    }
+}
+
+int is_codevar (const char *s)
+{
+    int i;
+
+    for (i=0; i<n_codevars; i++) {
+	if (!strcmp(s, codevars[i])) {
+	    return 1;
+	}
+    }
+
+    return 0;
+}
+
+int set_codevars (const char *s)
+{
+    char chunk[32];
+    const char *p;
+    int err = 0;
+
+    p = strstr(s, "codevars");
+    if (p != NULL) {
+	s = p + 9;
+    }
+
+    *chunk = '\0';
+    sscanf(s, "%31s", chunk);
+
+    if (*chunk == '\0') {
+	err = E_DATA;
+    } else {
+	clean_up_codevars();
+	if (strcmp(chunk, "null")) {
+	    codevars = gretl_string_split(s, &n_codevars);
+	    if (codevars == NULL) {
+		err = E_ALLOC;
+	    } 
+	}
+    }
+
+    return err;
+}
