@@ -1,20 +1,20 @@
-/*
- *  Copyright (c) by Ramu Ramanathan and Allin Cottrell
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
- *
- *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
- *
+/* 
+ *  gretl -- Gnu Regression, Econometrics and Time-series Library
+ *  Copyright (C) 2001 Allin Cottrell and Riccardo "Jack" Lucchetti
+ * 
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ * 
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ * 
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 #include "libgretl.h"
@@ -35,7 +35,6 @@
 #ifndef WIN32
 # include <signal.h>
 # include <glib.h>
-# define USE_GSPAWN
 #endif
 
 /**
@@ -264,7 +263,33 @@ int gretl_isunits (int t1, int t2, const double *x)
     return ret;
 }
 
-#define FEWVALS 8
+/**
+ * gretl_isint:
+ * @x: data series to examine.
+ * @t1: starting observation.
+ * @t2: ending observation. 
+ * 
+ * Check whether variable @x contains only integer values over the
+ * given sample range (aside from any missing values).
+ *
+ * Returns: 1 if so, otherwise 0.
+ */
+
+int gretl_isint (int t1, int t2, const double *x)
+{
+    int t, ret = 1;
+
+    for (t=t1; t<=t2; t++) {
+	if (!na(x[t]) && x[t] != floor(x[t])) {
+	    ret = 0;
+	    break;
+	}
+    }
+
+    return ret;
+}
+
+#define FEWVALS 32
 
 static int few_vals (int t1, int t2, const double *x)
 {
@@ -307,7 +332,7 @@ static int few_vals (int t1, int t2, const double *x)
  * values.
  * 
  * Returns: 0 if test (a) is not passed or the number of distinct values
- * is > 8; else 1 if the number of distinct values is <= 8; else 2 if 
+ * is > 32; else 1 if the number of distinct values is <= 32; else 2 if 
  * the number of distinct values is <= 4.  A return of 1 is supposed
  * to indicate that it's "reasonable" to treat @x as discrete, while
  * a return of 2 indicates that it's probably ureasonable _not_ to
@@ -352,6 +377,43 @@ int gretl_isdiscrete (int t1, int t2, const double *x)
     return d;
 }
 
+ /**
+ * gretl_is_oprobit_ok:
+ * @x: data series to examine.
+ * @t1: starting observation.
+ * @t2: ending observation. 
+ *
+ * Checks the variable @x over the range @t1 to @t2 for its
+ * suitability as the dependent variable in an ordered probit
+ * analysis.  The criterion used is that the variable has a
+ * minimum value of zero and has only non-negative integer 
+ * values.
+ * 
+ * Returns: 1 if the test succeeds, otherwise 0.
+ */
+
+int gretl_is_oprobit_ok (int t1, int t2, const double *x)
+{
+    int t, n = 0, d = 1;
+    double xmin = 1;
+
+    for (t=t1; t<=t2; t++) {
+	if (na(x[t])) {
+	    continue;
+	}
+	n++;
+	if (x[t] != floor(x[t]) || x[t] < 0) {
+	    d = 0;
+	    break;
+	}
+	if (x[t] < xmin) {
+	    xmin = x[t];
+	}
+    }
+
+    return (d && xmin == 0 && n > 0);
+}
+
 /**
  * true_const:
  * @v: index number of variable to test.
@@ -1288,6 +1350,9 @@ int re_estimate (char *model_spec, MODEL *tmpmod,
     case POISSON:
 	*tmpmod = poisson_model(cmd.list, pZ, pdinfo, NULL);
 	break;
+    case HECKIT:
+	*tmpmod = heckit_model(cmd.list, pZ, pdinfo, cmd.opt, NULL);
+	break;
     case OLS:
     case WLS:
     case HCCM:
@@ -1319,7 +1384,6 @@ double get_xvalue (int i, const double **Z, const DATAINFO *pdinfo)
 }
 
 #ifndef WIN32
-# ifdef USE_GSPAWN
 
 static int font_not_found (const char *s)
 {
@@ -1336,7 +1400,8 @@ static int font_not_found (const char *s)
 int gretl_spawn (char *cmdline)
 {
     GError *error = NULL;
-    gchar *errout = NULL, *sout = NULL;
+    gchar *errout = NULL;
+    gchar *sout = NULL;
     int ok, status;
     int ret = 0;
 
@@ -1351,26 +1416,24 @@ int gretl_spawn (char *cmdline)
 				    &error);
 
     if (!ok) {
-	strcpy(gretl_errmsg, error->message);
+	gretl_errmsg_set(error->message);
 	fprintf(stderr, "gretl_spawn: '%s'\n", error->message);
 	g_error_free(error);
 	ret = 1;
     } else if (errout && *errout) {
 	fprintf(stderr, "stderr: '%s'\n", errout);
 	if (!font_not_found(errout)) {
-	    strcpy(gretl_errmsg, errout);
+	    gretl_errmsg_set(errout);
 	    fprintf(stderr, "gretl_errmsg: '%s'\n", gretl_errmsg);
 	    ret = 1;
 	}
     } else if (status != 0) {
-	if (sout != NULL) {
-	    sprintf(gretl_errmsg, "%s\n%s", 
-		    _("Command failed"),
-		    sout);
-	    fprintf(stderr, "status=%d: '%s'\n", status, sout);
+	if (sout != NULL && *sout) {
+	    gretl_errmsg_set(sout);
+	    fprintf(stderr, "gretl_spawn: status = %d: '%s'\n", status, sout);
 	} else {
 	    strcpy(gretl_errmsg, _("Command failed"));
-	    fprintf(stderr, "status=%d\n", status);
+	    fprintf(stderr, "gretl_spawn: status = %d\n", status);
 	}
 	ret = 1;
     }
@@ -1385,26 +1448,6 @@ int gretl_spawn (char *cmdline)
     return ret;
 }
 
-# else /* now non-glib2 version */
-
-int gretl_spawn (char *cmdline)
-{
-    int err;
-
-    errno = 0;
-
-    signal(SIGCHLD, SIG_DFL);
-
-    err = system(cmdline);
-    if (err) {
-	fprintf(stderr, "Failed command: '%s'\n", cmdline);
-	perror(NULL);
-    }
-
-    return err;
-}
-
-# endif
 #endif /* !WIN32 */
 
 /* file copying */
@@ -1506,36 +1549,49 @@ void libgretl_cleanup (void)
 enum {
     SET_TEST_STAT,
     GET_TEST_STAT,
-    GET_TEST_PVAL
+    GET_TEST_PVAL,
+    GET_TEST_LNL
 };
 
+#define getcode(c) (c == GET_TEST_STAT || c == GET_TEST_PVAL || \
+                    c == GET_TEST_LNL)
+
 static double
-record_or_get_test_result (double teststat, double pval, char *instr,
-			   int code)
+record_or_get_test_result (double teststat, double pval, double lnl,
+			   char *instr, int code)
 {
     static char savestr[MAXLABEL] = {0};
     static double val = NADBL;
     static double pv = NADBL;
+    static double ll = NADBL;
 
     double ret = NADBL;
 
     if (code == SET_TEST_STAT) {
 	val = teststat;
 	pv = pval;
+	ll = lnl;
 	*savestr = '\0';
 	if (instr != NULL) {
 	    strncat(savestr, instr, MAXLABEL - 1);
 	} 
-    } else if (code == GET_TEST_STAT || code == GET_TEST_PVAL) {
+    } else if (getcode(code)) {
 	if (instr != NULL) {
 	    if (code == GET_TEST_STAT) {
 		sprintf(instr, _("%s test"), savestr);
-	    } else {
-		/* GET_TEST_PVAL */
+	    } else if (code == GET_TEST_PVAL) {
 		sprintf(instr, _("p-value for %s test"), savestr);
+	    } else if (code == GET_TEST_LNL) {
+		sprintf(instr, _("log-likelihood for %s test"), savestr);
 	    }
 	}
-	ret = (code == GET_TEST_STAT)? val : pv;
+	if (code == GET_TEST_STAT) {
+	    ret = val;
+	} else if (code == GET_TEST_PVAL) {
+	    ret = pv;
+	} else if (code == GET_TEST_LNL) {
+	    ret = ll;
+	}
     } 
 	
     return ret;
@@ -1543,16 +1599,27 @@ record_or_get_test_result (double teststat, double pval, char *instr,
 
 void record_test_result (double teststat, double pval, char *blurb)
 {
-    record_or_get_test_result(teststat, pval, blurb, SET_TEST_STAT);
+    record_or_get_test_result(teststat, pval, NADBL, blurb, SET_TEST_STAT);
+}
+
+void record_LR_test_result (double teststat, double pval, double lnl,
+			    char *blurb)
+{
+    record_or_get_test_result(teststat, pval, lnl, blurb, SET_TEST_STAT);
 }
 
 double get_last_test_statistic (char *blurb)
 {
-    return record_or_get_test_result(0, 0, blurb, GET_TEST_STAT);
+    return record_or_get_test_result(0, 0, 0, blurb, GET_TEST_STAT);
 }
 
 double get_last_pvalue (char *blurb)
 {
-    return record_or_get_test_result(0, 0, blurb, GET_TEST_PVAL);
+    return record_or_get_test_result(0, 0, 0, blurb, GET_TEST_PVAL);
+}
+
+double get_last_lnl (char *blurb)
+{
+    return record_or_get_test_result(0, 0, 0, blurb, GET_TEST_LNL);
 }
 
