@@ -1,20 +1,20 @@
-/*
- * Copyright (C) 1999-2004 Ramu Ramanathan and Allin Cottrell
- *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License 
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
- *
- * This software is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License 
- * along with this software; if not, write to the 
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+/* 
+ *  gretl -- Gnu Regression, Econometrics and Time-series Library
+ *  Copyright (C) 2001 Allin Cottrell and Riccardo "Jack" Lucchetti
+ * 
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ * 
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ * 
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 #ifndef GRETL_UTILS_H
@@ -50,8 +50,12 @@ int gretl_isconst (int t1, int t2, const double *x);
 
 int gretl_isunits (int t1, int t2, const double *x);
 
+int gretl_isint (int t1, int t2, const double *x);
+
 int gretl_isdiscrete (int t1, int t2, const double *x);
 
+int gretl_is_oprobit_ok (int t1, int t2, const double *x);
+
 int true_const (int v, const double **Z, const DATAINFO *pdinfo);
 
 /* setting observations */
@@ -127,10 +131,15 @@ int ls_criteria (MODEL *pmod);
 
 void record_test_result (double teststat, double pval, char *blurb);
 
+void record_LR_test_result (double teststat, double pval, double lnl,
+			    char *blurb);
+
 double get_last_test_statistic (char *blurb);
 
 double get_last_pvalue (char *blurb);
 
+double get_last_lnl (char *blurb);
+
 /* timer */
 
 double gretl_stopwatch (void);
