@@ -1,20 +1,20 @@
-/*
- *  Copyright (c) by Allin Cottrell
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
- *
- *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
- *
+/* 
+ *  gretl -- Gnu Regression, Econometrics and Time-series Library
+ *  Copyright (C) 2001 Allin Cottrell and Riccardo "Jack" Lucchetti
+ * 
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ * 
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ * 
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 #include "libgretl.h"
@@ -89,6 +89,7 @@ int gretl_xml_open_doc_root (const char *fname,
 	if (xmlStrcmp(node->name, (XUC) rootname)) {
 	    sprintf(gretl_errmsg, _("File of the wrong type, root node not %s"),
 		    rootname);
+	    fprintf(stderr, "Unexpected root node '%s'\n", (char *) node->name);
 	    xmlFreeDoc(doc);
 	    err = 1;
 	}
@@ -184,10 +185,24 @@ static int savenum (const int *list, int i)
     }
 }
 
+/**
+ * gretl_xml_put_int:
+ * @tag: name to give value.
+ * @i: value to put (as attribute)
+ * @fp: file to which to write.
+ * 
+ * Writes to @fp a string of the form "%s=%d".
+ */
+
+void gretl_xml_put_int (const char *tag, int i, FILE *fp)
+{
+    fprintf(fp, "%s=\"%d\" ", tag, i);
+}
+
 /**
  * gretl_xml_put_double:
  * @tag: name to give value.
- * @x: value to put.
+ * @x: value to put (as attribute)
  * @fp: file to which to write.
  * 
  * Writes to @fp a string of the form "%s=%.15g" if the value of
@@ -713,7 +728,10 @@ int *gretl_xml_node_get_list (xmlNodePtr node, xmlDocPtr doc, int *err)
 	p += strspn(p, " \r\n");
 	if (sscanf(p, "%d", &n) != 1) {
 	    *err = E_DATA;
-	} else if (n <= 0) {
+	} else if (n == 0) {
+	    free(tmp);
+	    return NULL;
+	} else if (n < 0) {
 	    *err = E_DATA;
 	} else {
 	    p += strcspn(p, " \r\n");
@@ -752,7 +770,7 @@ static const char *skip_to_next (const char *s)
 }
 
 static void *gretl_xml_get_array (xmlNodePtr node, xmlDocPtr doc,
-				  ModelDataType type,
+				  GretlType type,
 				  int *nelem, int *err)
 {
     xmlChar *tmp = xmlGetProp(node, (XUC) "count");
@@ -767,13 +785,13 @@ static void *gretl_xml_get_array (xmlNodePtr node, xmlDocPtr doc,
 	n = atoi((const char *) tmp);
 	free(tmp);
 	if (n > 0) {
-	    if (type == MODEL_DATA_INT_ARRAY) {
+	    if (type == GRETL_TYPE_INT_ARRAY) {
 		ivals = malloc(n * sizeof *ivals);
 		ptr = ivals;
-	    } else if (type == MODEL_DATA_DOUBLE_ARRAY) {
+	    } else if (type == GRETL_TYPE_DOUBLE_ARRAY) {
 		xvals = malloc(n * sizeof *xvals);
 		ptr = xvals;
-	    } else if (type == MODEL_DATA_CMPLX_ARRAY) {
+	    } else if (type == GRETL_TYPE_CMPLX_ARRAY) {
 		cvals = malloc(n * sizeof *cvals);
 		ptr = cvals;
 	    }
@@ -787,17 +805,17 @@ static void *gretl_xml_get_array (xmlNodePtr node, xmlDocPtr doc,
 		    p = (const char *) tmp;
 		    p += strspn(p, " \r\n");
 		    for (i=0; i<n && !*err; i++) {
-			if (type == MODEL_DATA_INT_ARRAY) {
+			if (type == GRETL_TYPE_INT_ARRAY) {
 			    if (sscanf(p, "%d", &ivals[i]) != 1) {
 				*err = E_DATA;
 			    }
-			} else if (type == MODEL_DATA_DOUBLE_ARRAY) {
+			} else if (type == GRETL_TYPE_DOUBLE_ARRAY) {
 			    if (!strncmp(p, "NA", 2)) {
 				xvals[i] = NADBL;
 			    } else if (sscanf(p, "%lf", &xvals[i]) != 1) {
 				*err = E_DATA;
 			    }
-			} else if (type == MODEL_DATA_CMPLX_ARRAY) {
+			} else if (type == GRETL_TYPE_CMPLX_ARRAY) {
 			    if (sscanf(p, "%lf %lf", &cvals[i].r, &cvals[i].i) != 2) {
 				*err = E_DATA;
 			    } 
@@ -837,7 +855,7 @@ static void *gretl_xml_get_array (xmlNodePtr node, xmlDocPtr doc,
 int *gretl_xml_get_int_array (xmlNodePtr node, xmlDocPtr doc,
 			      int *nelem, int *err)
 {
-    return gretl_xml_get_array(node, doc, MODEL_DATA_INT_ARRAY,
+    return gretl_xml_get_array(node, doc, GRETL_TYPE_INT_ARRAY,
 			       nelem, err);
 }
 
@@ -855,7 +873,7 @@ int *gretl_xml_get_int_array (xmlNodePtr node, xmlDocPtr doc,
 double *gretl_xml_get_double_array (xmlNodePtr node, xmlDocPtr doc,
 				    int *nelem, int *err)
 {
-    return gretl_xml_get_array(node, doc, MODEL_DATA_DOUBLE_ARRAY,
+    return gretl_xml_get_array(node, doc, GRETL_TYPE_DOUBLE_ARRAY,
 			       nelem, err);
 }
 
@@ -873,7 +891,7 @@ double *gretl_xml_get_double_array (xmlNodePtr node, xmlDocPtr doc,
 cmplx *gretl_xml_get_cmplx_array (xmlNodePtr node, xmlDocPtr doc,
 				  int *nelem, int *err)
 {
-    return gretl_xml_get_array(node, doc, MODEL_DATA_CMPLX_ARRAY,
+    return gretl_xml_get_array(node, doc, GRETL_TYPE_CMPLX_ARRAY,
 			       nelem, err);
 }
 
@@ -2033,31 +2051,33 @@ static void data_read_message (const char *fname, DATAINFO *pdinfo, PRN *prn)
  * @ppdinfo: pointer to data information struct.
  * @fname: name of file to try.
  * @ppaths: path information struct.
- * @ocode: %DATA_NONE: no datafile currently open; %DATA_CLEAR: datafile
- * is open, should be cleared; %DATA_APPEND: add to current dataset.
+ * @opt: use %OPT_P to display gui progress bar.
  * @prn: where messages should be written.
- * @gui: should = 1 if the function is launched from the GUI, else 0.
  * 
  * Read data from file into gretl's work space, allocating space as
- * required.
+ * required.  If the array to which @pZ points is not %NULL, attempt
+ * to merge the new data with the original data.
  * 
  * Returns: 0 on successful completion, non-zero otherwise.
  */
 
 int gretl_read_gdt (double ***pZ, DATAINFO **ppdinfo, char *fname,
-		    PATHS *ppaths, DataOpenCode ocode, PRN *prn, int gui) 
+		    PATHS *ppaths, gretlopt opt, PRN *prn) 
 {
     DATAINFO *tmpdinfo;
     double **tmpZ = NULL;
     xmlDocPtr doc = NULL;
     xmlNodePtr cur;
+    int newdata = (*pZ == NULL);
     int gotvars = 0, gotobs = 0, err = 0;
     int caldata = 0;
     long fsz, progress = 0L;
 
     gretl_error_clear();
 
+#ifdef ENABLE_NLS
     check_for_console(prn);
+#endif
 
     tmpdinfo = datainfo_new();
     if (tmpdinfo == NULL) {
@@ -2074,7 +2094,7 @@ int gretl_read_gdt (double ***pZ, DATAINFO **ppdinfo, char *fname,
 	fprintf(stderr, "%s %ld bytes %s...\n", 
 		(is_gzipped(fname))? I_("Uncompressing") : I_("Reading"),
 		fsz, I_("of data"));
-	if (gui) progress = fsz;
+	if (opt & OPT_P) progress = fsz;
     }
 
     doc = gretl_xmlParseFile(fname);
@@ -2175,24 +2195,10 @@ int gretl_read_gdt (double ***pZ, DATAINFO **ppdinfo, char *fname,
 
     data_read_message(fname, tmpdinfo, prn);
 
-    if (ocode == DATA_APPEND) {
-	err = merge_data(pZ, *ppdinfo, tmpZ, tmpdinfo, prn);
-	if (err) {
-	    tmpZ = NULL;
-	    free(tmpdinfo);
-	    tmpdinfo = NULL;
-	}
-    } else {
-	if (ppaths != NULL && fname != ppaths->datfile) {
-	    strcpy(ppaths->datfile, fname);
-	}
-	free_Z(*pZ, *ppdinfo);
-	if (ocode == DATA_CLEAR) {
-	    clear_datainfo(*ppdinfo, CLEAR_FULL);
-	}
-	free(*ppdinfo);
-	*ppdinfo = tmpdinfo;
-	*pZ = tmpZ;
+    err = merge_or_replace_data(pZ, ppdinfo, &tmpZ, &tmpdinfo, prn);
+
+    if (!err && newdata && ppaths != NULL && fname != ppaths->datfile) {
+	strcpy(ppaths->datfile, fname);
     }
 
  bailout:
@@ -2205,25 +2211,25 @@ int gretl_read_gdt (double ***pZ, DATAINFO **ppdinfo, char *fname,
     /* pre-process stacked cross-sectional panels: put into canonical
        stacked time series form
     */
-    if (!err && tmpdinfo->structure == STACKED_CROSS_SECTION) {
-	err = switch_panel_orientation(tmpZ, tmpdinfo);
+    if (!err && (*ppdinfo)->structure == STACKED_CROSS_SECTION) {
+	err = switch_panel_orientation(*pZ, *ppdinfo);
     }
 
-    if (!err && tmpdinfo->structure == STACKED_TIME_SERIES) {
-	if (tmpdinfo->paninfo == NULL) {
-	    err = dataset_add_default_panel_indices(tmpdinfo);
+    if (!err && (*ppdinfo)->structure == STACKED_TIME_SERIES) {
+	if ((*ppdinfo)->paninfo == NULL) {
+	    err = dataset_add_default_panel_indices(*ppdinfo);
 	} else {
-	    err = dataset_finalize_panel_indices(tmpdinfo);
+	    err = dataset_finalize_panel_indices(*ppdinfo);
 	}
     }
 
-    if (err) {
-	free_Z(tmpZ, tmpdinfo);
-	clear_datainfo(tmpdinfo, CLEAR_FULL);
-	free(tmpdinfo);
+    if (err && tmpdinfo != NULL) {
+	destroy_dataset(tmpZ, tmpdinfo);
     }
 
+#ifdef ENABLE_NLS
     console_off();
+#endif
 
     return err;
 }
