@@ -1,20 +1,20 @@
-/*
- *  Copyright (c) by Allin Cottrell
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
- *
- *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
- *
+/* 
+ *  gretl -- Gnu Regression, Econometrics and Time-series Library
+ *  Copyright (C) 2001 Allin Cottrell and Riccardo "Jack" Lucchetti
+ * 
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ * 
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ * 
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 #ifndef GRETL_XML_H
@@ -31,6 +31,8 @@ int gretl_xml_open_doc_root (const char *fname,
 			     xmlDocPtr *pdoc, 
 			     xmlNodePtr *pnode);
 
+void gretl_xml_put_int (const char *tag, int i, FILE *fp);
+
 void gretl_xml_put_double (const char *tag, double x, FILE *fp);
 
 void gretl_xml_put_double_array (const char *tag, double *x, int n,
@@ -111,7 +113,7 @@ int gretl_write_gdt (const char *fname, const int *list,
 		     GretlDataFormat fmt, PATHS *ppaths);
 
 int gretl_read_gdt (double ***pZ, DATAINFO **ppdinfo, char *fname,
-		    PATHS *ppaths, DataOpenCode ocode, PRN *prn, int gui);
+		    PATHS *ppaths, gretlopt opt, PRN *prn);
 
 char *gretl_get_gdt_description (const char *fname);
 
