@@ -1,20 +1,20 @@
-/*
- *  Copyright (c) by Allin Cottrell
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
- *
- *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
- *
+/* 
+ *  gretl -- Gnu Regression, Econometrics and Time-series Library
+ *  Copyright (C) 2001 Allin Cottrell and Riccardo "Jack" Lucchetti
+ * 
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ * 
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ * 
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 #include "libgretl.h"
@@ -324,6 +324,8 @@ kalman *kalman_new (const gretl_matrix *S, const gretl_matrix *P,
     K->sumVt = NADBL;
     K->loglik = NADBL;
 
+    clear_gretl_matrix_err();
+
     K->S0 = gretl_matrix_copy(S);
     K->S1 = gretl_matrix_copy(S);
 
@@ -372,15 +374,8 @@ kalman *kalman_new (const gretl_matrix *S, const gretl_matrix *P,
     K->Tmprr_2a = gretl_matrix_alloc(K->r, K->r);
     K->Tmprr_2b = gretl_matrix_alloc(K->r, K->r);
 
-    if (K->S0 == NULL || K->S1 == NULL || K->P0 == NULL || K->P1 == NULL ||
-	K->e == NULL || K->F == NULL || K->A == NULL ||
-	K->H == NULL || K->Q == NULL || 
-	K->PH == NULL || K->HPH == NULL ||
-	K->FPH == NULL || K->V == NULL || K->VE == NULL ||
-	K->PHV == NULL || K->Ax == NULL ||
-	K->Tmpnn == NULL || K->Tmprr == NULL || 
-	K->Tmprr_2a == NULL || K->Tmprr_2b == NULL) {
-	*err = E_ALLOC;
+    *err = get_gretl_matrix_err();
+    if (*err) {
 	kalman_free(K);
 	K = NULL;
     } else {
