@@ -1,20 +1,20 @@
-/*
- *  Copyright (c) by Allin Cottrell and Riccardo "Jack" Lucchetti
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
- *
- *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- *
+/* 
+ *  gretl -- Gnu Regression, Econometrics and Time-series Library
+ *  Copyright (C) 2001 Allin Cottrell and Riccardo "Jack" Lucchetti
+ * 
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ * 
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ * 
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 #ifndef MATRIX_EXTRA_H
@@ -47,7 +47,8 @@ int gretl_matrix_row_to_array (const gretl_matrix *m, int i, double *x);
 
 
 gretl_matrix *gretl_matrix_data_subset (const int *list, const double **Z,
-					int t1, int t2, const char *mask);
+					int t1, int t2, const char *mask,
+					int *err);
 
 gretl_matrix *
 gretl_matrix_data_subset_no_missing (const int *list, const double **Z,
@@ -61,4 +62,10 @@ int gretl_plotfit_matrices (int yno, int xno, FitType fit,
 			    const double **Z, int t1, int t2, 
 			    gretl_matrix **py, gretl_matrix **pX);
 
+int gretl_matrix_delete_columns (gretl_matrix *X, int *list);
+
+gretl_matrix *gretl_matrix_read_from_text (const char *fname, int *err);
+
+int gretl_matrix_write_as_text (gretl_matrix *A, const char *fname);
+
 #endif /* MATRIX_EXTRA_H */
