@@ -1,20 +1,20 @@
-/*
- *  Copyright (c) by Ramu Ramanathan and Allin Cottrell
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
- *
- *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111, USA.
- *
+/* 
+ *  gretl -- Gnu Regression, Econometrics and Time-series Library
+ *  Copyright (C) 2001 Allin Cottrell and Riccardo "Jack" Lucchetti
+ * 
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ * 
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ * 
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 /* modelprint.c */ 
@@ -24,7 +24,8 @@
 #include "system.h"
 #include "texprint.h"
 
-#define NO_RBAR_SQ(a) (a == AUX_SQ || a == AUX_LOG || a == AUX_WHITE || a == AUX_AR)
+#define NO_RBAR_SQ(a) (a == AUX_SQ || a == AUX_LOG || a == AUX_WHITE || \
+                       a == AUX_AR || a == AUX_VAR || a == AUX_HET_1)
 
 static int 
 print_coefficients (const MODEL *pmod, const DATAINFO *pdinfo, PRN *prn);
@@ -36,6 +37,7 @@ static void print_binary_statistics (const MODEL *pmod,
 static void print_arma_stats (const MODEL *pmod, PRN *prn);
 static void print_arma_roots (const MODEL *pmod, PRN *prn);
 static void print_tobit_stats (const MODEL *pmod, PRN *prn);
+static void print_heckit_stats (const MODEL *pmod, PRN *prn);
 static void print_poisson_offset (const MODEL *pmod, const DATAINFO *pdinfo, 
 				  PRN *prn);
 static void print_ll (const MODEL *pmod, PRN *prn);
@@ -186,21 +188,25 @@ real_essline (const MODEL *pmod, double ess, double sigma, PRN *prn)
 
 static int GMM_crit_line (const MODEL *pmod, PRN *prn)
 {
-    if (plain_format(prn)) {    
-	pprintf(prn, "  %s = %.*g\n", _("GMM criterion"), 
-		XDIGITS(pmod), pmod->ess);
-    } else if (rtf_format(prn)) {
-	pprintf(prn, RTFTAB "%s = %g\n", I_("GMM criterion"), 
-		pmod->ess);
-    } else if (tex_format(prn)) {
-	char xstr[32];
+    double Q = pmod->ess;
+    double TQ = pmod->ess * pmod->nobs;
 
-	tex_dcolumn_double(pmod->ess, xstr);
-	pprintf(prn, "%s & %s \\\\\n",
-		I_("GMM criterion"), xstr);
+    if (plain_format(prn)) {    
+	pprintf(prn, "  %s: Q = %.*g (TQ = %.*g)\n", _("GMM criterion"), 
+		XDIGITS(pmod), Q, XDIGITS(pmod), TQ);
+    } else if (rtf_format(prn)) {
+	pprintf(prn, RTFTAB "%s: Q = %g (TQ = %g)\n", I_("GMM criterion"), 
+		Q, TQ);
+    } else if (tex_format(prn)) {
+	char x1[32], x2[32];
+
+	tex_dcolumn_double(Q, x1);
+	tex_dcolumn_double(TQ, x2);
+	pprintf(prn, "%s, $Q$ & %s ($TQ$ = %s)\\\\\n",
+		I_("GMM criterion"), x1, x2);
     } else if (csv_format(prn)) {
 	pprintf(prn, "\"%s\"%c%.15g\n", I_("GMM criterion"), 
-		prn_delim(prn), pmod->ess);
+		prn_delim(prn), Q);
     }	
 
     return 0;
@@ -329,11 +335,13 @@ static void info_stats_lines (const MODEL *pmod, PRN *prn)
     const double *crit = pmod->criterion;
 
     if (pmod->aux == AUX_SQ || pmod->aux == AUX_LOG ||
-	pmod->aux == AUX_WHITE || pmod->aux == AUX_AR) {
+	pmod->aux == AUX_WHITE || pmod->aux == AUX_AR ||
+	pmod->aux == AUX_HET_1) {
 	return;
     }
 
-    if (na(crit[C_AIC]) || na(crit[C_BIC]) || na(crit[C_HQC])) {
+    if (na(crit[C_AIC]) || na(crit[C_BIC]) || na(crit[C_HQC]) ||
+	pmod->aux == AUX_VAR) {
 	return;
     }
 
@@ -651,7 +659,7 @@ static void Fline (const MODEL *pmod, PRN *prn)
 	    pprintf(prn, "  %s %s\n", tmp, _("undefined"));
 	} else {
 	    pprintf(prn, "  %s = %.*g", tmp, XDIGITS(pmod), pmod->fstt);
-	    print_f_pval_str(f_cdf_comp(pmod->fstt, pmod->dfn, pmod->dfd), prn);
+	    print_f_pval_str(snedecor_cdf_comp(pmod->fstt, pmod->dfn, pmod->dfd), prn);
 	}
     } else if (tex_format(prn)) {
 	if (na(pmod->fstt)) {
@@ -662,7 +670,7 @@ static void Fline (const MODEL *pmod, PRN *prn)
 
 	    tex_dcolumn_double(pmod->fstt, x1str);
 	    pprintf(prn, "$F(%d, %d)$ & %s \\\\\n", pmod->dfn, pmod->dfd, x1str);
-	    print_f_pval_str(f_cdf_comp(pmod->fstt, pmod->dfn, pmod->dfd), prn);
+	    print_f_pval_str(snedecor_cdf_comp(pmod->fstt, pmod->dfn, pmod->dfd), prn);
 	}
     } else if (rtf_format(prn)) {
 	char tmp[32];
@@ -672,7 +680,7 @@ static void Fline (const MODEL *pmod, PRN *prn)
 	    pprintf(prn, RTFTAB "%s %s\n", tmp, I_("undefined"));
 	} else {
 	    pprintf(prn, RTFTAB "%s = %g", tmp, pmod->fstt);
-	    print_f_pval_str(f_cdf_comp(pmod->fstt, pmod->dfn, pmod->dfd), prn);
+	    print_f_pval_str(snedecor_cdf_comp(pmod->fstt, pmod->dfn, pmod->dfd), prn);
 	}
     } else if (csv_format(prn)) {
 	pprintf(prn, "\"%s (%d, %d)\"%c", I_("F-statistic"), pmod->dfn, pmod->dfd,
@@ -681,7 +689,7 @@ static void Fline (const MODEL *pmod, PRN *prn)
 	    pprintf(prn, "\"%s\"\n", I_("undefined"));
 	} else {
 	    pprintf(prn, "%.15g%c", pmod->fstt, prn_delim(prn));
-	    print_f_pval_str(f_cdf_comp(pmod->fstt, pmod->dfn, pmod->dfd), prn);
+	    print_f_pval_str(snedecor_cdf_comp(pmod->fstt, pmod->dfn, pmod->dfd), prn);
 	}
     }	
 }
@@ -768,7 +776,7 @@ static void dhline (const MODEL *pmod, PRN *prn)
     }
 }
 
-static int least_signif_coeff (const MODEL *pmod)
+static int least_significant_coeff (const MODEL *pmod)
 {
     double tstat, tmin = 4.0;
     int i, k = 0;
@@ -795,7 +803,7 @@ static void pval_max_line (const MODEL *pmod, const DATAINFO *pdinfo,
 
     if (k < 3) return;
 
-    if ((k = least_signif_coeff(pmod))) {
+    if ((k = least_significant_coeff(pmod))) {
 	char tmp[128];
 
 	if (pmod->ifc) {
@@ -819,6 +827,8 @@ static const char *aux_string (int aux, PRN *prn)
 		 "(log terms)");
     } else if (aux == AUX_WHITE) {
 	return N_("White's test for heteroskedasticity");
+    } else if (aux == AUX_HET_1) {
+	return N_("Pesaran-Taylor test for heteroskedasticity");
     } else if (aux == AUX_CHOW) {
 	return N_("Augmented regression for Chow test");
     } else if (aux == AUX_COINT) {
@@ -844,16 +854,17 @@ static const char *aux_string (int aux, PRN *prn)
 static const char *simple_estimator_string (int ci, PRN *prn)
 {
     if (ci == OLS || ci == VAR) return N_("OLS");
-    else if (ci == WLS) return N_("WLS"); 
+    else if (ci == WLS)  return N_("WLS"); 
     else if (ci == ARCH) return N_("WLS (ARCH)");
     else if (ci == TSLS) return N_("TSLS");
-    else if (ci == HSK) return N_("Heteroskedasticity-corrected");
-    else if (ci == AR) return N_("AR");
-    else if (ci == LAD) return N_("LAD");
+    else if (ci == HSK)  return N_("Heteroskedasticity-corrected");
+    else if (ci == AR)   return N_("AR");
+    else if (ci == LAD)  return N_("LAD");
     else if (ci == MPOLS) return N_("High-Precision OLS");
     else if (ci == PROBIT) return N_("Probit");
-    else if (ci == LOGIT) return N_("Logit");
-    else if (ci == TOBIT) return N_("Tobit");
+    else if (ci == LOGIT)  return N_("Logit");
+    else if (ci == TOBIT)  return N_("Tobit");
+    else if (ci == HECKIT) return N_("Heckit");
     else if (ci == POISSON) return N_("Poisson");
     else if (ci == NLS) return N_("NLS");
     else if (ci == MLE) return N_("ML");
@@ -888,17 +899,17 @@ const char *estimator_string (const MODEL *pmod, PRN *prn)
 	} else {
 	    return N_("ARMA");
 	}
-    } else if (pmod->ci == WLS) {
-	if (gretl_model_get_int(pmod, "iters")) {
-	    return N_("Maximum Likelihood");
-	} else {
-	    return N_("WLS");
-	}
     } else if (pmod->ci == PANEL) {
 	if (gretl_model_get_int(pmod, "fixed-effects")) {
 	    return N_("Fixed-effects");
 	} else if (gretl_model_get_int(pmod, "random-effects")) {
 	    return N_("Random-effects (GLS)");
+	} else if (gretl_model_get_int(pmod, "unit-weights")) {
+	    if (gretl_model_get_int(pmod, "iters")) {
+		return N_("Maximum Likelihood");
+	    } else {
+		return N_("WLS");
+	    }
 	} else {
 	    return N_("Between-groups");
 	}
@@ -911,12 +922,14 @@ const char *estimator_string (const MODEL *pmod, PRN *prn)
 	    return N_("1-step Arellano-Bond");
 	}
     } else if (pmod->ci == GMM) {
-	int s = gretl_model_get_int(pmod, "step");
-
-	if (s > 2) {
-	    return N_("Iterated GMM");
-	} else if (s == 2) {
+	if (gretl_model_get_int(pmod, "two-step")) {
 	    return N_("2-step GMM");
+	} else if (gretl_model_get_int(pmod, "iterated")) {
+	    return N_("Iterated GMM");
+	} else if (gretl_model_get_int(pmod, "step") == 2) {
+	    return N_("2-step GMM");
+	} else if (gretl_model_get_int(pmod, "step") > 2) {
+	    return N_("Iterated GMM");
 	} else {
 	    return N_("1-step GMM");
 	}	
@@ -932,6 +945,12 @@ const char *estimator_string (const MODEL *pmod, PRN *prn)
 	} else {
 	    return N_("Probit");
 	}
+    } else if (pmod->ci == HECKIT) {
+	if (gretl_model_get_int(pmod, "two-step")) {
+	    return N_("Two-step Heckit");
+	} else {
+	    return N_("ML Heckit");
+	}	
     } else {
 	return simple_estimator_string(pmod->ci, prn);
     }
@@ -1135,12 +1154,12 @@ static void hac_vcv_line (const MODEL *pmod, PRN *prn)
 	N_("Parzen kernel"),
 	N_("QS kernel")
     };
-    int kern = gretl_model_get_int(pmod, "hac_kernel");
+    int k = gretl_model_get_int(pmod, "hac_kernel");
     int h = gretl_model_get_int(pmod, "hac_lag");
     int white = gretl_model_get_int(pmod, "hac_prewhiten");
     double bt;
 
-    if (kern == KERNEL_QS) {
+    if (k == KERNEL_QS) {
 	bt = gretl_model_get_double(pmod, "qs_bandwidth");
 	if (plain_format(prn)) {
 	    pprintf(prn, _("HAC standard errors, "
@@ -1161,9 +1180,9 @@ static void hac_vcv_line (const MODEL *pmod, PRN *prn)
 
     pputc(prn, ' ');
     if (plain_format(prn)) {
-	pprintf(prn, "(%s", _(kstrs[kern - 1]));
+	pprintf(prn, "(%s", _(kstrs[k]));
     } else {
-	pprintf(prn, "(%s", I_(kstrs[kern - 1]));
+	pprintf(prn, "(%s", I_(kstrs[k]));
 		    
     }
     if (white) {
@@ -1274,7 +1293,9 @@ static void tex_arbond_depvar_name (char *s, const char *vname)
 
 void print_model_vcv_info (const MODEL *pmod, PRN *prn)
 {
-    if (gretl_model_get_int(pmod, "hac_kernel")) {
+    if (gretl_model_get_int(pmod, "using_hac") ||
+	gretl_model_get_int(pmod, "hac_kernel") ||
+	gretl_model_get_int(pmod, "hac_lag")) {
 	hac_vcv_line(pmod, prn);
     } else if (gretl_model_get_int(pmod, "hc")) {
 	hc_vcv_line(pmod, prn);
@@ -1461,6 +1482,7 @@ static void print_model_heading (const MODEL *pmod,
     case AUX_SQ:
     case AUX_LOG:
     case AUX_WHITE:
+    case AUX_HET_1:	
     case AUX_CHOW:
     case AUX_COINT:
     case AUX_ADF:
@@ -1480,11 +1502,19 @@ static void print_model_heading (const MODEL *pmod,
 	break;
     case AUX_AR:
 	order = gretl_model_get_int(pmod, "BG_order");
-	if (utf) { 	
-	    pprintf(prn, "\n%s ", _("Breusch-Godfrey test for"));
+	if (pmod->ci == TSLS) {
+	    if (utf) { 	
+		pprintf(prn, "\n%s ", _("Godfrey (1994) test for"));
+	    } else {
+		pprintf(prn, "\n%s ", I_("Godfrey (1994) test for"));
+	    } 
 	} else {
-	    pprintf(prn, "\n%s ", I_("Breusch-Godfrey test for"));
-	} 
+	    if (utf) { 	
+		pprintf(prn, "\n%s ", _("Breusch-Godfrey test for"));
+	    } else {
+		pprintf(prn, "\n%s ", I_("Breusch-Godfrey test for"));
+	    } 
+	}
 	if (order > 1) {
 	    pprintf(prn, "%s %d\n", (utf)? _("autocorrelation up to order") :
 		    I_("autocorrelation up to order"), 
@@ -1513,6 +1543,9 @@ static void print_model_heading (const MODEL *pmod,
 	pprintf(prn, "%s %d: ", 
 		(utf)? _("Equation") : I_("Equation"), pmod->ID);
 	break;
+    case AUX_AUX:
+	pputc(prn, '\n');
+	break;
     case AUX_ADD:
     default:
 	if (pmod->ID < 0 || (opt & OPT_S)) {
@@ -1541,21 +1574,29 @@ static void print_model_heading (const MODEL *pmod,
 	pprintf(prn, (utf)?
 		_("%s estimates using the %d observations %s%s%s") :
 		I_("%s estimates using the %d observations %s%s%s"),
-		_(gretl_system_short_string(pmod)),
+		_(system_short_string(pmod)),
 		pmod->nobs, startdate, (tex)? "--" : "-", enddate);
     } else if (!dataset_is_panel(pdinfo)) {
 	if (pmod->missmask != NULL) {
 	    int mc = model_missval_count(pmod);
 
+	    if (pmod->ci == HECKIT) {
+		int Tmax = pmod->t2 - pmod->t1 + 1;
+		
+		mc = Tmax - gretl_model_get_int(pmod, "totobs");
+	    }
+
 	    pprintf(prn, (utf)?
 		    _("%s estimates using %d observations from %s%s%s") :
 		    I_("%s estimates using %d observations from %s%s%s"),
 		    _(estimator_string(pmod, prn)), 
 		    pmod->nobs, startdate, (tex)? "--" : "-", enddate);
-	    gretl_prn_newline(prn);
-	    pprintf(prn, "%s: %d",
-		    (utf)? _("Missing or incomplete observations dropped") :
-		    I_("Missing or incomplete observations dropped"), mc);
+	    if (mc > 0) {
+		gretl_prn_newline(prn);
+		pprintf(prn, "%s: %d",
+			(utf)? _("Missing or incomplete observations dropped") :
+			I_("Missing or incomplete observations dropped"), mc);
+	    }
 	} else {
 	    pprintf(prn, (utf)?
 		    _("%s estimates using the %d observations %s%s%s") :
@@ -1608,7 +1649,7 @@ static void print_model_heading (const MODEL *pmod,
 	pprintf(prn, "%s: %s", 
 		(utf)? _("Dependent variable") : I_("Dependent variable"),
 		(tex)? "$\\hat{u}$" : "uhat");
-    } else if (pmod->aux == AUX_WHITE) {
+    } else if (pmod->aux == AUX_WHITE || pmod->aux == AUX_HET_1) {
 	pprintf(prn, "%s: %s", 
 		(utf)? _("Dependent variable") : I_("Dependent variable"),
 		(tex)? "$\\hat{u}^2$" : "uhat^2");
@@ -1634,24 +1675,25 @@ static void print_model_heading (const MODEL *pmod,
     } else if (pmod->ci == ARMA) {
 	print_arma_depvar(pmod, pdinfo, prn);
     } else { 
-	int v = gretl_model_get_depvar(pmod);
+	const char *dvname = 
+	    gretl_model_get_depvar_name(pmod, pdinfo);
 
 	if (tex) {
 	    if (pmod->aux == AUX_VECM) {
-		tex_vecm_depvar_name(vname, pdinfo->varname[v]);
+		tex_vecm_depvar_name(vname, dvname);
 	    } else if (pmod->ci == ARBOND) {
-		tex_arbond_depvar_name(vname, pdinfo->varname[v]);
+		tex_arbond_depvar_name(vname, dvname);
 	    } else {
-		tex_escape(vname, pdinfo->varname[v]);
+		tex_escape(vname, dvname);
 	    }
 	}
 
 	if (pmod->aux == AUX_VAR || pmod->aux == AUX_VECM) {
-	    pputs(prn, (tex)? vname : pdinfo->varname[v]);
+	    pputs(prn, (tex)? vname : dvname);
 	} else {
 	    pprintf(prn, "%s: %s", 
 		    (utf)? _("Dependent variable") : I_("Dependent variable"),
-		    (tex)? vname : pdinfo->varname[v]);
+		    (tex)? vname : dvname);
 	}
     }
 
@@ -1667,7 +1709,7 @@ static void print_model_heading (const MODEL *pmod,
     if (pmod->ci == TSLS) {
 	int method = gretl_model_get_int(pmod, "method");
 
-	if (method != SYS_FIML && method != SYS_LIML) {
+	if (method != SYS_METHOD_FIML && method != SYS_METHOD_LIML) {
 	    print_tsls_instruments(pmod->list, pdinfo, prn);
 	}
     }
@@ -1676,7 +1718,7 @@ static void print_model_heading (const MODEL *pmod,
     print_model_vcv_info(pmod, prn);
 
     /* WLS on panel data */
-    if (gretl_model_get_int(pmod, "unit_weights") && !pmod->aux) {
+    if (gretl_model_get_int(pmod, "unit-weights") && !pmod->aux) {
 	if (tex) {
 	    pputs(prn, "\\\\\n");
 	}
@@ -1806,7 +1848,7 @@ void print_coeff_heading (int mode, PRN *prn)
 
 static void print_coeff_table_start (const MODEL *pmod, PRN *prn)
 {
-    int binary = binary_model(pmod);
+    int slopes = binary_model(pmod) && !gretl_model_get_int(pmod, "show-pvals");
     int use_param = pmod->ci == NLS || pmod->ci == MLE || pmod->ci == GMM;
 
     if (plain_format(prn)) {
@@ -1814,7 +1856,7 @@ static void print_coeff_table_start (const MODEL *pmod, PRN *prn)
 	    pputs(prn, _("      VARIABLE            COEFFICIENT          "
 		       "        STDERROR\n"));
 	    pputc(prn, '\n');
-	} else if (binary) {
+	} else if (slopes) {
 	    pputs(prn, _("      VARIABLE       COEFFICIENT        STDERROR"
 			 "      T STAT       SLOPE\n"));
 	    pprintf(prn, "                                                 "
@@ -1829,7 +1871,7 @@ static void print_coeff_table_start (const MODEL *pmod, PRN *prn)
 	if (pmod->ci == MPOLS) {
 	    pprintf(prn, "\"%s\"%c\"%s\"%c\"%s\"\n",
 		    I_("VARIABLE"), d, I_("COEFFICIENT"), d, I_("STDERROR"));
-	} else if (binary) {
+	} else if (slopes) {
 	    pprintf(prn, "\"%s\"%c\"%s\"%c\"%s\"%c\"%s\"%c\"%s\"\n",
 		    I_("VARIABLE"), d, I_("COEFFICIENT"), d, I_("STDERROR"),
 		    d, I_("T STAT"), d, I_("SLOPE at mean"));
@@ -1854,12 +1896,12 @@ static void print_coeff_table_start (const MODEL *pmod, PRN *prn)
 	}	    
 
 	if (tex_format(prn)) {
-	    tex_coeff_table_start(col1, col2, binary, prn);
+	    tex_coeff_table_start(col1, col2, slopes, prn);
 	    return;
 	}   
 
 	if (rtf_format(prn)) {
-	    if (binary) {
+	    if (slopes) {
 		pprintf(prn, "{" RTF_BINARY_ROW
 			" \\qc {\\i %s}\\cell"
 			" \\qc {\\i %s}\\cell"
@@ -1939,6 +1981,17 @@ static void print_middle_table_end (PRN *prn)
     }
 }
 
+static void addconst_message (const MODEL *pmod, PRN *prn)
+{
+    if (gretl_model_get_int(pmod, "addconst")) {
+	pprintf(prn, "\n\n%s\n", 
+		_("WARNING: The constant was present among the regressors but not among the\n"
+		  "instruments, so it has been automatically added to the instrument list.\n"
+		  "This behavior may change in future versions, so you may want to adjust your\n"
+		  "scripts accordingly.\n"));
+    }
+}
+
 static void r_squared_message (const MODEL *pmod, PRN *prn)
 {
     if (na(pmod->rsq)) {
@@ -1991,27 +2044,44 @@ static void rho_differenced_stats_message (PRN *prn)
 
 static void print_whites_results (const MODEL *pmod, PRN *prn)
 {
+    double X = pmod->rsq * pmod->nobs;
+    int df = pmod->ncoeff - 1;
+    double pv = chisq_cdf_comp(X, df);
+
     if (plain_format(prn)) {
-	pprintf(prn, "\n%s: TR^2 = %f,\n", _("Test statistic"), 
-		pmod->rsq * pmod->nobs);
+	pprintf(prn, "\n%s: TR^2 = %f,\n", _("Test statistic"), X);
 	pprintf(prn, "%s = P(%s(%d) > %f) = %f\n\n", 
-		_("with p-value"), _("Chi-square"), 
-		pmod->ncoeff - 1, pmod->rsq * pmod->nobs,
-		chisq_cdf_comp(pmod->rsq * pmod->nobs, pmod->ncoeff - 1)); 
+		_("with p-value"), _("Chi-square"), df, X, pv);
     } else if (rtf_format(prn)) { /* FIXME */
 	pprintf(prn, "\\par \\ql\n%s: TR{\\super 2} = %f,\n", I_("Test statistic"), 
-		pmod->rsq * pmod->nobs);
+		X);
 	pprintf(prn, "%s = P(%s(%d) > %f) = %f\n\n", 
-		I_("with p-value"), I_("Chi-square"), 
-		pmod->ncoeff - 1, pmod->rsq * pmod->nobs,
-		chisq_cdf_comp(pmod->rsq * pmod->nobs, pmod->ncoeff - 1)); 
+		I_("with p-value"), I_("Chi-square"), df, X, pv);
     } else if (tex_format(prn)) {
-	pprintf(prn, "\n%s: $TR^2$ = %f,\n", I_("Test statistic"), 
-		pmod->rsq * pmod->nobs);
+	pprintf(prn, "\n%s: $TR^2$ = %f,\n", I_("Test statistic"), X);
 	pprintf(prn, "%s = $P$($\\chi^2(%d)$ > %f) = %f\n\n",
-		I_("with p-value"), 
-		pmod->ncoeff - 1, pmod->rsq * pmod->nobs,
-		chisq_cdf_comp(pmod->rsq * pmod->nobs, pmod->ncoeff - 1)); 
+		I_("with p-value"), df, X, pv);
+    }
+}
+
+static void print_HET_1_results (const MODEL *pmod, PRN *prn)
+{
+    double z = fabs(pmod->coeff[1]) / pmod->sderr[1];
+    double pv = 2.0 * (1 - normal_cdf(z));
+
+    if (plain_format(prn)) {
+	pprintf(prn, "\n%s: HET_1 = |%f| / %f = %f,\n", _("Test statistic"), 
+		pmod->coeff[1], pmod->sderr[1], z);
+	pprintf(prn, "%s = 2 * P(z > %f) = %.3g\n\n", 
+		_("with p-value"), z, pv);
+    } else if (rtf_format(prn)) { /* FIXME */
+	pprintf(prn, "\\par \\ql\n%s: HET_1 = %f,\n", I_("Test statistic"), z);
+	pprintf(prn, "%s = 2 * P(z > %f) = %.3g\n\n", 
+		I_("with p-value"), z, pv);
+    } else if (tex_format(prn)) {
+	pprintf(prn, "\n%s: \verb|HET_1| = %f,\n", I_("Test statistic"), z);
+	pprintf(prn, "%s = $2 \times P$($z$ > %f) = %f\n\n",
+		I_("with p-value"), z, pv);
     }
 }
 
@@ -2071,6 +2141,17 @@ static char active_decpoint (void)
 #define between_model(m) (m->ci == PANEL && \
                           gretl_model_get_int(m, "between"))
 
+#define weighted_model(m) (m->ci == HSK || m->ci == ARCH || \
+			   (m->ci == WLS && !gretl_model_get_int(m, "wt_dummy")) || \
+                           (m->ci == PANEL && gretl_model_get_int(m, "unit-weights")))
+
+#define panel_ML_model(m) (m->ci == PANEL && \
+                           gretl_model_get_int(m, "unit-weights") && \
+			   gretl_model_get_int(m, "iters"))
+
+#define non_weighted_panel(m) (m->ci == PANEL && \
+			       !gretl_model_get_int(m, "unit-weights"))
+
 /**
  * printmodel:
  * @pmod: pointer to gretl model.
@@ -2199,11 +2280,20 @@ int printmodel (MODEL *pmod, const DATAINFO *pdinfo, gretlopt opt,
 	goto close_format;
     }   
 
+    if (pmod->ci == HECKIT) {
+	print_middle_table_start(prn);
+	depvarstats(pmod, prn);
+	print_heckit_stats(pmod, prn);
+	print_ll(pmod, prn);
+	print_middle_table_end(prn);
+	goto close_format;
+    }
+	
     if (pmod->aux == AUX_SYS) {
 	print_middle_table_start(prn);
 	depvarstats(pmod, prn);
 	essline(pmod, prn);
-	if (gretl_model_get_int(pmod, "method") == SYS_LIML) {
+	if (gretl_model_get_int(pmod, "method") == SYS_METHOD_LIML) {
 	    print_liml_equation_data(pmod, prn);
 	}
 	print_middle_table_end(prn);
@@ -2233,6 +2323,12 @@ int printmodel (MODEL *pmod, const DATAINFO *pdinfo, gretlopt opt,
 	goto close_format;
     }
 
+    if (pmod->aux == AUX_HET_1) {
+	rsqline(pmod, prn);
+	print_HET_1_results(pmod, prn);
+	goto close_format;
+    }
+
     if (pmod->aux == AUX_WHITE) { 
 	rsqline(pmod, prn);
 	print_whites_results(pmod, prn);
@@ -2258,12 +2354,10 @@ int printmodel (MODEL *pmod, const DATAINFO *pdinfo, gretlopt opt,
 	|| pmod->ci == NLS || pmod->ci == MPOLS
 	|| (pmod->ci == AR && pmod->arinfo->arlist[0] == 1)
 	|| pmod->ci == LOGISTIC || pmod->ci == TOBIT
-	|| pmod->ci == PANEL 
+	|| non_weighted_panel(pmod)
 	|| (pmod->ci == WLS && gretl_model_get_int(pmod, "wt_dummy"))) {
 	print_middle_table_start(prn);
-	if (pmod->ci != VAR) {
-	    depvarstats(pmod, prn);
-	}
+	depvarstats(pmod, prn);
 	if (essline(pmod, prn)) {
 	    print_middle_table_end(prn);
 	    goto close_format;
@@ -2305,6 +2399,10 @@ int printmodel (MODEL *pmod, const DATAINFO *pdinfo, gretlopt opt,
 	    info_stats_lines(pmod, prn);
 	}
 
+	if (pmod->ci == TSLS && plain_format(prn)) {
+	    addconst_message(pmod, prn);
+	}
+
 	print_middle_table_end(prn);
 
 	if (pmod->ci == TSLS && plain_format(prn)) {
@@ -2312,8 +2410,7 @@ int printmodel (MODEL *pmod, const DATAINFO *pdinfo, gretlopt opt,
 	}
     }
 
-    else if (pmod->ci == WLS && gretl_model_get_int(pmod, "iters")) {
-	/* panel ML estimation */
+    else if (panel_ML_model(pmod)) {
 	print_middle_table_start(prn);
 	depvarstats(pmod, prn);
 	print_ll(pmod, prn);
@@ -2328,8 +2425,7 @@ int printmodel (MODEL *pmod, const DATAINFO *pdinfo, gretlopt opt,
 	print_middle_table_end(prn);
     }	
 
-    else if (pmod->ci == HSK || pmod->ci == ARCH ||
-	     (pmod->ci == WLS && !gretl_model_get_int(pmod, "wt_dummy"))) {
+    else if (weighted_model(pmod)) {
 
 	weighted_stats_message(prn);
 	print_middle_table_start(prn);
@@ -2391,7 +2487,7 @@ int printmodel (MODEL *pmod, const DATAINFO *pdinfo, gretlopt opt,
 	outcovmx(pmod, pdinfo, prn);
     }
 
-    if (any_tests(pmod)) {
+    if (any_tests(pmod) && !(opt & OPT_S)) {
 	print_model_tests(pmod, prn);
     }
 
@@ -2438,8 +2534,11 @@ static void plain_print_pval (double pval, PRN *prn)
 
 static void plain_print_tval (double tval, PRN *prn)
 {
-    if (fabs(tval) >= 1000) { 
-	/* || t < .001 ? */
+    double abst = fabs(tval);
+
+    if (abst >= 1000 && abst < 100000) {
+	pprintf(prn, " %7.0f", tval);
+    } else if (abst >= 1000) { 
 	char numstr[9];
 
 	sprintf(numstr, "%#8.2G", tval);
@@ -2459,7 +2558,7 @@ prepare_model_coeff (const MODEL *pmod, const DATAINFO *pdinfo,
 
     model_coeff_init(mc);
 
-    mc->show_pval = !binary_model(pmod);
+    mc->show_pval = !binary_model(pmod) || gretl_model_get_int(pmod, "show-pvals");
 
     if (tex_format(prn)) {
 	make_tex_coeff_name(pmod, pdinfo, i, mc->name);
@@ -2495,7 +2594,9 @@ prepare_model_coeff (const MODEL *pmod, const DATAINFO *pdinfo,
 	}
     }
 
-    if (!gotnan && binary_model(pmod) && pmod->list[i+2] != 0) { 
+    if (!gotnan && !mc->show_pval && 
+	binary_model(pmod) && 
+	pmod->list[i+2] != 0) { 
 	double *slopes = gretl_model_get_data(pmod, "slopes");
 
 	if (slopes != NULL) {
@@ -2700,7 +2801,7 @@ void print_arch_coeffs (const double *a, const double *se,
 	mc.b = a[i];
 	mc.se = se[i];
 	mc.tval = a[i] / se[i];
-	mc.pval = t_pvalue_2(mc.tval, T - (order + 1));
+	mc.pval = student_pvalue_2(mc.tval, T - (order + 1));
 
 	if (tex_format(prn)) {
 	    sprintf(mc.name, "$\\alpha_%d$", i);
@@ -2712,47 +2813,54 @@ void print_arch_coeffs (const double *a, const double *se,
     }
 }
 
+static void print_coeff_separator (const char *s, PRN *prn)
+{
+    if (tex_format(prn)) {
+	/* FIXME */
+	pputs(prn, "\\\\ \n");
+    } else {
+	if (s != NULL) {
+	    pputc(prn, '\n');
+	    /* FIXME RTF */
+	    print_centered((rtf_format(prn))? I_(s) : _(s), 78, prn);
+	    pputc(prn, '\n');
+	}
+	pputc(prn, '\n');
+    }
+}
+
 static int 
 print_coefficients (const MODEL *pmod, const DATAINFO *pdinfo, PRN *prn)
 {
+    const char *sepstr = NULL;
+    int seppos = -1;
     model_coeff mc;
     int nc = pmod->ncoeff;
     int i, err = 0, gotnan = 0;
-    int gn = -1;
 
-    if (pmod->ci == GARCH) {
-	gn = pmod->list[0] - 4;
-    } else if (pmod->ci == PANEL) {
+    if (pmod->ci == PANEL) {
 	nc = pmod->list[0] - 1;
     }
 
+    gretl_model_get_coeff_separator(pmod, &sepstr, &seppos);
+    if (seppos == -1 && pmod->ci == GARCH) {
+	seppos = pmod->list[0] - 4;
+    }
+
     for (i=0; i<nc; i++) {
 
 	err = prepare_model_coeff(pmod, pdinfo, i, &mc, prn);
 	if (err) gotnan = 1;
 
-	if (plain_format(prn)) {
-	    if (i == gn) {
-		pputc(prn, '\n');
-	    }
-	    if (pmod->ci == MPOLS) {
-		print_mp_coeff(&mc, prn);
-		continue;
-	    } 
-	} else if (tex_format(prn)) {
-	    if (i == gn) {
-		pputs(prn, "\\\\ \n");
-	    }
-	} else if (rtf_format(prn)) {
-	    if (i == gn) {
-		pputc(prn, '\n');
-	    }
-	} else if (csv_format(prn)) {
-	    if (i == gn) {
-		pputc(prn, '\n');
-	    }
+	if (i == seppos) {
+	    print_coeff_separator(sepstr, prn);
 	}
 
+	if (plain_format(prn) && pmod->ci == MPOLS) {
+	    print_mp_coeff(&mc, prn);
+	    continue;
+	}	    
+
 	print_coeff(&mc, prn);
     }
 
@@ -2784,7 +2892,7 @@ static void print_rho (const ARINFO *arinfo, int c, int dfd, PRN *prn)
 	gretl_print_value (arinfo->rho[c], prn);
 	bufspace(2, prn);
 	gretl_print_value (arinfo->sderr[c], prn); 
-	pval = t_pvalue_2(xx, dfd);
+	pval = student_pvalue_2(xx, dfd);
 	pprintf(prn, " %7.3f ", xx, pval);
 	print_pval_str(pval, pvalstr);
 	pprintf(prn, "%*s\n", UTF_WIDTH(pvalstr, 12), pvalstr);
@@ -2805,7 +2913,7 @@ static void print_rho (const ARINFO *arinfo, int c, int dfd, PRN *prn)
 		coeff,
 		sderr,
 		arinfo->rho[c] / arinfo->sderr[c],
-		t_pvalue_2(xx, dfd));
+		student_pvalue_2(xx, dfd));
     } else if (rtf_format(prn)) {
 	char pvalstr[16];
 	double pval;
@@ -2815,7 +2923,7 @@ static void print_rho (const ARINFO *arinfo, int c, int dfd, PRN *prn)
 	rtf_print_double(arinfo->rho[c], prn);
 	rtf_print_double(arinfo->sderr[c], prn);
 	pprintf(prn, " \\qc %.4f\\cell", xx);
-	pval = t_pvalue_2(xx, dfd);
+	pval = student_pvalue_2(xx, dfd);
 	print_pval_str(pval, pvalstr);
 	pprintf(prn, " \\qc %s\\cell", pvalstr);
 	if (pval < 0.01) {
@@ -3056,6 +3164,36 @@ static void print_tobit_stats (const MODEL *pmod, PRN *prn)
     }
 }
 
+static void print_heckit_stats (const MODEL *pmod, PRN *prn)
+{
+    int totobs = gretl_model_get_int(pmod, "totobs");
+    int cenobs = totobs - pmod->nobs;
+    double cenpc = 100.0 * cenobs / totobs;
+
+    if (plain_format(prn)) {
+	pprintf(prn, "  %s: %d\n", _("Total observations"), totobs);
+	pprintf(prn, "  %s: %d (%.1f%%)\n", _("Censored observations"), cenobs, cenpc);
+	pprintf(prn, "  %s = %.*g\n", _("sigma"), GRETL_DIGITS, pmod->sigma);
+	pprintf(prn, "  %s = %.*g\n", _("rho"), GRETL_DIGITS, pmod->rho);
+    } else if (rtf_format(prn)) {
+	pprintf(prn, RTFTAB "%s: %d\n", I_("Total observations"), totobs);
+	pprintf(prn, RTFTAB "%s: %d (%.1f%%)\n", I_("Censored observations"), cenobs, cenpc);
+	pprintf(prn, RTFTAB "%s = %g\n", I_("sigma"), pmod->sigma);
+	pprintf(prn, RTFTAB "%s = %g\n", I_("rho"), pmod->rho);
+    } else if (tex_format(prn)) {
+	char xstr[32];
+	
+	pprintf(prn, "%s & \\multicolumn{1}{r}{%d} \\\\\n", 
+		I_("Total observations"), totobs);
+	pprintf(prn, "%s & \\multicolumn{1}{r}{%.1f\\%%} \\\\\n", 
+		I_("Censored observations"), cenpc);
+	tex_dcolumn_double(pmod->sigma, xstr);
+	pprintf(prn, "$\\hat{\\sigma}$ & %s \\\\\n", xstr);
+	tex_dcolumn_double(pmod->rho, xstr);
+	pprintf(prn, "$\\hat{\\rho}$ & %s \\\\\n", xstr);
+    }
+}
+
 static void 
 print_poisson_offset (const MODEL *pmod, const DATAINFO *pdinfo, PRN *prn)
 {
@@ -3138,6 +3276,7 @@ static void print_binary_statistics (const MODEL *pmod,
 				     const DATAINFO *pdinfo,
 				     PRN *prn)
 {
+    int slopes = !gretl_model_get_int(pmod, "show-pvals");
     double model_chisq = gretl_model_get_double(pmod, "chisq");
     const double *crit = pmod->criterion;
     const int *act_pred;
@@ -3181,11 +3320,11 @@ static void print_binary_statistics (const MODEL *pmod,
 	if (act_pred != NULL) {
 	    plain_print_act_pred(act_pred, prn);
 	}
-    }
-
-    else if (rtf_format(prn)) {
+    } else if (rtf_format(prn)) {
 	pputc(prn, '\n');
-	pprintf(prn, "\\par {\\super *}%s\n", I_("Evaluated at the mean"));
+	if (slopes) {
+	    pprintf(prn, "\\par {\\super *}%s\n", I_("Evaluated at the mean"));
+	}
 	pprintf(prn, "\\par %s %s = %.3f\n", I_("Mean of"), 
 		pdinfo->varname[pmod->list[1]], pmod->ybar);
 	if (correct >= 0) {
@@ -3211,13 +3350,13 @@ static void print_binary_statistics (const MODEL *pmod,
 	pprintf(prn, "\\par %s (%s) = %g\\par\n", I_(hqc_str), I_(hqc_abbrev),
 		crit[C_HQC]);
 	pputc(prn, '\n');
-    }
-
-    else if (tex_format(prn)) {
+    } else if (tex_format(prn)) {
 	char lnlstr[16];
 
-	pprintf(prn, "\\begin{center}\n$^*$%s\n\\end{center}\n", 
-		I_("Evaluated at the mean"));
+	if (slopes) {
+	    pprintf(prn, "\\begin{center}\n$^*$%s\n\\end{center}\n", 
+		    I_("Evaluated at the mean"));
+	}
 
 	pputs(prn, "\\vspace{1em}\n\\begin{raggedright}\n");
 	pprintf(prn, "%s %s = %.3f\\\\\n", I_("Mean of"), 
