@@ -1,20 +1,20 @@
-/*
- *  Copyright (c) by Ramu Ramanathan and Allin Cottrell
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
- *
- *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- *
+/* 
+ *  gretl -- Gnu Regression, Econometrics and Time-series Library
+ *  Copyright (C) 2001 Allin Cottrell and Riccardo "Jack" Lucchetti
+ * 
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ * 
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ * 
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 /*  monte_carlo.c - loop simulation procedures */
@@ -126,6 +126,7 @@ struct LOOPSET_ {
     char ichar;
     int ival;
     char brk;
+    char listname[VNAMELEN];
 
     controller init;
     controller test;
@@ -160,18 +161,16 @@ struct LOOPSET_ {
 #define loop_is_quiet(l) (l->flags & LOOP_QUIET)
 #define loop_set_quiet(l) (l->flags |= LOOP_QUIET)
 
-static void gretl_loop_init (LOOPSET *loop);
+#define is_list_loop(l) (l->listname[0] != '\0')
+
+#define model_print_deferred(o) (o & OPT_F)
+
+static void controller_init (controller *clr);
 static int gretl_loop_prepare (LOOPSET *loop);
 static void loop_model_free (LOOP_MODEL *lmod);
 static void loop_print_free (LOOP_PRINT *lprn);
 static void loop_store_free (LOOPSET *loop);
 static void controller_free (controller *clr);
-static void print_loop_model (LOOP_MODEL *lmod, int loopnum,
-			      const DATAINFO *pdinfo, PRN *prn);
-static void print_loop_coeff (const DATAINFO *pdinfo, const LOOP_MODEL *lmod, 
-			      int c, int n, PRN *prn);
-static void print_loop_prn (LOOP_PRINT *lprn, int n,
-			    const DATAINFO *pdinfo, PRN *prn);
 static int loop_store_save (LOOPSET *loop, PRN *prn);
 static void set_active_loop (LOOPSET *loop);
 
@@ -333,11 +332,28 @@ static void set_loop_opts (LOOPSET *loop, gretlopt opt)
     }
 }
 
-#define OK_LOOP_MODEL(c) (c == ARMA || c == CORC || c == GARCH || \
-                          c == HCCM || c == HILU || c == HSK || \
-                          c == LAD || c == OLS || c == TSLS || \
-                          c == PWE || c == WLS || c == MPOLS || \
-                          c == TOBIT)
+#define plain_model_ci(c) (c == AR ||		\
+			   c == ARBOND ||	\
+			   c == ARCH ||		\
+			   c == ARMA ||		\
+			   c == CORC ||		\
+			   c == GARCH ||	\
+			   c == HCCM ||		\
+			   c == HECKIT ||	\
+			   c == HILU ||		\
+			   c == HSK ||		\
+			   c == LAD ||		\
+			   c == LOGISTIC ||	\
+			   c == LOGIT ||	\
+			   c == MPOLS ||	\
+			   c == OLS ||		\
+			   c == PANEL ||	\
+			   c == POISSON ||	\
+			   c == PROBIT ||	\
+			   c == PWE ||		\
+			   c == TOBIT ||	\
+			   c == TSLS ||		\
+			   c == WLS)
 
 /**
  * ok_in_loop:
@@ -349,83 +365,35 @@ static void set_loop_opts (LOOPSET *loop, gretlopt opt)
 
 int ok_in_loop (int c)
 {
-    if (c == APPEND ||
-        c == GENR ||
-	c == LOOP ||
-	c == STORE ||
-	c == PRINT ||
-	c == PRINTF ||
-	c == PVALUE ||
-	c == SMPL ||
-	c == IF ||
-	c == ELSE ||
-	c == ENDIF ||
-	c == BREAK ||
-	c == ENDLOOP) { 
-	return 1;
+    if (c == BXPLOT ||
+	c == CORRGM ||
+	c == CUSUM ||
+	c == DATA ||
+	c == DATAMOD ||
+	c == DELEET ||
+	c == EQNPRINT ||
+	c == FIT ||
+	c == FUNC ||
+	c == GNUPLOT ||
+	c == HURST ||
+	c == INCLUDE ||
+	c == LEVERAGE ||
+	c == MODELTAB ||
+	c == NULLDATA ||
+	c == OPEN ||
+	c == QLRTEST ||
+	c == RMPLOT ||
+	c == RUN ||
+	c == SCATTERS ||
+	c == SETMISS ||
+	c == SETOBS ||
+	c == TABPRINT ||
+	c == VIF ||
+	c == XCORRGM) {
+	return 0;
     }
 
-    /* "simple_commands" */
-    if (c == ADF || 
-	c == COINT || 
-	c == COINT2 || 
-	c == CORR ||
-	c == CRITERIA || 
-	c == DIFF || 
-	c == HURST ||	
-	c == KPSS ||
-        c == LABELS ||
-	c == LAGS || 
-	c == LDIFF || 
-	c == LOGS ||
-	c == MEANTEST || 
-	c == MULTIPLY || 
-	c == OUTFILE ||
-	c == PCA ||
-	c == PERGM ||
-	c == REMEMBER ||
-        c == RENAME || 
-	c == RHODIFF ||
-	c == RUNS || 
-        c == SET ||
-	c == SETINFO ||
-	c == SHELL || 
-	c == SPEARMAN || 
-        c == STRING ||
-	c == SQUARE || 
-	c == SUMMARY ||
-	c == VARLIST ||
-	c == VARTEST ||
-	c == XTAB ) {
-	return 1;
-    }
-
-    /* frequencies --- no graph allowed */
-    if (c == FREQ) {
-	return 1;
-    }
-
-    /* modeling commands */
-    if (OK_LOOP_MODEL(c)) {
-	return 1;
-    }
-
-    /* vector models */
-    if (c == VAR || c == VECM) {
-	return 1;
-    }
-    
-    /* nonlinear models */
-    if (c == NLS || c == MLE || c == GMM || c == END) {
-	return 1;
-    }
-
-    /* basic model tests */
-    if (c == ADD || c == OMIT || c == TESTUHAT) {
-	return 1;
-    }
-
-    return 0;
+    return 1;
 }
 
 static int loop_attach_child (LOOPSET *loop, LOOPSET *child)
@@ -453,6 +421,52 @@ static int loop_attach_child (LOOPSET *loop, LOOPSET *child)
     return 0;
 }
 
+static void gretl_loop_init (LOOPSET *loop)
+{
+#if LOOP_DEBUG
+    fprintf(stderr, "gretl_loop_init: initing loop at %p\n", (void *) loop);
+#endif
+
+    loop->flags = 0;
+    loop->level = 0;
+
+    loop->itermax = 0;
+    loop->iter = 0;
+    loop->err = 0;
+    loop->ichar = 0;
+    loop->ival = 0;
+    loop->brk = 0;
+    *loop->listname = '\0';
+
+    controller_init(&loop->init);
+    controller_init(&loop->test);
+    controller_init(&loop->delta);
+    controller_init(&loop->final);
+
+    loop->n_cmds = 0;
+    loop->n_models = 0;
+    loop->n_loop_models = 0;
+    loop->n_prints = 0;
+
+    loop->lines = NULL;
+    loop->ci = NULL;
+
+    loop->eachstrs = NULL;
+
+    loop->models = NULL;
+    loop->lmodels = NULL;
+    loop->prns = NULL;
+
+    loop->storefile[0] = '\0';
+    loop->storeopt = OPT_NONE;
+    loop->sZ = NULL;
+    loop->sdinfo = NULL;
+
+    loop->parent = NULL;
+    loop->children = NULL;
+    loop->n_children = 0;
+}
+
 static LOOPSET *gretl_loop_new (LOOPSET *parent)
 {
     LOOPSET *loop = malloc(sizeof *loop);
@@ -500,11 +514,8 @@ static void gretl_loop_destroy (LOOPSET *loop)
     }
 
     if (loop->eachstrs != NULL) {
-	for (i=0; i<loop->itermax; i++) {
-	    free(loop->eachstrs[i]);
-	}
-	free(loop->eachstrs);
-    }    
+	free_strings_array(loop->eachstrs, loop->itermax);
+    }
 
     if (loop->models != NULL) {
 	free(loop->models);
@@ -908,53 +919,84 @@ static int allocate_each_strings (LOOPSET *loop, int n)
     return (loop->eachstrs == NULL)? E_ALLOC : 0;
 }
 
-static int each_strings_from_named_list (LOOPSET *loop, const DATAINFO *pdinfo,
-					 char *s, int *nf)
+static int list_vars_to_strings (LOOPSET *loop, const int *list,
+				 const DATAINFO *pdinfo)
 {
-    int *list;
-    int err = 0;
+    int i, vi, n = list[0];
+    int err;
 
-    while (isspace(*s)) s++;
+    err = allocate_each_strings(loop, n);
 
-    tailstrip(s);
-    list = get_list_by_name(s);
-
-    if (list == NULL) {
-	err = E_UNKVAR;
-    } else {
-	err = allocate_each_strings(loop, list[0]);
-    }
-
-    /* when cashing out list-members, use varnames rather than numbers
-    */
-    if (!err) {
-	int i, li;
-
-	for (i=0; i<list[0] && !err; i++) {
-	    li = list[i+1];
-	    if (li < 0 || li >= pdinfo->v) {
-		err = 1;
-	    } else {
-		loop->eachstrs[i] = gretl_strdup(pdinfo->varname[li]);
-	    }
-	    if (!err && loop->eachstrs[i] == NULL) {
+    for (i=0; i<n && !err; i++) {
+	vi = list[i+1];
+	if (vi < 0 || vi >= pdinfo->v) {
+	    err = E_DATA;
+	} else {
+	    loop->eachstrs[i] = gretl_strdup(pdinfo->varname[vi]);
+	    if (loop->eachstrs[i] == NULL) {
 		err = E_ALLOC;
 	    }
 	}
     }
 
-    if (err && loop->eachstrs != NULL) {
-	free_strings_array(loop->eachstrs, list[0]);
+    return err;
+}
+
+/* At loop runtime, check the named list and insert the names
+   of the variables as "eachstrs"; flag an error if the list
+   has disappeared.
+*/
+
+static int loop_list_refresh (LOOPSET *loop, const DATAINFO *pdinfo)
+{
+    int *list = get_list_by_name(loop->listname);
+    int err = 0;
+
+    if (loop->eachstrs != NULL) {
+	free_strings_array(loop->eachstrs, loop->itermax);
 	loop->eachstrs = NULL;
     }
 
-    if (!err) {
-	*nf = list[0];
+    loop->itermax = loop->final.val = 0;
+
+    if (list == NULL) {
+	err = E_UNKVAR;
+    } else if (list[0] > 0) {
+	err = list_vars_to_strings(loop, list, pdinfo);
+	if (!err) {
+	    loop->final.val = list[0];
+	}
     }
 
     return err;
 }
 
+/* Identify the named list, if any, but do not yet fill out the
+   variable-name strings: these will be set when the loop is
+   actually run, since the list may hae changed in the meantime.
+*/
+
+static int list_loop_setup (LOOPSET *loop, char *s, int *nf)
+{
+    int *list;
+    int err = 0;
+
+    while (isspace(*s)) s++;
+    tailstrip(s);
+
+    list = get_list_by_name(s);
+
+    if (list == NULL) {
+	err = E_UNKVAR;
+    } else {
+	*loop->listname = '\0';
+	strncat(loop->listname, s, VNAMELEN - 1);
+	*nf = list[0];
+    } 
+
+    return err;
+}
+
 static int
 each_strings_from_list_of_vars (LOOPSET *loop, const DATAINFO *pdinfo, 
 				char *s, int *nf)
@@ -1008,6 +1050,7 @@ parse_as_each_loop (LOOPSET *loop, const DATAINFO *pdinfo, char *s)
 {
     char ivar[8] = {0};
     char ichar = 0;
+    int done = 0;
     int i, nf, err = 0;
 
     if (*s == '\0') {
@@ -1048,10 +1091,16 @@ parse_as_each_loop (LOOPSET *loop, const DATAINFO *pdinfo, char *s)
     if (nf <= 3 && strstr(s, "..") != NULL) {
 	/* range of values, foo..quux */
 	err = each_strings_from_list_of_vars(loop, pdinfo, s, &nf);
-    } else if (nf == 1) {
-	/* named list? */
-	err = each_strings_from_named_list(loop, pdinfo, s, &nf);
-    } else {
+	done = 1;
+    }
+
+    if (!done && nf == 1) {
+	/* try for a named list? */
+	err = list_loop_setup(loop, s, &nf);
+	done = (err == 0);
+    }
+
+    if (!done) {
 	/* simple list of values */
 	err = allocate_each_strings(loop, nf);
 
@@ -1171,7 +1220,7 @@ static int parse_first_loopline (char *s, LOOPSET *loop,
     if (!err && !na(loop->init.val) && !na(loop->final.val)) {
 	int nt = loop->final.val - loop->init.val + 1;
 
-	if (loop->type != FOR_LOOP && nt <= 0) {
+	if (loop->type != FOR_LOOP && loop->type != EACH_LOOP && nt <= 0) {
 	    strcpy(gretl_errmsg, _("Loop count missing or invalid"));
 	    err = 1;
 	}
@@ -1246,38 +1295,22 @@ start_new_loop (char *s, LOOPSET *inloop,
     return loop;
 }
 
-#define DEFAULT_MAX_ITER 250
 #define MAX_FOR_TIMES  100000
 
-static int get_max_iters (void)
-{
-    static int ml = 0;
-
-    if (ml == 0) {
-	char *mlstr = getenv("GRETL_MAX_ITER");
-
-	if (mlstr != NULL && sscanf(mlstr, "%d", &ml)) ;
-	else ml = DEFAULT_MAX_ITER;
-    }
-
-    return ml;
-}
-
 static int loop_count_too_high (LOOPSET *loop)
 {
     static int max_iters = 0;
     int nt = loop->iter + 1;
 
     if (loop->type == FOR_LOOP) {
-	/* FIXME? */
 	if (nt >= MAX_FOR_TIMES) {
 	    sprintf(gretl_errmsg, _("Reached maximum interations, %d"),
 		    MAX_FOR_TIMES);
 	    loop->err = 1;
 	}
-    } else {  
+    } else {
 	if (max_iters == 0) {
-	    max_iters = get_max_iters();
+	    max_iters = libset_get_int(LOOP_MAXITER);
 	}
 
 	if (nt >= max_iters) {
@@ -1334,14 +1367,6 @@ loop_condition (LOOPSET *loop, double ***pZ, DATAINFO *pdinfo)
 	}
     }
 
-    if (!ok) {
-	if (loop->iter == 0) {
-	    strcpy(gretl_errmsg, _("Loop condition not satisfied at first round"));
-	    loop->err = 1;
-	    loop->brk = 1;
-	}
-    }
-
     return ok;
 }
 
@@ -1362,51 +1387,6 @@ static void controller_free (controller *clr)
     }
 }
 
-static void gretl_loop_init (LOOPSET *loop)
-{
-#if LOOP_DEBUG
-    fprintf(stderr, "gretl_loop_init: initing loop at %p\n", (void *) loop);
-#endif
-
-    loop->flags = 0;
-    loop->level = 0;
-
-    loop->itermax = 0;
-    loop->iter = 0;
-    loop->err = 0;
-    loop->ichar = 0;
-    loop->ival = 0;
-    loop->brk = 0;
-
-    controller_init(&loop->init);
-    controller_init(&loop->test);
-    controller_init(&loop->delta);
-    controller_init(&loop->final);
-
-    loop->n_cmds = 0;
-    loop->n_models = 0;
-    loop->n_loop_models = 0;
-    loop->n_prints = 0;
-
-    loop->lines = NULL;
-    loop->ci = NULL;
-
-    loop->eachstrs = NULL;
-
-    loop->models = NULL;
-    loop->lmodels = NULL;
-    loop->prns = NULL;
-
-    loop->storefile[0] = '\0';
-    loop->storeopt = OPT_NONE;
-    loop->sZ = NULL;
-    loop->sdinfo = NULL;
-
-    loop->parent = NULL;
-    loop->children = NULL;
-    loop->n_children = 0;
-}
-
 static int gretl_loop_prepare (LOOPSET *loop)
 {
 #ifdef ENABLE_GMP
@@ -1840,65 +1820,6 @@ static void loop_print_update (LOOPSET *loop, int i,
 #endif
 }
 
-/**
- * print_loop_results:
- * @loop: pointer to loop struct.
- * @pdinfo: data information struct.
- * @prn: gretl printing struct.
- *
- * Print out the results after completion of the loop @loop.
- */
-
-static void print_loop_results (LOOPSET *loop, const DATAINFO *pdinfo, 
-				PRN *prn)
-{
-    char linecpy[MAXLINE];
-    int iters = loop->iter;
-    int i, j, k;
-
-    if (loop->type != COUNT_LOOP && !(loop_is_quiet(loop))) {
-	pprintf(prn, _("\nNumber of iterations: %d\n\n"), iters);
-    }
-
-    j = 0;
-    k = 0;
-
-    for (i=0; i<loop->n_cmds; i++) {
-#if LOOP_DEBUG
-	fprintf(stderr, "print_loop_results: loop command %d (i=%d): %s\n", 
-		i+1, i, loop->lines[i]);
-#endif
-	if (!loop_is_progressive(loop) && loop->ci[i] == OLS) {
-	    gretlopt opt;
-
-	    strcpy(linecpy, loop->lines[i]);
-	    opt = get_gretl_options(linecpy, NULL);
-	    
-	    if (opt & OPT_P) {
-		/* deferred printing of model was requested */
-		MODEL *pmod = loop->models[j++];
-
-		set_model_id(pmod);
-		printmodel(pmod, pdinfo, opt, prn);
-	    }	    
-	}
-
-	if (loop_is_progressive(loop)) {
-	    if (OK_LOOP_MODEL(loop->ci[i])) {
-		print_loop_model(&loop->lmodels[j], iters, pdinfo, prn);
-		loop_model_zero(&loop->lmodels[j]);
-		j++;
-	    } else if (loop->ci[i] == PRINT) {
-		print_loop_prn(&loop->prns[k], iters, pdinfo, prn);
-		loop_print_zero(&loop->prns[k]);
-		k++;
-	    } else if (loop->ci[i] == STORE) {
-		loop_store_save(loop, prn);
-	    }
-	}
-    }
-}
-
 static void loop_model_free (LOOP_MODEL *lmod)
 {
 #if LOOP_DEBUG
@@ -2103,42 +2024,11 @@ int gretl_loop_append_line (ExecState *s, double ***pZ,
     return err;
 }
 
-static void print_loop_model (LOOP_MODEL *lmod, int loopnum,
-			      const DATAINFO *pdinfo, PRN *prn)
-{
-    char startdate[OBSLEN], enddate[OBSLEN];
-    int i;
-
-    ntodate(startdate, lmod->model0->t1, pdinfo);
-    ntodate(enddate, lmod->model0->t2, pdinfo);
-
-    pputc(prn, '\n');
-    pprintf(prn, _("%s estimates using the %d observations %s-%s\n"),
-	    _(estimator_string(lmod->model0, prn)), lmod->model0->nobs, 
-	    startdate, enddate);
-    print_model_vcv_info(lmod->model0, prn);
-    pprintf(prn, _("Statistics for %d repetitions\n"), loopnum); 
-    pprintf(prn, _("Dependent variable: %s\n\n"), 
-	    pdinfo->varname[lmod->model0->list[1]]);
-
-    pputs(prn, _("                     mean of      std. dev. of     mean of"
-		 "     std. dev. of\n"
-		 "                    estimated      estimated"
-		 "      estimated      estimated\n"
-		 "      Variable     coefficients   coefficients   std. errors"
-		 "    std. errors\n\n"));
-
-    for (i=0; i<lmod->model0->ncoeff; i++) {
-	print_loop_coeff(pdinfo, lmod, i, loopnum, prn);
-    }
-
-    pputc(prn, '\n');
-}
-
 static void print_loop_coeff (const DATAINFO *pdinfo, 
 			      const LOOP_MODEL *lmod, 
 			      int c, int n, PRN *prn)
 {
+    char pname[VNAMELEN];
 #ifdef ENABLE_GMP
     mpf_t c1, c2, m, sd1, sd2;
     unsigned long ln = n;
@@ -2179,8 +2069,8 @@ static void print_loop_coeff (const DATAINFO *pdinfo,
 	}
     }
 
-    pprintf(prn, "%*s", VNAMELEN - 1, 
-	    pdinfo->varname[lmod->model0->list[c+2]]);
+    gretl_model_get_param_name(lmod->model0, pdinfo, c, pname);
+    pprintf(prn, "%*s", VNAMELEN - 1, pname);
     pprintf(prn, "%#14g %#14g %#14g %#14g\n", mpf_get_d(c1), mpf_get_d(sd1), 
 	    mpf_get_d(c2), mpf_get_d(sd2));
 
@@ -2208,13 +2098,45 @@ static void print_loop_coeff (const DATAINFO *pdinfo,
 	sd2 = (sd2 <= 0.0)? 0 : sqrt((double) sd2);
     }
 
-    pprintf(prn, "%*s", VNAMELEN - 1, 
-	   pdinfo->varname[lmod->model0->list[c+2]]);
+    gretl_model_get_param_name(lmod->model0, pdinfo, c, pname);
+    pprintf(prn, "%*s", VNAMELEN - 1, pname);
     pprintf(prn, "%#14g %#14g %#14g %#14g\n", (double) m1, (double) sd1, 
 	    (double) m2, (double) sd2);
 #endif
 }
 
+static void print_loop_model (LOOP_MODEL *lmod, int loopnum,
+			      const DATAINFO *pdinfo, PRN *prn)
+{
+    char startdate[OBSLEN], enddate[OBSLEN];
+    int i;
+
+    ntodate(startdate, lmod->model0->t1, pdinfo);
+    ntodate(enddate, lmod->model0->t2, pdinfo);
+
+    pputc(prn, '\n');
+    pprintf(prn, _("%s estimates using the %d observations %s-%s\n"),
+	    _(estimator_string(lmod->model0, prn)), lmod->model0->nobs, 
+	    startdate, enddate);
+    print_model_vcv_info(lmod->model0, prn);
+    pprintf(prn, _("Statistics for %d repetitions\n"), loopnum); 
+    pprintf(prn, _("Dependent variable: %s\n\n"), 
+	    gretl_model_get_depvar_name(lmod->model0, pdinfo));
+
+    pputs(prn, _("                     mean of      std. dev. of     mean of"
+		 "     std. dev. of\n"
+		 "                    estimated      estimated"
+		 "      estimated      estimated\n"
+		 "      Variable     coefficients   coefficients   std. errors"
+		 "    std. errors\n\n"));
+
+    for (i=0; i<lmod->model0->ncoeff; i++) {
+	print_loop_coeff(pdinfo, lmod, i, loopnum, prn);
+    }
+
+    pputc(prn, '\n');
+}
+
 static void print_loop_prn (LOOP_PRINT *lprn, int n,
 			    const DATAINFO *pdinfo, PRN *prn)
 {
@@ -2273,6 +2195,64 @@ static void print_loop_prn (LOOP_PRINT *lprn, int n,
     pputc(prn, '\n');
 }
 
+/**
+ * print_loop_results:
+ * @loop: pointer to loop struct.
+ * @pdinfo: data information struct.
+ * @prn: gretl printing struct.
+ *
+ * Print out the results after completion of the loop @loop.
+ */
+
+static void print_loop_results (LOOPSET *loop, const DATAINFO *pdinfo, 
+				PRN *prn)
+{
+    char linecpy[MAXLINE];
+    int iters = loop->iter;
+    int i, j, k;
+
+    if (loop->type != COUNT_LOOP && !(loop_is_quiet(loop))) {
+	pprintf(prn, _("\nNumber of iterations: %d\n\n"), iters);
+    }
+
+    j = 0;
+    k = 0;
+
+    for (i=0; i<loop->n_cmds; i++) {
+#if LOOP_DEBUG
+	fprintf(stderr, "print_loop_results: loop command %d (i=%d): %s\n", 
+		i+1, i, loop->lines[i]);
+#endif
+	if (!loop_is_progressive(loop) && loop->ci[i] == OLS) {
+	    gretlopt opt;
+
+	    strcpy(linecpy, loop->lines[i]);
+	    opt = get_gretl_options(linecpy, NULL);
+	    
+	    if (model_print_deferred(opt)) {
+		MODEL *pmod = loop->models[j++];
+
+		set_model_id(pmod);
+		printmodel(pmod, pdinfo, opt, prn);
+	    }	    
+	}
+
+	if (loop_is_progressive(loop)) {
+	    if (plain_model_ci(loop->ci[i])) {
+		print_loop_model(&loop->lmodels[j], iters, pdinfo, prn);
+		loop_model_zero(&loop->lmodels[j]);
+		j++;
+	    } else if (loop->ci[i] == PRINT) {
+		print_loop_prn(&loop->prns[k], iters, pdinfo, prn);
+		loop_print_zero(&loop->prns[k]);
+		k++;
+	    } else if (loop->ci[i] == STORE) {
+		loop_store_save(loop, prn);
+	    }
+	}
+    }
+}
+
 static int loop_store_save (LOOPSET *loop, PRN *prn)
 {
     char fname[MAXLEN];
@@ -2374,7 +2354,7 @@ substitute_dollar_targ (char *str, const LOOPSET *loop,
 	    ntodate(ins, idx, pdinfo);
 	} else if (loop->type == EACH_LOOP) {
 	    pins = loop->eachstrs[idx - 1];
-	}
+	} 
 
 	strcpy(p, pins);
 	strcpy(p + strlen(pins), q);
@@ -2479,6 +2459,10 @@ connect_loop_control_vars (LOOPSET *loop, const DATAINFO *pdinfo)
 	err = clr_attach_var(&loop->final, pdinfo);
     }
 
+    if (!err && is_list_loop(loop)) {
+	loop_list_refresh(loop, pdinfo);
+    }
+
     return err;
 }
 
@@ -2617,11 +2601,23 @@ static int next_command (char *targ, LOOPSET *loop, int *j)
     return ret;
 }
 
+static int block_model (CMD *cmd)
+{
+    return cmd->ci == END && 
+	(!strcmp(cmd->param, "mle") || 
+	 !strcmp(cmd->param, "nls") ||
+	 !strcmp(cmd->param, "gmm"));
+}
+
+#define not_ok_in_progloop(c) (NEEDS_MODEL_CHECK(c) || \
+			       c == NLS ||  \
+			       c == MLE ||  \
+			       c == GMM)
+
 int gretl_loop_exec (ExecState *s, double ***pZ, DATAINFO **ppdinfo) 
 {
     LOOPSET *loop = currloop;
     DATAINFO *pdinfo = *ppdinfo;
-    MODEL **models = s->models;
     char *line = s->line;
     CMD *cmd = s->cmd;
     PRN *prn = s->prn;
@@ -2638,12 +2634,7 @@ int gretl_loop_exec (ExecState *s, double ***pZ, DATAINFO **ppdinfo)
 	return 1;
     }
 
-    if (loop->n_cmds == 0) {
-	pputs(prn, _("No commands in loop\n"));
-	return 0;
-    }
-
-    set_loop_on(); /* libset.c */
+    set_loop_on(loop_is_quiet(loop)); /* libset.c */
 
 #if LOOP_DEBUG
     fprintf(stderr, "loop_exec: loop = %p\n", (void *) loop);
@@ -2703,9 +2694,11 @@ int gretl_loop_exec (ExecState *s, double ***pZ, DATAINFO **ppdinfo)
 		}
 	    }
 
-	    switch (cmd->ci) {
+	    /* now branch based on the command index: some commands
+	       require special treatment
+	    */
 
-	    case LOOP:
+	    if (cmd->ci == LOOP) {
 		if (childnum < loop->n_children) {
 		    currloop = loop->children[childnum++];
 		    err = gretl_loop_exec(s, pZ, ppdinfo);
@@ -2714,171 +2707,91 @@ int gretl_loop_exec (ExecState *s, double ***pZ, DATAINFO **ppdinfo)
 		    fprintf(stderr, "Got a LOOP command, don't know what to do!\n");
 		    err = 1;
 		}
-		break;
-
-	    case BREAK:
+	    } else if (cmd->ci == BREAK) {
 		loop->brk = 1;
-		break;
-
-	    case ENDLOOP:
-		break;
-
-	    case FREQ:
-		/* note: no graphs in loops */
+	    } else if (cmd->ci == ENDLOOP) {
+		; /* implicit break */
+	    } else if (cmd->ci == FREQ) {
 		err = freqdist(cmd->list[1], (const double **) *pZ, pdinfo, 0, 
 			       cmd->opt, prn);
-		break;
-
-	    case ARMA:
-	    case CORC:
-	    case GARCH:
-	    case HCCM:
-	    case HILU:
-	    case HSK:
-	    case LAD:
-	    case MPOLS:
-	    case OLS:
-	    case TOBIT:
-	    case TSLS:
-	    case PWE:
-	    case WLS:
+	    } else if (plain_model_ci(cmd->ci)) {
 		/* if this is the first time round, allocate space
 		   for each loop model */
 		if (loop->iter == 0) {
 		    if (loop_is_progressive(loop)) {
 			err = add_loop_model(loop);
-		    } else if (cmd->opt & OPT_P) {
+		    } else if (model_print_deferred(cmd->opt)) {
 			err = add_model_record(loop);
 		    }
-		    if (err) {
-			break;
-		    }
 		} 
-
-		/* estimate the model called for, using models[0] */
-		clear_model(models[0]);
-
-		if (cmd->ci == OLS || cmd->ci == WLS || cmd->ci == HCCM) {
-		    *models[0] = lsq(cmd->list, pZ, pdinfo, cmd->ci, cmd->opt);
-		} else if (cmd->ci == LAD) {
-		    *models[0] = lad(cmd->list, pZ, pdinfo);
-		} else if (cmd->ci == MPOLS) {
-		    *models[0] = mp_ols(cmd->list, (const double **) *pZ, pdinfo);
-		} else if (cmd->ci == HSK) {
-		    *models[0] = hsk_func(cmd->list, pZ, pdinfo);
-		} else if (cmd->ci == ARMA) {
-		    *models[0] = arma(cmd->list, (const double **) *pZ, pdinfo, 
-				      cmd->opt, prn);
-		} else if (cmd->ci == TOBIT) {
-		    *models[0] = tobit_model(cmd->list, pZ, pdinfo,
-					     (cmd->opt & OPT_V)? prn : NULL);
-		} else if (cmd->ci == TSLS) {
-		    *models[0] = tsls_func(cmd->list, TSLS, pZ, pdinfo, cmd->opt);
-		} else if (cmd->ci == GARCH) {
-		    *models[0] = garch(cmd->list, pZ, pdinfo, cmd->opt, prn);
-		} else if (cmd->ci == CORC || cmd->ci == HILU || cmd->ci == PWE) {
-		    double rho = estimate_rho(cmd->list, pZ, pdinfo, cmd->ci,
-					      &err, cmd->opt, prn);
-		    if (err) {
-			break;
-		    }
-		    *models[0] = ar1_lsq(cmd->list, pZ, pdinfo, cmd->ci, cmd->opt, rho);
+		/* estimate the model called for */
+		if (!err) {
+		    err = gretl_cmd_exec(s, pZ, ppdinfo);
+		    pdinfo = *ppdinfo;
 		}
+		if (!err) {
+		    if (loop_is_progressive(loop)) {
+			int m = lmodnum++;
 
-		if ((err = models[0]->errcode)) {
-		    break;
-		} 
-
-		if (loop_is_progressive(loop)) {
-		    int m = lmodnum++;
-
-		    if (loop->iter == 0) {
-			err = loop_model_init(&loop->lmodels[m], models[0]);
-			if (err) {
-			    gretl_errmsg_set(_("Failed to initialize model for loop\n"));
-			    break;
+			if (loop->iter == 0) {
+			    err = loop_model_init(&loop->lmodels[m], s->models[0]);
+			    if (err) {
+				gretl_errmsg_set(_("Failed to initialize model for loop\n"));
+			    }
 			}
-		    }
-		    loop_model_update(loop, m, models[0]);
-		    set_as_last_model(models[0], GRETL_OBJ_EQN);
-		} else if (cmd->opt & OPT_P) {
-		    /* deferred printing of model results */
-		    int m = modnum++;
-
-		    swap_models(models[0], loop->models[m]);
-		    loop->models[m]->ID = j;
-		    set_as_last_model(loop->models[m], GRETL_OBJ_EQN);
-		    model_count_minus();
-		} else {
-		    models[0]->ID = ++mod_id;
-		    printmodel(models[0], pdinfo, cmd->opt, prn);
-		    set_as_last_model(models[0], GRETL_OBJ_EQN);
-		}
-		break;
-
-	    case ADD:
-	    case OMIT:
-	    case GMM:
-	    case MLE:
-	    case NLS:
-		if (loop_is_progressive(loop)) {
-		    err = 1;
-		} else {
-		    goto cmd_exec;
-		}
-		break;
-
-	    case PRINT:
-		if (cmd->param[0] != '\0') {
-		    goto cmd_exec;
-		} else if (loop_is_progressive(loop)) {
-		    int p = printnum++;
-
-		    if (loop->iter == 0) {
-			err = loop_print_add(loop, cmd->list);
-			if (err) {
-			    break;
+			if (!err) {
+			    loop_model_update(loop, m, s->models[0]);
+			    set_as_last_model(s->models[0], GRETL_OBJ_EQN);
 			}
+		    } else if (model_print_deferred(cmd->opt)) {
+			int m = modnum++;
+			
+			swap_models(s->models[0], loop->models[m]);
+			loop->models[m]->ID = j;
+			set_as_last_model(loop->models[m], GRETL_OBJ_EQN);
+			model_count_minus();
+		    } else {
+			s->models[0]->ID = ++mod_id; /* ?? */
+			printmodel(s->models[0], pdinfo, cmd->opt, prn);
+			set_as_last_model(s->models[0], GRETL_OBJ_EQN);
 		    }
-		    loop_print_update(loop, p, cmd->list, (const double **) *pZ, 
-				      pdinfo);
-		} else {
-		    err = printdata(cmd->list, cmd->extra,
-				    (const double **) *pZ, pdinfo, 
-				    cmd->opt, prn);
 		}
-		break;
-
-	    case STORE:
-		if (loop_is_progressive(loop)) {
-		    if (loop->iter == 0) {
-			err = loop_store_init(loop, cmd->param, cmd->list, 
-					      pdinfo, cmd->opt);
-			if (err) {
-			    break;
-			}
-		    }
+	    } else if (cmd->ci == PRINT && *cmd->param == '\0' &&
+		       loop_is_progressive(loop)) {
+		if (loop->iter == 0) {
+		    err = loop_print_add(loop, cmd->list);
+		}
+		if (!err) {
+		    loop_print_update(loop, printnum++, cmd->list, 
+				      (const double **) *pZ, pdinfo);
+		}
+	    } else if (loop_is_progressive(loop) && cmd->ci == STORE) {
+		if (loop->iter == 0) {
+		    err = loop_store_init(loop, cmd->param, cmd->list, 
+					  pdinfo, cmd->opt);
+		}
+		if (!err) {
 		    err = loop_store_update(cmd->list, loop, (const double **) *pZ, 
 					    pdinfo);
-		} else {
-		    goto cmd_exec;
 		}
-		break;
-
-	    default: 
-	    cmd_exec:
+	    } else if (loop_is_progressive(loop) && not_ok_in_progloop(cmd->ci)) {
+		err = 1;
+	    } else {
 		if (cmd->ci == GENR && !loop_is_verbose(loop)) {
 		    cmd->opt |= OPT_Q;
 		}
 		err = gretl_cmd_exec(s, pZ, ppdinfo);
 		pdinfo = *ppdinfo;
-		break;
-
-	    } /* end switch on specific command number */
-
+		if (!err && block_model(cmd)) {
+		    /* NLS, etc. */
+		    s->models[0]->ID = ++mod_id;
+		    printmodel(s->models[0], pdinfo, cmd->opt, prn);
+		    set_as_last_model(s->models[0], GRETL_OBJ_EQN);
+		}
+	    }
 	} /* end execution of commands within loop */
 
-	if (err && get_halt_on_error() == 0) {
+	if (err && !libset_get_bool(HALT_ON_ERR)) {
 	    errmsg(err, prn);
 	    err = 0;
 	}
@@ -2892,8 +2805,10 @@ int gretl_loop_exec (ExecState *s, double ***pZ, DATAINFO **ppdinfo)
     } /* end iterations of loop */
 
     if (err) {
-	errmsg(err, prn);
-	pprintf(prn, ">> %s\n", errline);
+	if (!s->funcerr) {
+	    errmsg(err, prn);
+	    pprintf(prn, ">> %s\n", errline);
+	}
     } else if (loop->err) {
 	errmsg(loop->err, prn);
 	err = loop->err;
@@ -2909,9 +2824,9 @@ int gretl_loop_exec (ExecState *s, double ***pZ, DATAINFO **ppdinfo)
 	void *ptr = get_last_model(&type);
 	int i;
 
-	if (type == GRETL_OBJ_EQN && models[0] != ptr) {
-	    swap_models(models[0], loop->models[loop->n_models - 1]);
-	    set_as_last_model(models[0], GRETL_OBJ_EQN);
+	if (type == GRETL_OBJ_EQN && s->models[0] != ptr) {
+	    swap_models(s->models[0], loop->models[loop->n_models - 1]);
+	    set_as_last_model(s->models[0], GRETL_OBJ_EQN);
 	}
 	for (i=0; i<loop->n_models; i++) {
 	    gretl_model_free(loop->models[i]);
@@ -2931,7 +2846,7 @@ int gretl_loop_exec (ExecState *s, double ***pZ, DATAINFO **ppdinfo)
 	currloop = NULL;
     }
 
-    if (get_halt_on_error()) {
+    if (libset_get_bool(HALT_ON_ERR)) {
 	return err;
     } else {
 	return 0;
@@ -2995,137 +2910,3 @@ int loop_scalar_read (int c)
 
     return (loop == NULL)? -1 : loop->ival;
 }
-
-#define IFDEBUG 0
-
-/* if-then stuff - conditional execution */
-
-int if_eval (const char *line, double ***pZ, DATAINFO *pdinfo)
-{
-    double val = NADBL;
-    int err, ret = -1;
-
-#if IFDEBUG
-    fprintf(stderr, "if_eval: line = '%s'\n", line);
-#endif
-
-    if (!strncmp(line, "if", 2)) {
-	line += 2;
-    } else if (!strncmp(line, "elif", 4)) {
-	/* dead code, for now */
-	line += 4;
-    }
-
-    while (*line == ' ') line++;
-
-    val = generate_scalar(line, pZ, pdinfo, &err);
-
-#if IFDEBUG
-    if (err) {
-	fprintf(stderr, "if_eval: generate returned %d\n", err);
-    }
-#endif
-
-    if (err) {
-	strcpy(gretl_errmsg, _("error evaluating 'if'"));
-    } else if (na(val)) {
-	strcpy(gretl_errmsg, _("indeterminate condition for 'if'"));
-    } else {
-	ret = (int) val;
-    }
-
-#if IFDEBUG
-    fprintf(stderr, "if_eval: returning %d\n", ret);
-#endif
-
-    return ret;
-}
-
-#if IFDEBUG
-static const char *ifstr (int c)
-{
-    if (c == SET_FALSE) return "SET_FALSE";
-    if (c == SET_TRUE)  return "SET_TRUE";
-    if (c == SET_ELSE)  return "SET_ELSE";
-    if (c == SET_ELIF)  return "SET_ELIF";
-    if (c == SET_ENDIF) return "SET_ENDIF";
-    if (c == IS_FALSE)  return "IS_FALSE";
-    if (c == DOINDENT)  return "DOINDENT";
-    if (c == UNINDENT)  return "UNINDENT";
-    if (c == GETINDENT) return "GETINDENT";
-    if (c == RELAX)     return "RELAX";
-    return "UNKNOWN";
-}
-#endif
-
-#define IF_DEPTH 32
-
-int ifstate (int code)
-{
-    static unsigned char T[IF_DEPTH];
-    static unsigned char got_if[IF_DEPTH];
-    static unsigned char got_else[IF_DEPTH];
-    static unsigned char indent;
-    int i, ret = 0;
-
-#if IFDEBUG
-    fprintf(stderr, "ifstate: code = %s\n", ifstr(code));
-#endif
-
-    if (code == RELAX) {
-	indent = 0;
-    } else if (code == DOINDENT) {
-	return ++indent;
-    } else if (code == UNINDENT) {
-	return --indent;
-    } else if (code == GETINDENT) {
-	return indent;
-    } else if (code == SET_FALSE || code == SET_TRUE) {
-	indent++;
-	if (indent >= IF_DEPTH) {
-	    fprintf(stderr, "IF depth (%d) exceeded\n", IF_DEPTH);
-	    ret = 1;
-	} else {
-	    T[indent] = (code == SET_TRUE);
-	    got_if[indent] = 1;
-	    got_else[indent] = 0;
-	}
-    } else if (code == SET_ELSE) {
-	if (got_else[indent] || !got_if[indent]) {
-	    strcpy(gretl_errmsg, "Unmatched \"else\"");
-	    ret = 1; 
-	} else {
-	    T[indent] = !T[indent];
-	    got_else[indent] = 1;
-	}
-    } else if (code == SET_ELIF) {
-	if (got_else[indent] || !got_if[indent]) {
-	    strcpy(gretl_errmsg, "Unmatched \"elif\"");
-	    ret = 1; 
-	} else {
-	    T[indent] = !T[indent];
-	}
-    } else if (code == SET_ENDIF) {
-	if (!got_if[indent] || indent == 0) {
-	    strcpy(gretl_errmsg, "Unmatched \"endif\"");
-	    ret = 1; 
-	} else {
-	    got_if[indent] = 0;
-	    got_else[indent] = 0;
-	    indent--;
-	}
-    } else if (code == IS_FALSE) {
-	for (i=1; i<=indent; i++) {
-	    if (T[i] == 0) {
-		ret = 1;
-		break;
-	    }
-	}
-    }
-
-#if IFDEBUG
-    fprintf(stderr, "ifstate: returning %d (indent = %d)\n", ret, indent);
-#endif
-
-    return ret;
-}
