@@ -1,20 +1,20 @@
-/*
- *  Copyright (c) 2003-2005 by Allin Cottrell
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
- *
- *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- *
+/* 
+ *  gretl -- Gnu Regression, Econometrics and Time-series Library
+ *  Copyright (C) 2001 Allin Cottrell and Riccardo "Jack" Lucchetti
+ * 
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ * 
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ * 
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 /* Nonlinear least squares for libgretl, using minpack; also
@@ -27,6 +27,7 @@
 #include "usermat.h"
 #include "gretl_func.h"
 #include "nlspec.h"
+#include "cmd_private.h"
 
 #include "../../minpack/minpack.h"  
 
@@ -58,7 +59,6 @@ struct parm_ {
 
 static nlspec private_spec;
 static integer one = 1;
-static int genr_err;
 
 static void destroy_genrs_array (GENERATOR **genrs, int n)
 {
@@ -142,7 +142,11 @@ static int nls_genr_setup (nlspec *s)
 	genrs[i] = NULL;
     }
 
+#if 0 
+    /* namespace problems when two or more non-nested functions
+       are called in an NLS/MLE/GMM block! */
     set_drop_function_vars(0);
+#endif
 
     j = 0;
 
@@ -169,56 +173,66 @@ static int nls_genr_setup (nlspec *s)
 	}
 	
 	genrs[i] = genr_compile(formula, s->Z, s->dinfo, &err);
-#if NLS_DEBUG
-	fprintf(stderr, "genrs[%d] = %p, err = %d\n", i, (void *) genrs[i], err);
-#endif
 
-	if (!err) {
-	    /* see if the formula actually works, and flush out NAs
-	       while we're at it
-	    */
-	    genr_set_na_check(genrs[i]);
-	    err = execute_genr(genrs[i], s->Z, s->dinfo, s->prn);
-	    genr_unset_na_check(genrs[i]);
+	if (err) {
+	    fprintf(stderr, "genr_compile: genrs[%d] = %p, err = %d\n", i, 
+		    (void *) genrs[i], err);
+	    fprintf(stderr, "formula: '%s'\n", formula);
+	    break;
 	}
 
-	if (!err) {
-	    v = genr_get_output_varnum(genrs[i]);
-	    m = genr_get_output_matrix(genrs[i]);
+	/* see if the formula actually works, and flush out NAs
+	   while we're at it
+	*/
+	genr_set_na_check(genrs[i]);
+	err = execute_genr(genrs[i], s->Z, s->dinfo, s->prn);
+	genr_unset_na_check(genrs[i]);
 
-	    if (v == 0 && m == NULL) {
+	if (err) {
+	    fprintf(stderr, "execute_genr: genrs[%d] = %p, err = %d\n", i, 
+		    (void *) genrs[i], err);
+	    fprintf(stderr, "formula: '%s'\n", formula);
+	    break;
+	}
+
+	v = genr_get_output_varnum(genrs[i]);
+	m = genr_get_output_matrix(genrs[i]);
+
+	if (v == 0 && m == NULL) {
+	    if (genr_is_print(genrs[i])) {
+		continue;
+	    } else {
+		fprintf(stderr, "nls_genr_setup: bad type: %s\n", formula);
 		err = E_TYPES;
 		break;
 	    }
-
-	    if (i == s->naux) {
-		s->lhv = v;
-		s->lvec = m;
-		if (s->lvec != NULL) {
-		    err = check_lhs_vec(s);
-		}
-	    } else if (j > 0) {
-		int k = j - 1;
-
-		s->params[k].dnum = v;
-		s->params[k].dmat = m;
-
-		if (m != NULL || !scalar_param(s, k)) {
-		    err = check_derivative_matrix(k, m, s);
-		} 
-	    }
-#if NLS_DEBUG
-	    fprintf(stderr, " formula '%s'\n", formula);
-	    fprintf(stderr, " v = %d, m = %p\n", v, (void *) m);
-	    if (v > 0) {
-		fprintf(stderr, " first value: Z[%d][%d] = %g\n", 
-			v, s->t1, (*s->Z)[v][s->t1]);
-	    }
-#endif
-	} else {
-	    fprintf(stderr, "execute_genr: formula '%s', error = %d\n", 
-		    formula, err);
 	}
+
+	if (i == s->naux) {
+	    s->lhv = v;
+	    s->lvec = m;
+	    if (s->lvec != NULL) {
+		err = check_lhs_vec(s);
+	    }
+	} else if (j > 0) {
+	    int k = j - 1;
+
+	    s->params[k].dnum = v;
+	    s->params[k].dmat = m;
+
+	    if (m != NULL || !scalar_param(s, k)) {
+		err = check_derivative_matrix(k, m, s);
+	    } 
+	}
+
+#if NLS_DEBUG
+	fprintf(stderr, " formula '%s'\n", formula);
+	fprintf(stderr, " v = %d, m = %p\n", v, (void *) m);
+	if (v > 0) {
+	    fprintf(stderr, " first value: Z[%d][%d] = %g\n", 
+		    v, s->t1, (*s->Z)[v][s->t1]);
+	}
+#endif
     }
 
     if (err) {
@@ -247,23 +261,23 @@ static int nls_auto_genr (nlspec *s, int i)
 #endif
 
     if (s->genrs == NULL) {
-	genr_err = nls_genr_setup(s);
-	if (genr_err) {
+	s->generr = nls_genr_setup(s);
+	if (s->generr) {
 	    fprintf(stderr, " nls_genr_setup failed\n");
 	}
-	return genr_err;
+	return s->generr;
     }
 
     for (j=0; j<s->naux; j++) {
 #if NLS_DEBUG
 	fprintf(stderr, " generating aux var %d:\n %s\n", j, s->aux[j]);
 #endif
-	genr_err = execute_genr(s->genrs[j], s->Z, s->dinfo, s->prn);
+	s->generr = execute_genr(s->genrs[j], s->Z, s->dinfo, s->prn);
     }
 
     if (i == 0 && s->nlfunc == NULL) {
 	/* we're done */
-	return genr_err;
+	return s->generr;
     }
 
     j = s->naux + i;
@@ -271,25 +285,25 @@ static int nls_auto_genr (nlspec *s, int i)
     fprintf(stderr, " j = naux + i = %d+%d = %d: executing genr[%d]\n", 
 	    s->naux, i, j, j);
 #endif
-    genr_err = execute_genr(s->genrs[j], s->Z, s->dinfo, s->prn);
+    s->generr = execute_genr(s->genrs[j], s->Z, s->dinfo, s->prn);
 
     /* make sure we have a correct pointer to matrix deriv */
-    if (!genr_err && i > 0 && matrix_deriv(s, i-1)) {
+    if (!s->generr && i > 0 && matrix_deriv(s, i-1)) {
 	gretl_matrix *m = genr_get_output_matrix(s->genrs[j]);
 
-	genr_err = check_derivative_matrix(i-1, m, s);
+	s->generr = check_derivative_matrix(i-1, m, s);
     }
 
 #if NLS_DEBUG
-    if (genr_err) {
+    if (s->generr) {
 	int v = genr_get_output_varnum(s->genrs[j]);
 
-	fprintf(stderr, " varnum = %d, err = %d\n", v, genr_err);
-	errmsg(genr_err, s->prn);
+	fprintf(stderr, " varnum = %d, err = %d\n", v, s->generr);
+	errmsg(s->generr, s->prn);
     } 
 #endif
 
-    return genr_err;    
+    return s->generr;    
 }
 
 /* wrappers for the above to enhance comprehensibility below */
@@ -1833,6 +1847,7 @@ static void clear_nlspec (nlspec *spec)
 	spec->genrs = NULL;
     }
     spec->ngenrs = 0;
+    spec->generr = 0;
 
     free(spec->nlfunc);
     spec->nlfunc = NULL;
@@ -2043,7 +2058,7 @@ static int mle_calculate (nlspec *s, double *fvec, double *jac, PRN *prn)
     if (!err) {
 	BFGS_GRAD_FUNC gradfun = (s->mode == ANALYTIC_DERIVS)?
 	    get_mle_gradient : NULL;
-	int maxit = get_bfgs_maxiter();
+	int maxit = libset_get_int(BFGS_MAXITER);
 
 	err = BFGS_max(s->coeff, s->ncoeff, maxit, s->tol, 
 		       &s->fncount, &s->grcount, 
@@ -2324,11 +2339,48 @@ nlspec_add_param_with_deriv (nlspec *spec, const char *dstr,
     return err;
 }
 
+static void get_aux_command_word (char *word, const char *line)
+{
+    int n = gretl_varchar_spn(line);
+
+    *word = '\0';
+
+    if (n > 0) {
+	if (n > FN_NAMELEN - 1) {
+	    n = FN_NAMELEN - 1;
+	}
+	strncat(word, line, n);
+    } 
+}
+
+static int screen_bad_aux (const char *line, const DATAINFO *pdinfo)
+{
+    char word[FN_NAMELEN];
+    int ci, err = E_DATA;
+
+    get_aux_command_word(word, line);
+    ci = gretl_command_number(word);
+
+    if (ci == GENR) {
+	err = 0;
+    } else if (plausible_genr_start(line, pdinfo)) {
+	err = 0;
+    } else if (get_user_function_by_name(word)) {
+	err = 0;
+    } else {
+	sprintf(gretl_errmsg, _("command '%s' not valid in this context"), 
+		word);
+    }
+
+    return err;
+}
+
 /**
  * nlspec_add_aux:
  * @spec: pointer to nls specification.
  * @s: string specifying an auxiliary command (primarily
  * for use in calculating function or derivatives).
+ * @pdinfo: pointer to dataset information.
  *
  * Adds the specification of an auxiliary command to @spec, 
  * which pointer must have previously been obtained by a call 
@@ -2337,7 +2389,7 @@ nlspec_add_param_with_deriv (nlspec *spec, const char *dstr,
  * Returns: 0 on success, non-zero error code on error.
  */
 
-int nlspec_add_aux (nlspec *spec, const char *s)
+int nlspec_add_aux (nlspec *spec, const char *s, const DATAINFO *pdinfo)
 {
     char **aux;
     char *this;
@@ -2348,6 +2400,11 @@ int nlspec_add_aux (nlspec *spec, const char *s)
     fprintf(stderr, "nlspec_add_aux: s = '%s'\n", s);
 #endif
 
+    err = screen_bad_aux(s, pdinfo);
+    if (err) {
+	return err;
+    }
+
     this = gretl_strdup(s);
     if (this == NULL) {
 	err = E_ALLOC;
@@ -2555,7 +2612,12 @@ int nls_parse_line (int ci, const char *line, const double **Z,
 	    }
 	}
     } else {
-	err = nlspec_add_aux(s, line);
+	err = nlspec_add_aux(s, line, pdinfo);
+    }
+
+    if (err) {
+	/* remember to clean up! */
+	clear_nlspec(s);
     }
 
     return err;
@@ -2609,7 +2671,6 @@ static MODEL real_nls (nlspec *spec, double ***pZ, DATAINFO *pdinfo,
     fprintf(stderr, "real_nls: starting\n");
 #endif
 
-    genr_err = 0;
     gretl_model_init(&nlsmod);
     gretl_model_smpl_init(&nlsmod, pdinfo);
 
@@ -2681,10 +2742,10 @@ static MODEL real_nls (nlspec *spec, double ***pZ, DATAINFO *pdinfo,
     }
 
     /* get tolerance from user setting or default */
-    if (spec->ci == MLE || spec->ci == GMM) {
-	spec->tol = get_bfgs_toler();
+    if (USES_BFGS(spec->ci)) {
+	spec->tol = libset_get_double(BFGS_TOLER);
     } else {
-	spec->tol = get_nls_toler();
+	spec->tol = libset_get_double(NLS_TOLER);
     }
 
     pputs(prn, (spec->mode == NUMERIC_DERIVS)?
@@ -2724,8 +2785,8 @@ static MODEL real_nls (nlspec *spec, double ***pZ, DATAINFO *pdinfo,
 	}
     } else {
 	if (nlsmod.errcode == 0) { 
-	    if (genr_err != 0) {
-		nlsmod.errcode = genr_err;
+	    if (spec->generr != 0) {
+		nlsmod.errcode = spec->generr;
 	    } else {
 		nlsmod.errcode = E_NOCONV;
 	    }
@@ -2830,6 +2891,7 @@ nlspec *nlspec_new (int ci, const DATAINFO *pdinfo)
     
     spec->genrs = NULL;
     spec->ngenrs = 0;
+    spec->generr = 0;
     
     spec->coeff = NULL;
     spec->ncoeff = 0;
@@ -3034,9 +3096,9 @@ double *numerical_hessian (double *b, int n, BFGS_CRIT_FUNC func, void *data,
 		goto bailout;
 	    }
 	    /* F'(i) */
-	    Dx[k] = (f1 - f2) / (2.0*h[i]); 
+	    Dx[k] = (f1 - f2) / (2.0 * h[i]); 
 	    /* F''(i) */
-	    Hx[k] = (f1 - 2.0*f0 + f2) / (h[i]*h[i]);
+	    Hx[k] = (f1 - 2.0*f0 + f2) / (h[i] * h[i]);
 	    hess_h_reduce(h, v, n);
 	}
 	p4m = 4;
@@ -3126,7 +3188,7 @@ double *numerical_hessian (double *b, int n, BFGS_CRIT_FUNC func, void *data,
 }
 
 #define stepfrac	0.2
-#define acctol		0.0001 
+#define acctol		1.0e-7 /* alt: 0.0001 or 1.0e-7 (?) */
 #define reltest		10.0
 
 #define coeff_unchanged(a,b) (reltest + a == reltest + b)
@@ -3140,6 +3202,76 @@ static void reverse_gradient (double *g, int n)
     }
 }
 
+static int broken_gradient (double *g, int n)
+{
+    int i;
+
+    for (i=0; i<n; i++) {
+	if (isnan(g[i])) {
+	    return 1;
+	}
+    }
+
+    return 0;
+}
+
+/* 
+   If "set inivals" has been used, supersede whatever initial
+   values were there by those given by the user (the customer is
+   always right).  In addition, respect user settings for the
+   maximum number of iterations and the convergence tolerance.
+*/
+
+static void BFGS_get_user_values (double *b, int n, int *maxit,
+				  double *reltol, gretlopt opt,
+				  PRN *prn)
+{
+    const gretl_matrix *uinit;
+    int uilen, umaxit;
+    double utol;
+    int i;
+
+    uinit = get_init_vals();
+    uilen = gretl_vector_get_length(uinit);
+
+    if (uilen > 0) {
+	/* the user has given something */
+	if (uilen < n) {
+	    fprintf(stderr, "Only %d initial values given, but %d "
+		    "are necessary\n", uilen, n);
+	} else {
+	    for (i=0; i<n; i++) {
+		b[i] = uinit->val[i];
+	    }
+	    if (opt & OPT_V) {
+		pputs(prn, _("\n\n*** User-specified starting values:\n"));
+		for (i=0; i<n; i++) {
+		    pprintf(prn, " %12.6f", b[i]);
+		    if (i%6 == 5) {
+			pputc(prn, '\n');
+		    }
+		}
+		pputs(prn, "\n\n");
+	    }
+	    free_init_vals();
+	}
+    }
+
+    umaxit = libset_get_int(BFGS_MAXITER);
+    if (umaxit > 0) {
+	*maxit = umaxit;
+    }
+    
+    utol = libset_get_double(BFGS_TOLER);
+    if (utol != get_default_nls_toler()) {
+	/* it has actually been set */
+	*reltol = utol;
+	if (!(opt & OPT_Q)) {
+	    fprintf(stderr, "user-specified BFGS tolerance = %g\n", utol);
+	}
+    }	
+}
+
 /**
  * BFGS_max:
  * @b: array of adjustable coefficients.
@@ -3171,20 +3303,23 @@ static void reverse_gradient (double *g, int n)
  * on error.
  */
 
-int BFGS_max (double *b, int n, int maxit, double reltol,
-	      int *fncount, int *grcount, BFGS_CRIT_FUNC cfunc, 
-	      int crittype, BFGS_GRAD_FUNC gradfunc, void *data, 
-	      gretlopt opt, PRN *prn)
+int BFGS_orig (double *b, int n, int maxit, double reltol,
+	       int *fncount, int *grcount, BFGS_CRIT_FUNC cfunc, 
+	       int crittype, BFGS_GRAD_FUNC gradfunc, void *data, 
+	       gretlopt opt, PRN *prn)
 {
     int crit_ok, done;
     double *g = NULL, *t = NULL, *X = NULL, *c = NULL, **H = NULL;
+    int verbose = (opt & OPT_V);
     int ndelta, fcount, gcount;
-    double d, fmax, f, sumgrad;
+    double d, fmax, f, f0, sumgrad;
     int i, j, ilast, iter;
     double s, steplen = 0.0;
     double D1, D2;
     int err = 0;
 
+    BFGS_get_user_values(b, n, &maxit, &reltol, opt, prn);
+
     if (gradfunc == NULL) {
 	gradfunc = BFGS_numeric_gradient;
     }
@@ -3208,17 +3343,18 @@ int BFGS_max (double *b, int n, int maxit, double reltol,
 	goto bailout;
     }
 
-    fmax = f;
+    f0 = fmax = f;
     iter = ilast = fcount = gcount = 1;
     gradfunc(b, g, n, cfunc, data);
     reverse_gradient(g, n);
 
     do {
-	if (opt & OPT_V) {
+	if (verbose) {
 	    reverse_gradient(g, n);
 	    print_iter_info(iter, f, crittype, n, b, g, steplen, prn);
 	    reverse_gradient(g, n);
 	}
+
 	if (ilast == gcount) {
 	    /* (re-)start: initialize curvature matrix */
 	    for (i=0; i<n; i++) {
@@ -3228,11 +3364,13 @@ int BFGS_max (double *b, int n, int maxit, double reltol,
 		H[i][i] = 1.0;
 	    }
 	}
+
 	for (i=0; i<n; i++) {
 	    /* copy coefficients to X, gradient to c */
 	    X[i] = b[i];
 	    c[i] = g[i];
 	}
+
 	sumgrad = 0.0;
 	for (i=0; i<n; i++) {
 	    s = 0.0;
@@ -3250,8 +3388,8 @@ int BFGS_max (double *b, int n, int maxit, double reltol,
 	    steplen = 1.0;
 	    crit_ok = 0;
 	    do {
-		/* loop so long as (a) we haven't achieved a definite
-		   improvement in the criterion and (b) there is
+		/* loop so long as (a) we haven't achieved an
+		   acceptable value of the criterion and (b) there is
 		   still some prospect of doing so */
 		ndelta = n;
 		for (i=0; i<n; i++) {
@@ -3315,8 +3453,7 @@ int BFGS_max (double *b, int n, int maxit, double reltol,
 		    D2 = 1.0 + D2 / D1;
 		    for (i=0; i<n; i++) {
 			for (j=0; j<=i; j++) {
-			    H[i][j] += (D2 * t[i]*t[j]
-					- X[i]*t[j] - t[i]*X[j]) / D1;
+			    H[i][j] += (D2 * t[i]*t[j] - X[i]*t[j] - t[i]*X[j]) / D1;
 			}
 		    }
 		} else {
@@ -3328,12 +3465,15 @@ int BFGS_max (double *b, int n, int maxit, double reltol,
 		ilast = gcount;
 	    }
 	} else {
-	    /* downhill search */
+	    /* heading in the wrong direction */
 	    if (ilast == gcount) {
 		/* we just reset: don't reset again; set ndelta = 0 so
 		   that we exit the main loop
 		*/
 		ndelta = 0;
+		if (gcount == 1) {
+		    err = (broken_gradient(g, n))? E_NAN : E_DATA;
+		}
 	    } else {
 		/* reset for another attempt */
 		ilast = gcount;
@@ -3360,15 +3500,18 @@ int BFGS_max (double *b, int n, int maxit, double reltol,
     if (iter >= maxit) {
 	fprintf(stderr, _("stopped after %d iterations\n"), iter);
 	err = E_NOCONV;
+    } else if (fmax < f0) {
+	/* FIXME this should never happen */
+	fprintf(stderr, "failed to match initial value of objective function, %g\n", f0);
+	err = E_NOCONV;
     }
 
     *fncount = fcount;
     *grcount = gcount;
 
-    if (opt & OPT_V) {
-	pputs(prn, _("\n--- FINAL VALUES: \n"));
+    if (verbose) {
 	reverse_gradient(g, n);
-	print_iter_info(iter, f, crittype, n, b, g, steplen, prn);
+	print_iter_info(-1, f, crittype, n, b, g, steplen, prn);
 	pputc(prn, '\n');
     }
 
@@ -3383,6 +3526,194 @@ int BFGS_max (double *b, int n, int maxit, double reltol,
     return err;
 }
 
+int LBFGS_max (double *b, int n, int maxit, double reltol,
+	       int *fncount, int *grcount, BFGS_CRIT_FUNC cfunc, 
+	       int crittype, BFGS_GRAD_FUNC gradfunc, void *data, 
+	       gretlopt opt, PRN *prn)
+{
+    double *g = NULL;
+    double *l = NULL;
+    double *u = NULL;
+    double *wa = NULL;
+    int *nbd = NULL;
+    int *iwa = NULL;
+
+    int i, m, wadim;
+    char task[60];
+    char csave[60];
+    double f, pgtol;
+    double dsave[29];
+    int isave[44];
+    int lsave[4];
+    int iter, ibak = 0;
+    int err = 0;
+
+    *fncount = *grcount = 0;    
+
+    BFGS_get_user_values(b, n, &maxit, &reltol, opt, prn);
+
+    /*
+      m: the number of corrections used in the limited memory matrix.
+      It is not altered by the routine.  Values of m < 3 are not
+      recommended, and large values of m can result in excessive
+      computing time. The range 3 <= m <= 20 is recommended.
+    */
+    m = 10; /* was initially set to 5 */
+
+    wadim = (2*m+4)*n + 12*m*m + 12*m;
+
+    g = malloc(n * sizeof *g);
+    l = malloc(n * sizeof *l);
+    u = malloc(n * sizeof *u);
+    wa = malloc(wadim * sizeof *wa);
+    nbd = malloc(n * sizeof *nbd);
+    iwa = malloc(3*n * sizeof *iwa);
+
+    if (g == NULL || l == NULL || u == NULL ||
+	wa == NULL || nbd == NULL || iwa == NULL) {
+	err = E_ALLOC;
+	goto bailout;
+    }
+
+    if (gradfunc == NULL) {
+	gradfunc = BFGS_numeric_gradient;
+    }
+
+    /* Gradient convergence criterion (not used -- we use reltol instead) */
+    pgtol = 0;
+
+    /* Bounds on the parameters: for now we just set them all to be
+       less than some ridiculously large number */
+    for (i=0; i<n; i++) {
+	nbd[i] = 3; /* case 3: upper bound only */
+	u[i] = NADBL / 100;
+    }	
+
+    /* Start the iteration by initializing 'task' */
+    strcpy(task, "START");
+
+    while (1) {
+	/* Call the L-BFGS-B code */
+	setulb_(&n, &m, b, l, u, nbd, &f, g, &reltol, &pgtol, wa, iwa, 
+		task, csave, lsave, isave, dsave);
+
+	iter = isave[29] + 1;
+
+	if (!strncmp(task, "FG", 2)) {
+
+	    /* Compute function value, f */
+	    f = cfunc(b, data);
+	    if (!na(f)) {
+		f = -f;
+	    } else if (*fncount == 0) {
+		fprintf(stderr, "initial value of f is not finite\n");
+		err = E_DATA;
+		break;
+	    }
+	    *fncount += 1;
+
+	    /* Compute gradient, g */
+	    gradfunc(b, g, n, cfunc, data);
+	    reverse_gradient(g, n);
+	    *grcount += 1;
+	    
+	} else if (!strncmp(task, "NEW_X", 5)) {
+	    /* The optimizer has produced a new set of parameter values */
+	    if (isave[33] >= maxit) {
+		strcpy(task, "STOP: TOTAL NO. of f AND g "
+		       "EVALUATIONS EXCEEDS LIMIT");
+		err = E_NOCONV;
+		break;
+	    } 
+	} else {
+	    fprintf(stderr, "%s\n", task);
+	    break;
+	}
+
+	if (opt & OPT_V) {
+	    if (iter != ibak) {
+		double steplen = (iter == 1)? NADBL : dsave[13];
+
+		reverse_gradient(g, n);
+		print_iter_info(iter, -f, crittype, n, b, g, steplen, prn);
+		reverse_gradient(g, n);
+	    }
+	    ibak = iter;
+	}
+    }
+
+    if (!err && crittype == C_GMM) {
+	/* finalize GMM computations */
+	f = cfunc(b, data);
+    }
+
+    if (opt & OPT_V) {
+	reverse_gradient(g, n);
+	print_iter_info(-1, -f, crittype, n, b, g, dsave[13], prn);
+	pputc(prn, '\n');
+    }
+
+ bailout:
+
+    free(g);
+    free(l);
+    free(u);
+    free(wa);
+    free(nbd);
+    free(iwa);
+
+    return err;
+}
+
+/**
+ * BFGS_max:
+ * @b: array of adjustable coefficients.
+ * @n: number elements in array @b.
+ * @maxit: the maximum number of iterations to allow.
+ * @reltol: relative tolerance for terminating iteration.
+ * @fncount: location to receive count of function evaluations.
+ * @grcount: location to receive count of gradient evaluations.
+ * @cfunc: pointer to function used to calculate maximand.
+ * @crittype: code for type of the maximand/minimand: should
+ * be %C_LOGLIK, %C_GMM or %C_OTHER.  Used only in printing
+ * iteration info.
+ * @gradfunc: pointer to function used to calculate the 
+ * gradient, or %NULL for default numerical calculation.
+ * @data: pointer that will be passed as the last
+ * parameter to the callback functions @cfunc and @gradfunc.
+ * @opt: may contain %OPT_V for verbose operation.
+ * @prn: printing struct (or %NULL).  Only used if @opt
+ * includes %OPT_V.
+ *
+ * Obtains the set of values for @b which jointly maximize the
+ * criterion value as calculated by @cfunc.  Uses the BFGS
+ * variable-metric method.  Based on Pascal code in J. C. Nash,
+ * "Compact Numerical Methods for Computers," 2nd edition, converted
+ * by p2c then re-crafted by B. D. Ripley for gnu R.  Revised for 
+ * gretl by Allin Cottrell.
+ * 
+ * Returns: 0 on successful completion, non-zero error code
+ * on error.
+ */
+
+int BFGS_max (double *b, int n, int maxit, double reltol,
+	      int *fncount, int *grcount, BFGS_CRIT_FUNC cfunc, 
+	      int crittype, BFGS_GRAD_FUNC gradfunc, void *data, 
+	      gretlopt opt, PRN *prn)
+{
+    if (libset_get_bool(USE_LBFGS)) {
+	return LBFGS_max(b, n, maxit, reltol,
+			 fncount, grcount, cfunc, 
+			 crittype, gradfunc, data, 
+			 opt, prn);
+    } else {
+	return BFGS_orig(b, n, maxit, reltol,
+			 fncount, grcount, cfunc, 
+			 crittype, gradfunc, data, 
+			 opt, prn);
+    }
+}
+
 /* user-level access to BFGS */
 
 typedef struct umax_ umax;
@@ -3506,10 +3837,10 @@ double user_BFGS (gretl_matrix *b, const char *fncall,
 	return NADBL;
     }
 
-    maxit = get_bfgs_maxiter();
-    tol = get_bfgs_toler();
+    maxit = libset_get_int(BFGS_MAXITER);
+    tol = libset_get_double(BFGS_TOLER);
 
-    if (get_max_verbose()) {
+    if (libset_get_bool(MAX_VERBOSE)) {
 	opt = OPT_V;
 	u.prn = prn;
     }
