@@ -1,218 +1,216 @@
-/*
- *  Copyright (c) by Allin Cottrell
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
- *
- *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
- *
+/* 
+ *  gretl -- Gnu Regression, Econometrics and Time-series Library
+ *  Copyright (C) 2001 Allin Cottrell and Riccardo "Jack" Lucchetti
+ * 
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ * 
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ * 
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 /* non-parametric stats for gretl */
 
 #include "libgretl.h"
 
-enum {
-    RUN_FIRST,
-    RUN_REPEAT,
-    RUN_CLEANUP
+/* Spearman, one-tailed alpha = .005, .025, .05 */
+
+static double rhocrit[18][3] = { 
+    { 0.8929, 0.7450, 0.6786 }, /*  n = 7 */
+    { 0.8571, 0.7143, 0.6190 }, /*  8 */
+    { 0.8167, 0.6833, 0.5833 }, /*  9 */
+    { 0.7818, 0.6364, 0.5515 }, /* 10 */
+    { 0.7545, 0.6091, 0.5273 }, /* 11 */
+    { 0.7273, 0.5804, 0.4965 }, /* 12 */
+    { 0.6978, 0.5549, 0.4780 }, /* 13 */
+    { 0.6747, 0.5341, 0.4593 }, /* 14 */
+    { 0.6536, 0.5179, 0.4429 }, /* 15 */
+    { 0.6324, 0.5000, 0.4265 }, /* 16 */
+    { 0.6152, 0.4853, 0.4118 }, /* 17 */
+    { 0.5975, 0.4716, 0.3994 }, /* 18 */
+    { 0.5825, 0.4579, 0.3895 }, /* 19 */
+    { 0.5684, 0.4451, 0.3789 }, /* 20 */
+    { 0.5545, 0.4351, 0.3688 }, /* 21 */
+    { 0.5426, 0.4241, 0.3597 }, /* 22 */
+    { 0.5306, 0.4150, 0.3518 }, /* 23 */
+    { 0.5200, 0.4061, 0.3435 }  /* 24 */
 };
 
-/* alpha = .001, .01, .025, .05, .1 */
-
-static double spearman_critical[18][5] = {      /* n = */
-    { 0.9643, 0.8571, 0.7450, 0.6786, 0.5357 }, /*  7 */
-    { 0.9286, 0.8095, 0.7143, 0.6190, 0.5000 }, /*  8 */
-    { 0.9000, 0.7667, 0.6833, 0.5833, 0.4667 }, /*  9 */
-    { 0.8667, 0.7333, 0.6364, 0.5515, 0.4424 }, /* 10 */
-    { 0.8364, 0.7000, 0.6091, 0.5273, 0.4182 }, /* 11 */
-    { 0.8182, 0.6713, 0.5804, 0.4965, 0.3986 }, /* 12 */
-    { 0.7912, 0.6429, 0.5549, 0.4780, 0.3791 }, /* 13 */
-    { 0.7670, 0.6220, 0.5341, 0.4593, 0.3626 }, /* 14 */
-    { 0.7464, 0.6000, 0.5179, 0.4429, 0.3500 }, /* 15 */
-    { 0.7265, 0.5824, 0.5000, 0.4265, 0.3382 }, /* 16 */
-    { 0.7083, 0.5637, 0.4853, 0.4118, 0.3260 }, /* 17 */
-    { 0.6904, 0.5480, 0.4716, 0.3994, 0.3148 }, /* 18 */
-    { 0.6737, 0.5333, 0.4579, 0.3895, 0.3070 }, /* 19 */
-    { 0.6586, 0.5203, 0.4451, 0.3789, 0.2977 }, /* 20 */
-    { 0.6455, 0.5078, 0.4351, 0.3688, 0.2909 }, /* 21 */
-    { 0.6318, 0.4963, 0.4241, 0.3597, 0.2829 }, /* 22 */
-    { 0.6186, 0.4852, 0.4150, 0.3518, 0.2767 }, /* 23 */
-    { 0.6070, 0.4748, 0.4061, 0.3435, 0.2704 }  /* 24 */
-};
+/* The values above are from Daniel and Terrell,
+   Business Statistics, 4e (Houghton Mifflin, 1986)
+*/
 
 static double spearman_signif (double rho, int n)
 {
-    const double *vals = spearman_critical[n - 7];
+    const double *x;
 
-    if (rho > vals[0]) return .001;
-    else if (rho > vals[1]) return .01;
-    else if (rho > vals[2]) return .025;
-    else if (rho > vals[3]) return .05;
-    else if (rho > vals[4]) return .1;
-    else return 1.0;
+    if (n < 7) {
+	return 1.0;
+    } 
+    
+    x = rhocrit[n - 7];
+
+    if (rho > x[0]) return .01;
+    if (rho > x[1]) return .05;
+    if (rho > x[2]) return .10;
+    return 1.0;
 }
 
-static int spearman_rho (const double *x, const double *y, int n, 
-			 double *rho, double *sd, double *pval,
-			 double **rxout, double **ryout, int *m,
-			 int code)
+enum {
+    RANK_X,
+    RANK_Y
+};
+
+/* m = number of sorted values (sz); n = number of raw values (either
+   x or y); in case of missing values in x and/or y we may have m < n.
+*/
+
+static void make_ranking (const double *sz, int m,
+			  const double *x, const double *y, 
+			  int n, double *rz, int *ties, 
+			  int which)
 {
-    static double *sx = NULL;
-    static double *sy = NULL;
-    static double *rx = NULL;
-    static double *ry = NULL;
-    static int nn;
+    const double *z;
+    int cases, k, i, j;
+    double avg, r = 1;
 
-    double xx, yy, r;
-    int i, j, t;
+    z = (which == RANK_X)? x : y;
 
-    if (code == RUN_CLEANUP) {
-	free(sx); sx = NULL;
-	free(sy); sy = NULL;
-	free(rx); rx = NULL;
-	free(ry); ry = NULL;
-	nn = 0;
-	return 0;
-    }
+    for (i=0; i<m; i++) {
+	/* scan sorted z */
+	cases = k = 0;
 
-    *rho = NADBL;
-    *sd = NADBL;
-    if (pval != NULL) {
-	*pval = NADBL;
-    }
-
-    if (code == RUN_FIRST) {
-	sx = malloc(n * sizeof *sx);
-	sy = malloc(n * sizeof *sy);
-	rx = malloc(n * sizeof *rx);
-	ry = malloc(n * sizeof *ry);
-
-	if (sx == NULL || sy == NULL || 
-	    rx == NULL || ry == NULL) { 
-	    return E_ALLOC;
+	if (i > 0 && sz[i] == sz[i-1]) {
+	    continue;
 	}
+
+	for (j=0; j<n; j++) {
+	    /* scan raw z for matches */
+	    if (!na(x[j]) && !na(y[j])) {
+		if (z[j] == sz[i]) {
+		    rz[k] = r;
+		    cases++;
+		}
+		k++;
+	    }
+	}
+
+	if (cases > 1) {
+	    avg = (r + r + cases - 1.0) / 2.0;
+	    for (j=0; j<m; j++) {
+		if (rz[j] == r) {
+		    rz[j] = avg;
+		}
+	    }
+	    *ties = 1;
+	} 
+
+	r += cases;
+    }
+}
+
+static int real_spearman_rho (const double *x, const double *y, int n, 
+			      double *rho, double *zval,
+			      double **rxout, double **ryout, 
+			      int *pm)
+{
+    double *sx = NULL, *sy = NULL;
+    double *rx = NULL, *ry = NULL;
+    double sd, sr;
+    int i, m, ties = 0;
+
+    *rho = *zval = NADBL;
+
+    /* count non-missing pairs */
+    m = 0;
+    for (i=0; i<n; i++) {
+	if (!na(x[i]) && !na(y[i])) {
+	    m++;
+	}
+    }    
+
+    if (m < 2) {
+	return E_DATA;
+    }
+
+    sx = malloc(m * sizeof *sx);
+    sy = malloc(m * sizeof *sy);
+    rx = malloc(m * sizeof *rx);
+    ry = malloc(m * sizeof *ry);
+
+    if (sx == NULL || sy == NULL || 
+	rx == NULL || ry == NULL) { 
+	free(sx);
+	free(sy);
+	free(rx);
+	free(ry);
+	return E_ALLOC;
     }
 
     /* copy non-missing x and y into sx, sy */
-    nn = 0;
-    for (t=0; t<n; t++) {
-	if (na(x[t]) || na(y[t])) {
-	    continue;
+    m = 0;
+    for (i=0; i<n; i++) {
+	if (!na(x[i]) && !na(y[i])) {
+	    sx[m] = x[i];
+	    sy[m] = y[i];
+	    m++;
 	}
-	sx[nn] = x[t];
-	sy[nn] = y[t];
-	nn++;
     }
 
     /* get sorted series */
-    qsort(sx, nn, sizeof *sx, gretl_inverse_compare_doubles);
-    qsort(sy, nn, sizeof *sy, gretl_inverse_compare_doubles);
+    qsort(sx, m, sizeof *sx, gretl_inverse_compare_doubles);
+    qsort(sy, m, sizeof *sy, gretl_inverse_compare_doubles);
 
-    for (t=0; t<n; t++) {
-	rx[t] = ry[t] = 0.0;
+    for (i=0; i<m; i++) {
+	rx[i] = ry[i] = 0.0;
     }
 
     /* make rankings by comparing "raw" x, y with sorted */
+    make_ranking(sx, m, x, y, n, rx, &ties, RANK_X);
+    make_ranking(sy, m, x, y, n, ry, &ties, RANK_Y);
 
-    r = 1.0;
-    for (i=0; i<nn; i++) {
-	/* scan sorted x */
-	int cases = 0, k = 0;
-
-	if (i > 0 && sx[i] == sx[i-1]) {
-	    continue;
+    if (ties == 0) {
+	/* calculate rho and z-score, no ties */
+	sd = 0.0;
+	for (i=0; i<m; i++) { 
+	    sd += (rx[i] - ry[i]) * (rx[i] - ry[i]);
 	}
+	sr = 1.0 - 6.0 * sd / (m * (m * m - 1));
+	sd = sqrt(1.0 / (m - 1.0));
 
-	for (j=0; j<n; j++) {
-	    /* scan raw x for matches */
-	    if (!na(x[j]) && !na(y[j])) {
-		if (x[j] == sx[i]) {
-		    rx[k] = r;
-		    cases++;
-		}
-		k++;
-	    }
-	}
-	if (cases > 1) {
-	    double avg = (r + r + cases - 1.0) / 2.0;
-
-	    for (t=0; t<nn; t++) {
-		if (rx[t] == r) {
-		    rx[t] = avg;
-		}
-	    }
-	} 
-	r += cases;
+	*rho = sr;
+	*zval = sr / sd;
+    } else {
+	/* use Pearson in case of ties */
+	sr = gretl_corr(0, m - 1, rx, ry, NULL);
+	*rho = sr;
     }
-		
-    r = 1.0;
-    for (i=0; i<nn; i++) {
-	/* scan sorted y */
-	int cases = 0, k = 0;
-
-	if (i > 0 && sy[i] == sy[i-1]) {
-	    continue;
-	}
-
-	for (j=0; j<n; j++) {
-	    /* scan raw y for matches */
-	    if (!na(x[j]) && !na(y[j])) {
-		if (y[j] == sy[i]) {
-		    ry[k] = r;
-		    cases++;
-		}
-		k++;
-	    }
-	}
-	if (cases > 1) {
-	    double avg = (r + r + cases - 1.0) / 2.0;
-
-	    for (t=0; t<nn; t++) {
-		if (ry[t] == r) {
-		    ry[t] = avg;
-		}
-	    }
-	} 
-	r += cases;
-    }	    
-	    
-    /* calculate rho and standard error */
-    xx = 0.0;
-    for (i=0; i<nn; i++) { 
-	xx += (rx[i] - ry[i]) * (rx[i] - ry[i]);
-    }
-    yy = 1.0 - 6.0 * xx / (nn * (nn * nn - 1));
-    xx = sqrt(1.0 / (nn - 1.0));
-
-    *rho = yy;
-    *sd = xx;
-
-    if (nn >= 20 && pval != NULL) {
-	*pval = normal_pvalue_1(fabs(yy / xx)); 
-    } 
 
     /* save the ranks, if wanted */
     if (rxout != NULL) {
 	*rxout = rx;
-    } 
+    } else {
+	free(rx);
+    }
     if (ryout != NULL) {
 	*ryout = ry;
-    } 
-
-    if (m != NULL) {
-	*m = nn;
+    } else {
+	free(ry);
     }
 
+    *pm = m;
+
+    free(sx);
+    free(sy);
+
     return 0;
 }
 
@@ -233,41 +231,55 @@ static int spearman_rho (const double *x, const double *y, int n,
 int spearman (const int *list, const double **Z, const DATAINFO *pdinfo, 
 	      gretlopt opt, PRN *prn)
 {
+    int T = pdinfo->t2 - pdinfo->t1 + 1;
     double *rx = NULL, *ry = NULL;
-    double rho, sd, pval;
+    const double *x, *y;
+    double rho, zval;
     int vx, vy, t, m;
+    int err;
 
     if (list[0] != 2) {
-	pputs(prn, _("spearman command requires two variables\n"));
+	pputs(prn, _("This command requires two variables\n"));
 	return 1;
     }
 
     vx = list[1];
     vy = list[2];
 
-    if (spearman_rho(Z[vx] + pdinfo->t1, 
-		     Z[vy] + pdinfo->t1, 
-		     pdinfo->t2 - pdinfo->t1 + 1, 
-		     &rho, &sd, &pval,
-		     (opt & OPT_V)? &rx : NULL,
-		     (opt & OPT_V)? &ry : NULL,
-		     &m, RUN_FIRST)) {
-	return E_ALLOC;
+    x = Z[vx] + pdinfo->t1;
+    y = Z[vy] + pdinfo->t1;
+
+    err = real_spearman_rho(x, y, T, &rho, &zval,
+			    (opt & OPT_V)? &rx : NULL,
+			    (opt & OPT_V)? &ry : NULL,
+			    &m);
+    if (err) {
+	return err;
     }
 
     pprintf(prn, _("\nFor the variables '%s' and '%s'\n"), pdinfo->varname[vx],
 	    pdinfo->varname[vy]);
-    pprintf(prn, _("Spearman's rank correlation coefficient (rho) = %f\n"), rho);
+    pprintf(prn, _("Spearman's rank correlation coefficient (rho) = %.8f\n"), rho);
 
-    if (!na(pval)) {
-	pprintf(prn, _("Under the null hypothesis of no correlation, rho "
-		       "follows N(0, %f)\n"), sd);
-	pprintf(prn, _("z-score = %f, with one-tailed p-value %f\n"), rho / sd,
-		pval);
+    if (rho == 0.0) {
+	goto skipit;
+    }
+
+    if (!na(zval)) {
+	pputs(prn, _("Under the null hypothesis of no correlation:\n "));
+	pprintf(prn, _("z-score = %g, with two-tailed p-value %.4f\n"), zval,
+		normal_pvalue_2(zval));
+    } else if (m > 24) {
+	double tval = rho * sqrt((m - 2) / (1 - rho*rho)); 
+
+	pputs(prn, _("Under the null hypothesis of no correlation:\n "));
+	pprintf(prn, _("t(%d) = %g, with two-tailed p-value %.4f\n"), m - 2,
+		tval, student_pvalue_2(tval, m - 2));
     } else if (m >= 7) {
-	pval = spearman_signif(m, fabs(rho));
+	double pval = spearman_signif(fabs(rho), m);
+
 	if (pval < 1.0) {
-	    pprintf(prn, _("significant at the %g%% level (one-tailed)\n"), 
+	    pprintf(prn, _("significant at the %g%% level (two-tailed)\n"), 
 		    100.0 * pval);
 	} else {
 	    /* xgettext:no-c-format */
@@ -275,37 +287,243 @@ int spearman (const int *list, const double **Z, const DATAINFO *pdinfo,
 	}
     } else {
 	pputs(prn, _("Sample is too small to calculate a p-value based on "
-		"the normal distribution\n"));
+		     "the normal distribution\n"));
     }
 
-    if (opt & OPT_V) { 
+ skipit:
+
+    if (rx != NULL && ry != NULL) { 
 	/* print raw and ranked data */
 	int i = 0;
 
 	obs_marker_init(pdinfo);
-
 	pprintf(prn, "\n     %s ", _("Obs"));
 	pprintf(prn, "%13s%13s%13s%13s\n\n", pdinfo->varname[vx], _("rank"),
 	       pdinfo->varname[vy], _("rank"));
-	for (t=pdinfo->t1; t<=pdinfo->t2; t++) {
-	    print_obs_marker(t, pdinfo, prn);
-	    if (!(na(Z[vx][t])) && !(na(Z[vy][t]))) {
-		gretl_printxn(Z[vx][t], 15, prn);
+
+	for (t=0; t<T; t++) {
+	    print_obs_marker(t + pdinfo->t1, pdinfo, prn);
+	    if (!(na(x[t])) && !(na(y[t]))) {
+		gretl_printxn(x[t], 15, prn);
 		pprintf(prn, "%15g", rx[i]);
-		gretl_printxn(Z[vy][t], 15, prn);
+		gretl_printxn(y[t], 15, prn);
 		pprintf(prn, "%15g", ry[i]);
 		i++;
 	    }
 	    pputc(prn, '\n');
 	}
+
+	free(rx);
+	free(ry);
     }
 
-    spearman_rho(NULL, NULL, 0, NULL, NULL, NULL, 
-		 NULL, NULL, NULL, RUN_CLEANUP);
+    pputc(prn, '\n');
 
     return 0;
 }
 
+struct xy_pair {
+    double x;
+    double y;
+};
+
+static int compare_pairs_x (const void *a, const void *b)
+{
+    const struct xy_pair *pa = a;
+    const struct xy_pair *pb = b;
+    int ret;
+
+    ret = (pa->x > pb->x) - (pa->x < pb->x);
+    if (ret == 0) {
+	ret = (pa->y > pb->y) - (pa->y < pb->y);
+    }
+     
+    return ret;
+}
+
+static int compare_pairs_y (const void *a, const void *b)
+{
+    const struct xy_pair *pa = a;
+    const struct xy_pair *pb = b;
+
+    return (pa->y > pb->y) - (pa->y < pb->y);
+}
+
+static int real_kendall_tau (const double *x, const double *y, 
+			     int n, struct xy_pair *xy, int nn,
+			     double *ptau, double *pz)
+{
+    double tau, nn1, s2, z;
+    int tt1, tx = 0, ty = 0;
+    int Tx = 0, Ty = 0;
+    int Tx2 = 0, Ty2 = 0;
+    int Tx25 = 0, Ty25 = 0;
+    int tie, N0, N1, S;
+    int i, j;
+
+    /* populate sorter */
+    j = 0;
+    for (i=0; i<n; i++) {
+	if (!na(x[i]) && !na(y[i])) {
+	    xy[j].x = x[i];
+	    xy[j].y = y[i];
+	    j++;
+	}
+    }
+
+    /* sort pairs by x */
+    qsort(xy, nn, sizeof *xy, compare_pairs_x);
+
+    /* make order counts */
+    N0 = N1 = 0;
+    for (i=0; i<nn; i++) {
+	for (j=i+1; j<nn; j++) {
+	    if (xy[j].y == xy[i].y) {
+		Ty++;
+	    } else if (xy[j].x != xy[i].x) {
+		if (xy[j].y > xy[i].y) {
+		    N0++;
+		} else if (xy[j].y < xy[i].y) {
+		    N1++;
+		}
+	    } 
+	}
+	if (i > 0) {
+	    /* account for ties in x */
+	    tie = (xy[i].x == xy[i-1].x);
+	    if (tie) {
+		tx++;
+	    }
+	    if (tx > 0 && (!tie || i == nn - 1)) {
+		tx++;
+		tt1 = tx * (tx - 1);
+		Tx += tt1;
+		Tx2 += tt1 * (tx - 2);
+		Tx25 += tt1 * (2 * tx + 5);
+		tx = 0;
+	    }
+	}
+    }
+
+    if (Ty > 0) {
+	/* account for ties in y */
+	Ty = 0;
+	qsort(xy, nn, sizeof *xy, compare_pairs_y);
+	for (i=1; i<nn; i++) {
+	    tie = (xy[i].y == xy[i-1].y);
+	    if (tie) {
+		ty++;
+	    }
+	    if (ty > 0 && (!tie || i == nn - 1)) {
+		ty++;
+		tt1 = ty * (ty - 1);
+		Ty += tt1;
+		Ty2 += tt1 * (ty - 2);
+		Ty25 += tt1 * (2 * ty + 5);
+		ty = 0;	
+	    }	
+	}
+    }
+
+    S = N0 - N1;
+
+#if 0
+    fprintf(stderr, "N0 = %d, N1 = %d, S = %d\n", N0, N1, S);
+    fprintf(stderr, "Tx = %d, Ty = %d\n", Tx, Ty);
+#endif
+
+    nn1 = nn * (nn - 1.0);
+
+    /* normal approximation as in Shapiro and Chen,
+       Journal of Quality Technology, 2001 */
+
+    if (Tx == 0 && Ty == 0) {
+	tau = 2 * S / nn1;
+	s2 = (1.0/18) * nn1 * (2 * nn + 5);
+    } else {
+	double den = (nn1 - Tx) * (nn1 - Ty);
+
+	tau = 2 * S / sqrt(den);
+	s2 = (1.0/18) * (nn1 * (2 * nn + 5) - Tx25 - Ty25);
+	if (Tx2 != 0 && Ty2 != 0) {
+	    s2 += (1.0/(9*nn1*(nn-2))) * Tx2 * Ty2;
+	}
+	if (Tx != 0 && Ty != 0) {
+	    s2 += (1.0/(2*nn1)) * Tx * Ty; 
+	}  
+    }
+
+    z = (S - 1) / sqrt(s2);
+
+    if (ptau != NULL) {
+	*ptau = tau;
+    }    
+
+    if (pz != NULL) {
+	*pz = z;
+    }
+
+    return 0;
+}
+
+int kendall (const int *list, const double **Z, const DATAINFO *pdinfo, 
+	     gretlopt opt, PRN *prn)
+{
+    struct xy_pair *xy = NULL;
+    int T = pdinfo->t2 - pdinfo->t1 + 1;
+    const double *x, *y;
+    double tau, z;
+    int vx, vy;
+    int t, nn = 0;
+    int err = 0;
+
+    if (list[0] != 2) {
+	pputs(prn, _("This command requires two variables\n"));
+	return 1;
+    }
+
+    vx = list[1];
+    vy = list[2];
+
+    x = Z[vx] + pdinfo->t1;
+    y = Z[vy] + pdinfo->t1;
+
+    /* count valid pairs */
+    for (t=0; t<T; t++) {
+	if (!na(x[t]) && !na(y[t])) {
+	    nn++;
+	}
+    } 
+
+    if (nn < 2) {
+	return E_MISSDATA;
+    }
+
+    /* allocate */
+    xy = malloc(nn * sizeof *xy);
+    if (xy == NULL) {
+	return E_ALLOC;
+    }
+
+    /* calculate */
+    err = real_kendall_tau(x, y, T, xy, nn, &tau, &z);
+
+    if (!err) {
+	pprintf(prn, _("\nFor the variables '%s' and '%s'\n"), pdinfo->varname[vx],
+		pdinfo->varname[vy]);
+	pprintf(prn, "Kendall's tau = %.8f\n", tau);
+	pputs(prn, _("Under the null hypothesis of no correlation:\n "));
+	pprintf(prn, _("z-score = %g, with two-tailed p-value %.4f\n"), z,
+		normal_pvalue_2(z));
+	pputc(prn, '\n');
+    }
+
+    /* clean up */
+    free(xy);
+
+    return err;
+}
+
 #define LOCKE_DEBUG 0
 
 static int randomize_doubles (const void *a, const void *b)
@@ -313,57 +531,47 @@ static int randomize_doubles (const void *a, const void *b)
     return gretl_rand_int_max(8096) - 4097;
 }
 
-/* put sample into random order */
+/* check for negative values, screen out missing values,
+   and load x into an array for randomization */
 
-static double *locke_shuffle (const double *x, int *n, int code)
+static int locke_shuffle_init (const double *x,
+			       int *n, double **psx)
 {
-    static double *sx;
-    static int m;
-    int i;
-
-    if (code == RUN_CLEANUP) {
-	free(sx);
-	sx = NULL;
-	m = 0;
-	return NULL;
-    }	
-
-    if (code == RUN_FIRST) {
-	m = *n;
+    double *sx = NULL;
+    int i, m = 0;
 	
-	sx = malloc(m * sizeof *sx);
-	if (sx == NULL) {
-	    return NULL;
+    for (i=0; i<*n; i++) {
+	if (x[i] < 0.0) {
+	    return E_DATA;
 	}
-
-	/* check for missing and negative values as we go */
-
-	m = 0;
-	for (i=0; i<*n; i++) {
-	    if (na(x[i])) {
-		continue;
-	    } else if (x[i] < 0.0) {
-		m = 0;
-		break;
-	    } else {
-		sx[m++] = x[i];
-	    }
+	if (!na(x[i])) {
+	    m++;
 	}
+    }
 
-	if (m == 0) {
-	    free(sx);
-	    return NULL;
+    if (m < 4) {
+	return E_DATA;
+    }
+
+    sx = malloc(m * sizeof *sx);
+    if (sx == NULL) {
+	return E_ALLOC;
+    }
+
+    *psx = sx;
+
+    m = 0;
+    for (i=0; i<*n; i++) {
+	if (!na(x[i])) {
+	    sx[m++] = x[i];
 	}
+    }    
 
-	m = 2 * m / 2;
+    m = 2 * m / 2;
+    *n = m;
 
-	*n = m;
-    } 
-
-    qsort(sx, m, sizeof *sx, randomize_doubles);
-
-    return sx;
-}
+    return 0;
+} 
 
 #define NREPEAT 100
 
@@ -386,13 +594,15 @@ static double *locke_shuffle (const double *x, int *n, int code)
 
 double lockes_test (const double *x, int t1, int t2)
 {
-    double rho, sd, z;
-    double *sx, *u = NULL, *v = NULL;
-    int i, j, t, m = t2 - t1 + 1;
+    struct xy_pair *uv = NULL;
+    double *sx = NULL, *u = NULL, *v = NULL;
+    double zj, z;
+    int m = t2 - t1 + 1;
+    int i, j, t;
+    int err;
 
-    sx = locke_shuffle(x + t1, &m, RUN_FIRST);
-
-    if (sx == NULL) {
+    err = locke_shuffle_init(x + t1, &m, &sx);
+    if (err) {
 	return NADBL;
     }
 
@@ -400,11 +610,13 @@ double lockes_test (const double *x, int t1, int t2)
 	
     u = malloc(m * sizeof *u);
     v = malloc(m * sizeof *v);
+    uv = malloc(m * sizeof *uv);
 
-    if (u == NULL || v == NULL) {
+    if (u == NULL || v == NULL || uv == NULL) {
 	free(u);
 	free(v);
-	locke_shuffle(NULL, NULL, RUN_CLEANUP);
+	free(uv);
+	free(sx);
 	return NADBL;
     }
 
@@ -415,6 +627,8 @@ double lockes_test (const double *x, int t1, int t2)
     */
 
     for (j=0; j<NREPEAT; j++) {
+	qsort(sx, 2 * m, sizeof *sx, randomize_doubles);
+
 	t = 0;
 	for (i=0; i<m; i++) {
 	    u[i] = sx[t] + sx[t+1];
@@ -424,27 +638,24 @@ double lockes_test (const double *x, int t1, int t2)
 	    }
 	    t += 2;
 	}
-	spearman_rho(u, v, m, &rho, &sd, NULL, NULL, NULL, NULL,
-		     (j == 0)? RUN_FIRST : RUN_REPEAT);
-	z += rho / sd;
-#if LOCKE_DEBUG
-	printf("z[%d] = %g/%g = %g\n", j, rho, sd, rho / sd);
-#endif
-	locke_shuffle(NULL, NULL, RUN_REPEAT);
-    }   
 
-    free(u);
-    free(v);
+	real_kendall_tau(u, v, m, uv, m, NULL, &zj);
+	z += zj;
+#if LOCKE_DEBUG
+	printf("z[%d] = %g\n", j, zj);
+#endif
+    }   
 
     z /= (double) NREPEAT;
 
 #if LOCKE_DEBUG
-    fprintf(stderr, "Spearman's rho: z = %g\n", z);
+    fprintf(stderr, "Kendall's tau: average z = %g\n", z);
 #endif 
 
-    locke_shuffle(NULL, NULL, RUN_CLEANUP);
-    spearman_rho(NULL, NULL, 0, NULL, NULL, NULL, NULL, NULL, NULL,
-		 RUN_CLEANUP);
+    free(u);
+    free(v);
+    free(uv);
+    free(sx);
 
     return z;
 }
@@ -526,8 +737,8 @@ int runs_test (int varno, const double **Z, const DATAINFO *pdinfo,
     pprintf(prn, _("\nNumber of runs (R) in the variable '%s' = %d\n"), 
 	    pdinfo->varname[varno], runs);
     pprintf(prn, _("Under the null hypothesis of randomness, R "
-	    "follows N(%f, %f)\n"), mean, sd);
-    pprintf(prn, _("z-score = %f, with two-tailed p-value %f\n"), z, pval);
+	    "follows N(%g, %g)\n"), mean, sd);
+    pprintf(prn, _("z-score = %g, with two-tailed p-value %g\n"), z, pval);
 
     record_test_result(z, pval, "runs");
   
@@ -538,7 +749,7 @@ int runs_test (int varno, const double **Z, const DATAINFO *pdinfo,
 
 static void print_z_prob (double z, PRN *prn)
 {
-    double p;
+    double p = NADBL;
 
     if (z > 0) {
 	p = normal_pvalue_1(z);
@@ -551,6 +762,18 @@ static void print_z_prob (double z, PRN *prn)
 	    pprintf(prn, "  Prob(Z < %g) = %g\n", z, p);
 	}
     }
+
+    if (!na(p) && p > 0) {
+	pprintf(prn, "  Two-tailed p-value = %g\n", 2 * p);
+    }
+}
+
+static void diff_test_header (int v1, int v2, const DATAINFO *pdinfo,
+			      PRN *prn)
+{
+    pputc(prn, '\n');
+    pprintf(prn, _("Test for difference between %s and %s"),
+	    pdinfo->varname[v1], pdinfo->varname[v2]);
 }
 
 static const int rank5[3][2] = {
@@ -637,6 +860,7 @@ signed_rank_test (const double *x, const double *y,
 	}
     }
 
+    diff_test_header(v1, v2, pdinfo, prn);
     pprintf(prn, "\n%s\n", _("Wilcoxon Signed-Rank Test"));
     pprintf(prn, "%s: %s\n\n", _("Null hypothesis"),
 	    _("the median difference is zero"));
@@ -768,6 +992,7 @@ static int rank_sum_test (const double *x, const double *y,
 	}
     }
 
+    diff_test_header(v1, v2, pdinfo, prn);
     pprintf(prn, "\n%s\n", _("Wilcoxon Rank-Sum Test"));
     pprintf(prn, "%s: %s\n\n", _("Null hypothesis"),
 	    _("the two medians are equal"));
@@ -792,7 +1017,7 @@ static int rank_sum_test (const double *x, const double *y,
     }
 
     pprintf(prn, "  n1 = %d, n2 = %d\n", na, nb);
-    pprintf(prn, "  w = %g\n", wa);
+    pprintf(prn, "  w (%s) = %g\n", _("sum of ranks, sample 1"), wa);
 
     if (na >= 10 && nb >= 10) {
 	double m, s, z;
@@ -841,6 +1066,7 @@ static int sign_test (const double *x, const double *y,
 	return E_MISSDATA;
     }
 
+    diff_test_header(v1, v2, pdinfo, prn);
     pprintf(prn, "\n%s\n\n", _("Sign Test"));
     pprintf(prn, _("Number of differences: n = %d\n"), n);
     pputs(prn, "  ");
