@@ -1,20 +1,20 @@
-/*
- *  Copyright (c) by Allin Cottrell
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
- *
- *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- *
+/* 
+ *  gretl -- Gnu Regression, Econometrics and Time-series Library
+ *  Copyright (C) 2001 Allin Cottrell and Riccardo "Jack" Lucchetti
+ * 
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ * 
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ * 
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 #include "libgretl.h"
@@ -83,7 +83,7 @@ void gretl_object_ref (void *ptr, GretlObjType type)
 	    var->refcount += 1;
 	}
     } else if (type == GRETL_OBJ_SYS) {
-	gretl_equation_system *sys = (gretl_equation_system *) ptr;
+	equation_system *sys = (equation_system *) ptr;
 
 	if (sys != NULL) {
 	    sys->refcount += 1;
@@ -132,7 +132,7 @@ static void gretl_object_destroy (void *ptr, GretlObjType type)
     } else if (type == GRETL_OBJ_VAR) {
 	gretl_VAR_free(ptr);
     } else if (type == GRETL_OBJ_SYS) {
-	gretl_equation_system_destroy(ptr);
+	equation_system_destroy(ptr);
     }
 }
 
@@ -154,7 +154,7 @@ static int gretl_object_get_refcount (void *ptr, GretlObjType type)
 	    rc = var->refcount;
 	}
     } else if (type == GRETL_OBJ_SYS) {
-	gretl_equation_system *sys = (gretl_equation_system *) ptr;
+	equation_system *sys = (equation_system *) ptr;
 
 	if (sys != NULL) {
 	    rc = sys->refcount;
@@ -293,7 +293,7 @@ void gretl_object_unref (void *ptr, GretlObjType type)
 	    rc = var->refcount;
 	}
     } else if (type == GRETL_OBJ_SYS) {
-	gretl_equation_system *sys = (gretl_equation_system *) ptr;
+	equation_system *sys = (equation_system *) ptr;
 
 	if (sys != NULL) {
 	    sys->refcount -= 1;
@@ -379,7 +379,7 @@ const char *gretl_object_get_name (void *p, GretlObjType type)
 
 	return var->name;
     } else if (type == GRETL_OBJ_SYS) {
-	gretl_equation_system *sys = p;
+	equation_system *sys = p;
 
 	return sys->name;
     }
@@ -532,7 +532,7 @@ GRETL_VAR *get_VECM_by_name (const char *vname)
     }
 }
 
-gretl_equation_system *get_equation_system_by_name (const char *sname)
+equation_system *get_equation_system_by_name (const char *sname)
 {
     return get_object_by_name(sname, GRETL_OBJ_SYS, NULL);
 }
@@ -557,10 +557,10 @@ int gretl_object_compose_name (void *p, GretlObjType type)
 	}
 	gretl_VAR_set_name(var, name);
     } else if (type == GRETL_OBJ_SYS) {
-	gretl_equation_system *sys = (gretl_equation_system *) p;
+	equation_system *sys = (equation_system *) p;
 
 	sprintf(name, "%s %d", _("System"), ++n_sys);
-	gretl_system_set_name(sys, name);
+	equation_system_set_name(sys, name);
     } else {
 	err = 1;
     }
@@ -577,7 +577,7 @@ int gretl_object_rename (void *p, GretlObjType type, const char *oname)
     } else if (type == GRETL_OBJ_VAR) {
 	gretl_VAR_set_name((GRETL_VAR *) p, oname);
     } else if (type == GRETL_OBJ_SYS) {
-	gretl_system_set_name((gretl_equation_system *) p, oname);
+	equation_system_set_name((equation_system *) p, oname);
     } else {
 	err = 1;
     }
@@ -713,6 +713,10 @@ int maybe_stack_var (GRETL_VAR *var, const CMD *cmd)
     char vname[MAXSAVENAME];
     int ret = 0;
 
+    if (var == NULL) {
+	return 0;
+    }
+
     gretl_cmd_get_savename(vname);
 
 #if ODEBUG
@@ -810,9 +814,11 @@ static double real_get_obj_scalar (void *p, GretlObjType type, int idx)
 	    x = INVALID_STAT;
 	}
     } else if (type == GRETL_OBJ_SYS) {
-	gretl_equation_system *sys = (gretl_equation_system *) p;
+	equation_system *sys = (equation_system *) p;
 
-	if (idx == M_LNL) {
+	if (idx == M_T) {
+	    x = sys->n_obs;
+	} else if (idx == M_LNL) {
 	    x = sys->ll;
 	} else if (idx == M_ESS) {
 	    x = sys->ess;
@@ -820,7 +826,9 @@ static double real_get_obj_scalar (void *p, GretlObjType type, int idx)
     } else if (type == GRETL_OBJ_VAR) {
 	GRETL_VAR *var = (GRETL_VAR *) p;
 
-	if (idx == M_LNL) {
+	if (idx == M_T) {
+	    x = var->T;
+	} else if (idx == M_LNL) {
 	    x = var->ll;
 	} else if (idx == M_AIC) {
 	    x = var->AIC;
@@ -854,6 +862,10 @@ real_get_obj_series (void *p, GretlObjType type, int idx,
     return x;
 }
 
+/* find out what sort of object we're dealing with, and call
+   the appropriate function to get the requested matrix 
+*/
+
 static gretl_matrix *
 real_get_obj_matrix (void *p, GretlObjType type, int idx, int *err)
 {
@@ -869,9 +881,9 @@ real_get_obj_matrix (void *p, GretlObjType type, int idx, int *err)
 
 	M = gretl_model_get_matrix(pmod, idx, err);
     } else if (type == GRETL_OBJ_SYS) {
-	gretl_equation_system *sys = (gretl_equation_system *) p;
+	equation_system *sys = (equation_system *) p;
 
-	M = gretl_equation_system_get_matrix(sys, idx, err);
+	M = equation_system_get_matrix(sys, idx, err);
     } else if (type == GRETL_OBJ_VAR) {
 	GRETL_VAR *var = (GRETL_VAR *) p;
 
@@ -983,6 +995,12 @@ double *saved_object_get_series (const char *oname, int idx,
     return x;
 }
 
+/* starting point for getting a matrix from a saved model
+   of one kind or another.  We start by looking for the right
+   model: this is either a match for @oname, or if @oname
+   is NULL or blank, the last model estimated.
+*/
+
 gretl_matrix *
 saved_object_get_matrix (const char *oname, int idx, int *err)
 {
@@ -1133,7 +1151,7 @@ static void saved_object_free (stacker *s)
     } else if (s->type == GRETL_OBJ_VAR) {
 	gretl_VAR_free(s->ptr);
     } else if (s->type == GRETL_OBJ_SYS) {
-	gretl_equation_system_destroy(s->ptr);
+	equation_system_destroy(s->ptr);
     }
 }
 
@@ -1173,7 +1191,7 @@ int last_model_test_ok (int ci, gretlopt opt, const DATAINFO *pdinfo,
 
 	if (ci == RESTRICT && r > 0) {
 	    err = 0;
-	} else if (ci == TESTUHAT) {
+	} else if (ci == TESTUHAT || ci == FCASTERR) {
 	    err = 0;
 	} else if (ci == LMTEST && ((opt & OPT_A) || (opt & OPT_H))) {
 	    err = 0;
