@@ -1,20 +1,20 @@
-/*
- *  Copyright (c) 2004 by Allin Cottrell
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
- *
- *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
- *
+/* 
+ *  gretl -- Gnu Regression, Econometrics and Time-series Library
+ *  Copyright (C) 2001 Allin Cottrell and Riccardo "Jack" Lucchetti
+ * 
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ * 
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ * 
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 #include "libgretl.h"
@@ -25,6 +25,7 @@
                        c == ARBOND || \
                        c == ARMA || \
                        c == CORC || \
+                       c == HECKIT || \
                        c == GARCH || \
                        c == GMM || \
                        c == HCCM || \
@@ -99,8 +100,12 @@ struct gretl_option gretl_opts[] = {
     { COINT2,   OPT_R, "rc" },
     { COINT2,   OPT_T, "ct" },
     { COINT2,   OPT_V, "verbose" },
+    { CORC,     OPT_Q, "quiet" },
+    { CORR,     OPT_K, "kendall" },
+    { CORR,     OPT_S, "spearman" },
     { CORR,     OPT_U, "uniform" },
     { CUSUM,    OPT_R, "squares" },
+    { DELEET,   OPT_D, "db" },
     { DIFFTEST, OPT_G, "sign" },
     { DIFFTEST, OPT_R, "rank-sum" },
     { DIFFTEST, OPT_I, "signed-rank" },
@@ -121,6 +126,8 @@ struct gretl_option gretl_opts[] = {
     { FCASTERR, OPT_S, "static" },
     { FREQ,     OPT_O, "gamma" },
     { FREQ,     OPT_Q, "quiet" },
+    { FREQ,     OPT_S, "silent" },
+    { FREQ,     OPT_Z, "normal" },
     { GARCH,    OPT_A, "arma-init" },    
     { GARCH,    OPT_R, "robust" },
     { GARCH,    OPT_V, "verbose" },
@@ -137,7 +144,11 @@ struct gretl_option gretl_opts[] = {
     { GNUPLOT,  OPT_T, "time-series" },
     { GNUPLOT,  OPT_Z, "dummy" },
     { GRAPH,    OPT_O, "tall" },
+    { HECKIT,   OPT_M, "ml" },
+    { HECKIT,   OPT_T, "two-step" },
+    { HECKIT,   OPT_V, "verbose" },
     { HILU,     OPT_B, "no-corc" },
+    { HSK,      OPT_Q, "quiet" },
     { KPSS,     OPT_T, "trend" },
     { KPSS,     OPT_V, "verbose" },
     { KPSS,     OPT_Q, "quiet" },
@@ -150,6 +161,7 @@ struct gretl_option gretl_opts[] = {
     { LMTEST,   OPT_P, "panel" },
     { LMTEST,   OPT_Q, "quiet" },
     { LMTEST,   OPT_W, "white" },
+    { LOGIT,    OPT_P, "p-values" },
     { LOGIT,    OPT_R, "robust" },
     { LOGIT,    OPT_V, "verbose" },
     { LOOP,     OPT_P, "progressive" },
@@ -167,12 +179,13 @@ struct gretl_option gretl_opts[] = {
     { NLS,      OPT_O, "vcv" },
     { NLS,      OPT_R, "robust" },
     { NLS,      OPT_V, "verbose" },
+    { OLS,      OPT_F, "print-final" },
     { OLS,      OPT_N, "no-df-corr" },
     { OLS,      OPT_O, "vcv" }, 
-    { OLS,      OPT_P, "print-final" },
     { OLS,      OPT_R, "robust" },
     { OLS,      OPT_Q, "quiet" },
     { OLS,      OPT_S, "simple-print" },
+    { OMIT,     OPT_A, "automatic" },
     { OMIT,     OPT_B, "both" },
     { OMIT,     OPT_I, "silent" },
     { OMIT,     OPT_P, "bootstrap" },
@@ -184,6 +197,7 @@ struct gretl_option gretl_opts[] = {
     { OMITFROM, OPT_Q, "quiet" },
     { OMITFROM, OPT_T, "inst" },
     { OMITFROM, OPT_W, "wald" },
+    { OPEN,     OPT_C, "coded" },
     { OPEN,     OPT_B, "box1" },
     { OPEN,     OPT_O, "octave" },
     { OPEN,     OPT_W, "www" },
@@ -212,11 +226,17 @@ struct gretl_option gretl_opts[] = {
     { PRINT,    OPT_L, "long" },
     { PRINT,    OPT_N, "no-dates" },
     { PRINT,    OPT_T, "ten" },
+    { PROBIT,   OPT_P, "p-values" },
     { PROBIT,   OPT_R, "robust" },
     { PROBIT,   OPT_V, "verbose" },
     { QUIT,     OPT_X, "exit" },
     { RESTRICT, OPT_B, "bootstrap" },
+    { RESTRICT, OPT_F, "full" },
+    { RESTRICT, OPT_J, "jitter" },
     { RESTRICT, OPT_Q, "quiet" },
+    { RESTRICT, OPT_V, "verbose" },
+    { RESTRICT, OPT_L, "lbfgs" },
+    { RESTRICT, OPT_N, "no-scaling" },
     { SCATTERS, OPT_L, "with-lines" },
     { SETINFO,  OPT_C, "continuous" },
     { SETINFO,  OPT_D, "discrete" },
@@ -375,7 +395,7 @@ struct flag_match flag_matches[] = {
     { OPT_G, 'g' },
     { OPT_H, 'h' },
     { OPT_I, 'i' },
-    { OPT_J, 'i' },
+    { OPT_J, 'j' },
     { OPT_K, 'k' },
     { OPT_L, 'l' },
     { OPT_M, 'm' },
@@ -396,7 +416,7 @@ struct flag_match flag_matches[] = {
 
 static const char *ok_flags = "abcdefghijklmnopqrstuvwxz";
 
-#define isflag(c) (strchr(ok_flags, c) != NULL)
+#define isflag(c) (c && (strchr(ok_flags, c) != NULL))
 
 static gretlopt opt_from_flag (unsigned char c)
 {
@@ -430,33 +450,61 @@ static int opt_is_valid (gretlopt opt, int ci, char c)
     return 0;
 }
 
+/* See if at point "p" (at which we've found '-') in string "s" we
+   might be at the start of an option flag: the previous character
+   must be a space, and we must not be inside a quoted string.
+*/
+
+static int maybe_opt_start (char *s, char *p)
+{
+    int i, n = p - s;
+    int quoted = 0;
+
+    if (n > 0 && !isspace(*(p-1))) {
+	return 0;
+    }
+
+    for (i=0; i<n; i++) {
+	if (s[i] == '"' && (i == 0 || s[i-1] != '\\')) {
+	    quoted = !quoted;
+	}
+    }
+
+    return !quoted;
+}
+
 static gretlopt get_short_opts (char *line, int ci, int *err)
 {
-    char *p = strchr(line, '-');
+    char *s = line;
     gretlopt opt, ret = 0L;
 
-    while (p != NULL) {
-	unsigned char c, prev;
-	int match = 0;
-	size_t n = strlen(p);
+    while ((s = strchr(s, '-')) != NULL) {
+	char *p = s + 1;
+	int i, n = 0;
 
-	c = *(p + 1);
-	prev = *(p - 1);
-	
-	if (isspace(prev) && isflag(c) && (n == 2 || isspace(*(p + 2)))) {
-	    opt = opt_from_flag(c);
-	    if (!opt_is_valid(opt, ci, c)) {
-		*err = 1;
-		return 0L;
+	if (maybe_opt_start(line, s)) {
+	    n = strspn(p, ok_flags);
+	    if (n > 0) {
+		if (isspace(p[n]) || p[n] == '\0') {
+		    for (i=0; i<n; i++) {
+			opt = opt_from_flag(p[i]);
+			if (!opt_is_valid(opt, ci, p[i])) {
+			    *err = 1;
+			    return 0L;
+			}
+			ret |= opt;
+		    }
+		} else {
+		    n = 0;
+		}
 	    }
-	    ret |= opt;
-	    gretl_delete(p, 0, 2);
-	    match = 1;
 	}
-	if (!match) {
-	    p++;
+
+	if (n > 0) {
+	    gretl_delete(s, 0, n + 1);
+	} else {
+	    s++;
 	}
-	p = strchr(p, '-');
     }
 
     return ret;
@@ -515,28 +563,33 @@ static int valid_long_opt (int ci, const char *lopt)
   
 static gretlopt get_long_opts (char *line, int ci, int *err)
 {
-    char *p = strstr(line, "--");
+    char *s = line;
+    char longopt[32];
     gretlopt match, ret = 0L;
 
-    while (p != NULL) {
-	char longopt[32];
-
-	sscanf(p + 2, "%31s", longopt);
-	match = valid_long_opt(ci, longopt);
+    while ((s = strstr(s, "--")) != NULL) {
+	match = 0;
+	*longopt = '\0';
+	if (maybe_opt_start(line, s)) {
+	    sscanf(s + 2, "%31s", longopt);
+	    match = valid_long_opt(ci, longopt);
+	    if (match > 0) {
+		/* recognized an acceptable option flag */
+		ret |= match;
+	    } else if (is_long_opt(longopt)) {
+		/* recognized option, but not valid for the command */
+		sprintf(gretl_errmsg, "Invalid option '--%s'", longopt);
+		fprintf(stderr, " line='%s', ci = %d\n", line, ci);
+		*err = 1;
+		return 0L;
+	    } 
+	}
 
 	if (match > 0) {
-	    ret |= match;
-	    gretl_delete(p, 0, 2 + strlen(longopt));
-	} else if (is_long_opt(longopt)) {
-	    /* recognized option, but not valid for the command */
-	    sprintf(gretl_errmsg, "Invalid option '--%s'", longopt);
-	    fprintf(stderr, " line='%s', ci = %d\n", line, ci);
-	    *err = 1;
-	    return 0L;
+	    gretl_delete(s, 0, 2 + strlen(longopt));
 	} else {
-	    p += 2;
+	    s += 2;
 	}
-	p = strstr(p, "--");
     }
 
     return ret;
@@ -607,6 +660,8 @@ gretlopt get_gretl_options (char *line, int *err)
 	ci = MLE;
     } else if (strstr(line, "end gmm")) {
 	ci = GMM;
+    } else if (strstr(line, "end restrict")) {
+	ci = RESTRICT;
     } else {
 	ci = gretl_command_number(cmdword);
     }
@@ -617,6 +672,11 @@ gretlopt get_gretl_options (char *line, int *err)
 	return oflags;
     }
 
+    /* smpl: in some contexts options don't make sense */
+    if (ci == SMPL && strchr(line, ';')) {
+	return oflags;
+    }
+
     if (ci != SETINFO && ci != TABPRINT && ci != EQNPRINT) {
 	/* try for short-form options (e.g. "-o") */
 	opt = get_short_opts(line, ci, &myerr);
