@@ -1,20 +1,20 @@
-/*
- *  Copyright (c) by Allin Cottrell and Riccardo Lucchetti
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
- *
- *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- *
+/* 
+ *  gretl -- Gnu Regression, Econometrics and Time-series Library
+ *  Copyright (C) 2001 Allin Cottrell and Riccardo "Jack" Lucchetti
+ * 
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ * 
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ * 
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 #include "libgretl.h"
@@ -45,24 +45,7 @@ GPT_SPEC *plotspec_new (void)
 	return NULL;
     }
 
-    spec->lines = malloc(MAX_PLOT_LINES * sizeof *spec->lines);
-
-    if (spec->lines == NULL) {
-	free(spec);
-	return NULL;
-    }
-
-    for (i=0; i<MAX_PLOT_LINES; i++) {
-	spec->lines[i].varnum = 0;
-	spec->lines[i].title[0] = 0;
-	spec->lines[i].formula[0] = 0;
-	spec->lines[i].style[0] = 0;
-	spec->lines[i].scale[0] = 0;
-	spec->lines[i].yaxis = 1;
-	spec->lines[i].type = 0;
-	spec->lines[i].width = 1;
-	spec->lines[i].ncols = 0;
-    }
+    spec->lines = NULL;
 
     for (i=0; i<4; i++) {
 	spec->titles[i][0] = 0;
@@ -82,9 +65,8 @@ GPT_SPEC *plotspec_new (void)
     spec->mxtics[0] = 0;
     spec->fname[0] = 0;
     strcpy(spec->keyspec, "left top");
-    spec->xzeroaxis = 0;
 
-    for (i=0; i<3; i++) {
+    for (i=0; i<4; i++) {
 	spec->range[i][0] = NADBL;
 	spec->range[i][1] = NADBL;
     }
@@ -106,7 +88,9 @@ GPT_SPEC *plotspec_new (void)
     spec->n_lines = 0;
     spec->nobs = 0;
     spec->okobs = 0;
+    spec->pd = 0;
     spec->boxwidth = 0;
+    spec->samples = 0;
 
     spec->termtype[0] = 0;
 
@@ -214,18 +198,27 @@ static char *escape_quotes (const char *s)
 static void 
 gp_string (FILE *fp, const char *fmt, const char *s, int png)
 {
-#ifdef ENABLE_NLS  
-    if (png && iso_latin_version() == 2) {
-	char htmlstr[128];
+    int done = 0;
 
-	sprint_l2_to_html(htmlstr, s, sizeof htmlstr);
-	fprintf(fp, fmt, htmlstr);
-    } else {
-	fprintf(fp, fmt, s); 
-    }
-#else
-    fprintf(fp, fmt, s);
+#ifdef ENABLE_NLS  
+    static int pngterm = -1;
+
+    if (png && iso_latin_version() == 2) {
+	if (pngterm < 0) {
+	    pngterm = gnuplot_png_terminal();
+	}
+	if (pngterm != GP_PNG_CAIRO) {
+	    char htmlstr[128];
+
+	    sprint_l2_to_html(htmlstr, s, sizeof htmlstr);
+	    fprintf(fp, fmt, htmlstr);
+	}
+    } 
 #endif
+
+    if (!done) {
+	fprintf(fp, fmt, s);
+    }
 }
 
 const char *gp_justification_string (int j)
@@ -353,6 +346,52 @@ void print_auto_fit_string (FitType fit, FILE *fp)
     }
 }
 
+void print_plot_ranges_etc (const GPT_SPEC *spec, FILE *fp)
+{
+    const char *rstrs[] = {
+	"x", "y", "y2", "t"
+    };
+    int i;
+
+    gretl_push_c_numeric_locale();
+
+    for (i=0; i<4; i++) {
+	if (na(spec->range[i][0]) || na(spec->range[i][1]) ||
+	    spec->range[i][0] == spec->range[i][1]) {
+	    continue;
+	}
+	if ((i == 2 && !(spec->flags & GPT_Y2AXIS)) ||
+	    (i == 3 && !(spec->flags & GPT_PARAMETRIC))) {
+	    continue;
+	}
+	fprintf(fp, "set %srange [%.7g:%.7g]\n", rstrs[i], 
+		spec->range[i][0], spec->range[i][1]);
+	if (i == 4 && spec->code == PLOT_PROB_DIST && spec->samples == 0) {
+	    int ns = spec->range[i][1] - spec->range[i][0] + 1;
+
+	    fprintf(fp, "set samples %d\n", ns);
+	}
+    }
+
+    if (spec->boxwidth > 0 && spec->boxwidth < 1) {
+	fprintf(fp, "set boxwidth %.3f\n", (double) spec->boxwidth);
+    }
+
+    gretl_pop_c_numeric_locale();
+}
+
+static int get_mono_output (const GPT_SPEC *spec)
+{
+    if (strstr(spec->termtype, " mono")) {
+	return 1;
+    } else if (strstr(spec->termtype, "postscr") && 
+	       !strstr(spec->termtype, "color")) {
+	return 1;
+    } else {
+	return 0;
+    }
+}
+
 #define show_fit(s) (s->fit == PLOT_FIT_OLS || \
                      s->fit == PLOT_FIT_QUADRATIC || \
                      s->fit == PLOT_FIT_INVERSE || \
@@ -362,6 +401,7 @@ int plotspec_print (const GPT_SPEC *spec, FILE *fp)
 {
     int i, k, t;
     int png = get_png_output(spec);
+    int mono = get_mono_output(spec);
     int started_data_lines = 0;
     int n_lines = spec->n_lines;
     double *x[4];
@@ -372,6 +412,19 @@ int plotspec_print (const GPT_SPEC *spec, FILE *fp)
 	n_lines--;
     }
 
+    if (spec->pd > 0) {
+	fprintf(fp, "# timeseries %d", spec->pd);
+	if (spec->flags & GPT_LETTERBOX) {
+	    fputs(" (letterbox)\n", fp);
+	} else {
+	    fputc('\n', fp);
+	}
+    }
+
+    if (!mono && gnuplot_has_rgb()) {
+	write_plot_line_styles(spec->code, fp);
+    }
+
     if (!string_is_blank(spec->titles[0])) {
 	gp_string(fp, "set title \"%s\"\n", spec->titles[0], png);
     }
@@ -394,11 +447,22 @@ int plotspec_print (const GPT_SPEC *spec, FILE *fp)
 	}
     }
 
-    if (spec->xzeroaxis) {
+    if (spec->flags & GPT_XZEROAXIS) {
 	fputs("set xzeroaxis\n", fp);
     }
 
-    fputs("set missing \"?\"\n", fp);
+    if (spec->flags & GPT_YZEROAXIS) {
+	fputs("set yzeroaxis\n", fp);
+    }    
+
+    if (spec->flags & GPT_PARAMETRIC) {
+	fputs("set parametric\n", fp);
+	if (spec->samples > 0) {
+	    fprintf(fp, "set samples %d\n", spec->samples);
+	}
+    }
+
+    gnuplot_missval_string(fp);
 
     if (strcmp(spec->keyspec, "none") == 0) {
 	fputs("set nokey\n", fp);
@@ -406,25 +470,7 @@ int plotspec_print (const GPT_SPEC *spec, FILE *fp)
 	fprintf(fp, "set key %s\n", spec->keyspec);
     }
 
-    k = (spec->flags & GPT_Y2AXIS)? 3 : 2;
-
-    gretl_push_c_numeric_locale();
-
-    for (i=0; i<k; i++) {
-	if (na(spec->range[i][0]) || na(spec->range[i][1]) ||
-	    spec->range[i][0] == spec->range[i][1]) {
-	    continue;
-	}
-	fprintf(fp, "set %srange [%.7g:%.7g]\n",
-		(i == 0)? "x" : (i == 1)? "y" : "y2",
-		spec->range[i][0], spec->range[i][1]);
-    }
-
-    if (spec->boxwidth > 0 && spec->boxwidth < 1) {
-	fprintf(fp, "set boxwidth %.3f\n", (double) spec->boxwidth);
-    }
-
-    gretl_pop_c_numeric_locale();
+    print_plot_ranges_etc(spec, fp);
 
     /* customized xtics? */
     if (!string_is_blank(spec->xtics)) {
@@ -472,7 +518,7 @@ int plotspec_print (const GPT_SPEC *spec, FILE *fp)
     if ((spec->code == PLOT_FREQ_SIMPLE ||
 	 spec->code == PLOT_FREQ_NORMAL ||
 	 spec->code == PLOT_FREQ_GAMMA) && gnuplot_has_style_fill()) {
-	if (strstr(spec->termtype, " mono")) {
+	if (mono) {
 	    fputs("set style fill solid 0.3\n", fp);
 	} else {
 	    fputs("set style fill solid 0.6\n", fp);
@@ -526,6 +572,7 @@ int plotspec_print (const GPT_SPEC *spec, FILE *fp)
 	}
 
 	fprintf(fp, "w %s", spec->lines[i].style);
+
 	if (spec->lines[i].type != 0) {
 	    fprintf(fp, " lt %d", spec->lines[i].type);
 	}
@@ -593,6 +640,10 @@ int plotspec_print (const GPT_SPEC *spec, FILE *fp)
 
     gretl_pop_c_numeric_locale();
 
+    if (png && gnuplot_png_terminal() == GP_PNG_CAIRO) {
+	print_plot_bounding_box_request(fp);
+    }
+
     return miss;
 }
 
