@@ -1,20 +1,20 @@
-/*
- *  Copyright (c) by Ramu Ramanathan and Allin Cottrell
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
- *
- *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
- *
+/* 
+ *  gretl -- Gnu Regression, Econometrics and Time-series Library
+ *  Copyright (C) 2001 Allin Cottrell and Riccardo "Jack" Lucchetti
+ * 
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ * 
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ * 
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 #include "libgretl.h"
@@ -30,6 +30,7 @@
 enum {
     P_EXCEL_IMPORT = 1,
     P_GNUMERIC_IMPORT,
+    P_ODS_IMPORT,
     P_JOHANSEN,
     P_LAD,
     P_VIF,
@@ -57,7 +58,8 @@ enum {
     P_JMULTI_IMPORT,
     P_ZIPFILE,
     P_OPROBIT,
-    P_ARBOND
+    P_ARBOND,
+    P_HECKIT
 } plugin_codes;
 
 struct plugin_info {
@@ -74,6 +76,7 @@ struct plugin_info plugins[] = {
     { 0,                 NULL },
     { P_EXCEL_IMPORT,    "excel_import" },
     { P_GNUMERIC_IMPORT, "gnumeric_import" },
+    { P_ODS_IMPORT,      "ods_import" },
     { P_JOHANSEN,        "johansen" },
     { P_LAD,             "lad" },
     { P_VIF,             "vif" },
@@ -101,7 +104,8 @@ struct plugin_info plugins[] = {
     { P_JMULTI_IMPORT,   "jmulti_import" },
     { P_ZIPFILE,         "gretlzip" },
     { P_OPROBIT,         "oprobit" },
-    { P_ARBOND,          "arbond" }
+    { P_ARBOND,          "arbond" },
+    { P_HECKIT,          "heckit" }
 };  
 
 struct plugin_function plugin_functions[] = { 
@@ -110,14 +114,18 @@ struct plugin_function plugin_functions[] = {
     { "cli_get_xls",      P_EXCEL_IMPORT },
     { "wbook_get_data",   P_GNUMERIC_IMPORT },
     { "cli_get_gnumeric", P_GNUMERIC_IMPORT },
+    { "ods_get_data",     P_ODS_IMPORT },
+    { "cli_get_ods",      P_ODS_IMPORT },
+
     { "wf1_get_data",     P_EVIEWS_IMPORT },
     { "dta_get_data",     P_STATA_IMPORT },
     { "jmulti_get_data",  P_JMULTI_IMPORT },
 
     /* Johansen cointegration test and VECM */
-    { "johansen_analysis",        P_JOHANSEN },
-    { "johansen_bootstrap_round", P_JOHANSEN },
-    { "vecm_beta_test",           P_JOHANSEN },
+    { "johansen_coint_test",   P_JOHANSEN },
+    { "johansen_estimate",     P_JOHANSEN },
+    { "johansen_boot_round",   P_JOHANSEN },
+    { "vecm_test_restriction", P_JOHANSEN },
 
     /* least absolute deviations */
     { "lad_driver", P_LAD },
@@ -165,6 +173,7 @@ struct plugin_function plugin_functions[] = {
     { "garch_model",       P_GARCH },
     { "poisson_estimate",  P_POISSON },
     { "ordered_estimate",  P_OPROBIT },
+    { "heckit_estimate",   P_HECKIT },
 
     /* audio graphs etc */
     { "midi_play_graph",   P_AUDIO },
