@@ -1,20 +1,20 @@
-/*
- *  Copyright (c) by Ramu Ramanathan and Allin Cottrell
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
- *
- *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111, USA.
- *
+/* 
+ *  gretl -- Gnu Regression, Econometrics and Time-series Library
+ *  Copyright (C) 2001 Allin Cottrell and Riccardo "Jack" Lucchetti
+ * 
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ * 
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ * 
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 /*  printout.c - simple text print routines for some gretl structs */ 
@@ -297,52 +297,111 @@ void print_freq (const FreqDist *freq, PRN *prn)
 	return;
     }
 
-    if (!freq->discrete) {
+    if (freq->discrete) {
+	pputs(prn, _("\n          frequency    rel.     cum.\n\n"));
+
+	for (k=0; k<=K; k++) {
+	    sprintf(word, "%4g", freq->midpt[k]);
+	    pputs(prn, word);
+	    nlw = 10 - strlen(word);
+	    bufspace(nlw, prn);
+
+	    pprintf(prn, "%6d  ", freq->f[k]);
+	    f = 100.0 * freq->f[k] / valid;
+	    cumf += f;
+	    pprintf(prn, "  %6.2f%% %7.2f%% ", f, cumf);
+	    i = 0.36 * f;
+	    while (i--) {
+		pputc(prn, '*');
+	    }
+	    pputc(prn, '\n');
+	}
+    } else {
+	int digits = 5;
+	int someneg = 0, somemneg = 0;
+	int len, xlen, mxlen;
+	double x;
+
 	pprintf(prn, _("number of bins = %d, mean = %g, sd = %g\n"), 
 		freq->numbins, freq->xbar, freq->sdx);
 	pputs(prn, 
 	      _("\n       interval          midpt   frequency    rel.     cum.\n\n"));
-    } else {
-	pputs(prn, _("\n          frequency    rel.     cum.\n\n"));
-    }
 
-    for (k=0; k<=K; k++) {
-	*word = '\0';
-	if (freq->discrete) {
-	    sprintf(word, "%4g", freq->midpt[k]);
-	} else {
+    tryagain:
+
+	xlen = mxlen = 0;
+
+	for (k=0; k<=K; k++) {
+	    x = freq->endpt[k];
+	    if (x < 0) {
+		someneg = 1;
+	    }
+	    sprintf(word, "%#.*g", digits, x);
+	    len = strlen(word);
+	    if (len > xlen) {
+		xlen = len;
+	    }
+	    x = freq->midpt[k];
+	    if (x < 0) {
+		somemneg = 1;
+	    }
+	    sprintf(word, "%#.*g", digits, x);
+	    len = strlen(word);
+	    if (len > mxlen) {
+		mxlen = len;
+	    }
+	}
+
+	if (xlen > 10 && digits == 5) {
+	    digits--;
+	    goto tryagain;
+	}
+
+	xlen++;
+	xlen = (xlen > 10)? xlen : 10;
+
+	mxlen++;
+	mxlen = (mxlen > 10)? mxlen : 10;
+	
+	for (k=0; k<=K; k++) {
+	    *word = '\0';
 	    if (k == 0) {
-		pputs(prn, "          <  ");
+		pprintf(prn, "%*s", xlen + 3, " < ");
 	    } else if (k == K) {
-		pputs(prn, "          >= ");
+		pprintf(prn, "%*s", xlen + 3, ">= ");
 	    } else {
-		pprintf(prn, "%#10.5g - ", freq->endpt[k]);
-	    }
-	    if (k == K) {
-		sprintf(word, "%#.5g", freq->endpt[k]);
-	    } else {
-		sprintf(word, "%#.5g", freq->endpt[k+1]);
+		sprintf(word, "%#.*g", digits, freq->endpt[k]);
+		pprintf(prn, "%*s", xlen, word);
+		pputs(prn, " - ");
 	    }
 
-	    pprintf(prn, "%s", word);
-	    nlw = 10 - strlen(word);
-	    bufspace(nlw, prn);
-	    sprintf(word, " %#.5g", freq->midpt[k]);
-	}
+	    x = (k == K)? freq->endpt[k] : freq->endpt[k+1];
+	    if (x > 0 && someneg) {
+		sprintf(word, " %#.*g", digits, x);
+	    } else {
+		sprintf(word, "%#.*g", digits, x);
+	    }
+	    pprintf(prn, "%-*s", xlen, word);
 
-	pputs(prn, word);
-	nlw = 10 - strlen(word);
-	bufspace(nlw, prn);
+	    x = freq->midpt[k];
+	    if (x > 0 && somemneg) {
+		sprintf(word, " %#.*g", digits, x);
+	    } else {
+		sprintf(word, "%#.*g", digits, x);
+	    }
+	    pprintf(prn, "%-*s", mxlen, word);
 
-	pprintf(prn, "%6d  ", freq->f[k]);
-	f = 100.0 * freq->f[k] / valid;
-	cumf += f;
-	pprintf(prn, "  %6.2f%% %7.2f%% ", f, cumf);
-	i = 0.36 * f;
-	while (i--) {
-	    pputc(prn, '*');
+	    pprintf(prn, "%6d  ", freq->f[k]);
+
+	    f = 100.0 * freq->f[k] / valid;
+	    cumf += f;
+	    pprintf(prn, "  %6.2f%% %7.2f%% ", f, cumf);
+	    i = 0.36 * f;
+	    while (i--) {
+		pputc(prn, '*');
+	    }
+	    pputc(prn, '\n');
 	}
-	pputc(prn, '\n');
     }
 
     missing = total - valid;
@@ -354,6 +413,8 @@ void print_freq (const FreqDist *freq, PRN *prn)
 
     if (!na(freq->test)) {
 	print_freq_test(freq, prn);
+    } else {
+	pputc(prn, '\n');
     }
 }
 
@@ -467,6 +528,10 @@ void print_xtab (const Xtab *tab, gretlopt opt, PRN *prn)
 		  "values of 5 or greater.\n");
 	}
     }
+
+    if (r == 2 && c == 2) {
+	fishers_exact_test(tab, prn);
+    }
 }
 
 /**
@@ -512,6 +577,32 @@ void print_smpl (const DATAINFO *pdinfo, int fulln, PRN *prn)
     pputc(prn, '\n');
 }
 
+static void print_var_smpl (int v, const double **Z, 
+			    const DATAINFO *pdinfo, 
+			    PRN *prn)
+{
+    int t, n = 0;
+
+    if (pdinfo->t1 > 0 || pdinfo->t2 < pdinfo->n - 1) {
+	char d1[OBSLEN], d2[OBSLEN];
+	ntodate_full(d1, pdinfo->t1, pdinfo);
+	ntodate_full(d2, pdinfo->t2, pdinfo);
+
+	pprintf(prn, "%s:  %s - %s", _("Current sample"), d1, d2);
+    } else {
+	pprintf(prn, "%s: %s - %s", _("Full data range"), 
+		pdinfo->stobs, pdinfo->endobs);
+    }
+
+    for (t=pdinfo->t1; t<=pdinfo->t2; t++) {
+	if (!na(Z[v][t])) {
+	    n++;
+	}
+    }
+
+    pprintf(prn, " (n = %d)\n", n);
+}
+
 /**
  * gretl_fix_exponent:
  * @s: string representation of floating-point number.
@@ -1040,7 +1131,7 @@ static void printz (const double *z, const DATAINFO *pdinfo,
     double xx;
 
     if (opt & OPT_L) {
-	dig = get_long_digits();
+	dig = libset_get_int(LONGDIGITS);
 	opt = OPT_T;
     }
 
@@ -1506,7 +1597,7 @@ static void print_scalar (double x, const char *vname,
 	    pputc(prn, ' ');
 	}
 	if (opt & OPT_L) {
-	    pprintf(prn, "%#.*E", get_long_digits(), x);
+	    pprintf(prn, "%#.*E", libset_get_int(LONGDIGITS), x);
 	} else if (opt & OPT_T) {
 	    pprintf(prn, "%#.10E", x);
 	} else {
@@ -1649,7 +1740,7 @@ int printdata (const int *list, const char *mstr,
 	    if (plist[0] > 1) {
 		pprintf(prn, _("Varname: %s\n"), pdinfo->varname[plist[j]]);
 	    }
-	    print_smpl(pdinfo, 0, prn);
+	    print_var_smpl(plist[j], Z, pdinfo, prn);
 	    pputc(prn, '\n');
 	    printz(Z[plist[j]], pdinfo, prn, opt);
 	    pputc(prn, '\n');
@@ -1877,7 +1968,7 @@ text_print_fit_resid (const FITRESID *fr, const DATAINFO *pdinfo, PRN *prn)
 {
     int onestep = fr->method == FC_ONESTEP;
     int t, anyast = 0;
-    double yt, yf;
+    double yt, yf, et;
     double MSE = 0.0;
     double AE = 0.0;
     int effn = 0;
@@ -1892,6 +1983,7 @@ text_print_fit_resid (const FITRESID *fr, const DATAINFO *pdinfo, PRN *prn)
 
 	yt = fr->actual[t];
 	yf = fr->fitted[t];
+	et = fr->resid[t];
 
 	if (na(yt)) {
 	    pputc(prn, '\n');
@@ -1901,8 +1993,13 @@ text_print_fit_resid (const FITRESID *fr, const DATAINFO *pdinfo, PRN *prn)
 	    } else {
 		pprintf(prn, "%13g\n", yt);
 	    }
+	} else if (na(et)) {
+	    if (fr->pmax != PMAX_NOT_AVAILABLE) {
+		pprintf(prn, "%13.*f%13.*f\n", fr->pmax, yt, yf);
+	    } else {
+		pprintf(prn, "%13g%13g\n", yt, yf);
+	    }
 	} else {
-	    double et = yt - yf;
 	    int ast = 0;
 
 	    if (onestep) {
@@ -1991,9 +2088,11 @@ int text_print_forecast (const FITRESID *fr,
 	}
     }
 
-    pputc(prn, '\n');
+    if (!(opt & OPT_Q)) {
+	pputc(prn, '\n');
+    }
 
-    if (do_errs) {
+    if (do_errs && !(opt & OPT_Q)) {
 	if (fr->model_ci == ARMA || fr->model_ci == VECM) {
 	    pprintf(prn, _(" For 95%% confidence intervals, z(.025) = %.2f\n"), 
 		    1.96);
@@ -2105,14 +2204,14 @@ int print_fit_resid (const MODEL *pmod, const double **Z,
     return 0;
 }
 
-static void print_iter_val (double x, int i, PRN *prn)
+static void print_iter_val (double x, int i, int k, PRN *prn)
 {
     if (na(x)) {
 	pprintf(prn, "%-12s", "NA");
     } else {
 	pprintf(prn, "%#12.5g", x);
     }
-    if (i && i % 5 == 0) {
+    if (i && i % 6 == 5 && i < k-1) {
 	pprintf(prn, "\n%12s", " ");
     }
 }
@@ -2145,33 +2244,44 @@ print_iter_info (int iter, double crit, int type, int k,
     const char *cstr = cstrs[type];
     int i;
 
-    if (na(crit)) {
-	pprintf(prn, "%s %d: %s = NA", _("Iteration"), iter, _(cstr));
-    } else {
-	if (type == C_GMM) {
-	    crit = -crit;
-	}
-	pprintf(prn, "%s %d: %s = %#.12g", _("Iteration"), iter, 
-		_(cstr), crit);
+    if (type == C_GMM) {
+	crit = -crit;
     }
 
-    if (sl > 0.0) {
-	pprintf(prn, _(" (steplength = %.8g)"), sl);
+    if (iter < 0) {
+	pputs(prn, _("\n--- FINAL VALUES: \n"));
+    } else {
+	pprintf(prn, "%s %d: ", _("Iteration"), iter);
+    }
+
+    if (na(crit) || na(-crit)) {
+	pprintf(prn, "%s = NA", _(cstr));
+    } else {
+	pprintf(prn, "%s = %#.12g", _(cstr), crit);
+    }
+
+    if (sl > 0.0 && !na(sl)) {
+	pprintf(prn, _(" (steplength = %g)"), sl);
     }	
 
     pputc(prn, '\n');
 	
     pputs(prn, _("Parameters: "));
     for (i=0; i<k; i++) {
-	print_iter_val(b[i], i, prn);
+	print_iter_val(b[i], i, k, prn);
     }
     pputc(prn, '\n');
 
     pputs(prn, _("Gradients:  "));
     for (i=0; i<k; i++) {
-	print_iter_val(g[i], i, prn);
+	print_iter_val(g[i], i, k, prn);
     }
     pputs(prn, "\n\n");
+
+    if (iter < 0 || (iter % 20 == 0)) {
+	/* experimental */
+	iter_print_callback((iter < 0)? 0 : iter, prn);
+    }
 }
 
 /* apparatus for user-defined printf statements */
@@ -2213,7 +2323,7 @@ static char *printf_get_string (const char *s, const double **Z,
 				int t, int *err)
 {
     char *ret = NULL;
-    char tstr[OBSLEN], darg[16];
+    char tmpstr[16], strarg[16];
     const char *p = NULL, *q = NULL;
     int v, offset = 0;
     int len = 0;
@@ -2251,20 +2361,31 @@ static char *printf_get_string (const char *s, const double **Z,
 	    len = strlen(p);
 	    q = strchr(s, ')') + 1;
 	}
-    } else if (sscanf(s, "date(%15[^)])", darg)) {
+    } else if (sscanf(s, "argname(%15[^)])", strarg)) {
+	/* name of function argument */
+	char *aname = gretl_func_get_arg_name(strarg);
+
+	if (aname != NULL) {
+	    strcpy(tmpstr, aname);
+	    p = tmpstr;
+	    len = strlen(p);
+	    q = strchr(s, ')') + 1;
+	    free(aname);
+	}
+    } else if (sscanf(s, "date(%15[^)])", strarg)) {
 	/* date string */
 	t = -1;
-	if (isdigit(*darg)) {
-	    t = atoi(darg);
+	if (isdigit(*strarg)) {
+	    t = atoi(strarg);
 	} else {
-	    v = varindex(pdinfo, darg);
+	    v = varindex(pdinfo, strarg);
 	    if (v < pdinfo->v) {
 		t = Z[v][0];
 	    }
 	}
 	if (t > 0 && t <= pdinfo->n) {
-	    ntodate(tstr, t - 1, pdinfo);
-	    p = tstr;
+	    ntodate(tmpstr, t - 1, pdinfo);
+	    p = tmpstr;
 	    len = strlen(p);
 	    q = strchr(s, ')') + 1;
 	}
