@@ -1,20 +1,20 @@
-/*
- *  Copyright (c) by Ramu Ramanathan and Allin Cottrell
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
- *
- *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- *
+/* 
+ *  gretl -- Gnu Regression, Econometrics and Time-series Library
+ *  Copyright (C) 2001 Allin Cottrell and Riccardo "Jack" Lucchetti
+ * 
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ * 
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ * 
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 /*  pvalues.c - routines relating to computation of pvalues
@@ -76,6 +76,47 @@ double binomial_cdf_comp (int k, int n, double p)
     return x;
 }
 
+/*  
+    gives the event probability p such that the sum of the terms 0
+    through k of the Binomial probability density, for n trials, is
+    equal to the given cumulative probability y.
+*/
+
+static double binomial_cdf_inverse (int k, int n, double y)
+{
+    double p = NADBL;
+
+    if (y >= 0 && n >= 0 && k >= 0) {
+	p = bdtri(k, n, y);
+	if (get_cephes_errno()) {
+	    p = NADBL;
+	}
+    }
+
+    return p;
+}
+
+/* The following is no doubt horribly inefficient */
+
+static double binomial_critval (double a, double p, int n)
+{
+    double pk, ac = 1 - a;
+    int k;
+
+    if (n <= 0 || p <= 0 || p >= 1 || a <= 0 || a >= 1) {
+	return NADBL;
+    }
+
+    for (k=n; k>0; k--) {
+	pk = binomial_cdf(k, n, p);
+	if (pk < ac) {
+	    break;
+	}
+    }
+
+    return (double) (k + 1);
+}
+
 /**
  * x_factorial:
  * @x: input value.
@@ -199,7 +240,7 @@ double rhocrit95 (int n)
  * standard normal distribution.
  *
  * Returns: 2 times (1 minus the value of the standard normal
- * CDF evaluated at abs(@x)).
+ * CDF evaluated at abs(@x)), or 0 on underflow.
  */
 
 double normal_pvalue_2 (double x)
@@ -223,20 +264,28 @@ double normal_pvalue_2 (double x)
 
 double normal_pvalue_1 (double x)
 {
-    return 1 - ndtr(x);
+    double p = ndtr(x);
+
+    if (get_cephes_errno()) {
+	p = NADBL;
+    } else {
+	p = 1 - p;
+    }
+
+    return p;
 }
 
 /**
- * t_cdf:
+ * student_cdf:
  * @x: the cutoff point in the distribution.
  * @df: degrees of freedom.
  * 
  * Returns: the integral from minus infinity to @x of
- * the t distribution with @df degrees of freedom, or
+ * the Student's t distribution with @df degrees of freedom, or
  * #NADBL on failure.
  */
 
-double t_cdf (double x, int df)
+double student_cdf (double x, int df)
 {
     double p = NADBL;
 
@@ -251,7 +300,7 @@ double t_cdf (double x, int df)
 }
 
 /**
- * t_cdf_comp:
+ * student_cdf_comp:
  * @x: the cutoff point in the distribution.
  * @df: degrees of freedom.
  * 
@@ -260,24 +309,52 @@ double t_cdf (double x, int df)
  * #NADBL on failure.
  */
 
-double t_cdf_comp (double x, int df)
+double student_cdf_comp (double x, int df)
 {
     double p = NADBL;
 
     if (df > 0) {
-	p = stdtr(df, x);
-	if (get_cephes_errno()) {
-	    p = NADBL;
+	if (x > 0) {
+	    p = stdtr(df, -x);
+	    if (get_cephes_errno()) {
+		p = NADBL;
+	    }
 	} else {
-	    p = 1 - p;
+	    p = stdtr(df, x);
+	    if (get_cephes_errno()) {
+		p = NADBL;
+	    } else {
+		p = 1 - p;
+	    }
 	}
     }
 
     return p;
 }
 
+static double normal_cdf_comp (double x)
+{
+    double p;
+
+    if (x > 0) {
+	p = ndtr(-x);
+	if (get_cephes_errno()) {
+	    p = NADBL;
+	}	
+    } else {
+	p = ndtr(x);
+	if (get_cephes_errno()) {
+	    p = NADBL;
+	} else {
+	    p = 1 - p;
+	}
+    } 
+
+    return p;
+}
+
 /**
- * t_pvalue_2:
+ * student_pvalue_2:
  * @x: the cutoff point in the distribution.
  * @df: degrees of freedom.
  * 
@@ -286,7 +363,7 @@ double t_cdf_comp (double x, int df)
  * #NADBL on failure.
  */
 
-double t_pvalue_2 (double x, int df)
+double student_pvalue_2 (double x, int df)
 {
     double p = NADBL;
 
@@ -306,8 +383,10 @@ double t_pvalue_2 (double x, int df)
     return p;
 }
 
+#define df_ok(d) (floor(d) == d && d < (double) INT_MAX)
+
 /**
- * t_critval:
+ * student_critval:
  * @a: right-tail probability.
  * @df: degrees of freedom.
  *
@@ -316,9 +395,48 @@ double t_pvalue_2 (double x, int df)
  * probability @a, or #NADBL on failure.
  */
 
-double t_critval (double a, int df)
+double student_critval (double a, double df)
 {
-    double x = stdtri(df, 1 - a);
+    double x;
+
+    if (df < 1) {
+	return NADBL;
+    }    
+
+    if (df_ok(df)) {
+	if (a > .10) {
+	    x = stdtri((int) df, 1 - a);
+	} else {
+	    x = -stdtri((int) df, a);
+	}
+    } else {
+	if (a > .10) {
+	    x = ndtri(1 - a);
+	} else {
+	    x = -ndtri(a);
+	}
+    }
+
+    if (get_cephes_errno()) {
+	x = NADBL;
+    } 
+
+    return x;
+}
+
+static double student_cdf_inverse (double a, double df)
+{
+    double x;
+
+    if (df < 1) {
+	return NADBL;
+    }
+
+    if (df_ok(df)) {
+	x = stdtri((int) df, a);
+    } else {
+	x = ndtri(a);
+    }    
 
     if (get_cephes_errno()) {
 	x = NADBL;
@@ -399,8 +517,22 @@ double chisq_critval (double a, int df)
     return x;
 }
 
+static double chisq_cdf_inverse (double a, int df)
+{
+    double x = NADBL;
+
+    if (df > 0 && a >= 0) {
+	x = chdtri(df, 1 - a);
+	if (get_cephes_errno()) {
+	    x = NADBL;
+	}
+    }
+
+    return x;
+}
+
 /**
- * f_cdf:
+ * snedecor_cdf:
  * @x: the cutoff point in the distribution.
  * @dfn: numerator degrees of freedom.
  * @dfd: denominator degrees of freedom.
@@ -409,7 +541,7 @@ double chisq_critval (double a, int df)
  * @dfd degrees of freedom, from 0 to @x, or #NADBL on failure.
  */
 
-double f_cdf (double x, int dfn, int dfd)
+double snedecor_cdf (double x, int dfn, int dfd)
 {
     double p = NADBL;
 
@@ -424,7 +556,7 @@ double f_cdf (double x, int dfn, int dfd)
 }
 
 /**
- * f_cdf_comp:
+ * snedecor_cdf_comp:
  * @x: the cutoff point in the distribution.
  * @dfn: numerator degrees of freedom.
  * @dfd: denominator degrees of freedom.
@@ -434,7 +566,7 @@ double f_cdf (double x, int dfn, int dfd)
  * on failure.
  */
 
-double f_cdf_comp (double x, int dfn, int dfd)
+double snedecor_cdf_comp (double x, int dfn, int dfd)
 {
     double p = NADBL;
 
@@ -449,7 +581,7 @@ double f_cdf_comp (double x, int dfn, int dfd)
 }
 
 /**
- * f_critval:
+ * snedecor_critval:
  * @a: right-tail probability.
  * @dfn: numerator degrees of freedom.
  * @dfd: denominator degrees of freedom.
@@ -459,7 +591,7 @@ double f_cdf_comp (double x, int dfn, int dfd)
  * to the given probability @a, or #NADBL on failure.
  */
 
-double f_critval (double a, int dfn, int dfd)
+double snedecor_critval (double a, int dfn, int dfd)
 {
     double x = NADBL;
 
@@ -473,6 +605,20 @@ double f_critval (double a, int dfn, int dfd)
     return x;
 }
 
+static double snedecor_cdf_inverse (double a, int dfn, int dfd)
+{
+    double x = NADBL;
+
+    if (dfn > 0 && dfd > 0 && a >= 0) {
+	x = fdtri(dfn, dfd, 1 - a);
+	if (get_cephes_errno()) {
+	    x = NADBL;
+	}
+    }
+
+    return x;
+}
+
 /**
  * normal_cdf:
  * @x: double-precision value.
@@ -536,7 +682,13 @@ double normal_pdf (double x)
 
 double normal_critval (double a)
 {
-    double z = ndtri(1.0 - a);
+    double z;
+
+    if (a > 0.10) {
+	z = ndtri(1.0 - a);
+    } else {
+	z = -ndtri(a);
+    }
 
     if (get_cephes_errno()) {
 	z = NADBL;
@@ -631,12 +783,49 @@ double bvnorm_cdf (double a, double b, double rho)
     return ret;
 }
 
-static double poisson_pmf (double lambda, int k)
+double gamma_cdf (double s1, double s2, double x, int control)
 {
-    double den = x_factorial((double) k);
-    double l1 = exp(-lambda);
+    double shape, scale, p;
 
-    return l1 * pow(lambda, (double) k) / den;
+    if (control == 1) {
+	shape = s1; 
+	scale = s2; 
+    } else {
+	scale = s2 / s1; 
+	shape = s1 / scale; 
+    }	
+
+    p = gdtr(s2, s1, x);
+    if (get_cephes_errno()) {
+	p = NADBL;
+    }
+
+    return p;
+}
+
+/* Control 1 : s1, s2 = shape, scale
+           2 : s1, s2 = mean, variance
+   Returns NADBL on error.
+*/
+
+double gamma_cdf_comp (double s1, double s2, double x, int control)
+{
+    double shape, scale, p;
+
+    if (control == 1) {
+	shape = s1; 
+	scale = s2; 
+    } else {
+	scale = s2 / s1;    /* variance / mean */
+	shape = s1 / scale; /* mean / scale */
+    }
+
+    p = gdtrc(1.0 / scale, shape, x);
+    if (get_cephes_errno()) {
+	p = NADBL;
+    }
+
+    return p;
 }
 
 /**
@@ -650,21 +839,172 @@ static double poisson_pmf (double lambda, int k)
 
 static double poisson_cdf (double lambda, int k)
 {
-    double l1, l2, den, x = 0.0;
-    int i;
+    double x = NADBL;
 
-    l1 = exp(-lambda);
-    l2 = 1.0;
-
-    for (i=0; i<=k; i++) {
-	den = x_factorial((double) i);
-	x += l1 * l2 / den;
-	l2 *= lambda;
+    if (lambda >= 0 && k >= 0) {
+	x = pdtr(k, lambda);
+	if (get_cephes_errno()) {
+	    x = NADBL;
+	}
     }
 
     return x;
 }
 
+/**
+ * poisson_cdf_comp:
+ * @lambda: mean (also variance).
+ * @k: test value.
+ *
+ * Returns: the probability of X > @k, for X an r.v. that follows
+ * the Poisson distribution with parameter @lambda.
+ */
+
+static double poisson_cdf_comp (double lambda, int k)
+{
+    double x = NADBL;
+
+    if (lambda >= 0 && k >= 0) {
+	x = pdtrc(k, lambda);
+	if (get_cephes_errno()) {
+	    x = NADBL;
+	}
+    }
+
+    return x;
+}
+
+/**
+ * poisson_cdf_inverse:
+ * @k: test value.
+ * @p: cumulative probability.
+ *
+ * Returns: the Poisson parameter such that the integral
+ * from 0 to @k of the Poisson density is equal to the
+ * given probability @p.
+ */
+
+static double poisson_cdf_inverse (int k, double p)
+{
+    double x = NADBL;
+
+    if (k >= 0 && p >= 0 && p <= 1) {
+	x = pdtri(k, p);
+	if (get_cephes_errno()) {
+	    x = NADBL;
+	}
+    }
+
+    return x;
+}
+
+static double poisson_pmf (double lambda, int k)
+{
+    double den, l0, p;
+
+    if (lambda <= 0 || k < 0) {
+	return NADBL;
+    }
+
+    den = x_factorial((double) k);
+    l0 = exp(-lambda);
+
+    if (na(den) || isinf(den) || isnan(den)) {
+	p = NADBL;
+    } else {
+	p = l0 * pow(lambda, (double) k) / den;
+    }
+
+    if (na(p) || isinf(p) || isnan(p)) {
+	int i;
+
+	p = l0;
+	for (i=1; i<=k; i++) {
+	    p *= lambda / i;
+	}
+    } 
+
+    return p;
+}
+
+/* The following is probably horribly inefficient --
+   investigate the possibility of using Chebyshev,
+   or perhaps binary search.
+*/
+
+static double poisson_critval (double a, double mu)
+{
+    double pk = 0.0;
+    double ac = 1 - a;
+    int k, k0 = 0;
+
+    if (mu <= 0 || a <= 0 || a >= 1) {
+	return NADBL;
+    }
+
+    if (mu >= 10 && a < 0.5) {
+	k0 = mu - 1;
+	pk = poisson_cdf(mu, k0++);
+    }     
+
+    for (k=k0; ; k++) {
+	pk = poisson_cdf(mu, k);
+	if (pk >= ac) {
+	    break;
+	}
+    }  
+
+    return (double) k;
+}
+
+/* order in x: [params], alpha, critval */
+
+void print_critval (char st, double *x, double c, PRN *prn)
+{
+    int apos = 0;
+    double a;
+
+    switch (st) {
+    case 'z':
+	pprintf(prn, "%s", _("Standard normal distribution"));
+	break;
+    case 't':
+	pprintf(prn, "t(%g)", x[0]);
+	apos = 1;
+	break;
+    case 'X':
+	pprintf(prn, _("Chi-square(%g)"), x[0]);
+	apos = 1;
+	break;
+    case 'F':
+	pprintf(prn, "F(%g, %g)", x[0], x[1]);
+	apos = 2;
+	break;
+    case 'B':
+	pprintf(prn, "Binomial (P = %g, %g trials)", x[0], x[1]);
+	apos = 2;
+	break;
+    case 'P':
+	pprintf(prn, "Poisson (mean = %g)", x[0]);
+	apos = 1;
+	break;
+    }
+
+    a = x[apos];
+
+    pputs(prn, "\n ");
+    pprintf(prn, _("right-tail probability = %g"), a);
+    pputs(prn, "\n ");
+    pprintf(prn, _("complementary probability = %g"), 1.0 - a);
+    if (a < 0.5 && (st == 'z' || st == 't')) {
+	pputs(prn, "\n ");
+	pprintf(prn, _("two-tailed probability = %g"), 2.0 * a);
+    }
+    pputs(prn, "\n\n ");
+    pprintf(prn, _("Critical value = %g"), c);
+    pputc(prn, '\n');    
+}
+
 static double dparm[3];
 
 static void dparm_set (const double *p)
@@ -681,21 +1021,17 @@ double gretl_get_critval (char st, double *p)
     double x = NADBL;
 
     if (st == 'z') {
-	if (p[0] > 0.5) {
-	    x = ndtri(1.0 - p[0]);
-	} else {
-	    x = -ndtri(p[0]);
-	}
+	x = normal_critval(p[0]);
     } else if (st == 't') {
-	if (p[1] > 0.5) {
-	    x = stdtri((int) p[0], 1 - p[1]);
-	} else {
-	    x = -stdtri((int) p[0], p[1]);
-	}
+	x = student_critval(p[1], p[0]);
     } else if (st == 'X') {	
 	x = chisq_critval(p[1], (int) p[0]);
     } else if (st == 'F') {
-	x = f_critval(p[2], (int) p[0], (int) p[1]);
+	x = snedecor_critval(p[2], (int) p[0], (int) p[1]);
+    } else if (st == 'B') {
+	x = binomial_critval(p[2], p[0], (int) p[1]);
+    } else if (st == 'P') {
+	x = poisson_critval(p[1], p[0]);
     }
 
     return x;
@@ -708,13 +1044,13 @@ double gretl_get_cdf (char st, double *p)
     if (st == 'z') {
 	x = normal_cdf(p[0]);
     } else if (st == 't') {
-	x = t_cdf(p[1], (int) p[0]);
+	x = student_cdf(p[1], (int) p[0]);
     } else if (st == 'X') {
 	x = chisq_cdf(p[1], (int) p[0]);
     } else if (st == 'F') {
-	x = f_cdf(p[2], (int) p[0], (int) p[1]);
+	x = snedecor_cdf(p[2], (int) p[0], (int) p[1]);
     } else if (st == 'G') {
-	x = 1.0 - gamma_cdf_comp(p[0], p[1], p[2], 2);
+	x = gamma_cdf(p[0], p[1], p[2], 1);
     } else if (st == 'B') {
 	x = binomial_cdf((int) p[2], (int) p[1], p[0]);
     } else if (st == 'D') {
@@ -726,50 +1062,131 @@ double gretl_get_cdf (char st, double *p)
     return x;
 }
 
-double gretl_get_pvalue (char st, const double *p)
+double gretl_get_cdf_inverse (char st, double *p)
 {
     double x = NADBL;
 
     if (st == 'z') {
-	x = 1.0 - normal_cdf(p[0]);
+	x = normal_cdf_inverse(p[0]);
     } else if (st == 't') {
-	x = t_cdf_comp(p[1], (int) p[0]);
+	x = student_cdf_inverse(p[1], p[0]);
     } else if (st == 'X') {
-	x = chisq_cdf_comp(p[1], (int) p[0]);
+	x = chisq_cdf_inverse(p[1], (int) p[0]);
     } else if (st == 'F') {
-	x = f_cdf_comp(p[2], (int) p[0], (int) p[1]);
-    } else if (st == 'G') {
-	x = gamma_cdf_comp(p[0], p[1], p[2], 2);
+	x = snedecor_cdf_inverse(p[2], (int) p[0], (int) p[1]);
     } else if (st == 'B') {
-	x = binomial_cdf_comp((int) p[2], (int) p[1], p[0]);
+	x = binomial_cdf_inverse((int) p[2], (int) p[1], p[0]);
     } else if (st == 'P') {
-	x = 1.0 - poisson_cdf(p[0], (int) p[1]);
-    }
-
-    if (!na(x)) {
-	dparm_set(p);
+	x = poisson_cdf_inverse((int) p[0], p[1]);
     }
 
     return x;
 }
 
-static void 
+double gretl_get_pvalue (char st, const double *p)
+{
+    double x = NADBL;
+
+    if (st == 'z') {
+	x = normal_cdf_comp(p[0]);
+    } else if (st == 't') {
+	x = student_cdf_comp(p[1], (int) p[0]);
+    } else if (st == 'X') {
+	x = chisq_cdf_comp(p[1], (int) p[0]);
+    } else if (st == 'F') {
+	x = snedecor_cdf_comp(p[2], (int) p[0], (int) p[1]);
+    } else if (st == 'G') {
+	x = gamma_cdf_comp(p[0], p[1], p[2], 1);
+    } else if (st == 'B') {
+	x = binomial_cdf_comp((int) p[2], (int) p[1], p[0]);
+    } else if (st == 'P') {
+	x = poisson_cdf_comp(p[0], (int) p[1]);
+    }
+
+    dparm_set(p);
+
+    return x;
+}
+
+double *gretl_get_random_series (char st, const double *p,
+				 const DATAINFO *pdinfo,
+				 int *err)
+{
+    double *x = malloc(pdinfo->n * sizeof *x);
+    int t;
+
+    if (x == NULL) {
+	*err = E_ALLOC;
+	return NULL;
+    }
+
+    for (t=0; t<pdinfo->n; t++) {
+	x[t] = NADBL;
+    }
+
+    if (st == 'u') {
+	double min = p[0], max = p[1];
+
+	*err = gretl_rand_uniform_minmax(x, pdinfo->t1, pdinfo->t2, min, max);
+    } else if (st == 'z') {
+	double mu = p[0], sd = p[1];
+
+	*err = gretl_rand_normal_full(x, pdinfo->t1, pdinfo->t2, mu, sd);
+    } else if (st == 't') {
+	int v = p[0];
+
+	*err = gretl_rand_student(x, pdinfo->t1, pdinfo->t2, v);
+    } else if (st == 'X') {
+	int v = p[0];
+
+	*err = gretl_rand_chisq(x, pdinfo->t1, pdinfo->t2, v);
+    } else if (st == 'F') {
+	int v1 = p[0], v2 = p[1];
+
+	*err = gretl_rand_F(x, pdinfo->t1, pdinfo->t2, v1, v2);
+    } else if (st == 'G') {
+	double shape = p[0], scale = p[1];
+
+	*err = gretl_rand_gamma(x, pdinfo->t1, pdinfo->t2, shape, scale);
+    } else if (st == 'B') {
+	int n = p[0];
+	double pr = p[1];
+
+	*err = gretl_rand_binomial(x, pdinfo->t1, pdinfo->t2, n, pr);
+    } else if (st == 'P') {
+	double m = p[0]; /* FIXME? */
+
+	gretl_rand_poisson(x, pdinfo->t1, pdinfo->t2, &m, 0);
+    }
+
+    return x;
+}
+
+static int 
 print_pv_string (double x, double p, PRN *prn)
 {
     char numstr[32];
 
+    if (na(p)) {
+	pprintf(prn, _("area to the right of %g: NA\n"), x);
+	return 1;
+    }
+
     sprintf(numstr, "%g", p);
 
-    if (!strcmp(numstr, "1")) {
+    if (!strcmp(numstr, "1") || !strcmp(numstr, "0")) {
 	pprintf(prn, _("area to the right of %g =~ %g\n"), x, p);
     } else {
 	pprintf(prn, _("area to the right of %g = %g\n"), x, p);
     }
+
+    return 0;
 }
 
 void print_pvalue (char st, double *p, double pv, PRN *prn)
 {
     double pc;
+    int err;
 
     switch (st) {
 
@@ -778,7 +1195,8 @@ void print_pvalue (char st, double *p, double pv, PRN *prn)
     case 'N':
     case '1':
 	pprintf(prn, "\n%s: ", _("Standard normal"));
-	print_pv_string(p[0], pv, prn);
+	err = print_pv_string(p[0], pv, prn);
+	if (err) return;
 	if (pv < 0.5) {
 	    pprintf(prn, _("(two-tailed value = %g; complement = %g)\n"), 
 		    2 * pv, 1 - 2 * pv);
@@ -793,12 +1211,13 @@ void print_pvalue (char st, double *p, double pv, PRN *prn)
     case 't':
     case '2':
 	pprintf(prn, "\nt(%d): ", (int) p[0]);
-	print_pv_string(p[1], pv, prn);
+	err = print_pv_string(p[1], pv, prn);
+	if (err) return;
 	if (pv < 0.5) {
 	    pprintf(prn, _("(two-tailed value = %g; complement = %g)\n"), 
 		    2 * pv, 1 - 2 * pv);
 	} else {
-	    pc = t_cdf(p[1], (int) p[0]);
+	    pc = student_cdf(p[1], (int) p[0]);
 	    pprintf(prn, _("(to the left: %g)\n"), pc);
 	    pprintf(prn, _("(two-tailed value = %g; complement = %g)\n"), 
 		    2 * pc, 1 - 2 * pc);
@@ -810,7 +1229,8 @@ void print_pvalue (char st, double *p, double pv, PRN *prn)
     case 'c':
     case '3':
 	pprintf(prn, "\n%s(%d): ", _("Chi-square"), (int) p[0]);
-	print_pv_string(p[1], pv, prn);
+	err = print_pv_string(p[1], pv, prn);
+	if (err) return;
 	pc = chisq_cdf(p[1], p[0]);
 	pprintf(prn, _("(to the left: %g)\n"), pc);
 	break;
@@ -819,18 +1239,19 @@ void print_pvalue (char st, double *p, double pv, PRN *prn)
     case 'f':
     case '4':
 	pprintf(prn, "\nF(%d, %d): ", (int) p[0], (int) p[1]);
-	print_pv_string(p[2], pv, prn);
-	pc = f_cdf(p[2], (int) p[0], (int) p[1]);
+	err = print_pv_string(p[2], pv, prn);
+	if (err) return;
+	pc = snedecor_cdf(p[2], (int) p[0], (int) p[1]);
 	pprintf(prn, _("(to the left: %g)\n"), pc);
 	break;
 
     case 'G':
     case 'g':
     case '5':
-	pprintf(prn, _("\nGamma (mean %g, variance %g, shape %g, scale %g):"
+	pprintf(prn, _("\nGamma (shape %g, scale %g, mean %g, variance %g):"
 		       "\n area to the right of %g = %g\n"), 
-		p[0], p[1], p[0] * p[0] / p[1], p[1] / p[0],
-		p[2], 1 - pv);
+		p[0], p[1], p[0] * p[1], p[0] * p[1] * p[1],
+		p[2], pv);
 	break;
 
     case 'B':
@@ -840,20 +1261,29 @@ void print_pvalue (char st, double *p, double pv, PRN *prn)
 		       "\n Prob(x > %d) = %g\n"), 
 		p[0], (int) p[1], (int) p[2], pv);
 	pc = binomial_cdf(p[2], p[1], p[0]);
-	pprintf(prn, _(" Prob(x <= %d) = %g\n"), (int) p[2], pc);
 	if (p[2] > 0) {
+	    pprintf(prn, _(" Prob(x <= %d) = %g\n"), (int) p[2], pc);
 	    pprintf(prn, _(" Prob(x = %d) = %g\n"), (int) p[2],
 		    pc - binomial_cdf(p[2] - 1, p[1], p[0]));
-	}		
+	} else {
+	    pprintf(prn, _(" Prob(x = %d) = %g\n"), (int) p[2], pc);
+	}
 	break;
 
     case 'p':
     case 'P':
     case '8':
 	pprintf(prn, _("\nPoisson (mean = %g): "), p[0]);
-	print_pv_string(p[1], pv, prn);
-	pprintf(prn, _(" Prob(x = %d) = %g\n"), (int) p[1],
-		poisson_pmf(p[0], (int) p[1]));
+	err = print_pv_string(p[1], pv, prn);
+	if (err) return;
+	pc = poisson_cdf(p[0], (int) p[1]);
+	if (p[1] > 0) {
+	    pprintf(prn, _(" Prob(x <= %d) = %g\n"), (int) p[1], pc);
+	    pprintf(prn, _(" Prob(x = %d) = %g\n"), (int) p[1],
+		    poisson_pmf(p[0], p[1]));
+	} else {
+	    pprintf(prn, _(" Prob(x = %d) = %g\n"), (int) p[1], pc);
+	}
 	break;	
 
     default:
@@ -929,154 +1359,3 @@ int batch_pvalue (const char *str,
     return err;
 }
 
-/* Functions relating to the gamma distribution.
-   Allin Cottrell (cottrell@wfu.edu), October 2000.
-   Draws upon the pascal code specialf.pas by Bent Nielsen.
-*/
-
-static const double gamma_tol = 1e-7;
-
-/* Gamma distribution function.
-   See Johnson, Kotz and Balakrishnan: 
-   Continuous Univariate Distributions
-   vol 1, 2nd ed, Wiley 1994
-*/
-
-static double gammadist_wilson_hilferty (double shape, double scale, double x)
-{
-    double df = 2 * shape;
-    double xscaled = x * 2 / scale;
-    double xx;
-
-    xx = exp(log(xscaled/df)/3) - 1 + (double)(2) / 9 / df;
-    xx *= sqrt(9 * df / 2);
-
-    return normal_cdf(xx);
-} 
-
-/* Expansion of Gamma Integral int_0^x t^lambda-1 exp(-t)
-   Abramowitz and Stegun p. 262
-   Note that the series is alternating.
-*/
-
-static double gamma_integral_expansion (double lambda, double x)
-{
-    double g, x1 = 1;
-    double x2, x3 = 1 / lambda;
-    int i = 0;
-
-    do {
-	i++;
-	x1 *= (-x)/i;
-	x2 = x1 / (lambda + i);
-	x3 += x2;
-    } while (fabs(x2) >= gamma_tol && i <= 100);
-
-    if (i == 100) {
-	g = NADBL;
-    } else {
-	g = x3 * exp(lambda * log(x));
-    }
-
-    return g;
-}
-
-/* Continued Fraction Expansion for Gamma Integral
-   int_0^x t^lambda-1 exp(-t) dx
-   Abramowitz and Stegun p. 263
-   Implemented in Fortran by
-   B. L. Shea (1988): Chi-squared and incomplete gamma integral,
-   Applied Statistics, vol 37, pp. 466-473.
-   See also Schwartz p. 120.      
-*/
-
-static double gamma_integral_fraction (double lambda, double x)
-{
-    double a = 1 - lambda;
-    double b = a + x + 1;
-    double p1 = 1, p2 = 1 + x;
-    double q1 = x, q2 = b * x;
-    double r2 = p2 / q2;
-    double d, p0, q0, r1, xx;
-    int c = 0;
-
-    do {
-	p0 = p1; p1 = p2; q0 = q1; q1 = q2; r1 = r2;
-	a = a + 1; b = b + 2; c = c + 1; d = a * c;
-	p2 = b * p1 - d * p0;
-	q2 = b * q1 - d * q0;
-	if (fabs(q2) > 0) {
-	    r2 = p2 / q2;
-	}
-	xx = fabs(r2 - r1);
-    } while (!((xx < gamma_tol ) || (xx < gamma_tol * r2) || (c == 100)));
-
-    if (c == 100) {
-	xx = NADBL;
-    } else {
-	xx = cephes_gamma(lambda);
-	xx -= exp(-x + lambda * log(x)) * r2;
-    }
-
-    return xx;
-}
-
-static double gamma_integral (double lambda, double x)
-{
-    double g;
-
-    if (x < 0)  { 
-	g = NADBL;
-    } else if (x < gamma_tol) {
-	g = 0;
-    } else if (x <= 1 || x < 0.9 * lambda) {
-	g = gamma_integral_expansion(lambda, x);
-    } else {
-	g = gamma_integral_fraction(lambda, x);
-    }
-
-    return g;
-}
-
-/* Control 1 : s1, s2 = shape, scale
-           2 : s1, s2 = expectation, variance
-   Returns NADBL on error 
-*/
-
-double gamma_cdf_comp (double s1, double s2, double x, int control)
-{
-    double shape, scale, xx;
-
-    if (control == 1) {
-	shape = s1; 
-	scale = s2; 
-    } else {
-	scale = s2 / s1; 
-	shape = s1 / scale; 
-    }	
-
-    if ((shape > 20) && (x / scale < 0.9 * shape) && (x > 1)) {
-	xx = gammadist_wilson_hilferty(shape, scale, x);
-    } else {
-	xx = gamma_integral(shape, x / scale);
-	if (!na(xx)) {
-	    xx /= cephes_gamma(shape);
-	}
-    }
-
-    return xx;
-}
-
-
-
-
-
-
-
-
-
-
-
-
-
-
