@@ -1,20 +1,20 @@
-/*
- *   Copyright (c) 2003-2004 by Allin Cottrell
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
- *
- *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- *
+/* 
+ *  gretl -- Gnu Regression, Econometrics and Time-series Library
+ *  Copyright (C) 2001 Allin Cottrell and Riccardo "Jack" Lucchetti
+ * 
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ * 
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ * 
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 #include "libgretl.h"
@@ -405,20 +405,22 @@ int newey_west_bandwidth (const gretl_matrix *f, int kern, int *h, double *bt)
     
     for (j=1; j<=n; j++) {
 	if (kern == KERNEL_BARTLETT) {
-	    sv += 2 * j * s[j];
+	    sv += 2.0 * j * s[j];
 	} else {
-	    sv += 2 * j * j * s[j];
+	    sv += 2.0 * j * j * s[j];
 	}
 	s0 += 2 * s[j];
     }
 
-    i = kern - 1;
-
-    p = 1.0 / (2 * v[i] + 1);
-    g = cg[i] * pow((sv / s0) * (sv / s0), p);
+    p = 1.0 / (2.0 * v[kern] + 1);
+    g = cg[kern] * pow((sv / s0) * (sv / s0), p);
 
     *bt = g * pow((double) T, p);
     *h = (int) floor(*bt);
+
+#if 0
+    fprintf(stderr, "bt = %g, h = %d\n", *bt, *h);
+#endif
     
  bailout:
 
@@ -469,8 +471,8 @@ static int qr_make_hac (MODEL *pmod, const double **Z, gretl_matrix *xpxinv)
 {
     gretl_matrix *vcv = NULL, *wtj = NULL, *gammaj = NULL;
     gretl_matrix *X;
-    int prewhiten = get_hac_prewhiten();
-    int kern = get_hac_kernel();
+    int prewhiten = libset_get_bool(PREWHITEN);
+    int kern = libset_get_int(HAC_KERNEL);
     int T = pmod->nobs;
     int k = pmod->ncoeff;
     int free_uhat = 0;
@@ -519,7 +521,7 @@ static int qr_make_hac (MODEL *pmod, const double **Z, gretl_matrix *xpxinv)
 	    goto bailout;
 	}
     } else if (kern == KERNEL_QS) {
-	bt = get_qs_bandwidth();
+	bt = libset_get_double(QS_BANDWIDTH);
 	p = pmod->nobs - 1;
     } else {
 	p = get_hac_lag(T);
@@ -643,7 +645,7 @@ static int qr_make_hccme (MODEL *pmod, const double **Z,
 	goto bailout;
     }  
 
-    hc_version = get_hc_version();
+    hc_version = libset_get_int(HC_VERSION);
     gretl_model_set_int(pmod, "hc", 1);
     if (hc_version > 0) {
 	gretl_model_set_int(pmod, "hc_version", hc_version);
@@ -678,7 +680,7 @@ static int qr_make_hccme (MODEL *pmod, const double **Z,
     /* Transcribe vcv into triangular representation */
     err = qr_make_vcv(pmod, vcv, VCV_ROBUST);
 
-    bailout:
+ bailout:
 
     gretl_matrix_free(X);
     gretl_matrix_free(diag);
@@ -960,7 +962,7 @@ int gretl_qr_regress (MODEL *pmod, const double **Z, DATAINFO *pdinfo,
     /* VCV and standard errors */
     if (opt & OPT_R) { 
 	gretl_model_set_int(pmod, "robust", 1);
-	if ((opt & OPT_T) && !get_force_hc()) {
+	if ((opt & OPT_T) && !libset_get_bool(FORCE_HC)) {
 	    qr_make_hac(pmod, Z, V);
 	} else {
 	    qr_make_hccme(pmod, Z, Q, V);
@@ -1025,7 +1027,8 @@ int qr_tsls_vcv (MODEL *pmod, const double **Z, const DATAINFO *pdinfo,
 	    if (!err) {
 		err = panel_tsls_robust_vcv(pmod, Z, pdinfo);
 	    }
-	} else if (dataset_is_time_series(pdinfo) && !get_force_hc()) {
+	} else if (dataset_is_time_series(pdinfo) && 
+		   !libset_get_bool(FORCE_HC)) {
 	    gretl_model_set_int(pmod, "robust", 1);
 	    err = qr_make_hac(pmod, Z, V);
 	} else {
