@@ -1,20 +1,20 @@
-/*
- *  Copyright (c) by Ramu Ramanathan and Allin Cottrell
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
- *
- *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
- *
+/* 
+ *  gretl -- Gnu Regression, Econometrics and Time-series Library
+ *  Copyright (C) 2001 Allin Cottrell and Riccardo "Jack" Lucchetti
+ * 
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ * 
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ * 
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 /* random.c for gretl */
@@ -25,10 +25,11 @@
 #include <glib.h>
 
 static GRand *gretl_rand;
-
 static unsigned int useed;
 
-unsigned int get_gretl_random_seed (void)
+#undef OLD_NORMAL
+
+unsigned int gretl_rand_get_seed (void)
 {
     return useed;
 }
@@ -88,13 +89,13 @@ double gretl_one_snormal (void)
     return (z * cos(M_2PI * y));
 }
 
+#define MOD_POLAR 1
+
 /**
  * gretl_two_snormals:
  *
  */
 
-#define MOD_POLAR 0
-
 void gretl_two_snormals (double *z1, double *z2) 
 {
     double x, y, z;
@@ -128,7 +129,7 @@ void gretl_two_snormals (double *z1, double *z2)
 }
 
 /**
- * gretl_uniform_dist_minmax:
+ * gretl_rand_uniform_minmax:
  * @a: target array.
  * @t1: start of the fill range.
  * @t2: end of the fill range.
@@ -142,7 +143,7 @@ void gretl_two_snormals (double *z1, double *z2)
  * Returns: 0 on success, 1 on invalid input.
  */
 
-int gretl_uniform_dist_minmax (double *a, int t1, int t2,
+int gretl_rand_uniform_minmax (double *a, int t1, int t2,
 			       double min, double max) 
 {
     int t;
@@ -162,7 +163,7 @@ int gretl_uniform_dist_minmax (double *a, int t1, int t2,
 }
 
 /**
- * gretl_uniform_dist:
+ * gretl_rand_uniform:
  * @a: target array
  * @t1: start of the fill range
  * @t2: end of the fill range
@@ -172,7 +173,7 @@ int gretl_uniform_dist_minmax (double *a, int t1, int t2,
  * Twister.
  */
 
-void gretl_uniform_dist (double *a, int t1, int t2) 
+void gretl_rand_uniform (double *a, int t1, int t2) 
 {
     int t;
 
@@ -182,7 +183,7 @@ void gretl_uniform_dist (double *a, int t1, int t2)
 }
 
 /**
- * gretl_normal_dist:
+ * gretl_rand_normal:
  * @a: target array
  * @t1: start of the fill range
  * @t2: end of the fill range
@@ -193,9 +194,9 @@ void gretl_uniform_dist (double *a, int t1, int t2)
  * normal distribution.
  */
 
-void gretl_normal_dist (double *a, int t1, int t2) 
+void gretl_rand_normal (double *a, int t1, int t2) 
 {
-#if 1
+#ifndef OLD_NORMAL
     double z1, z2;
     int t;
 
@@ -216,7 +217,7 @@ void gretl_normal_dist (double *a, int t1, int t2)
 }
 
 /**
- * gretl_normal_dist_with_params:
+ * gretl_rand_normal_full:
  * @a: target array
  * @t1: start of the fill range
  * @t2: end of the fill range
@@ -232,8 +233,8 @@ void gretl_normal_dist (double *a, int t1, int t2)
  */
 
 int
-gretl_normal_dist_with_params (double *a, int t1, int t2,
-			       double mean, double sd) 
+gretl_rand_normal_full (double *a, int t1, int t2,
+			double mean, double sd) 
 {
     int t;
 
@@ -244,7 +245,7 @@ gretl_normal_dist_with_params (double *a, int t1, int t2,
 	return E_INVARG;
     }
 
-    gretl_normal_dist(a, t1, t2);
+    gretl_rand_normal(a, t1, t2);
 
     if (mean != 0.0 || sd != 1.0) {
 	for (t=t1; t<=t2; t++) {
@@ -256,7 +257,7 @@ gretl_normal_dist_with_params (double *a, int t1, int t2,
 }
 
 /**
- * gretl_chisq_dist:
+ * gretl_rand_chisq:
  * @a: target array.
  * @t1: start of the fill range.
  * @t2: end of the fill range.
@@ -270,7 +271,7 @@ gretl_normal_dist_with_params (double *a, int t1, int t2,
  * Returns: 0 on success, non-zero on error.
  */
 
-int gretl_chisq_dist (double *a, int t1, int t2, int v) 
+int gretl_rand_chisq (double *a, int t1, int t2, int v) 
 {
     double z;
     int i, t;
@@ -291,7 +292,7 @@ int gretl_chisq_dist (double *a, int t1, int t2, int v)
 }
 
 /**
- * gretl_t_dist:
+ * gretl_rand_student:
  * @a: target array.
  * @t1: start of the fill range.
  * @t2: end of the fill range.
@@ -305,7 +306,7 @@ int gretl_chisq_dist (double *a, int t1, int t2, int v)
  * Returns: 0 on success, non-zero on error.
  */
 
-int gretl_t_dist (double *a, int t1, int t2, int v) 
+int gretl_rand_student (double *a, int t1, int t2, int v) 
 {
     double *X2 = NULL;
     int T = t2 - t1 + 1;
@@ -320,8 +321,8 @@ int gretl_t_dist (double *a, int t1, int t2, int v)
 	return E_ALLOC;
     }
 
-    gretl_normal_dist(a, t1, t2);
-    gretl_chisq_dist(X2, 0, T-1, v);
+    gretl_rand_normal(a, t1, t2);
+    gretl_rand_chisq(X2, 0, T-1, v);
 
     for (t=0; t<T; t++) {
 	a[t + t1] /= sqrt(X2[t] / v);
@@ -333,7 +334,51 @@ int gretl_t_dist (double *a, int t1, int t2, int v)
 }
 
 /**
- * gretl_binomial_dist:
+ * gretl_rand_F:
+ * @a: target array.
+ * @t1: start of the fill range.
+ * @t2: end of the fill range.
+ * @v1: numerator degrees of freedom.
+ * @v2: denominator degrees of freedom.
+ *
+ * Fill the selected range of array @a with pseudo-random drawings
+ * from the F distribution with @v1 and @v2 degrees of freedom, 
+ * using the Mersenne Twister for uniform input and the Box-Muller 
+ * method for converting to the normal distribution.
+ *
+ * Returns: 0 on success, non-zero on error.
+ */
+
+int gretl_rand_F (double *a, int t1, int t2, int v1, int v2) 
+{
+    double *b = NULL;
+    int T = t2 - t1 + 1;
+    int s, t;
+
+    if (v1 < 1 || v2 < 1) {
+	return E_INVARG;
+    }
+
+    b = malloc(T * sizeof *b);
+    if (b == NULL) {
+	return E_ALLOC;
+    }
+
+    gretl_rand_chisq(a, t1, t2, v1);
+    gretl_rand_chisq(b, 0, T-1, v2);
+
+    for (t=0; t<T; t++) {
+	s = t + t1;
+	a[s] = (a[s]/v1) / (b[t]/v2);
+    }
+
+    free(b);
+
+    return 0;
+}
+
+/**
+ * gretl_rand_binomial:
  * @a: target array.
  * @t1: start of the fill range.
  * @t2: end of the fill range.
@@ -346,7 +391,7 @@ int gretl_t_dist (double *a, int t1, int t2, int v)
  * Returns: 0 on success, non-zero on error.
  */
 
-int gretl_binomial_dist (double *a, int t1, int t2, int n, double p) 
+int gretl_rand_binomial (double *a, int t1, int t2, int n, double p) 
 {
     double *b;
     int i, t;
@@ -362,7 +407,7 @@ int gretl_binomial_dist (double *a, int t1, int t2, int n, double p)
 
     for (t=t1; t<=t2; t++) {
 	a[t] = 0.0;
-	gretl_uniform_dist(b, 0, n - 1);
+	gretl_rand_uniform(b, 0, n - 1);
 	for (i=0; i<n; i++) {
 	    if (b[i] <= p) {
 		a[t] += 1;
@@ -375,6 +420,110 @@ int gretl_binomial_dist (double *a, int t1, int t2, int n, double p)
     return 0;
 }
 
+/**
+ * gretl_rand_gamma:
+ * @a: target array.
+ * @t1: start of the fill range.
+ * @t2: end of the fill range.
+ * @shape: shape parameter.
+ * @scale: scale parameter.
+ *
+ * Fill the selected range of array @a with pseudo-random drawings
+ * from the specified gamma distribution.
+ *
+ * Returns: 0 on success, non-zero on error.
+ */
+
+int gretl_rand_gamma (double *a, int t1, int t2, 
+		      double shape, double scale) 
+{
+    double *U = NULL;
+    double e = 2.718281828459045235;
+    double delta, dinv = 0, d1 = 0;
+    double u, v, x, y, u0 = 0;
+    int k, i, t;
+
+    if (shape <= 0 || scale <= 0) {
+	return E_DATA;
+    }
+
+    k = shape;
+    if (k > 0) {
+	U = malloc(k * sizeof *U);
+	if (U == NULL) {
+	    return E_ALLOC;
+	}
+    }
+
+    delta = shape - k;
+
+    if (delta > 0) {
+	d1 = delta - 1;
+	dinv = 1 / delta;
+	u0 = e / (e + delta);
+    }
+
+    /* 
+       Case of shape < 1 from Kundu and Gupta, "A convenient way of
+       generating gamma random variables using generalized exponential
+       distribution", Computational Statistics and Data Analysis, 51
+       (2007).  Case of shape >= 1 taken from the Wikipedia entry on
+       the gamma distribution.
+    */
+
+    for (t=t1; t<=t2; t++) {
+	a[t] = 0.0;
+	if (shape < 1) {
+	    double ex2;
+
+	    while (1) {
+		u = gretl_one_uniform();
+		v = gretl_one_uniform();
+		x = -2 * log(1 - pow(u, dinv));
+		ex2 = exp(-x/2);
+		u0 = pow(x, d1) * ex2;
+		u0 /= pow(2.0, d1) * pow((1-ex2), d1);
+		if (v <= u0) {
+		    a[t] = x;
+		    break;
+		}
+	    }
+	} else {
+	    for (i=0; i<k; i++) {
+		U[i] = gretl_one_uniform();
+		while (U[i] == 0.0) {
+		    U[i] = gretl_one_uniform();
+		}
+	    }
+	    if (delta > 0) {
+		while (1) {
+		    u = gretl_one_uniform();
+		    v = gretl_one_uniform();
+		    if (u <= u0) {
+			x = pow(u, dinv);
+			y = v * pow(x, d1);
+		    } else {
+			x = 1 - log(u);
+			y = v * exp(-x);
+		    }
+		    if (y <= pow(x, d1) * exp(-x)) {
+			a[t] = x;
+			break;
+		    }
+		}
+	    } 
+	    for (i=0; i<k; i++) {
+		a[t] -= log(U[i]);
+	    }
+	}
+	a[t] *= scale;	
+    }
+
+    free(U);
+	
+    return 0;
+}
+
 /* Poisson rv with mean m */
 
 static double genpois (double m)
@@ -399,7 +548,7 @@ static double genpois (double m)
 }
 
 /**
- * gretl_poisson_dist:
+ * gretl_rand_poisson:
  * @a: target array.
  * @t1: start of the fill range.
  * @t2: end of the fill range.
@@ -412,7 +561,7 @@ static double genpois (double m)
  * of length greater than or equal to @t2 + 1.  
  */
 
-void gretl_poisson_dist (double *a, int t1, int t2, double *m,
+void gretl_rand_poisson (double *a, int t1, int t2, double *m,
 			 int vec) 
 {
     int t;
