@@ -1,20 +1,20 @@
-/*
- *  Copyright (c) by Ramu Ramanathan and Allin Cottrell
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
- *
- *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- *
+/* 
+ *  gretl -- Gnu Regression, Econometrics and Time-series Library
+ *  Copyright (C) 2001 Allin Cottrell and Riccardo "Jack" Lucchetti
+ * 
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ * 
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ * 
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 /* texprint.c for gretl - LaTeX output from modeling */
@@ -150,12 +150,19 @@ char *tex_escape (char *targ, const char *src)
 {
     char *p = targ;
 
+    if (src == NULL) {
+	fprintf(stderr, "tex_escape: src is NULL\n");
+	*p = '\0';
+	return p;
+    }
+
     while (*src) {
 	if (*src == '$' || *src == '&' || *src == '_' || 
 	    *src == '%' || *src == '#')
 	    *targ++ = '\\';
 	*targ++ = *src++;
     }
+
     *targ = '\0';
 
     return p;
@@ -369,51 +376,33 @@ static void tex_arma_coeff_name (char *targ, const char *src,
     }
 }
 
-static void tex_lagname (char *s, const DATAINFO *pdinfo, int v)
+static void tex_VAR_varname (char *s, const MODEL *pmod,
+			     const DATAINFO *pdinfo, int v)
 {
-    const char *lbl = VARLABEL(pdinfo, v);
-    int gotit = 0;
+    char tmp[32], base[12];
+    int lag;
 
-    if (strlen(lbl) > 2) {
-	char myvar[32], tmp[VNAMELEN];
-	int lag;
+    gretl_model_get_param_name(pmod, pdinfo, v, tmp);
 
-	lbl += 2;
-	if (sscanf(lbl, "%15[^(](t - %d)", tmp, &lag) == 2) {
-	    tex_escape(myvar, tmp);
-	    sprintf(s, "%s$_{t-%d}$", myvar, lag);
-	    gotit = 1;
-	}
-    }
-	
-    if (!gotit) {
-	tex_escape(s, pdinfo->varname[v]); 
+    if (sscanf(tmp, "%11[^_]_%d", base, &lag) == 2) {
+	sprintf(s, "%s$_{t-%d}$", base, lag);
+    } else {
+	tex_escape(s, tmp);
     }
 }
 
-static void tex_vecm_varname (char *s, const DATAINFO *pdinfo, int v)
+static void tex_VECM_varname (char *s, const MODEL *pmod,
+			      const DATAINFO *pdinfo, int v)
 {
-    const char *lbl = VARLABEL(pdinfo, v);
-    int cvnum;
-    int gotit = 0;
+    char tmp[32], base[12];
+    int lag;
 
-    if (sscanf(pdinfo->varname[v], "EC%d", &cvnum)) {
-	sprintf(s, "EC%d$_{t-1}$", cvnum);
-	gotit = 1;
-    } else if (strlen(lbl) > 2) {
-	char myvar[32], tmp[VNAMELEN];
-	int lag;
+    gretl_model_get_param_name(pmod, pdinfo, v, tmp);
 
-	lbl += 2;
-	if (sscanf(lbl, "d_%15[^(](t - %d)", tmp, &lag) == 2) {
-	    tex_escape(myvar, tmp);
-	    sprintf(s, "$\\Delta$%s$_{t-%d}$", myvar, lag);
-	    gotit = 1;
-	}
-    }
-
-    if (!gotit) {
-	tex_escape(s, pdinfo->varname[v]); 
+    if (sscanf(tmp, "d_%11[^_]_%d", base, &lag) == 2) {
+	sprintf(s, "$\\Delta$%s$_{t-%d}$", base, lag);
+    } else {
+	tex_escape(s, tmp);
     }
 }
 
@@ -491,7 +480,7 @@ void make_tex_coeff_name (const MODEL *pmod, const DATAINFO *pdinfo, int i,
     int j = i + 2;
 
     if (pmod->aux == AUX_ARCH) {
-	tex_make_cname(name, pdinfo->varname[pmod->list[i+2]]);
+	tex_make_cname(name, pdinfo->varname[pmod->list[j]]);
     } else if (pmod->ci == NLS) {
 	if (!tex_greek_param(name, pmod->params[i])) {
 	    tex_escape(name, pmod->params[i]);
@@ -501,16 +490,15 @@ void make_tex_coeff_name (const MODEL *pmod, const DATAINFO *pdinfo, int i,
     } else if (pmod->ci == GARCH) {
 	tex_garch_coeff_name(name, pmod->params[i], 0);
     } else if (pmod->ci == VAR) {
-	tex_lagname(name, pdinfo, pmod->list[j]);
+	tex_VAR_varname(name, pmod, pdinfo, i);
     } else if (pmod->aux == AUX_VECM) {
-	tex_vecm_varname(name, pdinfo, pmod->list[j]);
+	tex_VECM_varname(name, pmod, pdinfo, i);
     } else if (pmod->ci == MPOLS && pmod->params != NULL) {
 	tex_mp_coeff_name(name, pmod->params[i], 0);
-    } else if ((pmod->ci == PROBIT || pmod->ci == LOGIT) &&
+    } else if ((pmod->ci == PROBIT || pmod->ci == LOGIT || pmod->ci == HECKIT) &&
 	       pmod->params != NULL) {
 	tex_escape(name, pmod->params[i]);
-    } else if (pmod->ci == PANEL || pmod->ci == MLE ||
-	       pmod->ci == GMM) {
+    } else if (pmod->ci == PANEL || pmod->ci == MLE || pmod->ci == GMM) {
 	tex_escape(name, pmod->params[i]);
     } else if (pmod->ci == ARBOND) {
 	tex_arbond_coeff_name(name, pmod->params[i], 0);
@@ -671,7 +659,7 @@ void tex_coeff_table_end (PRN *prn)
 void tex_print_VECM_omega (GRETL_VAR *vecm, const DATAINFO *pdinfo, PRN *prn)
 {
     char vname[48];
-    const int *list = vecm->jinfo->list;
+    const int *list = vecm->ylist;
     double x;
     int i, j;
 
@@ -719,6 +707,24 @@ void tex_print_VECM_omega (GRETL_VAR *vecm, const DATAINFO *pdinfo, PRN *prn)
     pputs(prn, "\\\\\n");
 }
 
+static void tex_beta_vname (char *s,
+			    const GRETL_VAR *v,
+			    const DATAINFO *pdinfo,
+			    int i, PRN *prn)
+{
+    if (i < v->neqns) {
+	tex_escape(s, pdinfo->varname[v->ylist[i+1]]);
+	pprintf(prn, "%s$_{t-1}$ & ", s);
+    } else if (auto_restr(v) && i == v->neqns) {
+	pprintf(prn, "%s & ", (jcode(v) == J_REST_CONST)? "const" : "trend");
+    } else if (v->rlist != NULL) {
+	int k = i - v->ylist[0] - auto_restr(v) + 1;
+
+	tex_escape(s, pdinfo->varname[v->rlist[k]]);
+	pprintf(prn, "%s$_{t-1}$ & ", s);
+    } 
+}
+
 void tex_print_VECM_coint_eqns (GRETL_VAR *vecm, const DATAINFO *pdinfo, PRN *prn)
 {
     char s[32];
@@ -731,7 +737,9 @@ void tex_print_VECM_coint_eqns (GRETL_VAR *vecm, const DATAINFO *pdinfo, PRN *pr
     pputs(prn, _("Cointegrating vectors"));
     if (jv->Bse != NULL) {
 	pprintf(prn, " (%s)\n", _("standard errors in parentheses"));
-    } 
+    } else {
+	pputc(prn, '\n');
+    }
 
     pputs(prn, "\n\\vspace{1em}\n");
 
@@ -743,14 +751,7 @@ void tex_print_VECM_coint_eqns (GRETL_VAR *vecm, const DATAINFO *pdinfo, PRN *pr
     pputs(prn, "}\n");
 
     for (i=0; i<rows; i++) {
-	if (i < jv->list[0]) {
-	    tex_escape(s, pdinfo->varname[jv->list[i+1]]);
-	    pprintf(prn, "%s$_{t-1}$ & ", s);
-	} else if (jv->code == J_REST_CONST) {
-	    pputs(prn, "const & ");
-	} else if (jv->code == J_REST_TREND) {
-	    pputs(prn, "trend & ");
-	}
+	tex_beta_vname(s, vecm, pdinfo, i, prn);
 
 	/* coefficients */
 	for (j=0; j<jv->rank; j++) {
@@ -770,11 +771,7 @@ void tex_print_VECM_coint_eqns (GRETL_VAR *vecm, const DATAINFO *pdinfo, PRN *pr
 	    /* standard errors */
 	    pputs(prn, " & ");
 	    for (j=0; j<jv->rank; j++) {
-		if (i < jv->rank) {
-		    x = 0.0;
-		} else {
-		    x = gretl_matrix_get(jv->Bse, i - jv->rank, j);
-		}
+		x = gretl_matrix_get(jv->Bse, i, j);
 		pputc(prn, '(');
 		tex_print_float(x, prn);
 		pputc(prn, ')');
@@ -788,6 +785,64 @@ void tex_print_VECM_coint_eqns (GRETL_VAR *vecm, const DATAINFO *pdinfo, PRN *pr
     }
 
     pputs(prn, "\\end{tabular}\n\n\\vspace{1em}\n");
+    pputc(prn, '\n');
+
+    rows = gretl_matrix_rows(jv->Alpha);
+
+    pputs(prn, "\\noindent\n");
+    pprintf(prn, _("Adjustment vectors"));
+    if (jv->Ase != NULL) {
+	pprintf(prn, " (%s)\n", _("standard errors in parentheses"));
+    } else {
+	pputc(prn, '\n');
+    }
+
+    pputs(prn, "\n\\vspace{1em}\n");
+
+    pputs(prn, "\\begin{tabular}{");
+    pputs(prn, "l");
+    for (i=0; i<jv->rank; i++) {
+	pputs(prn, "r");
+    }
+    pputs(prn, "}\n");
+
+    for (i=0; i<rows; i++) {
+	tex_beta_vname(s, vecm, pdinfo, i, prn);
+
+	/* coefficients */
+	for (j=0; j<jv->rank; j++) {
+	    x = gretl_matrix_get(jv->Alpha, i, j);
+	    if (jv->Ase == NULL) {
+		x /= gretl_matrix_get(jv->Alpha, j, j);
+	    }
+	    tex_print_signed_float(x, prn);
+	    if (j == jv->rank - 1) {
+		pputs(prn, "\\\\\n");
+	    } else {
+		pputs(prn, "& ");
+	    }	    
+	}
+
+	if (jv->Ase != NULL) {
+	    /* standard errors */
+	    pputs(prn, " & ");
+	    for (j=0; j<jv->rank; j++) {
+		x = gretl_matrix_get(jv->Ase, i, j);
+		pputc(prn, '(');
+		tex_print_float(x, prn);
+		pputc(prn, ')');
+		if (j == jv->rank - 1) {
+		    pputs(prn, "\\\\\n");
+		} else {
+		    pputs(prn, "& ");
+		}
+	    }
+	}
+    }
+
+    pputs(prn, "\\end{tabular}\n\n\\vspace{1em}\n");
+    pputc(prn, '\n');
+
 }
 
 void tex_print_VAR_ll_stats (GRETL_VAR *var, PRN *prn)
