@@ -1,20 +1,20 @@
-/*
- *  Copyright (c) by Allin Cottrell and Riccardo Lucchetti
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
- *
- *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- *
+/* 
+ *  gretl -- Gnu Regression, Econometrics and Time-series Library
+ *  Copyright (C) 2001 Allin Cottrell and Riccardo "Jack" Lucchetti
+ * 
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ * 
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ * 
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 /* tsls.c - Two-Stage Least Squares for gretl */
@@ -130,11 +130,11 @@ tsls_save_data (MODEL *pmod, const int *hatlist, const int *exolist,
 
     /* now attach X and endog to the model */
     if (!recycle_X) {
-	gretl_model_set_data(pmod, "tslsX", X, MODEL_DATA_DOUBLE_ARRAY,
+	gretl_model_set_data(pmod, "tslsX", X, GRETL_TYPE_DOUBLE_ARRAY,
 			     Xsize);
     }
     if (!recycle_e) {
-	gretl_model_set_data(pmod, "endog", endog, MODEL_DATA_CHAR_ARRAY,
+	gretl_model_set_data(pmod, "endog", endog, GRETL_TYPE_CHAR_ARRAY,
 			     esize);
     }
 
@@ -387,8 +387,7 @@ tsls_make_hatlist (const int *reglist, int *instlist, int *hatlist)
 	}
 	if (endog) {
 	    if (reglist[i] == 0) {
-		/* found const in reglist but not instlist: needs fixing
-		   FIXME non-official const? */
+		/* found const in reglist but not instlist: needs fixing */
 		addconst = 1;
 	    } else {
 		hatlist[++k] = reglist[i];
@@ -407,7 +406,7 @@ tsls_make_hatlist (const int *reglist, int *instlist, int *hatlist)
 	instlist[1] = 0;
     }
 
-    return 0;
+    return addconst;
 }
 
 /* perform the Sargan overidentification test for a model
@@ -501,7 +500,12 @@ tsls_hausman_test (MODEL *tmod, int *reglist, int *hatlist,
 
     HT_list = gretl_list_add(reglist, hatlist, &err);
     if (err) {
-	fprintf(stderr, "tsls_hausman_test: gretl_list_add: err = %d\n", err);
+	if (err == E_NOADD) {
+	    /* more of a no-op than an error */
+	    err = 0;
+	} else {
+	    fprintf(stderr, "tsls_hausman_test: gretl_list_add: err = %d\n", err);
+	}
 	goto bailout;
     } 
 
@@ -551,9 +555,8 @@ static gretl_matrix *tsls_Q (int *instlist, int *reglist, int **pdlist,
     double test;
     int i, j, k;
 
-    Q = gretl_matrix_data_subset(instlist, Z, t1, t2, mask);
-    if (Q == NULL) {
-	*err = E_ALLOC;
+    Q = gretl_matrix_data_subset(instlist, Z, t1, t2, mask, err);
+    if (*err) {
 	return NULL;
     }
 
@@ -609,9 +612,11 @@ static gretl_matrix *tsls_Q (int *instlist, int *reglist, int **pdlist,
 
 	k = instlist[0];
 	gretl_matrix_free(Q);
-	Q = gretl_matrix_data_subset(instlist, Z, t1, t2, mask);
-	R = gretl_matrix_reuse(R, k, k);
-	*err = gretl_matrix_QR_decomp(Q, R);
+	Q = gretl_matrix_data_subset(instlist, Z, t1, t2, mask, err);
+	if (!*err) {
+	    R = gretl_matrix_reuse(R, k, k);
+	    *err = gretl_matrix_QR_decomp(Q, R);
+	}
     }
 
  bailout:
@@ -930,7 +935,7 @@ tsls_adjust_sample (const int *list, int *t1, int *t2,
     if (missobs == T) {
 	err = E_MISSDATA;
     } else if (missobs > 0) {
-	mask = calloc(T, 1);
+	mask = calloc(T, 1); /* all NUL bytes */
 	if (mask == NULL) {
 	    err = E_ALLOC;
 	} else {
@@ -989,6 +994,7 @@ MODEL tsls_func (const int *list, int ci, double ***pZ, DATAINFO *pdinfo,
     int *hatlist = NULL, *s2list = NULL;
     int *exolist = NULL;
     int *droplist = NULL;
+    int addconst = 0;
     int pos, orig_nvar = pdinfo->v;
     int rlen, ninst, ev = 0;
     int OverIdRank = 0;
@@ -1077,7 +1083,7 @@ MODEL tsls_func (const int *list, int ci, double ***pZ, DATAINFO *pdinfo,
     /* determine the list of variables (hatlist) for which we need to
        obtain fitted values in the first stage 
     */
-    tsls_make_hatlist(reglist, instlist, hatlist);
+    addconst = tsls_make_hatlist(reglist, instlist, hatlist);
 
     Q = tsls_Q(instlist, reglist, &droplist,
 	       (const double **) *pZ, pdinfo->t1, pdinfo->t2,
@@ -1165,7 +1171,7 @@ MODEL tsls_func (const int *list, int ci, double ***pZ, DATAINFO *pdinfo,
     tsls_residuals(&tsls, reglist, (const double **) *pZ, opt);
 
     /* compute standard errors */
-    if ((opt & OPT_R) || get_use_qr()) {
+    if ((opt & OPT_R) || libset_get_bool(USE_QR)) {
 	/* QR decomp in force, or robust standard errors called for */
 	qr_tsls_vcv(&tsls, (const double **) *pZ, pdinfo, opt);
 	if (tsls.errcode) {
@@ -1196,6 +1202,12 @@ MODEL tsls_func (const int *list, int ci, double ***pZ, DATAINFO *pdinfo,
 	gretl_model_set_list_as_data(&tsls, "inst_droplist", droplist); 
     }
 
+#if 0
+    if (addconst) {
+	gretl_model_set_int(&tsls, "addconst", addconst);
+    }
+#endif
+
  bailout:
 
     gretl_matrix_free(Q);
