@@ -1,20 +1,20 @@
-/*
- *  Copyright (c) by Allin Cottrell
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
- *
- *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- *
+/* 
+ *  gretl -- Gnu Regression, Econometrics and Time-series Library
+ *  Copyright (C) 2001 Allin Cottrell and Riccardo "Jack" Lucchetti
+ * 
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ * 
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ * 
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 #include "libgretl.h"
@@ -193,7 +193,7 @@ matrix_insert_diagonal (gretl_matrix *M, const gretl_matrix *S,
     return 0;
 }
 
-/* If slevel is LEVEL_AUTO search at the current function execution
+/* If slevel is LEVEL_AUTO, search at the current function execution
    depth, otherwise search at the function execution depth given by
    slevel.
 */
@@ -335,6 +335,34 @@ gretl_matrix *get_matrix_by_name (const char *name)
     return real_get_matrix_by_name(name, LEVEL_AUTO);
 }
 
+/**
+ * get_matrix_copy_by_name:
+ * @name: name of the matrix.
+ * @err: location to receive error code;
+ *
+ * Looks up a user-defined matrix by name.
+ *
+ * Returns: a copy of the named matrix, or %NULL if not found.
+ */
+
+gretl_matrix *get_matrix_copy_by_name (const char *name, int *err)
+{
+    gretl_matrix *m;
+
+    m = real_get_matrix_by_name(name, LEVEL_AUTO);
+
+    if (m == NULL) {
+	*err = E_UNKVAR;
+    } else {
+	m = gretl_matrix_copy(m);
+	if (m == NULL) {
+	    *err = E_ALLOC;
+	}
+    }
+
+    return m;
+}
+
 /**
  * get_matrix_by_name_at_level:
  * @name: name of the matrix.
@@ -635,6 +663,11 @@ gretl_matrix *matrix_get_submatrix (const gretl_matrix *M,
     int *cslice = NULL;
     int r, c;
 
+    if (gretl_is_null_matrix(M)) {
+	*err = E_DATA;
+	return NULL;
+    }
+
     if (spec->type[0] == SEL_DIAG) {
 	return gretl_matrix_get_diagonal(M, err);
     }
@@ -675,6 +708,10 @@ gretl_matrix *matrix_get_submatrix (const gretl_matrix *M,
 	}
     }
 
+    if (S != NULL && S->rows == M->rows) {
+	S->t1 = M->t1;
+    }
+
     free(rslice);
     free(cslice);
 
@@ -885,7 +922,7 @@ int user_matrix_destroy (user_matrix *u)
 	for (i=0; i<n_matrices; i++) {
 	    if (matrices[i] == u) {
 		destroy_user_matrix(matrices[i]);
-		for (j=i; j<n_matrices - 1; j++) {
+		for (j=i; j<nm; j++) {
 		    matrices[j] = matrices[j+1];
 		}
 		matrices[nm] = NULL;
@@ -899,7 +936,6 @@ int user_matrix_destroy (user_matrix *u)
 	} else {
 	    user_matrix **tmp = realloc(matrices, nm * sizeof *tmp);
 
-	    n_matrices = nm;
 	    if (tmp == NULL) {
 		err = E_ALLOC;
 	    } else {
@@ -907,7 +943,7 @@ int user_matrix_destroy (user_matrix *u)
 	    }
 	}
 
-	n_matrices--;
+	n_matrices = nm;
     }
 
     return err;
@@ -933,7 +969,7 @@ real_user_matrix_get_determinant (const gretl_matrix *m, int log, int *err)
 {
     double d = NADBL;
 
-    if (m != NULL) {
+    if (!gretl_is_null_matrix(m)) {
 	gretl_matrix *tmp = gretl_matrix_copy(m);
 
 	if (tmp != NULL) {
@@ -959,27 +995,43 @@ double user_matrix_get_log_determinant (const gretl_matrix *m, int *err)
     return real_user_matrix_get_determinant(m, 1, err);
 }
 
-gretl_matrix *user_matrix_get_inverse (const gretl_matrix *m)
+static gretl_matrix *
+real_user_matrix_get_inverse (const gretl_matrix *m, int moore, int *err)
 {
     gretl_matrix *R = NULL;
 
-    if (m != NULL) {
+    if (gretl_is_null_matrix(m)) {
+	*err = E_DATA;
+    } else {
 	R = gretl_matrix_copy(m);
-	if (R != NULL) {
-	    if (gretl_invert_matrix(R)) {
+	if (R == NULL) {
+	    *err = E_ALLOC;
+	} else {
+	    if (moore) {
+		*err = gretl_matrix_moore_penrose(R);
+	    } else {
+		*err = gretl_invert_matrix(R);
+	    } 
+	    if (*err) {
 		gretl_matrix_free(R);
 		R = NULL;
 	    }
 	} 
-    }
-
-    if (R == NULL) {
-   	strcpy(gretl_errmsg, _("Matrix inversion failed"));
-    }
+    } 
    
     return R;
 }
 
+gretl_matrix *user_matrix_get_inverse (const gretl_matrix *m, int *err)
+{
+    return real_user_matrix_get_inverse(m, 0, err);
+}
+
+gretl_matrix *user_matrix_moore_penrose (const gretl_matrix *m, int *err)
+{
+    return real_user_matrix_get_inverse(m, 1, err);
+}
+
 static void matrix_cannibalize (gretl_matrix *targ, gretl_matrix *src)
 {
     targ->rows = src->rows;
@@ -1069,34 +1121,40 @@ int matrix_XTX_in_place (gretl_matrix *m)
     return err;
 }
 
-gretl_matrix *user_matrix_cholesky_decomp (const gretl_matrix *m)
+gretl_matrix *user_matrix_cholesky_decomp (const gretl_matrix *m,
+					   int *err)
 {
     gretl_matrix *R = NULL;
 
-    if (m != NULL) {
+    if (gretl_is_null_matrix(m)) {
+	*err = E_DATA;
+    } else {
 	R = gretl_matrix_copy(m);
-	if (R != NULL) {
-	    if (gretl_matrix_cholesky_decomp(R)) {
+	if (R == NULL) {
+	    *err = E_ALLOC;
+	} else {
+	    *err = gretl_matrix_cholesky_decomp(R);
+	    if (*err) {
 		gretl_matrix_free(R);
 		R = NULL;
 	    }
 	} 
     }
 
-    if (R == NULL) {
-	strcpy(gretl_errmsg, _("Matrix decomposition failed"));
-    }
-
     return R;
 }
 
-gretl_matrix *user_matrix_column_demean (const gretl_matrix *m)
+gretl_matrix *user_matrix_column_demean (const gretl_matrix *m, int *err)
 {
     gretl_matrix *R = NULL;
 
-    if (m != NULL) {
+    if (gretl_is_null_matrix(m)) {
+	*err = E_DATA;
+    } else {
 	R = gretl_matrix_copy(m);
-	if (R != NULL) {
+	if (R == NULL) {
+	    *err = E_ALLOC;
+	} else {
 	    gretl_matrix_demean_by_column(R);
 	} 
     }
@@ -1104,17 +1162,23 @@ gretl_matrix *user_matrix_column_demean (const gretl_matrix *m)
     return R;
 }
 
-gretl_matrix *user_matrix_vec (const gretl_matrix *m)
+gretl_matrix *user_matrix_vec (const gretl_matrix *m, int *err)
 {
     gretl_matrix *R = NULL;
 
-    if (m != NULL) {
+    if (gretl_is_null_matrix(m)) {
+	R = gretl_null_matrix_new();
+    } else {
 	R = gretl_matrix_alloc(m->rows * m->cols, 1);
 	if (R != NULL) {
 	    gretl_matrix_vectorize(R, m);
 	} 
     }
 
+    if (R == NULL) {
+	*err = E_ALLOC;
+    }
+
     return R;
 }
 
@@ -1122,18 +1186,22 @@ gretl_matrix *user_matrix_vech (const gretl_matrix *m, int *err)
 {
     gretl_matrix *R = NULL;
 
-    if (m != NULL) {
-	if (m->rows != m->cols) {
-	    *err = E_NONCONF;
-	} else {
-	    int n = m->rows;
-	    int k = n * (n + 1) / 2;
+    if (gretl_is_null_matrix(m)) {
+	R = gretl_null_matrix_new();
+    } else if (m->rows != m->cols) {
+	*err = E_NONCONF;
+    } else {
+	int n = m->rows;
+	int k = n * (n + 1) / 2;
 
-	    R = gretl_matrix_alloc(k, 1);
-	    if (R != NULL) {
-		*err = gretl_matrix_vectorize_h(R, m);
-	    }
-	} 
+	R = gretl_matrix_alloc(k, 1);
+	if (R != NULL) {
+	    *err = gretl_matrix_vectorize_h(R, m);
+	}
+    } 
+
+    if (R == NULL && !*err) {
+	*err = E_ALLOC;
     }
 
     return R;
@@ -1142,16 +1210,40 @@ gretl_matrix *user_matrix_vech (const gretl_matrix *m, int *err)
 gretl_matrix *user_matrix_unvech (const gretl_matrix *m, int *err)
 {
     gretl_matrix *R = NULL;
-    int n;
 
-    if (m != NULL && m->cols == 1) {
-	n = (int) ((sqrt(1.0 + 8.0 * m->rows) - 1.0) / 2.0);
+    if (gretl_is_null_matrix(m)) {
+	R = gretl_null_matrix_new();
+    } else if (m->cols != 1) {
+	*err = E_NONCONF;
+    } else {
+	int n = (int) ((sqrt(1.0 + 8.0 * m->rows) - 1.0) / 2.0);
+
 	R = gretl_matrix_alloc(n, n);
 	if (R != NULL) {
 	    *err = gretl_matrix_unvectorize_h(R, m);
 	} 
     }
 
+    if (R == NULL && !*err) {
+	*err = E_ALLOC;
+    }
+
+    return R;
+}
+
+gretl_matrix *user_matrix_upper (const gretl_matrix *m, int *err)
+{
+    gretl_matrix *R = gretl_matrix_copy(m);
+
+    *err = gretl_matrix_zero_lower(R);
+    return R;
+}
+
+gretl_matrix *user_matrix_lower (const gretl_matrix *m, int *err)
+{
+    gretl_matrix *R = gretl_matrix_copy(m);
+
+    *err = gretl_matrix_zero_upper(R);
     return R;
 }
 
@@ -1197,7 +1289,7 @@ user_matrix_QR_decomp (const gretl_matrix *m, const char *rname, int *err)
     gretl_matrix *R = NULL;
     int wantR = 0;
 
-    if (m == NULL) {
+    if (gretl_is_null_matrix(m)) {
 	*err = E_DATA;
 	return NULL;
     }
@@ -1220,6 +1312,34 @@ user_matrix_QR_decomp (const gretl_matrix *m, const char *rname, int *err)
     return Q;
 }
 
+#define SVD_RESIZE 1
+
+#if SVD_RESIZE
+static int revise_SVD_V (gretl_matrix **pV, int r, int c)
+{
+    gretl_matrix *V;
+    double x;
+    int i, j;
+
+    V = gretl_matrix_alloc(r, c);
+    if (V == NULL) {
+	return E_ALLOC;
+    }
+
+    for (i=0; i<r; i++) {
+	for (j=0; j<c; j++) {
+	    x = gretl_matrix_get((*pV), i, j);
+	    gretl_matrix_set(V, i, j, x);
+	}
+    }
+
+    gretl_matrix_free(*pV);
+    *pV = V;
+
+    return 0;
+}
+#endif
+
 gretl_matrix *user_matrix_SVD (const gretl_matrix *m, 
 			       const char *uname, 
 			       const char *vname, 
@@ -1230,10 +1350,8 @@ gretl_matrix *user_matrix_SVD (const gretl_matrix *m,
     gretl_matrix *V = NULL;
     gretl_matrix **pU = NULL;
     gretl_matrix **pV = NULL;
-    int wantU = 0;
-    int wantV = 0;
 
-    if (m == NULL) {
+    if (gretl_is_null_matrix(m)) {
 	*err = E_DATA;
 	return NULL;
     }
@@ -1242,7 +1360,6 @@ gretl_matrix *user_matrix_SVD (const gretl_matrix *m,
 	if (get_matrix_by_name(uname) == NULL) {
 	    *err = E_UNKVAR;
 	} else {
-	    wantU = 1;
 	    pU = &U;
 	}
     }
@@ -1251,7 +1368,6 @@ gretl_matrix *user_matrix_SVD (const gretl_matrix *m,
 	if (get_matrix_by_name(vname) == NULL) {
 	    *err = E_UNKVAR;
 	} else {
-	    wantV = 1;
 	    pV = &V;
 	}
     }
@@ -1260,18 +1376,101 @@ gretl_matrix *user_matrix_SVD (const gretl_matrix *m,
 	*err = gretl_matrix_SVD(m, pU, &S, pV);
     }
 
+#if SVD_RESIZE
+    if (!*err && (U != NULL || V != NULL)) {
+	int tall = m->rows - m->cols;
+	int minrc = (m->rows > m->cols)? m->cols : m->rows;
+
+	if (U != NULL) {
+	    if (tall > 0) {
+		*err = gretl_matrix_realloc(U, m->rows, minrc);
+	    }
+	    if (!*err) {
+		user_matrix_replace_matrix_by_name(uname, U);
+	    }
+	}
+	if (V != NULL) {
+	    if (tall < 0) {
+		*err = revise_SVD_V(&V, minrc, m->cols);
+	    } 
+	    if (!*err) {
+		user_matrix_replace_matrix_by_name(vname, V);
+	    }
+	}
+    }
+#else
     if (!*err) {
-	if (wantU) {
+	if (U != NULL) {
 	    user_matrix_replace_matrix_by_name(uname, U);
 	}
-	if (wantV) {
+	if (V != NULL) {
 	    user_matrix_replace_matrix_by_name(vname, V);
 	}
     }
+#endif
 
     return S;
 }
 
+gretl_matrix *user_matrix_ols (const gretl_matrix *Y, 
+			       const gretl_matrix *X, 
+			       const char *Uname, 
+			       int *err)
+{
+    gretl_matrix *B = NULL;
+    gretl_matrix *U = NULL;
+    int newU = 0;
+
+    if (gretl_is_null_matrix(X) || gretl_is_null_matrix(X)) {
+	*err = E_DATA;
+	return NULL;
+    }
+
+    if (X->rows != Y->rows) {
+	*err = E_NONCONF;
+	return NULL;
+    }
+
+    if (Uname != NULL && strcmp(Uname, "null")) {
+	U = get_matrix_by_name(Uname);
+	if (U == NULL) {
+	    *err = E_UNKVAR;
+	    return NULL;
+	} 
+	if (U->rows != Y->rows || U->cols != Y->cols) {
+	    newU = 1;
+	}
+    }
+
+    if (newU) {
+	U = gretl_matrix_alloc(Y->rows, Y->cols);
+	if (U == NULL) {
+	    *err = E_ALLOC;
+	    return NULL;
+	}
+    }
+
+    B = gretl_matrix_alloc(X->cols, Y->cols);
+    if (B == NULL) {
+	*err = E_ALLOC;
+    }
+
+    if (!*err) {
+	*err = gretl_matrix_multi_ols(Y, X, B, U, NULL);
+    }
+
+    if (*err) {
+	gretl_matrix_free(B);
+	if (newU) {
+	    gretl_matrix_free(U);
+	}
+    } else if (newU) {
+	user_matrix_replace_matrix_by_name(Uname, U);
+    }
+
+    return B;
+}
+
 gretl_matrix *
 user_matrix_eigen_analysis (const gretl_matrix *m, const char *rname, int symm,
 			    int *err)
@@ -1280,7 +1479,7 @@ user_matrix_eigen_analysis (const gretl_matrix *m, const char *rname, int symm,
     gretl_matrix *E = NULL;
     int vecs = 0;
 
-    if (m == NULL) {
+    if (gretl_is_null_matrix(m)) {
 	*err = E_DATA;
 	return NULL;
     }
