@@ -1,20 +1,20 @@
-/* gretl - The Gnu Regression, Econometrics and Time-series Library
- * Copyright (C) 1999-2000 Ramu Ramanathan and Allin Cottrell
- *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License 
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
- *
- * This software is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License 
- * along with this software; if not, write to the 
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+/* 
+ *  gretl -- Gnu Regression, Econometrics and Time-series Library
+ *  Copyright (C) 2001 Allin Cottrell and Riccardo "Jack" Lucchetti
+ * 
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ * 
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ * 
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 #ifndef VAR_H_
@@ -22,15 +22,16 @@
 
 #include "gretl_matrix.h"
 #include "johansen.h"
+#include "gretl_restrict.h"
 
 int var_max_order (const int *list, const DATAINFO *pdinfo);
 
 GRETL_VAR *gretl_VAR (int order, int *list, 
-		      double ***pZ, DATAINFO *pdinfo,
+		      const double **Z, const DATAINFO *pdinfo,
 		      gretlopt opt, PRN *prn, int *err);
 
 GRETL_VAR *gretl_VECM (int order, int rank, int *list, 
-		       double ***pZ, DATAINFO *pdinfo,
+		       const double **Z, const DATAINFO *pdinfo,
 		       gretlopt opt, PRN *prn, int *err);
 
 const gretl_matrix *
@@ -55,12 +56,17 @@ int gretl_VAR_arch_test (GRETL_VAR *var, int order,
 
 int gretl_VAR_normality_test (const GRETL_VAR *var, PRN *prn);
 
-int *gretl_VAR_get_exo_list (const GRETL_VAR *var, int *err);
+int set_VAR_model_stats (GRETL_VAR *var, int i);
+
+const int *gretl_VAR_get_exo_list (const GRETL_VAR *var);
 
 GRETL_VAR *gretl_VAR_omit_test (const int *omitvars, const GRETL_VAR *orig, 
-				double ***pZ, DATAINFO *pdinfo, 
+				const double **Z, DATAINFO *pdinfo, 
 				PRN *prn, int *err);
 
+double *gretl_VECM_get_EC (GRETL_VAR *vecm, int j, const double **Z, 
+			   const DATAINFO *pdinfo, int *err);
+
 void gretl_VAR_free (GRETL_VAR *var);
 
 int default_VAR_horizon (const DATAINFO *pdinfo);
@@ -74,8 +80,6 @@ gretl_VAR_get_impulse_response (GRETL_VAR *var,
 
 void gretl_VAR_set_name (GRETL_VAR *var, const char *name);
 
-int gretl_VAR_attach_restrictions (GRETL_VAR *var, gretl_matrix *D);
-
 const char *gretl_VAR_get_name (const GRETL_VAR *var);
 
 int gretl_VAR_get_variable_number (const GRETL_VAR *var, int k);
@@ -94,27 +98,59 @@ int gretl_VAR_add_resids_to_dataset (GRETL_VAR *var, int eqnum,
 int gretl_VAR_do_irf (GRETL_VAR *var, const char *line,
 		      const double **Z, const DATAINFO *pdinfo);
 
-int gretl_VAR_get_highest_variable (const GRETL_VAR *var,
-				    const DATAINFO *pdinfo);
+int gretl_VAR_get_highest_variable (const GRETL_VAR *var);
 
 int gretl_VECM_n_beta (const GRETL_VAR *vecm);
 
-int gretl_VECM_test_beta (GRETL_VAR *vecm, const DATAINFO *pdinfo, 
-			  PRN *prn);
+int gretl_VECM_n_alpha (const GRETL_VAR *vecm);
+
+int gretl_VECM_test (GRETL_VAR *vecm, 
+		     gretl_restriction *rset,
+		     const DATAINFO *pdinfo, 
+		     gretlopt opt,
+		     PRN *prn);
+
+GRETL_VAR *
+real_gretl_restricted_vecm (GRETL_VAR *orig, 
+			    gretl_restriction *rset,
+			    const double **Z, const DATAINFO *pdinfo, 
+			    PRN *prn, int *err);
 
 int gretl_VECM_rank (const GRETL_VAR *vecm);
 
 const int *gretl_VECM_list (const GRETL_VAR *vecm);
 
+int beta_restricted_VECM (const GRETL_VAR *vecm);
+
+int alpha_restricted_VECM (const GRETL_VAR *vecm);
+
+int restricted_VECM (const GRETL_VAR *vecm);
+
+const gretl_matrix *gretl_VECM_R_matrix (const GRETL_VAR *vecm);
+
+const gretl_matrix *gretl_VECM_q_matrix (const GRETL_VAR *vecm);
+
+const gretl_matrix *gretl_VECM_Ra_matrix (const GRETL_VAR *vecm);
+
+const gretl_matrix *gretl_VECM_qa_matrix (const GRETL_VAR *vecm);
+
 double *gretl_VAR_get_series (const GRETL_VAR *var, const DATAINFO *pdinfo, 
 			      int idx, const char *key, int *err);
 
 gretl_matrix *gretl_VAR_get_matrix (const GRETL_VAR *var, int idx, 
 				    int *err);
 
+void gretl_VAR_param_names (GRETL_VAR *v, char **params, 
+			    const DATAINFO *pdinfo);
+
 int gretl_VAR_serialize (const GRETL_VAR *var, SavedObjectFlags flags,
 			 FILE *fp);
 
+int transcribe_VAR_models (GRETL_VAR *var, 
+			   const double **Z,
+			   const DATAINFO *pdinfo,
+			   const gretl_matrix *XTX);
+
 #ifndef GRETLCLI
 
 GRETL_VAR *gretl_VAR_from_XML (xmlNodePtr node, xmlDocPtr doc, int *err);
