@@ -1,20 +1,20 @@
-/*
- *  Copyright (c) by Allin Cottrell
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
- *
- *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111, USA.
- *
+/* 
+ *  gretl -- Gnu Regression, Econometrics and Time-series Library
+ *  Copyright (C) 2001 Allin Cottrell and Riccardo "Jack" Lucchetti
+ * 
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ * 
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ * 
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 #include "libgretl.h" 
@@ -226,11 +226,7 @@ static int varprint_namelen (const GRETL_VAR *var, const DATAINFO *pdinfo,
 	if (k >= var->neqns) {
 	    break;
 	}
-	if (var->ci == VECM) {
-	    v = var->jinfo->list[k + 1];
-	} else {
-	    v = (var->models[k])->list[1];
-	}
+	v = var->ylist[k+1];
 	len = strlen(pdinfo->varname[v]);
 	if (len > maxlen) {
 	    maxlen = len;
@@ -264,14 +260,13 @@ gretl_VAR_print_impulse_response (GRETL_VAR *var, int shock,
 				  int periods, const DATAINFO *pdinfo, 
 				  int pause, PRN *prn)
 {
-    int i, t;
-    int vsrc;
-    int rows = var->neqns * (var->order + var->ecm);
     gretl_matrix *rtmp, *ctmp;
+    int rows = var->neqns * effective_order(var);
     int block, blockmax;
     int tex = tex_format(prn);
     int rtf = rtf_format(prn);
-    int err = 0;
+    int vsrc;
+    int i, t, err = 0;
 
     if (prn == NULL) {
 	return 0;
@@ -293,11 +288,7 @@ gretl_VAR_print_impulse_response (GRETL_VAR *var, int shock,
 	return E_ALLOC;
     }
 
-    if (var->ci == VECM) {
-	vsrc = var->jinfo->list[shock + 1];
-    } else {
-	vsrc = (var->models[shock])->list[1];
-    }
+    vsrc = var->ylist[shock + 1];
 
     blockmax = var->neqns / IRF_ROW_MAX;
     if (var->neqns % IRF_ROW_MAX) {
@@ -319,11 +310,7 @@ gretl_VAR_print_impulse_response (GRETL_VAR *var, int shock,
 	    if (k >= var->neqns) {
 		break;
 	    }
-	    if (var->ci == VECM) {
-		vtarg = var->jinfo->list[k + 1];
-	    } else {
-		vtarg = (var->models[k])->list[1];
-	    }
+	    vtarg = var->ylist[k+1];
 	    endrow = !(i < IRF_ROW_MAX - 1 && k < var->neqns - 1);
 	    VAR_info_print_vname(i, vtarg, endrow, width, pdinfo, prn);
 	}
@@ -451,11 +438,7 @@ gretl_VAR_print_fcast_decomp (GRETL_VAR *var, int targ,
 	return E_ALLOC;
     }
 
-    if (var->ci == VECM) {
-	vtarg = var->jinfo->list[targ + 1];
-    } else {
-	vtarg = (var->models[targ])->list[1];
-    }
+    vtarg = var->ylist[targ + 1];
 
     blockmax = (var->neqns + 1) / VDC_ROW_MAX;
     if ((var->neqns + 1) % VDC_ROW_MAX) {
@@ -487,11 +470,7 @@ gretl_VAR_print_fcast_decomp (GRETL_VAR *var, int targ,
 	    if (k >= var->neqns) {
 		break;
 	    }
-	    if (var->ci == VECM) {
-		vsrc = var->jinfo->list[k + 1];
-	    } else {
-		vsrc = (var->models[k])->list[1];
-	    }
+	    vsrc = var->ylist[k+1];
 	    endrow = !(i < VDC_ROW_MAX - 1 && k < var->neqns - 1);
 	    VAR_info_print_vname(i, vsrc, endrow, width, pdinfo, prn);
 	}
@@ -597,44 +576,80 @@ void print_Johansen_test_case (JohansenCode jcode, PRN *prn)
     }
 }
 
-static void 
-print_VECM_coint_eqns (JohansenInfo *jv, const DATAINFO *pdinfo, PRN *prn)
+static char *make_beta_vname (char *vname,
+			      const GRETL_VAR *v,
+			      const DATAINFO *pdinfo,
+			      int i)
 {
+    if (i < v->neqns) {
+	strcpy(vname, pdinfo->varname[v->ylist[i+1]]);
+    } else if (auto_restr(v) && i == v->neqns) {
+	strcpy(vname, (jcode(v) == J_REST_CONST)? "const" : "trend");
+    } else if (v->rlist != NULL) {
+	int k = i - v->ylist[0] - auto_restr(v) + 1;
+
+	strcpy(vname, pdinfo->varname[v->rlist[k]]);
+    } 
+
+    return vname;
+}
+
+static int max_beta_namelen (GRETL_VAR *v, 
+			     const DATAINFO *pdinfo)
+{
+    int r = gretl_matrix_rows(v->jinfo->Beta);
+    char s[32];
+    int i, ni, n = 0;
+
+    for (i=0; i<r; i++) {    
+	make_beta_vname(s, v, pdinfo, i);
+	ni = strlen(s);
+	if (ni > n) {
+	    n = ni;
+	}
+    }
+
+    return n;
+}
+
+static void 
+print_VECM_coint_eqns (GRETL_VAR *jvar, 
+		       const DATAINFO *pdinfo, 
+		       PRN *prn)
+{
+    JohansenInfo *jv = jvar->jinfo;
     int rtf = rtf_format(prn);
-    char s[16];
+    char namefmt[8];
+    char s[16], vname[32];
     int rows = gretl_matrix_rows(jv->Beta);
+    int nwid;
     int i, j;
     double x;
 
-    pputs(prn, _("Cointegrating vectors"));
+    pprintf(prn, "beta (%s", _("cointegrating vectors"));
     if (jv->Bse != NULL) {
-	pprintf(prn, " (%s)", _("standard errors in parentheses"));
-    } 
+	pprintf(prn, ", %s)", _("standard errors in parentheses"));
+    } else {
+	pputc(prn, ')');
+    }
+
     gretl_prn_newline(prn);
     gretl_prn_newline(prn);
 
+    nwid = max_beta_namelen(jvar, pdinfo) + 1;
+    sprintf(namefmt, "%%-%ds", nwid);
+
     for (i=0; i<rows; i++) {
-	char vname[32];
-
-	if (i < jv->list[0]) {
-	    sprintf(vname, "%s(-1)", pdinfo->varname[jv->list[i+1]]);
-	} else if (jv->code == J_REST_CONST) {
-	    strcpy(vname, "const");
-	} else if (jv->code == J_REST_TREND) {
-	    strcpy(vname, "trend");
-	}
+	make_beta_vname(vname, jvar, pdinfo, i);
 	if (rtf) {
 	    pputs(prn, vname);
 	} else {
-	    pprintf(prn, "%-12s", vname); /* FIXME */
+	    pprintf(prn, namefmt, vname);
 	}
 
 	/* coefficients */
 	for (j=0; j<jv->rank; j++) {
 	    x = gretl_matrix_get(jv->Beta, i, j);
-	    if (jv->Bse == NULL) {
-		x /= gretl_matrix_get(jv->Beta, j, j);
-	    }
 	    if (rtf) {
 		pprintf(prn, "\t%#.5g ", x);
 	    } else {
@@ -648,14 +663,61 @@ print_VECM_coint_eqns (JohansenInfo *jv, const DATAINFO *pdinfo, PRN *prn)
 	    if (rtf) {
 		pputs(prn, "\t");
 	    } else {
-		bufspace(VECM_WIDTH, prn);
+		bufspace(nwid + 1, prn);
 	    }
 	    for (j=0; j<jv->rank; j++) {
-		if (i < jv->rank) {
-		    x = 0.0;
+		x = gretl_matrix_get(jv->Bse, i, j);
+		sprintf(s, "(%#.5g)", x);
+		if (rtf) {
+		    pprintf(prn, "\t%s", s);
 		} else {
-		    x = gretl_matrix_get(jv->Bse, i - jv->rank, j);
+		    pprintf(prn, "%12s ", s);
 		}
+	    }
+	    gretl_prn_newline(prn);
+	}
+    }
+
+    gretl_prn_newline(prn);
+
+    rows = gretl_matrix_rows(jv->Alpha);
+
+    pprintf(prn, "alpha (%s", _("adjustment vectors"));
+    if (jv->Ase != NULL) {
+	pprintf(prn, ", %s)", _("standard errors in parentheses"));
+    } else {
+	pputc(prn, ')');
+    }
+
+    gretl_prn_newline(prn);
+    gretl_prn_newline(prn);
+
+    for (i=0; i<rows; i++) {
+	sprintf(vname, "%s", pdinfo->varname[jvar->ylist[i+1]]);
+	if (rtf) {
+	    pputs(prn, vname);
+	} else {
+	    pprintf(prn, namefmt, vname);
+	}
+
+	for (j=0; j<jv->rank; j++) {
+	    x = gretl_matrix_get(jv->Alpha, i, j);
+	    if (rtf) {
+		pprintf(prn, "\t%#.5g ", x);
+	    } else {
+		pprintf(prn, "%#12.5g ", x);
+	    }
+	}
+	gretl_prn_newline(prn);
+
+	if (jv->Ase != NULL) {
+	    if (rtf) {
+		pputs(prn, "\t");
+	    } else {
+		bufspace(nwid + 1, prn);
+	    }
+	    for (j=0; j<jv->rank; j++) {
+		x = gretl_matrix_get(jv->Ase, i, j);
 		sprintf(s, "(%#.5g)", x);
 		if (rtf) {
 		    pprintf(prn, "\t%s", s);
@@ -670,49 +732,77 @@ print_VECM_coint_eqns (JohansenInfo *jv, const DATAINFO *pdinfo, PRN *prn)
     gretl_prn_newline(prn);
 }
 
+static int max_vlen (const int *list, const DATAINFO *pdinfo)
+{
+    int i, ni, n = 0;
+
+    for (i=1; i<=list[0]; i++) {
+	ni = strlen(pdinfo->varname[list[i]]);
+	if (ni > n) {
+	    n = ni;
+	}
+    }
+
+    return n;
+}
+
 static void print_VECM_omega (GRETL_VAR *jvar, const DATAINFO *pdinfo, PRN *prn)
 {
     int rtf = rtf_format(prn);
-    int *list = jvar->jinfo->list;
+    int *list = jvar->ylist;
     char s[32];
+    int w0, wi = 12;
     int i, j;
 
     pprintf(prn, "%s\n", _("Cross-equation covariance matrix"));
     gretl_prn_newline(prn);
 
+    w0 = max_vlen(list, pdinfo) + 1;
+
+    /* top row: names of Y variables */
+
     for (i=0; i<jvar->neqns; i++) {
-	sprintf(s, "d_%s", pdinfo->varname[list[i+1]]);
+	sprintf(s, "%s", pdinfo->varname[list[i+1]]);
 	if (i == 0) {
 	    if (rtf) {
 		pprintf(prn, "\t\t%s", s);
 	    } else {
-		pprintf(prn, "%25s", s);
+		wi = strlen(s);
+		if (wi < VECM_WIDTH) wi = VECM_WIDTH;
+		pprintf(prn, "%*s", w0 + wi, s);
 	    }
 	} else {
 	    if (rtf) {
 		pprintf(prn, "\t%s", s);
 	    } else {
-		pprintf(prn, "%*s", VECM_WIDTH, s);
+		wi = strlen(s) + 1;
+		if (wi < VECM_WIDTH) wi = VECM_WIDTH;
+		pprintf(prn, "%*s", wi, s);
 	    }
 	}
     }
+
     gretl_prn_newline(prn);
 
+    /* subsequent rows: Y name plus values */
+
     for (i=0; i<jvar->neqns; i++) {
-	sprintf(s, "d_%s", pdinfo->varname[list[i+1]]);
+	sprintf(s, "%s", pdinfo->varname[list[i+1]]);
 	if (rtf) {
 	    pputs(prn, s);
 	    if (strlen(s) < 8) {
 		pputc(prn, '\t');
 	    }	    
 	} else {
-	    pprintf(prn, "%-*s", VECM_WIDTH, s);
+	    pprintf(prn, "%-*s ", w0, s);
 	}
 	for (j=0; j<jvar->neqns; j++) {
 	    if (rtf) {
 		pprintf(prn, "\t%#.5g", gretl_matrix_get(jvar->S, i, j));
 	    } else {
-		pprintf(prn, "%#12.5g ", gretl_matrix_get(jvar->S, i, j));
+		wi = strlen(pdinfo->varname[list[j+1]]);
+		if (wi < VECM_WIDTH - 1) wi = VECM_WIDTH - 1;
+		pprintf(prn, "%#*.5g ", wi, gretl_matrix_get(jvar->S, i, j));
 	    }
 	}
 	gretl_prn_newline(prn);
@@ -724,9 +814,104 @@ static void print_VECM_omega (GRETL_VAR *jvar, const DATAINFO *pdinfo, PRN *prn)
     gretl_prn_newline(prn);
 }
 
-static void 
-print_vecm_header_info (GRETL_VAR *vecm, PRN *prn)
+/* FIXME TeX and RTF */
+
+static void vecm_print_restrictions (GRETL_VAR *vecm, PRN *prn)
 {
+    if (vecm->jinfo->R != NULL) {
+	pputs(prn, "\n\n");
+	pputs(prn, _("Restrictions on beta:"));
+	pputc(prn, '\n');
+	print_restriction_from_matrices(vecm->jinfo->R, vecm->jinfo->q, 
+					'b', gretl_VECM_n_beta(vecm), 
+					prn);
+	pputc(prn, '\n');
+    }
+
+    if (vecm->jinfo->Ra != NULL) {
+	if (vecm->jinfo->R == NULL) {
+	    pputs(prn, "\n\n");
+	}
+	pputs(prn, _("Restrictions on alpha:"));
+	pputc(prn, '\n');
+	print_restriction_from_matrices(vecm->jinfo->Ra, vecm->jinfo->qa, 
+					'a', gretl_VECM_n_alpha(vecm), 
+					prn);
+	pputc(prn, '\n');
+    }	  
+}
+
+static void print_LR_stat (double x, int df, PRN *prn)
+{
+    double pv = chisq_cdf_comp(x, df);
+
+    if (tex_format(prn)) {
+	pprintf(prn, "$2 (l_u - l_r) = %g$", x);
+	gretl_prn_newline(prn);
+	pprintf(prn, "$P(\\chi^2_{%d} > %g) = %g$", df, x, pv);
+    } else if (rtf_format(prn)) {
+	pprintf(prn, "2 * (lu - lr) = %g", x);
+	gretl_prn_newline(prn);
+	pprintf(prn, I_("P(Chi-Square(%d) > %g) = %g"), df, x, pv);
+    } else {
+	pprintf(prn, "2 * (lu - lr) = %g", x);
+	gretl_prn_newline(prn);
+	pprintf(prn, _("P(Chi-Square(%d) > %g) = %g"), df, x, pv);
+    }
+}
+
+enum {
+    LR_TOTAL,
+    LR_RELATIVE
+};
+
+static void 
+vecm_print_LR_test (GRETL_VAR *vecm, PRN *prn, int code)
+{
+    double ll0, x;
+    int df;
+
+    if (code == LR_RELATIVE) {
+	ll0 = vecm->jinfo->prior_ll;
+	df = vecm->jinfo->lrdf - vecm->jinfo->prior_df;
+	gretl_prn_newline(prn);
+	if (tex_format(prn) || rtf_format(prn)) {
+	    pputs(prn, I_("Relative to prior restriction"));
+	} else {
+	    pputs(prn, _("Relative to prior restriction"));
+	} 
+	pputc(prn, ':');
+	gretl_prn_newline(prn);
+    } else {
+	ll0 = vecm->jinfo->ll0;
+	df = vecm->jinfo->lrdf;
+    }
+
+    x = 2.0 * (ll0 - vecm->ll);
+
+    if (tex_format(prn)) {
+	pprintf(prn, I_("Unrestricted loglikelihood $(l_u) = %.8g$"), ll0);
+	gretl_prn_newline(prn);
+	pprintf(prn, I_("Restricted loglikelihood $(l_r) = %.8g$"), vecm->ll);
+    } else if (rtf_format(prn)) {
+	pprintf(prn, I_("Unrestricted loglikelihood (lu) = %.8g"), ll0);
+	gretl_prn_newline(prn);
+	pprintf(prn, I_("Restricted loglikelihood (lr) = %.8g"), vecm->ll);
+    } else {
+	pprintf(prn, _("Unrestricted loglikelihood (lu) = %.8g"), ll0);
+	gretl_prn_newline(prn);
+	pprintf(prn, _("Restricted loglikelihood (lr) = %.8g"), vecm->ll);
+    }
+
+    gretl_prn_newline(prn);
+    print_LR_stat(x, df, prn);
+    gretl_prn_newline(prn);
+}
+
+static void 
+print_vecm_header_info (GRETL_VAR *vecm, int *lldone, PRN *prn)
+{
+    JohansenInfo *J = vecm->jinfo;
     gretl_prn_newline(prn);
 
     if (vecm->jinfo == NULL) {
@@ -738,6 +923,44 @@ print_vecm_header_info (GRETL_VAR *vecm, PRN *prn)
 	    jrank(vecm));
     gretl_prn_newline(prn);
     print_Johansen_test_case(jcode(vecm), prn); 
+
+    if (J->R != NULL || J->Ra != NULL) {
+	vecm_print_restrictions(vecm, prn);
+	if (!na(J->ll0) && J->lrdf > 0) {
+	    vecm_print_LR_test(vecm, prn, LR_TOTAL);
+	    *lldone = 1;
+	} 
+	if (!na(J->prior_ll) && J->prior_df > 0) {
+	    vecm_print_LR_test(vecm, prn, LR_RELATIVE);
+	    *lldone = 1;
+	} 	
+	if (!*lldone) {
+	    pputc(prn, '\n');
+	}
+    } else {
+	pputc(prn, '\n');
+    }
+}
+
+static void VAR_print_LB_stat (const GRETL_VAR *var, PRN *prn)
+{
+    int k = var->order + (var->ci == VECM);
+    int df = var->neqns * var->neqns * (var->LBs - k);
+    double pv = chisq_cdf_comp(var->LB, df);
+
+    if (tex_format(prn)) {
+	pprintf(prn, "\\noindent\n%s: LB(%d) = %g (%s = %d, %s %f)\\par\n",
+		I_("Portmanteau test"), var->LBs, var->LB,
+		I_("df"), df, I_("p-value"), pv);	    
+    } else if (rtf_format(prn)) {
+	pprintf(prn, "%s: LB(%d) = %g (%s = %d, %s %f)\\par\n",
+		I_("Portmanteau test"), var->LBs, var->LB,
+		I_("df"), df, I_("p-value"), pv);
+    } else {
+	pprintf(prn, "%s: LB(%d) = %g (%s = %d, %s %f)\n",
+		_("Portmanteau test"), var->LBs, var->LB,
+		_("df"), df, _("p-value"), pv);
+    }
 }
 
 /**
@@ -760,11 +983,12 @@ int gretl_VAR_print (GRETL_VAR *var, const DATAINFO *pdinfo, gretlopt opt,
 {
     char startdate[OBSLEN], enddate[OBSLEN];
     char Vstr[72];
-    int vecm = var->ci == VECM;
+    int vecm = (var->ci == VECM);
     int dfd = var->models[0]->dfd;
     int tex = tex_format(prn);
     int rtf = rtf_format(prn);
     int quiet = (opt & OPT_Q);
+    int lldone = 0;
     int pause = 0;
     int i, j, k, v;
 
@@ -799,7 +1023,7 @@ int gretl_VAR_print (GRETL_VAR *var, const DATAINFO *pdinfo, gretlopt opt,
 	pprintf(prn, I_("%s estimates, observations %s--%s ($T=%d$)"),
 		(vecm)? I_("Maximum likelihood") : I_("OLS"), startdate, enddate, var->T);
 	if (vecm) {
-	    print_vecm_header_info(var, prn);
+	    print_vecm_header_info(var, &lldone, prn);
 	}
 	pputs(prn, "\n\\end{center}\n");
     } else if (rtf) {
@@ -808,7 +1032,7 @@ int gretl_VAR_print (GRETL_VAR *var, const DATAINFO *pdinfo, gretlopt opt,
 	pprintf(prn, I_("%s estimates, observations %s-%s (T = %d)"),
 		(vecm)? I_("Maximum likelihood") : I_("OLS"), startdate, enddate, var->T);
 	if (vecm) {
-	    print_vecm_header_info(var, prn);
+	    print_vecm_header_info(var, &lldone, prn);
 	}	
 	pputs(prn, "\\par\n\n");
     } else {
@@ -817,37 +1041,45 @@ int gretl_VAR_print (GRETL_VAR *var, const DATAINFO *pdinfo, gretlopt opt,
 	pprintf(prn, _("%s estimates, observations %s-%s (T = %d)"),
 		(vecm)? ("Maximum likelihood") : _("OLS"), startdate, enddate, var->T);
 	if (vecm) {
-	    print_vecm_header_info(var, prn);
+	    print_vecm_header_info(var, &lldone, prn);
 	}
-	pputs(prn, "\n\n");
+	pputc(prn, '\n');
     }
 
     if (vecm) {
 	if (tex_format(prn)) {
 	    tex_print_VECM_coint_eqns(var, pdinfo, prn);
 	} else {
-	    print_VECM_coint_eqns(var->jinfo, pdinfo, prn);
+	    print_VECM_coint_eqns(var, pdinfo, prn);
 	}
     }
 
     if (tex) {
 	tex_print_VAR_ll_stats(var, prn);
     } else if (rtf) {
-	pprintf(prn, "%s = %#g\\par\n", I_("Log-likelihood"), var->ll);
-	pprintf(prn, "%s = %#g\\par\n", I_("Determinant of covariance matrix"), 
+	if (!lldone) {
+	    pprintf(prn, "%s = %.8g\\par\n", I_("Log-likelihood"), var->ll);
+	}
+	pprintf(prn, "%s = %.8g\\par\n", I_("Determinant of covariance matrix"), 
 		exp(var->ldet));
 	pprintf(prn, "%s = %.4f\\par\n", I_("AIC"), var->AIC);
 	pprintf(prn, "%s = %.4f\\par\n", I_("BIC"), var->BIC);
 	pprintf(prn, "%s = %.4f\\par\n", I_("HQC"), var->HQC);
     } else {
-	pprintf(prn, "%s = %#g\n", _("Log-likelihood"), var->ll);
-	pprintf(prn, "%s = %#g\n", _("Determinant of covariance matrix"), exp(var->ldet));
+	if (!lldone) {
+	    pprintf(prn, "%s = %.8g\n", _("Log-likelihood"), var->ll);
+	}
+	pprintf(prn, "%s = %.8g\n", _("Determinant of covariance matrix"), exp(var->ldet));
 	pprintf(prn, "%s = %.4f\n", _("AIC"), var->AIC);
 	pprintf(prn, "%s = %.4f\n", _("BIC"), var->BIC);
 	pprintf(prn, "%s = %.4f\n", _("HQC"), var->HQC);
     }
 
-    if (vecm) {
+    if (var->LBs > 0 && !na(var->LB)) {
+	VAR_print_LB_stat(var, prn);
+    }
+
+    if (vecm && !quiet) {
 	pputc(prn, '\n');
     }
 
@@ -856,10 +1088,10 @@ int gretl_VAR_print (GRETL_VAR *var, const DATAINFO *pdinfo, gretlopt opt,
     for (i=0; i<var->neqns; i++) {
 	char Fstr[24];
 
-	if(!quiet) {
+	if (!quiet) {
 	    printmodel(var->models[i], pdinfo, OPT_NONE, prn);
 	} else {
-	    if (!var->ecm) {
+	    if (var->ci != VECM) {
 	        v = var->models[i]->list[1];
 		if (tex) {
 		    pputs(prn, "\n\\begin{center}\n");
@@ -901,19 +1133,19 @@ int gretl_VAR_print (GRETL_VAR *var, const DATAINFO *pdinfo, gretlopt opt,
 		pputs(prn, "& ");
 		pprintf(prn, "$F(%d, %d) = %g$ & ", var->order, dfd, var->Fvals[k]);
 		pprintf(prn, "%s %.4f\\\\\n", I_("p-value"), 
-			f_cdf_comp(var->Fvals[k], var->order, dfd));
+			snedecor_cdf_comp(var->Fvals[k], var->order, dfd));
 	    } else if (rtf) {
 		pprintf(prn, I_("All lags of %-15s "), pdinfo->varname[v]);
 		pprintf(prn, "F(%d, %d) = %8.5g, ", var->order, dfd, var->Fvals[k]);
 		pprintf(prn, "%s %.4f\\par\n", I_("p-value"), 
-			f_cdf_comp(var->Fvals[k], var->order, dfd));
+			snedecor_cdf_comp(var->Fvals[k], var->order, dfd));
 	    } else {
 		pputs(prn, "  ");
 		pprintf(prn, _("All lags of %-15s "), pdinfo->varname[v]);
 		sprintf(Fstr, "F(%d, %d)", var->order, dfd);
 		pprintf(prn, "%12s = %#8.5g, ", Fstr, var->Fvals[k]);
 		pprintf(prn, "%s %.4f\n", _("p-value"), 
-			f_cdf_comp(var->Fvals[k], var->order, dfd));
+			snedecor_cdf_comp(var->Fvals[k], var->order, dfd));
 	    }
 	    k++;
 	}
@@ -924,19 +1156,19 @@ int gretl_VAR_print (GRETL_VAR *var, const DATAINFO *pdinfo, gretlopt opt,
 		pputs(prn, "& ");
 		pprintf(prn, "$F(%d, %d) = %g$ & ", var->neqns, dfd, var->Fvals[k]);
 		pprintf(prn, "%s %.4f\\\\\n", I_("p-value"), 
-			f_cdf_comp(var->Fvals[k], var->neqns, dfd));
+			snedecor_cdf_comp(var->Fvals[k], var->neqns, dfd));
 	    } else if (rtf) {
 		pprintf(prn, I_("All vars, lag %-13d "), var->order);
 		pprintf(prn, "F(%d, %d) = %8.5g, ", var->neqns, dfd, var->Fvals[k]);
 		pprintf(prn, "%s %.4f\\par\n", I_("p-value"), 
-			f_cdf_comp(var->Fvals[k], var->neqns, dfd));
+			snedecor_cdf_comp(var->Fvals[k], var->neqns, dfd));
 	    } else {
 		pputs(prn, "  ");
 		pprintf(prn, _("All vars, lag %-13d "), var->order);
 		sprintf(Fstr, "F(%d, %d)", var->neqns, dfd);
 		pprintf(prn, "%12s = %#8.5g, ", Fstr, var->Fvals[k]);
 		pprintf(prn, "%s %.4f\n", _("p-value"), 
-			f_cdf_comp(var->Fvals[k], var->neqns, dfd));
+			snedecor_cdf_comp(var->Fvals[k], var->neqns, dfd));
 	    } 
 	    k++;
 	}
