@@ -1,16 +1,22 @@
-#define MAX_ARMA_ORDER 6
+#include "usermat.h"
+
+#define MAX_ARMA_ORDER 128
 #define MAX_ARIMA_DIFF 2
 
 struct arma_info {
     int yno;         /* ID of dependent variable */
     ArmaFlags flags; /* specification flags */
     int ifc;         /* specification includes a constant? */
-    int p;           /* non-seasonal AR order */
+    int p;           /* max non-seasonal AR order */
     int d;           /* non-seasonal difference */
-    int q;           /* non-seasonal MA order */
+    int q;           /* max non-seasonal MA order */
     int P;           /* seasonal AR order */
     int D;           /* seasonal difference */
     int Q;           /* seasonal MA order */
+    char *pmask;     /* specific AR lags included */
+    char *qmask;     /* specific MA lags included */
+    int np;          /* total non-seasonal AR lags */
+    int nq;          /* total non-seasonal MA lags */
     int maxlag;      /* longest lag in model */
     int nexo;        /* number of other regressors (ARMAX) */
     int nc;          /* total number of coefficients */
@@ -19,6 +25,7 @@ struct arma_info {
     int pd;          /* periodicity of data */
     int T;           /* full length of data series */
     double *dy;      /* differenced dependent variable */
+    const char *pqspec; /* input string with specific AR, MA lags */
 };
 
 #define arma_has_seasonal(a)   ((a)->flags & ARMA_SEAS)
@@ -32,8 +39,12 @@ struct arma_info {
 #define unset_arma_is_arima(a)    ((a)->flags &= ~ARMA_DSPEC)
 #define set_arma_use_vech(a)      ((a)->flags |= ARMA_VECH)
 
+#define AR_included(a,i) (a->pmask == NULL || a->pmask[i] == '1')
+#define MA_included(a,i) (a->qmask == NULL || a->qmask[i] == '1')
+
 static void 
-arma_info_init (struct arma_info *ainfo, char flags, const DATAINFO *pdinfo)
+arma_info_init (struct arma_info *ainfo, char flags, 
+		const char *pqspec, const DATAINFO *pdinfo)
 {
     ainfo->yno = 0;
     ainfo->flags = flags;
@@ -45,6 +56,12 @@ arma_info_init (struct arma_info *ainfo, char flags, const DATAINFO *pdinfo)
     ainfo->D = 0;
     ainfo->Q = 0; 
 
+    ainfo->pmask = NULL;
+    ainfo->qmask = NULL;
+    
+    ainfo->np = 0;
+    ainfo->nq = 0;
+
     ainfo->maxlag = 0;
     ainfo->ifc = 0;
     ainfo->nexo = 0;
@@ -56,6 +73,167 @@ arma_info_init (struct arma_info *ainfo, char flags, const DATAINFO *pdinfo)
     ainfo->T = pdinfo->n;
 
     ainfo->dy = NULL;
+    ainfo->pqspec = pqspec;
+}
+
+static void arma_info_cleanup (struct arma_info *ainfo)
+{
+    free(ainfo->pmask);
+    free(ainfo->qmask);
+    free(ainfo->dy);
+}
+
+enum {
+    AR_MASK,
+    MA_MASK
+};
+
+/* Create a mask for skipping certain intermediate lags, 
+   AR or MA.  This function also sets ainfo->np and ainfo->nq,
+   which record the actual number of non-seasonal AR and MA
+   lags used.
+*/
+
+static char *mask_from_vec (const gretl_vector *v, 
+			    struct arma_info *ainfo,
+			    int m, int *err)
+{
+    int vlen = gretl_vector_get_length(v);
+    int mlen = (m == AR_MASK)? ainfo->p : ainfo->q;
+    int nv = 0, nmax = 0;
+    char *mask;
+    int i, k;
+
+    mask = malloc(mlen + 1);
+    if (mask == NULL) {
+	*err = E_ALLOC;
+	return NULL;
+    }
+
+    for (i=0; i<mlen; i++) {
+	mask[i] = '0';
+    }
+    mask[mlen] = '\0';
+
+    for (i=0; i<vlen; i++) {
+	k = v->val[i] - 1; /* convert to zero-based */
+	if (k >= 0 && k < mlen) {
+	    mask[k] = '1'; 
+	    nv++;
+	    if (k + 1 > nmax) {
+		nmax = k + 1;
+	    }
+	}
+    }
+
+    if (m == AR_MASK) {
+	ainfo->p = nmax;
+	ainfo->np = nv;
+    } else {
+	ainfo->q = nmax;
+	ainfo->nq = nv;
+    }
+
+    if (nv == 0) {
+	free(mask);
+	mask = NULL;
+    }
+
+    return mask;
+}
+
+/* construct vector of specific lags from string spec */
+
+static gretl_matrix *
+matrix_from_spec (const char *s, int *tmp, int *err)
+{
+    gretl_matrix *m = NULL;
+    const char *p = s;
+    char *rem;
+    int i, k, n = 0;
+
+    while (*s != '\0' && *s != '}' && n < 20) {
+	strtol(s, &rem, 10);
+	n++;
+	s = rem;
+    }
+
+    m = gretl_vector_alloc(n);
+    if (m == NULL) {
+	*err = E_ALLOC;
+	return NULL;
+    }
+
+    s = p;
+    i = 0;
+    while (*s != '\0' && *s != '}' && i < 20) {
+	k = (int) strtol(s, &rem, 10);
+	m->val[i++] = k;
+	s = rem;
+    }
+
+    *tmp = 1;
+
+    return m;
+}
+
+static gretl_matrix *get_arma_pq_vec (struct arma_info *ainfo,
+				      int t, int *tmp, int *err)
+{
+    gretl_matrix *m = NULL;
+    const char *test = (t == AR_MASK)? "p=" : "q=";
+    const char *s = strstr(ainfo->pqspec, test);
+
+    *tmp = 0;
+
+    if (s != NULL) {
+	s += 2;
+	if (*s == '{') {
+	    m = matrix_from_spec(s + 1, tmp, err);
+	} else {
+	    m = get_matrix_by_name(s);
+	    if (m == NULL) {
+		*err = E_UNKVAR;
+	    }
+	} 
+    }
+
+    return m;
+}
+
+static int 
+arma_make_masks (struct arma_info *ainfo, int *list)
+{
+    gretl_matrix *m;
+    int tmp, err = 0;
+
+    if (ainfo->p > 0) {
+	ainfo->np = ainfo->p;
+	if (ainfo->pqspec != NULL) {
+	    m = get_arma_pq_vec(ainfo, AR_MASK, &tmp, &err);
+	    if (m != NULL) {
+		ainfo->pmask = mask_from_vec(m, ainfo, AR_MASK, &err);
+		if (tmp) {
+		    gretl_matrix_free(m);
+		}
+	    }
+	}
+    }
+
+    if (ainfo->q > 0 && !err) {
+	ainfo->nq = ainfo->q;
+	if (ainfo->pqspec != NULL) {
+	    m = get_arma_pq_vec(ainfo, MA_MASK, &tmp, &err);
+	    if (m != NULL) {
+		ainfo->qmask = mask_from_vec(m, ainfo, MA_MASK, &err);
+		if (tmp) {
+		    gretl_matrix_free(m);
+		}
+	    }
+	}
+    }
+
+    return err;
 }
 
 static int arma_list_y_position (struct arma_info *ainfo)
@@ -171,6 +349,16 @@ static void ainfo_data_to_model (struct arma_info *ainfo, MODEL *pmod)
     if (ainfo->nexo > 0) {
 	gretl_model_set_int(pmod, "armax", 1);
     }
+
+    if (ainfo->pmask != NULL) {
+	gretl_model_set_string_as_data(pmod, "pmask", 
+				       gretl_strdup(ainfo->pmask));
+    }
+
+    if (ainfo->qmask != NULL) {
+	gretl_model_set_string_as_data(pmod, "qmask", 
+				       gretl_strdup(ainfo->qmask));
+    }
 }
 
 /* write the various statistics from ARMA estimation into
@@ -233,6 +421,7 @@ static void write_arma_model_stats (MODEL *pmod, const int *list,
 
     gretl_model_add_arma_varnames(pmod, pdinfo, ainfo->yno,
 				  ainfo->p, ainfo->q, 
+				  ainfo->pmask, ainfo->qmask,
 				  ainfo->P, ainfo->Q,
 				  ainfo->nexo);
 }
@@ -240,7 +429,7 @@ static void write_arma_model_stats (MODEL *pmod, const int *list,
 static void calc_max_lag (struct arma_info *ainfo)
 {
     int pmax = ainfo->p;
-    int dmax = ainfo->d;
+    int dmax = ainfo->d; /* FIXME what about seasonal D? */
 
     if (arma_has_seasonal(ainfo)) {
 	pmax += ainfo->P * ainfo->pd;
@@ -437,6 +626,8 @@ static int check_arma_sep (int *list, int sep1, struct arma_info *ainfo)
     return err;
 }
 
+#define count_arma_coeffs(a) (a->ifc + a->np + a->nq + a->P + a->Q + a->nexo)
+
 static int check_arma_list (int *list, gretlopt opt, 
 			    const double **Z, const DATAINFO *pdinfo,
 			    struct arma_info *ainfo)
@@ -477,6 +668,12 @@ static int check_arma_list (int *list, gretlopt opt,
 	ainfo->Q = list[5];
     }
 
+    /* now that we have p and q we can check for masked lags */
+
+    if (!err) {
+	err = arma_make_masks(ainfo, list);
+    }
+
     /* If there's an explicit constant in the list here, we'll remove
        it, since it is added implicitly later.  But if we're supplied
        with OPT_N (meaning: no intercept) we'll flag this by
@@ -501,8 +698,7 @@ static int check_arma_list (int *list, gretlopt opt,
 	gretl_errmsg_set(_("Error in arma command"));
     } else {
 	ainfo->nexo = list[0] - ((arma_has_seasonal(ainfo))? 7 : 4);
-	ainfo->nc = ainfo->p + ainfo->q + ainfo->P + ainfo->Q
-	    + ainfo->nexo + ainfo->ifc;
+	ainfo->nc = count_arma_coeffs(ainfo);
 	ainfo->yno = (arma_has_seasonal(ainfo))? list[7] : list[4];
     }
 
@@ -559,6 +755,12 @@ static int check_arima_list (int *list, gretlopt opt,
 	ainfo->Q = list[7];
     }
 
+    /* now that we have p and q we can check for masked lags */
+
+    if (!err) {
+	err = arma_make_masks(ainfo, list);
+    }
+
     /* If there's an explicit constant in the list here, we'll remove
        it, since it is added implicitly later.  But if we're supplied
        with OPT_N (meaning: no intercept) we'll flag this by
@@ -583,8 +785,7 @@ static int check_arima_list (int *list, gretlopt opt,
 	gretl_errmsg_set(_("Error in arma command"));
     } else {
 	ainfo->nexo = list[0] - ((arma_has_seasonal(ainfo))? 9 : 5);
-	ainfo->nc = ainfo->p + ainfo->q + ainfo->P + ainfo->Q
-	    + ainfo->nexo + ainfo->ifc;
+	ainfo->nc = count_arma_coeffs(ainfo);
 	ainfo->yno = (arma_has_seasonal(ainfo))? list[9] : list[5];
     }
 
