@@ -1,20 +1,20 @@
-/*
- *  Copyright (c) by Allin Cottrell
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
- *
- *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
- *
+/* 
+ *  gretl -- Gnu Regression, Econometrics and Time-series Library
+ *  Copyright (C) 2001 Allin Cottrell and Riccardo "Jack" Lucchetti
+ * 
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ * 
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ * 
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 #include "../cephes/mconf.h"
@@ -124,7 +124,7 @@ static int add_unique_output_file (MODEL *pmod, const char *path)
     err = rename(fname, unique);
     if (!err) {
 	gretl_model_set_data(pmod, "x12a_output", g_strdup(unique),
-			     MODEL_DATA_STRING, strlen(fname) + 1);
+			     GRETL_TYPE_STRING, strlen(fname) + 1);
     } 
 
     return err;
@@ -282,7 +282,7 @@ static int get_roots (const char *fname, MODEL *pmod,
     }
 
     if (roots != NULL) {
-	gretl_model_set_data(pmod, "roots", roots, MODEL_DATA_CMPLX_ARRAY,
+	gretl_model_set_data(pmod, "roots", roots, GRETL_TYPE_CMPLX_ARRAY,
 			     nr * sizeof *roots);
     }
 
@@ -352,12 +352,12 @@ static int
 get_estimates (const char *fname, MODEL *pmod, struct arma_info *ainfo)
 {
     double *ar_coeff = pmod->coeff + ainfo->ifc;
-    double *ma_coeff = ar_coeff + ainfo->p + ainfo->P;
-    double *x_coeff = ma_coeff + ainfo->q + ainfo->Q;
+    double *ma_coeff = ar_coeff + ainfo->np + ainfo->P;
+    double *x_coeff = ma_coeff + ainfo->nq + ainfo->Q;
 
     double *ar_sderr = pmod->sderr + ainfo->ifc;
-    double *ma_sderr = ar_sderr + ainfo->p + ainfo->P;
-    double *x_sderr = ma_sderr + ainfo->q + ainfo->Q;
+    double *ma_sderr = ar_sderr + ainfo->np + ainfo->P;
+    double *x_sderr = ma_sderr + ainfo->nq + ainfo->Q;
 
     FILE *fp;
     char line[128], word[16];
@@ -601,6 +601,49 @@ make_x12a_date_string (int t, const DATAINFO *pdinfo, char *str)
     }    
 }
 
+static void x12_pdq_string (struct arma_info *ainfo, FILE *fp)
+{
+    fputc('(', fp);
+    
+    if (ainfo->pmask == NULL) {
+	fprintf(fp, "%d", ainfo->p);
+    } else {
+	int i;
+
+	fputc('[', fp);
+	for (i=0; i<ainfo->p; i++) {
+	    if (AR_included(ainfo, i)) {
+		fprintf(fp, "%d", i+1);
+		if (i < ainfo->p - 1) {
+		    fputc(' ', fp);
+		}
+	    }
+	}
+	fputc(']', fp);
+    }
+
+    fprintf(fp, " %d ", ainfo->d);
+
+    if (ainfo->qmask == NULL) {
+	fprintf(fp, "%d", ainfo->q);
+    } else {
+	int i;
+
+	fputc('[', fp);
+	for (i=0; i<ainfo->q; i++) {
+	    if (MA_included(ainfo, i)) {
+		fprintf(fp, "%d", i+1);
+		if (i < ainfo->q - 1) {
+		    fputc(' ', fp);
+		}
+	    }
+	}
+	fputc(']', fp);
+    }
+
+    fputc(')', fp);
+}
+
 #define MAXOBS 720
 #define MAXFCAST 60
 
@@ -695,13 +738,12 @@ static int write_spc_file (const char *fname,
     fputs("}\n", fp);
 
     /* arima specification */
+    fputs("arima {\n model = ", fp);
+    x12_pdq_string(ainfo, fp);
     if (ainfo->P > 0 || ainfo->Q > 0) {
-	fprintf(fp, "arima {\n model = (%d %d %d)(%d %d %d)\n}\n", 
-		ainfo->p, ainfo->d, ainfo->q, 
-		ainfo->P, ainfo->D, ainfo->Q);
+	fprintf(fp, "(%d %d %d)\n}\n", ainfo->P, ainfo->D, ainfo->Q);
     } else {
-	fprintf(fp, "arima {\n model = (%d %d %d)\n}\n", 
-		ainfo->p, ainfo->d, ainfo->q); 
+	fputs("\n}\n", fp);
     }
 
     fputs("estimate {\n", fp);
@@ -749,7 +791,8 @@ static void delete_old_files (const char *path)
     }
 }
 
-MODEL arma_x12_model (const int *list, const double **Z, const DATAINFO *pdinfo,
+MODEL arma_x12_model (const int *list, const char *pqspec,
+		      const double **Z, const DATAINFO *pdinfo,
 		      gretlopt opt, PRN *prn)
 {
     int verbose = (opt & OPT_V);
@@ -757,7 +800,7 @@ MODEL arma_x12_model (const int *list, const double **Z, const DATAINFO *pdinfo,
     const char *workdir = gretl_x12_arima_dir();
     char yname[VNAMELEN], path[MAXLEN];
     int *alist = NULL;
-    PRN *aprn = NULL;
+    PRN *vprn = NULL;
     MODEL armod;
     struct arma_info ainfo;
 #ifdef WIN32
@@ -765,30 +808,30 @@ MODEL arma_x12_model (const int *list, const double **Z, const DATAINFO *pdinfo,
 #endif
     int err = 0;
 
-    if (verbose) {
-	aprn = prn;
-    } 
+    vprn = set_up_verbose_printer(opt, prn);
 
     if (pdinfo->t2 < pdinfo->n - 1) {
 	/* FIXME this is temporary (OPT_F -> generate forecast) */
 	opt |= OPT_F;
     }
 
-    arma_info_init(&ainfo, ARMA_X12A, pdinfo);
+    arma_info_init(&ainfo, ARMA_X12A, pqspec, pdinfo);
     gretl_model_init(&armod); 
     gretl_model_smpl_init(&armod, pdinfo);
 
     alist = gretl_list_copy(list);
     if (alist == NULL) {
-	armod.errcode = E_ALLOC;
-	goto bailout;
+	err = E_ALLOC;
+    }
+
+    if (!err) {
+	err = arma_check_list(alist, opt, Z, pdinfo, &ainfo);
     }
 
-    err = arma_check_list(alist, opt, Z, pdinfo, &ainfo);
     if (err) {
 	armod.errcode = err;
 	goto bailout;
-    }
+    }     
 
     /* calculate maximum lag */
     calc_max_lag(&ainfo);
@@ -802,7 +845,7 @@ MODEL arma_x12_model (const int *list, const double **Z, const DATAINFO *pdinfo,
     /* create differenced series if needed */
     if (ainfo.d > 0 || ainfo.D > 0) {
 	err = arima_difference(Z[ainfo.yno], &ainfo);
-    }    
+    }  
 
     sprintf(yname, pdinfo->varname[ainfo.yno]);
 
@@ -830,7 +873,7 @@ MODEL arma_x12_model (const int *list, const double **Z, const DATAINFO *pdinfo,
 	armod.nobs = armod.t2 - armod.t1 + 1;
 	populate_arma_model(&armod, alist, path, Z, pdinfo, &ainfo);
 	if (verbose && !armod.errcode) {
-	    print_iterations(path, aprn);
+	    print_iterations(path, vprn);
 	}
 	if (!armod.errcode) {
 	    int acode = ARMA_X12A;
@@ -848,10 +891,15 @@ MODEL arma_x12_model (const int *list, const double **Z, const DATAINFO *pdinfo,
 	gretl_errmsg_set(_("Failed to execute x12arima"));
     }
 
+    if (vprn != NULL && vprn != prn) {
+	iter_print_callback(0, vprn);
+	close_down_verbose_printer(vprn);
+    }
+
  bailout:
 
     free(alist);
-    free(ainfo.dy);
+    arma_info_cleanup(&ainfo);
 
     return armod;
 }
