@@ -29,10 +29,10 @@ static int audioprint_coeff (const DATAINFO *pdinfo, const MODEL *pmod,
 
     /* std error is well-defined, but is it positive? */
     if (pmod->sderr[i] > 0.) {
-	double t, pval;
+	double tval, pval;
 
-	t = pmod->coeff[i] / pmod->sderr[i];
-	pval = coeff_pval(pmod, t, pmod->dfd);
+	tval = pmod->coeff[i] / pmod->sderr[i];
+	pval = coeff_pval(pmod->ci, tval, pmod->dfd);
 	pprintf(prn, "P-value %.3g.\n", pval);
     } else { /* zero standard error */
 	pputs(prn, "Standard error is zero.\n");
@@ -199,13 +199,15 @@ static int speak_buffer (const char *buf, int (*should_stop)())
 
     bufgets_init(buf);
 
-    while (bufgets(line, 127, buf)) {
+    while (bufgets(line, sizeof line, buf)) {
 	if (should_stop()) {
 	    flite_text_to_speech("OK, stopping", v, "play");
 	    break;
 	}
 	flite_text_to_speech(line, v, "play");
     }
+
+    bufgets_finalize(buf);
 	
     return 0;
 }
@@ -266,7 +268,7 @@ static int speak_buffer (const char *buf, int (*should_stop)())
 
     bufgets_init(buf);
 
-    while (bufgets(line, 127, buf)) {
+    while (bufgets(line, sizeof line, buf)) {
 	if (should_stop()) {
 	    ISpVoice_Speak(v, L"OK, stopping", 0, NULL);
 	    break;
@@ -276,6 +278,8 @@ static int speak_buffer (const char *buf, int (*should_stop)())
 	free(w);
     }
 
+    bufgets_finalize(buf);
+
     release_sapi_voice(v);
 
     return 0;
