@@ -1,20 +1,20 @@
-/*
- *  Copyright (c) by Allin Cottrell 2002-2004
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
- *
- *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
- *
+/* 
+ *  gretl -- Gnu Regression, Econometrics and Time-series Library
+ *  Copyright (C) 2001 Allin Cottrell and Riccardo "Jack" Lucchetti
+ * 
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ * 
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ * 
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 /*
@@ -997,7 +997,7 @@ static int fix_varname (char *vname)
 }   
 
 static int 
-check_all_varnames (wbook *book, int *ncols, const char *blank_col)
+check_all_varnames (wbook *book, int totcols, const char *blank_col)
 {
     int j, i = book->row_offset;
     int startcol = book->col_offset;
@@ -1019,7 +1019,7 @@ check_all_varnames (wbook *book, int *ncols, const char *blank_col)
 	}
     }
 
-    for (j=startcol; j<*ncols; j++) { 
+    for (j=startcol; j<totcols; j++) { 
 	if (blank_col[j]) {
 	    gotcols++;
 	    continue;
@@ -1060,10 +1060,7 @@ check_all_varnames (wbook *book, int *ncols, const char *blank_col)
 	realcols++;
     }
 
-    fprintf(stderr, "realcols = %d, vnames = %d\n", realcols, vnames);
-
     if (vnames == realcols) {
-	*ncols = gotcols;
 	ret = VARNAMES_OK;
     } else if (vnames > 0) {
 	ret = VARNAMES_NOTSTR;
@@ -1102,7 +1099,7 @@ struct string_err {
 /* check for invalid data in the selected data block */
 
 static int 
-check_data_block (wbook *book, int ncols, const char *blank_col,
+check_data_block (wbook *book, int totcols, const char *blank_col,
 		  struct string_err *err)
 {
     int startcol = book->col_offset;
@@ -1117,7 +1114,7 @@ check_data_block (wbook *book, int ncols, const char *blank_col,
     err->column = 0;
     err->str = NULL;
 
-    for (j=startcol; j<ncols; j++) {
+    for (j=startcol; j<totcols; j++) {
 	if (blank_col[j]) {
 	    continue;
 	}
@@ -1185,9 +1182,9 @@ static int transcribe_data (wbook *book, double **Z, DATAINFO *pdinfo,
 
     if (book_time_series(book) || book_obs_labels(book)) {
 	startcol++;
-    }    
+    } 
 
-    for (i=startcol; i<totcols; i++) {
+    for (i=startcol; i<=totcols; i++) {
 	int ts, missing = 0;
 
 	if (blank_col[i]) {
@@ -1201,6 +1198,8 @@ static int transcribe_data (wbook *book, double **Z, DATAINFO *pdinfo,
 	pdinfo->varname[j][0] = 0;
 	if (book_auto_varnames(book)) {
 	    sprintf(pdinfo->varname[j], "v%d", j);
+	} else if (rows[roff].cells[i] == NULL) {
+	    sprintf(pdinfo->varname[j], "v%d", j);
 	} else {
 	    strncat(pdinfo->varname[j], rows[roff].cells[i] + 1, 
 		    VNAMELEN - 1);
@@ -1341,7 +1340,7 @@ static void book_time_series_setup (wbook *book, DATAINFO *newinfo, int pd)
 }
 
 static int 
-real_excel_get_data (const char *fname, double ***pZ, DATAINFO *pdinfo,
+real_excel_get_data (const char *fname, double ***pZ, DATAINFO **ppdinfo,
 		     int gui, PRN *prn)
 {
     wbook xbook;
@@ -1406,16 +1405,16 @@ real_excel_get_data (const char *fname, double ***pZ, DATAINFO *pdinfo,
 	goto getout;
     }
 
-    /* locate any blank columns */
+    /* get sizes and locate any blank columns */
     err = get_sheet_dimensions(&totcols, &datacols, &blank_col, prn);
 
     if (err) goto getout;
 
     /* check feasibility of offsets */
-    if (book->row_offset > nrows) {
+    if (book->row_offset >= nrows) {
 	pputs(prn, _("Starting row is out of bounds.\n"));
 	err = 1;
-    } else if (book->col_offset > totcols) {
+    } else if (book->col_offset >= totcols) {
 	pputs(prn, _("Starting column is out of bounds.\n"));
 	err = 1;
     }
@@ -1429,7 +1428,7 @@ real_excel_get_data (const char *fname, double ***pZ, DATAINFO *pdinfo,
     }
 
     /* any bad or missing variable names? */
-    err = check_all_varnames(book, &totcols, blank_col);
+    err = check_all_varnames(book, totcols, blank_col);
 
     if (err == VARNAMES_NULL || err == VARNAMES_NOTSTR) {
 	pputs(prn, _("One or more variable names are missing.\n"));
@@ -1489,16 +1488,10 @@ real_excel_get_data (const char *fname, double ***pZ, DATAINFO *pdinfo,
 	goto getout;
     }
 
-    set_all_missing(newZ, newinfo);
-
     if (book_time_series(book)) {
 	ntodate_full(newinfo->endobs, newinfo->n - 1, newinfo);
     } else {
-	strcpy(newinfo->stobs, "1");
-	sprintf(newinfo->endobs, "%d", newinfo->n);
-	newinfo->sd0 = 1.0;
-	newinfo->pd = 1;
-	newinfo->structure = CROSS_SECTION;
+	dataset_obs_info_default(newinfo);
     } 
 
     /* OK: actually populate the dataset */
@@ -1520,13 +1513,7 @@ real_excel_get_data (const char *fname, double ***pZ, DATAINFO *pdinfo,
 	}
     }
 
-    if (*pZ == NULL) {
-	*pZ = newZ;
-	*pdinfo = *newinfo;
-	free(newinfo);
-    } else {
-	err = merge_data(pZ, pdinfo, newZ, newinfo, prn);
-    }
+    err = merge_or_replace_data(pZ, ppdinfo, &newZ, &newinfo, prn);
 
  getout:
     
@@ -1542,19 +1529,23 @@ real_excel_get_data (const char *fname, double ***pZ, DATAINFO *pdinfo,
     }
 #endif
 
+    if (newinfo != NULL) {
+	destroy_dataset(newZ, newinfo);
+    }
+
     return err;
 }  
 
-int excel_get_data (const char *fname, double ***pZ, DATAINFO *pdinfo,
+int excel_get_data (const char *fname, double ***pZ, DATAINFO **ppdinfo,
 		    PRN *prn)
 {
-    return real_excel_get_data(fname, pZ, pdinfo, 1, prn);
+    return real_excel_get_data(fname, pZ, ppdinfo, 1, prn);
 }
 
-int cli_get_xls (const char *fname, double ***pZ, DATAINFO *pdinfo,
+int cli_get_xls (const char *fname, double ***pZ, DATAINFO **ppdinfo,
 		 PRN *prn)
 {
-    return real_excel_get_data(fname, pZ, pdinfo, 0, prn);
+    return real_excel_get_data(fname, pZ, ppdinfo, 0, prn);
 }
 
 
