@@ -1,20 +1,20 @@
-/* gretl - The Gnu Regression, Econometrics and Time-series Library
- * Copyright (C) 1999-2004 by Allin Cottrell
- *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License 
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
- *
- * This software is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License 
- * along with this software; if not, write to the 
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+/* 
+ *  gretl -- Gnu Regression, Econometrics and Time-series Library
+ *  Copyright (C) 2001 Allin Cottrell and Riccardo "Jack" Lucchetti
+ * 
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ * 
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ * 
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 /* GARCH plugin for gretl using the Fiorentini, Calzolari and 
@@ -25,10 +25,9 @@
 #include "libset.h"
 #include "var.h"
 
-#include "fcp.h"
-#include "mod_garch.h"
+#include "garch.h"
 
-#undef VPARM_DEBUG
+#define VPARM_DEBUG 0
 
 #define VPARM_MAX 6            /* max number of variance parameters */
 #define GARCH_PARAM_MAX 0.999
@@ -71,77 +70,86 @@ static void add_garch_varnames (MODEL *pmod, const DATAINFO *pdinfo,
     }
 }
 
-static int make_packed_vcv (MODEL *pmod, double *vcv, int np,
-			    int nc, double scale)
+static void rescale_results (double *theta, gretl_matrix *V,
+			     double scale, int npar, int nc)
 {
-    const int nterms = np * (np + 1) / 2;
-    double sfi, sfj;
-    int i, j, k;
+    double vij, sfi, sf, sc2 = scale * scale;
+    int i, j;
 
-    free(pmod->vcv);
-    pmod->vcv = malloc(nterms * sizeof *pmod->vcv);
-    if (pmod->vcv == NULL) {
-	return 1;  
+    for (i=0; i<nc; i++) {
+	theta[i] *= scale;
     }
 
-    for (i=0; i<np; i++) {
-	if (i < nc) {
-	    sfi = scale;
-	} else if (i == nc) {
-	    sfi = scale * scale;
-	} else {
-	    sfi = 1.0;
-	}
+    theta[nc] *= sc2;
+
+    for (i=0; i<npar; i++) {
+	sfi = (i < nc)? scale : (i == nc)? sc2 : 1.0;
 	for (j=0; j<=i; j++) {
-	    if (j < nc) {
-		sfj = scale;
-	    } else if (j == nc) {
-		sfj = scale * scale;
-	    } else {
-		sfj = 1.0;
-	    }
-	    k = ijton(i, j, np);
-	    pmod->vcv[k] = vcv[i + np * j] * sfi * sfj;
+	    sf = (j < nc)? scale*sfi : (j == nc)? sc2*sfi : sfi;
+	    vij = gretl_matrix_get(V, i, j) * sf;
+	    gretl_matrix_set(V, i, j, vij);
+	    gretl_matrix_set(V, j, i, vij);
 	}
     }
-
-    return 0;
 }
 
-static int write_garch_stats (MODEL *pmod, const double **Z,
-			      double scale, const DATAINFO *pdinfo,
-			      const int *list, const double *theta, 
-			      int nparam, int pad, const double *res,
-			      const double *h)
+static int 
+write_garch_stats (MODEL *pmod, const int *list, 
+		   const double **Z, const DATAINFO *pdinfo,
+		   double *theta, gretl_matrix *V, double scale,
+		   const double *e, const double *h, 
+		   int npar, int nc, int pad, PRN *prn)
 {
     int err = 0;
-    double *coeff, *sderr, *garch_h;
-    double den;
+    double *coeff, *sderr, *vcv, *garch_h;
+    double x, den;
     int ynum = list[4];
     int nvp = list[1] + list[2];
     int xvars = list[0] - 4;
-    int i;
+    int nv = npar * (npar + 1) / 2;
+    int i, j, k;
 
-    coeff = realloc(pmod->coeff, nparam * sizeof *pmod->coeff);
-    sderr = realloc(pmod->sderr, nparam * sizeof *pmod->sderr);
+    coeff = realloc(pmod->coeff, npar * sizeof *pmod->coeff);
+    sderr = realloc(pmod->sderr, npar * sizeof *pmod->sderr);
+    vcv = realloc(pmod->vcv, nv * sizeof *pmod->vcv);
 
-    if (coeff == NULL || sderr == NULL) return 1;
-
-    for (i=0; i<nparam; i++) {
-	coeff[i] = theta[i+1];
-	sderr[i] = theta[i+nparam+1];
+    if (coeff == NULL || sderr == NULL || vcv == NULL) {
+	return E_ALLOC;
     }
-    
+
+    if (scale != 1.0) {
+	rescale_results(theta, V, scale, npar, nc);
+    }
+
+    for (i=0; i<npar; i++) {
+	coeff[i] = theta[i];
+	x = gretl_matrix_get(V, i, i);
+	sderr[i] = (x > 0.0)? sqrt(x) : 0.0;
+	for (j=0; j<=i; j++) {
+	    k = ijton(i, j, npar);
+	    vcv[k] = gretl_matrix_get(V, i, j);
+	}
+    }
+
+    /* verbose? */
+    if (prn != NULL) {
+	for (i=0; i<npar; i++) {
+	    pprintf(prn, "theta[%d]: %#14.6g (%#.6g)\n", i, theta[i], 
+		    sderr[i]);
+	}
+	pputc(prn, '\n'); 
+    }   
+
     pmod->coeff = coeff;
     pmod->sderr = sderr;
-   
-    pmod->ncoeff = nparam;
+    pmod->vcv = vcv;
+    pmod->ncoeff = npar;
 
     pmod->ess = 0.0;
     for (i=pmod->t1; i<=pmod->t2; i++) {
-	pmod->uhat[i] = res[i + pad] * scale;
+	pmod->uhat[i] = e[i + pad] * scale;
 	pmod->ess += pmod->uhat[i] * pmod->uhat[i];
-	pmod->yhat[i] =  Z[ynum][i] * scale - pmod->uhat[i];
+	pmod->yhat[i] = Z[ynum][i] * scale - pmod->uhat[i];
     }
 
     /* set sigma to its unconditional or steady-state value */
@@ -172,7 +180,7 @@ static int write_garch_stats (MODEL *pmod, const double **Z,
 	    }
 	}
 	gretl_model_set_data(pmod, "garch_h", garch_h, 
-			     MODEL_DATA_DOUBLE_ARRAY,
+			     GRETL_TYPE_DOUBLE_ARRAY,
 			     pdinfo->n * sizeof *garch_h);
     }
 
@@ -200,16 +208,15 @@ static int make_garch_dataset (const int *list, double **Z,
 	*py = y;
     } 
 
-    if (nx > 0) {
-	if (pad) {
-	    X = doubles_array_new(nx, bign);
-	} else {
-	    X = malloc(nx * sizeof *X);
-	}
-	if (X == NULL) {
-	    free(y);
-	    return E_ALLOC;
-	}
+    if (pad) {
+	X = doubles_array_new(nx, bign);
+    } else {
+	X = malloc(nx * sizeof *X);
+    }
+
+    if (X == NULL) {
+	free(y);
+	return E_ALLOC;
     }
 
     if (pad > 0) {
@@ -226,9 +233,6 @@ static int make_garch_dataset (const int *list, double **Z,
 		k = 5;
 		for (i=0; i<nx; i++) {
 		    vx = list[k++]; 
-		    if (vx == 0) {
-			vx = list[k++];
-		    }
 		    X[i][t] = Z[vx][s];
 		}
 	    }
@@ -239,9 +243,6 @@ static int make_garch_dataset (const int *list, double **Z,
 	k = 5;
 	for (i=0; i<nx; i++) {
 	    vx = list[k++]; 
-	    if (vx == 0) {
-		vx = list[k++];
-	    }
 	    X[i] = Z[vx];
 	}
     }
@@ -253,8 +254,8 @@ static int make_garch_dataset (const int *list, double **Z,
 
 static int get_vopt (int robust)
 {
-    int vopt = get_garch_vcv_version();
-    int ropt = get_garch_robust_vcv_version();
+    int vopt = libset_get_int(GARCH_VCV);
+    int ropt = libset_get_int(GARCH_ROBUST_VCV);
 
     /* The defaults: QML if "robust" option is in force,
        otherwise negative Hessian */
@@ -273,11 +274,11 @@ static int get_vopt (int robust)
     return vopt;
 }
 
-static void garch_print_init (const double *coeff, int k,
-			      const double *a, int p, int q,
-			      int manual, PRN *prn)
+static void garch_print_init (const double *theta, int k,
+			      int p, int q, int manual, 
+			      PRN *prn)
 {
-    int i;
+    int i, j = 0;
 
     pputc(prn, '\n');
 
@@ -290,17 +291,17 @@ static void garch_print_init (const double *coeff, int k,
     pputs(prn, "\n\n Regression coefficients:\n");
 
     for (i=0; i<k; i++) {
-	pprintf(prn, "  theta[%d] = %g\n", i, coeff[i]);
+	pprintf(prn, "  theta[%d] = %g\n", i, theta[j++]);
     }
 
     pputs(prn, "\n Variance parameters:\n");
 
-    pprintf(prn, "  alpha[0] = %g\n", a[0]);
+    pprintf(prn, "  alpha[0] = %g\n", theta[j++]);
     for (i=0; i<p; i++) {
-	pprintf(prn, "  alpha[%d] = %g\n", i+1, a[i+3]);
+	pprintf(prn, "  alpha[%d] = %g\n", i+1, theta[j++]);
     }
     for (i=0; i<q; i++) {
-	pprintf(prn, "   beta[%d] = %g\n", i, a[i+3+p]);
+	pprintf(prn, "   beta[%d] = %g\n", i, theta[j++]);
     }
 
     pputc(prn, '\n');
@@ -309,14 +310,11 @@ static void garch_print_init (const double *coeff, int k,
 /* pick up any manually set initial values (if these
    have been set via "set initvals") */
 
-static int garch_manual_init (double *a, int p, int q, 
-			      double *coeff, double *b,
-			      int k, PRN *prn)
+static int garch_manual_init (double *theta, int k, int p, int q, 
+			      PRN *prn)
 {
-    const gretl_matrix *m = get_init_vals();
-    int mlen = gretl_vector_get_length(m);
-    int n = p + q + 1 + k;
-    int i, j;
+    int mlen = n_init_vals();
+    int n = k + p + q + 1;
 
     if (mlen != n) {
 	if (mlen > 0) {
@@ -326,49 +324,52 @@ static int garch_manual_init (double *a, int p, int q,
 	return 0;
     }
 
-    /* coefficients on regressors */
-    for (i=0; i<k; i++) {
-	coeff[i] = m->val[i];
-	b[i] = 0.0;
+    /* if we're not using FCP, the following is handled 
+       within the BFGS routine */
+
+    if (libset_get_bool("fcp")) {
+	const gretl_matrix *m = get_init_vals();
+	int i;
+	
+	/* order: coeffs on regressors; variance params */
+
+	for (i=0; i<n; i++) {
+	    theta[i] = m->val[i];
+	}
+
+	garch_print_init(theta, k, p, q, 1, prn);
+
+	free_init_vals();
     }
 
-
-    /* variance parameters */
-    a[0] = m->val[i++];
-    for (j=0; j<p+q; j++) {
-	a[j+3] = m->val[i++];
-    }
-
-    garch_print_init(coeff, k, a, p, q, 1, prn);
-
     return 1;
 }
 
-int do_fcp (const int *list, double **Z, double scale,
-	    const DATAINFO *pdinfo, MODEL *pmod,
-	    PRN *prn, gretlopt opt)
+static int 
+garch_driver (const int *list, double **Z, double scale,
+	      const DATAINFO *pdinfo, MODEL *pmod,
+	      PRN *prn, gretlopt opt)
 {
     int t1 = pmod->t1, t2 = pmod->t2;
-    int ncoeff = pmod->ncoeff;
+    int nc = pmod->ncoeff;
     int p = list[1];
     int q = list[2];
     double *y = NULL;
     double **X = NULL;
     double *h = NULL;
-    double *amax = NULL; 
-    double *res = NULL, *res2 = NULL;
-    double *coeff = NULL, *b = NULL;
-    double *vcv = NULL;
+    double *e = NULL, *e2 = NULL;
+    double *theta = NULL;
+    double ll = NADBL;
+    gretl_matrix *V = NULL;
     int fnc = 0, grc = 0, iters = 0;
     int nobs, maxlag, bign, pad = 0;
-    int i, nx, nparam, vopt;
+    int i, npar, vopt;
     int err = 0;
 
     vopt = get_vopt(opt & OPT_R);
 
-    nx = ncoeff - 1;
     maxlag = (p > q)? p : q; 
-    nparam = ncoeff + p + q + 1;
+    npar = nc + p + q + 1;
 
     nobs = t2 + 1; /* number of obs in full dataset */
 
@@ -380,107 +381,72 @@ int do_fcp (const int *list, double **Z, double scale,
     /* length of series to pass to garch_estimate */
     bign = nobs + pad;
 	
-    res2 = malloc(bign * sizeof *res2);
-    res = malloc(bign * sizeof *res);
+    e = malloc(bign * sizeof *e);
+    e2 = malloc(bign * sizeof *e2);
     h = malloc(bign * sizeof *h);
-    amax = malloc(bign * sizeof *amax);
-    if (res2 == NULL || res == NULL || 
-	amax == NULL || h == NULL) {
+    if (e == NULL || e2 == NULL || h == NULL) {
 	err = E_ALLOC;
 	goto bailout;
     }
 
     for (i=0; i<bign; i++) {
-	res2[i] = res[i] = amax[i] = 0.0;
+	e[i] = e2[i] = h[i] = 0.0;
     }   
  
-    coeff = malloc(ncoeff * sizeof *coeff);
-    b = malloc(ncoeff * sizeof *b);
-    if (coeff == NULL || b == NULL) {
+    theta = malloc(npar * sizeof *theta);
+    if (theta == NULL) {
 	err = E_ALLOC;
 	goto bailout;	
     }
 
-    vcv = malloc((nparam * nparam) * sizeof *vcv);
-    if (vcv == NULL) {
+    V = gretl_zero_matrix_new(npar, npar);
+    if (V == NULL) {
 	err = E_ALLOC;
 	goto bailout;
     }
 
-    for (i=0; i<nparam * nparam; i++) {
-	vcv[i] = 0.0;
-    } 
-
     /* create dataset for garch estimation */
-    err = make_garch_dataset(list, Z, bign, pad, nx, &y, &X);
+    err = make_garch_dataset(list, Z, bign, pad, nc, &y, &X);
     if (err) {
 	goto bailout;
     }
 
-    /* a bit odd, but for compatibility with FCP... */
-    amax[1] = q;
-    amax[2] = p; 
-
-    if (!garch_manual_init(amax, p, q, coeff, b, ncoeff, prn)) {
+    if (!garch_manual_init(theta, nc, p, q, prn)) {
 
 	/* initial coefficients from OLS */
-	for (i=0; i<ncoeff; i++) {
-	    coeff[i] = pmod->coeff[i];
-	    b[i] = 0.0;
+	for (i=0; i<nc; i++) {
+	    theta[i] = pmod->coeff[i];
 	}
 
 	/* initialize variance parameters */
-	amax[0] = vparm_init[0];
-	for (i=0; i<p+q; i++) {
-	    amax[i+3] = vparm_init[i+1];
+	for (i=0; i<p+q+1; i++) {
+	    theta[i+nc] = vparm_init[i];
 	}
 
 	if (opt & OPT_V) {
-	    garch_print_init(coeff, ncoeff, amax, p, q, 0, prn);
+	    garch_print_init(theta, nc, p, q, 0, prn);
 	}
     }
 
-#ifdef USE_FCP
-    err = garch_estimate(t1 + pad, t2 + pad, bign, 
-			 (const double **) X, nx, coeff, ncoeff, 
-			 vcv, res2, res, h, y, amax, b, scale, &iters,
-			 prn, vopt);
-#else
-    if (getenv("FCP_GARCH") != NULL) {
-	err = garch_estimate(t1 + pad, t2 + pad, bign, 
-			     (const double **) X, nx, coeff, ncoeff, 
-			     vcv, res2, res, h, y, amax, b, scale, &iters,
-			     prn, vopt);
+    if (libset_get_bool("fcp")) {
+	err = garch_estimate(y, (const double **) X,
+			     t1 + pad, t2 + pad, bign, nc, 
+			     p, q, theta, V, e, e2, h,
+			     scale, &ll, &iters, vopt, prn);
     } else {
-	err = garch_estimate_mod(t1 + pad, t2 + pad, bign, 
-				 (const double **) X, nx, coeff, ncoeff, 
-				 vcv, res2, res, h, y, amax, b, scale, &fnc,
-				 &grc, prn, vopt);
+	err = garch_estimate_mod(y, (const double **) X,
+				 t1 + pad, t2 + pad, bign, nc, 
+				 p, q, theta, V, e, e2, h, 
+				 scale, &ll, &fnc, &grc, vopt, prn);
     }
-#endif
 
     if (err != 0) {
 	pmod->errcode = err;
     } else {
-	int nparam = ncoeff + p + q + 1;
-
-	for (i=1; i<=nparam; i++) {
-	    if (i <= ncoeff) {
-		amax[i] *= scale;
-		amax[i + nparam] *= scale;
-	    } else if (i == ncoeff + 1) {
-		amax[i] *= scale * scale;
-		amax[i + nparam] *= scale * scale;
-	    }
-	    pprintf(prn, "theta[%d]: %#14.6g (%#.6g)\n", i-1, amax[i], 
-		    amax[i + nparam]);
-	}
-	pputc(prn, '\n');
-
-	pmod->lnL = amax[0];
-	write_garch_stats(pmod, (const double **) Z, scale, pdinfo, 
-			  list, amax, nparam, pad, res, h);
-	make_packed_vcv(pmod, vcv, nparam, ncoeff, scale);
+	pmod->lnL = ll;
+	write_garch_stats(pmod, list, (const double **) Z, pdinfo, 
+			  theta, V, scale, e, h, npar, nc, pad,
+			  prn);
 	if (iters > 0) {
 	    gretl_model_set_int(pmod, "iters", iters);
 	} else if (fnc > 0) {
@@ -492,17 +458,15 @@ int do_fcp (const int *list, double **Z, double scale,
 
  bailout:
 
-    free(res2);
-    free(res);
+    free(e);
+    free(e2);
     free(h);
-    free(amax);    
-    free(coeff);
-    free(b);
-    free(vcv); 
+    free(theta);
+    gretl_matrix_free(V);
 
     if (pad > 0) {
 	free(y);
-	doubles_array_free(X, nx);
+	doubles_array_free(X, nc);
     } else {
 	free(X);
     }
@@ -556,7 +520,7 @@ garchpar_from_armapar (const double *armapar, int q, int p)
     int mo = q;
     int i;
 
-#ifdef VPARM_DEBUG
+#if VPARM_DEBUG
     for (i=0; i<1+ao+mo; i++) {
 	fprintf(stderr, "armapar[%d] = %#12.6g\n", i, armapar[i]);
     }
@@ -580,7 +544,7 @@ garchpar_from_armapar (const double *armapar, int q, int p)
 	sum_ab += vparm_init[p+i];
     }
 
-#ifdef VPARM_DEBUG
+#if VPARM_DEBUG
     fprintf(stderr, "sum_ab = %#12.6g\n", sum_ab);
 #endif
 
@@ -625,7 +589,7 @@ garch_init_by_arma (const MODEL *pmod, const int *garchlist,
     /* dep var is squared OLS residual */
     list[4] = v;
 
-    amod = arma(list, (const double **) *pZ, pdinfo, OPT_C, NULL);
+    amod = arma(list, NULL, (const double **) *pZ, pdinfo, OPT_C, NULL);
     if (amod.errcode) {
 	err = amod.errcode;
 	goto bailout;
@@ -764,7 +728,7 @@ MODEL garch_model (const int *cmdlist, double ***pZ, DATAINFO *pdinfo,
     double LMF = NADBL;
     double pvF = NADBL;
     double scale = 1.0;
-    int t, err, init_err, yno = 0;
+    int i, t, err, init_err, yno = 0;
 
     gretl_model_init(&model);
 
@@ -814,9 +778,16 @@ MODEL garch_model (const int *cmdlist, double ***pZ, DATAINFO *pdinfo,
 #endif 
 
     /* default variance parameter initialization */
-    vparm_init[1] = 0.2;
-    vparm_init[list[2]+1] = 0.7;
+    int q = list[1], p = list[2];
+
     vparm_init[0] = model.sigma * model.sigma * 0.1;
+    if (q>0) {
+	vparm_init[p+1] = 0.7;
+    }
+    vparm_init[1] = (q==0 ? 0.9 : 0.2) / p;
+    for (i=2; i<=p; i++) {
+	vparm_init[i] = vparm_init[1];
+    }
 
     if (opt & OPT_A) {
 	/* "--arma-init": try initializing params via ARMA */
@@ -824,7 +795,7 @@ MODEL garch_model (const int *cmdlist, double ***pZ, DATAINFO *pdinfo,
 				      pZ, pdinfo);
     }
 
-    do_fcp(list, *pZ, scale, pdinfo, &model, prn, opt); 
+    garch_driver(list, *pZ, scale, pdinfo, &model, prn, opt); 
 
     if (scale != 1.0) {
 	/* undo scaling of dependent variable */
