@@ -1,34 +1,30 @@
-/*
- *  Copyright (c) by Allin Cottrell 2002
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
- *
- *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
- *
+/* 
+ *  gretl -- Gnu Regression, Econometrics and Time-series Library
+ *  Copyright (C) 2001 Allin Cottrell and Riccardo "Jack" Lucchetti
+ * 
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ * 
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ * 
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 #include <gtk/gtk.h>
 
 #include "libgretl.h"
-
-#include <libxml/xmlmemory.h>
-#include <libxml/parser.h>
+#include "gretl_xml.h"
 #include "importer.h"
 
 #undef IDEBUG
 
-#define UTF const xmlChar *
-
 /* from gnumeric's value.h */
 typedef enum {
     VALUE_EMPTY     = 10,
@@ -47,11 +43,13 @@ static void wsheet_init (wsheet *sheet)
 {
     sheet->maxcol = sheet->maxrow = 0;
     sheet->text_cols = sheet->text_rows = 0;
+    sheet->colheads = 0;
+    sheet->ID = 0;
+    sheet->flags = 0;
+    sheet->name = NULL;
     sheet->Z = NULL;
     sheet->varname = NULL;
     sheet->label = NULL;
-    sheet->name = NULL;
-    sheet->flags = 0;
 }
 
 static void wsheet_free (wsheet *sheet)
@@ -122,7 +120,6 @@ static void wsheet_print_info (wsheet *sheet)
                              (v) == VALUE_INTEGER || \
                              (v) == VALUE_FLOAT)
 
-
 static int wsheet_allocate (wsheet *sheet, int cols, int rows)
 {
     int i, j, t;
@@ -132,11 +129,11 @@ static int wsheet_allocate (wsheet *sheet, int cols, int rows)
 	    cols, rows);
 #endif
 
-    sheet->Z = malloc(cols * sizeof *(sheet->Z));
+    sheet->Z = malloc(cols * sizeof *sheet->Z);
     if (sheet->Z == NULL) return 1;
 
     for (i=0; i<cols; i++) {
-	sheet->Z[i] = malloc(rows * sizeof **(sheet->Z));
+	sheet->Z[i] = malloc(rows * sizeof **sheet->Z);
 	if (sheet->Z[i] == NULL) {
 	    for (j=0; j<i; j++) {
 		free(sheet->Z[j]);
@@ -153,7 +150,7 @@ static int wsheet_allocate (wsheet *sheet, int cols, int rows)
     if (sheet->varname == NULL) return 1;
 
     for (i=0; i<cols; i++) {
-	sheet->varname[i] = malloc(VNAMELEN * sizeof **(sheet->varname));
+	sheet->varname[i] = malloc(VNAMELEN * sizeof **sheet->varname);
 	if (sheet->varname[i] == NULL) {
 	    for (j=0; j<i; j++) {
 		free(sheet->varname[j]);
@@ -191,10 +188,10 @@ static int wsheet_get_real_size (xmlNodePtr node, wsheet *sheet)
     sheet->maxcol = 0;
 
     while (p) {
-	if (!xmlStrcmp(p->name, (UTF) "Cell")) {
+	if (!xmlStrcmp(p->name, (XUC) "Cell")) {
 	    int i, j;
 
-	    tmp = (char *) xmlGetProp(p, (UTF) "Row");
+	    tmp = (char *) xmlGetProp(p, (XUC) "Row");
 	    if (tmp) {
 		i = atoi(tmp);
 		free(tmp);
@@ -202,7 +199,7 @@ static int wsheet_get_real_size (xmlNodePtr node, wsheet *sheet)
 		    sheet->maxrow = i;
 		}
 	    }
-	    tmp = (char *) xmlGetProp(p, (UTF) "Col");
+	    tmp = (char *) xmlGetProp(p, (XUC) "Col");
 	    if (tmp) {
 		j = atoi(tmp);
 		free(tmp);
@@ -276,26 +273,26 @@ static int wsheet_parse_cells (xmlNodePtr node, wsheet *sheet, PRN *prn)
     sheet->colheads = 0;
 
     while (p && !err) {
-	if (!xmlStrcmp(p->name, (UTF) "Cell")) {
+	if (!xmlStrcmp(p->name, (XUC) "Cell")) {
 	    int i_real = 0, t_real = 0;
 
 	    x = NADBL;
 	    i = 0; t = 0;
 
-	    tmp = (char *) xmlGetProp(p, (UTF) "Col");
+	    tmp = (char *) xmlGetProp(p, (XUC) "Col");
 	    if (tmp) {
 		i = atoi(tmp);
 		i_real = i - colmin;
 		free(tmp);
 	    }
-	    tmp = (char *) xmlGetProp(p, (UTF) "Row");
+	    tmp = (char *) xmlGetProp(p, (XUC) "Row");
 	    if (tmp) {
 		t = atoi(tmp);
 		t_real = t - rowmin;
 		free(tmp);
 	    }
 	    if (i_real >= 0 && t_real >= 0) {
-		tmp = (char *) xmlGetProp(p, (UTF) "ValueType");
+		tmp = (char *) xmlGetProp(p, (XUC) "ValueType");
 
 		if (tmp) {
 		    vtype = atoi(tmp);
@@ -316,7 +313,7 @@ static int wsheet_parse_cells (xmlNodePtr node, wsheet *sheet, PRN *prn)
 		    }
 
 		    if (i_real == 0 && t_real == 1 && VTYPE_IS_NUMERIC(vtype)) {
-			char *fmt = (char *) xmlGetProp(p, (UTF) "ValueFormat");
+			char *fmt = (char *) xmlGetProp(p, (XUC) "ValueFormat");
 
 			if (fmt) {
 			    check_for_date_format(sheet, fmt);
@@ -362,6 +359,7 @@ static int wsheet_parse_cells (xmlNodePtr node, wsheet *sheet, PRN *prn)
 		sheet->text_cols += 1;
 	    }
 	}
+
 	for (t=0; t<rows; t++) {
 	    if (toprows[t]) {
 		sheet->text_rows += 1;
@@ -373,6 +371,7 @@ static int wsheet_parse_cells (xmlNodePtr node, wsheet *sheet, PRN *prn)
 	    pputc(prn, '\n');
 	    err = 1;
 	}
+
 	if (sheet->text_cols > 1) {
 	    pputs(prn, _("Found an extraneous column of text"));
 	    pputc(prn, '\n');
@@ -409,7 +408,7 @@ static int wsheet_get_data (const char *fname, wsheet *sheet, PRN *prn)
 	return 1;
     }
 
-    if (xmlStrcmp(cur->name, (UTF) "Workbook")) {
+    if (xmlStrcmp(cur->name, (XUC) "Workbook")) {
         pputs(prn, _("File of the wrong type, root node not Workbook"));
 	xmlFreeDoc(doc);
 	return 1;
@@ -418,17 +417,17 @@ static int wsheet_get_data (const char *fname, wsheet *sheet, PRN *prn)
     /* Now walk the tree */
     cur = cur->xmlChildrenNode;
     while (!err && cur != NULL && !got_sheet) {
-	if (!xmlStrcmp(cur->name, (UTF) "Sheets")) {
+	if (!xmlStrcmp(cur->name, (XUC) "Sheets")) {
 	    int sheetcount = 0;
 
 	    sub = cur->xmlChildrenNode;
 
 	    while (sub != NULL && !got_sheet && !err) {
-		if (!xmlStrcmp(sub->name, (UTF) "Sheet")) {
+		if (!xmlStrcmp(sub->name, (XUC) "Sheet")) {
 		    xmlNodePtr snode = sub->xmlChildrenNode;
 
 		    while (snode != NULL && !err) {
-			if (!xmlStrcmp(snode->name, (UTF) "Name")) {
+			if (!xmlStrcmp(snode->name, (XUC) "Name")) {
 			    sheetcount++;
 			    tmp = (char *) xmlNodeGetContent(snode);
 			    if (tmp) {
@@ -438,19 +437,19 @@ static int wsheet_get_data (const char *fname, wsheet *sheet, PRN *prn)
 				}
 				free(tmp);
 			    }
-			} else if (got_sheet && !xmlStrcmp(snode->name, (UTF) "MaxCol")) {
+			} else if (got_sheet && !xmlStrcmp(snode->name, (XUC) "MaxCol")) {
 			    tmp = (char *) xmlNodeGetContent(snode);
 			    if (tmp) {
 				sheet->maxcol = atoi(tmp);
 				free(tmp);
 			    }
-			} else if (got_sheet && !xmlStrcmp(snode->name, (UTF) "MaxRow")) {
+			} else if (got_sheet && !xmlStrcmp(snode->name, (XUC) "MaxRow")) {
 			    tmp = (char *) xmlNodeGetContent(snode);
 			    if (tmp) {
 				sheet->maxrow = atoi(tmp);
 				free(tmp);
 			    }
-			} else if (got_sheet && !xmlStrcmp(snode->name, (UTF) "Cells")) {
+			} else if (got_sheet && !xmlStrcmp(snode->name, (XUC) "Cells")) {
 			    wsheet_get_real_size(snode, sheet);
 			    err = wsheet_parse_cells(snode, sheet, prn);
 			}
@@ -515,7 +514,7 @@ static int wbook_get_info (const char *fname, wbook *book, PRN *prn)
 	return 1;
     }
 
-    if (xmlStrcmp(cur->name, (UTF) "Workbook")) {
+    if (xmlStrcmp(cur->name, (XUC) "Workbook")) {
         pputs(prn, _("File of the wrong type, root node not Workbook"));
 	xmlFreeDoc(doc);
 	return 1;
@@ -524,11 +523,11 @@ static int wbook_get_info (const char *fname, wbook *book, PRN *prn)
     /* Now walk the tree */
     cur = cur->xmlChildrenNode;
     while (cur != NULL && !got_index && !err) {
-        if (!xmlStrcmp(cur->name, (UTF) "SheetNameIndex")) {
+        if (!xmlStrcmp(cur->name, (XUC) "SheetNameIndex")) {
 	    got_index = 1;
 	    sub = cur->xmlChildrenNode;
 	    while (sub != NULL && !err) {
-		if (!xmlStrcmp(sub->name, (UTF) "SheetName")) {
+		if (!xmlStrcmp(sub->name, (XUC) "SheetName")) {
 		    tmp = (char *) xmlNodeGetContent(sub);
 		    if (tmp != NULL) {
 			if (wbook_record_name(tmp, book)) {
@@ -640,7 +639,7 @@ sheet_time_series_setup (wsheet *sheet, wbook *book, DATAINFO *newinfo, int pd)
 }
 
 static int 
-real_wbook_get_data (const char *fname, double ***pZ, DATAINFO *pdinfo,
+real_wbook_get_data (const char *fname, double ***pZ, DATAINFO **ppdinfo,
 		     int gui, PRN *prn)
 {
     wbook gbook;
@@ -725,7 +724,7 @@ real_wbook_get_data (const char *fname, double ***pZ, DATAINFO *pdinfo,
 	    }
 	}
 
-	if (book_numeric_dates(book)) {
+	if (book != NULL && book_numeric_dates(book)) {
 	    pd = pd_from_numeric_dates(nrows, r0, 0, sheet->label, book);
 	} else if (sheet->colheads > 0) {
 	    if (obs_column_heading(sheet->label[0])) {
@@ -754,16 +753,12 @@ real_wbook_get_data (const char *fname, double ***pZ, DATAINFO *pdinfo,
 	    goto getout;
 	}
 
-	if (!book_time_series(book)) {
-	    strcpy(newinfo->stobs, "1");
-	    sprintf(newinfo->endobs, "%d", newinfo->n);
-	    newinfo->sd0 = 1.0;
-	    newinfo->pd = 1;
-	    newinfo->structure = CROSS_SECTION;
-	} else {
+	if (book_time_series(book)) {
 	    ntodate_full(newinfo->endobs, newinfo->n - 1, newinfo);
 	    fprintf(stderr, "endobs='%s'\n", newinfo->endobs);
-	}
+	} else {
+	    dataset_obs_info_default(newinfo);
+	} 
 
 	j = 1;
 
@@ -812,13 +807,7 @@ real_wbook_get_data (const char *fname, double ***pZ, DATAINFO *pdinfo,
 	    }
 	}
 
-	if (*pZ == NULL) {
-	    *pZ = newZ;
-	    *pdinfo = *newinfo;
-	    free(newinfo);
-	} else {
-	    err = merge_data(pZ, pdinfo, newZ, newinfo, prn);
-	}
+	err = merge_or_replace_data(pZ, ppdinfo, &newZ, &newinfo, prn);
     } 
 
  getout:
@@ -828,22 +817,22 @@ real_wbook_get_data (const char *fname, double ***pZ, DATAINFO *pdinfo,
 
     gretl_pop_c_numeric_locale();
 
-    if (err) {
-	free(newinfo);
+    if (err && newinfo != NULL) {
+	destroy_dataset(newZ, newinfo);
     }
 
     return err;
 }
 
-int wbook_get_data (const char *fname, double ***pZ, DATAINFO *pdinfo,
+int wbook_get_data (const char *fname, double ***pZ, DATAINFO **ppdinfo,
 		    PRN *prn)
 {
-    return real_wbook_get_data(fname, pZ, pdinfo, 1, prn);
+    return real_wbook_get_data(fname, pZ, ppdinfo, 1, prn);
 }
 
-int cli_get_gnumeric (const char *fname, double ***pZ, DATAINFO *pdinfo,
+int cli_get_gnumeric (const char *fname, double ***pZ, DATAINFO **ppdinfo,
 		      PRN *prn)
 {
-    return real_wbook_get_data(fname, pZ, pdinfo, 0, prn);
+    return real_wbook_get_data(fname, pZ, ppdinfo, 0, prn);
 }
 
