@@ -1,43 +1,30 @@
-/*
- *  Copyright (c) by Allin Cottrell 2002
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
- *
- *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
- *
+/* 
+ *  gretl -- Gnu Regression, Econometrics and Time-series Library
+ *  Copyright (C) 2001 Allin Cottrell and Riccardo "Jack" Lucchetti
+ * 
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ * 
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ * 
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
-#ifdef EXCEL_IMPORTER
-
-static void set_all_missing (double **Z, DATAINFO *pdinfo)
-{
-    int i, t;
-
-    for (i=1; i<pdinfo->v; i++) {
-	for (t=0; t<pdinfo->n; t++) {
-	    Z[i][t] = NADBL;
-	}
-    }
-}
-
-#endif /* EXCEL_IMPORTER */
-
 static void invalid_varname (PRN *prn)
 {
     pputs(prn, gretl_errmsg_get());
     pputs(prn, _("\nPlease rename this variable and try again"));
 }
 
+#ifndef ODS_IMPORTER
+
 static int label_is_date (char *str, int *submax)
 {
     int len = strlen(str);
@@ -376,6 +363,8 @@ static void wbook_free (wbook *book)
     free(book->missmask);
 }
 
+#endif /* !ODS_IMPORTER */
+
 static void wbook_init (wbook *book)
 {
     book->version = 0;
@@ -388,6 +377,8 @@ static void wbook_init (wbook *book)
     book->xf_list = NULL;
     book->totmiss = 0;
     book->missmask = NULL;
+    book->get_min_offset = NULL;
+    book->data = NULL;
 }
 
 static const char *column_label (int col)
@@ -472,6 +463,15 @@ void debug_callback (GtkWidget *w, wbook *book)
 
 #endif /* EXCEL_IMPORTER */
 
+static int book_get_min_offset (wbook *book, int k)
+{
+    if (book->get_min_offset != NULL) {
+	return book->get_min_offset(book, k);
+    } else {
+	return 1;
+    }
+}
+
 static
 void wsheet_menu_select_row (GtkTreeSelection *selection, wbook *book)
 {
@@ -480,10 +480,31 @@ void wsheet_menu_select_row (GtkTreeSelection *selection, wbook *book)
     GtkTreePath *path;
     gint *idx;
 
-    gtk_tree_selection_get_selected (selection, &model, &iter);
-    path = gtk_tree_model_get_path (model, &iter);
+    gtk_tree_selection_get_selected(selection, &model, &iter);
+    path = gtk_tree_model_get_path(model, &iter);
     idx = gtk_tree_path_get_indices(path);
-    book->selected = idx[0];
+
+    if (book->selected != idx[0]) {
+	int offmin, offcurr;
+
+	book->selected = idx[0];
+
+	offmin = book_get_min_offset(book, COL_OFFSET);
+	offcurr = gtk_spin_button_get_value_as_int
+	    (GTK_SPIN_BUTTON(book->colspin));
+	if (offcurr != offmin) {
+	    gtk_spin_button_set_value(GTK_SPIN_BUTTON(book->colspin),
+				      offmin);
+	}
+
+	offmin = book_get_min_offset(book, ROW_OFFSET);
+	offcurr = gtk_spin_button_get_value_as_int
+	    (GTK_SPIN_BUTTON(book->rowspin));
+	if (offcurr != offmin) {
+	    gtk_spin_button_set_value(GTK_SPIN_BUTTON(book->rowspin),
+				      offmin);
+	}	
+    }
 }
 
 static 
@@ -502,9 +523,9 @@ void wsheet_menu_make_list (GtkTreeView *view, wbook *book)
 			   0, book->sheetnames[i], -1);
     }
 
-    gtk_tree_model_get_iter_first (model, &iter);
-    gtk_tree_selection_select_iter (gtk_tree_view_get_selection (view), 
-				    &iter);
+    gtk_tree_model_get_iter_first(model, &iter);
+    gtk_tree_selection_select_iter(gtk_tree_view_get_selection(view), 
+				   &iter);
 }
 
 static 
@@ -514,14 +535,14 @@ void wsheet_menu_cancel (GtkWidget *w, wbook *book)
 }
 
 static 
-void wbook_get_col_offset (GtkWidget *w, wbook *book)
+void wbook_set_col_offset (GtkWidget *w, wbook *book)
 {
     book->col_offset = gtk_spin_button_get_value_as_int
 	(GTK_SPIN_BUTTON(book->colspin)) - 1;
 }
 
 static 
-void wbook_get_row_offset (GtkWidget *w, wbook *book)
+void wbook_set_row_offset (GtkWidget *w, wbook *book)
 {
     book->row_offset = gtk_spin_button_get_value_as_int
 	(GTK_SPIN_BUTTON(book->rowspin)) - 1;
@@ -551,9 +572,9 @@ void add_sheets_list (GtkWidget *vbox, wbook *book)
 
     select = gtk_tree_view_get_selection(GTK_TREE_VIEW(view));
     gtk_tree_selection_set_mode (select, GTK_SELECTION_SINGLE);
-    g_signal_connect (G_OBJECT(select), "changed",
-		      G_CALLBACK(wsheet_menu_select_row),
-		      book);
+    g_signal_connect(G_OBJECT(select), "changed",
+		     G_CALLBACK(wsheet_menu_select_row),
+		     book);
 
     wsheet_menu_make_list(GTK_TREE_VIEW(view), book);
 
@@ -581,6 +602,7 @@ static void wsheet_menu (wbook *book, int multisheet)
     GtkWidget *w, *tmp, *label;
     GtkWidget *vbox, *hbox;
     GtkObject *c_adj, *r_adj;
+    int offmin;
 
     w = gtk_dialog_new();
     gtk_window_set_title(GTK_WINDOW(w), _("gretl: spreadsheet import"));
@@ -602,10 +624,11 @@ static void wsheet_menu (wbook *book, int multisheet)
 
     /* starting column spinner */
     tmp = gtk_label_new(_("column:"));
-    c_adj = gtk_adjustment_new(1, 1, 256, 1, 1, 1);
+    offmin = book->col_offset + 1;
+    c_adj = gtk_adjustment_new(offmin, offmin, 256, 1, 1, 1);
     book->colspin = gtk_spin_button_new(GTK_ADJUSTMENT(c_adj), 1, 0);
     g_signal_connect(c_adj, "value_changed",
-		     G_CALLBACK(wbook_get_col_offset), book);
+		     G_CALLBACK(wbook_set_col_offset), book);
     gtk_spin_button_set_update_policy(GTK_SPIN_BUTTON(book->colspin),
 				      GTK_UPDATE_IF_VALID);
     gtk_box_pack_start(GTK_BOX(hbox), tmp, FALSE, FALSE, 5);
@@ -613,10 +636,11 @@ static void wsheet_menu (wbook *book, int multisheet)
 
     /* starting row spinner */
     tmp = gtk_label_new(_("row:"));
-    r_adj = gtk_adjustment_new(1, 1, 256, 1, 1, 1);
-    book->rowspin = gtk_spin_button_new (GTK_ADJUSTMENT(r_adj), 1, 0);
+    offmin = book->row_offset + 1;
+    r_adj = gtk_adjustment_new(offmin, offmin, 256, 1, 1, 1);
+    book->rowspin = gtk_spin_button_new(GTK_ADJUSTMENT(r_adj), 1, 0);
     g_signal_connect(r_adj, "value_changed",
-		     G_CALLBACK(wbook_get_row_offset), book);
+		     G_CALLBACK(wbook_set_row_offset), book);
     gtk_spin_button_set_update_policy(GTK_SPIN_BUTTON(book->rowspin),
 				      GTK_UPDATE_IF_VALID);
     gtk_box_pack_start(GTK_BOX(hbox), tmp, FALSE, FALSE, 5);
@@ -627,8 +651,8 @@ static void wsheet_menu (wbook *book, int multisheet)
     gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
     label = gtk_label_new("(A)");
     gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 5);
-    g_signal_connect (GTK_EDITABLE(book->colspin), "changed",
-		      G_CALLBACK (colspin_changed), label);
+    g_signal_connect(GTK_EDITABLE(book->colspin), "changed",
+		     G_CALLBACK(colspin_changed), label);
 
     /* choose the worksheet (if applicable) */
     if (multisheet) {
