@@ -1,20 +1,20 @@
-/*
- *  Copyright (c) by Allin Cottrell 2002
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
- *
- *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
- *
+/* 
+ *  gretl -- Gnu Regression, Econometrics and Time-series Library
+ *  Copyright (C) 2001 Allin Cottrell and Riccardo "Jack" Lucchetti
+ * 
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ * 
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ * 
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 typedef struct wbook_ wbook;
@@ -26,9 +26,15 @@ typedef enum {
     BOOK_AUTO_VARNAMES   = 1 << 2,
     BOOK_TIME_SERIES     = 1 << 3,
     BOOK_OBS_LABELS      = 1 << 4,
-    BOOK_DEBUG           = 1 << 5
+    BOOK_OBS_BLANK       = 1 << 5,
+    BOOK_DEBUG           = 1 << 6
 } BookFlag;
 
+enum {
+    COL_OFFSET,
+    ROW_OFFSET
+};
+
 #define book_numeric_dates(b) ((b)->flags & BOOK_NUMERIC_DATES)
 #define book_base_1904(b)     ((b)->flags & BOOK_DATE_BASE_1904)
 #define book_auto_varnames(b) ((b)->flags & BOOK_AUTO_VARNAMES)
@@ -57,6 +63,8 @@ struct wbook_ {
     BookFlag flags;
     int totmiss;
     char *missmask;
+    int (*get_min_offset)();
+    void *data;
 };
 
 struct wsheet_ {
