@@ -1,20 +1,20 @@
-/*
- *  Copyright (c) 2003 by Allin Cottrell
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
- *
- *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- *
+/* 
+ *  gretl -- Gnu Regression, Econometrics and Time-series Library
+ *  Copyright (C) 2001 Allin Cottrell and Riccardo "Jack" Lucchetti
+ * 
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ * 
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ * 
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 #include "libgretl.h"
@@ -233,7 +233,7 @@ static int leverage_plot (const MODEL *pmod, gretl_matrix *S,
 
     /* upper plot: leverage factor */
     fputs("set origin 0.0,0.50\n", fp);
-    fputs("set missing '?'\n", fp);
+    gnuplot_missval_string(fp);
     fputs("set yrange [0:1]\n", fp);
     fprintf(fp, "set title '%s'\n", G_("leverage"));
     fputs("plot \\\n'-' using 1:2 w impulses\n", fp);
@@ -259,7 +259,7 @@ static int leverage_plot (const MODEL *pmod, gretl_matrix *S,
 
     /* lower plot: influence factor */
     fputs("set origin 0.0,0.0\n", fp);
-    fputs("set missing '?'\n", fp);
+    gnuplot_missval_string(fp);
     fputs("set yrange [*:*]\n", fp);
     fprintf(fp, "set title '%s'\n", G_("influence")); 
     fputs("plot \\\n'-' using 1:2 w impulses\n", fp);
