@@ -1,20 +1,20 @@
-/*
- *  Copyright (c) 2002 by Allin Cottrell
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
- *
- *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- *
+/* 
+ *  gretl -- Gnu Regression, Econometrics and Time-series Library
+ *  Copyright (C) 2001 Allin Cottrell and Riccardo "Jack" Lucchetti
+ * 
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ * 
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ * 
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 /* mp_ols.c - gretl least squares with multiple precision (GMP) */
@@ -80,8 +80,7 @@ static void set_gretl_mp_bits (void);
 static void set_gretl_mpfr_bits (void);
 #endif
 static MPXPXXPY mp_xpxxpy_func (const int *list, int n, mpf_t **mpZ);
-static void mp_regress (MPMODEL *pmod, MPXPXXPY xpxxpy, mpf_t **mpZ, int n,
-			char *errbuf);
+static void mp_regress (MPMODEL *pmod, MPXPXXPY xpxxpy, char *errbuf);
 static MPCHOLBETA mp_cholbeta (MPXPXXPY xpxxpy);
 static void mp_diaginv (MPXPXXPY xpxxpy, mpf_t *diag);
 static int mp_rearrange (int *list);
@@ -1062,7 +1061,7 @@ int mplsq (const int *list, const int *polylist,
     xpxxpy = mp_xpxxpy_func(mpmod.list, mpmod.nobs, mpZ);
     mpf_set(mpmod.tss, xpxxpy.xpy[l0]);
 
-    mp_regress(&mpmod, xpxxpy, mpZ, mpmod.nobs, errbuf);
+    mp_regress(&mpmod, xpxxpy, errbuf);
 
     for (i=0; i<=l0; i++) {
 	mpf_clear(xpxxpy.xpy[i]);
@@ -1180,8 +1179,7 @@ static MPXPXXPY mp_xpxxpy_func (const int *list, int n, mpf_t **mpZ)
     return xpxxpy; 
 }
 
-static void mp_regress (MPMODEL *pmod, MPXPXXPY xpxxpy, mpf_t **mpZ, int n,
-			char *errbuf)
+static void mp_regress (MPMODEL *pmod, MPXPXXPY xpxxpy, char *errbuf)
 {
     int i, v, nobs, nv, yno;
     mpf_t *diag, ysum, ypy, zz, rss, tss;
@@ -1192,7 +1190,8 @@ static void mp_regress (MPMODEL *pmod, MPXPXXPY xpxxpy, mpf_t **mpZ, int n,
     nv = xpxxpy.nv;
     yno = pmod->list[1];
 
-    if ((pmod->sderr = malloc(nv * sizeof *pmod->sderr)) == NULL) {
+    pmod->sderr = malloc(nv * sizeof *pmod->sderr);
+    if (pmod->sderr == NULL) {
         pmod->errcode = E_ALLOC;
         return;
     }
@@ -1223,7 +1222,7 @@ static void mp_regress (MPMODEL *pmod, MPXPXXPY xpxxpy, mpf_t **mpZ, int n,
     mpf_set(ysum, xpxxpy.xpy[0]);
     mpf_set(ypy, xpxxpy.xpy[nv + 1]);
     if (mpf_sgn(ypy) == 0) { 
-        pmod->errcode = E_YPY;
+        pmod->errcode = E_ZERO;
         return; 
     }
 
