@@ -4,7 +4,7 @@
    Based on stataread.c from the GNU R "foreign" package with the 
    following original info:
 
-     * $Id: stata_import.c,v 1.16 2006/09/24 23:56:43 allin Exp $
+     * $Id: stata_import.c,v 1.20 2007/11/05 22:20:23 allin Exp $
   
      (c) 1999, 2000, 2001, 2002 Thomas Lumley. 
      2000 Saikat DebRoy
@@ -265,7 +265,8 @@ static int check_variable_types (FILE *fp, int *types, int nvar, int *nsv)
 
 /* mechanism for handling (coding) non-numeric variables */
 
-static gretl_string_table *dta_make_string_table (int *types, int nvar, int ncols)
+static gretl_string_table *
+dta_make_string_table (int *types, int nvar, int ncols)
 {
     gretl_string_table *st;
     int *list;
@@ -541,8 +542,11 @@ static int read_dta_data (FILE *fp, double **Z, DATAINFO *dinfo,
 #endif
 		if (*strbuf != '\0' && strcmp(strbuf, ".") && *pst != NULL) {
 		    ix = gretl_string_table_index(*pst, strbuf, v, 0, prn);
-		    if (ix >= 0) {
+		    if (ix > 0) {
 			Z[v][t] = ix;
+			if (t == 0) {
+			    set_var_discrete(dinfo, v, 1);
+			}
 		    }	
 		}
 	    }
@@ -644,7 +648,7 @@ static int parse_dta_header (FILE *fp, int *namelen, int *nvar, int *nobs)
     return err;
 }
 
-int dta_get_data (const char *fname, double ***pZ, DATAINFO *pdinfo,
+int dta_get_data (const char *fname, double ***pZ, DATAINFO **ppdinfo,
 		  PRN *prn)
 {
     int namelen = 0;
@@ -711,15 +715,10 @@ int dta_get_data (const char *fname, double ***pZ, DATAINFO *pdinfo,
 
 	if (st != NULL) {
 	    gretl_string_table_print(st, newinfo, fname, prn);
+	    gretl_string_table_destroy(st);
 	}
 
-	if (*pZ == NULL) {
-	    *pZ = newZ;
-	    *pdinfo = *newinfo;
-	    free(newinfo);
-	} else {
-	    err = merge_data(pZ, pdinfo, newZ, newinfo, prn);
-	}
+	err = merge_or_replace_data(pZ, ppdinfo, &newZ, &newinfo, prn);
     }
 
     fclose(fp);
