@@ -1,20 +1,20 @@
-/*
- *  Copyright (c) by Allin Cottrell
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
- *
- *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
- *
+/* 
+ *  gretl -- Gnu Regression, Econometrics and Time-series Library
+ *  Copyright (C) 2001 Allin Cottrell and Riccardo "Jack" Lucchetti
+ * 
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ * 
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ * 
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 /* TRAMO/SEATS, X-12-ARIMA plugin for gretl */
@@ -98,6 +98,8 @@ static int glib_spawn (const char *workdir, const char *fmt, ...)
     }
 #endif
 
+    gretl_error_clear();
+
     signal(SIGCHLD, SIG_DFL);
 
     ok = g_spawn_sync (workdir,
@@ -112,14 +114,22 @@ static int glib_spawn (const char *workdir, const char *fmt, ...)
 		       &error);
 
     if (!ok) {
+	gretl_errmsg_set(error->message);
 	fprintf(stderr, "spawn: '%s'\n", error->message);
 	g_error_free(error);
 	ret = 1;
     } else if (serr && *serr) {
+	gretl_errmsg_set(serr);
 	fprintf(stderr, "stderr: '%s'\n", serr);
 	ret = 1;
     } else if (status != 0) {
-	fprintf(stderr, "status=%d: stdout: '%s'\n", status, sout);
+	if (sout && *sout) {
+	    gretl_errmsg_set(sout);
+	    fprintf(stderr, "spawn: status = %d: '%s'\n", status, sout);
+	} else {
+	    strcpy(gretl_errmsg, _("Command failed"));
+	    fprintf(stderr, "spawn: status = %d\n", status);
+	}
 	ret = 1;
     }
 
