@@ -336,9 +336,9 @@ static int zipfile_write_check (zfile *zf, int task, int *attr)
 	fmode = "r+";
     }
 
-    trace(2, "testing gretl_fopen on '%s', mode %s\n", zf->fname, fmode);
+    trace(2, "testing fopen on '%s', mode %s\n", zf->fname, fmode);
 
-    fp = gretl_fopen(zf->fname, fmode);
+    fp = fopen(zf->fname, fmode);
     if (fp == NULL) {
 	err = ziperr(ZE_CREAT, zf->fname);
     } else {
@@ -567,7 +567,7 @@ static int real_archive_files (const char *targ, const char **filenames,
 
     if (zfiles != NULL || zf.zstart) {
 	trace(1, "opening original zip file for reading\n");
-	fr = gretl_fopen(zf.fname, "rb");
+	fr = fopen(zf.fname, "rb");
 	if (fr == NULL) {
 	    err = ziperr(ZE_NAME, zf.fname);
 	    goto bailout;
@@ -581,7 +581,7 @@ static int real_archive_files (const char *targ, const char **filenames,
     }
 
     trace(1, "opening %s for writing\n", tempzip);
-    zf.fp = gretl_fopen(tempzip, "wb");
+    zf.fp = fopen(tempzip, "wb");
     if (zf.fp == NULL) {
 	err = ziperr(ZE_TEMP, tempzip);
 	goto bailout;
@@ -757,8 +757,9 @@ int zipfile_archive_files (const char *targ, const char **filenames,
 static int process_zipfile (zfile *zf, const char *targ, int task)
 {
     int err = 0;
-    
+
     zf->fname = g_strdup(targ);
+
     if (zf->fname == NULL) {
 	err = ziperr(ZE_MEM, "was processing arguments");
     }
@@ -1047,7 +1048,7 @@ static int real_delete_files (zfile *zf)
     }
 
     trace(1, "opening original zip file for reading\n");
-    fr = gretl_fopen(zf->fname, "rb");
+    fr = fopen(zf->fname, "rb");
     if (fr == NULL) {
 	return ziperr(ZE_NAME, zf->fname);
     }
@@ -1059,7 +1060,7 @@ static int real_delete_files (zfile *zf)
     }
 
     trace(1, "opening %s for writing\n", tempzip);
-    zf->fp = gretl_fopen(tempzip, "wb");
+    zf->fp = fopen(tempzip, "wb");
     if (zf->fp == NULL) {
 	fclose(fr);
 	ziperr(ZE_TEMP, tempzip);
