@@ -36,6 +36,7 @@
 
 #define MIN_DIGITS 4
 #define MP_CHECK_DIGITS 12
+#define MP_PRINT_DIGITS 15
 
 #ifdef USE_GMP
 # define LIBGRETLSTR "Standard libgretl:"
@@ -48,6 +49,62 @@ static int noint;
 
 static char datadir[FILENAME_MAX];
 
+typedef struct mp_results_ mp_results;
+
+struct mp_results_ {
+    int ncoeff;
+    double *coeff;
+    double *sderr;
+    double sigma;
+    double ess;
+    double rsq;
+    double fstt;
+};
+
+void free_mp_results (mp_results *mpvals)
+{
+    if (mpvals != NULL) {
+	free(mpvals->coeff);
+	free(mpvals->sderr);
+	free(mpvals);
+    }
+}
+
+mp_results *mp_results_new (int nc)
+{
+    mp_results *mpvals;
+    int i;
+
+    mpvals = malloc(sizeof *mpvals);
+    if (mpvals == NULL) {
+	return NULL;
+    }
+
+    mpvals->coeff = NULL;
+    mpvals->sderr = NULL;
+
+    mpvals->ncoeff = nc;
+
+    mpvals->coeff = malloc(nc * sizeof *mpvals->coeff);
+    mpvals->sderr = malloc(nc * sizeof *mpvals->sderr);
+
+    if (mpvals->coeff == NULL || 
+	mpvals->sderr == NULL) {
+	free_mp_results(mpvals);
+	return NULL;
+    }
+
+    for (i=0; i<nc; i++) {
+	mpvals->coeff[i] = NADBL;
+	mpvals->sderr[i] = NADBL;
+    }
+
+    mpvals->sigma = mpvals->ess = NADBL;
+    mpvals->rsq = mpvals->fstt = NADBL;
+
+    return mpvals;
+}
+
 /* special stuff:
 
 Noint1, NoInt2: no intercept, use alternative R^2 calculation   
@@ -103,10 +160,12 @@ int grab_nist_data (FILE *fp, double **Z, DATAINFO *dinfo,
 		pputs(prn, "Data ended prematurely\n");
 		return 1;
 	    } else {
+#if 1
 		d = get_data_digits(numstr);
 		if (digits != NULL && digits[i] != NULL) {
 		    digits[i][t] = d;
 		}
+#endif
 		xx = atof(numstr);
 	    }
 	    Z[i][t] = xx;
@@ -191,6 +250,38 @@ void get_difficulty_level (const char *line, char *s)
     }
 }
 
+#define mylog10(x) (log(x) / 2.3025850929940459)
+
+static double log_error (double q, double c, PRN *prn)
+{
+    double le = 0.0;
+    int lae = 0;
+
+    if (q == c) {
+	le = 15.0;
+    } else if (isinf(c)) {
+	/* certval is inf (e.g. F-stat in some cases): 
+	   can't really handle this? */
+	if (na(q) || isinf(q)) {
+	    le = 15.0;
+	} else {
+	    le = -log(0);
+	}
+    } else if (c == 0.0) {
+	le = -mylog10(fabs(q));
+    } else {
+	le = -mylog10(fabs(q - c) / fabs(c));
+    }
+
+    if (isnan(le)) {
+	pprintf(prn, "q = %g, c = %g\n", q, c);
+    } else {
+	pprintf(prn, "%10.3f %s\n", le, (lae)? "(log abs error)" : "");
+    }
+
+    return le;
+}
+
 static 
 void free_data_digits (DATAINFO *dinfo)
 {
@@ -271,12 +362,21 @@ int read_nist_file (const char *fname,
     if (verbose) pputc(prn, '\n');
 
     /* allow for generated data: powers of x */
-    if (strstr(fname, "Pontius")) npoly = 1;
-    if (strstr(fname, "Filip")) npoly = 9;
-    if (strstr(fname, "Wampler")) npoly = 4;
+    if (strstr(fname, "Pontius")) {
+	npoly = 1;
+    }
+    if (strstr(fname, "Filip")) {
+	npoly = 9;
+    }
+    if (strstr(fname, "Wampler")) {
+	npoly = 4;
+    }
 
-    if (strstr(fname, "NoInt")) noint = 1;
-    else noint = 0;
+    if (strstr(fname, "NoInt")) {
+	noint = 1;
+    } else {
+	noint = 0;
+    }
 
     *difficulty = 0;
     
@@ -311,7 +411,7 @@ int read_nist_file (const char *fname,
 	/* how many variables are there? */
 	if (nvar == 0 && (p = strstr(line, "Predictor")) != NULL) {
 	    if (sscanf(line, "%d Predictor Variable", &nvar) == 1) {
-		nvar++;
+		nvar++; 
 		if (verbose) pprintf(prn, " Number of variables: %d\n", nvar);
 	    }
 	}
@@ -325,10 +425,10 @@ int read_nist_file (const char *fname,
 
 	/* allocate results struct once we know its size */
 	if (nvar > 0 && nobs > 0 && certvals == NULL) {	
-	    int nc = nvar + npoly;
+	    int nc = nvar - 1 + npoly;
 	    
 	    if (!noint) nc++;
-	    certvals = gretl_mp_results_new(nc);
+	    certvals = mp_results_new(nc);
 	    if (certvals == NULL) {
 		fclose(fp);
 		return 1;
@@ -341,16 +441,18 @@ int read_nist_file (const char *fname,
 	    dinfo = create_new_dataset(&Z, nvar + 1 + npoly, 
 				       nobs, 0);
 	    if (dinfo == NULL) {
-		free_gretl_mp_results(certvals);
+		free_mp_results(certvals);
 		fclose(fp);
 		return 1;
 	    }
+#if 1
 	    if (allocate_data_digits(dinfo)) {
-		free_gretl_mp_results(certvals);
+		free_mp_results(certvals);
 		free_datainfo(dinfo);
 		fclose(fp);
 		return 1;
-	    }		
+	    }	
+#endif	
 	}
 
 	/* read the certified results */
@@ -378,11 +480,9 @@ int read_nist_file (const char *fname,
 		      "allocated: file is problematic\n");
 		fclose(fp);
 		return 1;
-	    } else {
-		if (grab_nist_data(fp, Z, dinfo, npoly, prn)) {
-		    fclose(fp);
-		    return 1;
-		}
+	    } else if (grab_nist_data(fp, Z, dinfo, npoly, prn)) {
+		fclose(fp);
+		return 1;
 	    } 
 	} /* end if ready to grab data */
 
@@ -401,7 +501,9 @@ int read_nist_file (const char *fname,
 	}
     }
 
-    if (npoly && verbose) pputc(prn, '\n');
+    if (npoly && verbose) {
+	pputc(prn, '\n');
+    }
 
     for (i=2; i<=npoly+1; i++) {
 	if (verbose) {
@@ -423,105 +525,59 @@ int read_nist_file (const char *fname,
     return 0;
 }
 
-static
-void print_result_error (int digits, 
-			 const char *v1, const char *v2, 
-			 const char *str,
-			 PRN *prn)
-{
-    if (verbose) {
-	pprintf(prn, "\nDisagreement at %d significant digits over %s:\n"
-		" Certified value = %s, libgretl value = %s\n",
-		digits, str, v1, v2);
-    }
-}
-
-static
-int doubles_differ (const char *v1, const char *v2)
-{
-    if ((!strcmp(v1, "inf") || !strcmp(v1, "nan")) && 
-	!strncmp(v2, "-999", 4)) {
-	return 0;
-    } else {
-	double diff = fabs(fabs(atof(v1)) - fabs(atof(v2)));
-    
-	return diff > DBL_EPSILON;
-    }
-}
-
-static
-int results_agree (MODEL *pmod, mp_results *certvals, DATAINFO *dinfo,
-		   int digits, PRN *prn)
+static 
+double get_accuracy (MODEL *pmod, mp_results *certvals, PRN *prn)
 {
+    PRN *vprn;
+    char label[32];
+    double le, lemin = 32;
     int i;
-    char v1[48], v2[48], s[24];
+
+    vprn = (verbose)? prn : NULL;
+
+    pprintf(vprn, "\nstatistic   log relative error\n\n");
 
     for (i=0; i<pmod->ncoeff; i++) {
-	sprintf(v1, "%#.*g", digits, certvals->coeff[i]);
-	sprintf(v2, "%#.*g", digits, pmod->coeff[i]);
-	if (doubles_differ(v1, v2)) {
-	    sprintf(s, "coeff[%d]", i);
-	    print_result_error(digits, v1, v2, s, prn);
-	    return 0;
-	}
-	/* special case: exact fit, zero standard error */
-	if (certvals->sderr[i] == 0.0) {
-	    double x = pow(10.0, -digits);
-
-	    if (pmod->sderr[i] < x) return 1;
-	}
-	sprintf(v1, "%#.*g", digits, certvals->sderr[i]);
-	sprintf(v2, "%#.*g", digits, pmod->sderr[i]);
-	if (doubles_differ(v1, v2)) {
-	    sprintf(s, "std_err[%d]", i);
-	    print_result_error(digits, v1, v2, s, prn);
-	    return 0; 
+	sprintf(label, "B[%d]", i);
+	pprintf(vprn, "%-12s", label);
+	le = log_error(pmod->coeff[i], certvals->coeff[i], vprn);
+	if (le < lemin) {
+	    lemin = le;
 	}
+	sprintf(label, "Std.Err.");
+	pprintf(vprn, "%-12s", label);
+	le = log_error(pmod->sderr[i], certvals->sderr[i], vprn);
+	if (le < lemin) {
+	    lemin = le;
+	}	
     }
 
-    sprintf(v1, "%#.*g", digits, certvals->sigma);
-    sprintf(v2, "%#.*g", digits, pmod->sigma);
-    if (doubles_differ(v1, v2)) {
-	print_result_error(digits, v1, v2, "sigma", prn);
-	return 0;
-    }
+    pprintf(vprn, "%-12s", "sigma");
+    le = log_error(pmod->sigma, certvals->sigma, vprn);
+    if (le < lemin) {
+	lemin = le;
+    }	
+    
 
-    sprintf(v1, "%#.*g", digits, certvals->ess);
-    sprintf(v2, "%#.*g", digits, pmod->ess);
-    if (doubles_differ(v1, v2)) {
-	print_result_error(digits, v1, v2, "ESS", prn);
-	return 0;
-    }
+    pprintf(vprn, "%-12s", "ESS");
+    le = log_error(pmod->ess, certvals->ess, vprn);
+    if (le < lemin) {
+	lemin = le;
+    }	
 
-    sprintf(v1, "%#.*g", digits, certvals->rsq);
-    sprintf(v2, "%#.*g", digits, pmod->rsq);
-    if (doubles_differ(v1, v2)) {
-	print_result_error(digits, v1, v2, "R-squared", prn);
-	return 0;
-    }
+    pprintf(vprn, "%-12s", "R-squared");
+    le = log_error(pmod->rsq, certvals->rsq, vprn);
+    if (le < lemin) {
+	lemin = le;
+    }	
 
-    sprintf(v1, "%#.*g", digits, certvals->fstt);
-    sprintf(v2, "%#.*g", digits, pmod->fstt);
-    if (doubles_differ(v1, v2)) {
-	print_result_error(digits, v1, v2, "F-stat", prn);
-	return 0;
-    }
+    pprintf(vprn, "%-12s", "F-stat");
+    le = log_error(pmod->fstt, certvals->fstt, vprn);
+    if (le < lemin) {
+	lemin = le;
+    }	
 
-    return 1;
-}
-
-static 
-int get_accuracy (MODEL *pmod, mp_results *certvals, DATAINFO *dinfo,
-		  PRN *prn)
-{
-    int digits;
-
-    for (digits=MAX_DIGITS; digits>=MIN_DIGITS; digits--) {
-	if (results_agree(pmod, certvals, dinfo, digits, prn)) {
-	    return digits;
-	}
-    }
-    return 0;
+    return lemin;
 }
 
 static 
@@ -560,50 +616,25 @@ void print_nist_summary (int ntests, int missing, int modelerrs,
 static void *get_mplsq (void **handle); 
 # endif
 
-static 
-int mp_vals_differ (double x, double y, double *diff, PRN *prn)
-     /* x is certified value, y is gretl MP value */
-{
-    char xstr[32], ystr[32];
-    int ret;
-
-    sprintf(xstr, "%#.*g", MP_CHECK_DIGITS, x);
-    sprintf(ystr, "%#.*g", MP_CHECK_DIGITS, y);
-
-    if (strcmp(xstr, "inf") == 0 && strncmp(ystr, "-999", 4) == 0) {
-	return 0;
-    }
-
-    if (x == 0.0 && y < DBL_EPSILON) return 0;
-
-    ret = (atof(xstr) != atof(ystr));
-
-    if (strcmp(xstr, "inf") && strncmp(xstr, "-999", 4)) { 
-	*diff = fabs (y - x);
-    }
-
-    if (ret && verbose && strcmp(xstr, "inf") && strncmp(xstr, "-999", 4)) {
-	pprintf(prn, " ** using gretl GMP plugin: results differ by "
-		"%#.*g\n", MP_CHECK_DIGITS, *diff);
-    }
-
-    return ret;
-}
-
 static
 int run_gretl_mp_comparison (double ***pZ, DATAINFO *dinfo, 
 			     mp_results *certvals, int npoly,
 			     int *mpfails, PRN *prn)
 {
+    PRN *vprn;
     void *handle = NULL;
-    int (*mplsq)(const int *, const int *, double ***, 
-                 DATAINFO *, PRN *, char *, mp_results *);
+    int (*mplsq)(const int *, const int *, const double **, 
+		 const DATAINFO *, char *, MODEL *, gretlopt);
     int *list = NULL, *polylist = NULL;
     char errbuf[MAXLEN];
     int i, err = 0;
-    int getvals = 1, realv = dinfo->v - npoly;
-    mp_results *gretlvals = NULL;
-    double diffmax = 0.0, diff = 0.0;
+    int realv = dinfo->v - npoly;
+    MODEL model;
+    double acc;
+
+    vprn = (verbose)? prn : NULL;
+
+    gretl_model_init(&model);
 
     /* create regression list */
     list = malloc((realv + 1) * sizeof *list);
@@ -636,8 +667,6 @@ int run_gretl_mp_comparison (double ***pZ, DATAINFO *dinfo,
 	}
     }
 
-    if (getvals) gretlvals = gretl_mp_results_new (certvals->ncoeff);
-
 #ifdef STANDALONE
     mplsq = get_mplsq(&handle);
 #else
@@ -649,102 +678,76 @@ int run_gretl_mp_comparison (double ***pZ, DATAINFO *dinfo,
     }
 
     if (!err) {
-        err = (*mplsq)(list, polylist, pZ, dinfo, prn, 
-                       errbuf, gretlvals); 
+        err = (*mplsq)(list, polylist, (const double **) *pZ, 
+		       dinfo, errbuf, &model, OPT_NONE); 
     }
 
     close_plugin(handle);
     free(list);
-    if (polylist) free(polylist);
+    free(polylist);
 
-    if (gretlvals != NULL) {
-
-	if (verbose) {
-	    pprintf(prn, "\nChecking gretl multiple-precision results:\n\n"
-		    "%44s%24s\n\n", "certified", "libgretl");
-	}
+    if (verbose) {
+	pprintf(prn, "\nChecking gretl multiple-precision results (%d coefficients):\n\n"
+		"%44s%24s\n\n", certvals->ncoeff, "certified", "libgretl");
 
 	for (i=0; i<certvals->ncoeff; i++) {
 	    char label[16];
 
-	    if (verbose && !na(certvals->coeff[i])) {
+	    if (!na(certvals->coeff[i])) {
 		sprintf(label, "B[%d] estimate", i);
 		pprintf(prn, " %-20s %#24.*g %#24.*g\n",
 			label,
-			MP_CHECK_DIGITS, certvals->coeff[i],
-			MP_CHECK_DIGITS, gretlvals->coeff[i]);
+			MP_PRINT_DIGITS, certvals->coeff[i],
+			MP_PRINT_DIGITS, model.coeff[i]);
 	    }
 
-	    if (mp_vals_differ(certvals->coeff[i], gretlvals->coeff[i],
-			       &diff, prn)) {
-		if (diff > diffmax) diffmax = diff;
-	    }
-		
-
-	    if (verbose && !na(certvals->sderr[i])) {
+	    if (!na(certvals->sderr[i])) {
 		pprintf(prn, " %-20s %#24.*g %#24.*g\n",
 			"(std. error)",
-			MP_CHECK_DIGITS, certvals->sderr[i],
-			MP_CHECK_DIGITS, gretlvals->sderr[i]);
+			MP_PRINT_DIGITS, certvals->sderr[i],
+			MP_PRINT_DIGITS, model.sderr[i]);
 	    }
-
-	    if (mp_vals_differ(certvals->sderr[i], gretlvals->sderr[i],
-			       &diff, prn)) {
-		if (diff > diffmax) diffmax = diff;
-	    }
-
 	}
 
-	if (verbose) {
-	    pputc(prn, '\n');
+	pputc(prn, '\n');
 
-	    pprintf(prn, " %-20s %#24.*g %#24.*g\n"
-		    " %-20s %#24.*g %#24.*g\n"
-		    " %-20s %#24.*g %#24.*g\n"
-		    " %-20s %#24.*g %#24.*g\n", 
-		    "standard error", 
-		    MP_CHECK_DIGITS, certvals->sigma, 
-		    MP_CHECK_DIGITS, gretlvals->sigma,
-		    "error sum of squares", 
-		    MP_CHECK_DIGITS, certvals->ess, 
-		    MP_CHECK_DIGITS, gretlvals->ess,
-		    "R-squared", 
-		    MP_CHECK_DIGITS, certvals->rsq, 
-		    MP_CHECK_DIGITS, gretlvals->rsq,
-		    "F", 
-		    MP_CHECK_DIGITS, certvals->fstt, 
-		    MP_CHECK_DIGITS, gretlvals->fstt);
-	}
-
-	if (mp_vals_differ(certvals->sigma, gretlvals->sigma, &diff, prn)) {
-	    if (diff > diffmax) diffmax = diff;
-	}
-	if (mp_vals_differ(certvals->ess, gretlvals->ess, &diff, prn)) {
-	    if (diff > diffmax) diffmax = diff;
-	}
-	if (mp_vals_differ(certvals->rsq, gretlvals->rsq, &diff, prn)) {
-	    if (diff > diffmax) diffmax = diff;
-	}
-	if (mp_vals_differ(certvals->fstt, gretlvals->fstt, &diff, prn)) {
-	    if (diff > diffmax) diffmax = diff;
-	}
-
-	if (verbose) pputc(prn, '\n');
-
-	if (diffmax > 0.0) {
-	    *mpfails += 1;
-	    pprintf(prn, "* Using gretl GMP plugin: errors found when using"
-		    " %d significant figures\n  (largest error = %g)\n",
-		    MP_CHECK_DIGITS, diffmax);
-	} else {
-	    pprintf(prn, "* Using gretl GMP plugin: results correct to"
-		    " at least %d digits\n",
-		    MP_CHECK_DIGITS);
-	}
-
-	free_gretl_mp_results (gretlvals);
+	pprintf(prn, " %-20s %#24.*g %#24.*g\n"
+		" %-20s %#24.*g %#24.*g\n"
+		" %-20s %#24.*g %#24.*g\n"
+		" %-20s %#24.*g %#24.*g\n", 
+		"standard error", 
+		MP_PRINT_DIGITS, certvals->sigma, 
+		MP_PRINT_DIGITS, model.sigma,
+		"error sum of squares", 
+		MP_PRINT_DIGITS, certvals->ess, 
+		MP_PRINT_DIGITS, model.ess,
+		"R-squared", 
+		MP_PRINT_DIGITS, certvals->rsq, 
+		MP_PRINT_DIGITS, model.rsq,
+		"F", 
+		MP_PRINT_DIGITS, certvals->fstt, 
+		MP_PRINT_DIGITS, model.fstt);
     }
 
+    acc = get_accuracy(&model, certvals, prn);
+
+    if (verbose) {
+	pputc(prn, '\n');
+    }
+
+    if (acc < 12.0) {
+	*mpfails += 1;
+	pprintf(prn, "* Using gretl GMP plugin: errors found when using"
+		" %d significant figures\n  (worst-case log relative error = %.3f)\n",
+		MP_CHECK_DIGITS, acc);
+    } else {
+	pprintf(prn, "* Using gretl GMP plugin: results correct to"
+		" at least %d digits\n", (int) acc);
+	pprintf(prn, "  (worst-case log relative error = %.3f)\n", acc);
+    }
+
+    clear_model(&model);
+
     return err;
 }
 
@@ -759,7 +762,9 @@ int run_gretl_comparison (const char *datname,
 {
     int *list = NULL;
     MODEL *model = NULL;
-    int i, acc;
+    double acc;
+    int wc = 0;
+    int i;
     static int modelnum;
 
     list = malloc((dinfo->v + 1) * sizeof *list);
@@ -784,14 +789,18 @@ int run_gretl_comparison (const char *datname,
     *model = lsq(list, pZ, dinfo, OLS, OPT_D | OPT_Z);
 
     if (model->errcode) {
-	if (verbose) pputc(prn, '\n');
+	if (verbose) {
+	    pputc(prn, '\n');
+	}
 	pprintf(prn, "gretl error code: %d\n", model->errcode);
 	errmsg(model->errcode, prn);
 
 	if (strcmp(datname, "Filip.dat") == 0 &&
 	    model->errcode == E_SINGULAR) {
 	    pputs(prn, "(This error was expected with standard libgretl)\n");
-	} else *errs += 1;
+	} else {
+	    *errs += 1;
+	}
 
 	goto free_stuff;
     }
@@ -813,25 +822,30 @@ int run_gretl_comparison (const char *datname,
 	double xx = 0.0;
 	int t;
 
-	for (t=0; t<dinfo->n; t++) xx += (*pZ)[1][t] *  (*pZ)[1][t];
+	for (t=0; t<dinfo->n; t++) {
+	    xx += (*pZ)[1][t] *  (*pZ)[1][t];
+	}
 	model->rsq = 1.0 - model->ess / xx;
     } 
 
-    acc = get_accuracy(model, certvals, dinfo, prn);
+    acc = get_accuracy(model, certvals, prn);
 
-    if (verbose) pputs(prn, "\n ***");
+    if (verbose) {
+	pputs(prn, "\n ***");
+    }
 
-    if (acc >= 6) {
+    if (acc >= 6.0) {
 	pprintf(prn, "* %s results correct to at least %d digits\n", 
-		LIBGRETLSTR, acc);
-    } else if (acc == 4 || acc == 5) {
-	if (strcmp(datname, "Filip.dat")) {
+		LIBGRETLSTR, (int) acc);
+    } else if (acc >= MIN_DIGITS) {
+	if (strcmp(datname, "Filip.dat") && strcmp(datname, "Wampler5.dat")) {
 	    pprintf(prn, "* %s results correct to only %d digits: "
-		    "POOR\n", LIBGRETLSTR, acc);
+		    "POOR\n", LIBGRETLSTR, (int) acc);
 	    *poor += 1;
 	} else {
-	    pprintf(prn, "* %s results correct to %d digits: "
-		    "OK on Filip.dat\n", LIBGRETLSTR, acc);
+	    pprintf(prn, "* %s results correct to at least %.2f digits\n"
+		    "  (OK on Filip.dat and Wampler5.dat)\n", LIBGRETLSTR, acc);
+	    wc = 1;
 	}	    
     } else {
 	pprintf(prn, "* %s results correct to less than "
@@ -839,7 +853,16 @@ int run_gretl_comparison (const char *datname,
 	*poor += 1;
     }
 
+    if (!wc) {
+	pprintf(prn, "  (worst-case log relative error = %.3f)\n", acc);
+    }
+
+    if (verbose) {
+	pputc(prn, '\n');
+    }
+
  free_stuff:
+
     free(list);
     gretl_model_free(model);
 
@@ -889,8 +912,7 @@ int main (int argc, char *argv[])
     strcpy(datadir, ".");
     if (argc == 2 && argv[1][0] != '-') {
 	strcpy(datadir, argv[1]);
-    }
-    else if (argc == 3) {
+    } else if (argc == 3) {
 	strcpy(datadir, argv[2]);
     }
 
@@ -914,9 +936,11 @@ int main (int argc, char *argv[])
 				     polyterms, &mpfails, prn);
 # endif
 
-	    free_gretl_mp_results(certvals);
+	    free_mp_results(certvals);
 	    certvals = NULL;
+#if 1
 	    free_data_digits(datainfo);
+#endif
 	    destroy_dataset(Z, datainfo);
 	    Z = NULL;
 	    datainfo = NULL;
@@ -945,13 +969,21 @@ static void nist_intro (PRN *prn)
 	  "there should be 0 values for \"data files missing\", \"unexpected "
 	  "errors\" and \"poor results\".\n\n");
 
+    pputs(prn, "The \"log relative error\" is defined as the negative of the "
+	  "base-10 logarithm of (|q - c| / |c|), where q denotes the result "
+	  "produced by gretl and c denotes the certified value.  It represents "
+	  "the number of correct digits in the gretl output.  For more details, "
+	  "see B. D. McCullough, \"Assessing the Reliability of "
+	  "Statistical Software: Part I\", The American Statistician, 52 "
+	  "(1998), pp. 358-366.\n\n");
+
 #ifdef USE_GMP
     pputs(prn, "Each test cases is run twice, once using the standard "
 	  "linear regression calculation in the gretl library and once "
 	  "using mulitple precision arithmetic.\n\n");
 #endif
 
-    pputs(prn, "For more information, please see "
+    pputs(prn, "For more information, please see\n"
 	  "http://www.itl.nist.gov/div898/strd/general/main.html");
 
     pputs(prn, "\n\n");
@@ -1008,9 +1040,11 @@ int run_nist_tests (const char *datapath, const char *outfile, int verbosity)
 				     polyterms, &mpfails, prn);
 # endif
 
-	    free_gretl_mp_results(certvals);
+	    free_mp_results(certvals);
 	    certvals = NULL;
+#if 1
 	    free_data_digits(datainfo);
+#endif
 	    destroy_dataset(Z, datainfo);
 	    Z = NULL;
 	    datainfo = NULL;
